// swad_deg.c: degrees

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <ctype.h>		// For isprint, isspace, etc.
#include <stdio.h>		// For fprintf, etc.
#include <stdlib.h>		// For exit, system, calloc, free, etc.
#include <string.h>		// For string functions
#include <mysql/mysql.h>	// To access MySQL databases

#include "swad_boo.h"
#include "swad_cfg.h"
#include "swad_crs.h"
#include "swad_deg.h"
#include "swad_db.h"
#include "swad_exa.h"
#include "swad_gbl.h"
#include "swad_log.h"
#include "swad_not.h"
#include "swad_par.h"
#include "swad_str.h"
#include "swad_the.h"
#include "swad_txt.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/*************************** Public constants *******************************/
/****************************************************************************/

const char *Deg_StrsDegYear[1+Deg_MAX_YEARS_PER_DEGREE] = {"Opt./Libre conf.","1&ordm;","2&ordm;","3&ordm;","4&ordm;","5&ordm;","6&ordm;"};

/****************************************************************************/
/***************************** Private types ********************************/
/****************************************************************************/

typedef enum {FIRST_YEAR,LAST_YEAR} tFirstOrLastYear;

// Related with range of courses
#define NUM_TYPES_OF_DEG_RANGE	2
typedef enum {DEG_RANGE_ALL_DEGREES=0,DEG_RANGE_THIS_DEGREE=1} tDegRange;

/****************************************************************************/
/**************************** Private constants *****************************/
/****************************************************************************/

#define Deg_MAX_LENGTH_FULL_NAME_DEGREE_ON_PAGE_HEAD 65	// Adjust depending on the size of the style used for the degree on the page head
#define Deg_MAX_LENGTH_FULL_NAME_COURSE_ON_PAGE_HEAD 55	// Adjust depending on the size of the style used for the course on the page head

const char *YearStrs[1+Deg_MAX_YEARS_PER_DEGREE] =
  {
   "Opt./Libre conf.",
   "1&ordm;",
   "2&ordm;",
   "3&ordm;",
   "4&ordm;",
   "5&ordm;",
   "6&ordm;"
  };
const char *SemesterStrs[1+2] =
  {
   "Anual",
   "1&ordm;",
   "2&ordm;",
  };
const char *Crs_FileNamesForInfoType[Crs_NUM_INFO_TYPES] =
  {
   Cfg_CRS_INFO_TEACHING_GUIDE,
   Cfg_CRS_INFO_LECTURES,
   Cfg_CRS_INFO_PRACTICALS,
   Cfg_CRS_INFO_BIBLIOGRAPHY,
   Cfg_CRS_INFO_FAQ,
   Cfg_CRS_INFO_LINKS,
   Cfg_CRS_INFO_ASSESSMENT,
  };

/****************************************************************************/
/**************************** Private prototypes ****************************/
/****************************************************************************/

static void Deg_EditDegreeTypes (void);
static void Deg_ListDegreeTypesForEdition (void);
static void Deg_ListDegreesForEdition (void);
static void Deg_PutFormToCreateDegType (void);
static void Deg_PutFormToCreateDegree (void);
static void Deg_PutHeadDegreeTypes (void);
static void Deg_PutHeadDegrees (void);
static unsigned Deg_ConvStrToYear (const char *StrYear);
static unsigned Deg_ConvStrToSemester (const char *StrSemester);
static void Deg_CreateDegreeType (struct DegreeType *DegTyp);
static void Deg_CreateDegree (struct Degree *Deg);
static void Deg_EditDegrees (void);
static void Deg_GetListDegTypes (void);
static void Deg_GetListDegsOfType (struct DegreeType *DegTyp);
static void Deg_FreeListDegsOfCurrentDegType (void);
static void Deg_GetListDegsAdminByMe (void);
static void Deg_PutParamOtherDegTypCod (long DegTypCod);
static void Deg_PutParamOtherDegCod (long DegCod);
static void Deg_PutParamDegCod (long DegCod);
static void Deg_PutParamOtherCrsCod (long CrsCod);
static long Deg_GetParamOtherDegTypCod (void);
static long Deg_GetParamOtherCrsCod (void);
static unsigned Deg_CountNumDegsOfType (long DegTypCod);
static void Deg_GetDataOfDegreeFromRow (struct Degree *Deg,MYSQL_ROW row);
static void Deg_RemoveDegreeTypeCompletely (long DegTypCod);
static void Deg_RemoveDegreeCompletely (long DegCod);
static void Deg_RenameDegree (struct Degree *Deg,Cte_ShortOrFullName_t ShortOrFullName);
static bool Deg_CheckIfDegreeTypeNameExists (const char *DegTypName,long DegTypCod);
static bool Deg_CheckIfDegreeNameExists (long DegTypCod,const char *FieldName,const char *Name,long DegCod);
static void Deg_ChangeDegYear (struct Degree *Deg,tFirstOrLastYear FirstOrLastYear);

static void Deg_GetListCoursesInDegree (struct Degree *Deg);
static void Deg_EditCourses (struct Degree *Deg);
static void Deg_ListCoursesForEdition (struct Degree *Deg);
static void Deg_PutFormToCreateCourse (struct Degree *Deg);
static void Deg_PutHeadCourses (void);
static bool Deg_CheckIfCourseNameExists (long DegCod,unsigned Year,const char *FieldName,const char *Name,long CrsCod);
static void Deg_CreateCourse (struct Course *Crs);
static void Deg_GetDataOfCourseFromRow (struct Course *Crs,MYSQL_ROW row);
static void Deg_RemoveCourseCompletely (long CrsCod);
static void Deg_EmptyCourseCompletely (long CrsCod);
static bool Deg_CheckIfYearIsValidInDeg (unsigned Year,struct Degree *Deg);
static void Deg_RenameCourse (struct Course *Crs,Cte_ShortOrFullName_t ShortOrFullName);

static void Deg_PutFormSearchCrsTch (const char *SearchTch,tDegRange DegRange);
static void Deg_SearchCrsTchInDB (const char *SearchTch,tDegRange DegRange);
static void Deg_WriteRowCrsData (long NumRow,MYSQL_ROW row,bool WriteColumnAccepted);

/****************************************************************************/
/****************** Write menu to select degree and course ******************/
/****************************************************************************/

void Deg_WriteMenuAllCourses (void)
  {
   extern const char *The_ClassMenuCourses[The_NUM_THEMES];
   extern const char *Txt_Course[Txt_NUM_LANGUAGES];
   extern const char *Txt_shortcut[Txt_NUM_LANGUAGES];

   /***** Start of table *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">\n");

   /***** Write title *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\"><br />%s</td></tr>\n",
            The_ClassMenuCourses[Gbl.Prefs.Theme],Txt_Course[Gbl.Prefs.Language]);

   /***** Write a first selector with all the degree types *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   Deg_WriteSelectorOfDegreeType ();
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Write a second selector with all the degrees of selected type *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   Deg_WriteSelectorOfDegree ();
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Write a third selector with all the years of selected degree *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   Deg_WriteSelectorOfYear ();
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Write a fourth selector with all the courses of selected year *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   Deg_WriteSelectorOfCourse ();
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Link to show XHTML code with a direct link to current course *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\" class=\"MENUD\">");
   if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
     {
      Act_FormStart (ActDirAcc);
      Act_LinkFormSubmit (Act_NEW_WINDOW,"Obtener c&oacute;digo HTML de acceso directo","MENUD");	// Need translation!
      fprintf (Gbl.F.Out,"(%s)</acronym></a></form>",Txt_shortcut[Gbl.Prefs.Language]);
     }
   else
      fprintf (Gbl.F.Out,"&nbsp;");
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** End of table *****/
   fprintf (Gbl.F.Out,"</table>\n");
  }

/****************************************************************************/
/************************ Write selector of degree type *********************/
/****************************************************************************/

void Deg_WriteSelectorOfDegreeType (void)
  {
   extern const char *Txt_Type_of_degree[Txt_NUM_LANGUAGES];
   char Query[128];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long DegTypCod;

   /***** Start form *****/
   Act_FormStart (ActUnk);
   fprintf (Gbl.F.Out,"<select name=\"DegTyp\" style=\"width:120px;\"" \
                      " onchange=\"javascript:document.forms[%d].submit();\">" \
                      "<option value=\"\">[%s]</option>",
	    Gbl.NumForm,Txt_Type_of_degree[Gbl.Prefs.Language]);

   /***** Get degree types from database *****/
   sprintf (Query,"SELECT DegTypCod,DegTypName FROM deg_types ORDER BY DegTypName");
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get types of degree");

   /***** Get degree types *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next degree type */
      row = mysql_fetch_row (mysql_res);

      /* Get degree type code (row[0]) */
      if ((DegTypCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of type of degree.");

      /* Write option */      
      fprintf (Gbl.F.Out,"<option value=\"%ld\"",DegTypCod);
      if (Gbl.CurrentDeg.Deg.DegTypCod >= 0 && DegTypCod == Gbl.CurrentDeg.Deg.DegTypCod)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",row[1]);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** End form *****/
   fprintf (Gbl.F.Out,"</select></form>");
  }

/****************************************************************************/
/*************************** Write selector of degree ***********************/
/****************************************************************************/

void Deg_WriteSelectorOfDegree (void)
  {
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   char Query[128];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long DegCod;

   /***** Start form *****/
   Act_FormStart (ActUnk);
   fprintf (Gbl.F.Out,"<select name=\"DegCod\" style=\"width:120px;\"");
   if (Gbl.CurrentDeg.Deg.DegTypCod >= 0)
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\"",Gbl.NumForm);
   else
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out,"><option value=\"\">[%s]</option>",Txt_Degree[Gbl.Prefs.Language]);

   if (Gbl.CurrentDeg.Deg.DegTypCod >= 0)
     {
      /***** Get degrees of a given type from database *****/
      sprintf (Query,"SELECT DegCod,ShortName FROM degrees" \
                     " WHERE DegTypCod='%ld' ORDER BY ShortName",
               Gbl.CurrentDeg.Deg.DegTypCod);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees of a type");

      /***** Get degrees of this type *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
         /* Get next degree */
         row = mysql_fetch_row (mysql_res);

         /* Get degree code (row[0]) */
         if ((DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong type of degree.");

         /* Write option */      
         fprintf (Gbl.F.Out,"<option value=\"%ld\"",DegCod);
         if (Gbl.CurrentDeg.Deg.DegCod >= 0 && (DegCod == Gbl.CurrentDeg.Deg.DegCod))
	    fprintf (Gbl.F.Out," selected=\"selected\"");
         fprintf (Gbl.F.Out,">%s</option>",row[1]);
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }

   /***** End form *****/
   fprintf (Gbl.F.Out,"</select></form>");
  }

/****************************************************************************/
/**************** Write selector of year in current degree ******************/
/****************************************************************************/

void Deg_WriteSelectorOfYear (void)
  {
   extern const char *Txt_Year[Txt_NUM_LANGUAGES];
   unsigned Year;

   Act_FormStart (ActUnk);
   Deg_PutParamDegCod (Gbl.CurrentDeg.Deg.DegCod);
   fprintf (Gbl.F.Out,"<select name=\"CrsYear\" style=\"width:120px;\"");
   if (Gbl.CurrentDeg.Deg.DegCod >= 0)
     {
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\"><option value=\"%u\">[%s]</option>",
               Gbl.NumForm,Gbl.CurrentCrs.Crs.Year,Txt_Year[Gbl.Prefs.Language]);
      for (Year=0; Year<=Gbl.CurrentDeg.Deg.LastYear; Year++)
         if (Deg_CheckIfYearIsValidInDeg (Year,&Gbl.CurrentDeg.Deg))
           {
	    fprintf (Gbl.F.Out,"<option value=\"%u\"",Year);
	    if (Gbl.YearOK && Year == Gbl.CurrentCrs.Crs.Year)
	       fprintf (Gbl.F.Out," selected=\"selected\"");
	    fprintf (Gbl.F.Out,">%s</option>",Deg_StrsDegYear[Year]);
	   }
     }
   else
      fprintf (Gbl.F.Out," disabled=\"disabled\"><option value=\"\">[%s]</option>",
               Txt_Year[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</select></form>");
  }

/****************************************************************************/
/********** Write selector of course in current degree and year *************/
/****************************************************************************/

void Deg_WriteSelectorOfCourse (void)
  {
   extern const char *Txt_Course[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long CrsCod;

   /***** Start form *****/
   Act_FormStart ((Gbl.Usrs.Me.Logged) ? ActMnu : ActUnk);
   fprintf (Gbl.F.Out,"<select name=\"CrsCod\" style=\"width:120px;\"");
   if (Gbl.YearOK)
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\"><option value=\"%ld",
               Gbl.NumForm,Gbl.CurrentCrs.Crs.CrsCod);
   else
      fprintf (Gbl.F.Out," disabled=\"disabled\"><option value=\"");
   fprintf (Gbl.F.Out,"\">[%s]</option>",Txt_Course[Gbl.Prefs.Language]);

   if (Gbl.YearOK)
     {
      /***** Get courses of degrees of a type from database *****/
      sprintf (Query,"SELECT CrsCod,ShortName FROM courses" \
                     " WHERE DegCod='%ld' and Year='%u' ORDER BY ShortName",
               Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.Year);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get courses of degrees of a type");

      /***** Get courses in this degree *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
         /* Get next course */
         row = mysql_fetch_row (mysql_res);

         /* Get course code (row[0]) */
         if ((CrsCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of course.");

         /* Write option */      
	 fprintf (Gbl.F.Out,"<option value=\"%ld\"",CrsCod);
	 if (Gbl.CurrentCrs.Crs.CrsCod >= 0 && (CrsCod == Gbl.CurrentCrs.Crs.CrsCod))
	    fprintf (Gbl.F.Out," selected=\"selected\"");
	 fprintf (Gbl.F.Out,">%s</option>",row[1]);
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }

   /***** End form *****/
   fprintf (Gbl.F.Out,"</select></form>");
  }

/****************************************************************************/
/************************* Write menu of my courses *************************/
/****************************************************************************/

void Deg_WriteSelectorMyCourses (void)
  {
   extern const char *The_ClassHead[The_NUM_THEMES];
   extern const char *Txt_My_courses[Txt_NUM_LANGUAGES];
   extern const char *Txt_COURSE_SELECTED_None[Txt_NUM_LANGUAGES];
   unsigned NumMyCrs;
   struct Course Crs;

   /***** Fill the list with the courses I belong to *****/ 
   Usr_GetMyCourses ();

   /***** Text message *****/
   fprintf (Gbl.F.Out,"<div align=\"center\" class=\"%s\">%s</div>",
            The_ClassHead[Gbl.Prefs.Theme],Txt_My_courses[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart ((Gbl.Usrs.Me.Logged) ? ActMnu : ActUnk);
   fprintf (Gbl.F.Out,"<select name=\"CrsCod\" style=\"width:120px;\"");
   if (Gbl.Usrs.Me.Logged)
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\">",
               Gbl.NumForm);
   else
      fprintf (Gbl.F.Out," disabled=\"disabled\">");
   fprintf (Gbl.F.Out,"<option value=\"-1\">[%s]</option>",Txt_COURSE_SELECTED_None[Gbl.Prefs.Language]);

   /***** Write an option for each user's course  *****/
   if (Gbl.Usrs.Me.Logged)
      for (NumMyCrs=0; NumMyCrs<Gbl.Usrs.Me.MyCourses.Num; NumMyCrs++)
        {
         /* Get data of this course */
         Crs.CrsCod = Gbl.Usrs.Me.MyCourses.CrsCods[NumMyCrs];
         Deg_GetDataOfCourseByCod (&Crs);

         fprintf (Gbl.F.Out,"<option value=\"%ld\"",Gbl.Usrs.Me.MyCourses.CrsCods[NumMyCrs]);
         if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
            if (Gbl.Usrs.Me.MyCourses.CrsCods[NumMyCrs] == Gbl.CurrentCrs.Crs.CrsCod)
               fprintf (Gbl.F.Out," selected=\"selected\"");
         fprintf (Gbl.F.Out,">%s</option>",Crs.ShortName);
        }

   fprintf (Gbl.F.Out,"</select></form>");
  }

/****************************************************************************/
/****** Link to get the XHTML code for direct access to current course ******/
/****************************************************************************/

void Deg_XHTMLDirAccToCrs (void)
  {
   fprintf (Gbl.F.Out,"<div align=\"center\">\n");
   sprintf (Gbl.Message,"El URL (direcci&oacute;n web) que permite acceder directamente" \
                        " a <strong>%s</strong> es:<br /><strong>https://%s/?CrsCod=%ld</strong>",	// Need translation!
            Gbl.CurrentCrs.Crs.FullName,Cfg_PLATFORM_URL,Gbl.CurrentCrs.Crs.CrsCod);
   Lay_ShowAlert (INFO,Gbl.Message);
   sprintf (Gbl.Message,"Si desea crear una p&aacute;gina web que le permita acceder directamente a <strong>%s</strong>" \
		        " puede copiar todo el c&oacute;digo XHTML siguiente en un archivo nuevo" \
		        " y guardarlo con la extensi&oacute;n &quot;.htm&quot; o &quot;.html&quot;.<br /><br />" \
		        "Si s&oacute;lo desea insertar un enlace en una p&aacute;gina web existente," \
		        " copie las l&iacute;neas indicadas abajo en el lugar deseado del archivo HTML.",	// Need translation!
	    Gbl.CurrentCrs.Crs.FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
//   fprintf (Gbl.F.Out,"<textarea cols=\"70\" rows=\"25\" wrap=\"off\" readonly>\n");
   fprintf (Gbl.F.Out,"<textarea cols=\"120\" rows=\"25\" readonly>\n");

   fprintf (Gbl.F.Out,"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
   fprintf (Gbl.F.Out,"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
   fprintf (Gbl.F.Out,"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n");
   fprintf (Gbl.F.Out," <head>\n");
   fprintf (Gbl.F.Out,"  <title>%s</title>\n",Gbl.CurrentCrs.Crs.FullName);
   fprintf (Gbl.F.Out,"  <meta name=\"generator\"   content=\"%s\" />\n",Cfg_PLATFORM_FULL_NAME);
   fprintf (Gbl.F.Out,"  <meta name=\"author\"      content=\"%s\" />\n",Log_COPYRIGHT);
   fprintf (Gbl.F.Out,"  <meta name=\"keywords\"    content=\"%s, %s\" />\n",Cfg_PLATFORM_SHORT_NAME,Gbl.CurrentCrs.Crs.FullName);
   fprintf (Gbl.F.Out,"  <meta name=\"description\" content=\"Acceso directo a %s (%s) en %s\" />\n",Gbl.CurrentCrs.Crs.FullName,Gbl.CurrentDeg.Deg.FullName,Cfg_PLATFORM_SHORT_NAME);	// Need translation!
   fprintf (Gbl.F.Out," </head>\n");
   fprintf (Gbl.F.Out," <body>\n");
   fprintf (Gbl.F.Out,"  <table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\"><tr><td align=\"center\">\n");
   fprintf (Gbl.F.Out,"   <table border=\"0\">\n");
   fprintf (Gbl.F.Out,"     <tr>\n");
   fprintf (Gbl.F.Out,"      <td align=\"center\">\n");
   fprintf (Gbl.F.Out,"        <!-- Copie desde aqu para incluir el enlace en una pgina existente -->\n");	// Need translation!
   fprintf (Gbl.F.Out,"        <a href=\"https://%s/?CrsCod=%ld\">\n",Cfg_PLATFORM_URL,Gbl.CurrentCrs.Crs.CrsCod);
   fprintf (Gbl.F.Out,"         <img src=\"http://%s/%s/swad112x32.png\" width=\"112\" height=\"32\"\n",Cfg_PLATFORM_URL,Cfg_FOLDER_PUBLIC_LOGO);
   fprintf (Gbl.F.Out,"              alt=\"%s\" style=\"border:0px;\" /><br />\n",Gbl.CurrentCrs.Crs.FullName);
   fprintf (Gbl.F.Out,"         %s<br />(%s)\n",Gbl.CurrentCrs.Crs.FullName,Gbl.CurrentDeg.Deg.FullName);
   fprintf (Gbl.F.Out,"        </a>\n");
   fprintf (Gbl.F.Out,"        <!-- Copie hasta aqu para incluir el enlace en una pgina existente -->\n");	// Need translation!
   fprintf (Gbl.F.Out,"      </td>\n");
   fprintf (Gbl.F.Out,"     </tr>\n");
   fprintf (Gbl.F.Out,"   </table>\n");
   fprintf (Gbl.F.Out,"  </td></tr></table>\n");
   fprintf (Gbl.F.Out," </body>\n");
   fprintf (Gbl.F.Out,"</html>\n");

   fprintf (Gbl.F.Out,"</textarea>\n");
   fprintf (Gbl.F.Out,"</div>\n");
  }

/****************************************************************************/
/********* Write degree and course full names in the top of the page ********/
/****************************************************************************/

void Deg_WriteDegreeAndCourseNames (void)
  {
   extern const char *The_ClassDegree[The_NUM_THEMES];
   extern const char *The_ClassCourse[The_NUM_THEMES];
   char DegreeFullName[Deg_MAX_LENGTH_DEGREE_FULL_NAME+1];	// Full name of degree
   char CourseFullName[Deg_MAX_LENGTH_COURSE_FULL_NAME+1];	// Full name of course

   if (Gbl.CurrentDeg.Deg.DegCod >= 0)
     {
      strcpy (DegreeFullName,Gbl.CurrentDeg.Deg.FullName);
      Str_LimitLengthHTMLStr (DegreeFullName,Deg_MAX_LENGTH_FULL_NAME_DEGREE_ON_PAGE_HEAD);
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">%s</a>",
               Gbl.CurrentDeg.Deg.WWW,The_ClassDegree[Gbl.Prefs.Theme],DegreeFullName);
      if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
        {
         strcpy (CourseFullName,Gbl.CurrentCrs.Crs.FullName);
         Str_LimitLengthHTMLStr (CourseFullName,Deg_MAX_LENGTH_FULL_NAME_COURSE_ON_PAGE_HEAD);
	 fprintf (Gbl.F.Out,"<span class=\"%s\"><br />%s</span>",
                  The_ClassCourse[Gbl.Prefs.Theme],CourseFullName);
        }
     }
  }

/*****************************************************************************/
/**************** Initialize values related to current course ****************/
/*****************************************************************************/

void Deg_InitCurrentCourse (void)
  {
   unsigned NumInfoType;

   if (Gbl.CurrentDeg.Deg.DegCod >= 0)
     {
      sprintf (Gbl.CurrentIns.Logo16,"%s/%s/%s16x16.gif",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Gbl.CurrentIns.Ins.Logo);
      sprintf (Gbl.CurrentIns.Logo32,"%s/%s/%s32x32.gif",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Gbl.CurrentIns.Ins.Logo);
      sprintf (Gbl.CurrentIns.Logo64,"%s/%s/%s64x64.gif",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_INSTITUTIONS,Gbl.CurrentIns.Ins.Logo);

      sprintf (Gbl.CurrentDeg.Logo16,"%s/%s/%s16x16.gif",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Gbl.CurrentDeg.Deg.Logo);
      sprintf (Gbl.CurrentDeg.Logo32,"%s/%s/%s32x32.gif",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Gbl.CurrentDeg.Deg.Logo);
      sprintf (Gbl.CurrentDeg.Logo64,"%s/%s/%s64x64.gif",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Gbl.CurrentDeg.Deg.Logo);

      if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
        {
         /* Paths of course directories */
         sprintf (Gbl.CurrentCrs.PathAbsPriv,"%s/%ld",Cfg_PATH_ABS_CRS_PRIVATE,Gbl.CurrentCrs.Crs.CrsCod);
         sprintf (Gbl.CurrentCrs.PathRelPriv,"%s/%ld",Cfg_PATH_REL_CRS_PRIVATE,Gbl.CurrentCrs.Crs.CrsCod);
         sprintf (Gbl.CurrentCrs.PathRelPubl,"%s/%ld",Cfg_PATH_REL_CRS_PUBLIC,Gbl.CurrentCrs.Crs.CrsCod);
         sprintf (Gbl.CurrentCrs.PathURLPubl,"https://%s/%s/%ld",Cfg_PLATFORM_URL,Cfg_FOLDER_CRS_PUBLIC,Gbl.CurrentCrs.Crs.CrsCod);

         /* Path of file with a list of DNI to register several students in the course */
         sprintf (Gbl.Usrs.PathRelListDNI,"%s/listaDNI.txt",Gbl.CurrentCrs.PathRelPriv);

         /* Paths to files with URL or HTML for theory, practices, etc. of the course */
         for (NumInfoType=0; NumInfoType<Crs_NUM_INFO_TYPES; NumInfoType++)
           {
            sprintf (Gbl.CurrentCrs.Info.Links[NumInfoType].PathRelFileURL,"%s/%s.url",Gbl.CurrentCrs.PathRelPriv,Crs_FileNamesForInfoType[NumInfoType]);
            sprintf (Gbl.CurrentCrs.Info.Links[NumInfoType].PathRelFileZIP,"%s/%s.zip",Gbl.CurrentCrs.PathRelPriv,Crs_FileNamesForInfoType[NumInfoType]);
            sprintf (Gbl.CurrentCrs.Info.Links[NumInfoType].PathRelWebPage,"%s/%s"    ,Gbl.CurrentCrs.PathRelPubl,Crs_FileNamesForInfoType[NumInfoType]);
            sprintf (Gbl.CurrentCrs.Info.Links[NumInfoType].PathAbsWebPage,"%s/%s"    ,Gbl.CurrentCrs.PathURLPubl,Crs_FileNamesForInfoType[NumInfoType]);
           }

         /***** If any of the course directories doesn't exists, create it *****/
         if (!Fil_CheckIfPathExists (Gbl.CurrentCrs.PathRelPriv))
            Fil_CreateDirIfNotExists (Gbl.CurrentCrs.PathRelPriv);
         if (!Fil_CheckIfPathExists (Gbl.CurrentCrs.PathRelPubl))
            Fil_CreateDirIfNotExists (Gbl.CurrentCrs.PathRelPubl);
        }
     }
  }

/****************************************************************************/
/********************** Request edition of degree types *********************/
/****************************************************************************/

void Deg_ReqEditDegreeTypes (void)
  {
   /***** Get list of degree types *****/
   Deg_GetListDegTypes ();

   /***** Put form to edit degree types *****/
   Deg_EditDegreeTypes ();

   /***** Free list of degree types *****/
   Deg_FreeListDegTypes ();
  }

/****************************************************************************/
/************************* Request edition of degrees ***********************/
/****************************************************************************/

void Deg_ReqEditDegrees (void)
  {
   if (Gbl.CurrentDeg.Deg.DegTypCod >= 0)
     {
      /***** Get list of degree types and degrees *****/
      Deg_GetListDegTypes ();
      Deg_GetListDegsOfType (&Gbl.CurrentDegTyp.DegTyp);

      /***** Put form to edit degrees *****/
      Deg_EditDegrees ();

      /***** Free list of degree types and degrees *****/
      Deg_FreeListDegsOfCurrentDegType ();
      Deg_FreeListDegTypes ();
     }
   else
      Lay_ShowAlert (WARNING,"Debe seleccionar un tipo de titulaci&oacute;n.");	// Need translation!
  }

/****************************************************************************/
/************************ Put forms to edit degree types ********************/
/****************************************************************************/

static void Deg_EditDegreeTypes (void)
  {
   extern const char *Txt_Types_of_degree[Txt_NUM_LANGUAGES];

   /***** Help message *****/
   if (Gbl.Degs.DegTypes.Num) // Degree types found...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir un nuevo tipo de titulaci&oacute;n rellene el nombre del tipo de titulaci&oacute;n" \
                          " (por ejemplo Licenciatura, Ingenier&iacute;a, etc.) y pulse el icono verde.<br />" \
                          "Para eliminar un tipo de titulaci&oacute;n existente pulse el aspa roja (no puede eliminar tipos de titulaci&oacute;n con titulaciones).<br />" \
                          "Para modificar un tipo de titulaci&oacute;n existente," \
                          " edite las casillas correspondientes y despu&eacute;s pulse con el rat&oacute;n" \
                          " fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s:",Txt_Types_of_degree[Gbl.Prefs.Language]);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// No degree types found
      Lay_ShowAlert (INFO,"No existen tipos de titulaci&oacute;n.<br />" \
                          "Para a&ntilde;adir un tipo de titulaci&oacute;n rellene el nombre del tipo de titulaci&oacute;n" \
                          " y pulse el icono verde.");	// Need translation!

   /***** Forms to edit degree types *****/
   /* Start table with current degree types */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List current degree types */
   if (Gbl.Degs.DegTypes.Num)
      Deg_ListDegreeTypesForEdition ();

   /* Put a form to create a new degree type */
   Deg_PutFormToCreateDegType ();

   /* End of table with current degree types */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/******************* List current degree types for edition ******************/
/****************************************************************************/

static void Deg_ListDegreeTypesForEdition (void)
  {
   extern const char *Txt_Remove_type_of_degree[Txt_NUM_LANGUAGES];
   unsigned NumDegTyp;

   /***** Write heading *****/
   Deg_PutHeadDegreeTypes ();

   /***** List degree types with forms for edition *****/
   for (NumDegTyp=0; NumDegTyp<Gbl.Degs.DegTypes.Num; NumDegTyp++)
     {
      /* Put icon to remove degree type */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      if (Gbl.Degs.DegTypes.Lst[NumDegTyp].NumDegs)	// Degree type has degrees ==> deletion forbidden
         fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                  Gbl.Prefs.IconsURL);
      else
        {
         Act_FormStart (ActRemDegTyp);
         Deg_PutParamOtherDegTypCod (Gbl.Degs.DegTypes.Lst[NumDegTyp].DegTypCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_type_of_degree[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Txt_Remove_type_of_degree[Gbl.Prefs.Language]);
        }

      /* Degree type code */
      fprintf (Gbl.F.Out,"</td><td align=\"center\" valign=\"middle\" class=\"DAT_SMALL\">%ld</td>",
               Gbl.Degs.DegTypes.Lst[NumDegTyp].DegTypCod);

      /* Name of degree type */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\">");
      Act_FormStart (ActRenDegTyp);
      Deg_PutParamOtherDegTypCod (Gbl.Degs.DegTypes.Lst[NumDegTyp].DegTypCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"DegTypName\" size=\"25\" maxlength=\"%u\" value=\"%s\"" \
                     " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Deg_MAX_LENGTH_DEGREE_TYPE_NAME,Gbl.Degs.DegTypes.Lst[NumDegTyp].DegTypName,Gbl.NumForm);

      /* Number of degrees of this type */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"DAT_SMALL\">%u</td></tr>",
               Gbl.Degs.DegTypes.Lst[NumDegTyp].NumDegs);
     }
  }

/****************************************************************************/
/********************* List current degrees for edition *********************/
/****************************************************************************/

static void Deg_ListDegreesForEdition (void)
  {
   extern const char *Txt_Remove_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_place[Txt_NUM_LANGUAGES];
   unsigned NumDeg;
   unsigned NumDegTypAux;
   struct DegreeType *DegTypAux;
   struct Degree *Deg;
   struct Institution Ins;
   unsigned NumIns;
   unsigned NumPlc;
   unsigned Year;

   /***** Write heading *****/
   Deg_PutHeadDegrees ();

   /***** List the degrees *****/
   for (NumDeg=0; NumDeg<Gbl.CurrentDegTyp.DegTyp.NumDegs; NumDeg++)
     {
      Deg = &(Gbl.CurrentDegTyp.LstDegs[NumDeg]);

      /* Get data of institution of this degree */
      Ins.InsCod = Deg->InsCod;
      Ins_GetDataOfInstitutionByCod (&Ins);

      /* Put icon to remove degree */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
      if (Deg->NumCourses)	// Degree has courses ==>  deletion forbidden
         fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                  Gbl.Prefs.IconsURL);
      else
        {
         Act_FormStart (ActRemDeg);
         Deg_PutParamOtherDegCod (Deg->DegCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_degree[Gbl.Prefs.Language],NULL);
         fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                  Gbl.Prefs.IconsURL,Txt_Remove_degree[Gbl.Prefs.Language]);
        }

      /* Degree code */
      fprintf (Gbl.F.Out,"</td><td align=\"center\" valign=\"middle\" class=\"DAT_SMALL\">%ld</td>",
               Deg->DegCod);

      /* Degree type */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegTyp);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<select name=\"OthDegTypCod\" style=\"width:50px;\" onchange=\"javascript:document.forms[%d].submit();\">",
               Gbl.NumForm);
      for (NumDegTypAux=0; NumDegTypAux<Gbl.Degs.DegTypes.Num; NumDegTypAux++)
        {
         DegTypAux = &Gbl.Degs.DegTypes.Lst[NumDegTypAux];
         fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
                  DegTypAux->DegTypCod,
                  (DegTypAux->DegTypCod == Gbl.CurrentDegTyp.DegTyp.DegTypCod) ? " selected=\"selected\"" : "",
                  DegTypAux->DegTypName);
        }
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Institution */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegIns);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<select name=\"InsCod\" style=\"width:50px;\" onchange=\"javascript:document.forms[%d].submit();\">",
               Gbl.NumForm);
      fprintf (Gbl.F.Out,"<option value=\"0\"");
      if (Deg->InsCod == 0)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_Another_institution[Gbl.Prefs.Language]);
      for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
         fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
                  Gbl.Inss.Lst[NumIns].InsCod,
                  (Gbl.Inss.Lst[NumIns].InsCod == Deg->InsCod) ? " selected=\"selected\"" : "",
                  Gbl.Inss.Lst[NumIns].ShortName);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Place */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegPlc);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<select name=\"PlcCod\" style=\"width:50px;\" onchange=\"javascript:document.forms[%d].submit();\">",
               Gbl.NumForm);
      fprintf (Gbl.F.Out,"<option value=\"0\"");
      if (Deg->PlcCod == 0)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_Another_place[Gbl.Prefs.Language]);
      for (NumPlc=0; NumPlc<Gbl.Plcs.Num; NumPlc++)
         fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
                  Gbl.Plcs.Lst[NumPlc].PlcCod,
                  (Gbl.Plcs.Lst[NumPlc].PlcCod == Deg->PlcCod) ? " selected=\"selected\"" : "",
                  Gbl.Plcs.Lst[NumPlc].ShortName);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Degree short name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenDegSho);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ShortName\" size=\"10\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Deg_MAX_LENGTH_DEGREE_SHORT_NAME,Deg->ShortName,Gbl.NumForm);

      /* Degree full name */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActRenDegFul);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FullName\" size=\"20\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Deg_MAX_LENGTH_DEGREE_FULL_NAME,Deg->FullName,Gbl.NumForm);

      /* Degree first year */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegFstYea);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<select name=\"FirstYear\" onchange=\"javascript:document.forms[%d].submit();\">",
               Gbl.NumForm);
      for (Year=0; Year<=Deg_MAX_YEARS_PER_DEGREE; Year++)
         fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%u</option>",
                  Year,(Year == Deg->FirstYear) ? " selected=\"selected\"" : "",Year);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Degree last year */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegLstYea);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<select name=\"LastYear\" onchange=\"javascript:document.forms[%d].submit();\">",
               Gbl.NumForm);
      for (Year=0; Year<=Deg_MAX_YEARS_PER_DEGREE; Year++)
         fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%u</option>",
                  Year,(Year == Deg->LastYear) ? " selected=\"selected\"" : "",Year);
      fprintf (Gbl.F.Out,"</select></form></td>");

      /* Degree optional year */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegOptYea);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"OptYear\" value=\"Y\"%s" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Deg->OptYear ? " checked=\"checked\"" : "",Gbl.NumForm);

      /* Degree logo */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegLog);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Logo\" size=\"4\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Deg_MAX_LENGTH_DEGREE_LOGO,Deg->Logo,Gbl.NumForm);

      /* Degree WWW */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
      Act_FormStart (ActChgDegWWW);
      Deg_PutParamOtherDegCod (Deg->DegCod);
      fprintf (Gbl.F.Out,"<input type=\"text\" name=\"WWW\" size=\"5\" maxlength=\"%u\" value=\"%s\"" \
                         " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
               Cte_MAX_LENGTH_WWW,Deg->WWW,Gbl.NumForm);

      /* Current number of courses in this degree */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"DAT_SMALL\">%d</td></tr>\n",
               Deg->NumCourses);
     }
  }

/****************************************************************************/
/******************** Put a form to create a new degree type ****************/
/****************************************************************************/

static void Deg_PutFormToCreateDegType (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_type_of_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_type_of_degree[Txt_NUM_LANGUAGES];

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"4\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"4\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_type_of_degree[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewDegTyp);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Deg_PutHeadDegreeTypes ();

   /***** New degree type *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_type_of_degree[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Degree type name *****/
   fprintf (Gbl.F.Out,"<td></td><td align=\"left\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"DegTypName\" size=\"25\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Deg_MAX_LENGTH_DEGREE_TYPE_NAME,Gbl.Degs.EditingDegTyp.DegTypName);
  }

/****************************************************************************/
/*********************** Put a form to create a new degree ******************/
/****************************************************************************/

static void Deg_PutFormToCreateDegree (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_place[Txt_NUM_LANGUAGES];
   struct Degree *Deg;
   unsigned NumIns;
   unsigned NumPlc;
   unsigned Year;

   Deg = &Gbl.Degs.EditingDeg;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"12\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"12\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_degree[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewDeg);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Deg_PutHeadDegrees ();

   /***** New degree *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_degree[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Degree code *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"DAT_SMALL\">&nbsp;</td>");

   /***** Degree type *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><select name=\"OthDegTypCod\" style=\"width:50px;\">");
   fprintf (Gbl.F.Out,"<option value=\"%ld\" selected=\"selected\">%s</option>",
            Gbl.CurrentDegTyp.DegTyp.DegTypCod,Gbl.CurrentDegTyp.DegTyp.DegTypName);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Institution *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"InsCod\" style=\"width:50px;\">");
   fprintf (Gbl.F.Out,"<option value=\"0\"");
   if (Deg->InsCod == 0)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option>",Txt_Another_institution[Gbl.Prefs.Language]);
   for (NumIns=0; NumIns<Gbl.Inss.Num; NumIns++)
      fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
               Gbl.Inss.Lst[NumIns].InsCod,
               (Gbl.Inss.Lst[NumIns].InsCod == Deg->InsCod) ? " selected=\"selected\"" : "",
               Gbl.Inss.Lst[NumIns].ShortName);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Place *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"PlcCod\" style=\"width:50px;\">");
   fprintf (Gbl.F.Out,"<option value=\"0\"");
   if (Deg->PlcCod == 0)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option>",Txt_Another_place[Gbl.Prefs.Language]);
   for (NumPlc=0; NumPlc<Gbl.Plcs.Num; NumPlc++)
      fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
               Gbl.Plcs.Lst[NumPlc].PlcCod,
               (Gbl.Plcs.Lst[NumPlc].PlcCod == Deg->PlcCod) ? " selected=\"selected\"" : "",
               Gbl.Plcs.Lst[NumPlc].ShortName);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Degree short name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"ShortName\" size=\"10\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            Deg_MAX_LENGTH_DEGREE_SHORT_NAME,Deg->ShortName);

   /***** Degree full name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"FullName\" size=\"20\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            Deg_MAX_LENGTH_DEGREE_FULL_NAME,Deg->FullName);

   /***** First year *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><select name=\"FirstYear\">");
   for (Year=0; Year<=Deg_MAX_YEARS_PER_DEGREE; Year++)
      fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%u</option>",
               Year,(Year == Deg->FirstYear) ? " selected=\"selected\"" : "",Year);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Last year *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><select name=\"LastYear\">");
   for (Year=0; Year<=Deg_MAX_YEARS_PER_DEGREE; Year++)
      fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%u</option>",Year,(Year == Deg->LastYear) ? " selected=\"selected\"" : "",Year);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Degree optional year *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"checkbox\" name=\"OptYear\" value=\"Y\"%s /></td>",
            (Deg->OptYear) ? " checked=\"checked\"" : "");

   /***** Degree logo *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"Logo\" size=\"4\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            Deg_MAX_LENGTH_DEGREE_LOGO,Deg->Logo);

   /***** Degree WWW *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"WWW\" size=\"5\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td></tr></table></form></td></tr>\n",
            Cte_MAX_LENGTH_WWW,Deg->WWW);
  }

/****************************************************************************/
/***************** Write header with fields of a degree type ****************/
/****************************************************************************/

static void Deg_PutHeadDegreeTypes (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type_of_BR_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_BR_degrees[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Type_of_BR_degree[Gbl.Prefs.Language],
            Txt_No_of_BR_degrees[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************** Write header with fields of a degree ******************/
/****************************************************************************/

static void Deg_PutHeadDegrees (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type_of_BR_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Institution[Txt_NUM_LANGUAGES];
   extern const char *Txt_Place[Txt_NUM_LANGUAGES];
   extern const char *Txt_Short_BR_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Full_BR_Name[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">Prim.<br />curso</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">&Uacute;lt.<br />curso</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">&iquest;Cur.<br />opt.?</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">Logo</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">WWW</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">N&ordm;<br />asig.</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Type_of_BR_degree[Gbl.Prefs.Language],
            Txt_Institution[Gbl.Prefs.Language],
            Txt_Place[Gbl.Prefs.Language],
            Txt_Short_BR_Name[Gbl.Prefs.Language],
            Txt_Full_BR_Name[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************** Convert string to year of a degree ********************/
/****************************************************************************/

static unsigned Deg_ConvStrToYear (const char *StrYear)
  {
   int Year;

   if (sscanf (StrYear,"%d",&Year) != 1)
      return 0;
   if (Year < 0)
      return 0;
   if (Year > Deg_MAX_YEARS_PER_DEGREE)
      return Deg_MAX_YEARS_PER_DEGREE;
   return (unsigned) Year;
  }

/****************************************************************************/
/*********************** Convert string to semester *************************/
/****************************************************************************/

static unsigned Deg_ConvStrToSemester (const char *StrSemester)
  {
   int Semester;

   if (sscanf (StrSemester,"%d",&Semester) != 1)
      return 0;
   if (Semester < 0)
      return 0;
   if (Semester > 2)
      return 0;
   return (unsigned) Semester;
  }

/****************************************************************************/
/************************** Create a new degree type ************************/
/****************************************************************************/

static void Deg_CreateDegreeType (struct DegreeType *DegTyp)
  {
   extern const char *Txt_Created_new_type_of_degree_X[Txt_NUM_LANGUAGES];
   char Query[64+Deg_MAX_LENGTH_DEGREE_TYPE_NAME];

   /***** Create a new degree type *****/
   sprintf (Query,"INSERT INTO deg_types SET DegTypName='%s'",
            DegTyp->DegTypName);
   DB_QueryINSERT (Query,"can not create a new type of degree");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_type_of_degree_X[Gbl.Prefs.Language],
            DegTyp->DegTypName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/***************************** Create a new degree **************************/
/****************************************************************************/

static void Deg_CreateDegree (struct Degree *Deg)
  {
   extern const char *Txt_Created_new_degree_X[Txt_NUM_LANGUAGES];
   char Query[1024];

   /***** Create a new degree *****/
   sprintf (Query,"INSERT INTO degrees (DegTypCod,InsCod,PlcCod,ShortName,FullName,FirstYear,LastYear,OptYear,Logo,WWW)" \
                  " VALUES ('%ld','%ld','%ld','%s','%s','%u','%u','%c','%s','%s')",
            Deg->DegTypCod,Deg->InsCod,Deg->PlcCod,Deg->ShortName,Deg->FullName,
            Deg->FirstYear,Deg->LastYear,Deg->OptYear ? 'Y' : 'N',Deg->Logo,Deg->WWW);
   DB_QueryINSERT (Query,"can not create a new degree");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_degree_X[Gbl.Prefs.Language],
            Deg->FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/*************************** Put forms to edit degrees **********************/
/****************************************************************************/

static void Deg_EditDegrees (void)
  {
   extern const char *Txt_Degrees_of_type[Txt_NUM_LANGUAGES];

   /***** Help message *****/
   if (Gbl.CurrentDegTyp.DegTyp.NumDegs) // If there are degrees of this type...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir una titulaci&oacute;n nueva" \
                          " rellene los nombres breve y completo de la titulaci&oacute;n y el resto de los campos," \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar una titulaci&oacute;n existente pulse el aspa roja (no puede eliminar titulaciones con asignaturas).<br />" \
                          "Para modificar una titulaci&oacute;n existente," \
                          " ed&iacute;te las casillas correspondientes" \
                          " y despu&eacute;s pulse con el rat&oacute;n fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (Gbl.Message,"%s <strong>%s</strong>:",Txt_Degrees_of_type[Gbl.Prefs.Language],Gbl.CurrentDegTyp.DegTyp.DegTypName);
      Lay_WriteTitle (Gbl.Message);
     }
   else	// There aren't degrees of this type
     {
      sprintf (Gbl.Message,"No existen titulaciones de tipo <strong>%s</strong>.<br />" \
                           "Para a&ntilde;adir una titulaci&oacute;n" \
                           " rellene los datos de la titulaci&oacute;n y pulse el icono verde.",	// Need translation!
               Gbl.CurrentDegTyp.DegTyp.DegTypName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Get list of institutions *****/
   Ins_GetListInstitutions (false);
   if (!Gbl.Inss.Num)
     {
      Lay_ShowAlert (WARNING,"No hay lista de instituciones." \
                             " Debe crear al menos una instituci&oacute;n antes de crear titulaciones.");	// Need translation!
      return;
     }

   /***** Get list of places *****/
   Plc_GetListPlaces ();
   if (!Gbl.Plcs.Num)
     {
      Lay_ShowAlert (WARNING,"No hay lista de lugares." \
                             " Debe crear al menos un lugar antes de crear titulaciones.");	// Need translation!
      return;
     }

   /***** Forms to edit degrees *****/
   /* Start table with current degrees */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List the current degrees */
   if (Gbl.CurrentDegTyp.DegTyp.NumDegs)
      Deg_ListDegreesForEdition ();

   /* Put a form to create a new degree */
   Deg_PutFormToCreateDegree ();

   /* End of table with degrees */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free list of places *****/
   Plc_FreeListPlaces ();

   /***** Free list of institutions *****/
   Ins_FreeListInstitutions ();
  }

/****************************************************************************/
/**************** Create a list with current degree types *******************/
/****************************************************************************/

static void Deg_GetListDegTypes (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow;

   /***** Get types of degree from database *****/
   sprintf (Query,"(SELECT deg_types.DegTypCod,deg_types.DegTypName AS DegTypName,COUNT(degrees.DegCod)" \
                  " FROM deg_types,degrees WHERE deg_types.DegTypCod=degrees.DegTypCod" \
                  " GROUP BY degrees.DegTypCod)" \
                  " UNION " \
                  "(SELECT DegTypCod,DegTypName,'0'" \
                  " FROM deg_types WHERE DegTypCod NOT IN (SELECT DegTypCod FROM degrees))" \
                  " ORDER BY DegTypName");
   Gbl.Degs.DegTypes.Num = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get types of degree");

   /***** Get degree types *****/
   if (Gbl.Degs.DegTypes.Num)
     {
      /***** Create a list of degree types *****/
      if ((Gbl.Degs.DegTypes.Lst = (struct DegreeType *) calloc (Gbl.Degs.DegTypes.Num,sizeof (struct DegreeType))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store types of degree.");

      /***** Get degree types *****/
      for (NumRow=0; NumRow<Gbl.Degs.DegTypes.Num; NumRow++)
        {
         /* Get next degree type */
         row = mysql_fetch_row (mysql_res);

         /* Get degree type code (row[0]) */
         if ((Gbl.Degs.DegTypes.Lst[NumRow].DegTypCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of type of degree.");

         /* Get degree type name (row[1]) */
         strncpy (Gbl.Degs.DegTypes.Lst[NumRow].DegTypName,row[1],Deg_MAX_LENGTH_DEGREE_TYPE_NAME);
         Gbl.Degs.DegTypes.Lst[NumRow].DegTypName[Deg_MAX_LENGTH_DEGREE_TYPE_NAME] = '\0';

         /* Number of degrees of this type (row[2]) */
         if (sscanf (row[2],"%u",&Gbl.Degs.DegTypes.Lst[NumRow].NumDegs) != 1)
            Lay_ShowErrorAndExit ("Error when getting number of degrees of a type");
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/********************* Create a list with all the degrees *******************/
/****************************************************************************/

void Deg_GetListAllDegs (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumDeg;

   /***** Get degrees admin by me from database *****/
   sprintf (Query,"SELECT DegCod,DegTypCod,InsCod,PlcCod,ShortName,FullName,FirstYear,LastYear,OptYear,Logo,WWW" \
                  " FROM degrees ORDER BY ShortName");
   Gbl.Degs.AllDegs.Num = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get degrees admin by you");

   if (Gbl.Degs.AllDegs.Num) // Degrees found...
     {
      /***** Create list with degrees *****/
      if ((Gbl.Degs.AllDegs.Lst = (struct Degree *) calloc (Gbl.Degs.AllDegs.Num,sizeof (struct Degree))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store degrees admin by you.");

      /***** Get the degrees *****/
      for (NumDeg=0; NumDeg<Gbl.Degs.AllDegs.Num; NumDeg++)
        {
         /* Get next degree */
         row = mysql_fetch_row (mysql_res);
         Deg_GetDataOfDegreeFromRow (&(Gbl.Degs.AllDegs.Lst[NumDeg]),row);
        }
     }
   else
      Gbl.Degs.AllDegs.Lst = NULL;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/*********************** Free list of all the degrees ************************/
/*****************************************************************************/

void Deg_FreeListAllDegs (void)
  {
   if (Gbl.Degs.AllDegs.Lst)
     {
      free ((void *) Gbl.Degs.AllDegs.Lst);
      Gbl.Degs.AllDegs.Lst = NULL;
      Gbl.Degs.AllDegs.Num = 0;
     }
  }

/****************************************************************************/
/****************** Create a list with the degrees of a type ****************/
/****************************************************************************/

static void Deg_GetListDegsOfType (struct DegreeType *DegTyp)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumDeg;

   /***** Get degrees of a given type from database *****/
   sprintf (Query,"SELECT DegCod,DegTypCod,InsCod,PlcCod,ShortName,FullName,FirstYear,LastYear,OptYear,Logo,WWW" \
                  " FROM degrees WHERE DegTypCod='%ld' ORDER BY ShortName",DegTyp->DegTypCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees of a type");

   /***** Count number of rows in result *****/
   if (NumRows) // Degrees found...
     {
      DegTyp->NumDegs = (unsigned) NumRows;

      /***** Create list with degrees of this type *****/
      if ((Gbl.CurrentDegTyp.LstDegs = (struct Degree *) calloc (DegTyp->NumDegs,sizeof (struct Degree))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store degrees of a type.");

      /***** Get the degrees of this type *****/
      for (NumDeg=0; NumDeg<DegTyp->NumDegs; NumDeg++)
        {
         /* Get next degree */
         row = mysql_fetch_row (mysql_res);
         Deg_GetDataOfDegreeFromRow (&(Gbl.CurrentDegTyp.LstDegs[NumDeg]),row);
        }
     }
   else	// Error: degrees should be found, but really they haven't be found. This never should happen.
      DegTyp->NumDegs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********* Free list of degree types and list of degrees of each type ********/
/*****************************************************************************/

static void Deg_FreeListDegsOfCurrentDegType (void)
  {
   if (Gbl.CurrentDegTyp.LstDegs)
     {
      free ((void *) Gbl.CurrentDegTyp.LstDegs);
      Gbl.CurrentDegTyp.LstDegs = NULL;
      Gbl.CurrentDegTyp.DegTyp.NumDegs = 0;
     }
  }

/*****************************************************************************/
/********* Free list of degree types and list of degrees of each type ********/
/*****************************************************************************/

void Deg_FreeListDegTypes (void)
  {
   /***** Free memory used by the list of degree types *****/
   if (Gbl.Degs.DegTypes.Lst)
     {
      free ((void *) Gbl.Degs.DegTypes.Lst);
      Gbl.Degs.DegTypes.Lst = NULL;
      Gbl.Degs.DegTypes.Num = 0;
     }
  }

/****************************************************************************/
/*********** Create a list with the degrees administrated by me *************/
/****************************************************************************/

static void Deg_GetListDegsAdminByMe (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumDeg;

   /***** Get degrees admin by me from database *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
      sprintf (Query,"SELECT DegCod,DegTypCod,InsCod,PlcCod,ShortName,FullName,FirstYear,LastYear,OptYear,Logo,WWW" \
                     " FROM degrees ORDER BY ShortName");
   else	// Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN
      sprintf (Query,"SELECT degrees.DegCod,degrees.DegTypCod,degrees.InsCod,degrees.PlcCod,degrees.ShortName,degrees.FullName,degrees.FirstYear,degrees.LastYear,degrees.OptYear,degrees.Logo,degrees.WWW" \
                     " FROM deg_admin,degrees" \
                     " WHERE deg_admin.UsrCod='%ld' AND deg_admin.DegCod<>'-1' AND deg_admin.DegCod=degrees.DegCod" \
                     " ORDER BY degrees.ShortName",Gbl.Usrs.Me.UsrDat.UsrCod);
   Gbl.Usrs.Me.MyAdminDegs.Num = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get degrees admin by you");

   /***** Count number of rows in result *****/
   if (Gbl.Usrs.Me.MyAdminDegs.Num) // Degrees found...
     {
      /***** Create list with degrees of this type *****/
      if ((Gbl.Usrs.Me.MyAdminDegs.Lst = (struct Degree *) calloc (Gbl.Usrs.Me.MyAdminDegs.Num,sizeof (struct Degree))) == NULL)
         Lay_ShowErrorAndExit ("Nout enough memory to store degrees admin by you.");

      /***** Get the degrees of this type *****/
      for (NumDeg=0; NumDeg<Gbl.Usrs.Me.MyAdminDegs.Num; NumDeg++)
        {
         /* Get next degree */
         row = mysql_fetch_row (mysql_res);
         Deg_GetDataOfDegreeFromRow (&(Gbl.Usrs.Me.MyAdminDegs.Lst[NumDeg]),row);
        }
     }
   else
      Gbl.Usrs.Me.MyAdminDegs.Lst = NULL;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/***************** Free list of degrees administrated by me ******************/
/*****************************************************************************/

void Deg_FreeListMyAdminDegs (void)
  {
   if (Gbl.Usrs.Me.MyAdminDegs.Lst)
     {
      free ((void *) Gbl.Usrs.Me.MyAdminDegs.Lst);
      Gbl.Usrs.Me.MyAdminDegs.Lst = NULL;
      Gbl.Usrs.Me.MyAdminDegs.Num = 0;
     }
  }

/****************************************************************************/
/***************** Receive form to create a new degree type *****************/
/****************************************************************************/

void Deg_RecFormNewDegTyp (void)
  {
   extern const char *Txt_The_type_of_degree_X_already_exists[Txt_NUM_LANGUAGES];
   struct DegreeType *DegTyp;

   DegTyp = &Gbl.Degs.EditingDegTyp;

   /***** Get parameters from form *****/
   /* Get the name of degree type */
   Par_GetParam (Par_PARAM_SINGLE,"DegTypName",DegTyp->DegTypName,Deg_MAX_LENGTH_DEGREE_TYPE_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,DegTyp->DegTypName,Deg_MAX_LENGTH_DEGREE_TYPE_NAME,true);

   if (DegTyp->DegTypName[0])	// If there's a degree type name
     {
      /***** If name of degree type was in database... *****/
      if (Deg_CheckIfDegreeTypeNameExists (DegTyp->DegTypName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_type_of_degree_X_already_exists[Gbl.Prefs.Language],
                  DegTyp->DegTypName);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else	// Add new degree type to database
         Deg_CreateDegreeType (DegTyp);
     }
   else	// If there isn't a degree type name
     {
      sprintf (Gbl.Message,"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   Deg_ReqEditDegreeTypes ();
  }

/****************************************************************************/
/******************* Receive form to create a new degree ********************/
/****************************************************************************/

void Deg_RecFormNewDeg (void)
  {
   extern const char *Txt_The_degree_X_already_exists[Txt_NUM_LANGUAGES];
   struct Degree *Deg;
   char YearStr[1+1];
   char YN[1+1];

   Deg = &Gbl.Degs.EditingDeg;

   /***** Get parameters from form *****/
   if ((Deg->DegTypCod = Deg_GetParamOtherDegTypCod ()) >= 0) // Degree type valid
     {
      /* Get institution */
      Deg->InsCod = Ins_GetParamInsCod ();

      /* Get place */
      Deg->PlcCod = Plc_GetParamPlcCod ();

      /* Get degree short name */
      Par_GetParam (Par_PARAM_SINGLE,"ShortName",Deg->ShortName,Deg_MAX_LENGTH_DEGREE_SHORT_NAME);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Deg->ShortName,Deg_MAX_LENGTH_DEGREE_SHORT_NAME,true);

      /* Get degree full name */
      Par_GetParam (Par_PARAM_SINGLE,"FullName",Deg->FullName,Deg_MAX_LENGTH_DEGREE_FULL_NAME);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Deg->FullName,Deg_MAX_LENGTH_DEGREE_FULL_NAME,true);

      /* Get first year */
      Par_GetParam (Par_PARAM_SINGLE,"FirstYear",YearStr,1);
      Deg->FirstYear = Deg_ConvStrToYear (YearStr);

      /* Get last year */
      Par_GetParam (Par_PARAM_SINGLE,"LastYear",YearStr,1);
      Deg->LastYear = Deg_ConvStrToYear (YearStr);

      /* Get optional year */
      Par_GetParam (Par_PARAM_SINGLE,"OptYear",YN,1);
      Deg->OptYear = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

      /* Get degree logo */
      Par_GetParam (Par_PARAM_SINGLE,"Logo",Deg->Logo,Deg_MAX_LENGTH_DEGREE_LOGO);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Deg->Logo,Deg_MAX_LENGTH_DEGREE_LOGO,true);

      /* Get degree WWW */
      Par_GetParam (Par_PARAM_SINGLE,"WWW",Deg->WWW,Cte_MAX_LENGTH_WWW);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Deg->WWW,Cte_MAX_LENGTH_WWW,true);

      if (Deg->ShortName[0] && Deg->FullName[0])	// If there's a degree name
        {
         if (Deg->Logo[0] && Deg->WWW[0])
           {
            /***** If name of degree was in database... *****/
            if (Deg_CheckIfDegreeNameExists (Deg->DegTypCod,"ShortName",Deg->ShortName,-1L))
              {
               sprintf (Gbl.Message,Txt_The_degree_X_already_exists[Gbl.Prefs.Language],
                        Deg->ShortName);
               Lay_ShowAlert (WARNING,Gbl.Message);
              }
            else if (Deg_CheckIfDegreeNameExists (Deg->DegTypCod,"FullName",Deg->FullName,-1L))
              {
               sprintf (Gbl.Message,Txt_The_degree_X_already_exists[Gbl.Prefs.Language],
                        Deg->FullName);
               Lay_ShowAlert (WARNING,Gbl.Message);
              }
            else	// Add new degree to database
               Deg_CreateDegree (Deg);
           }
         else	// If there isn't a degree logo or web
           {
            sprintf (Gbl.Message,"Debe especificar el logo y la direcci&oacute;n web de la nueva titulaci&oacute;n.");	// Need translation!
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
        }
      else	// If there isn't a degree name
        {
         sprintf (Gbl.Message,"Debe especificar el nombre breve y el nombre completo de la nueva titulaci&oacute;n.");	// Need translation!
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else	// Degree type not valid
      Lay_ShowAlert (ERROR,"Tipo de titulaci&oacute;n no v&aacute;lido.");	// Need translation!

   /***** Show the form again *****/
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/**************************** Remove a degree type **************************/
/****************************************************************************/

void Deg_RemoveDegreeType (void)
  {
   extern const char *Txt_Type_of_degree_X_removed[Txt_NUM_LANGUAGES];
   struct DegreeType DegTyp;

   /***** Get the code of the degree type *****/
   if ((DegTyp.DegTypCod = Deg_GetParamOtherDegTypCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of type of degree is missing.");

   /***** Get data of the degree type from database *****/
   if (Deg_GetDataOfDegreeTypeByCod (&DegTyp) == -1L)
      Lay_ShowErrorAndExit ("Code of type of degree not found.");

   /***** Check if this degree type has degrees *****/
   if (DegTyp.NumDegs)	// Degree type has degrees ==> don't remove
      Lay_ShowAlert (WARNING,"Para eliminar un tipo de titulaci&oacute;n" \
                             " primero debe eliminar todas las titulaciones de ese tipo.");	// Need translation!
   else	// Degree type has no degrees ==> remove it
     {
      /***** Remove degree type *****/
      Deg_RemoveDegreeTypeCompletely (DegTyp.DegTypCod);

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_Type_of_degree_X_removed[Gbl.Prefs.Language],
               DegTyp.DegTypName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Deg_ReqEditDegreeTypes ();
  }

/****************************************************************************/
/************************ Request removing of a degree **********************/
/****************************************************************************/

void Deg_RemoveDegree (void)
  {
   extern const char *Txt_Degree_X_removed[Txt_NUM_LANGUAGES];
   struct Degree Deg;

   /***** Get degree code *****/
   if ((Deg.DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /***** Get data of the degree from database *****/
   Deg_GetDataOfDegreeByCod (&Deg);

   /***** Check if this degree has courses *****/
   if (Deg.NumCourses)	// Degree has courses ==> don't remove
      Lay_ShowAlert (WARNING,"Para eliminar una titulaci&oacute;n" \
                             " primero debe eliminar todas las asignaturas de esa titulaci&oacute;n.");	// Need translation!
   else	// Degree has no courses ==> remove it
     {
      /***** Remove degree *****/
      Deg_RemoveDegreeCompletely (Deg.DegCod);

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_Degree_X_removed[Gbl.Prefs.Language],
               Deg.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/***************** Write parameter with code of degree type *****************/
/****************************************************************************/

static void Deg_PutParamOtherDegTypCod (long DegTypCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OthDegTypCod\" value=\"%ld\" />",DegTypCod);
  }

/****************************************************************************/
/******************** Write parameter with code of degree *******************/
/****************************************************************************/

static void Deg_PutParamOtherDegCod (long DegCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OthDegCod\" value=\"%ld\" />",DegCod);
  }

/****************************************************************************/
/******************** Write parameter with code of degree *******************/
/****************************************************************************/

static void Deg_PutParamDegCod (long DegCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"DegCod\" value=\"%ld\" />",DegCod);
  }

/****************************************************************************/
/******************** Write parameter with code of course *******************/
/****************************************************************************/

static void Deg_PutParamOtherCrsCod (long CrsCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OthCrsCod\" value=\"%ld\" />",CrsCod);
  }

/****************************************************************************/
/******************* Get parameter with code of degree type *****************/
/****************************************************************************/

static long Deg_GetParamOtherDegTypCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of degree type *****/
   Par_GetParam (Par_PARAM_SINGLE,"OthDegTypCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/********************* Get parameter with code of degree ********************/
/****************************************************************************/

long Deg_GetParamOtherDegCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of degree *****/
   Par_GetParam (Par_PARAM_SINGLE,"OthDegCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/********************* Get parameter with code of course ********************/
/****************************************************************************/

static long Deg_GetParamOtherCrsCod (void)
  {
   char LongStr[1+10+1];

   /***** Get parameter with code of course *****/
   Par_GetParam (Par_PARAM_SINGLE,"OthCrsCod",LongStr,1+10);
   return Str_ConvertStrCodToLongCod (LongStr);
  }

/****************************************************************************/
/**************** Count number of degrees in a degree type ******************/
/****************************************************************************/

static unsigned Deg_CountNumDegsOfType (long DegTypCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   int NumDegs;

   /***** Get number of degrees of a type from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM degrees WHERE DegTypCod='%ld'",
            DegTypCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get number of degrees of a type");

   /***** Get number of degrees of this type *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumDegs) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of degrees of a type");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumDegs;
  }

/****************************************************************************/
/****************** Get data of a degree type from its code *****************/
/****************************************************************************/
// Returns the code DegTypCod of the degree type, or -1L if not exists (for example, after degree type deletion)

long Deg_GetDataOfDegreeTypeByCod (struct DegreeType *DegTyp)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long DegTypCod = -1L;

   /***** Get the name of a type of degree from database *****/
   sprintf (Query,"SELECT DegTypName FROM deg_types WHERE DegTypCod='%ld'",
            DegTyp->DegTypCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the name of a type of degree");

   if (NumRows > 1)
      Lay_ShowErrorAndExit ("Type of degree repeated in database.");
   else if (NumRows == 1)
     {
      /***** Get data of degree type *****/
      /* Get name of the degree type */
      row = mysql_fetch_row (mysql_res);
      strcpy (DegTyp->DegTypName,row[0]);

      /* Count number of degrees of this type */
      DegTyp->NumDegs = Deg_CountNumDegsOfType (DegTyp->DegTypCod);

      /* Set return value */
      DegTypCod = DegTyp->DegTypCod;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return DegTypCod;
  }

/****************************************************************************/
/********************* Get data of a degree from its code *******************/
/****************************************************************************/
// Returns true if degree found

bool Deg_GetDataOfDegreeByCod (struct Degree *Deg)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool DegFound = false;

   if (Deg->DegCod < 0)
     {
      Deg->DegCod = -1L;
      Deg->InsCod = -1L;
      Deg->PlcCod = -1L;
      Deg->ShortName[0] = '\0';
      Deg->FullName[0] = '\0';
      Deg->FirstYear = 0;
      Deg->LastYear = 0;
      Deg->OptYear = false;
      Deg->Logo[0] = '\0';
      Deg->WWW[0] = '\0';
      Deg->NumCourses = 0;
      Deg->LstCrss = NULL;
      return false;
     }

   /***** Get data of a degree from database *****/
   sprintf (Query,"SELECT DegCod,DegTypCod,InsCod,PlcCod,ShortName,FullName,FirstYear,LastYear,OptYear,Logo,WWW" \
                  " FROM degrees WHERE DegCod ='%ld'",Deg->DegCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a degree");

   if (NumRows == 1)
     {
      /***** Get data of degree *****/
      row = mysql_fetch_row (mysql_res);
      Deg_GetDataOfDegreeFromRow (Deg,row);

      DegFound = true;
     }
   else if (NumRows == 0)
     {
      Deg->DegCod = -1L;
      Deg->InsCod = -1L;
      Deg->PlcCod = -1L;
      Deg->ShortName[0] = '\0';
      Deg->FullName[0] = '\0';
      Deg->FirstYear = 0;
      Deg->LastYear = 0;
      Deg->OptYear = false;
      Deg->Logo[0] = '\0';
      Deg->WWW[0] = '\0';
      Deg->NumCourses = 0;
      Deg->LstCrss = NULL;
      return false;
     }
   else if (NumRows > 1)
      Lay_ShowErrorAndExit ("Degree repeated in database.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return DegFound;
  }

/****************************************************************************/
/********** Get data of a degree from a row resulting of a query ************/
/****************************************************************************/

static void Deg_GetDataOfDegreeFromRow (struct Degree *Deg,MYSQL_ROW row)
  {
   /***** Get degree code (row[0]) *****/
   if ((Deg->DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of degree.");

   /***** Get the code of the degree type (row[1]) *****/
   if ((Deg->DegTypCod = Str_ConvertStrCodToLongCod (row[1])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of type of degree.");

   /***** Get institution code (row[2]) *****/
   if ((Deg->InsCod = Str_ConvertStrCodToLongCod (row[2])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of institution.");

   /***** Get place code (row[3]) *****/
   if ((Deg->PlcCod = Str_ConvertStrCodToLongCod (row[3])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of place.");

   /***** Get degree short name (row[4]) *****/
   strcpy (Deg->ShortName,row[4]);

   /***** Get degree full name (row[5]) *****/
   strcpy (Deg->FullName,row[5]);

   /***** Get first year (row[6]) *****/
   Deg->FirstYear = Deg_ConvStrToYear (row[6]);

   /***** Get last year (row[7]) *****/
   Deg->LastYear = Deg_ConvStrToYear (row[7]);
        
   /***** Get optional year (row[8]) *****/
   Deg->OptYear = (Str_ConvertToUpperLetter (row[8][0]) != 'N');

   /***** Get logo (row[9]) *****/
   strcpy (Deg->Logo,row[9]);

   /***** Get logo (row[10]) *****/
   strcpy (Deg->WWW,row[10]);

   /***** Get number of courses *****/
   Deg->NumCourses = Deg_GetNumCoursesInDeg (Deg->DegCod);
  }

/****************************************************************************/
/******************** Remove a degree type and its degrees ******************/
/****************************************************************************/

static void Deg_RemoveDegreeTypeCompletely (long DegTypCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long DegCod;

   /***** Get degrees of a type from database *****/
   sprintf (Query,"SELECT DegCod FROM degrees WHERE DegTypCod='%ld'",
            DegTypCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees of a type");

   /* Get degrees of this type */
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next degree */
      row = mysql_fetch_row (mysql_res);

      /* Get degree code (row[0]) */
      if ((DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of degree.");

      /* Remove degree */
      Deg_RemoveDegreeCompletely (DegCod);
     }

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   /***** Remove the degree type *****/
   sprintf (Query,"DELETE FROM deg_types WHERE DegTypCod='%ld'",DegTypCod);
   DB_QueryDELETE (Query,"can not remove a type of degree");
  }

/****************************************************************************/
/***************************** Remove a degree ******************************/
/****************************************************************************/

static void Deg_RemoveDegreeCompletely (long DegCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long CrsCod;

   /***** Get courses of a degree from database *****/
   sprintf (Query,"SELECT CrsCod FROM courses WHERE DegCod='%ld'",DegCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get courses of a degree");

   /* Get courses in this degree */
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next course */
      row = mysql_fetch_row (mysql_res);

      /* Get course code (row[0]) */
      if ((CrsCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of course.");

      /* Remove course */
      Deg_RemoveCourseCompletely (CrsCod);
     }

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   /***** Remove all the threads and posts in degree forums *****/
   /* Remove disabled posts */
   sprintf (Query,"DELETE FROM forum_disabled_post USING forum_thread,forum_post,forum_disabled_post" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_post.ThrCod AND forum_post.MsgCod=forum_disabled_post.MsgCod",
            For_FORUM_DEGREE_USRS,For_FORUM_DEGREE_TCHS,DegCod);
   DB_QueryDELETE (Query,"can not remove the disabled posts in forums of a degree");

   /* Remove posts */
   sprintf (Query,"DELETE FROM forum_post USING forum_thread,forum_post" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_post.ThrCod",
            For_FORUM_DEGREE_USRS,For_FORUM_DEGREE_TCHS,DegCod);
   DB_QueryDELETE (Query,"can not remove posts in forums of a degree");

   /* Remove threads read */
   sprintf (Query,"DELETE FROM forum_thr_read USING forum_thread,forum_thr_read" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_thr_read.ThrCod",
            For_FORUM_DEGREE_USRS,For_FORUM_DEGREE_TCHS,DegCod);
   DB_QueryDELETE (Query,"can not remove read threads in forums of a degree");

   /* Remove threads */
   sprintf (Query,"DELETE FROM forum_thread" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND Location='%ld'",
            For_FORUM_DEGREE_USRS,For_FORUM_DEGREE_TCHS,DegCod);
   DB_QueryDELETE (Query,"can not remove threads in forums of a degree");

   /***** Remove administrators of this degree *****/
   sprintf (Query,"DELETE FROM deg_admin WHERE DegCod='%ld'",DegCod);
   DB_QueryDELETE (Query,"can not remove administrators of a degree");

   /***** Remove the degree *****/
   sprintf (Query,"DELETE FROM degrees WHERE DegCod='%ld'",DegCod);
   DB_QueryDELETE (Query,"can not remove a degree");

   /***** Delete all the degrees in sta_degrees table not present in degrees table *****/
   Sta_RemoveObsoleteStatDegrees ();
  }

/****************************************************************************/
/**************************** Rename a degree type **************************/
/****************************************************************************/

void Deg_RenameDegreeType (void)
  {
   extern const char *Txt_The_type_of_degree_X_already_exists[Txt_NUM_LANGUAGES];
   struct DegreeType *DegTyp;
   char Query[1024];
   char NewNameDegTyp[Deg_MAX_LENGTH_DEGREE_TYPE_NAME+1];

   DegTyp = &Gbl.Degs.EditingDegTyp;

   /***** Get parameters from form *****/
   /* Get the code of the degree type */
   if ((DegTyp->DegTypCod = Deg_GetParamOtherDegTypCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of type of degree is missing.");

   /* Get the new name for the degree type */
   Par_GetParam (Par_PARAM_SINGLE,"DegTypName",NewNameDegTyp,Deg_MAX_LENGTH_DEGREE_TYPE_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewNameDegTyp,Deg_MAX_LENGTH_DEGREE_TYPE_NAME,true);

   /***** Get from the database the old name of the degree type *****/
   if (Deg_GetDataOfDegreeTypeByCod (DegTyp) == -1L)
      Lay_ShowErrorAndExit ("Code of type of degree not found.");

   /***** Check if new name is empty *****/
   if (!NewNameDegTyp[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre" \
                           " del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.",	// Need translation!
               DegTyp->DegTypName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (DegTyp->DegTypName,NewNameDegTyp))	// Different names
        {
         /***** If degree type was in database... *****/
         if (Deg_CheckIfDegreeTypeNameExists (NewNameDegTyp,DegTyp->DegTypCod))
           {
            sprintf (Gbl.Message,Txt_The_type_of_degree_X_already_exists[Gbl.Prefs.Language],
                     NewNameDegTyp);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE deg_types SET DegTypName='%s' WHERE DegTypCod='%ld'",
                     NewNameDegTyp,DegTyp->DegTypCod);
            DB_QueryUPDATE (Query,"can not update the type of a degree");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"El tipo de titulaci&oacute;n <strong>%s</strong>" \
                                 " ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                     DegTyp->DegTypName,NewNameDegTyp);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre del tipo de titulaci&oacute;n <strong>%s</strong>" \
                              " no ha cambiado.",	// Need translation!
                  NewNameDegTyp);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (DegTyp->DegTypName,NewNameDegTyp);
   Deg_ReqEditDegreeTypes ();
  }

/****************************************************************************/
/********************* Change the short name of a degree ********************/
/****************************************************************************/

void Deg_RenameDegreeShort (void)
  {
   Deg_RenameDegree (&Gbl.Degs.EditingDeg,Cte_SHORT_NAME);
  }

/****************************************************************************/
/********************* Change the full name of a degree *********************/
/****************************************************************************/

void Deg_RenameDegreeFull (void)
  {
   Deg_RenameDegree (&Gbl.Degs.EditingDeg,Cte_FULL_NAME);
  }

/****************************************************************************/
/************************ Change the name of a degree ***********************/
/****************************************************************************/

static void Deg_RenameDegree (struct Degree *Deg,Cte_ShortOrFullName_t ShortOrFullName)
  {
   extern const char *Txt_The_degree_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned MaxLength = 0;		// Initialized to avoid warning
   char *CurrentDegName = NULL;		// Initialized to avoid warning
   char NewDegName[Deg_MAX_LENGTH_DEGREE_FULL_NAME+1];

   switch (ShortOrFullName)
     {
      case Cte_SHORT_NAME:
         ParamName = "ShortName";
         FieldName = "ShortName";
         MaxLength = Deg_MAX_LENGTH_DEGREE_SHORT_NAME;
         CurrentDegName = Deg->ShortName;
         break;
      case Cte_FULL_NAME:
         ParamName = "FullName";
         FieldName = "FullName";
         MaxLength = Deg_MAX_LENGTH_DEGREE_FULL_NAME;
         CurrentDegName = Deg->FullName;
     }

   /***** Get parameters from form *****/
   /* Get the code of the degree */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get the new name for the degree */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,NewDegName,MaxLength);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewDegName,MaxLength,true);

   /***** Get from the database the type and the old names of the degree *****/
   Deg_GetDataOfDegreeByCod (&Gbl.Degs.EditingDeg);

   /***** Check if new name is empty *****/
   if (!NewDegName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.",	// Need translation!
               CurrentDegName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (CurrentDegName,NewDegName))	// Different names
        {
         /***** If degree was in database... *****/
         if (Deg_CheckIfDegreeNameExists (Deg->DegTypCod,ParamName,NewDegName,Deg->DegCod))
           {
            sprintf (Gbl.Message,Txt_The_degree_X_already_exists[Gbl.Prefs.Language],
                     NewDegName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE degrees SET %s='%s' WHERE DegCod='%ld'",
                     FieldName,NewDegName,Deg->DegCod);
            DB_QueryUPDATE (Query,"can not update the name of a degree");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"La titulaci&oacute;n <strong>%s</strong> ha sido renombrada como <strong>%s</strong>.",	// Need translation!
                     CurrentDegName,NewDegName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// The same name
        {
         sprintf (Gbl.Message,"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.",	// Need translation!
                  CurrentDegName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show the form again *****/
   strcpy (CurrentDegName,NewDegName);
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/****************** Check if name of degree type exists *********************/
/****************************************************************************/

static bool Deg_CheckIfDegreeTypeNameExists (const char *DegTypName,long DegTypCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of degree types with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM deg_types" \
                  " WHERE DegTypName='%s' AND DegTypCod<>'%ld'",
            DegTypName,DegTypCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of a type of degree already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of a type of degree already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/********************* Check if the name of degree exists *******************/
/****************************************************************************/

static bool Deg_CheckIfDegreeNameExists (long DegTypCod,const char *FieldName,const char *Name,long DegCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of degrees with a type and a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM degrees" \
                  " WHERE DegTypCod='%ld' AND %s='%s' AND DegCod<>'%ld'",
            DegTypCod,FieldName,Name,DegCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of a degree already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Internal error while checking if the name of a degree already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/************************ Change the type of a degree ***********************/
/****************************************************************************/

void Deg_ChangeDegreeType (void)
  {
   extern const char *Txt_The_degree_X_already_exists[Txt_NUM_LANGUAGES];
   struct Degree *Deg;
   long NewDegTypCod;
   char Query[512];

   Deg = &Gbl.Degs.EditingDeg;

   /***** Get parameters from form *****/
   /* Get degree code */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get the new degree type */
   NewDegTypCod = Deg_GetParamOtherDegTypCod ();

   /* Get from the database the type and the name of the editing degree */
   Deg_GetDataOfDegreeByCod (Deg);

   /***** If degree was in database... *****/
   if (Deg_CheckIfDegreeNameExists (NewDegTypCod,"ShortName",Deg->ShortName,-1L))
     {
      sprintf (Gbl.Message,Txt_The_degree_X_already_exists[Gbl.Prefs.Language],
               Deg->ShortName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else if (Deg_CheckIfDegreeNameExists (NewDegTypCod,"FullName",Deg->FullName,-1L))
     {
      sprintf (Gbl.Message,Txt_The_degree_X_already_exists[Gbl.Prefs.Language],
               Deg->FullName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /* Update the table of degrees changing old type by new type */
      sprintf (Query,"UPDATE degrees SET DegTypCod='%ld' WHERE DegCod='%ld'",
               NewDegTypCod,Deg->DegCod);
      DB_QueryUPDATE (Query,"can not update the type of a degree");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"Se ha cambiado el tipo de titulaci&oacute;n" \
                           " de la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
               Deg->FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Gbl.Degs.EditingDegTyp.DegTypCod = NewDegTypCod;
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/********************* Change the institution of a degree *******************/
/****************************************************************************/

void Deg_ChangeDegreeIns (void)
  {
   struct Degree *Deg;
   char Query[512];

   Deg = &Gbl.Degs.EditingDeg;

   /***** Get parameters from form *****/
   /* Get degree code */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get parameter with institution code */
   Deg->InsCod = Ins_GetParamInsCod ();

   /***** Update institution in table of degrees *****/
   sprintf (Query,"UPDATE degrees SET InsCod='%ld' WHERE DegCod='%ld'",
            Deg->InsCod,Deg->DegCod);
   DB_QueryUPDATE (Query,"can not update the institution of a degree");

   /***** Write message to show the change made *****/
   Lay_ShowAlert (INFO,"Se ha cambiado la instituci&oacute;n de la titulaci&oacute;n.");	// Need translation!

   /***** Show the form again *****/
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/************************ Change the place of a degree **********************/
/****************************************************************************/

void Deg_ChangeDegreePlace (void)
  {
   struct Degree *Deg;
   char Query[512];

   Deg = &Gbl.Degs.EditingDeg;

   /***** Get parameters from form *****/
   /* Get degree code */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get parameter with place code */
   Deg->PlcCod = Plc_GetParamPlcCod ();

   /***** Update place in table of degrees *****/
   sprintf (Query,"UPDATE degrees SET PlcCod='%ld' WHERE DegCod='%ld'",
            Deg->PlcCod,Deg->DegCod);
   DB_QueryUPDATE (Query,"can not update the place of a degree");

   /***** Write message to show the change made *****/
   Lay_ShowAlert (INFO,"Se ha cambiado el lugar de la titulaci&oacute;n.");	// Need translation!

   /***** Show the form again *****/
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/********************** Change the first year of a degree *******************/
/****************************************************************************/

void Deg_ChangeDegFirstYear (void)
  {
   Deg_ChangeDegYear (&Gbl.Degs.EditingDeg,FIRST_YEAR);
  }

/****************************************************************************/
/*********************** Change the last year of a degree *******************/
/****************************************************************************/

void Deg_ChangeDegLastYear (void)
  {
   Deg_ChangeDegYear (&Gbl.Degs.EditingDeg,LAST_YEAR);
  }

/****************************************************************************/
/******************* Change the first/last year of a degree *****************/
/****************************************************************************/

static void Deg_ChangeDegYear (struct Degree *Deg,tFirstOrLastYear FirstOrLastYear)
  {
   char Query[512];
   char YearStr[1+1];
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned *PtrYear = NULL;		// Initialized to avoid warning

   switch (FirstOrLastYear)
     {
      case FIRST_YEAR:
         ParamName = "FirstYear";
         FieldName = "FirstYear";
         PtrYear = &(Deg->FirstYear);
         break;
      case LAST_YEAR:
         ParamName = "LastYear";
         FieldName = "LastYear";
         PtrYear = &(Deg->LastYear);
     }

   /***** Get parameters from form *****/
   /* Get degree code */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get parameter with first/last year */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,YearStr,1);
   *PtrYear = Deg_ConvStrToYear (YearStr);

   /***** Update first/last year in table of degrees *****/
   sprintf (Query,"UPDATE degrees SET %s='%u' WHERE DegCod='%ld'",FieldName,*PtrYear,Deg->DegCod);
   DB_QueryUPDATE (Query,"can not update the number of years of a degree");

   /***** Write message to show the change made *****/
   Lay_ShowAlert (INFO,"Se han cambiado los a&ntilde;os de la titulaci&oacute;n.");	// Need translation!

   /***** Show the form again *****/
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/******************** Change the optional year of a degree ******************/
/****************************************************************************/

void Deg_ChangeDegOptYear (void)
  {
   struct Degree *Deg;
   char Query[512];
   char YN[1+1];

   Deg = &Gbl.Degs.EditingDeg;

   /***** Get parameters from form *****/
   /* Get degree code */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get parameter with optional year */
   Par_GetParam (Par_PARAM_SINGLE,"OptYear",YN,1);
   Deg->OptYear = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /***** Update optional year in table of degrees *****/
   sprintf (Query,"UPDATE degrees SET OptYear='%c' WHERE DegCod='%ld'",
            Deg->OptYear ? 'Y' : 'N',Deg->DegCod);
   DB_QueryUPDATE (Query,"can not update the optional year of a degree");

   /***** Write message to show the change made *****/
   sprintf (Gbl.Message,"La titulaci&oacute;n ahora %s a&ntilde;o de optativas.",	// Need translation!
            Deg->OptYear ? "tiene" : "no tiene");
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Show the form again *****/
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/************************ Change the logo of a degree ***********************/
/****************************************************************************/

void Deg_ChangeDegLogo (void)
  {
   struct Degree *Deg;
   char Query[256+Deg_MAX_LENGTH_DEGREE_LOGO];
   char NewLogo[Deg_MAX_LENGTH_DEGREE_LOGO+1];

   Deg = &Gbl.Degs.EditingDeg;

   /***** Get parameters from form *****/
   /* Get the code of the degree */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get the new logo for the degree */
   Par_GetParam (Par_PARAM_SINGLE,"Logo",NewLogo,Deg_MAX_LENGTH_DEGREE_LOGO);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewLogo,Deg_MAX_LENGTH_DEGREE_LOGO,true);

   /***** Check if new logo is empty *****/
   if (NewLogo[0])
     {
      /* Update the table changing old logo by new logo */
      sprintf (Query,"UPDATE degrees SET Logo='%s' WHERE DegCod='%ld'",
               NewLogo,Deg->DegCod);
      DB_QueryUPDATE (Query,"can not update the logo of a degree");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"El nuevo logo es <strong>%s</strong>.",	// Need translation!
               NewLogo);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar el logo vac&iacute;o.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Deg->Logo,NewLogo);
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/************************* Change the WWW of a degree ***********************/
/****************************************************************************/

void Deg_ChangeDegWWW (void)
  {
   struct Degree *Deg;
   char Query[256+Cte_MAX_LENGTH_WWW];
   char NewWWW[Cte_MAX_LENGTH_WWW+1];

   Deg = &Gbl.Degs.EditingDeg;

   /***** Get parameters from form *****/
   /* Get the code of the degree */
   if ((Deg->DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Get the new WWW for the degree */
   Par_GetParam (Par_PARAM_SINGLE,"WWW",NewWWW,Cte_MAX_LENGTH_WWW);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewWWW,Cte_MAX_LENGTH_WWW,true);

   /***** Check if new WWW is empty *****/
   if (NewWWW[0])
     {
      /* Update the table changing old WWW by new WWW */
      sprintf (Query,"UPDATE degrees SET WWW='%s' WHERE DegCod='%ld'",
               NewWWW,Deg->DegCod);
      DB_QueryUPDATE (Query,"can not update the web of a degree");

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,"La nueva direcci&oacute;n web es <strong>%s</strong>.",	// Need translation!
               NewWWW);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No puede dejar la direcci&oacute;n web vac&iacute;a.");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   strcpy (Deg->WWW,NewWWW);
   Deg_ReqEditDegrees ();
  }

/****************************************************************************/
/*********************** Get total number of degrees ************************/
/****************************************************************************/

unsigned Deg_GetTotalNumberOfDegrees (void)
  {
   char Query[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumDegs;

   /***** Get total number of degrees from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM degrees");
   DB_QuerySELECT (Query,&mysql_res,"can not get the total number of degrees");

   /***** Get total number of degrees *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumDegs) != 1)
      NumDegs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumDegs;
  }

/****************************************************************************/
/****************** Get number of degrees in a institution ******************/
/****************************************************************************/

unsigned Deg_GetNumberOfDegreesInInstitution (long InsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumDegs;

   /***** Get number of degrees of an institution from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM degrees WHERE InsCod='%ld'",InsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of degrees of an institution");

   /***** Get number of degrees *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumDegs) != 1)
      NumDegs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumDegs;
  }

/****************************************************************************/
/******************** Get number of degrees in a place **********************/
/****************************************************************************/

unsigned Deg_GetNumberOfDegreesInPlace (long PlcCod)
  {
   char Query[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumDegs;

   /***** Get number of degrees of a place from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM degrees WHERE PlcCod='%ld'",PlcCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of degrees of a place");

   /***** Get number of degrees *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumDegs) != 1)
      NumDegs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumDegs;
  }

/****************************************************************************/
/*********************** Get total number of courses ************************/
/****************************************************************************/

unsigned Deg_GetTotalNumberOfCourses (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumCourses;

   /***** Get total number of courses  from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM courses");
   DB_QuerySELECT (Query,&mysql_res,"can not get the total number of courses");

   /* Get number of courses */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumCourses) != 1)
      Lay_ShowErrorAndExit ("Error when getting the total number of courses.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   return NumCourses;
  }

/****************************************************************************/
/******************** Get number of courses in a degree *********************/
/****************************************************************************/

unsigned Deg_GetNumCoursesInDeg (long DegCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   int NumCrss;

   /***** Get number of courses in a degree from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM courses WHERE DegCod='%ld'",DegCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of courses in a degree");

   /***** Get number of degrees of this type *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumCrss) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of courses in a degree.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumCrss;
  }

/****************************************************************************/
/************************* Request edition of courses ***********************/
/****************************************************************************/

void Deg_ReqEditCourses (void)
  {
   if (Gbl.CurrentDeg.Deg.DegCod >= 0)
     {
      /***** Get list of courses in this degree *****/
      Deg_GetListCoursesInDegree (&Gbl.CurrentDeg.Deg);

      /***** Get list of degrees administrated by me *****/
      Deg_GetListDegsAdminByMe ();

      /***** Put form to edit courses *****/
      Deg_EditCourses (&Gbl.CurrentDeg.Deg);

      /***** Free list of courses in this degree *****/
      Deg_FreeListCoursesInDegree (&Gbl.CurrentDeg.Deg);

      /***** Free list of degrees administrated by me *****/
      Deg_FreeListMyAdminDegs ();
     }
   else
      Lay_ShowAlert (WARNING,"Debe seleccionar una titulaci&oacute;n.");	// Need translation!
  }

/****************************************************************************/
/**************** Create a list with courses in this degree *****************/
/****************************************************************************/

static void Deg_GetListCoursesInDegree (struct Degree *Deg)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumCrs;
   struct Course *Crs;

   /***** Get courses of a degree from database *****/
   sprintf (Query,"SELECT CrsCod,DegCod,Year,Semester,ShortName,FullName" \
                  " FROM courses WHERE DegCod='%ld' ORDER BY Year,ShortName",Deg->DegCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the courses of a degree");

   if (NumRows) // Courses found...
     {
      // NumRows should be equal to Deg->NumCourses
      Deg->NumCourses = (unsigned) NumRows;

      /***** Create list with courses in degree *****/
      if ((Deg->LstCrss = (struct Course *) calloc (NumRows,sizeof (struct Course))) == NULL)
          Lay_ShowErrorAndExit ("Not enough memory to store the courses of a degree.");

      /***** Get the courses in degree *****/
      for (NumCrs=0; NumCrs<Deg->NumCourses; NumCrs++)
        {
         Crs = &(Deg->LstCrss[NumCrs]);

         /* Get next course */
         row = mysql_fetch_row (mysql_res);
         Deg_GetDataOfCourseFromRow (Crs,row);
        }
     }
   else
      Deg->NumCourses = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********************* Free list of courses in this degree *******************/
/*****************************************************************************/

void Deg_FreeListCoursesInDegree (struct Degree *Deg)
  {
   if (Deg->LstCrss)
     {
      /***** Free memory used by the list of courses in degree *****/
      free ((void *) Deg->LstCrss);
      Deg->LstCrss = NULL;
      Deg->NumCourses = 0;
     }
  }

/****************************************************************************/
/****************** Put forms to edit courses in this degree ****************/
/****************************************************************************/

static void Deg_EditCourses (struct Degree *Deg)
  {
   extern const char *Txt_Courses[Txt_NUM_LANGUAGES];
   char StrCourses[256];

   /***** Help message *****/
   if (Deg->NumCourses) // If there are courses...
     {
      Lay_ShowAlert (INFO,"Para a&ntilde;adir una asignatura nueva en esta titulaci&oacute;n," \
                          " rellene los nombres breve y completo de la asignatura y el resto de los campos," \
                          " y pulse el icono verde.<br />" \
                          "Para eliminar una asignatura existente pulse el aspa roja (no puede eliminar asignaturas con usuarios).<br />" \
                          "Para modificar una asignatura existente," \
                          " ed&iacute;te las casillas correspondientes" \
                          " y despu&eacute;s pulse con el rat&oacute;n fuera de ellas o pulse INTRO.");	// Need translation!
      sprintf (StrCourses,"%s:",Txt_Courses[Gbl.Prefs.Language]);
      Lay_WriteTitle (StrCourses);
     }
   else	// There aren't courses
      Lay_ShowAlert (INFO,"No existen asignaturas en esta titulaci&oacute;n.<br />" \
                          "Para a&ntilde;adir una asignatura" \
                          " rellene los datos de la asignatura y pulse el icono verde.");	// Need translation!

   /***** Forms to edit courses *****/
   /* Start table with current courses */
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /* List the current courses */
   if (Deg->NumCourses)
      Deg_ListCoursesForEdition (Deg);

   /* Put a form to create a new course */
   Deg_PutFormToCreateCourse (Deg);

   /* End of table with current courses */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/********************* List current courses for edition *********************/
/****************************************************************************/

static void Deg_ListCoursesForEdition (struct Degree *Deg)
  {
   extern const char *Txt_Remove_course[Txt_NUM_LANGUAGES];
   struct Course *Crs;
   unsigned Year,YearAux,Semester;
   unsigned NumDeg;
   unsigned NumCrs;

   /***** Write heading *****/
   Deg_PutHeadCourses ();

   /***** List the courses *****/
   for (Year=0; Year<=Deg_MAX_YEARS_PER_DEGREE; Year++)
      for (NumCrs=0; NumCrs<Deg->NumCourses; NumCrs++)
        {
         Crs = &(Deg->LstCrss[NumCrs]);
         if (Crs->Year == Year)
           {
            /* Put icon to remove course */
            fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
            if (Crs->NumUsrs)	// Course has users ==> deletion forbidden
               fprintf (Gbl.F.Out,"<img src=\"%s/deloff12x12.gif\" width=\"12\" height=\"12\" alt=\"\" />",
                        Gbl.Prefs.IconsURL);
            else
              {
               Act_FormStart (ActRemCrs);
               Deg_PutParamOtherCrsCod (Crs->CrsCod);
               sprintf (Gbl.MsgWindowStatus,"Eliminar asignatura");
               Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_course[Gbl.Prefs.Language],NULL);
               fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form>",
                        Gbl.Prefs.IconsURL,Txt_Remove_course[Gbl.Prefs.Language]);
              }

            /* Course code */
            fprintf (Gbl.F.Out,"</td><td align=\"center\" valign=\"middle\" class=\"DAT_SMALL\">%ld</td>",Crs->CrsCod);

            /* Institutional code of the course */
            fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" class=\"DAT_SMALL\">" \
                               "<input type=\"text\" name=\"InstitutionalCrsCod\" size=\"10\" maxlength=\"%u\" value=\"%s\" disabled=\"disabled\" />" \
                               "</td>",
                     Deg_MAX_LENGTH_INSTITUTIONAL_CRS_COD,Crs->InstitutionalCrsCod);

            /* Degree */
            fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
            Act_FormStart (ActChgCrsDeg);
            Deg_PutParamOtherCrsCod (Crs->CrsCod);
            fprintf (Gbl.F.Out,"<select name=\"OthDegCod\" style=\"width:80px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
            for (NumDeg=0; NumDeg<Gbl.Usrs.Me.MyAdminDegs.Num; NumDeg++)
               fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s</option>",
                        Gbl.Usrs.Me.MyAdminDegs.Lst[NumDeg].DegCod,
                        (Gbl.Usrs.Me.MyAdminDegs.Lst[NumDeg].DegCod == Deg->DegCod) ? " selected=\"selected\"" : "",
                        Gbl.Usrs.Me.MyAdminDegs.Lst[NumDeg].ShortName);
            fprintf (Gbl.F.Out,"</select></form></td>");

            /* Course year */
            fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
            Act_FormStart (ActChgCrsYea);
            Deg_PutParamOtherCrsCod (Crs->CrsCod);
            fprintf (Gbl.F.Out,"<select name=\"OthCrsYear\" style=\"width:40px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
            for (YearAux=0; YearAux<=Deg_MAX_YEARS_PER_DEGREE; YearAux++)	// All the years are permitted because it's possible to move this course to another degree (with other active years)
               fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%s</option>",
                        YearAux,(YearAux == Crs->Year) ? " selected=\"selected\"" : "",
                        YearStrs[YearAux]);
            fprintf (Gbl.F.Out,"</select></form></td>");

            /* Course semester */
            fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
            Act_FormStart (ActChgCrsSem);
            Deg_PutParamOtherCrsCod (Crs->CrsCod);
            fprintf (Gbl.F.Out,"<select name=\"OthCrsSem\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
            for (Semester=0; Semester<=2; Semester++)
              {
               fprintf (Gbl.F.Out,"<option value=\"%u\"",Semester);
               if (Semester == Crs->Semester)
                  fprintf (Gbl.F.Out," selected=\"selected\"");
               fprintf (Gbl.F.Out,">%s</option>",SemesterStrs[Semester]);
              }
            fprintf (Gbl.F.Out,"</select></form></td>");

            /* Course short name */
            fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
            Act_FormStart (ActRenCrsSho);
            Deg_PutParamOtherCrsCod (Crs->CrsCod);
            fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ShortName\" size=\"14\" maxlength=\"%u\" value=\"%s\"" \
                           " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
                     Deg_MAX_LENGTH_COURSE_SHORT_NAME,Crs->ShortName,Gbl.NumForm);

            /* Course full name */
            fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">");
            Act_FormStart (ActRenCrsFul);
            Deg_PutParamOtherCrsCod (Crs->CrsCod);
            fprintf (Gbl.F.Out,"<input type=\"text\" name=\"FullName\" size=\"38\" maxlength=\"%u\" value=\"%s\"" \
                           " onchange=\"javascript:document.forms[%d].submit();\" /></form></td>",
                     Deg_MAX_LENGTH_COURSE_FULL_NAME,Crs->FullName,Gbl.NumForm);

            /* Current number of students in this course */
            fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"middle\" class=\"DAT_SMALL\">%u</td>\n",
                     Crs->NumStds);

            /* Current number of teachers in this course */
            fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"middle\" class=\"DAT_SMALL\">%u</td></tr>",
                     Crs->NumTchs);
           }
        }
  }

/****************************************************************************/
/*********************** Put a form to create a new course ******************/
/****************************************************************************/

static void Deg_PutFormToCreateCourse (struct Degree *Deg)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_Create_course[Txt_NUM_LANGUAGES];
   struct Course *Crs;
   unsigned Year;
   unsigned Semester;

   Crs = &Gbl.Degs.EditingCrs;

   /***** Message *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"10\" class=\"%s\">%s:</td></tr>" \
                      "<tr><td align=\"left\" colspan=\"10\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_New_course[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActNewCrs);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   Deg_PutHeadCourses ();

   /***** New course *****/
   fprintf (Gbl.F.Out,"<tr><td class=\"BM\">");
   sprintf (Gbl.MsgWindowStatus,Txt_Create_course[Gbl.Prefs.Language]);
   Lay_PutButtonCreate ();
   fprintf (Gbl.F.Out,"</td>");

   /***** Course code *****/
   fprintf (Gbl.F.Out,"<td></td>");

   /***** Institutional code of the course *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"InstitutionalCrsCod\" size=\"10\" maxlength=\"%u\" value=\"%s\" disabled=\"disabled\" />" \
                      "</td>",
            Deg_MAX_LENGTH_INSTITUTIONAL_CRS_COD,Crs->InstitutionalCrsCod);

   /***** Degree *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<select name=\"OthDegCod\" style=\"width:80px;\" disabled=\"disabled\">" \
                      "<option value=\"%ld\">%s</option></select></td>",
            Deg->DegCod,Deg->ShortName);

   /***** Year *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><select name=\"OthCrsYear\" style=\"width:40px;\">");
   for (Year=0; Year<=Deg->LastYear; Year++)
      if (Deg_CheckIfYearIsValidInDeg (Year,Deg))
         fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%s</option>",
                  Year,
                  (Year == Crs->Year) ? " selected=\"selected\"" : "",
                  YearStrs[Year]);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Semester *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\"><select name=\"OthCrsSem\">");
   for (Semester=0; Semester<=2; Semester++)
      fprintf (Gbl.F.Out,"<option value=\"%u\"%s>%s</option>",
               Semester,(Semester == Crs->Semester) ? " selected=\"selected\"" : "",SemesterStrs[Semester]);
   fprintf (Gbl.F.Out,"</select></td>");

   /***** Course short name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                  "<input type=\"text\" name=\"ShortName\" size=\"14\" maxlength=\"%u\" value=\"%s\" />" \
                  "</td>",
            Deg_MAX_LENGTH_COURSE_SHORT_NAME,Crs->ShortName);

   /***** Course full name *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"FullName\" size=\"38\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td><td></td><td></td></tr></table></form></td></tr>\n",
            Deg_MAX_LENGTH_COURSE_FULL_NAME,Crs->FullName);
  }

/****************************************************************************/
/******************** Write header with fields of a course ******************/
/****************************************************************************/

static void Deg_PutHeadCourses (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Institutional_course_code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Year[Txt_NUM_LANGUAGES];
   extern const char *Txt_Semester[Txt_NUM_LANGUAGES];
   extern const char *Txt_Short_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Full_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_Teach[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td class=\"BM\"></td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Institutional_course_code[Gbl.Prefs.Language],
            Txt_Degree[Gbl.Prefs.Language],
            Txt_Year[Gbl.Prefs.Language],
            Txt_Semester[Gbl.Prefs.Language],
            Txt_Short_Name[Gbl.Prefs.Language],
            Txt_Full_Name[Gbl.Prefs.Language],
            Txt_Stud[Gbl.Prefs.Language],
            Txt_Teach[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/******************* Receive form to create a new course ********************/
/****************************************************************************/

void Deg_RecFormNewCrs (void)
  {
   extern const char *Txt_The_course_X_already_exists[Txt_NUM_LANGUAGES];
   struct Course *Crs;
   struct Degree Deg;
   char YearStr[1+1];
   char SemesterStr[1+1];

   Crs = &Gbl.Degs.EditingCrs;

   /***** Get degree code from form *****/
   if ((Deg.DegCod = Crs->DegCod = Gbl.CurrentDeg.Deg.DegCod) >= 0) // Current degree valid (this is the normal behaviour)
     {
      /***** Get parameters of the course from form *****/
      /* Get year */
      Par_GetParam (Par_PARAM_SINGLE,"OthCrsYear",YearStr,1);
      Crs->Year = Deg_ConvStrToYear (YearStr);

      /* Get semester */
      Par_GetParam (Par_PARAM_SINGLE,"OthCrsSem",SemesterStr,1);
      Crs->Semester = Deg_ConvStrToSemester (SemesterStr);

      /* Get course short name */
      Par_GetParam (Par_PARAM_SINGLE,"ShortName",Crs->ShortName,Deg_MAX_LENGTH_COURSE_SHORT_NAME);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Crs->ShortName,Deg_MAX_LENGTH_COURSE_SHORT_NAME,true);

      /* Get course full name */
      Par_GetParam (Par_PARAM_SINGLE,"FullName",Crs->FullName,Deg_MAX_LENGTH_COURSE_FULL_NAME);
      Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Crs->FullName,Deg_MAX_LENGTH_COURSE_FULL_NAME,true);

      /***** Check if year is correct *****/
      Deg_GetDataOfDegreeByCod (&Deg);
      if (Deg_CheckIfYearIsValidInDeg (Crs->Year,&Deg))	// If year is valid
        { 
         if (Crs->ShortName[0] && Crs->FullName[0])	// If there's a course name
           {
            /***** If name of course was in database... *****/
            if (Deg_CheckIfCourseNameExists (Crs->DegCod,Crs->Year,"ShortName",Crs->ShortName,-1L))
              {
               sprintf (Gbl.Message,Txt_The_course_X_already_exists[Gbl.Prefs.Language],
                        Crs->ShortName);
               Lay_ShowAlert (WARNING,Gbl.Message);
              }
            else if (Deg_CheckIfCourseNameExists (Crs->DegCod,Crs->Year,"FullName",Crs->FullName,-1L))
              {
               sprintf (Gbl.Message,Txt_The_course_X_already_exists[Gbl.Prefs.Language],
                        Crs->FullName);
               Lay_ShowAlert (WARNING,Gbl.Message);
              }
            else	// Add new course to database
               Deg_CreateCourse (Crs);
           }
         else	// If there isn't a course name
           {
            sprintf (Gbl.Message,"Debe especificar el nombre breve" \
                                 " y el nombre completo de la nueva asignatura.");	// Need translation!
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
        }
      else	// Year not valid
        {
         sprintf (Gbl.Message,"La titulaci&oacute;n <strong>%s</strong>" \
                              " no tiene habilitado el curso <strong>%s</strong>.",	// Need translation!
                  Deg.FullName,Deg_StrsDegYear[Crs->Year]);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else	// Current degree not valid! This never should happen
      Lay_ShowErrorAndExit ("Wrong code of degree.");

   /***** Show the form again *****/
   Deg_ReqEditCourses ();
  }

/****************************************************************************/
/********************* Check if the name of course exists *******************/
/****************************************************************************/

static bool Deg_CheckIfCourseNameExists (long DegCod,unsigned Year,const char *FieldName,const char *Name,long CrsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of courses in a year of a degree and with a name from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM courses" \
                  " WHERE DegCod='%ld' AND Year='%u' AND %s='%s' AND CrsCod<>'%ld'",
            DegCod,Year,FieldName,Name,CrsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if the name of a course already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if the name of a course already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/***************************** Create a new course **************************/
/****************************************************************************/

static void Deg_CreateCourse (struct Course *Crs)
  {
   extern const char *Txt_Created_new_course_X[Txt_NUM_LANGUAGES];
   char Query[2048];

   /***** Create a new course *****/
   sprintf (Query,"INSERT INTO courses (DegCod,Year,Semester,ShortName,FullName)" \
                  " VALUES ('%ld','%u','%u','%s','%s')",
            Crs->DegCod,Crs->Year,Crs->Semester,Crs->ShortName,Crs->FullName);
   DB_QueryINSERT (Query,"can not create a new course");

   /***** Write success message *****/
   sprintf (Gbl.Message,Txt_Created_new_course_X[Gbl.Prefs.Language],
            Crs->FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/************************ Request removing of a course **********************/
/****************************************************************************/

void Deg_RemoveCourse (void)
  {
   extern const char *Txt_Course_X_removed[Txt_NUM_LANGUAGES];
   struct Course Crs;

   /***** Get course code *****/
   if ((Crs.CrsCod = Deg_GetParamOtherCrsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of course is missing.");

   /***** Get data of the course from database *****/
   Deg_GetDataOfCourseByCod (&Crs);

   /***** Check if this course has users *****/
   if (Crs.NumUsrs)	// Course has users ==> don't remove
      Lay_ShowAlert (WARNING,"Para eliminar una asignatura primero debe eliminar" \
                             " todos los usuarios de esa asignatura.");	// Need translation!
   else	// Course has no users ==> remove it
     {
      /***** Remove course *****/
      Deg_RemoveCourseCompletely (Crs.CrsCod);

      /***** Write message to show the change made *****/
      sprintf (Gbl.Message,Txt_Course_X_removed[Gbl.Prefs.Language],
               Crs.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Show the form again *****/
   Deg_ReqEditCourses ();
  }

/****************************************************************************/
/********************* Get data of a course from its code *******************/
/****************************************************************************/

bool Deg_GetDataOfCourseByCod (struct Course *Crs)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool CrsFound = false;

   if (Crs->CrsCod < 0)
     {
      Crs->CrsCod = -1L;
      Crs->DegCod = -1L;
      Crs->Year = 0;
      Crs->Semester = 0;
      Crs->ShortName[0] = '\0';
      Crs->FullName[0] = '\0';
      Crs->NumStds = 0;
      Crs->NumTchs = 0;
      Crs->NumUsrs = 0;
      return false;
     }

   /***** Get data of a course from database *****/
   sprintf (Query,"SELECT CrsCod,DegCod,Year,Semester,ShortName,FullName" \
                  " FROM courses WHERE CrsCod='%ld'",Crs->CrsCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of a course");

   if (NumRows == 1)		// Course found
     {
      /***** Get data of the course *****/
      row = mysql_fetch_row (mysql_res);
      Deg_GetDataOfCourseFromRow (Crs,row);

      CrsFound = true;
     }
   else if (NumRows == 0)	// Course not found
     {
      Crs->CrsCod = -1L;
      Crs->DegCod = -1L;
      Crs->Year = 0;
      Crs->Semester = 0;
      Crs->ShortName[0] = '\0';
      Crs->FullName[0] = '\0';
      Crs->NumStds = 0;
      Crs->NumTchs = 0;
      Crs->NumUsrs = 0;
     }
   else if (NumRows > 1)	// Course duplicated
      Lay_ShowErrorAndExit ("Course is repeated in database.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return CrsFound;
  }

/****************************************************************************/
/********** Get data of a course from a row resulting of a query ************/
/****************************************************************************/

static void Deg_GetDataOfCourseFromRow (struct Course *Crs,MYSQL_ROW row)
  {
   /* Get course code (row[0]) */
   if ((Crs->CrsCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of course.");

   /* Get code of degree (row[1]) */
   Crs->DegCod = Str_ConvertStrCodToLongCod (row[1]);

   /* Get year (row[2]) */
   Crs->Year = Deg_ConvStrToYear (row[2]);

   /* Get semester (row[3]) */
   Crs->Semester = Deg_ConvStrToSemester (row[3]);

   /* Get the short name of the course (row[4]) */
   strcpy (Crs->ShortName,row[4]);

   /* Get the full name of the course (row[5]) */
   strcpy (Crs->FullName,row[5]);

   /* Get number of students */
   Crs->NumStds = Usr_GetNumStdsInCrs (Crs->CrsCod);

   /* Get number of teachers */
   Crs->NumTchs = Usr_GetNumTchsInCrs (Crs->CrsCod);

   Crs->NumUsrs = Crs->NumStds + Crs->NumTchs;
  }

/****************************************************************************/
/***************************** Remove a course ******************************/
/****************************************************************************/

static void Deg_RemoveCourseCompletely (long CrsCod)
  {
   char Query[128];

   /***** Empty course *****/
   Deg_EmptyCourseCompletely (CrsCod);

   /***** Remove course from table of courses in database *****/
   sprintf (Query,"DELETE FROM courses WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove a course");
  }

/****************************************************************************/
/********** Empty a course (remove all its information and users) ***********/
/****************************************************************************/
// Start removing less important things to more important things;
// so, in case of failure, important things can been removed in the future

static void Deg_EmptyCourseCompletely (long CrsCod)
  {
   char PathRelCrs[PATH_MAX+1];
   char Query[512];
   
   /***** Remove information of the course ****/
   /* Remove timetable of the course */
   sprintf (Query,"DELETE FROM timetable_crs WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove the timetable of a course");

   /* Remove other information of the course */
   sprintf (Query,"DELETE FROM crs_info_src WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove info sources of a course");
   
   sprintf (Query,"DELETE FROM crs_info_txt WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove info of a course");

   /***** Remove exam announcements in the course *****/
   /* Mark all exam announcements in the course as deleted */
   sprintf (Query,"UPDATE exam_announcements SET Status='%u' WHERE CrsCod='%ld'",
            (unsigned) Exa_DELETED_EXAM_ANNOUNCEMENT,CrsCod);
   DB_QueryUPDATE (Query,"can not remove exam announcements of a course");

   /***** Remove course cards of the course *****/
   /* Remove content of course cards */
   sprintf (Query,"DELETE FROM crs_records USING crs_record_fields,crs_records" \
                  " WHERE crs_record_fields.CrsCod='%ld' AND crs_record_fields.FieldCod=crs_records.FieldCod",CrsCod);
   DB_QueryDELETE (Query,"can not remove content of cards in a course");

   /* Remove definition of fields in course cards */
   sprintf (Query,"DELETE FROM crs_record_fields WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove fields of cards in a course");

   /***** Remove assignments of the course *****/
   Asg_RemoveCrsAssignments (CrsCod);

   /***** Remove notices in the course *****/
   /* Mark all notices from the course as deleted */
   sprintf (Query,"UPDATE notices SET Status='%u' WHERE CrsCod='%ld'",
            (unsigned) Not_DELETED_NOTICE,CrsCod);
   DB_QueryUPDATE (Query,"can not remove notices in a course");

   /***** Remove all the threads and posts in course forums *****/
   /* Remove disabled posts */
   sprintf (Query,"DELETE FROM forum_disabled_post USING forum_thread,forum_post,forum_disabled_post" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_post.ThrCod AND forum_post.MsgCod=forum_disabled_post.MsgCod",
            For_FORUM_COURSE_USRS,For_FORUM_COURSE_TCHS,CrsCod);
   DB_QueryDELETE (Query,"can not remove disbled posts in forums of a course");

   /* Remove posts */
   sprintf (Query,"DELETE FROM forum_post USING forum_thread,forum_post" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_post.ThrCod",
            For_FORUM_COURSE_USRS,For_FORUM_COURSE_TCHS,CrsCod);
   DB_QueryDELETE (Query,"can not remove posts in forums of a course");

   /* Remove threads read */
   sprintf (Query,"DELETE FROM forum_thr_read USING forum_thread,forum_thr_read" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND forum_thread.Location='%ld' AND forum_thread.ThrCod=forum_thr_read.ThrCod",
            For_FORUM_COURSE_USRS,For_FORUM_COURSE_TCHS,CrsCod);
   DB_QueryDELETE (Query,"can not remove read threads in forums of a course");

   /* Remove threads */
   sprintf (Query,"DELETE FROM forum_thread" \
                  " WHERE (forum_thread.ForumType='%u' OR forum_thread.ForumType='%u') AND Location='%ld'",
            For_FORUM_COURSE_USRS,For_FORUM_COURSE_TCHS,CrsCod);
   DB_QueryDELETE (Query,"can not remove threads in forums of a course");

   /***** Remove tests in the course *****/
   /* Remove tests status in the course */
   sprintf (Query,"DELETE FROM tst_status WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove status of tests of a course");

   /* Remove test configuration of the course */
   sprintf (Query,"DELETE FROM tst_config WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove configuration of tests of a course");

   /* Remove associations between test questions and test tags in the course */
   sprintf (Query,"DELETE FROM tst_question_tags USING tst_questions,tst_question_tags" \
                  " WHERE tst_questions.CrsCod='%ld' AND tst_questions.QstCod=tst_question_tags.QstCod",CrsCod);
   DB_QueryDELETE (Query,"can not remove tags associated to questions of tests of a course");

   /* Remove test tags in the course */
   sprintf (Query,"DELETE FROM tst_tags WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove tags of test of a course");

   /* Remove test answers in the course */
   sprintf (Query,"DELETE FROM tst_answers USING tst_questions,tst_answers" \
                  " WHERE tst_questions.CrsCod='%ld' AND tst_questions.QstCod=tst_answers.QstCod",CrsCod);
   DB_QueryDELETE (Query,"can not remove answers of tests of a course");

   /* Remove test questions in the course */
   sprintf (Query,"DELETE FROM tst_questions WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove test questions of a course");

   /***** Remove groups in the course *****/
   /* Remove all the users in groups in the course */
   sprintf (Query,"DELETE FROM crs_grp_usr USING crs_grp_types,crs_grp,crs_grp_usr" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod AND crs_grp.GrpCod=crs_grp_usr.GrpCod",CrsCod);
   DB_QueryDELETE (Query,"can not remove users from groups of a course");

   /* Remove all the groups in the course */
   sprintf (Query,"DELETE FROM crs_grp USING crs_grp_types,crs_grp" \
                  " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod",CrsCod);
   DB_QueryDELETE (Query,"can not remove groups of a course");

   /* Remove all the group types in the course */
   sprintf (Query,"DELETE FROM crs_grp_types WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove types of group of a course");

   /***** Remove users in the course (really the course should have no users when removing it) *****/
   sprintf (Query,"DELETE FROM crs_usr WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove users from a course");

   /***** Remove information related to files in course *****/
   /* Remove clipboards related to the course */
   sprintf (Query,"DELETE FROM clipboard WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove clipboards in a course");

   /* Remove expanded folders in the course */
   sprintf (Query,"DELETE FROM expanded_folders WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove expanded folders in a course");

   /* Remove hidden downloads in the course */
   sprintf (Query,"DELETE FROM hidden_downloads WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove hidden downloads in a course");

   /* Remove owners of common files of the course */
   sprintf (Query,"DELETE FROM crs_common_files WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove owners of files in common zones of a course");

   /* Remove format of files of marks of the course */
   sprintf (Query,"DELETE FROM marks WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove files of marks associated to a course");

   /***** Remove directories of the course *****/
   sprintf (PathRelCrs,"%s/%ld",Cfg_PATH_REL_CRS_PRIVATE,CrsCod);
   Brw_RemoveTree (PathRelCrs);
   sprintf (PathRelCrs,"%s/%ld",Cfg_PATH_REL_CRS_PUBLIC,CrsCod);
   Brw_RemoveTree (PathRelCrs);

   /* Remove size of file zones in the course from database */
   Brw_RemoveSizeOfFileTreeFromDB (CrsCod,-1L,-1L);
  }

/****************************************************************************/
/****** Change the degree of a course (move course to another degree) *******/
/****************************************************************************/

void Deg_ChangeCrsDegree (void)
  {
   struct Course *Crs;
   bool ICanChangeCrsToNewDeg;
   char Query[512];
   struct Degree NewDeg;

   Crs = &Gbl.Degs.EditingCrs;

   /***** Get parameters from form *****/
   /* Get course code */
   if ((Crs->CrsCod = Deg_GetParamOtherCrsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of course is missing.");

   /* Get new degree code */
   if ((NewDeg.DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /* Check if I have permission to change course to this degree */
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
      ICanChangeCrsToNewDeg = true;
   else	// Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN
      ICanChangeCrsToNewDeg = Usr_CheckIfUsrIsAdmOfDeg (Gbl.Usrs.Me.UsrDat.UsrCod,NewDeg.DegCod);

   /***** Get data of course and new degree *****/
   Deg_GetDataOfCourseByCod (Crs);
   Deg_GetDataOfDegreeByCod (&NewDeg);

   /***** If I have permission to change course to this new degree... *****/
   if (ICanChangeCrsToNewDeg)
     {
      /***** If new degree has current course year... *****/
      if (Deg_CheckIfYearIsValidInDeg (Crs->Year,&NewDeg))
        {
         /***** If name of course was in database in the new degree... *****/
         if (Deg_CheckIfCourseNameExists (NewDeg.DegCod,Crs->Year,"ShortName",Crs->ShortName,-1L))
           {
            sprintf (Gbl.Message,"En el curso %s de la titulaci&oacute;n <strong>%s</strong>" \
                                 " ya exista una asignatura con el nombre <strong>%s</strong>.",	// Need translation!
                     YearStrs[Crs->Year],NewDeg.FullName,Crs->ShortName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else if (Deg_CheckIfCourseNameExists (NewDeg.DegCod,Crs->Year,"FullName",Crs->FullName,-1L))
           {
            sprintf (Gbl.Message,"En el curso %s de la titulaci&oacute;n <strong>%s</strong>" \
                                 " ya exista una asignatura con el nombre <strong>%s</strong>.",	// Need translation!
                     YearStrs[Crs->Year],NewDeg.FullName,Crs->FullName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
         else	// Update degree in database
           {
            /***** Update degree in table of courses *****/
            sprintf (Query,"UPDATE courses SET DegCod='%ld' WHERE CrsCod='%ld'",
                     NewDeg.DegCod,Crs->CrsCod);
            DB_QueryUPDATE (Query,"can not move course to another degree");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"La asignatura <strong>%s</strong>" \
                                 " se ha movido a la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
                     Crs->FullName,NewDeg.FullName);
            Lay_ShowAlert (INFO,Gbl.Message);
           }
        }
      else	// New degree has no current course year
        {
         sprintf (Gbl.Message,"El curso %s no est&aacute; disponible" \
                              " en la titulaci&oacute;n <strong>%s</strong>",	// Need translation!
                  YearStrs[Crs->Year],NewDeg.FullName);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else	// I have no permission to change course to this new degree
     {
      sprintf (Gbl.Message,"Usted no tiene permiso para mover asignaturas" \
                           " a la titulaci&oacute;n <strong>%s</strong>",	// Need translation!
               NewDeg.FullName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Show the form again *****/
   Deg_ReqEditCourses ();
  }

/****************************************************************************/
/************************ Change the year of a course ***********************/
/****************************************************************************/

void Deg_ChangeCrsYear (void)
  {
   extern const char *Txt_The_course_X_already_exists_in_year_Y[Txt_NUM_LANGUAGES];
   struct Course *Crs;
   struct Degree Deg;
   char Query[512];
   char YearStr[1+1];
   unsigned NewYear;

   Crs = &Gbl.Degs.EditingCrs;

   /***** Get parameters from form *****/
   /* Get course code */
   if ((Crs->CrsCod = Deg_GetParamOtherCrsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of course is missing.");

   /* Get parameter with year */
   Par_GetParam (Par_PARAM_SINGLE,"OthCrsYear",YearStr,1);
   NewYear = Deg_ConvStrToYear (YearStr);

   Deg_GetDataOfCourseByCod (Crs);
   Deg.DegCod = Crs->DegCod;
   Deg_GetDataOfDegreeByCod (&Deg);

   if (Deg_CheckIfYearIsValidInDeg (Crs->Year,&Deg))	// If year is valid
     { 
      /***** If name of course was in database in the new year... *****/
      if (Deg_CheckIfCourseNameExists (Crs->DegCod,NewYear,"ShortName",Crs->ShortName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_course_X_already_exists_in_year_Y[Gbl.Prefs.Language],
                  Crs->ShortName,YearStrs[NewYear]);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else if (Deg_CheckIfCourseNameExists (Crs->DegCod,NewYear,"FullName",Crs->FullName,-1L))
        {
         sprintf (Gbl.Message,Txt_The_course_X_already_exists_in_year_Y[Gbl.Prefs.Language],
                  Crs->FullName,YearStrs[NewYear]);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else	// Update year in database
        {
         /***** Update year in table of courses *****/
         sprintf (Query,"UPDATE courses SET Year='%u' WHERE CrsCod='%ld'",NewYear,Crs->CrsCod);
         DB_QueryUPDATE (Query,"can not update the year of a course");

         Crs->Year = NewYear;

         /***** Write message to show the change made *****/
         Lay_ShowAlert (INFO,"Se ha cambiado el curso de la asignatura.");	// Need translation!
        }
     }
   else	// Year not valid
     {
      sprintf (Gbl.Message,"La titulaci&oacute;n <strong>%s</strong>" \
                           " no tiene habilitado el curso <strong>%s</strong>.",	// Need translation!
               Deg.FullName,Deg_StrsDegYear[Crs->Year]);
      Lay_ShowAlert (ERROR,Gbl.Message);
     }

   /***** Show the form again *****/
   Deg_ReqEditCourses ();
  }

/****************************************************************************/
/******************* Check if a year is valid in a degree *******************/
/****************************************************************************/

static bool Deg_CheckIfYearIsValidInDeg (unsigned Year,struct Degree *Deg)
  {
   return (Year == 0 && Deg->OptYear) || (Year != 0 && Year >= Deg->FirstYear && Year <= Deg->LastYear);
  }

/****************************************************************************/
/********************** Change the semester of a course *********************/
/****************************************************************************/

void Deg_ChangeCrsSemester (void)
  {
   struct Course *Crs;
   char Query[512];
   char SemesterStr[1+1];
   unsigned NewSemester;

   Crs = &Gbl.Degs.EditingCrs;

   /***** Get parameters from form *****/
   /* Get course code */
   if ((Crs->CrsCod = Deg_GetParamOtherCrsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of course is missing.");

   /* Get parameter with semester */
   Par_GetParam (Par_PARAM_SINGLE,"OthCrsSem",SemesterStr,1);
   NewSemester = Deg_ConvStrToSemester (SemesterStr);

   /***** Update semester in table of courses *****/
   sprintf (Query,"UPDATE courses SET Semester='%u' WHERE CrsCod='%ld'",
            NewSemester,Crs->CrsCod);
   DB_QueryUPDATE (Query,"can not update the semester of a course");

   Crs->Semester = NewSemester;

   /***** Write message to show the change made *****/
   Lay_ShowAlert (INFO,"Se ha cambiado el cuatrimestre de la asignatura.");	// Need translation!

   /***** Show the form again *****/
   Deg_ReqEditCourses ();
  }

/****************************************************************************/
/********************* Change the short name of a course ********************/
/****************************************************************************/

void Deg_RenameCourseShort (void)
  {
   struct Course *Crs;

   Crs = &Gbl.Degs.EditingCrs;

   Deg_RenameCourse (Crs,Cte_SHORT_NAME);

   if (Crs->CrsCod == Gbl.CurrentCrs.Crs.CrsCod)	// If renaming current course...
      strcpy (Gbl.CurrentCrs.Crs.ShortName,Crs->ShortName);	// Overwrite current course name in order to show correctly in page title and heading
  }

/****************************************************************************/
/********************* Change the full name of a course *********************/
/****************************************************************************/

void Deg_RenameCourseFull (void)
  {
   struct Course *Crs;

   Crs = &Gbl.Degs.EditingCrs;

   Deg_RenameCourse (Crs,Cte_FULL_NAME);

   if (Crs->CrsCod == Gbl.CurrentCrs.Crs.CrsCod)	// If renaming current course...
      strcpy (Gbl.CurrentCrs.Crs.FullName,Crs->FullName);	// Overwrite current course name in order to show correctly in page title and heading
  }

/****************************************************************************/
/************************ Change the name of a course ***********************/
/****************************************************************************/

static void Deg_RenameCourse (struct Course *Crs,Cte_ShortOrFullName_t ShortOrFullName)
  {
   extern const char *Txt_The_course_X_already_exists[Txt_NUM_LANGUAGES];
   char Query[512];
   const char *ParamName = NULL;	// Initialized to avoid warning
   const char *FieldName = NULL;	// Initialized to avoid warning
   unsigned MaxLength = 0;		// Initialized to avoid warning
   char *CurrentCrsName = NULL;		// Initialized to avoid warning
   char NewCrsName[Deg_MAX_LENGTH_COURSE_FULL_NAME+1];

   switch (ShortOrFullName)
     {
      case Cte_SHORT_NAME:
         ParamName = "ShortName";
         FieldName = "ShortName";
         MaxLength = Deg_MAX_LENGTH_COURSE_SHORT_NAME;
         CurrentCrsName = Crs->ShortName;
         break;
      case Cte_FULL_NAME:
         ParamName = "FullName";
         FieldName = "FullName";
         MaxLength = Deg_MAX_LENGTH_COURSE_FULL_NAME;
         CurrentCrsName = Crs->FullName;
     }

   /***** Get parameters from form *****/
   /* Get the code of the course */
   if ((Crs->CrsCod = Deg_GetParamOtherCrsCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of course is missing.");

   /* Get the new name for the course */
   Par_GetParam (Par_PARAM_SINGLE,ParamName,NewCrsName,MaxLength);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewCrsName,MaxLength,true);

   /***** Get from the database the data of the degree *****/
   Deg_GetDataOfCourseByCod (Crs);

   /***** Check if new name is empty *****/
   if (!NewCrsName[0])
     {
      sprintf (Gbl.Message,"No puede dejar el nombre de la asignatura <strong>%s</strong> vac&iacute;o.",	// Need translation!
               CurrentCrsName);
      Gbl.Error = true;
     }
   else
     {
      /***** Check if old and new names are the same (this happens when user press enter with no changes in the form) *****/
      if (strcmp (CurrentCrsName,NewCrsName))	// Different names
        {
         /***** If course was in database... *****/
         if (Deg_CheckIfCourseNameExists (Crs->DegCod,Crs->Year,ParamName,NewCrsName,Crs->CrsCod))
           {
            sprintf (Gbl.Message,Txt_The_course_X_already_exists[Gbl.Prefs.Language],
                     NewCrsName);
            Gbl.Error = true;
           }
         else
           {
            /* Update the table changing old name by new name */
            sprintf (Query,"UPDATE courses SET %s='%s' WHERE CrsCod='%ld'",
                     FieldName,NewCrsName,Crs->CrsCod);
            DB_QueryUPDATE (Query,"can not update the name of a course");

            /***** Write message to show the change made *****/
            sprintf (Gbl.Message,"La asignatura <strong>%s</strong>" \
                                 " ha sido renombrada como <strong>%s</strong>.",	// Need translation!
                     CurrentCrsName,NewCrsName);
           }
        }
      else	// The same name
         sprintf (Gbl.Message,"El nombre de la asignatura <strong>%s</strong> no ha cambiado.",	// Need translation!
                  CurrentCrsName);
     }

   /***** Prepare to show the form again *****/
   strcpy (CurrentCrsName,NewCrsName);
  }

/****************************************************************************/
/************* Show message of success after changing a course **************/
/****************************************************************************/

void Deg_ContEditAfterChgCrs (void)
  {
   /***** Write error/success message showing the change made *****/
   Lay_ShowAlert (Gbl.Error ? WARNING : INFO,Gbl.Message);

   /***** Show the form again *****/
   Deg_ReqEditCourses ();
  }

/****************************************************************************/
/********************** Show a form to search courses ***********************/
/****************************************************************************/

void Deg_PutFormSearchCourse (void)
  {
   extern const char *Txt_Search_course[Txt_NUM_LANGUAGES];

   /***** Title *****/
   fprintf (Gbl.F.Out,"%s",
            Txt_Search_course[Gbl.Prefs.Language]);

   /***** Start form *****/
   Act_FormStart (ActSchCrs);

   /***** Course name *****/
   fprintf (Gbl.F.Out,"<input type=\"text\" name=\"SearchCrs\"" \
                      " style=\"width:94px;\" size=\"10\" maxlength=\"%u\" value=\"%s\" />",
            Deg_MAX_LENGTH_COURSE_FULL_NAME,Gbl.Search.Crs);

   fprintf (Gbl.F.Out,"<input type=\"image\" src=\"%s/search20x20.gif\" align=\"top\" title=\"%s\" alt=\"%s\" style=\"width:20px;height:20px;\" />",
            Gbl.Prefs.PathTheme,Txt_Search_course[Gbl.Prefs.Language],Txt_Search_course[Gbl.Prefs.Language]);

   /***** End form *****/
   fprintf (Gbl.F.Out,"</form>");
  }

/****************************************************************************/
/************************* Request the search of courses ********************/
/****************************************************************************/

void Deg_ReqSearchCrsTch (void)
  {
   /***** Request names of course / teacher *****/
   Deg_PutFormSearchCrsTch ("",DEG_RANGE_ALL_DEGREES);
  }

/****************************************************************************/
/************************* Request the search of courses ********************/
/****************************************************************************/

static void Deg_PutFormSearchCrsTch (const char *SearchTch,tDegRange DegRange)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Course[Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_in_any_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_in[Txt_NUM_LANGUAGES];
   extern const char *Txt_Search[Txt_NUM_LANGUAGES];

   /***** Form to request search *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActSchCrsTch);
   fprintf (Gbl.F.Out,"<table cellspacing=\"2\" cellpadding=\"0\">");

   /* Course name */
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"%s\">%s:&nbsp;</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"SearchCrs\" size=\"40\" maxlength=\"%u\" value=\"%s\" /></td></tr>",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Course[Gbl.Prefs.Language],
            Deg_MAX_LENGTH_COURSE_FULL_NAME,Gbl.Search.Crs);

   /* Course teacher */
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"%s\">%s:&nbsp;</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"SearchTch\" size=\"40\" maxlength=\"%u\" value=\"%s\" /></td></tr>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_USR_TYPES_SINGULAR_Abc[Usr_TEACHER][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3,SearchTch);

   /* All the degrees */
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\"><input type=\"radio\" name=\"DegRange\" value=\"%u\"",(unsigned) DEG_RANGE_ALL_DEGREES);
   if (Gbl.CurrentDeg.Deg.DegCod < 0 || DegRange == DEG_RANGE_ALL_DEGREES)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   if (Gbl.CurrentDeg.Deg.DegCod < 0)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," /></td><td align=\"left\" class=\"%s\">%s</td></tr>",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_in_any_degree[Gbl.Prefs.Language]);

   /* Only this degree */
   if (Gbl.CurrentDeg.Deg.DegCod >= 0)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\"><input type=\"radio\" name=\"DegRange\" value=\"%u\"",(unsigned) DEG_RANGE_THIS_DEGREE);
      if (DegRange == DEG_RANGE_THIS_DEGREE)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," /></td><td align=\"left\" class=\"%s\">%s %s</td></tr>",
               The_ClassFormul[Gbl.Prefs.Theme],
               Txt_in[Gbl.Prefs.Language],
               Gbl.CurrentDeg.Deg.FullName);
     }

   fprintf (Gbl.F.Out,"</table>");

   /***** Send button *****/
   Lay_PutSendButton (Txt_Search[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form></div>\n");
  }

/****************************************************************************/
/**************** Get course and teacher from search form *******************/
/****************************************************************************/
// This function is called as a priori function, before writing HTML page, in order to fill input text for search a course

void Deg_GetParamSearchCrs (void)
  {
   /***** Get part of course full name *****/
   Par_GetParam (Par_PARAM_SINGLE,"SearchCrs",Gbl.Search.Crs,Deg_MAX_LENGTH_COURSE_FULL_NAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Search.Crs,Deg_MAX_LENGTH_COURSE_FULL_NAME,true);
  }

/****************************************************************************/
/****************************** Search courses ******************************/
/****************************************************************************/

void Deg_SearchCrs (void)
  {
   /***** Show search form *****/
   Deg_PutFormSearchCrsTch ("",DEG_RANGE_ALL_DEGREES);

   /***** Show results of search *****/
   if (Gbl.Search.Crs[0])			// Course specified
      Deg_SearchCrsTchInDB ("",DEG_RANGE_ALL_DEGREES);
  }

/****************************************************************************/
/*********************** Search courses / teachers **************************/
/****************************************************************************/

void Deg_SearchCrsTch (void)
  {
   char SearchTch[Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3+1];
   tDegRange DegRange = DEG_RANGE_ALL_DEGREES;
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   /***** Get part of course full name *****/
   Deg_GetParamSearchCrs ();

   /***** Get part of teacher full name *****/
   Par_GetParam (Par_PARAM_SINGLE,"SearchTch",SearchTch,Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,SearchTch,Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3,true);

   /***** Get range of degrees *****/
   Par_GetParam (Par_PARAM_SINGLE,"DegRange",UnsignedStr,10);
   if (UnsignedStr[0])
     {
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         DegRange = DEG_RANGE_ALL_DEGREES;
      else if (UnsignedNum >= NUM_TYPES_OF_DEG_RANGE)
         DegRange = DEG_RANGE_ALL_DEGREES;
      else
         DegRange = (tDegRange) UnsignedNum;
     }
   else
      DegRange = DEG_RANGE_ALL_DEGREES;

   /***** Show search form again *****/
   Deg_PutFormSearchCrsTch (SearchTch,DegRange);

   /***** Show results of search *****/
   if (Gbl.Search.Crs[0] ||			// Course specified
       SearchTch[0] ||				// Teacher specified
       DegRange == DEG_RANGE_THIS_DEGREE ||	// Only current degree
       Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)	// Usr_SUPERUSER can see all the courses
      Deg_SearchCrsTchInDB (SearchTch,DegRange);
  }

/****************************************************************************/
/******************************** Search courses ****************************/
/****************************************************************************/
// Return true if course(s) found
#define MAX_LENGTH_SEARCH_QUERY (10*(Deg_MAX_LENGTH_COURSE_FULL_NAME+(Usr_MAX_LENGTH_USR_NAME_OR_SURNAME*3)))
#define MAX_WORDS_IN_SEARCH 10
static void Deg_SearchCrsTchInDB (const char *SearchTch,tDegRange DegRange)
  {
   extern const char *Txt_any_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_SEARCH_Courses_of_DEG_whose_names_contain_X_taught_by_teachers_whose_names_contain_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_SEARCH_Courses_of_DEG_whose_names_contain_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_SEARCH_Teachers_whose_names_contain_X_and_teach_courses_of_DEG[Txt_NUM_LANGUAGES];
   extern const char *Txt_SEARCH_Courses_of_DEG[Txt_NUM_LANGUAGES];
   extern const char *Txt_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_courses[Txt_NUM_LANGUAGES];
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Year[Txt_NUM_LANGUAGES];
   extern const char *Txt_Semester[Txt_NUM_LANGUAGES];
   extern const char *Txt_Course[Txt_NUM_LANGUAGES];
   extern const char *Txt_Stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_Teach[Txt_NUM_LANGUAGES];
   const char *Ptr;
   unsigned NumWords;
   char SearchWord[Deg_MAX_LENGTH_COURSE_FULL_NAME+1];
   char CrsQuery[MAX_LENGTH_SEARCH_QUERY+1];
   char TchQuery[MAX_LENGTH_SEARCH_QUERY+1];
   Usr_ListUsrsRange_t ListUsrsRange;
   char DegQuery[256];
   const char *DegRangeMsg;
   char Query[1024+MAX_LENGTH_SEARCH_QUERY*2];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;

   /***** Split course string into words *****/
   CrsQuery[0] = '\0';
   Ptr = Gbl.Search.Crs;
   for (NumWords=0; NumWords<MAX_WORDS_IN_SEARCH && *Ptr; NumWords++)
     {
      Str_GetNextStringUntilSpace (&Ptr,SearchWord,Deg_MAX_LENGTH_COURSE_FULL_NAME);
      if (strlen (CrsQuery) + strlen (SearchWord) + 256 > MAX_LENGTH_SEARCH_QUERY)	// Prevent string overflow
         break;
      if (NumWords)
         strcat (CrsQuery," AND ");
      strcat (CrsQuery,"courses.FullName LIKE '%");
      strcat (CrsQuery,SearchWord);
      strcat (CrsQuery,"%'");
     }

   /***** Split teacher string into words *****/
   TchQuery[0] = '\0';
   Ptr = SearchTch;
   for (NumWords=0; NumWords<MAX_WORDS_IN_SEARCH && *Ptr; NumWords++)
     {
      Str_GetNextStringUntilSpace (&Ptr,SearchWord,Usr_MAX_LENGTH_USR_NAME_OR_SURNAME);
      if (strlen (TchQuery) + strlen (SearchWord) + 256 > MAX_LENGTH_SEARCH_QUERY)	// Prevent string overflow
         break;
      if (NumWords)
         strcat (TchQuery," AND ");
      strcat (TchQuery,"CONCAT_WS(' ',FirstName,Surname1,Surname2) LIKE '%");
      strcat (TchQuery,SearchWord);
      strcat (TchQuery,"%'");
     }

   /***** Select courses in all the degrees or in current degree *****/
   if (DegRange == DEG_RANGE_ALL_DEGREES)
     {
      DegQuery[0] = '\0';
      DegRangeMsg = Txt_any_degree[Gbl.Prefs.Language];
      ListUsrsRange = Usr_LIST_USRS_RANGE_ANY_DEGREE_SEARCH;
     }
   else
     {
      sprintf (DegQuery," AND courses.DegCod='%ld'",Gbl.CurrentDeg.Deg.DegCod);
      DegRangeMsg = Gbl.CurrentDeg.Deg.FullName;
      ListUsrsRange = Usr_LIST_USRS_RANGE_THIS_DEGREE_SEARCH;
     }

   /***** Get courses from database *****/
   // Important for speed: do the teachers subquery in the FROM clause instead of the WHERE clause
   if (CrsQuery[0] && TchQuery[0])
     {
      sprintf (Gbl.Message,Txt_SEARCH_Courses_of_DEG_whose_names_contain_X_taught_by_teachers_whose_names_contain_X[Gbl.Prefs.Language],
               DegRangeMsg,Gbl.Search.Crs,SearchTch);
      Lay_WriteTitle (Gbl.Message);
      sprintf (Query,"SELECT DISTINCTROW degrees.DegCod,courses.CrsCod,degrees.Logo,degrees.ShortName,degrees.FullName," \
                     "courses.Year,courses.Semester,courses.FullName" \
                     " FROM (SELECT UsrCod FROM usr_data WHERE UsrType='%u' AND %s) AS teachers,crs_usr,courses,degrees" \
                     " WHERE %s AND teachers.UsrCod=crs_usr.UsrCod AND crs_usr.CrsCod=courses.CrsCod%s AND courses.DegCod=degrees.DegCod" \
                     " ORDER BY degrees.FullName,courses.Year,courses.Semester,courses.FullName",
               Usr_TEACHER,TchQuery,CrsQuery,DegQuery);
     }
   else if (CrsQuery[0] && !TchQuery[0])
     {
      sprintf (Gbl.Message,Txt_SEARCH_Courses_of_DEG_whose_names_contain_X[Gbl.Prefs.Language],
               DegRangeMsg,Gbl.Search.Crs);
      Lay_WriteTitle (Gbl.Message);
      sprintf (Query,"SELECT degrees.DegCod,courses.CrsCod,degrees.Logo,degrees.ShortName,degrees.FullName," \
                     "courses.Year,courses.Semester,courses.FullName" \
                     " FROM courses,degrees" \
                     " WHERE %s%s AND courses.DegCod=degrees.DegCod" \
                     " ORDER BY degrees.FullName,courses.Year,courses.Semester,courses.FullName",
               CrsQuery,DegQuery);
     }
   else if (!CrsQuery[0] && TchQuery[0])
     {
      sprintf (Gbl.Message,Txt_SEARCH_Teachers_whose_names_contain_X_and_teach_courses_of_DEG[Gbl.Prefs.Language],
               SearchTch,DegRangeMsg);
      Lay_WriteTitle (Gbl.Message);
/*      sprintf (Query,"SELECT DISTINCTROW degrees.DegCod,courses.CrsCod,degrees.Logo,degrees.ShortName,degrees.FullName," \
                     "courses.Year,courses.Semester,courses.FullName" \
                     " FROM (SELECT UsrCod FROM usr_data WHERE UsrType='%u' AND %s) AS teachers,crs_usr,courses,degrees" \
                     " WHERE teachers.UsrCod=crs_usr.UsrCod AND crs_usr.CrsCod=courses.CrsCod%s AND courses.DegCod=degrees.DegCod" \
                     " ORDER BY degrees.FullName,courses.Year,courses.Semester,courses.FullName",
               Usr_TEACHER,TchQuery,DegQuery); */
      Usr_ListTchsFound (ListUsrsRange,TchQuery);
      return;
     }
   else if (!CrsQuery[0] && !TchQuery[0])
     {
      sprintf (Query,"SELECT degrees.DegCod,courses.CrsCod,degrees.Logo,degrees.ShortName,degrees.FullName," \
                     "courses.Year,courses.Semester,courses.FullName" \
                     " FROM courses,degrees" \
                     " WHERE courses.DegCod=degrees.DegCod%s" \
                     " ORDER BY degrees.FullName,courses.Year,courses.Semester,courses.FullName",
               DegQuery);
      sprintf (Gbl.Message,Txt_SEARCH_Courses_of_DEG[Gbl.Prefs.Language],
               DegRangeMsg);
      Lay_WriteTitle (Gbl.Message);
     }
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get courses");

   /***** List courses found *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write header with number of courses found *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\" colspan=\"7\">");
   if (NumRows == 1)
      fprintf (Gbl.F.Out,"1 %s",Txt_course[Gbl.Prefs.Language]);
   else
      fprintf (Gbl.F.Out,"%ld %s",NumRows,Txt_courses[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** List the courses (one row per course) *****/
   if (NumRows)
     {
      /* Heading row */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\"></td>" \
                         "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"right\" class=\"TIT_TBL\">%s</td></tr>",
               Txt_Degree[Gbl.Prefs.Language],
               Txt_Year[Gbl.Prefs.Language],
               Txt_Semester[Gbl.Prefs.Language],
               Txt_Course[Gbl.Prefs.Language],
               Txt_Stud[Gbl.Prefs.Language],
               Txt_Teach[Gbl.Prefs.Language]);

      /* Write courses */
      for (NumRow=1; NumRow<=NumRows; NumRow++)
        {
         /* Get next course */
         row = mysql_fetch_row (mysql_res);

         /* Write data of this course */
         Deg_WriteRowCrsData (NumRow,row,false);
        }
     }
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************************** Write courses of a user **************************/
/*****************************************************************************/

void Deg_GetAndWriteCoursesOf (long UsrCod)
  {
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Year[Txt_NUM_LANGUAGES];
   extern const char *Txt_Semester[Txt_NUM_LANGUAGES];
   extern const char *Txt_Course[Txt_NUM_LANGUAGES];
   extern const char *Txt_Stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_Teach[Txt_NUM_LANGUAGES];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;

   /***** Get courses of a user from database *****/
   sprintf (Query,"SELECT degrees.DegCod,courses.CrsCod,degrees.Logo,degrees.ShortName,degrees.FullName," \
                  "courses.Year,courses.Semester,courses.FullName,crs_usr.Accepted" \
                  " FROM crs_usr,courses,degrees" \
                  " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod=courses.CrsCod AND courses.DegCod=degrees.DegCod" \
                  " ORDER BY degrees.FullName,courses.Year,courses.Semester,courses.FullName",UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get courses of a user");

   /***** List the courses (one row per course) *****/
   if (NumRows)
     {
      /* Heading row */
      fprintf (Gbl.F.Out,"<tr><td class=\"BM\"></td><td class=\"BM\"></td>" \
                         "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"right\" class=\"TIT_TBL\">%s</td>" \
                         "<td align=\"right\" class=\"TIT_TBL\">%s</td></tr>",
               Txt_Degree[Gbl.Prefs.Language],
               Txt_Year[Gbl.Prefs.Language],
               Txt_Semester[Gbl.Prefs.Language],
               Txt_Course[Gbl.Prefs.Language],
               Txt_Stud[Gbl.Prefs.Language],
               Txt_Teach[Gbl.Prefs.Language]);

      /* Write courses */
      for (NumRow=1; NumRow<=NumRows; NumRow++)
        {
         /* Get next course */
         row = mysql_fetch_row (mysql_res);

         /* Write data of this course */
         Deg_WriteRowCrsData (NumRow,row,true);
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/************** Write the data of a course (result of a query) **************/
/****************************************************************************/

static void Deg_WriteRowCrsData (long NumRow,MYSQL_ROW row,bool WriteColumnAccepted)
  {
   extern const char *Txt_Go_to_X[Txt_NUM_LANGUAGES];
   long DegCod;
   long CrsCod;
   unsigned NumStds;
   unsigned NumTchs;
   const char *Style;
   const char *StyleNoBR;
   bool Accepted;

   /***** Get degree code (row[0]) *****/
   if ((DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of degree.");

   /***** Get course code (row[1]) *****/
   if ((CrsCod = Str_ConvertStrCodToLongCod (row[1])) < 0)
      Lay_ShowErrorAndExit ("Wrong code of course.");

   /***** Get number of students and teachers in this course *****/
   NumStds = Usr_GetNumStdsInCrs (CrsCod);
   NumTchs = Usr_GetNumTchsInCrs (CrsCod);
   if (NumStds + NumTchs)
     {
      Style = "DAT_SMALL_N";
      StyleNoBR = "DAT_SMALL_NOBR_N";
     }
   else
     {
      Style = "DAT_SMALL";
      StyleNoBR = "DAT_SMALL_NOBR";
     }

   /***** Start row *****/
   fprintf (Gbl.F.Out,"<tr>");

   /***** Teacher has accepted joining to this course/to any course in degree/to any course? *****/
   if (WriteColumnAccepted)
     {
      Accepted = (Str_ConvertToUpperLetter (row[8][0]) == 'Y');
      fprintf (Gbl.F.Out,"<td class=\"BT\"><img src=\"%s/%s12x12.gif\" alt=\"\" title=\"Alta %s\" width=\"12\" height=\"12\" /></td>",
               Gbl.Prefs.IconsURL,
               Accepted ? "new" : "tr",
               Accepted ? "confirmada" : "pendiente de confirmaci&oacute;n");	// Need translation!
     }

   /***** Write number of course in this search *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"%s\">%ld&nbsp;</td>",StyleNoBR,NumRow);

   /***** Write degree logo (row[2]) and degree short name (row[3]) *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">",StyleNoBR);
   Act_FormStart (ActUnk);
   Deg_PutParamDegCod (DegCod);
   sprintf (Gbl.MsgWindowStatus,Txt_Go_to_X[Gbl.Prefs.Language],row[4]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,StyleNoBR);
   fprintf (Gbl.F.Out,"<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"%s\" />&nbsp;%s</acronym></a>&nbsp;</form></td>",
            Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,row[2],row[3],row[3]);

   /***** Write year (row[5]) *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" class=\"%s\">%s</td>",Style,YearStrs[Deg_ConvStrToYear (row[5])]);

   /***** Write semester (row[6]) *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" class=\"%s\">%s</td>",Style,SemesterStrs[Deg_ConvStrToSemester (row[6])]);

   /***** Write course full name (row[7]) *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">",Style);
   Act_FormStart (ActUnk);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CrsCod\" value=\"%ld\" />",CrsCod);
   sprintf (Gbl.MsgWindowStatus,Txt_Go_to_X[Gbl.Prefs.Language],row[7]);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,Style);
   fprintf (Gbl.F.Out,"%s</acronym></a>&nbsp;</form></td>",row[7]);

   /***** Write number of students in course *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"%s\">%u</td>",Style,NumStds);

   /***** Write number of teachers in course *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" class=\"%s\">%u</td></tr>",Style,NumTchs);
  }

/****************************************************************************/
/********** Write the degrees administrated by an administrator *************/
/****************************************************************************/

void Deg_GetAndWriteDegreesAdminBy (long UsrCod,unsigned ColSpan)
  {
   extern const char *Txt_Go_to_X[Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long DegCod;

   /***** Get degrees admin by a user from database *****/
   sprintf (Query,"(SELECT DegCod,'','' AS ShortName,'' FROM deg_admin WHERE UsrCod='%ld' AND DegCod<'0')" \
                  " UNION " \
                  "(SELECT degrees.DegCod,degrees.ShortName AS ShortName,degrees.FullName,degrees.Logo FROM deg_admin,degrees" \
                  " WHERE deg_admin.UsrCod='%ld' AND deg_admin.DegCod>='0' AND deg_admin.DegCod=degrees.DegCod) ORDER BY ShortName",
            UsrCod,UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees admin by a user");

   if (NumRows) // If degrees found for this administrator
     {
      /***** Get the list of degrees *****/
      for (NumRow=1; NumRow<=NumRows; NumRow++)
	{
         /***** Indent *****/
         fprintf (Gbl.F.Out,"<tr><td align=\"right\" bgcolor=\"#%s\"><img src=\"%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"\" /></td>",
                  Gbl.ColorRows[Gbl.GlobalRowEvenOdd],Gbl.Prefs.IconsURL,(NumRow == NumRows) ? "subend" : "submid");

         /***** Write degree *****/
         fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" colspan=\"%u\" class=\"DAT_SMALL_NOBR\" bgcolor=\"#%s\">",
                  ColSpan-1,Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);

         /* Get next degree */
         row = mysql_fetch_row (mysql_res);
         DegCod = Str_ConvertStrCodToLongCod (row[0]);

         if (DegCod >= 0)
           {
            /* Write degree logo and degree short name */
            Act_FormStart (ActUnk);
            Deg_PutParamDegCod (DegCod);
            sprintf (Gbl.MsgWindowStatus,Txt_Go_to_X[Gbl.Prefs.Language],row[2]);
            Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,"DAT_SMALL_NOBR");
            fprintf (Gbl.F.Out,"<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"%s\" /> %s</acronym></a></form>",
                     Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,row[3],row[1],row[2]);
           }
         else
            fprintf (Gbl.F.Out,"<img src=\"%s/ugr16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"todas las titulaciones\" /> todas las titulaciones",
                     Gbl.Prefs.IconsURL);

         fprintf (Gbl.F.Out,"</td></tr>");
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }
