// swad_fil.c: files

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <ctype.h>		// For isprint, isspace, etc.
#include <dirent.h>		// For scandir, etc.
#include <errno.h>		// For errno
#include <stdio.h>		// For FILE,fprintf
#include <stdlib.h>		// For exit, system, malloc, calloc, free, etc.
#include <string.h>		// For string functions
#include <sys/stat.h>		// For mkdir
#include <sys/types.h>		// For mkdir
#include <unistd.h>		// For unlink

#include "swad_cfg.h"
#include "swad_gbl.h"
#include "swad_fil.h"
#include "swad_str.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

// Maximum allowed size (in bytes) of a file when uploading it.
// Must be < 2 GiB, because off_t type is of type long int
#define MAX_FILE_SIZE (1536ULL*1024ULL*1024ULL)

/****************************************************************************/
/******************************* Private types ******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private variables ****************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

/****************************************************************************/
/******** Create HTML output file for the web page sent by this CGI *********/
/****************************************************************************/

void Fil_CreateFileForXHTMLOutput (void)
  {      
   /***** Check if exists the directory for HTML output. If not exists, create it *****/
   Fil_CreateDirIfNotExists (Cfg_PATH_REL_XHTML_OUTPUT_PRIVATE);

   /***** Create a unique name for the file *****/
   sprintf (Gbl.HTMLOutput.FileName,"%s/%s.html",
            Cfg_PATH_REL_XHTML_OUTPUT_PRIVATE,Gbl.UniqueNameEncrypted);

   /***** Open file for writing and reading *****/
   if ((Gbl.F.Out = fopen (Gbl.HTMLOutput.FileName,"w+t")) == NULL)
     {
      Gbl.F.Out = stdout;
      Lay_ShowErrorAndExit ("Can not create output file.");
     }

   /***** Remove old files *****/
   Fil_RemoveOldXHTMLOutputFiles ();
  }

/*****************************************************************************/
/************************ Remove old XHTML output files **********************/
/*****************************************************************************/

void Fil_RemoveOldXHTMLOutputFiles (void)
  {
   struct dirent **ListNames;
   int NumFich,NumFiles;
   char PathFileRel[PATH_MAX+1];
   struct stat Estado;

   /***** Scan the directory of HTML output *****/
   NumFiles = scandir (Cfg_PATH_REL_XHTML_OUTPUT_PRIVATE, &ListNames, 0, 0);

   /***** Analyze the files borrando the antiguos *****/
   for (NumFich=0; NumFich<NumFiles; NumFich++)
      if (strcmp (ListNames[NumFich]->d_name,".") &&
          strcmp (ListNames[NumFich]->d_name,".."))	// Skip directories "." and ".."
	{
	 sprintf (PathFileRel,"%s/%s",Cfg_PATH_REL_XHTML_OUTPUT_PRIVATE,ListNames[NumFich]->d_name);
	 lstat (PathFileRel,&Estado);
	 if (Gbl.TimeStartExecution - Estado.st_mtime > Cfg_TIME_TO_DELETE_HTML_OUTPUT)	// If the file no se ha borrado...
	    unlink (PathFileRel);
	}
  }

/****************************************************************************/
/***************** Close and remove the HTML output file ********************/
/****************************************************************************/

void Fil_CloseAndRemoveFileForHTMLOutput (void)
  {
   if (Gbl.F.Out)
     {
      fclose (Gbl.F.Out);
      unlink (Gbl.HTMLOutput.FileName);
     }
   Gbl.F.Out = stdout;
  }

/****************************************************************************/
/********** Open temporary file and write on it reading from stdin **********/
/****************************************************************************/

bool Fil_ReadStdinIntoTmpFile (void)
  {
   extern const char *Txt_File_too_large[Txt_NUM_LANGUAGES];
   extern const char *Txt_Upload_time_too_long[Txt_NUM_LANGUAGES];
   extern const char *Txt_maximum[Txt_NUM_LANGUAGES];
   extern const char *Txt_minutes[Txt_NUM_LANGUAGES];
   unsigned long long TmpFileSize;
   bool FileIsTooBig = false;
   bool TimeExceeded = false;

   if ((Gbl.F.Tmp = tmpfile ()) == NULL)
     {
      Fil_EndOfReadingStdin ();
      Lay_ShowErrorAndExit ("Can not create temporary file.");
     }
   for (TmpFileSize=0; !feof (stdin) && !FileIsTooBig && !TimeExceeded; TmpFileSize++)
      if (TmpFileSize < MAX_FILE_SIZE)
        {
         if (!(TmpFileSize % (64ULL*1024ULL)))	// Se comprueba the tiempo cada cierto tiempo
            if (time (NULL) - Gbl.TimeStartExecution >= Cfg_TIME_TO_ABORT_FILE_UPLOAD)
               TimeExceeded = true;
         fputc (fgetc (stdin),Gbl.F.Tmp);
        }
      else
         FileIsTooBig = true;
   if (FileIsTooBig || TimeExceeded)
     {
      Fil_EndOfReadingStdin ();  // If stdin were not fully read, there will be problems with buffers
      if (FileIsTooBig)
         sprintf (Gbl.Message,"%s (%s: %lu MiB).",
                  Txt_File_too_large[Gbl.Prefs.Language],Txt_maximum[Gbl.Prefs.Language],
                  (unsigned long) (MAX_FILE_SIZE/(1024ULL*1024ULL)));
      else
         sprintf (Gbl.Message,"%s (%s: %ld %s).",
                  Txt_Upload_time_too_long[Gbl.Prefs.Language],Txt_maximum[Gbl.Prefs.Language],
                  Cfg_TIME_TO_ABORT_FILE_UPLOAD/60,Txt_minutes[Gbl.Prefs.Language]);
      Lay_ShowAlert (WARNING,Gbl.Message);
      return false;
     }
   rewind (Gbl.F.Tmp);

   return true;
  }

/****************************************************************************/
/********** End the reading of all the characters coming from stdin *********/
/****************************************************************************/

void Fil_EndOfReadingStdin (void)
  {
   while (!feof (stdin))
      fgetc (stdin);
  }

/****************************************************************************/
/********         Start the reception of data from a file            ********/
/******** once the rest of parameters from the form have been readed ********/
/****************************************************************************/
/*
-----------------------------7d113610948
Content-Disposition: form-data; name="CURSO_ACADEMICO"

2000-2001
-----------------------------7d113610948
Content-Disposition: form-data; name="ASIGNATURA"

Estructura de los computadores II
-----------------------------7d113610948
Content-Disposition: form-data; name="FICH_FICHAS"

../swad/fichas.htm
-----------------------------7d113610948
Content-Disposition: form-data; name="FICH_LOG"

../public_html/docencia/ecii/ecii-log.html
-----------------------------7d113610948
Content-Disposition: form-data; name="DNI"

L384261
-----------------------------7d113610948
Content-Disposition: form-data; name="UsrPwd"

13282
-----------------------------7d113610948
Content-Disposition: form-data; name="NxtAct"

/~acanas/docencia/fotos
-----------------------------7d113610948
Content-Disposition: form-data; name="Archivo"; filename="D:\Usr\Antonio\Docencia\Estruct\Notas\Notas 2000-2001.ec\fotos\R157550.jpg"
Content-Type: image/pjpeg

000000  FFD8FFE0 00104A46 49460001 01010048  ෷JFIFH
000010  00480000 FFDB0043 00030202 03020203  H۷C
000020  03030304 03030405 08050504 04050A07  
etc, etc.
*/
void Fil_StartReceptionOfFile (char *SrcFileName,char *FileType)
  {
   char *Ptr;
   int Ch,i;

   /* At this point, a heading has been read from Gbl.F.Tmp
      with all the variables passed by form */
   rewind (Gbl.F.Tmp);
   if (!Str_FindStrInFile (Gbl.F.Tmp,Brw_NAME_OF_PARAM_FILENAME_ORG,Str_NO_SKIP_HTML_COMMENTS))
     {
      sprintf (Gbl.Message,"Error uploading file: parameter <strong>%s</strong> not found.",
               Brw_NAME_OF_PARAM_FILENAME_ORG);
      Lay_ShowErrorAndExit (Gbl.Message);
     }

   /* Go to the name of the source file */
   if (!Str_FindStrInFile (Gbl.F.Tmp,"filename=\"",Str_NO_SKIP_HTML_COMMENTS))
      Lay_ShowErrorAndExit ("Error uploading file: parameter &quot;filename&quot; not found.");

   /* Get the name of the source file */
   Ptr = SrcFileName;
   while ((Ch = fgetc (Gbl.F.Tmp)) != (int) '\"')
      *Ptr++ = Ch;
   *Ptr = '\0';

   /* Get and check the type of data */
   if (!Str_FindStrInFile (Gbl.F.Tmp,"Content-Type:",Str_NO_SKIP_HTML_COMMENTS))
      Lay_ShowErrorAndExit ("Error uploading file: &quot;Content-Type&quot; not found.");

   /* Skip spaces and get the type of file */
   while (isspace (Ch = fgetc (Gbl.F.Tmp)));
   for (i=0, Ptr = FileType; !isspace (Ch) && i<Brw_MAX_BYTES_MIME_TYPE; i++)
     {
      *Ptr++ = Str_ConvertToLowerLetter ((char) Ch);
      Ch = fgetc (Gbl.F.Tmp);
     }
   *Ptr = '\0';
  }

/****************************************************************************/
/****************** End the reception of data of a file *********************/
/****************************************************************************/

bool Fil_EndReceptionOfFile (char *FileNameDatosTmp)
  {
   extern const char *Txt_File_too_large[Txt_NUM_LANGUAGES];
   extern const char *Txt_maximum[Txt_NUM_LANGUAGES];
   extern const char *Txt_Another_upload_problem[Txt_NUM_LANGUAGES];
   FILE *FileDataTmp;
   int Result;  // Result of the reception of the file

   /***** Open a new file temporary *****/
   if ((FileDataTmp = fopen (FileNameDatosTmp,"wb")) == NULL)
      Lay_ShowErrorAndExit ("Can not open temporary file.");

   /***** Skip carriage returns, spaces, etc. *****/
   Str_SkipSpacesInFile (Gbl.F.Tmp);

   /* At this point, the data of the file begin */

   /***** Write the file *****/
   Result = Str_ReceiveFileUntilDelimitStr (Gbl.F.Tmp, FileDataTmp, (char *) NULL, Gbl.DelimiterStringIncludingInitialRet, MAX_FILE_SIZE);
   fclose (FileDataTmp);
   if (Result != 1)
     /* 0 ==> File too long; -1 ==> Unfinished transmission */
     {
      unlink (FileNameDatosTmp);
      sprintf (Gbl.Message,"%s (%s: %lu MiB) / %s.",
               Txt_File_too_large[Gbl.Prefs.Language],Txt_maximum[Gbl.Prefs.Language],
               (unsigned long) (MAX_FILE_SIZE/(1024ULL*1024ULL)),
               Txt_Another_upload_problem[Gbl.Prefs.Language]);
      Lay_ShowAlert (ERROR,Gbl.Message);
      return false;	// Error
     }
   return true;		// Success
  }

/*****************************************************************************/
/************* Create names and new file to update a existing file ***********/
/*****************************************************************************/
// CurrentName doesn't change
// OldName is created
// NewName is created

void Fil_CreateUpdateFile (const char *CurrentName,const char *ExtensionOldName,char *OldName,char *NewName,FILE **NewFile)
  {
   int LongRaizFich = Str_GetLengthRootFileName (CurrentName);

   strncpy (NewName,CurrentName,LongRaizFich);
   NewName[LongRaizFich] = '\0';
   sprintf (OldName,"%s%s",NewName,ExtensionOldName);
   strcat (NewName,".new");

   /* The new file shouldn't exist. If it exists is due to any error when running this CGI formerly
      and the file was not renamed successfully. In this case, remove it! */
   if (Fil_CheckIfPathExists (NewName))
      unlink (NewName);

   /* Open the new file */
   if ((*NewFile = fopen (NewName,"wt")) == NULL)
     {
      sprintf (Gbl.Message,"Can not create file <strong>%s</strong>.",NewName);
      Lay_ShowErrorAndExit (Gbl.Message);
     }
  }

/*****************************************************************************/
/****************** Close and rename files related to an update **************/
/*****************************************************************************/

void Fil_CloseUpdateFile (const char *CurrentName,const char *OldName,const char *NewName,FILE *NewFile)
  {
   /* Close file */
   fclose (NewFile);

   /* Rename the old file and the new file */
   if (rename (CurrentName,OldName)) // mv CurrentName OldName Ej: mv file.html file.old
     {
      sprintf (Gbl.Message,"Can not rename the file <strong>%s</strong> as <strong>%s</strong>.",
               CurrentName,OldName);
      Lay_ShowErrorAndExit (Gbl.Message);
     }
   if (rename (NewName,CurrentName)) // mv NewName CurrentName Ej: mv file.new file.html
     {
      sprintf (Gbl.Message,"Can not rename the file <strong>%s</strong> as <strong>%s</strong>.",
               NewName,CurrentName);
      Lay_ShowErrorAndExit (Gbl.Message);
     }
  }

/****************************************************************************/
/*********************** Rename a file or directory *************************/
/****************************************************************************/

bool Fil_RenameFileOrDir (const char *PathOld,const char *PathNew)
  {
   /* Rename the file or directory */
   if (rename (PathOld,PathNew) == 0)
      return true;
   else
     {
      switch (errno)
        {
	 case ENOTEMPTY:
	 case EEXIST:
	    sprintf (Gbl.Message,"Error: ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.",	// Need translation!
	             PathNew);
	    break;
	 case ENOTDIR:
	    sprintf (Gbl.Message,"Error: ya existe un archivo con el nombre <strong>%s</strong>.",	// Need translation!
	             PathNew);
	    break;
	 case EACCES:
	    Lay_ShowErrorAndExit ("Write is forbidden.");
	    break;
	  default:
	    Lay_ShowErrorAndExit ("Can not rename file or folder.");
	    break;
	}
      return false;
     }
  }

/****************************************************************************/
/***************** Check if existe a file or directory **********************/
/****************************************************************************/
/* Return true if exists and false if not exists */

bool Fil_CheckIfPathExists (const char *Path)
  {
   return access (Path,F_OK) ? false : true;
  }

/*****************************************************************************/
/********** Check if a directory exists. If not exists, create it! ***********/
/*****************************************************************************/

void Fil_CreateDirIfNotExists (const char *Path)
  {
   if (!Fil_CheckIfPathExists (Path))
      if (mkdir (Path,(mode_t) 0xFFF) != 0)
        {
	 sprintf (Gbl.Message,"Can not create folder <strong>%s</strong>.",Path);
	 Lay_ShowErrorAndExit (Gbl.Message);
        }
  }

/****************************************************************************/
/**************************** Fast copy of files ****************************/
/****************************************************************************/

void Fil_FastCopyOfFiles (const char *PathSrc,const char *PathTgt)
  {
   FILE *FileSrc,*FileTgt;

   /***** Open source file *****/
   if ((FileSrc = fopen (PathSrc,"rb")) == NULL)
      Lay_ShowErrorAndExit ("Can not open source file.");

   /***** Open destination file *****/
   if ((FileTgt = fopen (PathTgt,"wb")) == NULL)
      Lay_ShowErrorAndExit ("Can not open target file.");

   /***** Copy source file into destination file *****/
   Fil_FastCopyOfOpenFiles (FileSrc,FileTgt);

   /***** Close the files *****/
   fclose (FileTgt);
   fclose (FileSrc);
  }

/****************************************************************************/
/************************* Fast copy of open files **************************/
/****************************************************************************/

#define NUM_BYTES_PER_COPY 4096

void Fil_FastCopyOfOpenFiles (FILE *FileSrc,FILE *FileTgt)
  {
   unsigned char Bytes[NUM_BYTES_PER_COPY];
   size_t NumBytesRead;

   while ((NumBytesRead = fread ((void *) Bytes,sizeof (Bytes[0]),(size_t) NUM_BYTES_PER_COPY,FileSrc)))
      fwrite ((void *) Bytes,sizeof (Bytes[0]),NumBytesRead,FileTgt);
  }

/****************************************************************************/
/***** Create a semaphore file to avoid other instance of the CGI ***********/
/***** acceding to the files. It's equivalent to a WAIT           ***********/
/****************************************************************************/

bool Fil_WaitSemaphore (void)
  {
   extern const char *Txt_X_is_busy_at_the_moment_Try_again_in_a_few_seconds[Txt_NUM_LANGUAGES];
   char Command[100];
   int ResultLockFile;

   /*
   Usage: lockfile -www | -r rrr | -l lll | -s nnn | -! | -ml | -mu | file ...
	-www	wait www seconds between locking attempts
	-r rrr	make at most rrr retries before giving up on a lock
	-l lll	set locktimeout to lll seconds
	-s nnn	suspend nnn seconds after a locktimeout occurred
	-!	invert the exit code of lockfile
	-ml	lock your system mail-spool file
	-mu	unlock your system mail-spool file

      lockfile tries to create the specified file. Usually ends in ".lock".
      If it doesn't create the file because it exists, it retries after www seconds.
      It makes rrr trys (www*rrr seconds in total).
      The file will be removed after lll seconds
      to ensure that it will be removed even when the CGI becomes a zombie and doesn't remove the file,
      so avoiding blocking other processes. */

   if (!Gbl.LockCreated)
     {
      sprintf (Command,"lockfile -1 -r20 -l30 %s",Gbl.PathLockFile);
      // sprintf (Command,"lockfile -10 -r1000 -l10000 %s",Gbl.PathLockFile);	// To execute long actions
      ResultLockFile = system (Command);
      if (ResultLockFile == 127 ||	// Can't execute /bin/sh
	  ResultLockFile == -1)		// Other error of system
	 Lay_ShowErrorAndExit ("Can not create lock file.");
      if (ResultLockFile != 0)		// Couldn't create the lock file after all the retries
      	{
      	 sprintf (Gbl.Message,Txt_X_is_busy_at_the_moment_Try_again_in_a_few_seconds[Gbl.Prefs.Language],
      	          Cfg_PLATFORM_SHORT_NAME);
	 Lay_ShowAlert (WARNING,Gbl.Message);
	 return false;
	}
      Gbl.LockCreated = true;
     }

   return true;
  }

/****************************************************************************/
/********* Remove the semaphore file. It's equivalent to a SIGNAL ***********/
/****************************************************************************/

void Fil_SignalSemaphore (void)
  {
   if (Gbl.LockCreated)
      if (Fil_CheckIfPathExists (Gbl.PathLockFile))  // If file exists
	{
	 if (unlink (Gbl.PathLockFile))
	    fprintf (Gbl.F.Out,"Can not remove lock file.\n");
	 else
	    Gbl.LockCreated = false;
	}
  }
