// swad_ses.c: sessions

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <mysql/mysql.h>	// To access MySQL databases
#include <stdio.h>		// For sprintf
#include <string.h>		// For string functions

#include "swad_con.h"
#include "swad_db.h"
#include "swad_gbl.h"

/****************************************************************************/
/*************************** Internal constants *****************************/
/****************************************************************************/

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/************************** Get number of open sessions *********************/
/****************************************************************************/

void Ses_GetNumSessions (void)
  {
   char Query[128];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;

   /***** Get the number of open sessions from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM sessions");
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of open sessions");

   /***** Get number of users *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&Gbl.Session.NumSessions) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of open sessions.");

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   Gbl.Usrs.Connected.TimeToRefreshInMs = (unsigned long) (Gbl.Session.NumSessions/Cfg_TIMES_PER_SECOND_REFRESH_CONNECTED) * 1000UL;
   if (Gbl.Usrs.Connected.TimeToRefreshInMs < Con_MIN_TIME_TO_REFRESH_CONNECTED_IN_MS)
      Gbl.Usrs.Connected.TimeToRefreshInMs = Con_MIN_TIME_TO_REFRESH_CONNECTED_IN_MS;
   else if (Gbl.Usrs.Connected.TimeToRefreshInMs > Con_MAX_TIME_TO_REFRESH_CONNECTED_IN_MS)
      Gbl.Usrs.Connected.TimeToRefreshInMs = Con_MAX_TIME_TO_REFRESH_CONNECTED_IN_MS;
  }

/****************************************************************************/
/*************************** Create a new session ***************************/
/****************************************************************************/

void Ses_CreateSession (void)
  {
   /***** Create a unique name for the session *****/
   strcpy (Gbl.Session.Id,Gbl.UniqueNameEncrypted);

   /***** Check that session is not open *****/
   if (Ses_CheckIfSessionExists (Gbl.Session.Id))
      Lay_ShowErrorAndExit ("Can not create session.");

   /***** Add session to database *****/
   Ses_InsertSessionInDB ();

   /***** Update time and course in connected list *****/
   Con_UpdateMeInConnectedList ();

   /***** Update number of connected users and number of open sessions in order to show them properly *****/
   Con_GetConnectedUsrsTotal ();
   Ses_GetNumSessions ();
  }

/****************************************************************************/
/*********** Check if the session already exists in the database ************/
/****************************************************************************/
// Return true if session exists
// Return false if session doesn't exists or error

bool Ses_CheckIfSessionExists (const char *IdSes)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if session already exists in database *****/
   sprintf (Query,"SELECT COUNT(*) FROM sessions WHERE SessionId='%s'",
            IdSes);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a session already existed");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a session already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/************************** Close current session ***************************/
/****************************************************************************/

void Ses_CloseSession (void)
  {
   if (Gbl.Usrs.Me.Logged)
     {
      /***** Remove session from database *****/
      Ses_RemoveSessionFromDB ();

      /***** If there are no more sessions for current user ==> remove user from connected list *****/
      Con_RemoveOldConnected ();

      Ses_RemoveHiddenParFromExpiredSessions ();

      /***** Now, user is not autentified *****/
      Gbl.Usrs.Me.Logged = Gbl.Usrs.Me.IBelongToThisCrs = Gbl.Usrs.Me.IHaveAccessToCurrentCrs = false;
      Gbl.Session.Id[0] = '\0';
      Gbl.CurrentTab = TabUnk;

      /***** Update number of connected users and number of open sessions in order to show them properly *****/
      Con_GetConnectedUsrsTotal ();
      Ses_GetNumSessions ();
     }
  }

/****************************************************************************/
/******************** Insert new session in the database ********************/
/****************************************************************************/

void Ses_InsertSessionInDB (void)
  {
   char Query[1024];

   /***** Insert session in the database *****/
   sprintf (Query,"INSERT INTO sessions (SessionId,UsrCod,Password,UsrType,DegTypCod,DegCod,CrsCod,LastTime,LastRefresh)" \
                  " VALUES ('%s','%ld','%s','%u','%ld','%ld','%ld',NOW(),NOW())",
            Gbl.Session.Id,
            Gbl.Usrs.Me.UsrDat.UsrCod,
            Gbl.Usrs.Me.UsrDat.Password,
            (unsigned) Gbl.Usrs.Me.UsrTypeLog,
            Gbl.CurrentDeg.Deg.DegTypCod,
            Gbl.CurrentDeg.Deg.DegCod,
            Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryINSERT (Query,"can not create session");
  }

/****************************************************************************/
/************** Modificar data of session in the database ***************/
/****************************************************************************/

void Ses_UpdateSessionDataInDB (void)
  {
   char Query[1024];

   /***** Update session in database *****/
   sprintf (Query,"UPDATE sessions SET UsrCod='%ld',Password='%s',UsrType='%u',DegTypCod='%ld',DegCod='%ld',CrsCod='%ld',LastTime=NOW(),LastRefresh=NOW()" \
                  " WHERE SessionId='%s'",
            Gbl.Usrs.Me.UsrDat.UsrCod,
            Gbl.Usrs.Me.UsrDat.Password,
            (unsigned) Gbl.Usrs.Me.UsrTypeLog,
            Gbl.CurrentDeg.Deg.DegTypCod,
            Gbl.CurrentDeg.Deg.DegCod,
            Gbl.CurrentCrs.Crs.CrsCod,
            Gbl.Session.Id);
   DB_QueryUPDATE (Query,"can not update session");
  }

/****************************************************************************/
/******************* Modify session last refresh in database ****************/
/****************************************************************************/

void Ses_UpdateSessionLastRefreshInDB (void)
  {
   char Query[512];

   /***** Update session in database *****/
   sprintf (Query,"UPDATE sessions SET LastRefresh=NOW() WHERE SessionId='%s'",
            Gbl.Session.Id);
   DB_QueryUPDATE (Query,"can not update session");
  }

/****************************************************************************/
/********************* Remove session from the database *********************/
/****************************************************************************/

void Ses_RemoveSessionFromDB (void)
  {
   char Query[512];

   /***** Remove current session *****/
   sprintf (Query,"DELETE FROM sessions WHERE SessionId='%s'",
            Gbl.Session.Id);
   DB_QueryDELETE (Query,"can not remove a session");
  }

/*****************************************************************************/
/*************************** Remove expired sessions *************************/
/*****************************************************************************/

void Ses_RemoveExpiredSessions (void)
  {
   char Query[1024];

   /***** Remove expired sessions *****/
   /* A session expire when last click (LastTime) is too old,
      or when there was at least one refresh (navigator supports AJAX) and last refresh is too old (browser probably was closed) */
   sprintf (Query,"DELETE LOW_PRIORITY FROM sessions WHERE" \
                  " (UNIX_TIMESTAMP() > UNIX_TIMESTAMP(LastTime)+%ld) OR" \
                  " ((UNIX_TIMESTAMP(LastRefresh) > UNIX_TIMESTAMP(LastTime)+1) AND (UNIX_TIMESTAMP() > UNIX_TIMESTAMP(LastRefresh)+%ld))",
            Cfg_TIME_TO_CLOSE_SESSION_FROM_LAST_CLICK,
            Cfg_TIME_TO_CLOSE_SESSION_FROM_LAST_REFRESH);
   DB_QueryDELETE (Query,"can not remove expired sessions");
  }

/*****************************************************************************/
/*** Get the data (user code and password) of an already initiated session ***/
/*****************************************************************************/

bool Ses_GetSessionData (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool Result = false;

   /***** Query data of session from database *****/
   sprintf (Query,"SELECT UsrCod,Password,UsrType,DegTypCod,DegCod,CrsCod FROM sessions" \
                  " WHERE SessionId='%s'",Gbl.Session.Id);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get data of session");

   /***** Check if the session existed in the database *****/
   if (NumRows)
     {
      row = mysql_fetch_row (mysql_res);

      /***** Get user code (row[0]) *****/
      Gbl.Session.UsrCod = Str_ConvertStrCodToLongCod (row[0]);

      /***** Get password (row[1]) *****/
      strcpy (Gbl.Usrs.Me.LoginEncryptedPassword,row[1]);

      /***** Get logged user type (row[2]) *****/
      if (sscanf (row[2],"%u",&Gbl.Usrs.Me.UsrTypeFromSession) != 1)
         Gbl.Usrs.Me.UsrTypeFromSession = Usr_UNKNOWN;

      /***** Get degree type code (row[3]) *****/
      Gbl.CurrentDeg.Deg.DegTypCod = Str_ConvertStrCodToLongCod (row[3]);

      /***** Get degree code (row[4]) *****/
      Gbl.CurrentDeg.Deg.DegCod = Str_ConvertStrCodToLongCod (row[4]);

      /***** Get course code (row[5]) *****/
      Gbl.CurrentCrs.Crs.CrsCod = Str_ConvertStrCodToLongCod (row[5]);

      Result = true;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return Result;
  }

/****************************************************************************/
/****************** Insert hidden parameter in the database *****************/
/****************************************************************************/

void Ses_InsertHiddenParInDB (Act_Action_t Action,const char *ParamName,const char *ParamValue)
  {
   char Query[512+Cte_MAX_BYTES_TEXT];

   if (Gbl.Session.IsOpen)
     {
      /***** Before of inserting the first hidden parameter passed to the next action,
             delete all the parameters coming from the previous action *****/
      if (!Gbl.HiddenParamsInsertedIntoDB)
         Ses_RemoveHiddenParFromThisSession ();

      /***** For a unique session-action-parameter, don't insert a parameter more than one time *****/
      if (!Ses_CheckIfHiddenParIsYetInDB (Action,ParamName))
        {
         /***** Insert parameter in the database *****/
         sprintf (Query,"INSERT INTO hidden_params (SessionId,Action,ParamName,ParamValue)" \
                        " VALUES ('%s','%d','%s','%s')",
                  Gbl.Session.Id,(int) Action,ParamName,ParamValue);
         DB_QueryINSERT (Query,"can not create hidden parameter");
         Gbl.HiddenParamsInsertedIntoDB = true;
        }
     }
  }

/****************************************************************************/
/*********** Remove hidden parameters of a session from database ************/
/****************************************************************************/

void Ses_RemoveHiddenParFromThisSession (void)
  {
   char Query[512];

   if (Gbl.Session.IsOpen)
     {
      /***** Remove hidden parameters of this session *****/
      sprintf (Query,"DELETE FROM hidden_params WHERE SessionId='%s'",
               Gbl.Session.Id);
      DB_QueryDELETE (Query,"can not remove hidden parameters of current session");
     }
  }

/****************************************************************************/
/******** Remove expired hidden parameters (from expired sessions) **********/
/****************************************************************************/

void Ses_RemoveHiddenParFromExpiredSessions (void)
  {
   char Query[512];

   /***** Remove hidden parameters from expired sessions *****/
   sprintf (Query,"DELETE FROM hidden_params" \
                  " WHERE SessionId NOT IN (SELECT SessionId FROM sessions)");
   DB_QueryDELETE (Query,"can not remove hidden parameters of expired sessions");
  }

/****************************************************************************/
/************** Check if a hidden parameter existed in database *************/
/****************************************************************************/
// Return true if the parameter existed yet in database

bool Ses_CheckIfHiddenParIsYetInDB (Act_Action_t Action,const char *ParamName)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get a hidden parameter from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM hidden_params" \
                  " WHERE SessionId='%s' AND Action='%d' AND ParamName='%s'",
            Gbl.Session.Id,(int) Action,ParamName);
   DB_QuerySELECT (Query,&mysql_res,"can not get a hidden parameter");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when getting a hidden parameter.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/****************************************************************************/
/**************** Get hidden parameter from the database ********************/
/****************************************************************************/
// Return true if the parameter is too big

bool Ses_GetHiddenParFromDB (Act_Action_t Action,const char *ParamName,char *ParamValue,size_t MaxBytes)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool ParameterIsTooBig = false;

   ParamValue[0] = '\0';
   if (Gbl.Session.IsOpen)	// If the session is not open, the parameter will be empty
     {
      /***** Get a hidden parameter from database *****/
      sprintf (Query,"SELECT ParamValue FROM hidden_params" \
                     " WHERE SessionId='%s' AND Action='%d' AND ParamName='%s'",
               Gbl.Session.Id,(int) Action,ParamName);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get a hidden parameter");

      /***** Check if the parameter is found in database *****/
      if (NumRows)
        {
         /***** Get the value del parameter *****/
         row = mysql_fetch_row (mysql_res);
         strncpy (ParamValue,row[0],MaxBytes);
         ParamValue[MaxBytes] = '\0';
         ParameterIsTooBig = (strlen (row[0]) > MaxBytes);
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }

   return ParameterIsTooBig;
  }
