// swad_tst.c: self-assessment tests

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <limits.h>		// For UINT_MAX
#include <locale.h>		// For setlocale, LC_NUMERIC...
#include <mysql/mysql.h>	// To access MySQL databases
#include <stdio.h>		// For fprintf, etc.
#include <stdlib.h>		// For exit, system, malloc, free, etc
#include <string.h>		// For string functions

#include "swad_act.h"
#include "swad_boo.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_the.h"
#include "swad_tst.h"
#include "swad_txt.h"

/****************************************************************************/
/*************************** Internal constants *****************************/
/****************************************************************************/

#define Tst_MAX_BYTES_TAGS_LIST		(16*1024)
#define Tst_MAX_LENGTH_ANSWER		1000
#define Txt_MAX_BYTES_ANSWER		(Tst_MAX_LENGTH_ANSWER*Str_MAX_CHARACTER)
#define Txt_MAX_BYTES_FLOAT_ANSWER		30	// Maximum length of the strings that store an floating point answer

// Feedback to students in exams
const char *Tst_FeedbackDB[Tst_NUM_TYPES_FEEDBACK] =
  {
   "nothing",
   "total_result",
   "each_result",
   "each_good_bad"
  };

const char *Tst_StrAnswerTypesDB[Tst_NUM_ANS_TYPES] =
  {
   "T/F",
   "unique_choice",
   "multiple_choice",
   "text",
   "int",
   "float"
  };

/****************************************************************************/
/******************************* Internal types *****************************/
/****************************************************************************/

#define Tst_NUM_STATUS 2
typedef enum
  {
   Tst_STATUS_SHOWN_BUT_NOT_ASSESSED	= 0,
   Tst_STATUS_ASSESSED			= 1,
   Tst_STATUS_ERROR			= 2,
  } Tst_Status_t;

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/************************* Internal global variables ************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Internal prototypes **************************/
/****************************************************************************/

static bool Tst_CheckIfNextTstAllowed (void);
static void Tst_SetTstStatus (unsigned NumTst,Tst_Status_t TstStatus);
Tst_Status_t Tst_GetTstStatus (unsigned NumTst);
unsigned Tst_GetNumHitsTst (void);
static void Tst_WriteTestHead (unsigned NumTst);
static void Tst_WriteTestFoot (void);
static void Tst_ShowTestQuestions (MYSQL_RES *mysql_res);
static void Tst_ShowTstResult (double *TotalMark);
static void Tst_WriteQstAndAnsExam (unsigned NumQst,long QstCod,MYSQL_ROW row,double *MarkThisQst);
static void Tst_WriteQstStem (const char *Stem,const char *TextStyle);
static void Tst_UpdateMarkQst (long QstCod,float MarkThisQst);
static void Tst_UpdateMyNumAccessTst (unsigned NumHitsTst);
static void Tst_UpdateNextAccTst (unsigned NumSecondsUntilNextAccess);
static long Tst_GetParamTagCode (void);
static unsigned long Tst_GetAllTagsFromCurrentCrs (MYSQL_RES **mysql_res);
static unsigned long Tst_GetEnabledTagsFromThisCrs (MYSQL_RES **mysql_res);
static void Tst_ShowFormSelTags (unsigned long NumRows,MYSQL_RES *mysql_res,bool ShowOnlyEnabledTags);
static void Tst_ShowFormEditTags (void);
static void Tst_PutIconEnable (long TagCod,const char *TagTxt);
static void Tst_PutIconDisable (long TagCod,const char *TagTxt);
static void Tst_ShowFormConfigTst (void);
static void Tst_GetConfigTstFromDB (void);
static Tst_Feedback_t Tst_GetFeedbackTypeFromForm (void);
static void Tst_CheckAndCorrectNumbersQst (void);
static void Tst_ShowFormAnswerTypes (void);
static unsigned long Tst_GetQuestionsForEdit (MYSQL_RES **mysql_res);
static unsigned long Tst_GetQuestionsForExam (MYSQL_RES **mysql_res);
static void Tst_ListOneQstToEdit (void);
static unsigned long Tst_GetOneQstToEdit (long QstCod,MYSQL_RES **mysql_res);
static unsigned long Tst_GetAQuestionForExam (long QstCod,MYSQL_RES **mysql_res);
static void Tst_ListOneOrMoreQuestionsToEdit (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Tst_WriteParamEditQst (void);
static unsigned Tst_GetAnswersQst (long QstCod,MYSQL_RES **mysql_res,bool Shuffle);
static void Tst_WriteAnswersOfAQstEdit (long QstCod);
static void Tst_WriteAnswersOfAQstSeeExam (unsigned NumQst,long QstCod,bool Shuffle);
static void Tst_WriteAnswersOfAQstAssessExam (unsigned NumQst,long QstCod,double *MarkThisQst);
static void Tst_WriteFormAnsTF (unsigned NumQst);
static void Tst_WriteAnsTF (char AnsTF);
static void Tst_WriteTFAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst);
static void Tst_WriteChoiceAnsSeeExam (unsigned NumQst,long QstCod,bool Shuffle);
static void Tst_WriteChoiceAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst);
static void Tst_WriteTextAnsSeeExam (unsigned NumQst);
static void Tst_WriteTextAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst);
static void Tst_WriteIntAnsSeeExam (unsigned NumQst);
static void Tst_WriteIntAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst);
static void Tst_WriteFloatAnsSeeExam (unsigned NumQst);
static void Tst_WriteFloatAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst);
static void Tst_WriteHeadUserCorrect (void);
static void Tst_WriteScoreStart (unsigned ColSpan);
static void Tst_WriteScoreEnd (void);
static void Tst_WriteParamQstCod (unsigned NumQst,long QstCod);
static void Tst_CheckIfNumberOfAnswersIsOne (unsigned NumAnswers);
static unsigned long Tst_GetTagsQst (long QstCod,MYSQL_RES **mysql_res);
static void Tst_GetAndWriteTagsQst (long QstCod);
static int Tst_GetParamsTst (void);
static unsigned Tst_GetAndCheckParamNumTst (void);
static unsigned Tst_GetParamNumQst (void);
static int Tst_CountNumTagsInList (void);
static int Tst_CountNumAnswerTypesInList (void);
static void Tst_InitQst (void);
static Tst_AnswerType_t Tst_ConvertFromStrAnsTypDBToAnsTyp (const char *StrAnsTypeBD);
static Tst_AnswerType_t Tst_ConvertFromUnsignedStrToAnsTyp (const char *UnsignedStr);
static double Tst_GetFloatReplyFromStr (char *Str);
static long Tst_GetTagCodeFromTagTxt (const char *TagTxt);
static long Tst_CreateNewTag (long CrsCod,const char *TagTxt);
static void Tst_EnableOrDisableTag (long TagCod,bool TagHidden);
static int Tst_GetQstCod (void);
static void Tst_RemAnsFromQst (long QstCod);
static void Tst_RemTagsFromQst (long QstCod);
static void Tst_RemoveUnusedTagsFromCurrentCrs (void);
static int Tst_AllocateTextChoiceAnswer (unsigned NumAns);
static void Tst_FreeTextChoiceAnswers (unsigned NumAnswers);
static void Tst_FreeTextChoiceAnswer (unsigned NumAns);

/*****************************************************************************/
/*************** Show form to generate a self-assessment test ****************/
/*****************************************************************************/

void Tst_ShowFormAskTst (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_No_of_questions[Txt_NUM_LANGUAGES];
   extern const char *Txt_Generate_test[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_test_questions_in_X[Txt_NUM_LANGUAGES];
   MYSQL_RES *mysql_res;
   unsigned long NumRows;

   /***** Read test configuration from database *****/
   Tst_GetConfigTstFromDB ();

   /***** Get tags *****/
   if ((NumRows = Tst_GetEnabledTagsFromThisCrs (&mysql_res)) != 0)
     {
      /***** Check if minimum date-time of next access to test is older than now *****/
      if (Tst_CheckIfNextTstAllowed ())
        {
         Act_FormStart (ActSeeTst);
         fprintf (Gbl.F.Out,"<div align=\"center\"><table>");

         /***** Selection of tags *****/
         Tst_ShowFormSelTags (NumRows,mysql_res,true);

         /***** Selection of types of answers *****/
         Tst_ShowFormAnswerTypes ();

         /***** Number of questions to generate ****/
         fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\">%s: " \
                            "<input type=\"text\" name=\"NumQst\" size=\"3\" maxlength=\"3\" value=\"%u\"",
                  The_ClassFormul[Gbl.Prefs.Theme],Txt_No_of_questions[Gbl.Prefs.Language],
                  Gbl.Test.Config.Def);
         if (Gbl.Test.Config.Min == Gbl.Test.Config.Max)
            fprintf (Gbl.F.Out," disabled=\"disabled\"");
         fprintf (Gbl.F.Out," /></td></tr>");

         /***** Send button *****/
         fprintf (Gbl.F.Out,"<tr><td align=\"center\">" \
                            "<input type=\"submit\" value=\"%s\" />" \
                            "</td></tr></table></div></form>\n",
                  Txt_Generate_test[Gbl.Prefs.Language]);
        }
     }
   else
     {
      sprintf (Gbl.Message,Txt_No_test_questions_in_X[Gbl.Prefs.Language],
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********************** Generate self-assessment test ************************/
/*****************************************************************************/

void Tst_ShowTestExam (void)
  {
   extern const char *Txt_No_questions_found_matching_your_search_criteria[Txt_NUM_LANGUAGES];
   extern const char *Txt_Send[Txt_NUM_LANGUAGES];
   MYSQL_RES *mysql_res;
   unsigned long NumRows;
   unsigned NumHitsTst;

   /***** Read test configuration from database *****/
   Tst_GetConfigTstFromDB ();

   if (Tst_CheckIfNextTstAllowed ())
     {
      /***** Check that all parameters used to generate a test are valid *****/
      if (Tst_GetParamsTst ())					// Get parameters from form
        {
         /***** Get questions *****/
         if ((NumRows = Tst_GetQuestionsForExam (&mysql_res)) == 0)	// Query database
           {
            Lay_ShowAlert (INFO,Txt_No_questions_found_matching_your_search_criteria[Gbl.Prefs.Language]);
            Tst_ShowFormAskTst ();					// Show the form again
           }
         else
           {
            /***** Get and update number of hits *****/
            NumHitsTst = Tst_GetNumHitsTst () + 1;
            if (Gbl.Usrs.Me.IBelongToThisCrs)
	       Tst_UpdateMyNumAccessTst (NumHitsTst);

            /***** List the questions *****/
            /* Start form */
            Act_FormStart (ActAssTst);
  	    Gbl.Test.NumQuestions = (unsigned) NumRows;
            fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"NumTst\" value=\"%u\" />\n",
                     NumHitsTst);
            fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"NumQst\" value=\"%u\" />\n",
                     Gbl.Test.NumQuestions);

            /* Write test heading */
            Tst_WriteTestHead (NumHitsTst);

            /* Write the questions */
            Tst_ShowTestQuestions (mysql_res);

            /* End of table */
            Tst_WriteTestFoot ();

            /* End form */
            Lay_PutSendButton (Txt_Send[Gbl.Prefs.Language]);
            fprintf (Gbl.F.Out,"</form>\n");

            /***** Set test status *****/
            Tst_SetTstStatus (NumHitsTst,Tst_STATUS_SHOWN_BUT_NOT_ASSESSED);

            /***** Update date-time of my next allowed access to test *****/
            if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT)
               Tst_UpdateNextAccTst (Gbl.Test.NumQuestions * Gbl.Test.Config.MinTimeNxtTstPerQst);
           }

         /***** Free structure that stores the query result *****/
         DB_FreeMySQLResult (&mysql_res);
        }
      else
         Tst_ShowFormAskTst ();							// Show the form again

      /***** Free memory used for por the list of tags *****/
      Tst_FreeTagsList ();
     }
  }

/*****************************************************************************/
/****************************** Assess a test exam ***************************/
/*****************************************************************************/

void Tst_AssessTestExam (void)
  {
   extern const char *Txt_The_number_of_questions_must_be_in_the_interval_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_Score[Txt_NUM_LANGUAGES];
   extern const char *Txt_out_of_PART_OF_A_SCORE[Txt_NUM_LANGUAGES];
   Tst_Status_t TstStatus;
   char UnsignedStr[10+1];
   char StrQstOrAns[3+10+1]; /* "Qstxx...x" or "Ansxx...x" */
   unsigned NumTst;
   unsigned NumQst;
   double TotalMark = 0.0,TotalMarkOver10;

   /***** Read test configuration from database *****/
   Tst_GetConfigTstFromDB ();

   /***** Get number of this test from form *****/
   NumTst = Tst_GetAndCheckParamNumTst ();

   /****** Get test status in database for this session-course-num.test *****/
   TstStatus = Tst_GetTstStatus (NumTst);

   switch (TstStatus)
     {
      case Tst_STATUS_SHOWN_BUT_NOT_ASSESSED:
         /***** Get the parameters of the form *****/
         /* Get number of questions */
         if ((Gbl.Test.NumQuestions = Tst_GetParamNumQst ()) == UINT_MAX)
           {
            sprintf (Gbl.Message,Txt_The_number_of_questions_must_be_in_the_interval_X[Gbl.Prefs.Language],
                     Gbl.Test.Config.Min,Gbl.Test.Config.Max);
            Lay_ShowAlert (ERROR,Gbl.Message);
           }
         else
           {
            /***** Get questions, indexes, and answers *****/
            for (NumQst=0; NumQst<Gbl.Test.NumQuestions; NumQst++)
              {
               /* Get question code */
              sprintf (StrQstOrAns,"Qst%06u",NumQst);
               Par_GetParam (Par_PARAM_SINGLE,StrQstOrAns,UnsignedStr,10);
               if (sscanf (UnsignedStr,"%ld",&Gbl.Test.QstCodes[NumQst]) != 1)
                  Lay_ShowErrorAndExit ("Code of question is missing.");

               /* Get answers selected by user for this question */
               sprintf (StrQstOrAns,"Ans%06u",NumQst);
               Par_GetParam (Par_PARAM_MULTIPLE,StrQstOrAns,Gbl.Test.StrAnswersUsr[NumQst],Tst_MAX_LENGTH_ANS_USR);  /* If answer type == T/F ==> " ", "T", "F"; if choice ==> "0", "2",... */
               Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Test.StrAnswersUsr[NumQst],Tst_MAX_LENGTH_ANS_USR,true);
              }

            /***** Write head of test *****/
            Tst_WriteTestHead (NumTst);

            /***** Write answers and solutions *****/
            Tst_ShowTstResult (&TotalMark);

            /***** Write total mark of test *****/
            if (Gbl.Test.Config.FeedbackType != Tst_FEEDBACK_NOTHING)
              {
               TotalMarkOver10 = TotalMark * 10.0 / (float) Gbl.Test.NumQuestions;
               fprintf (Gbl.F.Out,"<tr><td colspan=\"3\" align=\"center\" class=\"DAT\">%s: " \
                                  "<span class=\"%s\">%.2lf (%.2lf %s 10)</span></td></tr>",
                        Txt_Score[Gbl.Prefs.Language],
                        (TotalMarkOver10 >= 5.0) ? "ANS_OK" : "ANS_BAD",TotalMark,
                        TotalMarkOver10,Txt_out_of_PART_OF_A_SCORE[Gbl.Prefs.Language]);
              }

            /***** End of table *****/
            Tst_WriteTestFoot ();
           }

         /***** Set test status *****/
         Tst_SetTstStatus (NumTst,Tst_STATUS_ASSESSED);
         break;
      case Tst_STATUS_ASSESSED:
         sprintf (Gbl.Message,"El test %u ya ha sido autoevaluado anteriormente.",	// Need translation!
                  NumTst);
         Lay_ShowAlert (WARNING,Gbl.Message);
         break;
      case Tst_STATUS_ERROR:
         sprintf (Gbl.Message,"Se ha producido un error al evaluar el test %u.",	// Need translation!
                  NumTst);
         Lay_ShowAlert (WARNING,Gbl.Message);
         break;
     }
  }

/*****************************************************************************/
/************** Check minimum date-time of next access to test ***************/
/*****************************************************************************/
// Return true if allowed date-time of next access to test is older than now

static bool Tst_CheckIfNextTstAllowed (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   int NumSecondsFromNowToNextAccTst = -1;	// Access allowed when this number <= 0
   unsigned Year,Month,Day,Hour,Minute,Second;
   
   /***** Superusers are allowed to do all test they want *****/
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
      return true;

   /***** Get number of hits to test from database *****/
   sprintf (Query,"SELECT UNIX_TIMESTAMP(NextAccTst)-UNIX_TIMESTAMP(),NextAccTst" \
                  " FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get number of hits to test");

   if (NumRows == 1)
     {
      /* Get seconds from now to next access to test */
      row = mysql_fetch_row (mysql_res);
      if (row[0])
         if (sscanf (row[0],"%d",&NumSecondsFromNowToNextAccTst) == 1)
            /* Datetime of next access allowed (row[1]) */
            if (sscanf (row[1],"%04u-%02u-%02u %02u:%02u:%02u",&Year,&Month,&Day,&Hour,&Minute,&Second) != 6)
               Lay_ShowErrorAndExit ("Wrong date of next allowed access to test.");
     }
   else
      Lay_ShowErrorAndExit ("Error when reading date of next allowed access to test.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Check if access is allowed *****/
   if (NumSecondsFromNowToNextAccTst > 0)
     {
      if (Year  == Gbl.Today.Year &&
          Month == Gbl.Today.Month &&
          Day   == Gbl.Today.Day)		// Next access allowed is today
         sprintf (Gbl.Message,"No puede hacer un nuevo test en <strong>%s</strong> hasta las %02uh%02u'%02u&quot;",	// Need translation!
                  Gbl.CurrentCrs.Crs.FullName,Hour,Minute,Second);
      else
         sprintf (Gbl.Message,"No puede hacer un nuevo test en <strong>%s</strong> hasta las %02uh%02u'%02u&quot; del %02u/%02u/%04u",	// Need translation!
                  Gbl.CurrentCrs.Crs.FullName,Hour,Minute,Second,Day,Month,Year);
      Lay_ShowAlert (WARNING,Gbl.Message);
      return false;
     }
   return true;
  }

/*****************************************************************************/
/****************************** Update test status ***************************/
/*****************************************************************************/

static void Tst_SetTstStatus (unsigned NumTst,Tst_Status_t TstStatus)
  {
   char Query[512];

   /***** Delete old status from expired sessions *****/
   sprintf (Query,"DELETE FROM tst_status" \
                  " WHERE SessionId NOT IN (SELECT SessionId FROM sessions)");
   DB_QueryDELETE (Query,"can not remove old status of tests");

   /***** Update database *****/
   sprintf (Query,"REPLACE INTO tst_status (SessionId,CrsCod,NumTst,Status) VALUES ('%s','%ld','%u','%u')",
            Gbl.Session.Id,Gbl.CurrentCrs.Crs.CrsCod,NumTst,(unsigned) TstStatus);
   DB_QueryREPLACE (Query,"can not update status of test");
  }

/*****************************************************************************/
/****************************** Update test status ***************************/
/*****************************************************************************/

Tst_Status_t Tst_GetTstStatus (unsigned NumTst)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned UnsignedNum;
   Tst_Status_t TstStatus = Tst_STATUS_ERROR;

   /***** Get status of test from database *****/
   sprintf (Query,"SELECT Status FROM tst_status" \
                  " WHERE SessionId='%s' AND CrsCod='%ld' AND NumTst='%u'",
            Gbl.Session.Id,Gbl.CurrentCrs.Crs.CrsCod,NumTst);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get status of test");

   if (NumRows == 1)
     {
      /* Get number of hits */
      row = mysql_fetch_row (mysql_res);
      if (row[0])
         if (sscanf (row[0],"%u",&UnsignedNum) == 1)
            if (UnsignedNum < Tst_NUM_STATUS)
               TstStatus = (Tst_Status_t) UnsignedNum;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return TstStatus;
  }

/*****************************************************************************/
/************************* Get number of hits to test ************************/
/*****************************************************************************/

unsigned Tst_GetNumHitsTst (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumHitsTst = 0;

   if (Gbl.Usrs.Me.IBelongToThisCrs)
     {
      /***** Get number of hits to test from database *****/
      sprintf (Query,"SELECT NumAccTst FROM crs_usr" \
                     " WHERE CrsCod='%ld' AND UsrCod='%ld'",
               Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get number of hits to test");

      if (NumRows == 0)
         NumHitsTst = 0;
      else if (NumRows == 1)
        {
         /* Get number of hits */
         row = mysql_fetch_row (mysql_res);
         if (row[0] == NULL)
            NumHitsTst = 0;
         else if (sscanf (row[0],"%u",&NumHitsTst) != 1)
            NumHitsTst = 0;
        }
      else
         Lay_ShowErrorAndExit ("Error when getting number of hits to test.");

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }

   return NumHitsTst;
  }

/*****************************************************************************/
/***************** Write the head of the table with the test **************/
/*****************************************************************************/

static void Tst_WriteTestHead (unsigned NumTst)
  {
   extern const char *Txt_Self_assessment_test[Txt_NUM_LANGUAGES];
   extern const char *Txt_Test_result[Txt_NUM_LANGUAGES];
   extern const char *Txt_Test_No_X_that_you_make_in_this_course[Txt_NUM_LANGUAGES];

   /***** Start table *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   Lay_WriteHeaderClassPhoto (3,false,false,
                            Gbl.CurrentDeg.Deg.InsCod,Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod);

   /***** Header row *****/
   fprintf (Gbl.F.Out,"<tr><td colspan=\"3\" align=\"center\" class=\"TST_TIT\">" \
                      "&nbsp;<br /><strong>%s</strong>",
            (Gbl.CurrentAct == ActSeeTst) ? Txt_Self_assessment_test[Gbl.Prefs.Language] :
                                            Txt_Test_result[Gbl.Prefs.Language]);
   if (Gbl.Usrs.Me.IBelongToThisCrs)
     {
      fprintf (Gbl.F.Out,"<br />");
      fprintf (Gbl.F.Out,Txt_Test_No_X_that_you_make_in_this_course[Gbl.Prefs.Language],NumTst);
     }
   fprintf (Gbl.F.Out,"<br />&nbsp;</td></tr>\n");
  }

/*****************************************************************************/
/******************* Write the foot of the table with the test ***************/
/*****************************************************************************/

static void Tst_WriteTestFoot (void)
  {
   /***** End table *****/
   Lay_WriteFootClassPhoto ();
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/*************************** Write the test questions ************************/
/*****************************************************************************/
// NumRows must hold the number of rows of a MySQL query
// In each row mysql_res holds: in the column 0 the code of a question, in the column 1 the type of answer, and in the column 2 the stem

static void Tst_ShowTestQuestions (MYSQL_RES *mysql_res)
  {
   unsigned NumQst;
   long QstCod;
   MYSQL_ROW row;

   /***** Write rows *****/
   for (NumQst=0; NumQst<Gbl.Test.NumQuestions; NumQst++)
     {
      Gbl.GlobalRowEvenOdd = NumQst % 2;

      /***** Get the row next of the result of the query in the database *****/
      row = mysql_fetch_row (mysql_res);

      /***** Get the code of question (row[0]) *****/
      if (sscanf (row[0],"%ld",&QstCod) != 1)
         Lay_ShowErrorAndExit ("Wrong code of question.");

      Tst_WriteQstAndAnsExam (NumQst,QstCod,row,NULL);
     }
  }

/*****************************************************************************/
/************************** Show the result of a test ************************/
/*****************************************************************************/

static void Tst_ShowTstResult (double *TotalMark)
  {
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumQst;
   long QstCod;
   double MarkThisQst;

   for (NumQst=0; NumQst<Gbl.Test.NumQuestions; NumQst++)
     {
      Gbl.GlobalRowEvenOdd = NumQst % 2;

      /***** Query database *****/
      if ((Tst_GetAQuestionForExam (Gbl.Test.QstCodes[NumQst],&mysql_res)) == 1)
        {
         /***** Get the row next of the result of the query in the database *****/
         row = mysql_fetch_row (mysql_res);

         /***** Get the code of question (row[0]) *****/
         if (sscanf (row[0],"%ld",&QstCod) != 1)
            Lay_ShowErrorAndExit ("Wrong code of question.");

         Tst_WriteQstAndAnsExam (NumQst,QstCod,row,&MarkThisQst);
         *TotalMark += MarkThisQst;

         /***** Update the number of accesos y the puntuacin of esta question *****/
         if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT)
            Tst_UpdateMarkQst (QstCod,MarkThisQst);
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
  }

/*****************************************************************************/
/******** Write a row of a test exam, with one question and its answer *******/
/*****************************************************************************/

static void Tst_WriteQstAndAnsExam (unsigned NumQst,long QstCod,MYSQL_ROW row,double *MarkThisQst)
  {
   extern const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES][Tst_NUM_ANS_TYPES];

   /***** Write number of question *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_N\">%u.&nbsp;</td>",
            Gbl.ColorRows[Gbl.GlobalRowEvenOdd],NumQst+1);

   /***** Write answer type (row[1]) *****/
   Gbl.Test.AnswerType = Tst_ConvertFromStrAnsTypDBToAnsTyp (row[1]);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_SMALL\">%s&nbsp;</td>",
            Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
            Txt_TST_STR_ANSWER_TYPES[Gbl.Test.AnswerType][Gbl.Prefs.Language]);

   /***** Write heading (row[3]) *****/
   Tst_WriteQstStem (row[3],"TEST_EXA");

   /***** Write the answer(s) *****/
   if (Gbl.CurrentAct == ActSeeTst)
      Tst_WriteAnswersOfAQstSeeExam (NumQst,QstCod,(Str_ConvertToUpperLetter (row[2][0]) != 'N'));
   else
      Tst_WriteAnswersOfAQstAssessExam (NumQst,QstCod,MarkThisQst);

   fprintf (Gbl.F.Out,"</td></tr>");
  }

/*****************************************************************************/
/******************* Write the heading of a test question ********************/
/*****************************************************************************/

static void Tst_WriteQstStem (const char *Stem,const char *TextStyle)
  {
   unsigned long LengthHeading;
   char *HeadingRigorousHTML;

   /* Convert the stem, that is in HTML, to rigorous HTML */
   LengthHeading = strlen (Stem) * Str_MAX_LENGTH_SPEC_CHAR_HTML;
   if ((HeadingRigorousHTML = malloc (LengthHeading+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store stem of question.");
   strcpy (HeadingRigorousHTML,Stem);
   Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,HeadingRigorousHTML,LengthHeading,false);

   /* Write the stem */
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"%s\">" \
                      "<p align=\"justify\"><tt>%s</tt></p>",
	    Gbl.ColorRows[Gbl.GlobalRowEvenOdd],TextStyle,HeadingRigorousHTML);

   /* Free memory allocated for the stem */
   free ((void *) HeadingRigorousHTML);
  }


/****************************************************************************/
/*********************** Update the score of a question *********************/
/****************************************************************************/

static void Tst_UpdateMarkQst (long QstCod,float MarkThisQst)
  {
   char Query[512];

   /***** Update number of clicks and score of the question *****/
   setlocale (LC_NUMERIC,"en_US.iso88591");	// To print the floating point as a dot
   sprintf (Query,"UPDATE tst_questions SET NumHits=NumHits+1,Score=Score+(%lf)" \
                  " WHERE QstCod='%ld'",
            MarkThisQst,QstCod);
   setlocale (LC_NUMERIC,"es_ES.iso88591");
   DB_QueryUPDATE (Query,"can not update the score of a question");
  }

/****************************************************************************/
/************** Update my number of hits to test in this course *************/
/****************************************************************************/

static void Tst_UpdateMyNumAccessTst (unsigned NumHitsTst)
  {
   char Query[512];

   /***** Update my number of hits to test in this course *****/
   sprintf (Query,"UPDATE crs_usr SET NumAccTst='%u' WHERE CrsCod='%ld' AND UsrCod='%ld'",
	    NumHitsTst,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update the number of hits to test");
  }

/****************************************************************************/
/************ Update date-time of my next allowed access to test ************/
/****************************************************************************/

static void Tst_UpdateNextAccTst (unsigned NumSecondsUntilNextAccess)
  {
   char Query[512];

   /***** Update date-time of my next allowed access to test *****/
   sprintf (Query,"UPDATE crs_usr SET NextAccTst=NOW()+INTERVAL %u SECOND WHERE CrsCod='%ld' AND UsrCod='%ld'",
	    NumSecondsUntilNextAccess,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update the time of next allowed access to test");
  }

/*****************************************************************************/
/******* Select tags and dates for edition of the self-assessment test *******/
/*****************************************************************************/

void Tst_ShowFormAskEditTsts (void)
  {
   extern const char *Txt_No_test_questions_in_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_Show_questions[Txt_NUM_LANGUAGES];
   MYSQL_RES *mysql_res;
   unsigned long NumRows;

   /***** Get tags already present in the table of questions *****/
   if ((NumRows = Tst_GetAllTagsFromCurrentCrs (&mysql_res)) == 0)
     {
      sprintf (Gbl.Message,Txt_No_test_questions_in_X[Gbl.Prefs.Language],
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      Act_FormStart (ActLstTstQst);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
               (unsigned) Tst_ORDER_HEADING);

      fprintf (Gbl.F.Out,"<div align=\"center\"><table>\n");

      /***** Selection of tags *****/
      Tst_ShowFormSelTags (NumRows,mysql_res,false);

      /***** Selection of types of answers *****/
      Tst_ShowFormAnswerTypes ();

      /***** Starting and ending dates in the search *****/
      Dat_WriteFormIniEndDates ();

      /***** Send button *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"center\"><input type=\"submit\" value=\"%s\" /></td></tr>" \
                         "</table></div></form>\n",
               Txt_Show_questions[Gbl.Prefs.Language]);
     }

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/***************************** Form to rename tags ***************************/
/*****************************************************************************/

void Tst_ShowFormConfig (void)
  {
   /***** Form to edit tags *****/
   Tst_ShowFormEditTags ();

   /***** Form to configure test *****/
   Tst_ShowFormConfigTst ();
  }

/*****************************************************************************/
/******************************* Enable a test tag ***************************/
/*****************************************************************************/

void Tst_EnableTag (void)
  {
   long TagCod = Tst_GetParamTagCode ();

   /***** Change tag status to enabled *****/
   Tst_EnableOrDisableTag (TagCod,false);

   /***** Show again the form to configure test *****/
   Tst_ShowFormConfig ();
  }

/*****************************************************************************/
/****************************** Disable a test tag ***************************/
/*****************************************************************************/

void Tst_DisableTag (void)
  {
   long TagCod = Tst_GetParamTagCode ();

   /***** Change tag status to disabled *****/
   Tst_EnableOrDisableTag (TagCod,true);

   /***** Show again the form to configure test *****/
   Tst_ShowFormConfig ();
  }

/*****************************************************************************/
/************************* Get parameter with tag code ***********************/
/*****************************************************************************/

static long Tst_GetParamTagCode (void)
  {
   char StrLong[1+10+1];
   long TagCod;

   Par_GetParam (Par_PARAM_SINGLE,"TagCod",StrLong,1+10);
   if ((TagCod = Str_ConvertStrCodToLongCod (StrLong)) == -1)
      Lay_ShowErrorAndExit ("Code of tag is missing.");

   return TagCod;
  }

/*****************************************************************************/
/************************ Rename a tag of test questions *********************/
/*****************************************************************************/

void Tst_RenameTag (void)
  {
   char OldTag[Tst_MAX_BYTES_TAG+1],NewTag[Tst_MAX_BYTES_TAG+1];
   char Query[1024+2*Tst_MAX_BYTES_TAG];
   MYSQL_RES *mysql_res;
   unsigned long NumRows;
   unsigned long NumRow;
   MYSQL_ROW row;
   bool RepeatedTag = false;

   /***** Get old and new tags from the form *****/
   Par_GetParam (Par_PARAM_SINGLE,"OldTagTxt",OldTag,Tst_MAX_BYTES_TAG);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,OldTag,Tst_MAX_BYTES_TAG,true);

   Par_GetParam (Par_PARAM_SINGLE,"NewTagTxt",NewTag,Tst_MAX_BYTES_TAG);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewTag,Tst_MAX_BYTES_TAG,true);

   /***** Check that the new tag is not empty *****/
   if (!NewTag[0])
     {
      sprintf (Gbl.Message,"No puede dejar el descriptor <strong>%s</strong> vac&iacute;o.",	// Need translation!
               OldTag);
      Lay_ShowAlert (ERROR,Gbl.Message);
     }
   else
     {
      /***** Check if the old tag is equal to the new one (this happens when user press INTRO without changing anything in the form) *****/
      if (strcmp (OldTag,NewTag))	// The tag has changed
        {
         /***** Check if the new tag is equal to any of the tags present in the database *****/
         if ((NumRows = Tst_GetAllTagsFromCurrentCrs (&mysql_res)) != 0)
            for (NumRow=0; NumRow<NumRows; NumRow++)
              {
               row = mysql_fetch_row (mysql_res);
               if (!strcmp (row[1],NewTag))	// If a tag from the database is equal to the new one...
                 {
                  RepeatedTag = true;
	          break;
                 }
	      }
         /* Free structure that stores the query result */
         DB_FreeMySQLResult (&mysql_res);

         if (RepeatedTag)	// The new tag was already in database
           {
            /***** Complex update made to not repeat tags in the same question *****/
            /* Step 1: Get all questions that already had the new tag from database */
            sprintf (Query,"SELECT tst_questions.QstCod FROM tst_questions,tst_question_tags,tst_tags" \
                           " WHERE tst_questions.CrsCod='%ld' AND tst_question_tags.QstCod=tst_questions.QstCod" \
                           " AND tst_tags.TagCod=tst_question_tags.TagCod AND tst_tags.CrsCod='%ld' AND tst_tags.TagTxt='%s'",
                     Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod,NewTag);
            NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get what questions already had a tag");

            /* Step 2: For each question that already had the new tag, remove the old tag in order to not repeat the new */
            for (NumRow=0; NumRow<NumRows; NumRow++)
              {
               row = mysql_fetch_row (mysql_res);

               /* Delete tag from tst_question_tags */
               sprintf (Query,"DELETE tst_question_tags FROM tst_question_tags,tst_tags" \
                              " WHERE tst_question_tags.QstCod='%s' AND tst_question_tags.TagCod=tst_tags.TagCod" \
                              " AND tst_tags.CrsCod='%ld' AND tst_tags.TagTxt='%s'",
                        row[0],Gbl.CurrentCrs.Crs.CrsCod,OldTag);
               DB_QueryDELETE (Query,"can not remove a tag from questions");

               /* Delete old tag from tst_tags if it is not used more */
               Tst_RemoveUnusedTagsFromCurrentCrs ();
              }
            /* Free structure that stores the query result */
            DB_FreeMySQLResult (&mysql_res);
           }
         /***** Simple update replacing each instance of the old tag by the new tag *****/
         sprintf (Query,"UPDATE tst_tags SET TagTxt='%s' WHERE tst_tags.CrsCod='%ld' AND tst_tags.TagTxt='%s'",
                  NewTag,Gbl.CurrentCrs.Crs.CrsCod,OldTag);
         DB_QueryUPDATE (Query,"can not update tag");

         /***** Write message to show the change made *****/
         sprintf (Gbl.Message,"El descriptor <strong>%s</strong>" \
                              " ha sido renombrado como <strong>%s</strong>.",	// Need translation!
                  OldTag,NewTag);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
      else	// The tags are the same
        {
         sprintf (Gbl.Message,"El descriptor <strong>%s</strong> no ha cambiado.",	// Need translation!
                  NewTag);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Show again the form to configure test *****/
   Tst_ShowFormConfig ();
  }

/*****************************************************************************/
/********* Get all (enabled or disabled) test tags for this course ***********/
/*****************************************************************************/
// Return the number of rows of the result

static unsigned long Tst_GetAllTagsFromCurrentCrs (MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get available tags from database *****/
   sprintf (Query,"SELECT TagCod,TagTxt,TagHidden FROM tst_tags" \
                  " WHERE CrsCod='%ld' ORDER BY TagTxt",
            Gbl.CurrentCrs.Crs.CrsCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get available tags");
  }

/*****************************************************************************/
/********************** Get enabled test tags for this course ****************/
/*****************************************************************************/
// Return the number of rows of the result

static unsigned long Tst_GetEnabledTagsFromThisCrs (MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get available enabled tags from database *****/
   sprintf (Query,"SELECT TagCod,TagTxt FROM tst_tags" \
                  " WHERE CrsCod='%ld' AND TagHidden='N' ORDER BY TagTxt",
            Gbl.CurrentCrs.Crs.CrsCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get available enabled tags");
  }

/*****************************************************************************/
/********************* Show a form to select test tags ***********************/
/*****************************************************************************/

static void Tst_ShowFormSelTags (unsigned long NumRows,MYSQL_RES *mysql_res,bool ShowOnlyEnabledTags)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_All_tags[Txt_NUM_LANGUAGES];
   unsigned long NumRow;
   MYSQL_ROW row;
   bool TagHidden = false;
   const char *Ptr;
   char TagText[Tst_MAX_BYTES_TAG+1];

   /***** Start table *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\">\n");
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Select all tags *****/
   fprintf (Gbl.F.Out,"<tr><td");
   if (!ShowOnlyEnabledTags)
      fprintf (Gbl.F.Out," colspan=\"2\"");
   fprintf (Gbl.F.Out," class=\"%s\"><input type=\"checkbox\" name=\"AllTags\" value=\"Y\"",
            The_ClassFormul[Gbl.Prefs.Theme]);
   if (Gbl.Test.AllTags)
    fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," onclick=\"togglecheckChildren(this,'ChkTag')\" />" \
                      "%s</td></tr>",
            Txt_All_tags[Gbl.Prefs.Language]);

   /***** Select tags one by one *****/
   for (NumRow=1; NumRow<=NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);
      fprintf (Gbl.F.Out,"<tr>");
      if (!ShowOnlyEnabledTags)
        {
         TagHidden = (Str_ConvertToUpperLetter (row[2][0]) != 'N');
         fprintf (Gbl.F.Out,"<td align=\"left\"><img src=\"%s/",Gbl.Prefs.IconsURL);
         if (TagHidden)
            fprintf (Gbl.F.Out,"proh_off12x12.gif\" alt=\"Descriptor prohibido");
         else
            fprintf (Gbl.F.Out,"finp_off12x12.gif\" alt=\"Descriptor permitido");
         fprintf (Gbl.F.Out,"\" width=\"12\" height=\"12\" /></td>");
        }
      fprintf (Gbl.F.Out,"<td align=\"left\" class=\"DAT\"><input type=\"checkbox\" name=\"ChkTag\" value=\"%s\"",row[1]);
      if (Gbl.Test.TagsList)
        {
         Ptr = Gbl.Test.TagsList;
         while (*Ptr)
           {
            Str_GetNextStrUntilSeparParamMult (&Ptr,TagText,Tst_MAX_BYTES_TAG);
            if (!strcmp (row[1],TagText))
               fprintf (Gbl.F.Out," checked=\"checked\"");
           }
        }
      fprintf (Gbl.F.Out," onclick=\"checkParent(this,'AllTags')\" />%s</td></tr>",row[1]);
     }

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr>\n");
  }

/*****************************************************************************/
/************* Show a form to enable/disable and rename test tags ************/
/*****************************************************************************/

static void Tst_ShowFormEditTags (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_No_test_questions_in_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_Tags[Txt_NUM_LANGUAGES];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long TagCod;

   /***** Get current tags in current course *****/
   if ((NumRows = Tst_GetAllTagsFromCurrentCrs (&mysql_res)) == 0)
     {
      sprintf (Gbl.Message,Txt_No_test_questions_in_X[Gbl.Prefs.Language],
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }
   else
     {
      /***** Start table *****/
      fprintf (Gbl.F.Out,"<div align=\"center\">");
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /***** Header of table with title *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" colspan=\"2\" class=\"%s\">%s</td></tr>\n",
               The_ClassFormul[Gbl.Prefs.Theme],Txt_Tags[Gbl.Prefs.Language]);

      /***** Show tags *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
         row = mysql_fetch_row (mysql_res);
         if (sscanf (row[0],"%ld",&TagCod) != 1)
            Lay_ShowErrorAndExit ("Wrong code of tag.");

         fprintf (Gbl.F.Out,"<tr>");

         /* Form to enable / disable this tag */
         if (Str_ConvertToUpperLetter (row[2][0]) != 'N')	// Tag disabled
            Tst_PutIconEnable (TagCod,row[1]);
         else
            Tst_PutIconDisable (TagCod,row[1]);

         /* Form to rename this tag */
         fprintf (Gbl.F.Out,"<td align=\"left\">");
         Act_FormStart (ActRenTag);
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OldTagTxt\" value=\"%s\" />",row[1]);
         fprintf (Gbl.F.Out,"<input type=\"text\" name=\"NewTagTxt\" size=\"36\" maxlength=\"%u\" value=\"%s\" onchange=\"javascript:document.forms[%d].submit();\" />",
                  Tst_MAX_TAG_LENGTH,row[1],Gbl.NumForm);
         fprintf (Gbl.F.Out,"</form></td></tr>");
        }

      /***** End table *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
      fprintf (Gbl.F.Out,"</div>\n");
     }

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/******************* Put a link and an icon to enable a tag ******************/
/*****************************************************************************/

static void Tst_PutIconEnable (long TagCod,const char *TagTxt)
  {
   fprintf (Gbl.F.Out,"<td class=\"BM\">");
   Act_FormStart (ActEnableTag);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"TagCod\" value=\"%ld\" />",TagCod);
   sprintf (Gbl.MsgWindowStatus,"Permitir descriptor %s",TagTxt);	// Need translation!
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/proh12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      "</acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
  }

/*****************************************************************************/
/****************** Put a link and an icon to disable a tag ******************/
/*****************************************************************************/

static void Tst_PutIconDisable (long TagCod,const char *TagTxt)
  {
   fprintf (Gbl.F.Out,"<td class=\"BM\">");
   Act_FormStart (ActDisableTag);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"TagCod\" value=\"%ld\" />",TagCod);
   sprintf (Gbl.MsgWindowStatus,"Prohibir descriptor %s",TagTxt);	// Need translation!
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/finp12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      "</acronym></a></form></td>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);
  }

/*****************************************************************************/
/********************* Show a form to to configure test **********************/
/*****************************************************************************/

static void Tst_ShowFormConfigTst (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_No_of_questions[Txt_NUM_LANGUAGES];
   extern const char *Txt_minimum[Txt_NUM_LANGUAGES];
   extern const char *Txt_default[Txt_NUM_LANGUAGES];
   extern const char *Txt_maximum[Txt_NUM_LANGUAGES];
   extern const char *Txt_Minimum_time_seconds_per_question_between_two_tests[Txt_NUM_LANGUAGES];
   extern const char *Txt_Feedback_to_students[Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_STR_FEEDBACK[Tst_NUM_TYPES_FEEDBACK][Txt_NUM_LANGUAGES];
   extern const char *Txt_Send[Txt_NUM_LANGUAGES];
   Tst_Feedback_t FeedbTyp;

   /***** Read test configuration from database *****/
   Tst_GetConfigTstFromDB ();

   /***** Start form *****/
   Act_FormStart (ActRecCfgTst);
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Number of questions *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\"><table cellspacing=\"5\" cellpadding=\"0\">" \
                      "<tr><td align=\"right\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\"><table cellspacing=\"2\" cellpadding=\"0\">",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_No_of_questions[Gbl.Prefs.Language]);

   /* Minimum number of questions in a test exam */
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"DAT\">%s</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"NumQstMin\" size=\"3\" maxlength=\"3\" value=\"%u\" /></td></tr>\n",
            Txt_minimum[Gbl.Prefs.Language],Gbl.Test.Config.Min);

   /* Default number of questions in a test exam */
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"DAT\">%s</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"NumQstDef\" size=\"3\" maxlength=\"3\" value=\"%u\" /></td></tr>\n",
            Txt_default[Gbl.Prefs.Language],Gbl.Test.Config.Def);

   /* Maximum number of questions in a test exam */
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"DAT\">%s</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"NumQstMax\" size=\"3\" maxlength=\"3\" value=\"%u\" /></td></tr>\n</table></td></tr>\n",
            Txt_maximum[Gbl.Prefs.Language],Gbl.Test.Config.Max);

   /***** Minimum time between test exams per question *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\"><input type=\"text\" name=\"MinTimeNxtTstPerQst\" size=\"4\" maxlength=\"4\" value=\"%u\" /></td></tr>\n",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Minimum_time_seconds_per_question_between_two_tests[Gbl.Prefs.Language],
            Gbl.Test.Config.MinTimeNxtTstPerQst);

   /***** Feedback to students *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"%s\">%s:</td><td align=\"left\" valign=\"top\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Feedback_to_students[Gbl.Prefs.Language]);
   for (FeedbTyp=(Sta_UsrTypeStat_t) 0; FeedbTyp<Tst_NUM_TYPES_FEEDBACK; FeedbTyp++)
     {
      fprintf (Gbl.F.Out,"<input type=\"radio\" name=\"Feedback\" value=\"%u\"",(unsigned) FeedbTyp);
      if (FeedbTyp == Gbl.Test.Config.FeedbackType)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," /><span class=\"DAT\">%s</span><br />",
               Txt_TST_STR_FEEDBACK[FeedbTyp][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** End of table *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" colspan=\"2\">");
   Lay_PutSendAndUndoButtons (Txt_Send[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</td></tr>\n</table></td></tr>\n");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</form>\n");
  }

/*****************************************************************************/
/*************** Get configuration of test for current course ****************/
/*****************************************************************************/

static void Tst_GetConfigTstFromDB (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   int IntNum;
   Tst_Feedback_t FeedbTyp;

   /***** Get configuration of test for current course from database *****/
   sprintf (Query,"SELECT Min,Def,Max,MinTimeNxtTstPerQst,Feedback FROM tst_config" \
                  " WHERE CrsCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get configuration of test");

   Gbl.Test.Config.FeedbackType = Tst_FEEDBACK_DEFAULT;
   Gbl.Test.Config.MinTimeNxtTstPerQst = 0;
   if (NumRows == 0)
     {
      Gbl.Test.Config.Min = Tst_CONFIG_DEFAULT_MIN_QUESTIONS;
      Gbl.Test.Config.Def = Tst_CONFIG_DEFAULT_DEF_QUESTIONS;
      Gbl.Test.Config.Max = Tst_CONFIG_DEFAULT_MAX_QUESTIONS;
     }
   else // NumRows == 1
     {
      /***** Get minimun, default and maximum *****/
      row = mysql_fetch_row (mysql_res);

      if (sscanf (row[0],"%d",&IntNum) == 1)
         Gbl.Test.Config.Min = (IntNum < 1) ? 1 : (unsigned) IntNum;
      else
         Gbl.Test.Config.Min = Tst_CONFIG_DEFAULT_MIN_QUESTIONS;

      if (sscanf (row[1],"%d",&IntNum) == 1)
         Gbl.Test.Config.Def = (IntNum < 1) ? 1 : (unsigned) IntNum;
      else
         Gbl.Test.Config.Def = Tst_CONFIG_DEFAULT_DEF_QUESTIONS;

      if (sscanf (row[2],"%d",&IntNum) == 1)
         Gbl.Test.Config.Max = (IntNum < 1) ? 1 : (unsigned) IntNum;
      else
         Gbl.Test.Config.Max = Tst_CONFIG_DEFAULT_MAX_QUESTIONS;

      /***** Check and correct numbers *****/
      Tst_CheckAndCorrectNumbersQst ();

      /***** Get minimum time between test exams per question (row[3]) *****/
      if (sscanf (row[3],"%d",&IntNum) == 1)
         Gbl.Test.Config.MinTimeNxtTstPerQst = (IntNum < 1) ? 0 : (unsigned) IntNum;

      /***** Get feedback type (row[4]) *****/
      for (FeedbTyp=(Sta_UsrTypeStat_t) 0; FeedbTyp<Tst_NUM_TYPES_FEEDBACK; FeedbTyp++)
         if (!strcmp (row[4],Tst_FeedbackDB[FeedbTyp]))
           {
            Gbl.Test.Config.FeedbackType = FeedbTyp;
            break;
           }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************* Receive configuration of test for current course **************/
/*****************************************************************************/

void Tst_ReceiveConfigTst (void)
  {
   char IntStr[1+10+1];
   int IntNum;
   char Query[512];

   /***** Get number of questions *****/
   /* Get minimum number of questions */
   Par_GetParam (Par_PARAM_SINGLE,"NumQstMin",IntStr,1+10);
   if (sscanf (IntStr,"%d",&IntNum) != 1)
      Lay_ShowErrorAndExit ("Minimum number of questions is missing.");
   Gbl.Test.Config.Min = (IntNum < 1) ? 1 : (unsigned) IntNum;

   /* Get default number of questions */
   Par_GetParam (Par_PARAM_SINGLE,"NumQstDef",IntStr,1+10);
   if (sscanf (IntStr,"%d",&IntNum) != 1)
      Lay_ShowErrorAndExit ("Default number of questions is missing.");
   Gbl.Test.Config.Def = (IntNum < 1) ? 1 : (unsigned) IntNum;

   /* Get maximum number of questions */
   Par_GetParam (Par_PARAM_SINGLE,"NumQstMax",IntStr,1+10);
   if (sscanf (IntStr,"%d",&IntNum) != 1)
      Lay_ShowErrorAndExit ("Maximum number of questions is missing.");
   Gbl.Test.Config.Max = (IntNum < 1) ? 1 : (unsigned) IntNum;

   /* Check and correct numbers */
   Tst_CheckAndCorrectNumbersQst ();

   /***** Get minimum time between test exams per question *****/
   Par_GetParam (Par_PARAM_SINGLE,"MinTimeNxtTstPerQst",IntStr,1+10);
   if (sscanf (IntStr,"%d",&IntNum) != 1)
      Lay_ShowErrorAndExit ("Minimum time per question between two test is missing.");
   Gbl.Test.Config.MinTimeNxtTstPerQst = (IntNum < 1) ? 0 : (unsigned) IntNum;

   /***** Get type of feedback from form *****/
   Gbl.Test.Config.FeedbackType = Tst_GetFeedbackTypeFromForm ();

   /***** Update database *****/
   sprintf (Query,"REPLACE INTO tst_config (CrsCod,Min,Def,Max,MinTimeNxtTstPerQst,Feedback)" \
                  " VALUES ('%ld','%u','%u','%u','%u','%s')",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Test.Config.Min,Gbl.Test.Config.Def,Gbl.Test.Config.Max,
            Gbl.Test.Config.MinTimeNxtTstPerQst,
            Tst_FeedbackDB[Gbl.Test.Config.FeedbackType]);
   DB_QueryREPLACE (Query,"can not save configuration of test");

   /***** Show confirmation message *****/
   Lay_ShowAlert (INFO,"Configuraci&oacute;n de test actualizada.");	// Need translation!

   /***** Show again the form to configure test *****/
   Tst_ShowFormConfig ();
  }

/****************************************************************************/
/*********************** Get type of feedback from form *********************/
/****************************************************************************/

static Tst_Feedback_t Tst_GetFeedbackTypeFromForm (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"Feedback",UnsignedStr,10);
   if (UnsignedStr[0])
     {
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         return (Tst_Feedback_t) 0; 
      if (UnsignedNum >= Tst_NUM_TYPES_FEEDBACK)
         return (Tst_Feedback_t) 0; 
      return (Tst_Feedback_t) UnsignedNum;
     }
   return (Tst_Feedback_t) 0; 
  }

/*****************************************************************************/
/**** Check and correct minimum, default and maximum numbers of questions ****/
/*****************************************************************************/

static void Tst_CheckAndCorrectNumbersQst (void)
  {
   /***** Check if minimum is correct *****/
   if (Gbl.Test.Config.Min < 1)
      Gbl.Test.Config.Min = 1;
   else if (Gbl.Test.Config.Min > Tst_MAX_QUESTIONS_PER_EXAM)
      Gbl.Test.Config.Min = Tst_MAX_QUESTIONS_PER_EXAM;

   /***** Check if maximum is correct *****/
   if (Gbl.Test.Config.Max < 1)
      Gbl.Test.Config.Max = 1;
   else if (Gbl.Test.Config.Max > Tst_MAX_QUESTIONS_PER_EXAM)
      Gbl.Test.Config.Max = Tst_MAX_QUESTIONS_PER_EXAM;

   /***** Check if minimum is lower than maximum *****/
   if (Gbl.Test.Config.Min > Gbl.Test.Config.Max)
      Gbl.Test.Config.Min = Gbl.Test.Config.Max;

   /***** Check if default is correct *****/
   if (Gbl.Test.Config.Def < Gbl.Test.Config.Min)
      Gbl.Test.Config.Def = Gbl.Test.Config.Min;
   else if (Gbl.Test.Config.Def > Gbl.Test.Config.Max)
      Gbl.Test.Config.Def = Gbl.Test.Config.Max;
  }

/*****************************************************************************/
/***************** Show form for select the types of answers *****************/
/*****************************************************************************/

static void Tst_ShowFormAnswerTypes (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_All_types_of_answers[Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES][Tst_NUM_ANS_TYPES];
   Tst_AnswerType_t AnsType;
   char UnsignedStr[10+1];
   const char *Ptr;

   /***** Table start *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\">\n");
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Select all types of answers *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"%s\">" \
                      "<input type=\"checkbox\" name=\"AllAnsTypes\" value=\"Y\"",
            The_ClassFormul[Gbl.Prefs.Theme]);
   if (Gbl.Test.AllAnsTypes)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," onclick=\"togglecheckChildren(this,'AnswerType')\" />" \
                      "%s</td></tr>",
            Txt_All_types_of_answers[Gbl.Prefs.Language]);

   /***** Type of answer *****/
   for (AnsType=(Tst_AnswerType_t) 0; AnsType<Tst_NUM_ANS_TYPES; AnsType++)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">" \
                         "<input type=\"checkbox\" name=\"AnswerType\" value=\"%u\"",
               (unsigned) AnsType);
      Ptr = Gbl.Test.ListAnsTypes;
      while (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,UnsignedStr,10);
         if (Tst_ConvertFromUnsignedStrToAnsTyp (UnsignedStr) == AnsType)
            fprintf (Gbl.F.Out," checked=\"checked\"");
        }
      fprintf (Gbl.F.Out," onclick=\"checkParent(this,'AllAnsTypes')\" />%s</td></tr>",
               Txt_TST_STR_ANSWER_TYPES[AnsType][Gbl.Prefs.Language]);
     }

   /***** End of table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr>\n");
  }

/*****************************************************************************/
/***************** List several test questions for edition *******************/
/*****************************************************************************/

void Tst_ListQuestionsToEdit (void)
  {
   MYSQL_RES *mysql_res;
   unsigned long NumRows;

   /***** Get parameters, query the database and list the questions *****/
   if (Tst_GetParamsTst ())							// Get parameters of the form
     {
      if ((NumRows = Tst_GetQuestionsForEdit (&mysql_res)) != 0)		// Query database
         Tst_ListOneOrMoreQuestionsToEdit (NumRows,mysql_res);			// Show the table with the questions

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
   else
      Tst_ShowFormAskEditTsts ();						// Show the form again

   /***** Free memory used by the list of tags *****/
   Tst_FreeTagsList ();
  }

/*****************************************************************************/
/********** Get from the database several test questions for listing *********/
/*****************************************************************************/

#define MAX_LENGTH_QUERY_TEST (16*1024)

static unsigned long Tst_GetQuestionsForEdit (MYSQL_RES **mysql_res)
  {
   extern const char *Txt_No_questions_found_matching_your_search_criteria[Txt_NUM_LANGUAGES];
   unsigned long NumRows;
   char Query[MAX_LENGTH_QUERY_TEST+1];
   long LengthQuery;
   unsigned NumItemInList;
   const char *Ptr;
   char TagText[Tst_MAX_BYTES_TAG+1];
   char UnsignedStr[10+1];
   Tst_AnswerType_t AnsType;
   char CrsCodStr[1+10+1];
   char StrDate[16+1];

   /***** Select questions *****/
   /* Start query */
   sprintf (Query,"SELECT tst_questions.QstCod,DATE_FORMAT(tst_questions.EditTime,'%%Y%%m%%d%%H%%i%%S') AS F," \
                  "tst_questions.AnsType,tst_questions.Shuffle,tst_questions.Stem,tst_questions.NumHits,tst_questions.Score" \
                 " FROM tst_questions");
   if (!Gbl.Test.AllTags)
      strcat (Query,",tst_question_tags,tst_tags");

   strcat (Query," WHERE tst_questions.CrsCod='");
   sprintf (CrsCodStr,"%ld",Gbl.CurrentCrs.Crs.CrsCod);
   strcat (Query,CrsCodStr);
   strcat (Query,"' AND tst_questions.EditTime>=");
   sprintf (StrDate,"'%04u%02u%02u'",
            Gbl.DateRange.DateIni.Year,
            Gbl.DateRange.DateIni.Month,
            Gbl.DateRange.DateIni.Day);
   strcat (Query,StrDate);
   strcat (Query," AND tst_questions.EditTime<=");
   sprintf (StrDate,"'%04u%02u%02u235959'",
            Gbl.DateRange.DateEnd.Year,
            Gbl.DateRange.DateEnd.Month,
            Gbl.DateRange.DateEnd.Day);
   strcat (Query,StrDate);

   /* Add the tags selected */
   if (!Gbl.Test.AllTags)
     {
      strcat (Query," AND tst_questions.QstCod=tst_question_tags.QstCod AND tst_question_tags.TagCod=tst_tags.TagCod" \
                    " AND tst_tags.CrsCod='");
      strcat (Query,CrsCodStr);
      strcat (Query,"'");
      LengthQuery = strlen (Query);
      NumItemInList = 0;
      Ptr = Gbl.Test.TagsList;
      while (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,TagText,Tst_MAX_BYTES_TAG);
         LengthQuery = LengthQuery + 35 + strlen (TagText) + 1;
         if (LengthQuery > MAX_LENGTH_QUERY_TEST - 256)
            Lay_ShowErrorAndExit ("Query size exceed.");
         strcat (Query,NumItemInList ? " OR tst_tags.TagTxt='" : " AND (tst_tags.TagTxt='");
         strcat (Query,TagText);
         strcat (Query,"'");
         NumItemInList++;
        }
      strcat (Query,")");
     }

   /* Add the types of answer selected */
   if (!Gbl.Test.AllAnsTypes)
     {
      LengthQuery = strlen (Query);
      NumItemInList = 0;
      Ptr = Gbl.Test.ListAnsTypes;
      while (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,UnsignedStr,Tst_MAX_BYTES_TAG);
	 AnsType = Tst_ConvertFromUnsignedStrToAnsTyp (UnsignedStr);
         LengthQuery = LengthQuery + 35 + strlen (Tst_StrAnswerTypesDB[AnsType]) + 1;
         if (LengthQuery > MAX_LENGTH_QUERY_TEST - 256)
            Lay_ShowErrorAndExit ("Query size exceed.");
         strcat (Query,NumItemInList ? " OR tst_questions.AnsType='" : " AND (tst_questions.AnsType='");
         strcat (Query,Tst_StrAnswerTypesDB[AnsType]);
         strcat (Query,"'");
         NumItemInList++;
        }
      strcat (Query,")");
     }

   /* End the query */
   strcat (Query," GROUP BY tst_questions.QstCod");

   switch (Gbl.Test.SelectedOrderType)
     {
      case Tst_ORDER_HEADING:
         strcat (Query," ORDER BY tst_questions.Stem");
         break;
      case Tst_ORDER_NUM_ACCESS:
         strcat (Query," ORDER BY tst_questions.NumHits DESC,tst_questions.Stem");
         break;
      case Tst_ORDER_AVERAGE_GRADE:
         strcat (Query," ORDER BY tst_questions.Score/tst_questions.NumHits DESC,tst_questions.NumHits DESC,tst_questions.Stem");
         break;
     }

   /* Make the query */
   NumRows = DB_QuerySELECT (Query,mysql_res,"can not get questions");

   if (NumRows == 0)
      Lay_ShowAlert (INFO,Txt_No_questions_found_matching_your_search_criteria[Gbl.Prefs.Language]);

   return NumRows;
  }

/*****************************************************************************/
/********* Get from the database several test questions to list them *********/
/*****************************************************************************/

static unsigned long Tst_GetQuestionsForExam (MYSQL_RES **mysql_res)
  {
   char Query[MAX_LENGTH_QUERY_TEST+1];
   long LengthQuery;
   unsigned NumItemInList;
   const char *Ptr;
   char TagText[Tst_MAX_BYTES_TAG+1];
   char UnsignedStr[10+1];
   Tst_AnswerType_t AnsType;
   char StrNumQuestions[10+1];

   /***** Select questions without hidden tags *****/
   /* Start query */
   if (Gbl.Test.AllTags) // User has selected all the tags
     {
/*
      sprintf (Query,"SELECT QstCod,AnsType,Shuffle,Stem" \
                     " FROM tst_questions" \
                     " WHERE CrsCod='%ld' AND QstCod NOT IN" \
                     " (SELECT DISTINCT tst_questions.QstCod FROM tst_questions,tst_question_tags,tst_tags" \
                     " WHERE tst_questions.CrsCod='%ld' AND tst_questions.QstCod=tst_question_tags.QstCod" \
                     " AND tst_question_tags.TagCod=tst_tags.TagCod AND tst_tags.CrsCod='%ld' AND tst_tags.TagHidden='Y')",
               Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod);
*/
      sprintf (Query,"SELECT QstCod,AnsType,Shuffle,Stem" \
                     " FROM tst_questions" \
                     " WHERE CrsCod='%ld' AND NOT EXISTS" \
                     " (SELECT * FROM tst_question_tags,tst_tags" \
                     " WHERE tst_question_tags.QstCod=tst_questions.QstCod" \
                     " AND tst_question_tags.TagCod=tst_tags.TagCod AND tst_tags.CrsCod='%ld' AND tst_tags.TagHidden='Y')",
               Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod);

     }
   else // User hasn't selected all the tags
     {
/*
      sprintf (Query,"SELECT tst_questions.QstCod AS QstCod,tst_questions.AnsType,tst_questions.Shuffle,tst_questions.Stem" \
                     " FROM tst_questions,tst_question_tags,tst_tags" \
                     " WHERE tst_questions.CrsCod='%ld' AND tst_questions.QstCod NOT IN" \
                     " (SELECT DISTINCT tst_questions.QstCod FROM tst_questions,tst_question_tags,tst_tags" \
                     " WHERE tst_questions.CrsCod='%ld' AND tst_questions.QstCod=tst_question_tags.QstCod" \
                     " AND tst_question_tags.TagCod=tst_tags.TagCod AND tst_tags.CrsCod='%ld' AND tst_tags.TagHidden='Y')" \
                     " AND tst_questions.QstCod=tst_question_tags.QstCod AND tst_question_tags.TagCod=tst_tags.TagCod AND tst_tags.CrsCod='%ld'",
               Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod);
*/
      sprintf (Query,"SELECT tst_questions.QstCod AS QstCod,tst_questions.AnsType,tst_questions.Shuffle,tst_questions.Stem" \
                     " FROM tst_questions,tst_question_tags,tst_tags" \
                     " WHERE tst_questions.CrsCod='%ld' AND NOT EXISTS" \
                     " (SELECT * FROM tst_question_tags,tst_tags" \
                     " WHERE tst_question_tags.QstCod=tst_questions.QstCod" \
                     " AND tst_question_tags.TagCod=tst_tags.TagCod AND tst_tags.CrsCod='%ld' AND tst_tags.TagHidden='Y')" \
                     " AND tst_questions.QstCod=tst_question_tags.QstCod AND tst_question_tags.TagCod=tst_tags.TagCod AND tst_tags.CrsCod='%ld'",
               Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod);

      /* Add selected tags */
      LengthQuery = strlen (Query);
      NumItemInList = 0;
      Ptr = Gbl.Test.TagsList;
      while (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,TagText,Tst_MAX_BYTES_TAG);
         LengthQuery = LengthQuery + 35 + strlen (TagText) + 1;
         if (LengthQuery > MAX_LENGTH_QUERY_TEST - 128)
            Lay_ShowErrorAndExit ("Query size exceed.");
         strcat (Query,NumItemInList ? " OR tst_tags.TagTxt='" : " AND (tst_tags.TagTxt='");
         strcat (Query,TagText);
         strcat (Query,"'");
         NumItemInList++;
        }
      strcat (Query,")");
     }

   /* Add answer types selected */
   if (!Gbl.Test.AllAnsTypes)
     {
      LengthQuery = strlen (Query);
      NumItemInList = 0;
      Ptr = Gbl.Test.ListAnsTypes;
      while (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,UnsignedStr,Tst_MAX_BYTES_TAG);
	 AnsType = Tst_ConvertFromUnsignedStrToAnsTyp (UnsignedStr);
         LengthQuery = LengthQuery + 35 + strlen (Tst_StrAnswerTypesDB[AnsType]) + 1;
         if (LengthQuery > MAX_LENGTH_QUERY_TEST - 128)
            Lay_ShowErrorAndExit ("Query size exceed.");
         if (Gbl.Test.AllTags) // User has selected all the tags
            strcat (Query,NumItemInList ? " OR AnsType='" : " AND (AnsType='");
         else
            strcat (Query,NumItemInList ? " OR tst_questions.AnsType='" : " AND (tst_questions.AnsType='");
         strcat (Query,Tst_StrAnswerTypesDB[AnsType]);
         strcat (Query,"'");
         NumItemInList++;
        }
      strcat (Query,")");
     }

   /* End query */
   strcat (Query," GROUP BY QstCod ORDER BY RAND(NOW()) LIMIT ");
   sprintf (StrNumQuestions,"%u",Gbl.Test.NumQuestions);
   strcat (Query,StrNumQuestions);

   /* Make the query */
   return DB_QuerySELECT (Query,mysql_res,"can not get questions");
  }

/*****************************************************************************/
/*********************** List a test question for edition ********************/
/*****************************************************************************/

static void Tst_ListOneQstToEdit (void)
  {
   MYSQL_RES *mysql_res;
   unsigned long NumRows;

   /***** Query database *****/
   if ((NumRows = Tst_GetOneQstToEdit (Gbl.Test.QstCod,&mysql_res)) == 1)
      /***** Show the question ready to edit it *****/
      Tst_ListOneOrMoreQuestionsToEdit (1,mysql_res);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********************* Get one test question for edition *********************/
/*****************************************************************************/

static unsigned long Tst_GetOneQstToEdit (long QstCod,MYSQL_RES **mysql_res)
  {
   unsigned long NumRows;
   char Query[512];

   /***** Get data of a question from database *****/
   sprintf (Query,"SELECT QstCod,DATE_FORMAT(tst_questions.EditTime,'%%Y%%m%%d%%H%%i%%S') AS F,AnsType,Shuffle,Stem,NumHits,Score" \
                  " FROM tst_questions WHERE QstCod='%ld'",QstCod);
   NumRows = DB_QuerySELECT (Query,mysql_res,"can not get data of a question");

   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting data of a question.");

   return NumRows;
  }

/*****************************************************************************/
/*********************** Get one test question for exam **********************/
/*****************************************************************************/

static unsigned long Tst_GetAQuestionForExam (long QstCod,MYSQL_RES **mysql_res)
  {
   unsigned long NumRows;
   char Query[512];

   /***** Get data of a question from database *****/
   sprintf (Query,"SELECT QstCod,AnsType,Shuffle,Stem" \
                  " FROM tst_questions WHERE QstCod='%ld'",
            QstCod);
   NumRows = DB_QuerySELECT (Query,mysql_res,"can not get data of a question");

   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting data of a question.");

   return NumRows;
  }

/*****************************************************************************/
/****************** List for edition one or more test questions **************/
/*****************************************************************************/

static void Tst_ListOneOrMoreQuestionsToEdit (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *Txt_Date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Tags[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type[Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_STR_ORDER_FULL[Tst_NUM_CRIT_ORDEN_TST][Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_STR_ORDER_SHORT[Tst_NUM_CRIT_ORDEN_TST][Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES][Tst_NUM_ANS_TYPES];
   extern const char *Txt_Shuffle[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_question[Txt_NUM_LANGUAGES];
   Tst_QuestionsOrder_t Order;
   unsigned long NumRow;
   MYSQL_ROW row;
   long QstCod;
   long NumHitsThisQst;
   double TotalScoreThisQst;

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

   /***** Write the heading *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"3\"></td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">N&ordm;&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>",
            Txt_Code[Gbl.Prefs.Language],
            Txt_Date[Gbl.Prefs.Language],
            Txt_Tags[Gbl.Prefs.Language],
            Txt_Type[Gbl.Prefs.Language]);

   /* Stem and answers of question */
   /* Number of times that the question has been answered */
   /* Average score */
   for (Order = (Tst_QuestionsOrder_t) 0; Order < (Tst_QuestionsOrder_t) Tst_NUM_CRIT_ORDEN_TST; Order++)
     {
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">");
      if (NumRows > 1)
        {
         Act_FormStart (ActLstTstQst);
         Sta_WriteParamsDatesSeeAccesses ();
         Tst_WriteParamEditQst ();
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OnlyThisQst\" value=\"N\" />");
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",(unsigned) Order);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_TST_STR_ORDER_FULL[Order][Gbl.Prefs.Language],"TIT_TBL");
         if (Order == Gbl.Test.SelectedOrderType)
            fprintf (Gbl.F.Out,"<u>");
        }
      fprintf (Gbl.F.Out,Txt_TST_STR_ORDER_SHORT[Order][Gbl.Prefs.Language]);
      if (NumRows > 1)
        {
         if (Order == Gbl.Test.SelectedOrderType)
            fprintf (Gbl.F.Out,"</u>");
         fprintf (Gbl.F.Out,"</acronym></a></form>");
        }
      fprintf (Gbl.F.Out,"</td>");
     }

   fprintf (Gbl.F.Out,"</tr>");

   /***** Write rows *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      Gbl.GlobalRowEvenOdd = NumRow % 2;

      row = mysql_fetch_row (mysql_res);

      /* row[0] holds the code of the question */
      if (sscanf (row[0],"%ld",&QstCod) != 1)
         Lay_ShowErrorAndExit ("Wrong code of question.");

      fprintf (Gbl.F.Out,"<tr>");

      /* Write icon to remove the question */
      fprintf (Gbl.F.Out,"<td class=\"BT%d\">",Gbl.GlobalRowEvenOdd);
      Act_FormStart (ActRemTstQst);
      Sta_WriteParamsDatesSeeAccesses ();
      Tst_WriteParamEditQst ();
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"QstCod\" value=\"%ld\" />",
               QstCod);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OnlyThisQst\" value=\"%c\" />",
               (NumRows == 1) ? 'Y' : 'N'); // If there are only one row, don't list again after removing
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Remove_question[Gbl.Prefs.Language],NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/delon12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                         "</acronym></a></form></td>",
               Gbl.Prefs.IconsURL,Txt_Remove_question[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"DAT_SMALL\">&nbsp;&nbsp;</td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);

      /* Write icon to edit the question */
      fprintf (Gbl.F.Out,"<td class=\"BT%d\">",Gbl.GlobalRowEvenOdd);
      Act_FormStart (ActWriTstQst);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"QstCod\" value=\"%ld\" />",QstCod);
      sprintf (Gbl.MsgWindowStatus,"Editar pregunta");	// Need translation!
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
      fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /></acronym></a></form></td>",
               Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);

      /* Write number of question */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_SMALL\">%lu&nbsp;</td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],NumRow+1);

      /* Write question code */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_SMALL\">%ld&nbsp;</td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],QstCod);

      /* Write the date (row[1] has the date in format YYYYMMDDHHMMSS) */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_SMALL\">",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      Dat_WriteDate (row[1]);
      fprintf (Gbl.F.Out,"<br />");
      Dat_WriteHourMinute (&row[1][8]);
      fprintf (Gbl.F.Out,"</td>");

      /* Write the question tags */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\">",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      Tst_GetAndWriteTagsQst (QstCod);
      fprintf (Gbl.F.Out,"</td>");

      /* Write the question type (row[2]) */
      Gbl.Test.AnswerType = Tst_ConvertFromStrAnsTypDBToAnsTyp (row[2]);
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_SMALL\">%s&nbsp;",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],
               Txt_TST_STR_ANSWER_TYPES[Gbl.Test.AnswerType][Gbl.Prefs.Language]);

      /* Write if shuffle is enabled (row[3]) */
      if (Gbl.Test.AnswerType == Tst_ANS_UNIQUE_CHOICE ||
          Gbl.Test.AnswerType == Tst_ANS_MULTIPLE_CHOICE)
        {
         Act_FormStart (ActShfTstQst);
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"QstCod\" value=\"%ld\" />",QstCod);
         Sta_WriteParamsDatesSeeAccesses ();
         Tst_WriteParamEditQst ();
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OnlyThisQst\" value=\"%c\" />",
                  (NumRows == 1) ? 'Y' : 'N'); // If editing only one question, don't edit others
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Order\" value=\"%u\" />",
                  (unsigned) Gbl.Test.SelectedOrderType);
         fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"Shuffle\" value=\"Y\"");
         if (Str_ConvertToUpperLetter (row[3][0]) != 'N')
            fprintf (Gbl.F.Out," checked=\"checked\"");
         fprintf (Gbl.F.Out," onclick=\"javascript:document.forms[%d].submit();\" />" \
                            "%s&nbsp;</form>",
                  Gbl.NumForm,Txt_Shuffle[Gbl.Prefs.Language]);
        }
      fprintf (Gbl.F.Out,"</td>");

      /* Write the stem (row[4]) */
      Tst_WriteQstStem (row[4],"TEST_EDI");

      /* Write the answers of esta question */
      Tst_WriteAnswersOfAQstEdit (QstCod);
      fprintf (Gbl.F.Out,"</td>");

      /* Get number of hits (number of times that the question has been answered) (row[5]) */
      if (sscanf (row[5],"%ld",&NumHitsThisQst) != 1)
         Lay_ShowErrorAndExit ("Wrong number of accesses to a question.");

      /* Get the puntuacin acumulada of the question (row[6]) */
      setlocale (LC_NUMERIC,"en_US.iso88591");	// To get decimal point
      if (sscanf (row[6],"%lf",&TotalScoreThisQst) != 1)
         Lay_ShowErrorAndExit ("Wrong score of a question.");

      setlocale (LC_NUMERIC,"es_ES.iso88591");

      /* Write n of times this question has been asnwered */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_SMALL\">%ld&nbsp;</td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],NumHitsThisQst);

      /* Write average score */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"DAT_SMALL\">",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      if (NumHitsThisQst)
         fprintf (Gbl.F.Out,"%.2f",TotalScoreThisQst / (double) NumHitsThisQst);
      else
         fprintf (Gbl.F.Out,"N.A.");
      fprintf (Gbl.F.Out,"</td>");

      fprintf (Gbl.F.Out,"</tr>");
     }

   /***** Table end *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/*********** Write hidden parameters for edition of test questions ***********/
/*****************************************************************************/

static void Tst_WriteParamEditQst (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AllTags\" value=\"%c\" />",
            Gbl.Test.AllTags ? 'Y' : 'N');
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ChkTag\" value=\"%s\" />",
            Gbl.Test.TagsList ? Gbl.Test.TagsList : "");
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AllAnsTypes\" value=\"%c\" />",
            Gbl.Test.AllAnsTypes ? 'Y' : 'N');
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AnswerType\" value=\"%s\" />",
            Gbl.Test.ListAnsTypes);
  }

/*****************************************************************************/
/*************** Get answers of a test question from database ****************/
/*****************************************************************************/

static unsigned Tst_GetAnswersQst (long QstCod,MYSQL_RES **mysql_res,bool Shuffle)
  {
   char Query[512];
   unsigned long NumRows;

   /***** Get answers of a question from database *****/
   sprintf (Query,"SELECT AnsInd,Answer,Correct FROM tst_answers" \
                  " WHERE QstCod='%ld' ORDER BY %s",
            QstCod,Shuffle ? "RAND(NOW())" : "AnsInd");
   NumRows = DB_QuerySELECT (Query,mysql_res,"can not get answers of a question");

   /***** Count number of rows of result *****/
   if (NumRows == 0)
      Lay_ShowErrorAndExit ("Error when getting answers of a question.");

   return (unsigned) NumRows;
  }

/*****************************************************************************/
/**************** Get and write the answers of a test question ***************/
/*****************************************************************************/

static void Tst_WriteAnswersOfAQstEdit (long QstCod)
  {
   unsigned NumAns,NumAnswers;
   unsigned i;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   char *Answer;
   size_t AnsLength;
   double NumReal[2];

   NumAnswers = Tst_GetAnswersQst (QstCod,&mysql_res,false);	// Result: AnsInd,Answer,Correct

   /***** Write the answers *****/
   switch (Gbl.Test.AnswerType)
     {
      case Tst_ANS_TRUE_FALSE:
         Tst_CheckIfNumberOfAnswersIsOne (NumAnswers);
         row = mysql_fetch_row (mysql_res);
         fprintf (Gbl.F.Out,"<span class=\"TEST_EDI\"><tt>(");
         Tst_WriteAnsTF (row[1][0]);
         fprintf (Gbl.F.Out,")</tt></span>");
         break;
      case Tst_ANS_UNIQUE_CHOICE:
      case Tst_ANS_MULTIPLE_CHOICE:
      case Tst_ANS_TEXT:
         fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");
         for (NumAns = 0; NumAns < NumAnswers; NumAns++)
           {
            row = mysql_fetch_row (mysql_res);
            /* Convert the answer (row[1]), that is in HTML, to HTML respetuoso */
            AnsLength = strlen (row[1]) * Str_MAX_LENGTH_SPEC_CHAR_HTML;
            if ((Answer = malloc (AnsLength+1)) == NULL)
               Lay_ShowErrorAndExit ("No enough memory to store answer.");
            strcpy (Answer,row[1]);
            Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Answer,AnsLength,false);
            /* Draw an icon that indicates wheter the answer is correct or wrong */
            fprintf (Gbl.F.Out,"<tr><td class=\"BT%d\">",Gbl.GlobalRowEvenOdd);
            if (Str_ConvertToUpperLetter (row[2][0]) == 'Y')
               fprintf (Gbl.F.Out,"<img src=\"%s/OKn12x12.gif\" width=\"12\" height=\"12\" alt=\"Respuesta correcta\" />",
                        Gbl.Prefs.IconsURL);
            fprintf (Gbl.F.Out,"</td>");
            /* Write the number of opcin */
            fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"DAT\">%c)&nbsp;</td>",
                     'a' + (char) NumAns);
            /* Write the text of the answer */
            fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TEST_EDI\"><tt>%s</tt></td></tr>",
                     Answer);
	    /* Free memory asignada for the answer */
	    free ((void *) Answer);
           }
         fprintf (Gbl.F.Out,"</table>");
	 break;
      case Tst_ANS_INT:
         Tst_CheckIfNumberOfAnswersIsOne (NumAnswers);
         row = mysql_fetch_row (mysql_res);
         fprintf (Gbl.F.Out,"<span class=\"TEST_EDI\"><tt>(%s)</tt></span>",row[1]);
         break;
      case Tst_ANS_FLOAT:
	 if (NumAnswers != 2)
            Lay_ShowErrorAndExit ("Wrong float range.");

         for (i=0; i<2; i++)
           {
            row = mysql_fetch_row (mysql_res);
     	    Str_ConvertStrFloatCommaToStrFloatPoint (row[1]);
            NumReal[i] = Tst_GetFloatReplyFromStr (row[1]);
           }
         fprintf (Gbl.F.Out,"<span class=\"TEST_EDI\"><tt>([%lg; %lg])</tt></span>",
                  NumReal[0],NumReal[1]);
         break;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/**************** Write answers of a question when seeing an exam ************/
/*****************************************************************************/

static void Tst_WriteAnswersOfAQstSeeExam (unsigned NumQst,long QstCod,bool Shuffle)
  {
   /***** Write parameter with question code *****/
   Tst_WriteParamQstCod (NumQst,QstCod);

   /***** Write answer depending on type *****/
   switch (Gbl.Test.AnswerType)
     {
      case Tst_ANS_TRUE_FALSE:
         Tst_WriteFormAnsTF (NumQst);
         break;
      case Tst_ANS_UNIQUE_CHOICE:
      case Tst_ANS_MULTIPLE_CHOICE:
         Tst_WriteChoiceAnsSeeExam (NumQst,QstCod,Shuffle);
         break;
      case Tst_ANS_TEXT:
         Tst_WriteTextAnsSeeExam (NumQst);
         break;
      case Tst_ANS_INT:
         Tst_WriteIntAnsSeeExam (NumQst);
         break;
      case Tst_ANS_FLOAT:
         Tst_WriteFloatAnsSeeExam (NumQst);
         break;
     }
  }

/*****************************************************************************/
/************* Write answers of a question when assessing an exam ************/
/*****************************************************************************/

static void Tst_WriteAnswersOfAQstAssessExam (unsigned NumQst,long QstCod,double *MarkThisQst)
  {
   unsigned NumAnswers;
   MYSQL_RES *mysql_res;

   /***** Get answers of a question from database *****/
   NumAnswers = Tst_GetAnswersQst (QstCod,&mysql_res,false);	// Result: AnsInd,Answer,Correct

   /***** Write answer depending on type *****/
   switch (Gbl.Test.AnswerType)
     {
      case Tst_ANS_TRUE_FALSE:
         Tst_WriteTFAnsAssessExam (NumQst,QstCod,NumAnswers,mysql_res,MarkThisQst);
         break;
      case Tst_ANS_UNIQUE_CHOICE:
      case Tst_ANS_MULTIPLE_CHOICE:
         Tst_WriteChoiceAnsAssessExam (NumQst,QstCod,NumAnswers,mysql_res,MarkThisQst);
         break;
      case Tst_ANS_TEXT:
         Tst_WriteTextAnsAssessExam (NumQst,QstCod,NumAnswers,mysql_res,MarkThisQst);
         break;
      case Tst_ANS_INT:
         Tst_WriteIntAnsAssessExam (NumQst,QstCod,NumAnswers,mysql_res,MarkThisQst);
         break;
      case Tst_ANS_FLOAT:
         Tst_WriteFloatAnsAssessExam (NumQst,QstCod,NumAnswers,mysql_res,MarkThisQst);
         break;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************** Write false / true answer when seeing an exam ****************/
/*****************************************************************************/

static void Tst_WriteFormAnsTF (unsigned NumQst)
  {
   extern const char *Txt_TST_TF_QST[2][Txt_NUM_LANGUAGES];

   /***** Write selector for the answer *****/
   fprintf (Gbl.F.Out,"<select name=\"Ans%06u\">" \
                      "<option value=\"\" selected=\"selected\">&nbsp;</option>" \
                      "<option value=\"T\">%s</option>" \
                      "<option value=\"F\">%s</option>" \
                      "</select>",
            NumQst,
            Txt_TST_TF_QST[0][Gbl.Prefs.Language],
            Txt_TST_TF_QST[1][Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/************** Write false / true answer when seeing an exam ****************/
/*****************************************************************************/

static void Tst_WriteAnsTF (char AnsTF)
  {
   extern const char *Txt_TST_TF_QST[2][Txt_NUM_LANGUAGES];

   switch (AnsTF)
     {
      case 'T':		// true
         fprintf (Gbl.F.Out,"%s",Txt_TST_TF_QST[0][Gbl.Prefs.Language]);
         break;
      case 'F':		// false
         fprintf (Gbl.F.Out,"%s",Txt_TST_TF_QST[1][Gbl.Prefs.Language]);
         break;
      default:		// no answer
         fprintf (Gbl.F.Out,"&nbsp;");
     }
  }

/*****************************************************************************/
/************* Write false / true answer when assessing an exam **************/
/*****************************************************************************/

static void Tst_WriteTFAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst)
  {
   MYSQL_ROW row;
   char AnsTF;

   /***** Check if number of rows is correct *****/
   Tst_CheckIfNumberOfAnswersIsOne (NumAnswers);

   /***** Get answer true or false *****/
   row = mysql_fetch_row (mysql_res);

   /***** Compute the mark for this question *****/
   AnsTF = Gbl.Test.StrAnswersUsr[NumQst][0];
   if (AnsTF == '\0')			// If user has omitted the answer
      *MarkThisQst = 0.0;
   else if (AnsTF == row[1][0])		// If correct
      *MarkThisQst = 1.0;
   else					// If wrong
      *MarkThisQst = -1.0;

   /***** Header with the title of each column *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   Tst_WriteHeadUserCorrect ();
   fprintf (Gbl.F.Out,"</tr>");

   /***** Write the user answer *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\">",
            (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD) ?
            ((AnsTF == row[1][0]) ? "ANS_OK" : "ANS_BAD") :
            "ANS");
   Tst_WriteAnsTF (AnsTF);
   fprintf (Gbl.F.Out,"</td>");

   /***** Write the correct answer *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" class=\"ANS\">");
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD)
      Tst_WriteAnsTF (row[1][0]);
   else
      fprintf (Gbl.F.Out,"?");
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Write the mark *****/
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD ||
       Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_RESULT)
     {
      Tst_WriteScoreStart (2);
      if (AnsTF == '\0')		// If user has omitted the answer
         fprintf (Gbl.F.Out,"ANS\">0,00");
      else if (AnsTF == row[1][0])	// If correct
         fprintf (Gbl.F.Out,"ANS_OK\">1,00");
      else				// If wrong
         fprintf (Gbl.F.Out,"ANS_BAD\">-1,00");
      Tst_WriteScoreEnd ();
     }

   fprintf (Gbl.F.Out,"</table>");
  }

/*****************************************************************************/
/******** Write single or multiple choice answer when seeing an exam *********/
/*****************************************************************************/

static void Tst_WriteChoiceAnsSeeExam (unsigned NumQst,long QstCod,bool Shuffle)
  {
   unsigned NumAns,NumAnswers;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned Index;
   bool ErrorInIndex = false;

   /***** Start of table *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Get answers of a question from database *****/
   NumAnswers = Tst_GetAnswersQst (QstCod,&mysql_res,Shuffle);	// Result: AnsInd,Answer,Correct
   for (NumAns=0; NumAns<NumAnswers; NumAns++)
     {
      /***** Get next answer *****/
      row = mysql_fetch_row (mysql_res);

      /***** Allocate memory for text in this choice answer *****/
      if (!Tst_AllocateTextChoiceAnswer (NumAns))
         Lay_ShowErrorAndExit (Gbl.Message);

      /***** Assign index (row[0]). Index is 0,1,2,3... if no shuffle or 1,3,0,2... (example) if shuffle *****/
      // Index is got from databse query
      if (sscanf (row[0],"%u",&Index) == 1)
        {
         if (Index >= Tst_MAX_ANSWERS_PER_QUESTION)
            ErrorInIndex = true;
        }
      else
         ErrorInIndex = true;
      if (ErrorInIndex)
         Lay_ShowErrorAndExit ("Wrong index of answer.");

      /***** Assign text (row[1]) and convert it, that is in HTML, to respetuoso HTML ******/
      strncpy (Gbl.Test.AnsChoice[NumAns].Text,row[1],Txt_MAX_BYTES_ANSWER);
      Gbl.Test.AnsChoice[NumAns].Text[Txt_MAX_BYTES_ANSWER] = '\0';
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.Test.AnsChoice[NumAns].Text,Txt_MAX_BYTES_ANSWER,false);

      /***** Write selectors and letter of this option *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\">");
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Ind%06u\" value=\"%u\" />",NumQst,Index);
      fprintf (Gbl.F.Out,"<input type=\"");
      if (Gbl.Test.AnswerType == Tst_ANS_UNIQUE_CHOICE)
         fprintf (Gbl.F.Out,"radio\" onclick=\"selectUnselectRadio(this,this.form.Ans%06u,%u)\"",NumQst,NumAnswers);
      else // Gbl.Test.AnswerType == Tst_ANS_MULTIPLE_CHOICE
         fprintf (Gbl.F.Out,"checkbox\"");
      fprintf (Gbl.F.Out," name=\"Ans%06u\" value=\"%u\" /> </td>",NumQst,Index);
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TEST\">%c)&nbsp;</td>",'a' + (char) NumAns);

      /***** Write the option text *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TEST_EXA\"><p align=\"justify\"><tt>%s</tt></p></td></tr>",
               Gbl.Test.AnsChoice[NumAns].Text);
     }

   /***** End of table *****/
   fprintf (Gbl.F.Out,"</table>");

   /***** Free answers *****/
   Tst_FreeTextChoiceAnswers (NumAnswers);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/******* Write single or multiple choice answer when assessing an exam *******/
/*****************************************************************************/

#define MAX_SIZE_INDEXES (Tst_MAX_ANSWERS_PER_QUESTION*(10+1))

static void Tst_WriteChoiceAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst)
  {
   unsigned NumAns;
   MYSQL_ROW row;
   char StrParamIndex[3+10+1]; // "Indxx...x"
   char StrAllIndexes[MAX_SIZE_INDEXES+1]; // 1 2 3 4, 4 3 1 2, etc.
   char StrOneIndex[10+1];
   const char *Ptr;
   unsigned Index[Tst_MAX_ANSWERS_PER_QUESTION];	// Indexes of all answers of this question
   int AnsUsr;
   bool AnswersUsr[Tst_MAX_ANSWERS_PER_QUESTION];
   unsigned NumOptTotInQst=0,NumOptCorrInQst=0,NumAnsGood=0,NumAnsBad=0;

   /***** Get text and correctness of answers for this question from database (one row per answer) *****/
   for (NumAns=0; NumAns<NumAnswers; NumAns++)
     {
      /***** Get next answer *****/
      row = mysql_fetch_row (mysql_res);

      /***** Allocate memory for text in this choice answer *****/
      if (!Tst_AllocateTextChoiceAnswer (NumAns))
         Lay_ShowErrorAndExit (Gbl.Message);

      /***** Assign text (row[1]) and convert it, that is in HTML, to respetuoso HTML ******/
      strncpy (Gbl.Test.AnsChoice[NumAns].Text,row[1],Txt_MAX_BYTES_ANSWER);
      Gbl.Test.AnsChoice[NumAns].Text[Txt_MAX_BYTES_ANSWER] = '\0';
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.Test.AnsChoice[NumAns].Text,Txt_MAX_BYTES_ANSWER,false);

      /***** Assign correctness (row[2]) of this answer (this option) *****/ 
      Gbl.Test.AnsChoice[NumAns].Correct = (Str_ConvertToUpperLetter (row[2][0]) == 'Y');
     }

   /***** Get indexes of answers for this question from form *****/
   sprintf (StrParamIndex,"Ind%06u",NumQst);
   Par_GetParam (Par_PARAM_MULTIPLE,StrParamIndex,StrAllIndexes,MAX_SIZE_INDEXES);  /* If choice ==> "0", "1", "2",... */
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,StrAllIndexes,MAX_SIZE_INDEXES,true);
   for (NumAns=0, Ptr=StrAllIndexes; NumAns<NumAnswers; NumAns++)
     {
      Str_GetNextStrUntilSeparParamMult (&Ptr,StrOneIndex,10);
      if (sscanf (StrOneIndex,"%u",&(Index[NumAns])) != 1)
         Lay_ShowErrorAndExit ("Wrong index of answer.");
     }

   /***** Get the user's answers from form *****/
   for (NumAns=0; NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
      AnswersUsr[NumAns] = false;
   for (NumAns=0, Ptr=Gbl.Test.StrAnswersUsr[NumQst]; NumAns<NumAnswers; NumAns++)
      if (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,StrOneIndex,10);
         if (sscanf (StrOneIndex,"%d",&AnsUsr) != 1)
            Lay_ShowErrorAndExit ("Bad user's answer.");
         if (AnsUsr < 0 || AnsUsr >= Tst_MAX_ANSWERS_PER_QUESTION)
            Lay_ShowErrorAndExit ("Bad user's answer.");
         AnswersUsr[AnsUsr] = true;
        }

   /***** Start of table *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   Tst_WriteHeadUserCorrect ();
   fprintf (Gbl.F.Out,"<td align=\"left\"></td><td align=\"left\"></td></tr>");

   /***** Write answers (one row per answer) *****/
   for (NumAns=0; NumAns<NumAnswers; NumAns++)
     {
      /* Draw icon depending on user's answer */
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\">");
      if (AnswersUsr[Index[NumAns]] == true)	// This answer has been selected by the user
         fprintf (Gbl.F.Out,"<img src=\"%s/OK%c12x12.gif\" width=\"12\" height=\"12\" alt=\"Respuesta usuario\" />",
                  Gbl.Prefs.IconsURL,
                  (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD) ?
                   (Gbl.Test.AnsChoice[Index[NumAns]].Correct ? 'v' : 'r') : 'n');
      fprintf (Gbl.F.Out,"</td>");

      /* Draw icon that indicates whether the answer is correct */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" class=\"ANS\">");
      if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD)
        {
         if (Gbl.Test.AnsChoice[Index[NumAns]].Correct)
            fprintf (Gbl.F.Out,"<img src=\"%s/OKn12x12.gif\" width=\"12\" height=\"12\" alt=\"Respuesta correcta\" />",
                     Gbl.Prefs.IconsURL);
        }
      else
         fprintf (Gbl.F.Out,"?");
      fprintf (Gbl.F.Out,"</td>");

      /* Answer letter (a, b, c,...) */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TEST\">%c)&nbsp;</td>",'a' + (char) NumAns);

      /* Answer text */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TEST_EXA\"><p align=\"justify\"><tt>%s</tt></p></td></tr>",
               Gbl.Test.AnsChoice[Index[NumAns]].Text);

      NumOptTotInQst++;
      if (AnswersUsr[Index[NumAns]] == true)	// This answer has been selected by the user
        {
         if (Gbl.Test.AnsChoice[Index[NumAns]].Correct)
            NumAnsGood++;
         else
            NumAnsBad++;
        }
      if (Gbl.Test.AnsChoice[Index[NumAns]].Correct)
         NumOptCorrInQst++;
     }

   /***** Compute and write the total mark of this question *****/
   /* Compute the mark */
   if (Gbl.Test.AnswerType == Tst_ANS_UNIQUE_CHOICE)
     {
      if (NumOptTotInQst >= 2)	// Debera haber siempre al menos dos opciones
         *MarkThisQst = (double) NumAnsGood - (double) NumAnsBad / (double) (NumOptTotInQst - 1);
      else			// Hay 0  1 opciones (caso imposible)
         *MarkThisQst = (double) NumAnsGood;
     }
   else	// Gbl.Test.AnswerType == Tst_ANS_MULTIPLE_CHOICE
     {
      if (NumOptCorrInQst)	// Si hay opciones correctas in the question
        {
         if (NumOptCorrInQst < NumOptTotInQst)	// Si hay opciones correctas y otras incorrectas (caso normal)
            *MarkThisQst = (double) NumAnsGood / (double) NumOptCorrInQst - (double) NumAnsBad / (double) (NumOptTotInQst-NumOptCorrInQst);
         else					// Si todas the opciones son correctas (caso raro)
            *MarkThisQst = (double) NumAnsGood / (double) NumOptCorrInQst;
        }
      else
        {
         if (NumOptTotInQst)	// There are options but none is correct (extrange case)
            *MarkThisQst = - (double) NumAnsBad / (double) NumOptTotInQst;
         else			// If there os no option (impossible!)
            *MarkThisQst = 0.0;
        }
     }

   /* Write the mark */
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD ||
       Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_RESULT)
     {
      Tst_WriteScoreStart (4);
      if (*MarkThisQst == 0.0)
         fprintf (Gbl.F.Out,"ANS");
      else if (*MarkThisQst > 0.0)
         fprintf (Gbl.F.Out,"ANS_OK");
      else
         fprintf (Gbl.F.Out,"ANS_BAD");
      fprintf (Gbl.F.Out,"\">%.2lf",*MarkThisQst);
      Tst_WriteScoreEnd ();
     }

   /***** End of table *****/
   fprintf (Gbl.F.Out,"</table>");

   /***** Free answers *****/
   Tst_FreeTextChoiceAnswers (NumAnswers);
  }

/*****************************************************************************/
/******************** Write text answer when seeing an exam ******************/
/*****************************************************************************/

static void Tst_WriteTextAnsSeeExam (unsigned NumQst)
  {
   /***** Write input field for the answer *****/
   fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Ans%06u\" size=\"40\" maxlength=\"%u\" value=\"\" />",
            NumQst,Tst_MAX_LENGTH_ANS_USR);
  }

/*****************************************************************************/
/***************** Write text answer when assessing an exam ******************/
/*****************************************************************************/

static void Tst_WriteTextAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst)
  {
   unsigned NumAns;
   MYSQL_ROW row;
   char TextAnsUsr[Tst_MAX_LENGTH_ANS_USR],TextAnsOK[Tst_MAX_LENGTH_ANS_USR];
   bool Correct = false;

   /***** Get text and correctness of answers for this question from database (one row per answer) *****/
   for (NumAns=0; NumAns<NumAnswers; NumAns++)
     {
      /***** Get next answer *****/
      row = mysql_fetch_row (mysql_res);

      /***** Allocate memory for text in this choice answer *****/
      if (!Tst_AllocateTextChoiceAnswer (NumAns))
         Lay_ShowErrorAndExit (Gbl.Message);

      /***** Assign text (row[1]) and convert it, that is in HTML, to respetuoso HTML ******/
      strncpy (Gbl.Test.AnsChoice[NumAns].Text,row[1],Txt_MAX_BYTES_ANSWER);
      Gbl.Test.AnsChoice[NumAns].Text[Txt_MAX_BYTES_ANSWER] = '\0';
      Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Gbl.Test.AnsChoice[NumAns].Text,Txt_MAX_BYTES_ANSWER,false);

      /***** Assign correctness (row[2]) of this answer (this option) *****/ 
      Gbl.Test.AnsChoice[NumAns].Correct = (Str_ConvertToUpperLetter (row[2][0]) == 'Y');
     }

   /***** Header with the title of each column *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   Tst_WriteHeadUserCorrect ();
   fprintf (Gbl.F.Out,"</tr>");

   /***** Write the user answer *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"top\"");
   if (Gbl.Test.StrAnswersUsr[NumQst][0])		// If user has answered the question
     {
      /* Filter the user answer */
      strcpy (TextAnsUsr,Gbl.Test.StrAnswersUsr[NumQst]);
      Str_ReplaceSeveralSpacesForOne (TextAnsUsr);
      Str_ConvertToComparable (TextAnsUsr);

      for (NumAns=0; NumAns<NumAnswers; NumAns++)
        {
         /* Filter this correct answer */
         strcpy (TextAnsOK,Gbl.Test.AnsChoice[NumAns].Text);
         Str_ConvertToComparable (TextAnsOK);

         /* Check is user answer is correct */
         if (!strcoll (TextAnsUsr,TextAnsOK))
           {
            Correct = true;
            break;
           }
        }
      fprintf (Gbl.F.Out," class=\"%s\">&nbsp;%s&nbsp;",
               (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD) ?
                (Correct ? "ANS_OK" : "ANS_BAD") : "ANS",Gbl.Test.StrAnswersUsr[NumQst]);
     }
   else							// If user has omitted the answer
      fprintf (Gbl.F.Out," class=\"ANS\">&nbsp;");
   fprintf (Gbl.F.Out,"</td>");

   /***** Write the correct answers *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\">");
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD)
     {
      fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" class=\"ANS\">");
      for (NumAns=0; NumAns<NumAnswers; NumAns++)
        {
         /* Answer letter (a, b, c,...) */
         fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TEST\">%c)&nbsp;</td>",
                  'a' + (char) NumAns);

         /* Answer text */
         fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"TEST_EXA\">" \
                            "<p align=\"justify\"><tt>%s</tt></p>" \
                            "</td></tr>",
                  Gbl.Test.AnsChoice[NumAns].Text);
        }
      fprintf (Gbl.F.Out,"</table>");
     }
   else
      fprintf (Gbl.F.Out,"?");
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Compute the mark *****/
   if (!Gbl.Test.StrAnswersUsr[NumQst][0])	// If user has omitted the answer
      *MarkThisQst = 0.0;
   else if (Correct)				// If correct
      *MarkThisQst = 1.0;
   else						// If wrong
      *MarkThisQst = 0.0;

   /***** Write the mark *****/
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD ||
       Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_RESULT)
     {
      Tst_WriteScoreStart (4);
      if (!Gbl.Test.StrAnswersUsr[NumQst][0])	// If user has omitted the answer
         fprintf (Gbl.F.Out,"ANS\">0,00");
      else if (Correct)	// If correct
         fprintf (Gbl.F.Out,"ANS_OK\">1,00");
      else					// If wrong
         fprintf (Gbl.F.Out,"ANS_BAD\">0,00");
      Tst_WriteScoreEnd ();
     }

   fprintf (Gbl.F.Out,"</table>");

   /***** Free answers *****/
   Tst_FreeTextChoiceAnswers (NumAnswers);
  }

/*****************************************************************************/
/****************** Write integer answer when seeing an exam *****************/
/*****************************************************************************/

static void Tst_WriteIntAnsSeeExam (unsigned NumQst)
  {
   /***** Write input field for the answer *****/
   fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Ans%06u\" size=\"11\" maxlength=\"11\" value=\"\" />",
            NumQst);
  }

/*****************************************************************************/
/**************** Write integer answer when assessing an exam ****************/
/*****************************************************************************/

static void Tst_WriteIntAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst)
  {
   MYSQL_ROW row;
   long IntAnswerUsr,IntAnswerCorr;

   /***** Check if number of rows is correct *****/
   Tst_CheckIfNumberOfAnswersIsOne (NumAnswers);

   /***** Get the numerical value of the correct answer *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[1],"%ld",&IntAnswerCorr) != 1)
      Lay_ShowErrorAndExit ("Wrong integer answer.");

   /***** Header with the title of each column *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   Tst_WriteHeadUserCorrect ();
   fprintf (Gbl.F.Out,"</tr>");

   /***** Write the user answer *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"");
   if (Gbl.Test.StrAnswersUsr[NumQst][0])		// If user has answered the question
     {
      if (sscanf (Gbl.Test.StrAnswersUsr[NumQst],"%ld",&IntAnswerUsr) == 1)
         fprintf (Gbl.F.Out," class=\"%s\">&nbsp;%ld&nbsp;",
                  (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD) ?
                   ((IntAnswerUsr == IntAnswerCorr) ? "ANS_OK" : "ANS_BAD") : "ANS",IntAnswerUsr);
      else
        {
         Gbl.Test.StrAnswersUsr[NumQst][0] = '\0';
         fprintf (Gbl.F.Out," class=\"ANS\">?");
        }
     }
   else							// If user has omitted the answer
      fprintf (Gbl.F.Out," class=\"ANS\">&nbsp;");
   fprintf (Gbl.F.Out,"</td>");

   /***** Write the correct answer *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" class=\"ANS\">");
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD)
      fprintf (Gbl.F.Out,"&nbsp;%ld&nbsp;",IntAnswerCorr);
   else
      fprintf (Gbl.F.Out,"?");
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Compute the mark *****/
   if (!Gbl.Test.StrAnswersUsr[NumQst][0])	// If user has omitted the answer
      *MarkThisQst = 0.0;
   else if (IntAnswerUsr == IntAnswerCorr)		// If correct
      *MarkThisQst = 1.0;
   else							// If wrong
      *MarkThisQst = 0.0;

   /***** Write the mark *****/
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD ||
       Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_RESULT)
     {
      Tst_WriteScoreStart (2);
      if (!Gbl.Test.StrAnswersUsr[NumQst][0])	// If user has omitted the answer
         fprintf (Gbl.F.Out,"ANS\">0,00");
      else if (IntAnswerUsr == IntAnswerCorr)	// If correct
         fprintf (Gbl.F.Out,"ANS_OK\">1,00");
      else					// If wrong
         fprintf (Gbl.F.Out,"ANS_BAD\">0,00");
      Tst_WriteScoreEnd ();
     }

   fprintf (Gbl.F.Out,"</table>");
  }

/*****************************************************************************/
/****************** Write float answer when seeing an exam *******************/
/*****************************************************************************/

static void Tst_WriteFloatAnsSeeExam (unsigned NumQst)
  {
   /***** Write input field for the answer *****/
   fprintf (Gbl.F.Out,"<input type=\"text\" name=\"Ans%06u\" size=\"11\" maxlength=\"%u\" value=\"\" />",
            NumQst,Txt_MAX_BYTES_FLOAT_ANSWER);
  }

/*****************************************************************************/
/***************** Write float answer when assessing an exam *****************/
/*****************************************************************************/

static void Tst_WriteFloatAnsAssessExam (unsigned NumQst,long QstCod,unsigned NumAnswers,MYSQL_RES *mysql_res,double *MarkThisQst)
  {
   MYSQL_ROW row;
   unsigned i;
   double FloatAnsUsr = 0.0,Tmp;
   double FloatAnsCorr[2];

   /***** Check if number of rows is correct *****/
   if (NumAnswers != 2)
      Lay_ShowErrorAndExit ("Wrong float range.");

   /***** Get the numerical value of the minimum and maximum correct answers *****/
   for (i=0; i<2; i++)
     {
      row = mysql_fetch_row (mysql_res);
      Str_ConvertStrFloatCommaToStrFloatPoint (row[1]);
      FloatAnsCorr[i] = Tst_GetFloatReplyFromStr (row[1]);
     }
   if (FloatAnsCorr[0] > FloatAnsCorr[1]) 	// The maximum and the minimum are swapped
    {
      /* Swap maximum and minimum */
      Tmp = FloatAnsCorr[0];
      FloatAnsCorr[0] = FloatAnsCorr[1];
      FloatAnsCorr[1] = Tmp;
     }

   /***** Header with the title of each column *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>");
   Tst_WriteHeadUserCorrect ();
   fprintf (Gbl.F.Out,"</tr>");

   /***** Write the user answer *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"");
   if (Gbl.Test.StrAnswersUsr[NumQst][0])	// If user has answered the question
     {
      Str_ConvertStrFloatCommaToStrFloatPoint (Gbl.Test.StrAnswersUsr[NumQst]);
      FloatAnsUsr = Tst_GetFloatReplyFromStr (Gbl.Test.StrAnswersUsr[NumQst]);
      if (Gbl.Test.StrAnswersUsr[NumQst][0])	// It's a correct floating point number
         fprintf (Gbl.F.Out," class=\"%s\">&nbsp;%lg&nbsp;",
                  (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD) ?
                   ((FloatAnsUsr >= FloatAnsCorr[0] && FloatAnsUsr <= FloatAnsCorr[1]) ? "ANS_OK" : "ANS_BAD") : "ANS",
                  FloatAnsUsr);
      else				// Not a floating point number
         fprintf (Gbl.F.Out," class=\"ANS\">?");
     }
   else					// If user has omitted the answer
      fprintf (Gbl.F.Out," class=\"ANS\">&nbsp;");
   fprintf (Gbl.F.Out,"</td>");

   /***** Write the correct answer *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" class=\"ANS\">");
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD)
      fprintf (Gbl.F.Out,"&nbsp;[%lg; %lg]&nbsp;",FloatAnsCorr[0],FloatAnsCorr[1]);
   else
      fprintf (Gbl.F.Out,"?");
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Compute mark *****/
   if (!Gbl.Test.StrAnswersUsr[NumQst][0])	// If user has omitted the answer
      *MarkThisQst = 0.0;
   else if (FloatAnsUsr >= FloatAnsCorr[0] &&
            FloatAnsUsr <= FloatAnsCorr[1])	// If correct (inside the interval)
      *MarkThisQst = 1.0;
   else						// If wrong (outside the interval)
      *MarkThisQst = 0.0;

   /***** Write mark *****/
   if (Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_GOOD_BAD ||
       Gbl.Test.Config.FeedbackType == Tst_FEEDBACK_EACH_RESULT)
     {
      Tst_WriteScoreStart (2);
      if (!Gbl.Test.StrAnswersUsr[NumQst][0])	// If user has omitted the answer
         fprintf (Gbl.F.Out,"ANS\">0,00");
      else if (FloatAnsUsr >= FloatAnsCorr[0] &&
               FloatAnsUsr <= FloatAnsCorr[1])	// If correct (inside the interval)
         fprintf (Gbl.F.Out,"ANS_OK\">1,00");
      else					// If wrong (outside the interval)
         fprintf (Gbl.F.Out,"ANS_BAD\">0,00");
      Tst_WriteScoreEnd ();
     }

   fprintf (Gbl.F.Out,"</table>");
  }

/*****************************************************************************/
/********* Write head with two columns:                               ********/
/********* one for the user's answer and other for the correct answer ********/
/*****************************************************************************/

static void Tst_WriteHeadUserCorrect (void)
  {
   extern const char *Txt_User[Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_Correct_ANSWER[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<td align=\"center\" class=\"DAT_SMALL\">&nbsp;%s&nbsp;</td>" \
                      "<td align=\"center\" class=\"DAT_SMALL\">&nbsp;%s&nbsp;</td>",
            Txt_User[Gbl.Prefs.Language],Txt_TST_Correct_ANSWER[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/*********** Write the start ans the end of the score of an answer ***********/
/*****************************************************************************/

static void Tst_WriteScoreStart (unsigned ColSpan)
  {
   extern const char *Txt_Score[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"%u\" class=\"DAT_SMALL\">%s: <span class=\"",
            ColSpan,Txt_Score[Gbl.Prefs.Language]);
  }

static void Tst_WriteScoreEnd (void)
  {
   fprintf (Gbl.F.Out,"</span></td></tr>");
  }

/*****************************************************************************/
/*************** Write parameter with the code of a question *****************/
/*****************************************************************************/

static void Tst_WriteParamQstCod (unsigned NumQst,long QstCod)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"Qst%06u\" value=\"%ld\" />",
            NumQst,QstCod);
  }

/*****************************************************************************/
/********************* Check if number of answers is one *********************/
/*****************************************************************************/

static void Tst_CheckIfNumberOfAnswersIsOne (unsigned NumAnswers)
  {
   if (NumAnswers != 1)
      Lay_ShowErrorAndExit ("Wrong answer.");
  }

/*****************************************************************************/
/************************* Get tags of a test question ***********************/
/*****************************************************************************/

static unsigned long Tst_GetTagsQst (long QstCod,MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get the tags of a question from database *****/
   sprintf (Query,"SELECT tst_tags.TagTxt FROM tst_question_tags,tst_tags" \
                  " WHERE tst_question_tags.QstCod='%ld' AND tst_question_tags.TagCod=tst_tags.TagCod AND tst_tags.CrsCod='%ld'" \
                  " ORDER BY tst_question_tags.IndDes",
            QstCod,Gbl.CurrentCrs.Crs.CrsCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get the tags of a question");
  }

/*****************************************************************************/
/******************** Get and write tags of a test question ******************/
/*****************************************************************************/

static void Tst_GetAndWriteTagsQst (long QstCod)
  {
   extern const char *Txt_no_tags[Txt_NUM_LANGUAGES];
   unsigned long NumRow,NumRows;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;

   if ((NumRows = Tst_GetTagsQst (QstCod,&mysql_res)))	// Result: TagTxt
     {
      /***** Write the answers *****/
      fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");
      for (NumRow=1; NumRow<=NumRows; NumRow++)
        {
         row = mysql_fetch_row (mysql_res);
         fprintf (Gbl.F.Out,"<tr>" \
                            "<td align=\"left\" valign=\"top\" class=\"DAT_SMALL\">&nbsp;&#8226;&nbsp;</td>" \
                            "<td align=\"left\" valign=\"top\" class=\"DAT_SMALL\">%s</td>" \
                            "</tr>",
                  row[0]);
        }
      fprintf (Gbl.F.Out,"</table>");
     }
   else
      fprintf (Gbl.F.Out,"<span class=\"DAT_SMALL\">&nbsp;(%s)&nbsp;</span>",
               Txt_no_tags[Gbl.Prefs.Language]);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************ Get parameters for the selection of test questions *************/
/*****************************************************************************/
// Return 1 (OK) if all parameters are found, or 0 (error) if any necessary parameter is not found

static int Tst_GetParamsTst (void)
  {
   extern const char *Txt_You_must_select_one_ore_more_tags[Txt_NUM_LANGUAGES];
   extern const char *Txt_You_must_select_one_ore_more_types_of_answer[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_number_of_questions_must_be_in_the_interval_X[Txt_NUM_LANGUAGES];
   char YN[1+1];
   bool Error = false;
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   /***** Tags *****/
   /* Get parameter that indicates whether all tags are selected */
   Par_GetParam (Par_PARAM_SINGLE,"AllTags",YN,1);
   Gbl.Test.AllTags = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /* Get the tags */
   if ((Gbl.Test.TagsList = malloc (Tst_MAX_BYTES_TAGS_LIST+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store tags.");
   Par_GetParam (Par_PARAM_MULTIPLE,"ChkTag",Gbl.Test.TagsList,Tst_MAX_BYTES_TAGS_LIST);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Test.TagsList,Tst_MAX_BYTES_TAGS_LIST,true);

   /* Check number of tags selected */
   if (Tst_CountNumTagsInList () == 0)	// If no tags selected...
     {					// ...write alert
      Lay_ShowAlert (WARNING,Txt_You_must_select_one_ore_more_tags[Gbl.Prefs.Language]);
      Error = true;
     }

   /***** Types of answer *****/
   /* Get parameter that indicates if all types of answer are selected */
   Par_GetParam (Par_PARAM_SINGLE,"AllAnsTypes",YN,1);
   Gbl.Test.AllAnsTypes = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /* Get types of answer */
   Par_GetParam (Par_PARAM_MULTIPLE,"AnswerType",Gbl.Test.ListAnsTypes,Tst_MAX_BYTES_LIST_ANSWER_TYPES);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Test.ListAnsTypes,Tst_MAX_BYTES_LIST_ANSWER_TYPES,true);

   /* Check number of types of answer */
   if (Tst_CountNumAnswerTypesInList () == 0)	// If no types of answer selected...
     {						// ...write warning alert
      Lay_ShowAlert (WARNING,Txt_You_must_select_one_ore_more_types_of_answer[Gbl.Prefs.Language]);
      Error = true;
     }

   /***** Get other parameters, depending on action *****/
   if (Gbl.CurrentAct == ActSeeTst)
     {
      Gbl.Test.NumQuestions = Tst_GetParamNumQst ();
      if (Gbl.Test.NumQuestions < Gbl.Test.Config.Min ||
          Gbl.Test.NumQuestions > Gbl.Test.Config.Max) 
        {
         sprintf (Gbl.Message,Txt_The_number_of_questions_must_be_in_the_interval_X[Gbl.Prefs.Language],
                  Gbl.Test.Config.Min,Gbl.Test.Config.Max);
         Lay_ShowAlert (WARNING,Gbl.Message);
         Error = true;
        }
     }
   else
     {
      /* Get starting and ending dates */
      Dat_GetIniEndDatesFromForm ();

      /* Get ordering criteria */
      Par_GetParam (Par_PARAM_MULTIPLE,"Order",UnsignedStr,10);
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
         Gbl.Test.SelectedOrderType = (Tst_QuestionsOrder_t) ((UnsignedNum < Tst_NUM_CRIT_ORDEN_TST) ? UnsignedNum : 0);
      else
         Gbl.Test.SelectedOrderType = (Tst_QuestionsOrder_t) 0;
     }

   return Error ? 0 : 1;
  }

/*****************************************************************************/
/******************** Get parameter with the number of test ******************/
/*****************************************************************************/

static unsigned Tst_GetAndCheckParamNumTst (void)
  {
   char IntStr[1+10+1];
   int IntNum;

   /***** Get number of questions *****/
   Par_GetParam (Par_PARAM_SINGLE,"NumTst",IntStr,10);
   if (!IntStr[0])
      Lay_ShowErrorAndExit ("Number of test is missing.");

   if (sscanf (IntStr,"%d",&IntNum) == 1)
     {
      if (IntNum < (int) 1)
         Lay_ShowErrorAndExit ("Wrong number of test.");
     }
   else
      Lay_ShowErrorAndExit ("Wrong number of test.");

   return (unsigned) IntNum;
  }

/*****************************************************************************/
/***** Get parameter with the number of questions to generate in an test *****/
/*****************************************************************************/

static unsigned Tst_GetParamNumQst (void)
  {
   char IntStr[1+10+1];
   int IntNum;

   /***** Get number of questions *****/
   Par_GetParam (Par_PARAM_SINGLE,"NumQst",IntStr,10);
   if (IntStr[0])
     {
      if (sscanf (IntStr,"%d",&IntNum) != 1)
         return UINT_MAX;
     }
   else	// Parameter NumQst not found
      return UINT_MAX;

   return (unsigned) IntNum;
  }

/*****************************************************************************/
/***************** Count number of tags in the list of tags ******************/
/*****************************************************************************/

static int Tst_CountNumTagsInList (void)
  {
   const char *Ptr;
   int NumTags = 0;
   char TagText[Tst_MAX_BYTES_TAG+1];

   /***** Go over the list Gbl.Test.TagsList counting the number of tags *****/
   if (Gbl.Test.TagsList)
     {
      Ptr = Gbl.Test.TagsList;
      while (*Ptr)
        {
         Str_GetNextStrUntilSeparParamMult (&Ptr,TagText,Tst_MAX_BYTES_TAG);
         NumTags++;
        }
     }
   return NumTags;
  }

/*****************************************************************************/
/**** Count the number of types of answers in the list of types of answers ***/
/*****************************************************************************/

static int Tst_CountNumAnswerTypesInList (void)
  {
   const char *Ptr;
   int NumTiposRes = 0;
   char UnsignedStr[10+1];
   Tst_AnswerType_t AnsType;

   /***** Go over the list Gbl.Test.ListAnsTypes counting the number of types of answer *****/
   Ptr = Gbl.Test.ListAnsTypes;
   while (*Ptr)
     {
      Str_GetNextStrUntilSeparParamMult (&Ptr,UnsignedStr,10);
      AnsType = Tst_ConvertFromUnsignedStrToAnsTyp (UnsignedStr);
      NumTiposRes++;
     }
   return NumTiposRes;
  }

/*****************************************************************************/
/**************** Free memory allocated for the list of tags *****************/
/*****************************************************************************/

void Tst_FreeTagsList (void)
  {
   if (Gbl.Test.TagsList)
     {
      free ((void *) Gbl.Test.TagsList);
      Gbl.Test.TagsList = NULL;
     }
  }

/*****************************************************************************/
/******************** Show form to edit one test question ********************/
/*****************************************************************************/

// This function can be called from three places:
// 1. Pressing edition of question in the menu
// 2. Pressing button to edit question in a listing of existing questions
// 3. From the action associated to reception of a question, on error in the parameters received from the form

void Tst_ShowFormEditOneQst (void)
  {
   extern const char *Txt_Code[Txt_NUM_LANGUAGES];
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Tags[Txt_NUM_LANGUAGES];
   extern const char *Txt_new_tag[Txt_NUM_LANGUAGES];
   extern const char *Txt_Stem[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type[Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES][Txt_NUM_LANGUAGES];
   extern const char *Txt_Answers[Txt_NUM_LANGUAGES];
   extern const char *Txt_TST_TF_QST[2][Txt_NUM_LANGUAGES];
   extern const char *Txt_Integer_number[Txt_NUM_LANGUAGES];
   extern const char *Txt_Real_number_between_A_and_B_1[Txt_NUM_LANGUAGES];
   extern const char *Txt_Real_number_between_A_and_B_2[Txt_NUM_LANGUAGES];
   extern const char *Txt_Shuffle[Txt_NUM_LANGUAGES];
   extern const char *Txt_Send[Txt_NUM_LANGUAGES];
   char Title[256];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   unsigned NumAns,NumAnswers = 0;
   Tst_AnswerType_t AnsType;
   bool Shuffle = false;
   unsigned NumTag;
   bool TagNotFound;

   if (Gbl.CurrentAct == ActWriTstQst) // If no receiving the question, but editing a new or existing question
     {
      Tst_InitQst ();
      if (Tst_GetQstCod ())	// If parameter QstCod received ==> question already exists in the database
        {
         sprintf (Title,"%s <strong>%ld</strong>",
                  Txt_Code[Gbl.Prefs.Language],Gbl.Test.QstCod);
         Lay_WriteTitle (Title);

         /***** Get the type of answer and the stem from the database *****/
         /* Get the question from database */
         sprintf (Query,"SELECT AnsType,Shuffle,Stem FROM tst_questions" \
                        " WHERE QstCod='%ld' AND CrsCod='%ld'",
                  Gbl.Test.QstCod,Gbl.CurrentCrs.Crs.CrsCod);
         DB_QuerySELECT (Query,&mysql_res,"can not get a question");

         row = mysql_fetch_row (mysql_res);

         /* Get the type of answer */
         Gbl.Test.AnswerType = Tst_ConvertFromStrAnsTypDBToAnsTyp (row[0]);

         /* Get shuffle */
         Shuffle = (Str_ConvertToUpperLetter (row[1][0]) != 'N');

         /* Get the stem of the question from the database */
         if (!Str_AllocateText ())
            Lay_ShowErrorAndExit (Gbl.Message);
         strncpy (Gbl.Txt,row[2],Cte_MAX_BYTES_TEXT);
         Gbl.Txt[Cte_MAX_BYTES_TEXT] = '\0';

         /* Free structure that stores the query result */
	 DB_FreeMySQLResult (&mysql_res);

         /***** Get the tags from the database *****/
         NumRows = Tst_GetTagsQst (Gbl.Test.QstCod,&mysql_res);
         for (NumRow = 0; NumRow < NumRows; NumRow++)
           {
            row = mysql_fetch_row (mysql_res);
            strncpy (Gbl.Test.TagText[NumRow],row[0],Tst_MAX_BYTES_TAG);
            Gbl.Test.TagText[NumRow][Tst_MAX_BYTES_TAG] = '\0';
           }

         /* Free structure that stores the query result */
	 DB_FreeMySQLResult (&mysql_res);

         /***** Get the answers from the database *****/
         NumAnswers = Tst_GetAnswersQst (Gbl.Test.QstCod,&mysql_res,false);	// Result: AnsInd,Answer,Correct
         for (NumAns=0; NumAns<NumAnswers; NumAns++)
           {
            row = mysql_fetch_row (mysql_res);
            switch (Gbl.Test.AnswerType)
              {
               case Tst_ANS_TRUE_FALSE:
                  if (NumAnswers != 1)
                     Lay_ShowErrorAndExit ("Wrong answer.");
                  Gbl.Test.AnsTF = row[1][0];
                  break;
               case Tst_ANS_UNIQUE_CHOICE:
               case Tst_ANS_MULTIPLE_CHOICE:
               case Tst_ANS_TEXT:
                  if (NumAnswers > Tst_MAX_ANSWERS_PER_QUESTION)
                     Lay_ShowErrorAndExit ("Wrong answer.");
                  if (!Tst_AllocateTextChoiceAnswer (NumAns))
                     Lay_ShowErrorAndExit (Gbl.Message);
                  strncpy (Gbl.Test.AnsChoice[NumAns].Text,row[1],Txt_MAX_BYTES_ANSWER);
                  Gbl.Test.AnsChoice[NumAns].Text[Txt_MAX_BYTES_ANSWER] = '\0';
                  Gbl.Test.AnsChoice[NumAns].Correct = (Str_ConvertToUpperLetter (row[2][0]) == 'Y');
                  break;
               case Tst_ANS_INT:
                  if (NumAnswers != 1)
                     Lay_ShowErrorAndExit ("Wrong answer.");
                  if (sscanf (row[1],"%ld",&Gbl.Test.AnsInt) != 1)
                     Gbl.Test.AnsInt = 0;
                  break;
               case Tst_ANS_FLOAT:
                  if (NumAnswers != 2)
                     Lay_ShowErrorAndExit ("Wrong answer.");
                  Str_ConvertStrFloatCommaToStrFloatPoint (row[1]);
                  Gbl.Test.AnsFloat[NumAns] = Tst_GetFloatReplyFromStr (row[1]);
                  break;
              }
           }
         /* Free structure that stores the query result */
	 DB_FreeMySQLResult (&mysql_res);
        }
     }

   /***** Start form *****/
   Act_FormStart (ActRecTstQst);
   if (Gbl.Test.QstCod != -1)	// If the question already has assigned a code
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"QstCod\" value=\"%ld\" />",
               Gbl.Test.QstCod);

   fprintf (Gbl.F.Out,"<div align=\"center\"><table>\n");

   /***** Get tags already existing for questions in current course *****/
   NumRows = Tst_GetAllTagsFromCurrentCrs (&mysql_res);

   /***** Write the tags *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\"><table cellspacing=\"0\" cellpadding=\"0\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Tags[Gbl.Prefs.Language]);
   for (NumTag=0; NumTag<Tst_MAX_TAGS_PER_QUESTION; NumTag++)
     {
      fprintf (Gbl.F.Out,"<tr>");

      /***** Write the tags already existing in a selector *****/
      fprintf (Gbl.F.Out,"<td align=\"left\"><select id=\"SelDesc%u\" name=\"SelDesc%u\" style=\"width:230px;\" onchange=\"changeTxtTag('%u')\">",
               NumTag,NumTag,NumTag);
      fprintf (Gbl.F.Out,"<option value=\"\">&nbsp;</option>");
      mysql_data_seek (mysql_res, 0);
      TagNotFound = true;
      for (NumRow=1; NumRow<=NumRows; NumRow++)
        {
         row = mysql_fetch_row (mysql_res);

         fprintf (Gbl.F.Out,"<option value=\"%s\"",row[1]);
         if (!strcasecmp (Gbl.Test.TagText[NumTag],row[1]))
           {
  	    fprintf (Gbl.F.Out," selected=\"selected\"");
            TagNotFound = false;
           }
         fprintf (Gbl.F.Out,">%s</option>",row[1]);
        }
      /* If it's a new tag received from the form */
      if (TagNotFound && Gbl.Test.TagText[NumTag][0])
         fprintf (Gbl.F.Out,"<option value=\"%s\" selected=\"selected\">%s</option>",
                  Gbl.Test.TagText[NumTag],Gbl.Test.TagText[NumTag]);
      fprintf (Gbl.F.Out,"<option value=\"\">[%s]</option></select></td>",
               Txt_new_tag[Gbl.Prefs.Language]);

      /***** Input of a new tag *****/
      fprintf (Gbl.F.Out,"<td align=\"right\">" \
                         "<input type=\"text\" id=\"TagTxt%u\" name=\"TagTxt%u\" size=\"36\" maxlength=\"%u\" value=\"%s\" onchange=\"changeSelTag('%u')\" />" \
                         "</td>",
	       NumTag,NumTag,Tst_MAX_TAG_LENGTH,Gbl.Test.TagText[NumTag],NumTag);

      fprintf (Gbl.F.Out,"</tr>\n");
     }
   fprintf (Gbl.F.Out,"</table></td></tr>\n");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   /***** Stem *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Stem[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\"><textarea name=\"Txt\" cols=\"60\" rows=\"8\">");
   if (Gbl.Txt)
      fprintf (Gbl.F.Out,Gbl.Txt);
   fprintf (Gbl.F.Out,"</textarea></td></tr>\n");

   /***** Type of answer *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Type[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">",
            The_ClassFormul[Gbl.Prefs.Theme]);
   for (AnsType=(Tst_AnswerType_t) 0; AnsType<Tst_NUM_ANS_TYPES; AnsType++)
     {
      fprintf (Gbl.F.Out,"<input type=\"radio\" name=\"AnswerType\" value=\"%u\"",
               (unsigned) AnsType);
      if (AnsType == Gbl.Test.AnswerType)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," onclick=\"enableDisableAns(this.form)\" />%s&nbsp;<br />",
               Txt_TST_STR_ANSWER_TYPES[AnsType][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Answers *****/
   /* T/F answer */
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" valign=\"top\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\" valign=\"top\"><table cellspacing=\"0\" cellpadding=\"0\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Answers[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\" class=\"%s\">",
            The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<input type=\"radio\" name=\"AnsTF\" value=\"T\"");
   if (Gbl.Test.AnsTF == 'T')
      fprintf (Gbl.F.Out," checked=\"checked\"");
   if (Gbl.Test.AnswerType != Tst_ANS_TRUE_FALSE)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," />%s&nbsp;",Txt_TST_TF_QST[0][Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<input type=\"radio\" name=\"AnsTF\" value=\"F\"");
   if (Gbl.Test.AnsTF == 'F')
      fprintf (Gbl.F.Out," checked=\"checked\"");
   if (Gbl.Test.AnswerType != Tst_ANS_TRUE_FALSE)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," />%s",Txt_TST_TF_QST[1][Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</td></tr>");

   fprintf (Gbl.F.Out,"</table></td></tr>\n");

   /* Integer answer */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\"></td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">%s: " \
                      "<input type=\"text\" name=\"AnsInt\" size=\"11\" maxlength=\"11\" value=\"%ld\"",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Integer_number[Gbl.Prefs.Language],
            Gbl.Test.AnsInt);
   if (Gbl.Test.AnswerType != Tst_ANS_INT)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," /></td></tr>\n");

   /* Floating point answer */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\"></td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">%s " \
                      "<input type=\"text\" name=\"AnsFloatMin\" size=\"11\" maxlength=\"%u\" value=\"%lg\"",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Real_number_between_A_and_B_1[Gbl.Prefs.Language],
            Txt_MAX_BYTES_FLOAT_ANSWER,Gbl.Test.AnsFloat[0]);
   if (Gbl.Test.AnswerType != Tst_ANS_FLOAT)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," /> %s <input type=\"text\" name=\"AnsFloatMax\" size=\"11\" maxlength=\"%u\" value=\"%lg\"",
            Txt_Real_number_between_A_and_B_2[Gbl.Prefs.Language],
            Txt_MAX_BYTES_FLOAT_ANSWER,Gbl.Test.AnsFloat[1]);
   if (Gbl.Test.AnswerType != Tst_ANS_FLOAT)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," /></td></tr>\n");

   /* Questions can be shuffled? */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\"></td>" \
                      "<td align=\"left\" valign=\"top\" class=\"%s\">" \
                      "<input type=\"checkbox\" name=\"Shuffle\" value=\"Y\"",
            The_ClassFormul[Gbl.Prefs.Theme]);
   if (Shuffle)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   if (Gbl.Test.AnswerType != Tst_ANS_UNIQUE_CHOICE &&
       Gbl.Test.AnswerType != Tst_ANS_MULTIPLE_CHOICE)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," />%s</td></tr>",
            Txt_Shuffle[Gbl.Prefs.Language]);

   /* Simple or multiple choice answers */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\"></td><td align=\"left\" valign=\"top\"><table cellspacing=\"0\" cellpadding=\"0\">");

   for (NumAns=0; NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
     {
      /* Selectores y rtulo with the letra of the answer */
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\" class=\"%s\">",The_ClassFormul[Gbl.Prefs.Theme]);
      fprintf (Gbl.F.Out,"<input type=\"radio\" name=\"AnsUni\" value=\"%u\"",NumAns);
      if (Gbl.Test.AnswerType != Tst_ANS_UNIQUE_CHOICE)
         fprintf (Gbl.F.Out," disabled=\"disabled\"");
      if (Gbl.Test.AnsChoice[NumAns].Correct)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," />");
      fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"AnsMulti\" value=\"%u\"",NumAns);
      if (Gbl.Test.AnswerType != Tst_ANS_MULTIPLE_CHOICE)
         fprintf (Gbl.F.Out," disabled=\"disabled\"");
      if (Gbl.Test.AnsChoice[NumAns].Correct)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," />");
      fprintf (Gbl.F.Out,"&nbsp;%c)&nbsp;</td>",'a' + (char) NumAns);

      /* Answer text */
      fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\"><textarea name=\"AnsStr%u\" cols=\"50\" rows=\"3\"",NumAns);
      if (Gbl.Test.AnswerType != Tst_ANS_UNIQUE_CHOICE && Gbl.Test.AnswerType != Tst_ANS_MULTIPLE_CHOICE && Gbl.Test.AnswerType != Tst_ANS_TEXT)
         fprintf (Gbl.F.Out," disabled=\"disabled\"");
      fprintf (Gbl.F.Out,">");
      if (Gbl.Test.AnsChoice[NumAns].Text)
         fprintf (Gbl.F.Out,Gbl.Test.AnsChoice[NumAns].Text);
      fprintf (Gbl.F.Out,"</textarea></td></tr>\n");
     }

   /***** Send and undo buttons *****/
   fprintf (Gbl.F.Out,"</table></tr></table></div>\n");
   Lay_PutSendAndUndoButtons (Txt_Send[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");

   Tst_FreeTextChoiceAnswers (NumAnswers);
   Str_FreeTxt ();
  }

/*****************************************************************************/
/********************* Initialize a new question to zero *********************/
/*****************************************************************************/

static void Tst_InitQst (void)
  {
   unsigned NumAns;
   unsigned NumTag;

   Gbl.Test.QstCod = -1;
   for (NumTag=0; NumTag<Tst_MAX_TAGS_PER_QUESTION; NumTag++)
      Gbl.Test.TagText[NumTag][0] = '\0';
   Gbl.Test.TagsList = NULL;
   Gbl.Txt = NULL;
   Gbl.Test.AnswerType = Tst_ANS_UNIQUE_CHOICE;
   Gbl.Test.AnsTF = ' ';
   Gbl.Test.AnsInt = 0;
   Gbl.Test.AnsFloat[0] = Gbl.Test.AnsFloat[1] = 0.0;
   for (NumAns=0; NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
     {
      Gbl.Test.AnsChoice[NumAns].Correct = false;
      Gbl.Test.AnsChoice[NumAns].Text    = NULL;
     }
  }

/*****************************************************************************/
/***** Convert una cadena with the tipo of answer a tipo of answer *****/
/*****************************************************************************/

static Tst_AnswerType_t Tst_ConvertFromStrAnsTypDBToAnsTyp (const char *StrAnsTypeBD)
  {
   Tst_AnswerType_t AnsType;

   for (AnsType=(Tst_AnswerType_t) 0; AnsType<Tst_NUM_ANS_TYPES; AnsType++)
      if (!strcmp (StrAnsTypeBD,Tst_StrAnswerTypesDB[AnsType]))
         return AnsType;

   return (Tst_AnswerType_t) 0;
  }

/*****************************************************************************/
/****** Convert una cadena with a unsigned a number tipo of answer ******/
/*****************************************************************************/

static Tst_AnswerType_t Tst_ConvertFromUnsignedStrToAnsTyp (const char *UnsignedStr)
  {
   unsigned AnsType;

   if (sscanf (UnsignedStr,"%u",&AnsType) != 1)
      Lay_ShowErrorAndExit ("Wrong type of answer.");
   if (AnsType >= Tst_NUM_ANS_TYPES)
      Lay_ShowErrorAndExit ("Wrong type of answer.");
   return (Tst_AnswerType_t) AnsType;
  }

/*****************************************************************************/
/*************** Receive a question of the self-assessment test **************/
/*****************************************************************************/

void Tst_ReceiveQst (void)
  {
   char UnsignedStr[10+1];
   char YN[1+1];
   char StrLong[1+10+1];
   char StrFloat[2][Txt_MAX_BYTES_FLOAT_ANSWER+1];
   char *Query;
   unsigned NumTag,NumTagRead,TagIdx,NumAns,i;
   char TagStr[7+10+1];
   char AnsStr[8+10+1];
   bool Error = false,ThereIsEndOfAnswers;
   unsigned NumCorrectAns,NumLastAns;
   char StrMultiAns[Tst_MAX_LENGTH_ANS_USR+1];
   const char *Ptr;
   long TagCod;
   bool Shuffle = false;

   /***** Initialize new question to zero *****/
   Tst_InitQst ();

   /***** Get parameters from form *****/
   /* Get question code */
   Tst_GetQstCod ();

   /* Get answer type */
   Par_GetParam (Par_PARAM_SINGLE,"AnswerType",UnsignedStr,10);
   Gbl.Test.AnswerType = Tst_ConvertFromUnsignedStrToAnsTyp (UnsignedStr);

   /* Get question tags */
   for (NumTag=0; NumTag<Tst_MAX_TAGS_PER_QUESTION; NumTag++)
     {
      sprintf (TagStr,"TagTxt%u",NumTag);
      Par_GetParam (Par_PARAM_SINGLE,TagStr,Gbl.Test.TagText[NumTag],Tst_MAX_BYTES_TAG);
      if (Gbl.Test.TagText[NumTag][0])
        {
         Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Test.TagText[NumTag],Tst_MAX_BYTES_TAG,true);
         /* Check if not repeated */
         for (NumTagRead=0; NumTagRead<NumTag; NumTagRead++)
            if (!strcmp (Gbl.Test.TagText[NumTagRead],Gbl.Test.TagText[NumTag]))
              {
               Gbl.Test.TagText[NumTag][0] = '\0';
               break;
              }
        }
     }

   /* Get question text */
   Str_AllocateAndGetText (Str_TO_HTML,true);

   /* Get answers */
   switch (Gbl.Test.AnswerType)
     {
      case Tst_ANS_TRUE_FALSE:
	 Par_GetParam (Par_PARAM_SINGLE,"AnsTF",YN,1);
         Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,YN,1,true);	// No es necesario
	 Gbl.Test.AnsTF = YN[0];
	 break;
      case Tst_ANS_UNIQUE_CHOICE:
      case Tst_ANS_MULTIPLE_CHOICE:
         /* Get shuffle */
         Par_GetParam (Par_PARAM_SINGLE,"Shuffle",YN,1);
         Shuffle = (Str_ConvertToUpperLetter (YN[0]) == 'Y');
         // No break
      case Tst_ANS_TEXT:
         /* Get the texts of the answers */
         for (NumAns=0; NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
           {
            Tst_AllocateTextChoiceAnswer (NumAns);
            sprintf (AnsStr,"AnsStr%u",NumAns);
	    Par_GetParam (Par_PARAM_SINGLE,AnsStr,Gbl.Test.AnsChoice[NumAns].Text,Txt_MAX_BYTES_ANSWER);
	    Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,Gbl.Test.AnsChoice[NumAns].Text,Txt_MAX_BYTES_ANSWER,true);
            /* Join several spaces into one in user answer */
            Str_ReplaceSeveralSpacesForOne (Gbl.Test.AnsChoice[NumAns].Text);
           }

         /* Get the numbers of correct answers */
         if (Gbl.Test.AnswerType == Tst_ANS_UNIQUE_CHOICE)
           {
	    Par_GetParam (Par_PARAM_SINGLE,"AnsUni",UnsignedStr,10);
            Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,UnsignedStr,10,true);	// Not necessary
            if (UnsignedStr[0])
              {
               if (sscanf (UnsignedStr,"%u",&NumCorrectAns) != 1)
	          Lay_ShowErrorAndExit ("Wrong selected answer.");
               if (NumCorrectAns >= Tst_MAX_ANSWERS_PER_QUESTION)
	          Lay_ShowErrorAndExit ("Wrong selected answer.");
               Gbl.Test.AnsChoice[NumCorrectAns].Correct = true;
              }
           }
      	 else if (Gbl.Test.AnswerType == Tst_ANS_MULTIPLE_CHOICE)
           {
	    Par_GetParam (Par_PARAM_MULTIPLE,"AnsMulti",StrMultiAns,Tst_MAX_LENGTH_ANS_USR);
            Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,StrMultiAns,Tst_MAX_LENGTH_ANS_USR,true);	// No es necesario
 	    Ptr = StrMultiAns;
            while (*Ptr)
              {
  	       Str_GetNextStrUntilSeparParamMult (&Ptr,UnsignedStr,10);
	       if (sscanf (UnsignedStr,"%u",&NumCorrectAns) != 1)
	          Lay_ShowErrorAndExit ("Wrong selected answer.");
               if (NumCorrectAns >= Tst_MAX_ANSWERS_PER_QUESTION)
	          Lay_ShowErrorAndExit ("Wrong selected answer.");
               Gbl.Test.AnsChoice[NumCorrectAns].Correct = true;
              }
           }
         else // Tst_ANS_TEXT
            for (NumAns=0; NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
               if (Gbl.Test.AnsChoice[NumAns].Text[0])
                  Gbl.Test.AnsChoice[NumAns].Correct = true;	// All the answers are correct
	 break;
      case Tst_ANS_INT:
	 Par_GetParam (Par_PARAM_SINGLE,"AnsInt",StrLong,1+10);
         Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,StrLong,1+10,true);
	 break;
      case Tst_ANS_FLOAT:
	 Par_GetParam (Par_PARAM_SINGLE,"AnsFloatMin",StrFloat[0],Txt_MAX_BYTES_FLOAT_ANSWER);
         Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,StrFloat[0],Txt_MAX_BYTES_FLOAT_ANSWER,true);
         Str_ConvertStrFloatCommaToStrFloatPoint (StrFloat[0]);
	 Par_GetParam (Par_PARAM_SINGLE,"AnsFloatMax",StrFloat[1],Txt_MAX_BYTES_FLOAT_ANSWER);
         Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,StrFloat[1],Txt_MAX_BYTES_FLOAT_ANSWER,true);
         Str_ConvertStrFloatCommaToStrFloatPoint (StrFloat[1]);
	 break;
     }

   /***** Make sure that tags, text and answer are not empty *****/
   for (NumTag=0; NumTag<Tst_MAX_TAGS_PER_QUESTION; NumTag++)
      if (Gbl.Test.TagText[NumTag][0])
         break;
   if (NumTag == Tst_MAX_TAGS_PER_QUESTION) // All tags revised ==> all are empty
     {
      Lay_ShowAlert (WARNING,"Debe escribir al menos un descriptor para la pregunta.");	// Need translation!
      Error = true;
     }
   else if (!Gbl.Txt[0])
     {
      Lay_ShowAlert (WARNING,"Debe escribir el enunciado de la pregunta.");	// Need translation!
      Error = true;
     }
   else
      switch (Gbl.Test.AnswerType)
        {
         case Tst_ANS_TRUE_FALSE:
	    if (Gbl.Test.AnsTF != 'T' &&
                Gbl.Test.AnsTF != 'F')
              {
	       Lay_ShowAlert (WARNING,"Debe seleccionar una respuesta verdadero o falso.");	// Need translation!
               Error = true;
              }
	    break;
         case Tst_ANS_UNIQUE_CHOICE:
         case Tst_ANS_MULTIPLE_CHOICE:
            if (Gbl.Test.AnsChoice[0].Text[0])	// Si se ha rellenado the primera answer
              {
               for (NumAns=0, NumLastAns=0, ThereIsEndOfAnswers=false; !Error && NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
                  if (Gbl.Test.AnsChoice[NumAns].Text[0])
                    {
                     if (ThereIsEndOfAnswers)
                       {
                        Lay_ShowAlert (ERROR,"No puede dejar respuestas intermedias sin rellenar.");	// Need translation!
                        Error = true;
                       }
                     else
                        NumLastAns = NumAns;
                    }
                  else
                     ThereIsEndOfAnswers = true;
               if (!Error)
                 {
                  if (NumLastAns < 1)
                    {
                     Lay_ShowAlert (ERROR,"Debe escribir al menos las dos primeras respuestas.");	// Need translation!
                     Error = true;
                    }
                  else
                    {
                     for (NumAns=0; NumAns<=NumLastAns; NumAns++)
                        if (Gbl.Test.AnsChoice[NumAns].Correct)
                           break;
                     if (NumAns > NumLastAns)
                       {
                        Lay_ShowAlert (ERROR,"Debe marcar alguna respuesta como correcta.");	// Need translation!
                        Error = true;
                       }
                    }
                 }
              }
            else	// If first answer is empty
              {
               Lay_ShowAlert (ERROR,"Debe escribir al menos las dos primeras respuestas.");	// Need translation!
               Error = true;
              }
	    break;
         case Tst_ANS_TEXT:
            if (Gbl.Test.AnsChoice[0].Text[0])	// Si se ha rellenado the primera answer
              {
               for (NumAns=0, ThereIsEndOfAnswers=false; !Error && NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
                  if (Gbl.Test.AnsChoice[NumAns].Text[0])
                    {
                     if (ThereIsEndOfAnswers)
                       {
                        Lay_ShowAlert (ERROR,"No puede dejar respuestas intermedias sin rellenar.");	// Need translation!
                        Error = true;
                       }
                    }
                  else
                     ThereIsEndOfAnswers = true;
              }
            else	// If first answer is empty
              {
               Lay_ShowAlert (ERROR,"Debe escribir al menos la primera respuesta.");	// Need translation!
               Error = true;
              }
	    break;
         case Tst_ANS_INT:
            if (sscanf (StrLong,"%ld",&Gbl.Test.AnsInt) != 1)
              {
               Lay_ShowAlert (WARNING,"Debe escribir un valor entero como respuesta correcta.");	// Need translation!
               Error = true;
              }
            break;
         case Tst_ANS_FLOAT:
            for (i=0; i<2; i++)
              {
               Str_ConvertStrFloatCommaToStrFloatPoint (StrFloat[i]);
               Gbl.Test.AnsFloat[i] = Tst_GetFloatReplyFromStr (StrFloat[i]);
              }
            if (!StrFloat[0][0] || !StrFloat[1][0])
              {
               Lay_ShowAlert (WARNING,"Debe escribir un intervalo de valores reales permitidos como respuesta.");	// Need translation!
               Error = true;
              }
            if (!Error)
               if (Gbl.Test.AnsFloat[0] > Gbl.Test.AnsFloat[1])
                 {
                  Lay_ShowAlert (WARNING,"El l&iacute;mite inferior del intervalo de respuestas correctas" \
                                         " debe ser menor o igual que el l&iacute;mite superior.");	// Need translation!
                  Error = true;
                 }
            break;
        }

   if (Error)
      Tst_ShowFormEditOneQst ();
   else
     {
      /***** Form is received OK ==> insert question and answer in the database *****/
      /* Allocate space for query */
      if ((Query = malloc (256+Cte_MAX_BYTES_TEXT+1)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store database query.");

      if (Gbl.Test.QstCod == -1)	// It's a new question
        {
         /* Insert question in the table of questions */
         sprintf (Query,"INSERT INTO tst_questions (CrsCod,EditTime,AnsType,Shuffle,Stem)" \
                        " VALUES ('%ld',NOW(),'%s','%c','%s')",
	          Gbl.CurrentCrs.Crs.CrsCod,Tst_StrAnswerTypesDB[Gbl.Test.AnswerType],Shuffle ? 'Y' : 'N',Gbl.Txt);
         Gbl.Test.QstCod = DB_QueryINSERTandReturnCode (Query,"can not create question");
        }
      else				// It's an existing question
        {
         /* Update question */
         sprintf (Query,"UPDATE tst_questions SET EditTime=NOW(),Stem='%s',AnsType='%s',Shuffle='%c'" \
                        " WHERE QstCod='%ld'",
                  Gbl.Txt,Tst_StrAnswerTypesDB[Gbl.Test.AnswerType],Shuffle ? 'Y' : 'N',Gbl.Test.QstCod);
         DB_QueryUPDATE (Query,"can not update question");

         /* Remove answers and tags from this test question */
         Tst_RemAnsFromQst (Gbl.Test.QstCod);
         Tst_RemTagsFromQst (Gbl.Test.QstCod);
        }

      /* Insert tags in the tags table */
      for (NumTag=0, TagIdx=0; NumTag<Tst_MAX_TAGS_PER_QUESTION; NumTag++)
         if (Gbl.Test.TagText[NumTag][0])
           {
	    /* Check if this tag exists for current course */
            if ((TagCod = Tst_GetTagCodeFromTagTxt (Gbl.Test.TagText[NumTag])) == -1)
               /* This tag is new for current course. Add it to tags table */
               TagCod = Tst_CreateNewTag (Gbl.CurrentCrs.Crs.CrsCod,Gbl.Test.TagText[NumTag]);

            /* Insert tag in tst_question_tags */
            sprintf (Query,"INSERT INTO tst_question_tags (QstCod,TagCod,IndDes) VALUES ('%ld','%ld','%u')",
                     Gbl.Test.QstCod,TagCod,TagIdx);
            DB_QueryINSERT (Query,"can not create tag");
            TagIdx++;
           }

      /* Remove unused tags in current course */
      Tst_RemoveUnusedTagsFromCurrentCrs ();

      /* Insert answers in the answers table */
      switch (Gbl.Test.AnswerType)
        {
         case Tst_ANS_TRUE_FALSE:
            sprintf (Query,"INSERT INTO tst_answers (QstCod,AnsInd,Answer,Correct)" \
                           " VALUES (%ld,0,'%c','Y')",
                     Gbl.Test.QstCod,Gbl.Test.AnsTF);
            DB_QueryINSERT (Query,"can not create answer");
           break;
	 case Tst_ANS_UNIQUE_CHOICE:
	 case Tst_ANS_MULTIPLE_CHOICE:
         case Tst_ANS_TEXT:
            for (NumAns=0; NumAns<Tst_MAX_ANSWERS_PER_QUESTION; NumAns++)
               if (Gbl.Test.AnsChoice[NumAns].Text[0])
                 {
                  sprintf (Query,"INSERT INTO tst_answers (QstCod,AnsInd,Answer,Correct)" \
                                 " VALUES (%ld,%u,'%s','%c')",
                           Gbl.Test.QstCod,NumAns,Gbl.Test.AnsChoice[NumAns].Text,Gbl.Test.AnsChoice[NumAns].Correct ? 'Y' : 'N');
                  DB_QueryINSERT (Query,"can not create answer");
                 }
	    break;
         case Tst_ANS_INT:
            sprintf (Query,"INSERT INTO tst_answers (QstCod,AnsInd,Answer,Correct)" \
                           " VALUES (%ld,0,'%ld','Y')",
                     Gbl.Test.QstCod,Gbl.Test.AnsInt);
            DB_QueryINSERT (Query,"can not create answer");
            break;
         case Tst_ANS_FLOAT:
   	    setlocale (LC_NUMERIC,"en_US.iso88591");	// To store in the database in floating point format
   	    
            sprintf (Query,"INSERT INTO tst_answers (QstCod,AnsInd,Answer,Correct)" \
                           " VALUES (%ld,0,'%lg','Y')",
                     Gbl.Test.QstCod,Gbl.Test.AnsFloat[0]);
            DB_QueryINSERT (Query,"can not create answer");
            
            sprintf (Query,"INSERT INTO tst_answers (QstCod,AnsInd,Answer,Correct)" \
                           " VALUES (%ld,1,'%lg','Y')",
                     Gbl.Test.QstCod,Gbl.Test.AnsFloat[1]);
            DB_QueryINSERT (Query,"can not create answer");
            
   	    setlocale (LC_NUMERIC,"es_ES.iso88591");
            break;
        }

      free ((void *) Query);

      /***** Listar the question recin introducida in the database *****/
      Tst_ListOneQstToEdit ();
     }

   /***** Free answers *****/
   Tst_FreeTextChoiceAnswers (Tst_MAX_ANSWERS_PER_QUESTION);

   /***** Free body utilizado for the stem of the question *****/
   Str_FreeTxt ();
  }

/****************************************************************************/
/************* Get a real number from a string in floating point ************/
/****************************************************************************/

static double Tst_GetFloatReplyFromStr (char *Str)
  {
   double NumReal;

   /***** The string is "scanned" in floating point (it must have a point, not a colon as decimal separator) *****/
   setlocale (LC_NUMERIC,"en_US.iso88591");	// To use the number in floating point
   if (sscanf (Str,"%lg",&NumReal) != 1)	// If the string doesn't hold a valid floating point number...
     {
      NumReal = 0.0;	// ...the number is reset to 0
      Str[0] = '\0';	// ...and the string is reset to ""
     }
   setlocale (LC_NUMERIC,"es_ES.iso88591");	// Return to spanish system (this should be internationalized!!!!!!!)

   return NumReal;
  }

/*****************************************************************************/
/***************** Check if this tag exists for current course ***************/
/*****************************************************************************/

static long Tst_GetTagCodeFromTagTxt (const char *TagTxt)
  {
   char Query[256+Tst_MAX_BYTES_TAG];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long TagCod = -1;
   bool Error = false;

   /***** Get tag code from database *****/
   sprintf (Query,"SELECT TagCod FROM tst_tags WHERE tst_tags.CrsCod='%ld' AND tst_tags.TagTxt='%s'",
            Gbl.CurrentCrs.Crs.CrsCod,TagTxt);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get tag");

   if (NumRows == 1)
     {
      /***** Get tag code *****/
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%ld",&TagCod) != 1)
        {
         strcpy (Gbl.Message,"Wrong code of tag.");
         Error = true;
        }
     }
   else if (NumRows > 1)
     {
      strcpy (Gbl.Message,"Wrong code of tag.");
      Error = true;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   if (Error)
      Lay_ShowErrorAndExit (Gbl.Message);

   return TagCod;
  }

/*****************************************************************************/
/********************* Insert new tag into tst_tags table ********************/
/*****************************************************************************/

static long Tst_CreateNewTag (long CrsCod,const char *TagTxt)
  {
   char Query[256+Tst_MAX_BYTES_TAG];

   /***** Insert new tag into tst_tags table *****/
   sprintf (Query,"INSERT INTO tst_tags (CrsCod,TagTxt,TagHidden)" \
                  " VALUES ('%ld','%s','N')",
            CrsCod,TagTxt);
   return DB_QueryINSERTandReturnCode (Query,"can not create new tag");
  }

/*****************************************************************************/
/********** Change visibility of an existing tag into tst_tags table *********/
/*****************************************************************************/

static void Tst_EnableOrDisableTag (long TagCod,bool TagHidden)
  {
   char Query[512];

   /***** Insert new tag into tst_tags table *****/
   sprintf (Query,"UPDATE tst_tags SET TagHidden='%c' WHERE TagCod='%ld' AND CrsCod='%ld'",
            TagHidden ? 'Y' : 'N',TagCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryUPDATE (Query,"can not update the visibility of a tag");
  }

/*****************************************************************************/
/******************************* Remove a question ***************************/
/*****************************************************************************/

void Tst_RemoveQst (void)
  {
   char Query[512];
   char YN[1+1];
   bool EditingOnlyThisQst;

   /***** Get the question code *****/
   if (!Tst_GetQstCod ())
      Lay_ShowErrorAndExit ("Wrong code of question.");

   /***** Get a parameter that indicates whether it's necessary to continue listing the rest of questions ******/
   Par_GetParam (Par_PARAM_SINGLE,"OnlyThisQst",YN,1);
   EditingOnlyThisQst = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /***** Remove the question from all the tables *****/
   /* Remove answers y tags from this test question */
   Tst_RemAnsFromQst (Gbl.Test.QstCod);
   Tst_RemTagsFromQst (Gbl.Test.QstCod);
   Tst_RemoveUnusedTagsFromCurrentCrs ();

   /* Remove the question itself */
   sprintf (Query,"DELETE FROM tst_questions" \
                  " WHERE QstCod='%ld' AND CrsCod='%ld'",
            Gbl.Test.QstCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryDELETE (Query,"can not remove a question");

   if (!mysql_affected_rows (&Gbl.mysql))
      Lay_ShowErrorAndExit ("The question to be removed don't exists or belongs to another course.");

   /***** Write message *****/
   sprintf (Gbl.Message,"Pregunta con c&oacute;digo <strong>%ld</strong> eliminada.",	// Need translation!
            Gbl.Test.QstCod);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Continue editing questions *****/
   if (!EditingOnlyThisQst)
      Tst_ListQuestionsToEdit ();
  }

/*****************************************************************************/
/*********************** Change the shuffle of a question ********************/
/*****************************************************************************/

void Tst_ChangeShuffleQst (void)
  {
   char Query[512];
   char YN[1+1];
   bool EditingOnlyThisQst;
   bool Shuffle;

   /***** Get the question code *****/
   if (!Tst_GetQstCod ())
      Lay_ShowErrorAndExit ("Wrong code of question.");

   /***** Get a parameter that indicates whether it's necessary to continue listing the rest of questions ******/
   Par_GetParam (Par_PARAM_SINGLE,"OnlyThisQst",YN,1);
   EditingOnlyThisQst = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /***** Get a parameter that indicates whether it's possible to shuffle the answers of this question ******/
   Par_GetParam (Par_PARAM_SINGLE,"Shuffle",YN,1);
   Shuffle = (Str_ConvertToUpperLetter (YN[0]) == 'Y');

   /***** Remove the question from all the tables *****/
   /* Update the question changing the current shuffle */
   sprintf (Query,"UPDATE tst_questions SET Shuffle='%c'" \
                  " WHERE QstCod='%ld' AND CrsCod='%ld'",
            Shuffle ? 'Y' : 'N',Gbl.Test.QstCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryUPDATE (Query,"can not update the shuffle type of a question");

   /***** Write message *****/
   sprintf (Gbl.Message,"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong> aparecer&aacute;n %s.",	// Need translation!
            Gbl.Test.QstCod,Shuffle ? "barajadas" : "sin barajar");	// Need translation!
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Continue editing questions *****/
   if (EditingOnlyThisQst)
      Tst_ListOneQstToEdit ();
   else
      Tst_ListQuestionsToEdit ();
  }

/*****************************************************************************/
/******** Get the parameter with the code of una test question *********/
/*****************************************************************************/
// Devuelve 1 si puede obtener the parameter; 0 si no lo encuentra

static int Tst_GetQstCod (void)
  {
   char IntStr[10+1];

   Par_GetParam (Par_PARAM_SINGLE,"QstCod",IntStr,1+10);
   if (sscanf (IntStr,"%ld",&Gbl.Test.QstCod) != 1)
      return 0;
   return 1;
  }

/*****************************************************************************/
/********************** Remove answers from a test question ******************/
/*****************************************************************************/

static void Tst_RemAnsFromQst (long QstCod)
  {
   char Query[512];

   /***** Remove answers *****/
   sprintf (Query,"DELETE FROM tst_answers WHERE QstCod='%ld'",QstCod);
   DB_QueryDELETE (Query,"can not remove the answers of a question");
  }

/*****************************************************************************/
/************************** Remove tags from a test question *****************/
/*****************************************************************************/

static void Tst_RemTagsFromQst (long QstCod)
  {
   char Query[512];

   /***** Remove tags *****/
   sprintf (Query,"DELETE FROM tst_question_tags WHERE QstCod='%ld'",QstCod);
   DB_QueryDELETE (Query,"can not remove the tags of a question");
  }

/*****************************************************************************/
/******************** Remove unused tags in current course *******************/
/*****************************************************************************/

static void Tst_RemoveUnusedTagsFromCurrentCrs (void)
  {
   char Query[1024];

   /* Remove unused tags from tst_tags  */
   sprintf (Query,"DELETE FROM tst_tags WHERE CrsCod='%ld' AND TagCod NOT IN" \
                  " (SELECT DISTINCT tst_question_tags.TagCod FROM tst_questions,tst_question_tags" \
                  " WHERE tst_questions.CrsCod='%ld' AND tst_questions.QstCod=tst_question_tags.QstCod)",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.CurrentCrs.Crs.CrsCod);
   DB_QueryDELETE (Query,"can not remove unused tags");
  }

/****************************************************************************/
/******************* Allocate memory for a choice answer ********************/
/****************************************************************************/

static int Tst_AllocateTextChoiceAnswer (unsigned NumAns)
  {
   Tst_FreeTextChoiceAnswer (NumAns);
   if ((Gbl.Test.AnsChoice[NumAns].Text = malloc (Txt_MAX_BYTES_ANSWER+1)) == NULL)
     {
      sprintf (Gbl.Message,"Not enough memory to store answer.");
      return 0;
     }
   Gbl.Test.AnsChoice[NumAns].Text[0] = '\0';
   return 1;
  }

/*****************************************************************************/
/******************** Free memory of all choice answers **********************/
/*****************************************************************************/

static void Tst_FreeTextChoiceAnswers (unsigned NumAnswers)
  {
   unsigned NumAns;

   for (NumAns=0; NumAns<NumAnswers; NumAns++)
      Tst_FreeTextChoiceAnswer (NumAns);
  }

/*****************************************************************************/
/********************** Free memory of a choice answer ***********************/
/*****************************************************************************/

static void Tst_FreeTextChoiceAnswer (unsigned NumAns)
  {
   if (Gbl.Test.AnsChoice[NumAns].Text)
     {
      free ((void *) Gbl.Test.AnsChoice[NumAns].Text);
      Gbl.Test.AnsChoice[NumAns].Text = NULL;
     }
  }
