// swad_tt.c: timetables

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <stdio.h>		// For fprintf, etc.
#include <string.h>		// For string functions

#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_par.h"
#include "swad_tt.h"
#include "swad_txt.h"

/****************************************************************************/
/*************************** External constants *****************************/
/****************************************************************************/

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;

/****************************************************************************/
/*************************** Internal constants *****************************/
/****************************************************************************/

#define TT_START_HOUR				  7	// Day starts at 7:00
#define TT_HOURS_PER_DAY			 15	// From 7:00 to 22:00
#define TT_DAYS					  5	// From monday to friday
#define TT_MAX_COLUMNS_PER_CELL			  3	// Maximum number of items (i.e. classes) in a timetable cell (1, 2, 3 or 4)
#define TT_NUM_MINICOLUMNS_PER_DAY		  6	// Least common multiple of 1,2,3,...,TT_MAX_COLUMNS_PER_CELL
#define TT_PERCENT_WIDTH_OF_A_MINICOLUMN	  3	// Width (%) of each minicolumn
#define TT_PERCENT_WIDTH_OF_A_DAY		(TT_PERCENT_WIDTH_OF_A_MINICOLUMN*TT_NUM_MINICOLUMNS_PER_DAY)	// Width (%) of each day
#define TT_PERCENT_WIDTH_OF_ALL_DAYS		(TT_PERCENT_WIDTH_OF_A_DAY*TT_DAYS)				// Width (%) of all days
#define TT_PERCENT_WIDTH_OF_A_SEPARATION_COLUMN	  1	// Width (%) of left and right columns (frame)
#define TT_PERCENT_WIDTH_OF_AN_HOUR_COLUMN 	 ((100-TT_PERCENT_WIDTH_OF_ALL_DAYS-TT_PERCENT_WIDTH_OF_A_SEPARATION_COLUMN*(1+TT_DAYS))/2)	// Width (%) of the four separation columns
#define TT_NUM_CLASS_TYPES			  4	// Libre, Teora, Prcticas, Tutoras
#define TT_MAX_BYTES_STR_CLASS_TYPE		256
#define TT_MAX_BYTES_STR_DURATION		((2+1+2+1+1)*Str_LENGTH_STR_HEX)	// "hh:mm h"
#define TT_MAX_LENGTH_PLACE			 32

/****************************************************************************/
/******************************* Internal types *****************************/
/****************************************************************************/

/****************************************************************************/
/************************* Internal global variables ************************/
/****************************************************************************/

char TimeTableCharsDays[TT_DAYS] = {'L','M','X','J','V'};
char *TimeTableStrsClassTypeScreen[TT_NUM_CLASS_TYPES] = {"Libre","Teora","Prct.","Tutoras"}; // No utilizar &iacute;, etc.
char *TimeTableStrsClassTypeDB[TT_NUM_CLASS_TYPES] = {"libre","teoria","practicas","tutorias"};
struct TimeTableColumn
  {
   long CrsCod;		// Course code (-1 if no course selected)
   long GrpCod;		// Group code (-1 if no group selected)
   TT_HourType_t HourType;
   TT_ClassType_t ClassType;
   unsigned Duration;
   char Place[TT_MAX_BYTES_PLACE+1];
   char Group[TT_MAX_BYTES_GROUP+1];
  };
struct
  {
   unsigned NumColumns;
   struct TimeTableColumn Columns[TT_MAX_COLUMNS_PER_CELL];
  } TimeTable[TT_DAYS][TT_HOURS_PER_DAY*2];
bool TimeTableHoursChecked[TT_HOURS_PER_DAY*2];

/****************************************************************************/
/***************************** Internal prototypes **************************/
/****************************************************************************/


/****************************************************************************/
/*********** Show whether only my groups or all groups are shown ************/
/****************************************************************************/

void TT_ShowTimeTableGrpsSelected (void)
  {
   extern const char *Txt_All_groups[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_ORL\">");
   switch (Gbl.CurrentCrs.Grps.WhichGrps)
     {
      case Grp_ONLY_MY_GROUPS:
        fprintf (Gbl.F.Out,"Grupos de %s",	// Need translation!
                 Gbl.Usrs.Me.UsrDat.FullName);
        break;
      case Grp_ALL_GROUPS:
        fprintf (Gbl.F.Out,Txt_All_groups[Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</td></tr>");
  }

/****************************************************************************/
/******************** Get paramaters for timetable editing ******************/
/****************************************************************************/

void TT_GetParamsTimeTable (void)
  {
   char UnsignedStr[10+1];
   char LongStr[1+10+1];
   char CadenaTipoClase[TT_MAX_BYTES_STR_CLASS_TYPE+1];
   char CadenaDuracionClase[TT_MAX_BYTES_STR_DURATION+1];
   unsigned Hours,Minutes;

   /***** Get day (0: monday, 1: tuesday,..., 4: friday *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModTTDay",UnsignedStr,2);
   if (sscanf (UnsignedStr,"%u",&Gbl.TimeTable.Day) != 1)
      Lay_ShowErrorAndExit ("Day is missing.");

   /***** Get hour *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModTTHour",UnsignedStr,2);
   if (sscanf (UnsignedStr,"%u",&Gbl.TimeTable.Hour) != 1)
      Lay_ShowErrorAndExit ("Hour is missing.");

   /***** Get number of column *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModTTCol",UnsignedStr,2);
   if (sscanf (UnsignedStr,"%u",&Gbl.TimeTable.Column) != 1)
      Lay_ShowErrorAndExit ("Column is missing.");

   /***** Get class type *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModTTCellType",CadenaTipoClase,TT_MAX_BYTES_STR_CLASS_TYPE);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,CadenaTipoClase,TT_MAX_BYTES_STR_CLASS_TYPE,true);
   for (Gbl.TimeTable.ClassType=(TT_ClassType_t) 0; Gbl.TimeTable.ClassType<(TT_ClassType_t) TT_NUM_CLASS_TYPES; Gbl.TimeTable.ClassType++)
      if (!strcmp (CadenaTipoClase,TimeTableStrsClassTypeScreen[Gbl.TimeTable.ClassType]))
         break;
   if (Gbl.TimeTable.ClassType == (TT_ClassType_t) TT_NUM_CLASS_TYPES)
      Lay_ShowErrorAndExit ("Type of timetable cell is missing.");

   /***** Get class duration *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModTTDur",CadenaDuracionClase,TT_MAX_BYTES_STR_DURATION);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,CadenaDuracionClase,TT_MAX_BYTES_STR_DURATION,true);
   if (sscanf (CadenaDuracionClase,"%u:%u",&Hours,&Minutes) != 2)
      Lay_ShowErrorAndExit ("Duration is missing.");
   Gbl.TimeTable.Duration = Hours*2+Minutes/30;

   /***** Get group *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModHorGrp",Gbl.TimeTable.Group,TT_MAX_BYTES_GROUP);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.TimeTable.Group,TT_MAX_BYTES_GROUP,true);

   /***** Get group code *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModTTGrpCod",LongStr,1+10);
   if (LongStr[0])
     {
      if (sscanf (LongStr,"%ld",&Gbl.TimeTable.GrpCod) != 1)
         Lay_ShowErrorAndExit ("Wrong code of group.");
     }
   else
      Gbl.TimeTable.GrpCod = -1;

   /***** Get place *****/
   Par_GetParam (Par_PARAM_SINGLE,"ModHorLugar",Gbl.TimeTable.Place,TT_MAX_BYTES_PLACE);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.TimeTable.Place,TT_MAX_BYTES_PLACE,true);
  }

/****************************************************************************/
/*************************** Show class timetable ***************************/
/****************************************************************************/

void TT_ShowClassTimeTable (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Edit[Txt_NUM_LANGUAGES];
   extern const char *Txt_Show_print_view[Txt_NUM_LANGUAGES];
   TT_TimeTableType_t TimeTableType = TT_COURSE_TIMETABLE;

   /***** Get whether to show only my groups or all groups *****/
   Grp_GetParamWhichGrps ();

   /***** Start of frame and title *****/
   switch (Gbl.CurrentAct)
     {
      case ActSeeCrsTimTbl:
      case ActPrnCrsTimTbl:
         TimeTableType = TT_COURSE_TIMETABLE;
	 break;
      case ActSeeMyTimTbl:
      case ActPrnMyTimTbl:
         TimeTableType = TT_MY_TIMETABLE;
     }

   /***** Put a message for editing class timetable *****/
   if ((Gbl.CurrentAct == ActSeeCrsTimTbl) && (Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER))
     {
      fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"center\">");
      Act_FormStart (ActEdiCrsTimTbl);
      Grp_PutParamWhichGrps ();
      strcpy (Gbl.MsgWindowStatus,Txt_Edit[Gbl.Prefs.Language]);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
      fprintf (Gbl.F.Out,"<img src=\"%s/pen12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" /> %s</acronym></a></form></td></tr></table></div>",
               Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
     }
   switch (Gbl.CurrentAct)
     {
      case ActSeeCrsTimTbl:
      case ActSeeMyTimTbl:
	 /***** Link to print view *****/
	 fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\">" \
	                    "<tr><td align=\"center\" valign=\"middle\">");
	 Act_FormStart ((Gbl.CurrentAct == ActSeeCrsTimTbl) ? ActPrnCrsTimTbl : ActPrnMyTimTbl);
         Grp_PutParamWhichGrps ();
         strcpy (Gbl.MsgWindowStatus,Txt_Show_print_view[Gbl.Prefs.Language]);
         Act_LinkFormSubmit (Act_NEW_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"<img src=\"%s/print12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                            " %s</acronym></a></form></td></tr></table></div>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);

         /***** Start time table drawing *****/
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
         if (TimeTableType == TT_COURSE_TIMETABLE)
            Lay_WriteHeaderClassPhoto (1,false,false,
                                     Gbl.CurrentDeg.Deg.InsCod,Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod);

         /***** Select whether show only my groups or all groups *****/
         fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
         Act_FormStart (Gbl.CurrentAct);
         Grp_ShowSelectorWhichGrps ();
         fprintf (Gbl.F.Out,"</form></td></tr>");
	 break;
      case ActPrnCrsTimTbl:
      case ActPrnMyTimTbl:
         /***** Start time table drawing *****/
	 Lay_StartTableFrame (DARK_BLUE,WHITE,"90%");
         if (TimeTableType == TT_COURSE_TIMETABLE)
            Lay_WriteHeaderClassPhoto (1,true,false,
                                     Gbl.CurrentDeg.Deg.InsCod,Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod);

         /***** Show whether only my groups or all groups are selected *****/
         TT_ShowTimeTableGrpsSelected ();
     }

   /***** Show the time table *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">\n");
   TT_ShowTimeTable (TimeTableType,Gbl.Usrs.Me.UsrDat.UsrCod);
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** End of frame *****/
   if (TimeTableType == TT_COURSE_TIMETABLE)
      Lay_WriteFootClassPhoto ();
   switch (Gbl.CurrentAct)
     {
      case ActSeeCrsTimTbl:
      case ActSeeMyTimTbl:
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
	 break;
      case ActPrnCrsTimTbl:
      case ActPrnMyTimTbl:
         Lay_EndTableFrame ();
     }
  }

/****************************************************************************/
/********************** Show course timetable for edition *******************/
/****************************************************************************/

void TT_EditCrsTimeTable (void)
  {
   /***** Start of table *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

   /***** Editable time table *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   TT_ShowTimeTable (TT_COURSE_TIMETABLE,Gbl.Usrs.Me.UsrDat.UsrCod);
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** End of table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/******* Show tutor timetable for all the teachers in current course ********/
/****************************************************************************/

void TT_ShowTutTimeTable (void)
  {
   unsigned NumUsr;
   struct UsrData *UsrDat;

   /***** Get the list of teachers from this course *****/
   Usr_GetAndOrderTchsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);
   if (!Gbl.Usrs.LstTchs.NumUsrs)
     {
      Lay_ShowAlert (WARNING,"No se han encontrado profesores.");	// Need translation!
      return;
     }

   /***** Table start *****/
   fprintf (Gbl.F.Out,"<div align=\"center\"><table width=\"100%%\" cellspacing=\"1\" cellpadding=\"0\">\n");

   /***** List the records and the tutor timetables of the teachers *****/
   Gbl.Usrs.Listing.Type = Usr_USER_LIST_RECORDS_COURSE_TEACHERS;
   for (NumUsr=0; NumUsr<Gbl.Usrs.LstTchs.NumUsrs; NumUsr++)
     {
      UsrDat = &Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.Ind[NumUsr]];

      fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
      Rec_ShowCommonRecord (RECORD_LIST,UsrDat,NULL);
      fprintf (Gbl.F.Out,"</td></tr><tr><td align=\"center\">");
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
      fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
      TT_ShowTimeTable (TT_TUTOR_TIMETABLE,UsrDat->UsrCod);
      fprintf (Gbl.F.Out,"</td></tr>");
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
      fprintf (Gbl.F.Out,"</td></tr><tr><td align=\"left\" class=\"DAT_SMALL\">&nbsp;</td></tr>\n");
     }

   /***** End of table *****/
   fprintf (Gbl.F.Out,"</table></div>\n");

   /***** Free memory for teachers list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
  }

/****************************************************************************/
/********************* Show tutor timetable for edition *********************/
/****************************************************************************/

void TT_ShowMyTutTimeTable (void)
  {
   Lay_ShowAlert (INFO,"Su horario de tutor&iacute;as aparece en esta pesta&ntilde;a junto a sus datos personales" \
                      " debido a que est&aacute; asociado a usted como profesor, y no a una asignatura concreta.");	// Need translation!
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   TT_ShowTimeTable (TT_TUTOR_TIMETABLE,Gbl.Usrs.Me.UsrDat.UsrCod);
   fprintf (Gbl.F.Out,"</td></tr>");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/*********** Show course timetable or tutor timetable of a teacher **********/
/****************************************************************************/

void TT_ShowTimeTable (TT_TimeTableType_t TimeTableType,long UsrCod)
  {
   /***** Create an internal table with the timetable from database *****/
   TT_CreatTimeTableFromDB (TimeTableType,UsrCod);

   /***** If timetable must be modified... *****/
   if (Gbl.CurrentAct == ActChgCrsTimTbl || Gbl.CurrentAct == ActChgTut)
     {
      /* Get parameters for time table editing */
      TT_GetParamsTimeTable ();

      /* Modify timetable in memory */
      TT_ModifTimeTable ();

      /* Write a new timetable in database */
      switch (TimeTableType)
        {
         case TT_COURSE_TIMETABLE:
            TT_WriteCrsTimeTableIntoDB (Gbl.CurrentCrs.Crs.CrsCod);
	    break;
         case TT_TUTOR_TIMETABLE:
            TT_WriteTutTimeTableIntoDB (UsrCod);
            break;
         default:
            break;
        }

      /* Get a new table from database */
      TT_CreatTimeTableFromDB (TimeTableType,UsrCod);
     }

   /***** Draw timetable *****/
   TT_DrawTimeTable ();
  }

/****************************************************************************/
/******************* Write course timetable into database *******************/
/****************************************************************************/

void TT_WriteCrsTimeTableIntoDB (long CrsCod)
  {
   char Query[1024];
   unsigned Hour,Day,Column;

   /***** Remove former timetable *****/
   sprintf (Query,"DELETE FROM timetable_crs WHERE CrsCod='%ld'",CrsCod);
   DB_QueryDELETE (Query,"can not remove former timetable");

   /***** Go across the timetable inserting classes into database *****/
   for (Day=0; Day<TT_DAYS; Day++)
      for (Hour=0; Hour<TT_HOURS_PER_DAY*2; Hour++)
         for (Column=0; Column<TT_MAX_COLUMNS_PER_CELL; Column++)
	    if (TimeTable[Day][Hour].Columns[Column].HourType == TT_FIRST_HOUR &&
                TimeTable[Day][Hour].Columns[Column].Duration > 0)
              {
               sprintf (Query,"INSERT INTO timetable_crs (CrsCod,GrpCod,Day,Hour,Duration,ClassType,Place,GroupName)" \
                              " VALUES ('%ld','%ld','%c','%u','%d','%s','%s','%s')",
                        CrsCod,
			TimeTable[Day][Hour].Columns[Column].GrpCod,
			TimeTableCharsDays[Day],
			Hour,
                        TimeTable[Day][Hour].Columns[Column].Duration,
                        TimeTableStrsClassTypeDB[TimeTable[Day][Hour].Columns[Column].ClassType],
                        TimeTable[Day][Hour].Columns[Column].Place,
		        TimeTable[Day][Hour].Columns[Column].Group);
               DB_QueryINSERT (Query,"can not create course timetable");
              }
  }

/****************************************************************************/
/********************* Write tutor timetable into database ******************/
/****************************************************************************/

void TT_WriteTutTimeTableIntoDB (long UsrCod)
  {
   char Query[1024];
   unsigned Hour,Day,Column;

   /***** Remove former timetable *****/
   sprintf (Query,"DELETE FROM timetable_tut WHERE UsrCod='%ld'",UsrCod);
   DB_QueryDELETE (Query,"can not remove former timetable");

   /***** Loop over timetable *****/
   for (Day=0; Day<TT_DAYS; Day++)
      for (Hour=0; Hour<TT_HOURS_PER_DAY*2; Hour++)
         for (Column=0; Column<TT_MAX_COLUMNS_PER_CELL; Column++)
	    if (TimeTable[Day][Hour].Columns[Column].HourType == TT_FIRST_HOUR &&
                TimeTable[Day][Hour].Columns[Column].Duration > 0)
              {
               sprintf (Query,"INSERT INTO timetable_tut (UsrCod,Day,Hour,Duration,Place)" \
                              " VALUES ('%ld','%c','%u','%d','%s')",
                        UsrCod,TimeTableCharsDays[Day],Hour,
                        TimeTable[Day][Hour].Columns[Column].Duration,
			TimeTable[Day][Hour].Columns[Column].Place);
               DB_QueryINSERT (Query,"can not create office timetable");
              }
  }

/****************************************************************************/
/********** Create an internal table with timetable from database ***********/
/****************************************************************************/

void TT_CreatTimeTableFromDB (TT_TimeTableType_t TimeTableType,long UsrCod)
  {
   char Query[4096];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   unsigned Hour,Day,Column,FirstFreeColumn,Duration,H;
   long GrpCod;
   TT_ClassType_t ClassType = TT_NO_CLASS;	// Initialized to avoid warning
   bool TimeTableIsIncomplete = false,TimeTableHasSpaceForThisClass;

   /***** Initialize timetable to all free *****/
   for (Day=0; Day<TT_DAYS; Day++)
      for (Hour=0; Hour<TT_HOURS_PER_DAY*2; Hour++)
        {
         TimeTable[Day][Hour].NumColumns = 0;
         for (Column=0; Column<TT_MAX_COLUMNS_PER_CELL; Column++)
	   {
	    TimeTable[Day][Hour].Columns[Column].CrsCod = -1L;
	    TimeTable[Day][Hour].Columns[Column].GrpCod = -1L;
	    TimeTable[Day][Hour].Columns[Column].HourType = TT_FREE_HOUR;
	    TimeTable[Day][Hour].Columns[Column].ClassType = TT_NO_CLASS;
	    TimeTable[Day][Hour].Columns[Column].Duration = 0;
	    TimeTable[Day][Hour].Columns[Column].Group[0] = '\0';
	    TimeTable[Day][Hour].Columns[Column].Place[0] = '\0';
	   }
        }

   /***** Get timetable from database *****/
   switch (TimeTableType)
     {
      case TT_MY_TIMETABLE:
         switch (Gbl.CurrentCrs.Grps.WhichGrps)
           {
            case Grp_ONLY_MY_GROUPS:
               sprintf (Query,"(SELECT timetable_crs.Day,timetable_crs.Hour,timetable_crs.Duration,timetable_crs.Place," \
                              "timetable_crs.ClassType,timetable_crs.GroupName,timetable_crs.GrpCod,timetable_crs.CrsCod" \
                              " FROM timetable_crs,crs_usr" \
                              " WHERE crs_usr.UsrCod='%ld' AND timetable_crs.GrpCod='-1' AND timetable_crs.CrsCod=crs_usr.CrsCod)" \
                              " UNION DISTINCT " \
                              "(SELECT timetable_crs.Day,timetable_crs.Hour,timetable_crs.Duration,timetable_crs.Place," \
                              "timetable_crs.ClassType,timetable_crs.GroupName,timetable_crs.GrpCod,timetable_crs.CrsCod" \
                              " FROM timetable_crs,crs_grp_usr" \
                              " WHERE crs_grp_usr.UsrCod='%ld' AND timetable_crs.GrpCod=crs_grp_usr.GrpCod)" \
                              " ORDER BY Day,Hour,ClassType,GroupName,GrpCod,Place,Duration DESC,CrsCod",UsrCod,UsrCod);
               break;
            case Grp_ALL_GROUPS:
               sprintf (Query,"SELECT timetable_crs.Day,timetable_crs.Hour,timetable_crs.Duration,timetable_crs.Place," \
                              "timetable_crs.ClassType,timetable_crs.GroupName,timetable_crs.GrpCod,timetable_crs.CrsCod" \
                              " FROM timetable_crs,crs_usr WHERE crs_usr.UsrCod='%ld' AND timetable_crs.CrsCod=crs_usr.CrsCod" \
                              " ORDER BY timetable_crs.Day,timetable_crs.Hour,timetable_crs.ClassType," \
                              "timetable_crs.GroupName,timetable_crs.GrpCod,timetable_crs.Place,timetable_crs.Duration DESC,timetable_crs.CrsCod",UsrCod);
           }
	 break;
      case TT_COURSE_TIMETABLE:
         if (Gbl.CurrentCrs.Grps.WhichGrps == Grp_ALL_GROUPS ||
             Gbl.CurrentAct == ActEdiCrsTimTbl || Gbl.CurrentAct == ActChgCrsTimTbl)	// Si estamos editando, se muestran siempre todos los grupos
            sprintf (Query,"SELECT Day,Hour,Duration,Place,ClassType,GroupName,GrpCod FROM timetable_crs" \
                           " WHERE CrsCod='%ld' ORDER BY Day,Hour,ClassType,GroupName,GrpCod,Place,Duration DESC",
                     Gbl.CurrentCrs.Crs.CrsCod);
         else
            sprintf (Query,"(SELECT timetable_crs.Day,timetable_crs.Hour,timetable_crs.Duration,timetable_crs.Place,timetable_crs.ClassType,timetable_crs.GroupName,timetable_crs.GrpCod" \
                           " FROM timetable_crs,crs_usr" \
                           " WHERE timetable_crs.CrsCod='%ld' AND timetable_crs.GrpCod='-1' AND crs_usr.UsrCod='%ld' AND timetable_crs.CrsCod=crs_usr.CrsCod)" \
                           " UNION DISTINCT " \
                           "(SELECT timetable_crs.Day,timetable_crs.Hour,timetable_crs.Duration,timetable_crs.Place," \
                           "timetable_crs.ClassType,timetable_crs.GroupName,timetable_crs.GrpCod" \
			   " FROM timetable_crs,crs_grp_usr" \
                           " WHERE timetable_crs.CrsCod='%ld' AND crs_grp_usr.UsrCod='%ld' AND timetable_crs.GrpCod=crs_grp_usr.GrpCod)" \
                           " ORDER BY Day,Hour,ClassType,GroupName,GrpCod,Place,Duration DESC",
                     Gbl.CurrentCrs.Crs.CrsCod,UsrCod,Gbl.CurrentCrs.Crs.CrsCod,UsrCod);
	 break;
      case TT_TUTOR_TIMETABLE:
         sprintf (Query,"SELECT Day,Hour,Duration,Place FROM timetable_tut" \
                        " WHERE UsrCod='%ld' ORDER BY Day,Hour,Place,Duration DESC",
                  UsrCod);
     }
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get timetable");

   /***** Build the table depending on the number of rows of the timetable *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      if (TimeTableType == TT_MY_TIMETABLE ||
          TimeTableType == TT_COURSE_TIMETABLE)
         /* Group code */
         if (sscanf (row[6],"%ld",&GrpCod) != 1)
            GrpCod = -1;

      /* Day of week (row[0]) */
      for (Day=0; Day<TT_DAYS; Day++)
	 if (Str_ConvertToUpperLetter (row[0][0]) == TimeTableCharsDays[Day])
	    break;
      if (Day == TT_DAYS)
	 Lay_ShowErrorAndExit ("Wrong day of week in timetable.");

      /* Hour (row[1]) */
      if (sscanf (row[1],"%u",&Hour) != 1)
	 Lay_ShowErrorAndExit ("Wrong hour in timetable.");

      /* Duracin (row[2]) */
      if (sscanf (row[2],"%u",&Duration) != 1)
	 Lay_ShowErrorAndExit ("Wrong duration in timetable.");

      /* Type of class */
      switch (TimeTableType)
        {
         case TT_COURSE_TIMETABLE:
         case TT_MY_TIMETABLE:
            for (ClassType=TT_THEORY_CLASS; ClassType<=TT_PRACT_CLASS; ClassType++)
  	       if (!strcmp (row[4],TimeTableStrsClassTypeDB[ClassType]))
	          break;
            if (ClassType > TT_PRACT_CLASS)
	       Lay_ShowErrorAndExit ("Wrong type of class in timetable.");
            break;
         case TT_TUTOR_TIMETABLE:
            ClassType = TT_TUTOR_CLASS;
        }

      /* Cell has been read without errors. */
      if (TimeTable[Day][Hour].NumColumns < TT_MAX_COLUMNS_PER_CELL)	// If there's place for another column in this cell...
        {
         /* Find the first free column for this day-hour */
         FirstFreeColumn = TT_MAX_COLUMNS_PER_CELL;
         for (Column=0; Column<TT_MAX_COLUMNS_PER_CELL; Column++)
            if (TimeTable[Day][Hour].Columns[Column].HourType == TT_FREE_HOUR)
              {
               FirstFreeColumn = Column;
               break;
              }
         if (FirstFreeColumn < TT_MAX_COLUMNS_PER_CELL)	// If there's place for another column in this cell
           {
            /* Check if there's place for all the rows of this class */
            TimeTableHasSpaceForThisClass = true;
            for (H=Hour+1; H<Hour+Duration && H<TT_HOURS_PER_DAY*2; H++)
               if (TimeTable[Day][H].Columns[FirstFreeColumn].HourType != TT_FREE_HOUR)
                {
                  TimeTableIsIncomplete = true;
                  TimeTableHasSpaceForThisClass = false;
                  break;
                 }
            if (TimeTableHasSpaceForThisClass)
              {
               TimeTable[Day][Hour].Columns[FirstFreeColumn].ClassType = ClassType;
               TimeTable[Day][Hour].Columns[FirstFreeColumn].Duration  = Duration;
               TimeTable[Day][Hour].Columns[FirstFreeColumn].HourType  = TT_FIRST_HOUR;
               for (H=Hour+1; H<Hour+Duration && H<TT_HOURS_PER_DAY*2; H++)
                 {
	          TimeTable[Day][H].Columns[FirstFreeColumn].HourType = TT_NEXT_HOUR;
	          TimeTable[Day][H].NumColumns++;
                 }

               /* Group and place */
               switch (TimeTableType)
                 {
                  case TT_MY_TIMETABLE:
                  case TT_COURSE_TIMETABLE:
                     TimeTable[Day][Hour].Columns[FirstFreeColumn].CrsCod =
                        (TimeTableType == TT_MY_TIMETABLE) ? Str_ConvertStrCodToLongCod (row[7]) :
                                                          Gbl.CurrentCrs.Crs.CrsCod;
                     strcpy (TimeTable[Day][Hour].Columns[FirstFreeColumn].Group,row[5]);
                     TimeTable[Day][Hour].Columns[FirstFreeColumn].GrpCod = GrpCod;
                  case TT_TUTOR_TIMETABLE:
                     strcpy (TimeTable[Day][Hour].Columns[FirstFreeColumn].Place,row[3]);
                 }

               /* Increment number of items in this cell */
               TimeTable[Day][Hour].NumColumns++;
              }
           }
         else
            TimeTableIsIncomplete = true;
        }
      else
         TimeTableIsIncomplete = true;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   if (TimeTableIsIncomplete)
      Lay_WriteTitle ("Horario incompleto por falta de espacio");	// Need trnaslation!!!!!!!!
  }

/****************************************************************************/
/*************** Modificar una clase in the tabla del horario ****************/
/****************************************************************************/

void TT_ModifTimeTable (void)
  {
   if (TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].HourType == TT_FIRST_HOUR)
     {
      /***** Free this cell *****/
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].GrpCod    = -1;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].HourType  = TT_FREE_HOUR;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].ClassType = TT_NO_CLASS;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].Duration  = 0;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].Group[0]  = '\0';
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].Place[0]  = '\0';
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].NumColumns--;
     }

   if (Gbl.TimeTable.ClassType != TT_NO_CLASS && Gbl.TimeTable.Duration > 0 &&
       TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].NumColumns < TT_MAX_COLUMNS_PER_CELL)
     {
      /***** Change this cell *****/
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].NumColumns++;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].GrpCod    = Gbl.TimeTable.GrpCod;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].HourType  = TT_FIRST_HOUR;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].ClassType = Gbl.TimeTable.ClassType;
      TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].Duration  = Gbl.TimeTable.Duration;
      strcpy (TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].Group,Gbl.TimeTable.Group);
      strcpy (TimeTable[Gbl.TimeTable.Day][Gbl.TimeTable.Hour].Columns[Gbl.TimeTable.Column].Place,Gbl.TimeTable.Place);
     }
  }

/****************************************************************************/
/********************* Draw timetable using internal table ******************/
/****************************************************************************/

void TT_DrawTimeTable (void)
  {
   bool Editando = false;
   unsigned Day,Hour,H,Column,ColumnsToDraw,ColumnsToDrawIncludingExtraColumn,ContinuousFreeMinicolumns;

   switch (Gbl.CurrentAct)
     {
      case ActSeeCrsTimTbl:	case ActPrnCrsTimTbl:
      case ActSeeMyTimTbl:	case ActPrnMyTimTbl:
      case ActSeeTut:
         Editando = false;
         break;
      case ActEdiCrsTimTbl:	case ActChgCrsTimTbl:
      case ActEdiTut:		case ActChgTut:
	 // If editing and there's place for more columns, a potential new column is added at the end of each day
         Editando = true;
     }

   /***** Table start *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\">\n");

   /***** Top row used for column adjustement *****/
   TT_TimeTableDrawAdjustRow (true);

   /***** Row with day names *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"middle\" rowspan=\"2\" width=\"%u%%\" class=\"HOR\">%u:00</td>",
            TT_PERCENT_WIDTH_OF_AN_HOUR_COLUMN,TT_START_HOUR);
   TT_DrawCellAlignTimeTable ();
   TT_TimeTableDrawDaysCells ();
   TT_DrawCellAlignTimeTable ();
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" rowspan=\"2\" width=\"%u%%\" class=\"HOR\">%u:00</td></tr>",
            TT_PERCENT_WIDTH_OF_AN_HOUR_COLUMN,TT_START_HOUR);

   /***** Get list of groups types and groups in this course *****/
   if (Gbl.CurrentAct == ActEdiCrsTimTbl || Gbl.CurrentAct == ActChgCrsTimTbl)
      Grp_GetListGrpTypesAndGrpsInThisCrs ();

   /***** Write the table fila por fila *****/
   for (Hour=0; Hour<TT_HOURS_PER_DAY*2; Hour++)
     {
      fprintf (Gbl.F.Out,"<tr>");

      /* Hour */
      if (Hour % 2)
         fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"middle\" rowspan=\"2\" width=\"%u%%\" class=\"HOR\">%2u:00</td>",
                  TT_PERCENT_WIDTH_OF_AN_HOUR_COLUMN,TT_START_HOUR + (Hour+1)/2);

      /* Empty column used to adjust height */
      TT_DrawCellAlignTimeTable ();

      /* Row for this hour */
      for (Day=0; Day<TT_DAYS; Day++)
        {
         /* Check how many colums are needed.
            For each item (class) in this hour from left to right, we must check the maximum of columns */
         for (H=0; H<TT_HOURS_PER_DAY*2; H++)
            TimeTableHoursChecked[H] = false;
         ColumnsToDraw = TT_TimeTableCalculateColsToDraw (Day,Hour);
         if (!Editando && ColumnsToDraw == 0)
            ColumnsToDraw = 1;
         ColumnsToDrawIncludingExtraColumn = ColumnsToDraw;
         if (Editando && ColumnsToDraw < TT_MAX_COLUMNS_PER_CELL)
            ColumnsToDrawIncludingExtraColumn++;

        /* Draw cells */
         for (ContinuousFreeMinicolumns = 0, Column=0; Column<ColumnsToDrawIncludingExtraColumn; Column++)
            if (TimeTable[Day][Hour].Columns[Column].HourType == TT_FREE_HOUR)
               ContinuousFreeMinicolumns += TT_NUM_MINICOLUMNS_PER_DAY/ColumnsToDrawIncludingExtraColumn;
            else
              {
               if (ContinuousFreeMinicolumns)
                 {
                  TT_TimeTableDrawCell (Day,Hour,Column-1,ContinuousFreeMinicolumns,-1L,TT_FREE_HOUR,TT_NO_CLASS,0,NULL,-1,NULL);
                  ContinuousFreeMinicolumns = 0;
                 }
               TT_TimeTableDrawCell (Day,Hour,Column,TT_NUM_MINICOLUMNS_PER_DAY/ColumnsToDrawIncludingExtraColumn,
	                             TimeTable[Day][Hour].Columns[Column].CrsCod,
				     TimeTable[Day][Hour].Columns[Column].HourType,
	                             TimeTable[Day][Hour].Columns[Column].ClassType,
                                     TimeTable[Day][Hour].Columns[Column].Duration,
	                             TimeTable[Day][Hour].Columns[Column].Group,
	                             TimeTable[Day][Hour].Columns[Column].GrpCod,
                                     TimeTable[Day][Hour].Columns[Column].Place);
              }
         if (ContinuousFreeMinicolumns)
            TT_TimeTableDrawCell (Day,Hour,Column-1,ContinuousFreeMinicolumns,-1L,TT_FREE_HOUR,TT_NO_CLASS,0,NULL,-1L,NULL);
        }

      /* Empty column used to adjust height */
      TT_DrawCellAlignTimeTable ();

      /* Hour */
      if (Hour % 2)
         fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" rowspan=\"2\" width=\"%u%%\" class=\"HOR\">%2u:00</td>",
                  TT_PERCENT_WIDTH_OF_AN_HOUR_COLUMN,TT_START_HOUR + (Hour+1)/2);

      fprintf (Gbl.F.Out,"</tr>");
     }

   /***** Free list of groups types and groups in this course *****/
   if (Gbl.CurrentAct == ActEdiCrsTimTbl || Gbl.CurrentAct == ActChgCrsTimTbl)
      Grp_FreeListGrpTypesAndGrps ();

   /***** Row with day names *****/
   fprintf (Gbl.F.Out,"<tr>");
   TT_DrawCellAlignTimeTable ();
   TT_TimeTableDrawDaysCells ();
   TT_DrawCellAlignTimeTable ();
   fprintf (Gbl.F.Out,"</tr>");

   /***** Bottom row used for column adjustement *****/
   TT_TimeTableDrawAdjustRow (false);

   /***** End of the table *****/
   fprintf (Gbl.F.Out,"</table>\n");
  }

/****************************************************************************/
/********** Draw a row used for column adjustement in a time table **********/
/****************************************************************************/

void TT_TimeTableDrawAdjustRow (bool DrawSeparationBetweenDays)
  {
   unsigned Day,Minicolumn;

   fprintf (Gbl.F.Out,"<tr>");
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u%%\" class=\"HOR\">&nbsp;</td>",
            TT_PERCENT_WIDTH_OF_AN_HOUR_COLUMN);
   TT_DrawCellAlignTimeTable ();
   for (Day=0; Day<TT_DAYS; Day++)
     {
      for (Minicolumn=0; Minicolumn<TT_NUM_MINICOLUMNS_PER_DAY; Minicolumn++)
         fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u%%\" class=\"HOR\">&nbsp;</td>",TT_PERCENT_WIDTH_OF_A_MINICOLUMN);
      if (DrawSeparationBetweenDays && Day < TT_DAYS-1)
         fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u%%\" rowspan=\"%u\" class=\"HOR\">&nbsp;</td>",
                  TT_PERCENT_WIDTH_OF_A_SEPARATION_COLUMN,2+TT_HOURS_PER_DAY*2+2);
     }
   TT_DrawCellAlignTimeTable ();
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u%%\" class=\"HOR\">&nbsp;</td>",
            TT_PERCENT_WIDTH_OF_AN_HOUR_COLUMN);
   fprintf (Gbl.F.Out,"</tr>");
  }

/****************************************************************************/
/***************** Draw cells with day names in a time table ****************/
/****************************************************************************/

void TT_TimeTableDrawDaysCells (void)
  {
   extern const char *Txt_DAYS_CAPS[7][Txt_NUM_LANGUAGES];
   unsigned Day;

   for (Day=0; Day<TT_DAYS; Day++)
      fprintf (Gbl.F.Out,"<td align=\"center\" width=\"%u%%\" colspan=\"%u\" class=\"HOR\">%s</td>",
               TT_PERCENT_WIDTH_OF_A_DAY,TT_NUM_MINICOLUMNS_PER_DAY,Txt_DAYS_CAPS[Day][Gbl.Prefs.Language]);
  }

/****************************************************************************/
/*** Calculate recursively number of columns to draw for a day and hour *****/
/****************************************************************************/

unsigned TT_TimeTableCalculateColsToDraw (unsigned Day,unsigned Hour)
  {
   unsigned ColumnsToDraw,Column,H,FirstHour,Cols;

   ColumnsToDraw = TimeTable[Day][Hour].NumColumns;

   if (!TimeTableHoursChecked[Hour])
     {
      TimeTableHoursChecked[Hour] = true;
      for (Column=0; Column<TT_MAX_COLUMNS_PER_CELL; Column++)
        {
         switch (TimeTable[Day][Hour].Columns[Column].HourType)
           {
            case TT_FREE_HOUR:
               break;
            case TT_FIRST_HOUR:
               /* Check from first hour (this one) to last hour searching maximum number of columns */
               for (H=Hour+1; H<Hour+TimeTable[Day][Hour].Columns[Column].Duration; H++)
                  if (!TimeTableHoursChecked[H])
                    {
                     Cols = TT_TimeTableCalculateColsToDraw (Day,H);
                     if (Cols > ColumnsToDraw)
                        ColumnsToDraw = Cols;
                    }
               break;
            case TT_NEXT_HOUR:
               /* Find first hour for this item (class) */
               for (FirstHour=Hour; TimeTable[Day][FirstHour].Columns[Column].HourType == TT_NEXT_HOUR; FirstHour--);
                 /* Check from first hour to last hour searching maximum number of columns */
               for (H=FirstHour; H<FirstHour+TimeTable[Day][FirstHour].Columns[Column].Duration; H++)
                  if (!TimeTableHoursChecked[H])
                    {
                     Cols = TT_TimeTableCalculateColsToDraw (Day,H);
                     if (Cols > ColumnsToDraw)
                        ColumnsToDraw = Cols;
                    }
               break;
           }
        }
     }
   return ColumnsToDraw;
  }

/****************************************************************************/
/******************** Write empty cell for alignment ************************/
/****************************************************************************/

void TT_DrawCellAlignTimeTable (void)
  {
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u%%\" class=\"HOR\">&nbsp;</td>",
            TT_PERCENT_WIDTH_OF_A_SEPARATION_COLUMN);
  }

/****************************************************************************/
/*********************** Write una celda del horario *********************/
/****************************************************************************/

void TT_TimeTableDrawCell (unsigned Day,unsigned Hour,unsigned Column,unsigned ColSpan,
                           long CrsCod,TT_HourType_t HourType,TT_ClassType_t ClassType,int Duration,char *Group,long GrpCod,char *Place)
  {
   extern const char *Txt_Group[Txt_NUM_LANGUAGES];
   extern const char *Txt_Place[Txt_NUM_LANGUAGES];
   static const char *TimeTableClasses[TT_NUM_CLASS_TYPES] = {"FREE","THEO","PRAC","TUTO"};
   enum
     {
      SEE_CRS_TIMETABLE,
      EDIT_CRS_TIMETABLE,
      SEE_TUT_TIMETABLE,
      EDIT_TUT_TIMETABLE,
     } TimeTableView = SEE_CRS_TIMETABLE;
   struct GroupData GrpDat;
   long NumGrpTyp,NumGrp;
   unsigned H,Dur,MaxDuration,RowSpan = 0;
   TT_ClassType_t CT;
   struct Course Crs;

   /***** Compute row span and background color depending on hour type *****/
   switch (HourType)
     {
      case TT_FREE_HOUR:		// Se escribe una casilla libre
         RowSpan = 1;
	 break;
      case TT_FIRST_HOUR:	// Se escribe una casilla normal
	 RowSpan = Duration;
	 break;
      case TT_NEXT_HOUR:	// No se escribe nada
	 break;
     }

   /***** If there's nothing to do... *****/
   if (RowSpan == 0)
      return;

   /***** Set type of view depending on current action *****/
   switch (Gbl.CurrentAct)
     {
      case ActSeeCrsTimTbl:	case ActPrnCrsTimTbl:
      case ActSeeMyTimTbl:	case ActPrnMyTimTbl:
	 TimeTableView = SEE_CRS_TIMETABLE;
	 break;
      case ActEdiCrsTimTbl:	case ActChgCrsTimTbl:
	 TimeTableView = EDIT_CRS_TIMETABLE;
	 break;
      case ActSeeTut:
	 TimeTableView = SEE_TUT_TIMETABLE;
	 break;
      case ActEdiTut:		case ActChgTut:
	 TimeTableView = EDIT_TUT_TIMETABLE;
     }

   /***** If group code is not -1, a group is selected ==> get group type and name *****/
   if (HourType == TT_FIRST_HOUR &&
       (TimeTableView == SEE_CRS_TIMETABLE ||
        TimeTableView == EDIT_CRS_TIMETABLE) &&
       GrpCod != -1)
     {
      /* Get group type and name */
      GrpDat.GrpCod = GrpCod;
      Grp_GetDataOfGroupByCod (&GrpDat);
     }

   /***** Cell start *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" rowspan=\"%u\" colspan=\"%d\" class=\"%s%s\">",
            RowSpan,ColSpan,TimeTableClasses[ClassType],(ClassType == TT_NO_CLASS) ? (((Hour/2) % 2) ? "1" : "0") : "");

   /***** Form to modify this cell *****/
   if (TimeTableView == EDIT_CRS_TIMETABLE)
      Act_FormStart (ActChgCrsTimTbl);
   else if (TimeTableView == EDIT_TUT_TIMETABLE)
      Act_FormStart (ActChgTut);

   /***** Draw cell depending on type of view *****/
   switch (TimeTableView)
     {
      case SEE_CRS_TIMETABLE:
      case SEE_TUT_TIMETABLE:
	 if (HourType == TT_FREE_HOUR) // If cell is empty...
	    fprintf (Gbl.F.Out,"<img src=\"%s/tr1x16.gif\" width=\"1\" height=\"16\" />",
                     Gbl.Prefs.IconsURL);
	 else
	   {
	    fprintf (Gbl.F.Out,"<span class=\"HOR\">");
	    if (Gbl.CurrentAct == ActSeeMyTimTbl ||
                Gbl.CurrentAct == ActPrnMyTimTbl)
              {
               Crs.CrsCod = CrsCod;
               Deg_GetDataOfCourseByCod (&Crs);
	       if (Crs.ShortName[0])
                  fprintf (Gbl.F.Out,"%s<br />",Crs.ShortName);
               else
                  fprintf (Gbl.F.Out,"Asignatura desconocida<br />");
              }
	    fprintf (Gbl.F.Out,TimeTableStrsClassTypeScreen[ClassType]);
	    fprintf (Gbl.F.Out," (%dh%s)",Duration/2,(Duration % 2) ? "30'" : "");
	    if (TimeTableView == SEE_CRS_TIMETABLE)
	      {
               if (GrpCod == -1)
                 {
	          if (Group[0])
		     fprintf (Gbl.F.Out,"<br />%s %s ",
                              Txt_Group[Gbl.Prefs.Language],
                              Group);
                 }
               else
                 {
                  Str_LimitLengthHTMLStr (GrpDat.GrpTypName,12);
                  Str_LimitLengthHTMLStr (GrpDat.GrpName,12);
		  fprintf (Gbl.F.Out,"<br />%s %s %s",
                           Txt_Group[Gbl.Prefs.Language],
                           GrpDat.GrpTypName,GrpDat.GrpName);
                 }
	       if (Place[0])
		  fprintf (Gbl.F.Out,"<br />Aula %s",Place);
	      }
	    else	// TimeTableView == SEE_TUT_TIMETABLE
	       if (Place[0])
		  fprintf (Gbl.F.Out,"<br />%s",Place);
            fprintf (Gbl.F.Out,"</span>");
	   }
	 break;
      case EDIT_CRS_TIMETABLE:
      case EDIT_TUT_TIMETABLE:
	 fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ModTTDay\" value=\"%u\" />",Day);
	 fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ModTTHour\" value=\"%u\" />",Hour);
	 fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ModTTCol\" value=\"%u\" />",Column);

	 fprintf (Gbl.F.Out,"<span class=\"DAT_SMALL\">");

	 /***** Class type *****/
	 fprintf (Gbl.F.Out,"<select name=\"ModTTCellType\" style=\"width:54px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
	 for (CT=(TT_ClassType_t) 0; CT<(TT_ClassType_t) TT_NUM_CLASS_TYPES; CT++)
	    if ((CT == TT_NO_CLASS) ||
		((TimeTableView == EDIT_CRS_TIMETABLE) && (CT == TT_THEORY_CLASS || CT == TT_PRACT_CLASS)) ||
		((TimeTableView == EDIT_TUT_TIMETABLE) && (CT == TT_TUTOR_CLASS)))
	      {
	       fprintf (Gbl.F.Out,"<option");
	       if (CT == ClassType)
		  fprintf (Gbl.F.Out," selected=\"selected\"");
	       fprintf (Gbl.F.Out,">%s</option>",TimeTableStrsClassTypeScreen[CT]);
	      }
	 fprintf (Gbl.F.Out,"</select>");
	 if (HourType == TT_FREE_HOUR)
	   {
	    fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ModTTDur\" value=\"");
            for (H=Hour+1; H<TT_HOURS_PER_DAY*2; H++)
              if (TimeTable[Day][H].NumColumns == TT_MAX_COLUMNS_PER_CELL)
                  break;
            MaxDuration = H - Hour;
	    if (MaxDuration > 1)
	       fprintf (Gbl.F.Out,"1:00");
	    else
	       fprintf (Gbl.F.Out,"0:30");
	    fprintf (Gbl.F.Out," h\" />");
	    fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ModHorGrp\"  value=\"\" />");
	    fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ModHorLugar\" value=\"\" />");
	   }
	 else
	   {
	    /***** Class duration *****/
	    fprintf (Gbl.F.Out,"<select name=\"ModTTDur\" style=\"width:46px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
            for (H=Hour+TimeTable[Day][Hour].Columns[Column].Duration; H<TT_HOURS_PER_DAY*2; H++)
               if (TimeTable[Day][H].NumColumns == TT_MAX_COLUMNS_PER_CELL)
                  break;
            MaxDuration = H - Hour;
            if (TimeTable[Day][Hour].Columns[Column].Duration > MaxDuration)
               MaxDuration = TimeTable[Day][Hour].Columns[Column].Duration;
	    for (Dur=0; Dur<=MaxDuration; Dur++)
	      {
	       fprintf (Gbl.F.Out,"<option");
	       if (Dur == Duration)
		  fprintf (Gbl.F.Out," selected=\"selected\"");
	       fprintf (Gbl.F.Out,">%d:%02d h</option>",Dur/2,(Dur % 2) ? 30 : 0);
	      }
	    fprintf (Gbl.F.Out,"</select>");

	    if (TimeTableView == EDIT_CRS_TIMETABLE)
	      {
	       /***** Group *****/
	       fprintf (Gbl.F.Out,"<br />%s",Txt_Group[Gbl.Prefs.Language]);
	       fprintf (Gbl.F.Out,"<select name=\"ModTTGrpCod\" style=\"width:88px;\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
               fprintf (Gbl.F.Out,"<option value=\"-1\"");
	       if (GrpCod == -1)
		  fprintf (Gbl.F.Out," selected=\"selected\"");
               fprintf (Gbl.F.Out,">Todos</option>");
               for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
                  for (NumGrp=0; NumGrp<Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps; NumGrp++)
                    {
	             fprintf (Gbl.F.Out,"<option value=\"%ld\"",Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].LstGrps[NumGrp].GrpCod);
	             if (GrpCod == Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].LstGrps[NumGrp].GrpCod)
		        fprintf (Gbl.F.Out," selected=\"selected\"");
                     Str_LimitLengthHTMLStr (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypName,8);
                     Str_LimitLengthHTMLStr (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].LstGrps[NumGrp].GrpName,8);
	             fprintf (Gbl.F.Out,">%s %s</option>",Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypName,
                              Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].LstGrps[NumGrp].GrpName);
                    }
	       fprintf (Gbl.F.Out,"</select>");

	       /***** Class room *****/
	       fprintf (Gbl.F.Out,"<br />Aula");
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ModHorLugar\" size=\"1\" maxlength=\"%u\" value=\"%s\" onchange=\"javascript:document.forms[%d].submit();\" />",
		        TT_MAX_LENGTH_PLACE,Place,Gbl.NumForm);
	      }
	    else // TimeTableView == EDIT_TUT_TIMETABLE
	      {
	       fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ModHorGrp\" value=\"\" />");
	       /***** Place *****/
	       fprintf (Gbl.F.Out,"<br />%s",
                        Txt_Place[Gbl.Prefs.Language]);
	       fprintf (Gbl.F.Out,"<input type=\"text\" name=\"ModHorLugar\" size=\"12\" maxlength=\"%u\" value=\"%s\" onchange=\"javascript:document.forms[%d].submit();\" />",
		        TT_MAX_LENGTH_PLACE,Place,Gbl.NumForm);
	      }
	   }
         fprintf (Gbl.F.Out,"</span>");
	 break;
     }

   /***** End of form *****/
   if (TimeTableView == EDIT_CRS_TIMETABLE ||
       TimeTableView == EDIT_TUT_TIMETABLE)
      fprintf (Gbl.F.Out,"</form>");

   /***** End of cell *****/
   fprintf (Gbl.F.Out,"</td>");
  }
