// swad_ws.c: SWAD web service provided to external plugins

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/******* How to use the gSOAP toolkit to create a web-service server ********/
/****************************************************************************/
/*
This code uses the gSOAP toolkit:
http://www.cs.fsu.edu/~engelen/soap.html
http://gsoap2.sourceforge.net/
Version: 2.7.13 stable, March 21, 2009

To install the toolkit:
1. gunzip gsoap_2.7.13.tar.gz 
2. ./configure
3. make
4. make install

Steps to generate swad.wsdl:
1. Go to directory soap
cd /home/acanas/swad/swad/soap

2. Inside soap, create a C header file with the web service. Example swad_web_service.h:
----------
// File: swad_web_service.h
//gsoap swad service name: swad
//gsoap swad service namespace: urn:swad
//gsoap swad service location: https://swad.ugr.es/
...
----------

3. Inside soap, execute soapcpp2 compiler:

soapcpp2 -c -S swad_web_service.h
4. Copy generated WSDL file (swad.wsdl) to public location (probably you must be root):
cp -f /home/acanas/swad/swad/soap/swad.wsdl /var/www/html/ws/

5. Put a '#include "soap/soapH.h"' line into the code of the web-service server (this file).

6. Put a '#include "soap/swad.nsmap"' into the code of the web-service server (this file).

7. Compile swad including the web service.
The makefile must include -lgsoap, and compile soapC.c and soapServer.c files generated by soapcpp2 in step 2
Example of Makefile:
---------------------
OBJS = swad.o swad_act.o swad_asg.o swad_brw.o swad_cht.o swad_con.o swad_crs.o swad_cry.o swad_ctr.o swad_dat.o swad_db.o swad_deg.o swad_dpt.o swad_exa.o swad_fil.o swad_for.o swad_grp.o swad_hld.o swad_imp.o swad_ins.o swad_lay.o swad_lnk.o swad_log.o swad_msg.o swad_not.o swad_ntf.o swad_pag.o swad_par.o swad_plc.o swad_plg.o swad_prf.o swad_rec.o swad_ses.o swad_sta.o swad_str.o swad_syl.o swad_the.o swad_tst.o swad_tt.o swad_txt.o swad_usr.o swad_ws.o sha2.o
SOAPOBJS = soap/soapC.o soap/soapServer.o
CC = gcc

LIBS = -lmysqlclient -lz -L/usr/lib64/mysql -lm -lgsoap

CFLAGS = -Wall -O2 -s

swad: $(OBJS) $(SOAPOBJS)
	$(CC) $(CFLAGS) -o $@ $(OBJS) $(SOAPOBJS) $(LIBS)
	chmod a+x $@
---------------------

8. Copy CGI (swad) to the cgi directory:
cp -f /home/acanas/swad/swad/swad /var/www/cgi-bin/

*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <string.h>

#include "soap/soapH.h"		// gSOAP header
#include "soap/swad.nsmap"	// Namespaces map used

#include "swad_gbl.h"
#include "swad_usr.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern const char Str_BIN_TO_BASE64URL[64];

/****************************************************************************/
/***************************** Private constants ****************************/
/****************************************************************************/

static const char *WS_UsrTypeName[Usr_NUM_USR_TYPES] =
  {
   "unknown",		// 0, Usr_UNKNOWN
   "invited",		// 1, Usr_INVITED
   "student",		// 2, Usr_STUDENT
   "teacher",		// 3, Usr_TEACHER
   "administrator",	// 4, Usr_DEG_ADMIN
   "superuser",		// 5, Usr_SUPERUSER
  };

/****************************************************************************/
/********************************* Data types *******************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Private prototypes ***************************/
/****************************************************************************/

static int WS_CheckIfIPIsAllowed (struct soap *soap);
static int WS_CheckIdSession (struct soap *soap,const char *IdSession);
static int WS_CheckCourseAndGroupCodes (struct soap *soap,long CrsCod,long GrpCod);
static int WS_GenerateNewWSKey (struct soap *soap,long UsrCod,char *WSKey);
static int WS_RemoveOldWSKeys (struct soap *soap);
static int WS_CheckWSKey (struct soap *soap,char *WSKey,long *UsrCod);
static int WS_GetSomeUsrDataFromUsrCod (struct soap *soap,struct UsrData *UsrDat);

/****************************************************************************/
/******* Function called when a web service if required by a plugin *********/
/****************************************************************************/

void WS_WebService (void)
  {
   struct soap soap; // gSOAP runtime environment

   /**** Create soap context and serve one CGI-based request *****/
  //  soap_serve (soap_new ());

   soap_init (&soap); // Initialize runtime environment (only once)
   soap_serve (&soap);
   soap_end (&soap);
  }

/****************************************************************************/
/********** Check if the IP of the requester of a web service ***************/
/********** is one of the IP allowed in the plugins           ***************/
/****************************************************************************/

static int WS_CheckIfIPIsAllowed (struct soap *soap)
  {
   char IP[Cte_MAX_LENGTH_IP+1];
   char Query[256+Cte_MAX_LENGTH_IP];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get IP *****/
   if (getenv ("REMOTE_ADDR"))
     {
      strncpy (IP,getenv ("REMOTE_ADDR"),Cte_MAX_LENGTH_IP);
      IP[Cte_MAX_LENGTH_IP] = '\0';
     }
   else
      IP[0] = '\0';

   /***** Get number of plugins with a IP address *****/
   sprintf (Query,"SELECT COUNT(*) FROM plugins WHERE IP='%s'",IP);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_sender_fault (soap,"can not check IP in database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_sender_fault (soap,"can not check IP in database",mysql_error (&Gbl.mysql));

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      return soap_sender_fault (soap,"can not check IP in database","Error reading from database");

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   if (!NumRows)
     {
      sprintf (Gbl.Message,"Sender's IP (%s) is forbidden",IP);
      return soap_sender_fault (soap,Gbl.Message,"The IP of the requester of a web service is not allowed");
     }

   return SOAP_OK;
  }

/****************************************************************************/
/****** Check if a session identifier is valid and exists in database *******/
/****************************************************************************/

static int WS_CheckIdSession (struct soap *soap,const char *IdSession)
  {
   const char *Ptr;
   unsigned i;
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows = 0;
   bool Error = false;

   /***** Check if pointer is NULL *****/
   if (IdSession == NULL)
      return soap_sender_fault (soap,"Bad session identifier","Session identifier is a null pointer"); 

   /***** Check length of session identifier *****/
   if (strlen (IdSession) != Ses_LENGTH_SESSION_ID)
      return soap_sender_fault (soap,"Bad session identifier","The length of the session identifier is wrong"); 

   /***** Check if session identifier is in base64url *****/
   for (Ptr=IdSession; *Ptr; Ptr++)
     {
      for (i=0; i<64; i++)	// Check if this character is one of the allowed characters
         if (*Ptr == Str_BIN_TO_BASE64URL[i])
            break;
      if (i == 64)	// Character not found
         return soap_sender_fault (soap,"Bad session identifier","The session identifier must contain only base64url characters"); 
     }

   /***** Query if session identifier already exists in database *****/
   sprintf (Query,"SELECT COUNT(*) FROM sessions WHERE SessionId='%s'",
            IdSession);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get session data from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get session data from database",mysql_error (&Gbl.mysql));

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Error = true;

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   if (Error || NumRows != 1)
      return soap_receiver_fault (soap,"Bad session identifier","Session identifier doesn't exist in database");
   
   return SOAP_OK;
  }

/****************************************************************************/
/********* Check if a course code is valid and exists in database ***********/
/****************************************************************************/

static int WS_CheckCourseAndGroupCodes (struct soap *soap,long CrsCod,long GrpCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows = 0;
   bool Error = false;

   /***** Check if course code is correct *****/
   if (CrsCod <= 0)
      return soap_sender_fault (soap,"Bad course code","Course code must be a integer greater than 0"); 

   /***** Query if course code already exists in database *****/
   sprintf (Query,"SELECT COUNT(*) FROM courses WHERE CrsCod='%ld'",
            CrsCod);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get course from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get course from database",mysql_error (&Gbl.mysql));

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Error = true;

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   if (Error || NumRows != 1)
      return soap_sender_fault (soap,"Bad course code","Course code doesn't exist in database");
   
   /***** Course code exists in database, so check if group code exists in database and belongs to course *****/
   if (GrpCod > 0)	// <=0 means "the whole course"
     {
      /***** Query if group code already exists in database *****/
      sprintf (Query,"SELECT COUNT(*) FROM crs_grp_types,crs_grp" \
                     " WHERE crs_grp_types.CrsCod='%ld' AND crs_grp_types.GrpTypCod=crs_grp.GrpTypCod AND crs_grp.GrpCod='%ld'",
               CrsCod,GrpCod);
      if (mysql_query (&Gbl.mysql,Query))
         return soap_receiver_fault (soap,"can not get group from database",mysql_error (&Gbl.mysql));

      /***** Store query result *****/
      if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
         return soap_receiver_fault (soap,"can not get group from database",mysql_error (&Gbl.mysql));

      /***** Get number of rows *****/
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%u",&NumRows) != 1)
         Error = true;

      /***** Free structure that stores the query result *****/
      mysql_free_result (mysql_res);

      if (Error || NumRows != 1)
         return soap_sender_fault (soap,"Bad group code","Group code doesn't exist in database or it's not a group of the specified course");
     }

   return SOAP_OK;
  }

/****************************************************************************/
/***** Generate a key used in subsequents calls to other web services *******/
/****************************************************************************/

static int WS_GenerateNewWSKey (struct soap *soap,long UsrCod,char *WSKey)
  {
   int ReturnCode;
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows = 0;
   bool Error = false;

   /***** Remove expired web service keys *****/
   if ((ReturnCode = WS_RemoveOldWSKeys (soap)) != SOAP_OK)
      return ReturnCode;

   /***** Create a unique name for the key *****/
   strcpy (WSKey,Gbl.UniqueNameEncrypted);

   /***** Check that key doesn't exist in database *****/
   sprintf (Query,"SELECT COUNT(*) FROM ws_keys WHERE WSKey='%s'",
            WSKey);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get existence of key from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get existence of key from database",mysql_error (&Gbl.mysql));

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Error = true;

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   if (Error || NumRows != 0)
      return soap_receiver_fault (soap,"Error when generating key","Generated key already existed in database");

   /***** Insert key into database *****/
   sprintf (Query,"INSERT INTO ws_keys (WSKey,UsrCod,LastTime)" \
                  " VALUES ('%s','%ld',NOW())",
            WSKey,UsrCod);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not insert new key into database",mysql_error (&Gbl.mysql));

   return SOAP_OK;
  }

/*****************************************************************************/
/************************ Remove old web service keys ************************/
/*****************************************************************************/

static int WS_RemoveOldWSKeys (struct soap *soap)
  {
   char Query[512];

   /***** Remove expired sessions *****/
   /* A session expire when last click (LastTime) is too old,
      or when there was at least one refresh (navigator supports AJAX) and last refresh is too old (browser probably was closed) */
   sprintf (Query,"DELETE LOW_PRIORITY FROM ws_keys WHERE" \
                  " UNIX_TIMESTAMP() > UNIX_TIMESTAMP(LastTime)+%ld",
            Cfg_TIME_TO_DELETE_WS_KEY);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not remove old web service keys from database",mysql_error (&Gbl.mysql));

   return SOAP_OK;
  }

/****************************************************************************/
/************** Check if a web service key exists in database ***************/
/****************************************************************************/

static int WS_CheckWSKey (struct soap *soap,char *WSKey,long *UsrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Set default user's code *****/
   *UsrCod = -1L;

   /***** Check that key doesn't exist in database *****/
   sprintf (Query,"SELECT UsrCod FROM ws_keys WHERE WSKey='%s'",
            WSKey);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get existence of key from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get existence of key from database",mysql_error (&Gbl.mysql));

   /***** Get number of rows *****/
   if ((NumRows = mysql_num_rows (mysql_res)))	// Session found in table of sessions
     {
      row = mysql_fetch_row (mysql_res);

      *UsrCod = Str_ConvertStrCodToLongCod (row[0]);
     }

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   return SOAP_OK;
  }

/****************************************************************************/
/************ Get user's data from database giving a user's code ************/
/****************************************************************************/
// UsrDat->UsrCod must contain an existing user's code

static int WS_GetSomeUsrDataFromUsrCod (struct soap *soap,struct UsrData *UsrDat)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;

   /***** Query database *****/
   sprintf (Query,"SELECT DNI,UsrType,Surname1,Surname2,FirstName" \
                  " FROM usr_data WHERE UsrCod='%ld'",
            UsrDat->UsrCod);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get user's data from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get user's data from database",mysql_error (&Gbl.mysql));

   /***** Check number of rows in result *****/
   if (mysql_num_rows (mysql_res) != 1)
      return soap_receiver_fault (soap,"can not get user's data from database","User doen't exist in database");

   /***** Read the data comunes a all the users *****/
   row = mysql_fetch_row (mysql_res);

   /* Get user's ID */
   strncpy (UsrDat->DNI,row[0],sizeof (UsrDat->DNI)-1);
   UsrDat->DNI[sizeof (UsrDat->DNI)-1] = '\0';

   /* Get user's type */
   UsrDat->UsrTypeInDB = Usr_ConvertUnsignedStrToUsrType (row[1]);

   /* Get user's name */
   strncpy (UsrDat->Surname1 ,row[2],sizeof (UsrDat->Surname1 )-1);
   strncpy (UsrDat->Surname2 ,row[3],sizeof (UsrDat->Surname2 )-1);
   strncpy (UsrDat->FirstName,row[4],sizeof (UsrDat->FirstName)-1);

   UsrDat->Surname1 [sizeof (UsrDat->Surname1 )-1] = '\0';
   UsrDat->Surname2 [sizeof (UsrDat->Surname2 )-1] = '\0';
   UsrDat->FirstName[sizeof (UsrDat->FirstName)-1] = '\0';

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   return SOAP_OK;
  }

/****************************************************************************/
/*********************** Return data of a session ***************************/
/****************************************************************************/

int swad__loginBySession (struct soap *soap,char *sessionID,struct swad__loginBySessionOutput *loginBySessionOut)
  {
   int ReturnCode;
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   struct UsrData UsrDat;
   struct DegreeType DegTyp;
   struct Degree Deg;
   struct Course Crs;

   /***** Allocate space for strings *****/
   loginBySessionOut->wsKey          = (char *) soap_malloc (soap,256); 
   loginBySessionOut->userID         = (char *) soap_malloc (soap,256); 
   loginBySessionOut->userFirstname  = (char *) soap_malloc (soap,256); 
   loginBySessionOut->userSurname1   = (char *) soap_malloc (soap,256); 
   loginBySessionOut->userSurname2   = (char *) soap_malloc (soap,256);
   loginBySessionOut->userTypeName   = (char *) soap_malloc (soap,256); 
   loginBySessionOut->degreeTypeName = (char *) soap_malloc (soap,256); 
   loginBySessionOut->degreeName     = (char *) soap_malloc (soap,256); 
   loginBySessionOut->courseName     = (char *) soap_malloc (soap,256); 

   /***** Default values returned on error *****/
   loginBySessionOut->userCode          = -1;
   loginBySessionOut->userTypeCode      = (int) Usr_UNKNOWN;
   loginBySessionOut->degreeTypeCode    = -1;
   loginBySessionOut->degreeCode        = -1;
   loginBySessionOut->courseCode        = -1;
   loginBySessionOut->wsKey[0]          = '\0';
   loginBySessionOut->userID[0]         = '\0';
   loginBySessionOut->userFirstname[0]  = '\0';
   loginBySessionOut->userSurname1[0]   = '\0';
   loginBySessionOut->userSurname2[0]   = '\0';
   strncpy (loginBySessionOut->userTypeName,WS_UsrTypeName[loginBySessionOut->userTypeCode],255);
   loginBySessionOut->userTypeName[255] = '\0';
   loginBySessionOut->degreeTypeName[0] = '\0';
   loginBySessionOut->degreeName[0]     = '\0';
   loginBySessionOut->courseName[0]     = '\0';

   /***** Check IP of the sender *****/
   if ((ReturnCode = WS_CheckIfIPIsAllowed (soap)) != SOAP_OK)
      return ReturnCode;

   /***** Check length of session identifier *****/
   if (sessionID == NULL)
      return soap_sender_fault (soap,"sessionID is null","login by session"); 

   /***** Check session identifier coming from an external plugin *****/
   if ((ReturnCode = WS_CheckIdSession (soap,sessionID)) != SOAP_OK)
      return ReturnCode;

   // Now, we know that sessionID is a valid session identifier
   /***** Query data of the session from database *****/
   sprintf (Query,"SELECT UsrCod,DegTypCod,DegCod,CrsCod FROM sessions" \
                  " WHERE SessionId='%s'",sessionID);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get session data from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get session data from database",mysql_error (&Gbl.mysql));

   /***** Get some data of session from database *****/
   if ((NumRows = mysql_num_rows (mysql_res)))	// Session found in table of sessions
     {
      row = mysql_fetch_row (mysql_res);

      /***** Get user code (row[0]) *****/
      UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);
      if (WS_GetSomeUsrDataFromUsrCod (soap,&UsrDat) == SOAP_OK)		// Get some user's data from database
        {
         loginBySessionOut->userCode = (int) UsrDat.UsrCod;
         strncpy (loginBySessionOut->userID,UsrDat.DNI,255);
         loginBySessionOut->userID[255] = '\0';

         strncpy (loginBySessionOut->userSurname1,UsrDat.Surname1,255);
         loginBySessionOut->userSurname1[255] = '\0';
         strncpy (loginBySessionOut->userSurname2,UsrDat.Surname2,255);
         loginBySessionOut->userSurname2[255] = '\0';
         strncpy (loginBySessionOut->userFirstname,UsrDat.FirstName,255);
         loginBySessionOut->userFirstname[255] = '\0';
         loginBySessionOut->userTypeCode = (int) UsrDat.UsrTypeInDB;
         strncpy (loginBySessionOut->userTypeName,WS_UsrTypeName[loginBySessionOut->userTypeCode],255);
         loginBySessionOut->userTypeName[255] = '\0';
        }

      /***** Get degree type (row[1]) *****/
      DegTyp.DegTypCod = Str_ConvertStrCodToLongCod (row[1]);
      Deg_GetDataOfDegreeTypeByCod (&DegTyp);
      loginBySessionOut->degreeTypeCode = (int) DegTyp.DegTypCod;
      strncpy (loginBySessionOut->degreeTypeName,DegTyp.DegTypName,255);
      loginBySessionOut->degreeTypeName[255] = '\0';

      /***** Get degree (row[2]) *****/
      Deg.DegCod = Str_ConvertStrCodToLongCod (row[2]);
      Deg_GetDataOfDegreeByCod (&Deg);
      loginBySessionOut->degreeCode = (int) Deg.DegCod;
      strncpy (loginBySessionOut->degreeName,Deg.FullName,255);
      loginBySessionOut->degreeName[255] = '\0';

      /***** Get course (row[3]) *****/
      Crs.CrsCod = Str_ConvertStrCodToLongCod (row[3]);
      Deg_GetDataOfCourseByCod (&Crs);
      loginBySessionOut->courseCode = (int) Crs.CrsCod;
      strncpy (loginBySessionOut->courseName,Crs.FullName,255);
      loginBySessionOut->courseName[255] = '\0';
     }

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   if (NumRows == 1)
     {
      /***** Generate a key used in subsequents calls to other web services *****/
      if ((ReturnCode = WS_GenerateNewWSKey (soap,(long) loginBySessionOut->userCode,loginBySessionOut->wsKey)) != SOAP_OK)
         return ReturnCode;

      return SOAP_OK;
     }
   else
      return soap_receiver_fault (soap,"Bad session identifier","Session identifier doesn't exist in database");
  }

/****************************************************************************/
/*********************** Return data of a session ***************************/
/****************************************************************************/

int swad__loginByUserPassword (struct soap *soap,char *userID,char *userPassword,struct swad__loginByUserPasswordOutput *loginByUserPasswordOut)
  {
   char DNIorNickname[256];
   int ReturnCode;
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   struct UsrData UsrDat;

   /***** Allocate space for strings *****/
   loginByUserPasswordOut->wsKey          = (char *) soap_malloc (soap,256); 
   loginByUserPasswordOut->userID         = (char *) soap_malloc (soap,256); 
   loginByUserPasswordOut->userFirstname  = (char *) soap_malloc (soap,256); 
   loginByUserPasswordOut->userSurname1   = (char *) soap_malloc (soap,256); 
   loginByUserPasswordOut->userSurname2   = (char *) soap_malloc (soap,256);
   loginByUserPasswordOut->userTypeName   = (char *) soap_malloc (soap,256); 

   /***** Default values returned on error *****/
   loginByUserPasswordOut->userCode          = -1;
   loginByUserPasswordOut->userTypeCode      = (int) Usr_UNKNOWN;
   loginByUserPasswordOut->wsKey[0]          = '\0';
   loginByUserPasswordOut->userID[0]         = '\0';
   loginByUserPasswordOut->userFirstname[0]  = '\0';
   loginByUserPasswordOut->userSurname1[0]   = '\0';
   loginByUserPasswordOut->userSurname2[0]   = '\0';
   strncpy (loginByUserPasswordOut->userTypeName,WS_UsrTypeName[loginByUserPasswordOut->userTypeCode],255);
   loginByUserPasswordOut->userTypeName[255] = '\0';

   /***** Check IP of the sender *****/
   if ((ReturnCode = WS_CheckIfIPIsAllowed (soap)) != SOAP_OK)
      return ReturnCode;

   /***** Check if user's ID or nickname are valid *****/
   strncpy (DNIorNickname,userID,255);
   DNIorNickname[255] = '\0';
   if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (userID))
     {
      /* User has typed a valid DNI (existing or not) */
      sprintf (Query,"SELECT UsrCod FROM usr_data" \
                     " WHERE DNI='%s' AND Password='%s'",
               DNIorNickname,userPassword);
     }
   else if (Usr_CheckIfNickIsValid (userID))
     {
      /* User has typed an existing nickname */
      sprintf (Query,"SELECT UsrCod FROM usr_data" \
                     " WHERE Nickname='%s' AND Password='%s'",
               DNIorNickname,userPassword);
     }
   else
      return soap_receiver_fault (soap,"Bad log in","User's ID or nickname don't exist or password is wrong");

   /***** Query data of the session from database *****/
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get user's data from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get user's data from database",mysql_error (&Gbl.mysql));

   /***** Get user's data from database *****/
   if ((NumRows = mysql_num_rows (mysql_res)))	// User found in table of users' data
     {
      row = mysql_fetch_row (mysql_res);

      /***** Get user code (row[0]) *****/
      UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);
      if (WS_GetSomeUsrDataFromUsrCod (soap,&UsrDat) == SOAP_OK)		// Get some user's data from database
        {
         loginByUserPasswordOut->userCode = (int) UsrDat.UsrCod;
         strncpy (loginByUserPasswordOut->userID,UsrDat.DNI,255);
         loginByUserPasswordOut->userID[255] = '\0';

         strncpy (loginByUserPasswordOut->userSurname1,UsrDat.Surname1,255);
         loginByUserPasswordOut->userSurname1[255] = '\0';
         strncpy (loginByUserPasswordOut->userSurname2,UsrDat.Surname2,255);
         loginByUserPasswordOut->userSurname2[255] = '\0';
         strncpy (loginByUserPasswordOut->userFirstname,UsrDat.FirstName,255);
         loginByUserPasswordOut->userFirstname[255] = '\0';
         loginByUserPasswordOut->userTypeCode = (int) UsrDat.UsrTypeInDB;
         strncpy (loginByUserPasswordOut->userTypeName,WS_UsrTypeName[loginByUserPasswordOut->userTypeCode],255);
         loginByUserPasswordOut->userTypeName[255] = '\0';
        }
     }

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   if (NumRows == 1)
     {
      /***** Generate a key used in subsequents calls to other web services *****/
      if ((ReturnCode = WS_GenerateNewWSKey (soap,(long) loginByUserPasswordOut->userCode,loginByUserPasswordOut->wsKey)) != SOAP_OK)
         return ReturnCode;

      return SOAP_OK;
     }
   else
      return soap_receiver_fault (soap,"Bad log in","User's ID or nickname don't exist or password is wrong");
  }

/****************************************************************************/
/*********************** Return courses of a user ***************************/
/****************************************************************************/

int swad__getCourses (struct soap *soap,char *wsKey,struct swad__getCoursesOutput *getCoursesOut)
  {
   long UsrCod;
   int ReturnCode;
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRow,NumRows;

   /***** Check IP of the sender *****/
   if ((ReturnCode = WS_CheckIfIPIsAllowed (soap)) != SOAP_OK)
      return ReturnCode;

   /***** Check web service key *****/
   if ((ReturnCode = WS_CheckWSKey (soap,wsKey,&UsrCod)) != SOAP_OK)
      return ReturnCode;
   if (UsrCod < 0)	// Web service key doesn't exist in database
      return soap_receiver_fault (soap,"Bad web service key","Web service key doesn't exist in database");

   /***** Query my courses from database *****/
   sprintf (Query,"SELECT courses.CrsCod,courses.FullName FROM crs_usr,courses" \
                  " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod=courses.CrsCod" \
                  " ORDER BY courses.FullName",
            UsrCod);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get user's courses from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get user's courses from database",mysql_error (&Gbl.mysql));

   /***** Return the number of rows of the result *****/
   NumRows = (unsigned) mysql_num_rows (mysql_res);

   getCoursesOut->numCourses = (int) NumRows;
   getCoursesOut->coursesArray.__size = (int) NumRows;

   if (NumRows == 0)
      getCoursesOut->coursesArray.__ptr = NULL;
   else	// Courses found
     {
      getCoursesOut->coursesArray.__ptr = soap_malloc (soap,(getCoursesOut->coursesArray.__size) * sizeof (*(getCoursesOut->coursesArray.__ptr)));

      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
	 /* Get next course */
	 row = mysql_fetch_row (mysql_res);

         /* Get course code (row[0]) */
	 getCoursesOut->coursesArray.__ptr[NumRow].courseCode = (int) Str_ConvertStrCodToLongCod (row[0]);

         /* Get course name (row[1]) */
         getCoursesOut->coursesArray.__ptr[NumRow].courseName = (char *) soap_malloc (soap,256);
	 strncpy (getCoursesOut->coursesArray.__ptr[NumRow].courseName,row[1],255);
         getCoursesOut->coursesArray.__ptr[NumRow].courseName[255] = '\0';
	}
     }

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   return SOAP_OK;
  }

/****************************************************************************/
/********************* Return students in a course **************************/
/****************************************************************************/
// If groupCode <= 0 ==> get users from the whole course

int swad__getUsers (struct soap *soap,char *wsKey,int courseCode,int groupCode,int userType,struct swad__getUsersOutput *getUsersOut)
  {
   int ReturnCode;
   long UsrCod;
   Usr_UsrType_t UsrTypeInDB;
   bool Error = false;
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRow,NumRows;

   /***** Check IP of the sender *****/
   if ((ReturnCode = WS_CheckIfIPIsAllowed (soap)) != SOAP_OK)
      return ReturnCode;

   /***** Check web service key *****/
   if ((ReturnCode = WS_CheckWSKey (soap,wsKey,&UsrCod)) != SOAP_OK)
      return ReturnCode;
   if (UsrCod < 0)	// Web service key doesn't exist in database
      return soap_receiver_fault (soap,"Bad web service key","Web service key doesn't exist in database");

   /***** Check course and group codes *****/
   if ((ReturnCode = WS_CheckCourseAndGroupCodes (soap,(long) courseCode,(long) groupCode)) != SOAP_OK)
      return ReturnCode;

   /***** Check requested users' type *****/
   if (userType != (int) Usr_STUDENT &&
       userType != (int) Usr_TEACHER)
     {
      sprintf (Gbl.Message,"Only students (user's type = %u) or teachers (user's type = %u) are allowed",
               (unsigned) Usr_STUDENT,(unsigned) Usr_TEACHER);
      return soap_sender_fault (soap,"Bad requested users' type",Gbl.Message);
     }

   /***** Check if the user in the web service key is a teacher *****/
   sprintf (Query,"SELECT UsrType FROM usr_data WHERE UsrCod='%ld'",UsrCod); 
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get user's type from database",mysql_error (&Gbl.mysql));
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get user's type from database",mysql_error (&Gbl.mysql));
   if (mysql_num_rows (mysql_res) != 1)
      return soap_receiver_fault (soap,"can not get user's type from database","User's not found in database");
   row = mysql_fetch_row (mysql_res);
   UsrTypeInDB = Usr_ConvertUnsignedStrToUsrType (row[0]);
   if (UsrTypeInDB != Usr_TEACHER)
      return soap_receiver_fault (soap,"Request forbidden","Requester must be a teacher");
   mysql_free_result (mysql_res);

   /***** Check if the user in the web service belongs to course *****/
   if (groupCode <= 0)	// Users belonging to the whole course
      sprintf (Query,"SELECT COUNT(*) FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
               (long) courseCode,UsrCod);
   else			// Users belonging to the group
      sprintf (Query,"SELECT COUNT(*) FROM crs_grp_usr WHERE GrpCod='%ld' AND UsrCod='%ld'",
               (long) groupCode,UsrCod);
   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get whether user belongs to course/group from database",mysql_error (&Gbl.mysql));
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get whether user belongs to course/group from database",mysql_error (&Gbl.mysql));
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Error = true;
   mysql_free_result (mysql_res);
   if (Error || NumRows != 1)
      return soap_receiver_fault (soap,"Request forbidden","Requester must belong to course");

   /***** Query users beloging to course or group from database *****/
   if (groupCode <= 0)	// Users belonging to the whole course
      sprintf (Query,"SELECT usr_data.UsrCod,usr_data.DNI,usr_data.Surname1,usr_data.Surname2,usr_data.FirstName" \
                     " FROM crs_usr,usr_data" \
                     " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%d'" \
                     " ORDER BY usr_data.Surname1,usr_data.Surname2,usr_data.FirstName,usr_data.DNI",
               (long) courseCode,userType);
   else			// Users belonging to the group
      sprintf (Query,"SELECT usr_data.UsrCod,usr_data.DNI,usr_data.Surname1,usr_data.Surname2,usr_data.FirstName" \
                     " FROM crs_grp_usr,usr_data" \
                     " WHERE crs_grp_usr.GrpCod='%ld' AND crs_grp_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%d'" \
                     " ORDER BY usr_data.Surname1,usr_data.Surname2,usr_data.FirstName,usr_data.DNI",
               (long) groupCode,userType);

   if (mysql_query (&Gbl.mysql,Query))
      return soap_receiver_fault (soap,"can not get users from database",mysql_error (&Gbl.mysql));

   /***** Store query result *****/
   if ((mysql_res = mysql_store_result (&Gbl.mysql)) == NULL)
      return soap_receiver_fault (soap,"can not get users courses from database",mysql_error (&Gbl.mysql));

   /***** Return the number of rows of the result *****/
   NumRows = (unsigned) mysql_num_rows (mysql_res);

   getUsersOut->numUsers = (int) NumRows;
   getUsersOut->usersArray.__size = (int) NumRows;

   if (NumRows == 0)
      getUsersOut->usersArray.__ptr = NULL;
   else	// Courses found
     {
      getUsersOut->usersArray.__ptr = soap_malloc (soap,(getUsersOut->usersArray.__size) * sizeof (*(getUsersOut->usersArray.__ptr)));

      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
	 /* Get next course */
	 row = mysql_fetch_row (mysql_res);

         /* Get user's code (row[0]) */
	 getUsersOut->usersArray.__ptr[NumRow].userCode = (int) Str_ConvertStrCodToLongCod (row[0]);

         /* Get user's ID (row[1]) */
         getUsersOut->usersArray.__ptr[NumRow].userID = (char *) soap_malloc (soap,256);
         switch (userType)
           {
            case Usr_STUDENT:
	       strncpy (getUsersOut->usersArray.__ptr[NumRow].userID,row[1],255);
               getUsersOut->usersArray.__ptr[NumRow].userID[255] = '\0';
               break;
            default:	// Hide user's ID
               strcpy (getUsersOut->usersArray.__ptr[NumRow].userID,"********");
               break;
           }

         /* Get user's surname1 (row[2]) */
         getUsersOut->usersArray.__ptr[NumRow].userSurname1 = (char *) soap_malloc (soap,256);
	 strncpy (getUsersOut->usersArray.__ptr[NumRow].userSurname1,row[2],255);
         getUsersOut->usersArray.__ptr[NumRow].userSurname1[255] = '\0';

         /* Get user's surname2 (row[3]) */
         getUsersOut->usersArray.__ptr[NumRow].userSurname2 = (char *) soap_malloc (soap,256);
	 strncpy (getUsersOut->usersArray.__ptr[NumRow].userSurname2,row[3],255);
         getUsersOut->usersArray.__ptr[NumRow].userSurname2[255] = '\0';

         /* Get user's first name (row[4]) */
         getUsersOut->usersArray.__ptr[NumRow].userFirstname = (char *) soap_malloc (soap,256);
	 strncpy (getUsersOut->usersArray.__ptr[NumRow].userFirstname,row[4],255);
         getUsersOut->usersArray.__ptr[NumRow].userFirstname[255] = '\0';
	}
     }

   /***** Free structure that stores the query result *****/
   mysql_free_result (mysql_res);

   return SOAP_OK;
  }
