// swad_sta.c: statistics

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/********************************* Headers **********************************/
/****************************************************************************/

#include <math.h>		// For log10, floor, ceil, modf, sqrt...
#include <stdlib.h>		// For system, getenv, etc.
#include <string.h>		// For string functions
#include <sys/wait.h>		// For the macro WEXITSTATUS
#include <unistd.h>		// For unlink

#include "swad_act.h"
#include "swad_brw.h"
#include "swad_cfg.h"
#include "swad_db.h"
#include "swad_for.h"
#include "swad_gbl.h"
#include "swad_not.h"
#include "swad_ntf.h"
#include "swad_par.h"
#include "swad_sta.h"

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Actions Actions[Act_NUM_ACTIONS];

/****************************************************************************/
/******************** Internal constants and types **************************/
/****************************************************************************/

#define Sta_SECONDS_IN_RECENT_LOG ((time_t)(Cfg_DAYS_IN_RECENT_LOG*24UL*60UL*60UL))	// Remove entries in recent log oldest than this time

const char *Sta_StrAvgPhotoTypes[Sta_NUM_AVERAGE_PHOTO_TYPES] =
  {
   "mediana de todas las fotos",
   "media aritm&eacute;tica de todas las fotos",	// Need translation!
  };
const char *Sta_StrAvgPhotoDirs[Sta_NUM_AVERAGE_PHOTO_TYPES] =
  {
   Cfg_FOLDER_DEGREE_PHOTO_MEDIAN,
   Cfg_FOLDER_DEGREE_PHOTO_AVERAGE,
  };
const char *Sta_StrAvgPhotoPrograms[Sta_NUM_AVERAGE_PHOTO_TYPES] =
  {
   Cfg_COMMAND_DEGREE_PHOTO_MEDIAN,
   Cfg_COMMAND_DEGREE_PHOTO_AVERAGE,
  };
const char *Sta_StrPhotoSizes[Sta_NUM_HOW_COMPUTE_PHOTO_SIZES] =
  {
   "proporcional al n&ordm; de alumnos",
   "proporcional al n&ordm; de alumnos con fotograf&iacute;a",
   "proporcional al porcentaje de alumnos con fotograf&iacute;a",
   "igual para todas las titulaciones",	// Need translation!
  };
const char *Sta_StrOrderDegrees[Sta_NUM_HOW_ORDER_DEGREES] =
  {
   "n&ordm; de alumnos",
   "n&ordm; de alumnos con fotograf&iacute;a",
   "porcentaje de alumnos con fotograf&iacute;a",
   "nombre de titulaci&oacute;n",	// Need translation!
  };

/****************************************************************************/
/***************************** Internal prototypes **************************/
/****************************************************************************/

static void Sta_PutSeeAccessesButton (void);
static void Sta_WriteSelectorDegCrs (Sta_LocationRange_t DefaultLocationRange);
static void Sta_PutSelectorLocationRange (void);
static void Sta_GetLocationRange (Sta_LocationRange_t DefaultLocationRange);
static void Sta_WriteSelectorCountType (void);
static void Sta_WriteSelectorAction (void);
static void Sta_RemoveOldEntriesRecentLog (void);
static bool Sta_SeeAccesses (void);
static void Sta_ShowDetailedAccessesList (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_ShowNumAccessesPerUsr (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_ShowNumAccessesPerDays (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_ShowDistrAccessesPerDaysAndHour (unsigned long NumRows,MYSQL_RES *mysql_res);
static Sta_ColorType_t Sta_GetStatColorType (void);
static void Sta_DrawBarColors (Sta_ColorType_t ColorType,float MaxPagesGenerated);
static void Sta_DrawAccessesPerHourForADay (Sta_ColorType_t ColorType,float NumPagesGenerated[24],float MaxPagesGenerated);
static void Sta_SetColor (Sta_ColorType_t ColorType,float NumPagesGenerated,float MaxPagesGenerated,unsigned *R,unsigned *G,unsigned *B);
static void Sta_ShowNumAccessesPerWeeks (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_ShowNumAccessesPerMonths (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_ShowNumAccessesPerHour (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_WriteAccessHour (unsigned Hour,float NumPagesGenerated,float MaxPagesGenerated,float TotalPagesGenerated,unsigned ColumnWidth);
static void Sta_ShowAverageAccessesPerMinute (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_WriteLabelsXAxisAccMin (float IncX,const char *Format);
static void Sta_WriteAccessMinute (unsigned Minute,float NumPagesGenerated,float MaxX);
static void Sta_ShowNumAccessesPerAction (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_ShowNumAccessesPerDegree (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_ShowNumAccessesPerCourse (unsigned long NumRows,MYSQL_RES *mysql_res);
static void Sta_WriteDegree (long DegCod);
static void Sta_DrawBarNumAccesses (char Color,float NumPagesGenerated,float MaxPagesGenerated,float TotalPagesGenerated,unsigned MaxBarWidth);
static void Sta_WriteSelectedRangeOfDates (unsigned NumDays);
static void Sta_WriteFloatNum (float Number);
static void Sta_GetAndShowNumDegsInSWAD (void);
static void Sta_GetAndShowNumCrssInSWAD (void);
static unsigned Sta_GetNumCoursesWithUsrs (Usr_UsrType_t UsrType);
static void Sta_GetAndShowNumUsrsInPlatform (Usr_UsrType_t UsrType);
static unsigned Sta_GetNumUsrsBelongingToAnyCrs (Usr_UsrType_t UsrType);
static float Sta_GetNumCrssPerUsr (Usr_UsrType_t UsrType);
static float Sta_GetNumUsrsPerCrs (Usr_UsrType_t UsrType);

static void Sta_ShowStatsExpTrees (void);
static void Sta_WriteStatsExpTreesTableHead (void);
static void Sta_WriteRowStatsExpTrees (Brw_FileBrowser_t FileZone,const char *NameOfFileZones);
static void Sta_GetSizeOfFileZoneFromDB (Sta_LocationRange_t LocationRange,Brw_FileBrowser_t FileBrowser,
                                         int *NumCrss,int *NumUsrs,unsigned *MaxLevels,unsigned long *NumFolders,unsigned long *NumFiles,unsigned long long int *Size);
static unsigned long Sta_GetNumFilesOfCrsFileZoneFromDB (Brw_FileBrowser_t FileBrowser,long CrsCod);

static void Sta_ShowMsgsStats (void);
static void Sta_ShowForumStats (void);
static void Sta_ShowStatOfAForumType (For_ForumType_t ForumType,long DegCod,long CrsCod);
static void Sta_WriteForumTitleAndStats (For_ForumType_t ForumType,long DegCod,long CrsCod,const char *Icon,const char *ForumName1,const char *ForumName2);

static void Sta_ShowNoticesStats (void);

static void Sta_GetAndShowNumUsrsPerLanguage (void);
static void Sta_GetAndShowNumUsrsPerTheme (void);
static void Sta_GetAndShowNumUsrsPerLayout (void);
static void Sta_GetAndShowNumUsrsPerNotifyEvent (void);

static void Sta_GetAndShowTableOfCourses (void);

static long Sta_GetDegWithAvgPhotoLeastRecentlyUpdated (void);
static long Sta_GetTimeAvgPhotoWasComputed (long DegCod);
static long Sta_GetTimeToComputeAvgPhoto (long DegCod);
static void Sta_ComputeAveragePhoto (long DegCod,Usr_Gender_t Gender,struct ListUsers *LstUsrs,
                                     Sta_AvgPhotoTypeOfAverage_t TypeOfAverage,const char *DirAvgPhotosRelPath,
                                     unsigned *NumStds,unsigned *NumStdsWithPhoto,long *TimeToComputeAvgPhotoInMicroseconds);
static void Sta_PutSelectorForTypeOfAvg (void);
static Sta_AvgPhotoTypeOfAverage_t Sta_GetPhotoAvgTypeFromForm (void);
static void Sta_PutSelectorForHowComputePhotoSize (void);
static Sta_HowComputePhotoSize_t Sta_GetHowComputePhotoSizeFromForm (void);
static void Sta_PutSelectorForHowOrderDegrees (void);
static Sta_HowOrderDegrees_t Sta_GetHowOrderDegreesFromForm (void);
static void Sta_PutLinkToPrintViewOfDegreeStats (void);
static void Sta_PutLinkToCalculateDegreeStats (void);
static void Sta_GetMaxStdsPerDegree (void);
static void Sta_ShowOrPrintClassPhotoDegrees (Sta_AvgPhotoSeeOrPrint_t SeeOrPrint);
static void Sta_ShowOrPrintListDegrees (Sta_AvgPhotoSeeOrPrint_t SeeOrPrint);
static void Sta_BuildQueryOfDegrees (char *Query);
static void Sta_GetNumStdsInDegree (long DegCod,Usr_Gender_t Gender,int *NumStds,int *NumStdsWithPhoto);
static void Sta_UpdateDegStats (long DegCod,Usr_Gender_t Gender,unsigned NumStds,unsigned NumStdsWithPhoto,long TimeToComputeAvgPhoto);
static void Sta_ShowDegreeStat (int NumStds,int NumStdsWithPhoto);
static void Sta_ShowDegreeAvgPhotoAndStat (struct Degree *Deg,Sta_AvgPhotoSeeOrPrint_t SeeOrPrint,Usr_Gender_t Gender,int NumStds,int NumStdsWithPhoto,struct Date *DateAvgPhoto);
static void Sta_ComputePhotoSize (int NumStds,int NumStdsWithPhoto,unsigned *PhotoWidth,unsigned *PhotoHeight);

static void Sta_WriteTime (char *Str,long TimeInMicroseconds);

/*****************************************************************************/
/*************** Read CGI environment variable REMOTE_ADDR *******************/
/*****************************************************************************/
/*
CGI Environment Variables:
REMOTE_ADDR
The IP address of the remote host making the request.
*/
void Sta_GetRemoteAddr (void)
  {
   if (getenv ("REMOTE_ADDR"))
     {
      strncpy (Gbl.IP,getenv ("REMOTE_ADDR"),Cte_MAX_LENGTH_IP);
      Gbl.IP[Cte_MAX_LENGTH_IP] = '\0';
     }
   else
      Gbl.IP[0] = '\0';
  }

/****************************************************************************/
/**************************** Log access in database ************************/
/****************************************************************************/

void Sta_LogAccess (const char *Comments)
  {
   char QueryLogHistoric[2048];
   char QueryLogRecent  [2048];
   char StrTimeGenerate[1+1+10+1+1];
   char StrTimeSend    [1+1+10+1+1];

   /***** Create strings with generation and sending times *****/
   if (Gbl.TimeGenerationInMicroseconds >= 0)
      sprintf (StrTimeGenerate,"'%ld'",Gbl.TimeGenerationInMicroseconds);
   else
      strcpy (StrTimeGenerate,"NULL");

   if (Gbl.TimeSendInMicroseconds >= 0)
      sprintf (StrTimeSend,"'%ld'",Gbl.TimeSendInMicroseconds);
   else
      strcpy (StrTimeSend,"NULL");

   /***** Sometimes, we delete old entries in recent log table *****/
   Sta_RemoveOldEntriesRecentLog ();

   /***** Insert access into database *****/
   if (Comments)
     {
      sprintf (QueryLogHistoric,"INSERT INTO log (DegCod,CrsCod,DNI,UsrType,ClickTime,TimeToGenerate,TimeToSend,Action,IP,Comments)" \
                                " VALUES('%ld','%ld','%s','%u',NOW(),%s,%s,'%s','%s','",
	       Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.DNI,(unsigned) Gbl.Usrs.Me.UsrTypeLog,
               StrTimeGenerate,StrTimeSend,Actions[Gbl.CurrentAct].LogMessage,
               Gbl.IP);
      sprintf (QueryLogRecent  ,"INSERT INTO log_recent (DegCod,CrsCod,DNI,UsrType,ClickTime,TimeToGenerate,TimeToSend,Action,IP,Comments)" \
                                " VALUES('%ld','%ld','%s','%u',NOW(),%s,%s,'%s','%s','",
	       Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.DNI,(unsigned) Gbl.Usrs.Me.UsrTypeLog,
               StrTimeGenerate,StrTimeSend,Actions[Gbl.CurrentAct].LogMessage,
               Gbl.IP);
      Str_AddStrToQuery (QueryLogHistoric,Comments,sizeof (QueryLogHistoric));
      Str_AddStrToQuery (QueryLogRecent  ,Comments,sizeof (QueryLogRecent  ));
      strcat (QueryLogHistoric,"')");
      strcat (QueryLogRecent  ,"')");
     }
   else
     {
      sprintf (QueryLogHistoric,"INSERT INTO log (DegCod,CrsCod,DNI,UsrType,ClickTime,TimeToGenerate,TimeToSend,Action,IP,Comments)" \
                                " VALUES('%ld','%ld','%s','%u',NOW(),%s,%s,'%s','%s',NULL)",
	       Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.DNI,Gbl.Usrs.Me.UsrTypeLog,
               StrTimeGenerate,StrTimeSend,Actions[Gbl.CurrentAct].LogMessage,
               Gbl.IP);
      sprintf (QueryLogRecent  ,"INSERT INTO log_recent (DegCod,CrsCod,DNI,UsrType,ClickTime,TimeToGenerate,TimeToSend,Action,IP,Comments)" \
                                " VALUES('%ld','%ld','%s','%u',NOW(),%s,%s,'%s','%s',NULL)",
	       Gbl.CurrentDeg.Deg.DegCod,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.DNI,Gbl.Usrs.Me.UsrTypeLog,
               StrTimeGenerate,StrTimeSend,Actions[Gbl.CurrentAct].LogMessage,
               Gbl.IP);
     }
   DB_QueryINSERT (QueryLogHistoric,"can not log access");
   DB_QueryINSERT (QueryLogRecent  ,"can not log access");
  }

/*****************************************************************************/
/************ Sometimes, we delete old entries in recent log table ***********/
/*****************************************************************************/

static void Sta_RemoveOldEntriesRecentLog (void)
  {
   char Query[512];

   if (!(Gbl.PID % 1000))	// Do this only one of 1000 times
     {
      /***** Remove all expired clipboards *****/
      sprintf (Query,"DELETE LOW_PRIORITY FROM log_recent" \
                     " WHERE UNIX_TIMESTAMP(ClickTime)<UNIX_TIMESTAMP()-%ld",
               Sta_SECONDS_IN_RECENT_LOG);
      DB_QueryDELETE (Query,"can not remove old entries from recent log");
     }
  }

/*****************************************************************************/
/******************** Show a form to make a query of clicks ******************/
/*****************************************************************************/

void Sta_AskSeeCrsAccesses (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_distributed_by[Txt_NUM_LANGUAGES];
   extern const char *Txt_STA_CLICK_STAT_TYPES[Sta_NUM_TYPES_CLICK_STATS][Txt_NUM_LANGUAGES];
   static unsigned long RowsPerPage[] = {10,20,30,40,50,100,500,1000,5000,10000,50000,100000};
#define NUM_OPTIONS_ROWS_PER_PAGE (sizeof (RowsPerPage) / sizeof (RowsPerPage[0]))
   Sta_ClicksStatType_t ClicksStatType;
   unsigned long i;

   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Show form to select the grupos *****/
   Grp_ShowFormToSelectSeveralGroups (ActReqAccCrs);

   /***** Form to select type of list used for select several users *****/
   Usr_ShowFormsToSelectUsrListType (ActReqAccCrs);

   /***** Get and order the lists of users of this course *****/
   Usr_GetAndOrderStdsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);
   Usr_GetAndOrderTchsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L);

   if (!Gbl.Usrs.LstTchs.NumUsrs && !Gbl.Usrs.LstStds.NumUsrs)
     {
      Lay_ShowAlert (WARNING,"No hay profesores ni alumnos.");	// Need translation!
      return;
     }

   /***** Get list of selected users *****/
   Usr_GetListSelectedUsrs ();

   Act_FormStart (ActSeeAccCrs);
   Grp_PutParamsCodGrps ();
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FirstRow\" value=\"0\" />");
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"LastRow\" value=\"0\" />");

   /***** Draw two class photographs with the users: one for teachers of the course and another one for students *****/
   /* Start the table */
   fprintf (Gbl.F.Out,"<div align=\"center\"><table><tr><td align=\"left\">\n");
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,"100%");

   /* Put list of users to select some users */
   Usr_ListUsersToSelect (Usr_TEACHER);
   Usr_ListUsersToSelect (Usr_STUDENT);

   /* Free memory used by las listas */
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /* Free the memory used by the list of users */
   Usr_FreeListsEncryptedUsrCods ();

   /* Free memory for list of selected groups */
   Grp_FreeListCodSelectedGrps ();

   /* End the table */
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Initial and final dates of the search *****/
   Dat_WriteFormIniEndDates ();

   /***** Selection of action *****/
   Sta_WriteSelectorAction ();

   /***** Selection of count type (number of pages generated, accesses per user, etc.) *****/
   Sta_WriteSelectorCountType ();

   /***** Type of statistic *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"><table><tr><td align=\"left\" class=\"%s\">%s</td><td align=\"left\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_distributed_by[Gbl.Prefs.Language]);
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   if ((Gbl.Stat.ClicksStatType < ACC_CRS_PER_USR ||
        Gbl.Stat.ClicksStatType > ACC_CRS_PER_ACTION) &&
        Gbl.Stat.ClicksStatType != ACC_CRS_LISTING) 
      Gbl.Stat.ClicksStatType = ACC_CRS_PER_USR;
   for (ClicksStatType=ACC_CRS_PER_USR; ClicksStatType<=ACC_CRS_PER_ACTION; ClicksStatType++) 
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\">" \
                         "<input type=\"radio\" name=\"ClickStatType\" value=\"%u\"",
               (unsigned) ClicksStatType);
      if (ClicksStatType == Gbl.Stat.ClicksStatType)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," onclick=\"disableRowsPage()\" /></td><td align=\"left\" class=\"%s\">%s</td></tr>",
               The_ClassFormul[Gbl.Prefs.Theme],Txt_STA_CLICK_STAT_TYPES[ClicksStatType][Gbl.Prefs.Language]);
     }
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr></table></td></tr>\n");

   /* Listing of clicks to this course  */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"middle\" class=\"%s\"><input type=\"radio\" name=\"ClickStatType\" value=\"%u\"",
            The_ClassFormul[Gbl.Prefs.Theme],(unsigned) ACC_CRS_LISTING);
   if (Gbl.Stat.ClicksStatType == ACC_CRS_LISTING)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," onclick=\"enableRowsPage()\">%s",
            Txt_STA_CLICK_STAT_TYPES[ACC_CRS_LISTING][Gbl.Prefs.Language]);

   /* Number of rows per page */
   fprintf (Gbl.F.Out," (resultados por p&aacute;gina: <select id=\"RowsPage\" name=\"RowsPage\""); // To use getElementById in Firefox, it's necessary to have the id attribute
   if (Gbl.Stat.ClicksStatType != ACC_CRS_LISTING)
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out,">");
   for (i=0; i<NUM_OPTIONS_ROWS_PER_PAGE; i++)
     {
      fprintf (Gbl.F.Out,"<option");
      if (RowsPerPage[i] == Gbl.Stat.RowsPerPage)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%lu",RowsPerPage[i]);
     }
   fprintf (Gbl.F.Out,"</select>)</td></tr>");

   /***** Submit button *****/
   Sta_PutSeeAccessesButton ();

   /***** Form end *****/
   fprintf (Gbl.F.Out,"</table></div></form>\n");
  }

/*****************************************************************************/
/********** Show a form to select the type of global stat of clics ***********/
/*****************************************************************************/

void Sta_AskSeeGblAccesses (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Users[Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPE_STATS[Sta_NUM_USR_TYPES_STAT][Txt_NUM_LANGUAGES];
   extern const char *Txt_distributed_by[Txt_NUM_LANGUAGES];
   extern const char *Txt_STA_CLICK_STAT_TYPES[Sta_NUM_TYPES_CLICK_STATS][Txt_NUM_LANGUAGES];
   Sta_UsrTypeStat_t UsrTypeStat;
   Sta_ClicksStatType_t ClicksStatType;

   /***** Start form *****/
   Act_FormStart (ActSeeAccGbl);
   fprintf (Gbl.F.Out,"<div align=\"center\"><table>\n");

   /***** Start and end dates for the search *****/
   Dat_WriteFormIniEndDates ();

   /***** Types of users whose accesses we want to see *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\">%s: <select name=\"UsrType\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Users[Gbl.Prefs.Language]);
   for (UsrTypeStat=(Sta_UsrTypeStat_t) 0; UsrTypeStat<Sta_NUM_USR_TYPES_STAT; UsrTypeStat++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) UsrTypeStat);
      if (UsrTypeStat == Gbl.Stat.UsrType)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s",Txt_USR_TYPE_STATS[UsrTypeStat][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</select></td></tr>");

   /***** Selection of action *****/
   Sta_WriteSelectorAction ();

   /***** Clicks made from anywhere, degree or course *****/
   Sta_WriteSelectorDegCrs (Sta_LOCATION_RANGE_ALL);

   /***** Count type for the statistic *****/
   Sta_WriteSelectorCountType ();

   /***** Type of statistic *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"><table><tr><td align=\"left\" class=\"%s\">%s</td><td align=\"left\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_distributed_by[Gbl.Prefs.Language]);
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   if (Gbl.Stat.ClicksStatType < ACC_GBL_PER_DAYS ||
       Gbl.Stat.ClicksStatType > ACC_GBL_PER_COURSE) 
      Gbl.Stat.ClicksStatType = ACC_GBL_PER_DAYS;
   for (ClicksStatType=ACC_GBL_PER_DAYS; ClicksStatType<=ACC_GBL_PER_COURSE; ClicksStatType++) 
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\">" \
                         "<input type=\"radio\" name=\"ClickStatType\" value=\"%u\"",
               (unsigned) ClicksStatType);
      if (ClicksStatType == Gbl.Stat.ClicksStatType)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," /></td><td align=\"left\" class=\"%s\">%s</td></tr>",
               The_ClassFormul[Gbl.Prefs.Theme],
               Txt_STA_CLICK_STAT_TYPES[ClicksStatType][Gbl.Prefs.Language]);
     }
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr></table></td></tr>\n");

   /***** Submit button *****/
   Sta_PutSeeAccessesButton ();

   /***** Form end *****/
   fprintf (Gbl.F.Out,"</table></div></form>\n");
  }

/*****************************************************************************/
/********************** Put submit button to see accesses ********************/
/*****************************************************************************/

static void Sta_PutSeeAccessesButton (void)
  {
   extern const char *Txt_Show_visits[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr><td align=\"center\">" \
                      "<input type=\"submit\" value=\"%s\" />" \
                      "</td></tr>\n",
            Txt_Show_visits[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/****** Put selectors for type of access count and for degree or course ******/
/*****************************************************************************/

static void Sta_WriteSelectorDegCrs (Sta_LocationRange_t DefaultLocationRange)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];

   /***** Get users range for access statistics *****/
   Sta_GetLocationRange (DefaultLocationRange);

   /***** Form to select range of users *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\">",
            The_ClassFormul[Gbl.Prefs.Theme]);
   Sta_PutSelectorLocationRange ();
   fprintf (Gbl.F.Out,"</td></tr>");
  }

/*****************************************************************************/
/** Put a selector to choice between ranges when getting users for listing ***/
/*****************************************************************************/

static void Sta_PutSelectorLocationRange (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Restrict_the_statistics_to[Txt_NUM_LANGUAGES];
   extern const char *Txt_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_course[Txt_NUM_LANGUAGES];

   /***** Start selector *****/
   fprintf (Gbl.F.Out,"<span class=\"%s\">%s: </span>" \
                      "<select name=\"LocationRange\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Restrict_the_statistics_to[Gbl.Prefs.Language]);

   /***** All accesses *****/
   fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) Sta_LOCATION_RANGE_ALL);
   if (Gbl.Stat.LocationRange == Sta_LOCATION_RANGE_ALL || Gbl.CurrentDeg.Deg.DegCod < 0)
      fprintf (Gbl.F.Out," selected=\"selected\"");
   fprintf (Gbl.F.Out,">%s</option>",Cfg_PLATFORM_SHORT_NAME);

   /***** Accesses to current degree *****/
   if (Gbl.CurrentDeg.Deg.DegCod >= 0)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) Sta_LOCATION_RANGE_THIS_DEGREE);
      if (Gbl.Stat.LocationRange == Sta_LOCATION_RANGE_THIS_DEGREE)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s: %s</option>",
               Txt_degree[Gbl.Prefs.Language],Gbl.CurrentDeg.Deg.ShortName);

      /***** Accesses to current course *****/
      if (Gbl.CurrentCrs.Crs.CrsCod >= 0)
        {
         fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) Sta_LOCATION_RANGE_THIS_COURSE);
         if (Gbl.Stat.LocationRange == Sta_LOCATION_RANGE_THIS_COURSE)
            fprintf (Gbl.F.Out," selected=\"selected\"");
         fprintf (Gbl.F.Out,">%s: %s</option>",
                  Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.ShortName);
        }
     }

   /***** End selector *****/
   fprintf (Gbl.F.Out,"</select>");
  }

/*****************************************************************************/
/****************** Get users range for access statistics ********************/
/*****************************************************************************/

static void Sta_GetLocationRange (Sta_LocationRange_t DefaultLocationRange)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   /***** Get users' range *****/
   Par_GetParam (Par_PARAM_SINGLE,"LocationRange",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      Gbl.Stat.LocationRange = DefaultLocationRange;
   else if (UnsignedNum >= NUM_TYPES_OF_LOCATION_RANGE)
      Gbl.Stat.LocationRange = DefaultLocationRange;
   else
      Gbl.Stat.LocationRange = (Sta_LocationRange_t) UnsignedNum;

   /***** Avoid impossible users' ranges *****/
   if (Gbl.CurrentDeg.Deg.DegCod < 0)
      Gbl.Stat.LocationRange = Sta_LOCATION_RANGE_ALL;
   else if (Gbl.CurrentCrs.Crs.CrsCod < 0 && Gbl.Stat.LocationRange == Sta_LOCATION_RANGE_THIS_COURSE)
      Gbl.Stat.LocationRange = Sta_LOCATION_RANGE_THIS_DEGREE;
  }

/*****************************************************************************/
/****** Put selectors for type of access count and for degree or course ******/
/*****************************************************************************/

static void Sta_WriteSelectorCountType (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_SMALL[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   Sta_CountType_t StatCountType;

   /**** Count type *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\">%s: <select name=\"CountType\" id=\"CountType\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Show[Gbl.Prefs.Language]);
   for (StatCountType=(Sta_CountType_t) 0; StatCountType<Sta_NUM_STAT_COUNT_TYPES; StatCountType++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) StatCountType);
      if (StatCountType == Gbl.Stat.CountType)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s",Txt_TYPE_COUNT_STAT_SMALL[StatCountType][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</select></td></tr>");
  }

/*****************************************************************************/
/****** Put selectors for type of access count and for degree or course ******/
/*****************************************************************************/

static void Sta_WriteSelectorAction (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Action[Txt_NUM_LANGUAGES];
   Act_Action_t NumAction;

   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"%s\">%s: <select name=\"StatAct\" id=\"StatAct\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Action[Gbl.Prefs.Language]);
   for (NumAction=(Act_Action_t) 0; NumAction<Act_NUM_ACTIONS; NumAction++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) NumAction);
      if (NumAction == Gbl.Stat.NumAction)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s",Actions[NumAction].LogMessage);
     }

   fprintf (Gbl.F.Out,"</select></td></tr>");
  }

/*****************************************************************************/
/************ Set end date to current date                        ************/
/************ and set initial date to end date minus several days ************/
/*****************************************************************************/

void Sta_SetIniEndDates (void)
  {
   extern const unsigned NumDaysMonth[1+12];			// Declaration in swad_dat.c

   Gbl.DateRange.DateEnd.Day   = Gbl.Today.Day;
   Gbl.DateRange.DateEnd.Month = Gbl.Today.Month;
   Gbl.DateRange.DateEnd.Year  = Gbl.Today.Year;

   if (Gbl.DateRange.DateEnd.Day >= Cfg_DAYS_IN_RECENT_LOG)
     {
      Gbl.DateRange.DateIni.Year  = Gbl.DateRange.DateEnd.Year;
      Gbl.DateRange.DateIni.Month = Gbl.DateRange.DateEnd.Month;
      Gbl.DateRange.DateIni.Day   = Gbl.DateRange.DateEnd.Day - (Cfg_DAYS_IN_RECENT_LOG-1);
     }
   else
     {
      Gbl.DateRange.DateIni.Year  = (Gbl.DateRange.DateEnd.Month == 1) ? Gbl.DateRange.DateEnd.Year - 1 :
                                                                         Gbl.DateRange.DateEnd.Year;
      Gbl.DateRange.DateIni.Month = (Gbl.DateRange.DateEnd.Month == 1) ? 12 :
                                                                         Gbl.DateRange.DateEnd.Month - 1;
      Gbl.DateRange.DateIni.Day   = ((Gbl.DateRange.DateIni.Month == 2) ? Dat_GetNumDaysFebruary (Gbl.DateRange.DateIni.Year) : NumDaysMonth[Gbl.DateRange.DateIni.Month]) +
                                    Gbl.DateRange.DateEnd.Day - (Cfg_DAYS_IN_RECENT_LOG-1);
     }
  }

/****************************************************************************/
/******************** Compute and show access statistics ********************/
/****************************************************************************/

void Sta_SeeCrsAccesses (void)
  {
   if (!Sta_SeeAccesses ())
      Sta_AskSeeCrsAccesses ();
  }

void Sta_SeeGblAccesses (void)
  {
   if (!Sta_SeeAccesses ())
      Sta_AskSeeGblAccesses ();
  }

/****************************************************************************/
/******************** Compute and show access statistics ********************/
/****************************************************************************/

#define MAX_LENGTH_QUERY_ACCESS (1024 + (10+Usr_MAX_LENGTH_USR_ID)*5000)

// Returns false on error

static bool Sta_SeeAccesses (void)
  {
   extern const char *Txt_User[Txt_NUM_LANGUAGES];
   extern const char *Txt_Users[Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPE_STATS[Sta_NUM_USR_TYPES_STAT][Txt_NUM_LANGUAGES];
   extern const char *Txt_Action[Txt_NUM_LANGUAGES];
   extern const char *Txt_The_graph_shows_the_NUMBER[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_SMALL[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   extern const char *Txt_distributed_by[Txt_NUM_LANGUAGES];
   extern const char *Txt_STA_CLICK_STAT_TYPES[Sta_NUM_TYPES_CLICK_STATS][Txt_NUM_LANGUAGES];
   extern const char *Txt_You_must_select_one_ore_more_users[Txt_NUM_LANGUAGES];
   enum {STAT_GLOBAL,STAT_COURSE} StatsGlobalOrCourse;
   char UnsignedStr[10+1];
   char LongStr[1+10+1];
   unsigned UnsignedNum;
   enum {LOG_HISTORIC,LOG_RECENT} LogType;
   MYSQL_RES *mysql_res;
   unsigned long NumRows;
   struct UsrData UsrDat;
   char Query[MAX_LENGTH_QUERY_ACCESS+1];
   char QueryAux[256];
   long LengthQuery;
   unsigned NumUsr = 0;
   const char *Ptr;
   char StrDate[16+1];
   char StrUsrType[16+1];
   char StrQueryCountType[256];
   char CrsCodStr[1+10+1];
   unsigned NumDays;

   /***** Initialize data structure of the user *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** Get initial and ending dates *****/
   Dat_GetIniEndDatesFromForm ();
  
   /***** Set table where to find depending on initial date *****/
   // If initial day is older than current day minus Cfg_DAYS_IN_RECENT_LOG, then use recent log table, else use historic log table */
   LogType = (Dat_GetNumDaysBetweenDates (&Gbl.DateRange.DateIni,&Gbl.Today) <= Cfg_DAYS_IN_RECENT_LOG) ? LOG_RECENT : LOG_HISTORIC;

   /***** Get the type of stat of clicks ******/
   Par_GetParam (Par_PARAM_SINGLE,"ClickStatType",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      Lay_ShowErrorAndExit ("Type of query of accesses is missing.");
   if (UnsignedNum >= Sta_NUM_TYPES_CLICK_STATS)
      Lay_ShowErrorAndExit ("Type of query of accesses is missing.");
   Gbl.Stat.ClicksStatType = (Sta_ClicksStatType_t) UnsignedNum;

   /***** Get the type of count of clicks *****/
   if (Gbl.Stat.ClicksStatType != ACC_CRS_LISTING)
     {
      Par_GetParam (Par_PARAM_SINGLE,"CountType",UnsignedStr,10);
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         Lay_ShowErrorAndExit ("Type of count is missing.");
      if (UnsignedNum >= Sta_NUM_STAT_COUNT_TYPES)
         Lay_ShowErrorAndExit ("Type of count is missing.");
      Gbl.Stat.CountType = (Sta_CountType_t) UnsignedNum;
     }

   /***** Get action *****/
   Par_GetParam (Par_PARAM_SINGLE,"StatAct",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      Lay_ShowErrorAndExit ("Action is missing.");
   if (UnsignedNum >= Act_NUM_ACTIONS)
      Lay_ShowErrorAndExit ("Action is missing.");
   Gbl.Stat.NumAction = (Act_Action_t) UnsignedNum;

   switch (Gbl.Stat.ClicksStatType)
     {
      case ACC_GBL_PER_DAYS:
      case ACC_GBL_PER_DAYS_AND_HOUR:
      case ACC_GBL_PER_WEEKS:
      case ACC_GBL_PER_MONTHS:
      case ACC_GBL_PER_HOUR:
      case ACC_GBL_PER_MINUTE:
      case ACC_GBL_PER_ACTION:
      case ACC_GBL_PER_DEGREE:
      case ACC_GBL_PER_COURSE:
         StatsGlobalOrCourse = STAT_GLOBAL;
	 break;
      default:
         StatsGlobalOrCourse = STAT_COURSE;
         break;
     }

   if (Gbl.CurrentAct == ActSeeAccCrs)
     {
      if (Gbl.Stat.ClicksStatType == ACC_CRS_LISTING)
	{
	 /****** Get the number of the first row to show ******/
	 Par_GetParam (Par_PARAM_SINGLE,"FirstRow",LongStr,1+10);
	 if (sscanf (LongStr,"%ld",&Gbl.Stat.FirstRow) != 1)
	    Lay_ShowErrorAndExit ("Number of start row is missing.");

	 /****** Get the number of the last row to show ******/
	 Par_GetParam (Par_PARAM_SINGLE,"LastRow",LongStr,1+10);
	 if (sscanf (LongStr,"%ld",&Gbl.Stat.LastRow) != 1)
	    Lay_ShowErrorAndExit ("Number of end row is missing.");

	 /****** Get the number of rows per page ******/
	 Par_GetParam (Par_PARAM_SINGLE,"RowsPage",LongStr,1+10);
	 if (sscanf (LongStr,"%ld",&Gbl.Stat.RowsPerPage) != 1)
	    Lay_ShowErrorAndExit ("Number of rows per page is missing.");
	}

      /****** Get list of selected users ******/
      Usr_GetListSelectedUsrs ();

      /* Check the number of users whose clicks will be shown */
      if (Usr_CountNumUsrsInEncryptedList () == 0)	// If there are no users selected...
	{						// ...write warning message and show the form again
	 Lay_ShowAlert (WARNING,Txt_You_must_select_one_ore_more_users[Gbl.Prefs.Language]);
	 return false;
	}
     }
   else // Gbl.CurrentAct == ActSeeAccGbl
     {
      /***** Get the type of user of clicks *****/
      Par_GetParam (Par_PARAM_SINGLE,"UsrType",UnsignedStr,10);
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         Lay_ShowErrorAndExit ("Type of users is missing.");
      if (UnsignedNum >= Sta_NUM_USR_TYPES_STAT)
         Lay_ShowErrorAndExit ("Type of users is missing.");
      Gbl.Stat.UsrType = (Sta_UsrTypeStat_t) UnsignedNum;
      /* The following types of query will never give a valid result */
      if ((Gbl.Stat.UsrType == Sta_ALL_USRS || Gbl.Stat.UsrType == Sta_UNKNOWN_USRS) &&
          (Gbl.Stat.CountType == Sta_DISTINCT_USRS || Gbl.Stat.CountType == Sta_CLICKS_PER_USR))
        {
         Lay_ShowAlert (WARNING,"No se puede saber cu&aacute;ntos usuarios distintos han accedido sin identificarse.");	// Need translation!
         Usr_UsrDataDestructor (&UsrDat);
         return false;
        }

      /***** Get users range for access statistics *****/
      Sta_GetLocationRange (Sta_LOCATION_RANGE_ALL);
     }

   /***** Check if range of dates is forbidden for me *****/
   NumDays = Dat_GetNumDaysBetweenDates (&Gbl.DateRange.DateIni,&Gbl.DateRange.DateEnd);
   if (!(Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER ||
         (Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER && StatsGlobalOrCourse == STAT_COURSE)))
      if (NumDays > Cfg_DAYS_IN_RECENT_LOG)
        {
         sprintf (Gbl.Message,"El rango de fechas debe ser menor o igual a %u d&iacute;as.",	// Need translation!
                  Cfg_DAYS_IN_RECENT_LOG);
         Lay_ShowAlert (WARNING,Gbl.Message);	// ...write warning message and show the form again
         return false;
        }

   /***** Query depending on the type of count *****/
   switch (Gbl.Stat.CountType)
     {
      case Sta_TOTAL_CLICKS:
         strcpy (StrQueryCountType,"COUNT(*)");
	 break;
      case Sta_DISTINCT_USRS:
         strcpy (StrQueryCountType,"COUNT(DISTINCT(DNI))");
	 break;
      case Sta_CLICKS_PER_USR:
         strcpy (StrQueryCountType,"COUNT(*)/GREATEST(COUNT(DISTINCT(DNI)),1)+0.000000");
	 break;
      case Sta_GENERATION_TIME:
         strcpy (StrQueryCountType,"(AVG(TimeToGenerate)/1E6)+0.000000");
	 break;
      case Sta_SEND_TIME:
         strcpy (StrQueryCountType,"(AVG(TimeToSend)/1E6)+0.000000");
	 break;
     }

   /***** Select clicks from the table of log *****/
   /* Start the query */
   switch (Gbl.Stat.ClicksStatType)
     {
      case ACC_CRS_LISTING:
   	 strcpy (Query,"SELECT DNI,UsrType,DATE_FORMAT(ClickTime,'%Y%m%d%H%i%S') AS F,Action,Comments");
	 break;
      case ACC_CRS_PER_USR:
	 sprintf (Query,"SELECT DNI,%s AS Num",StrQueryCountType);
	 break;
      case ACC_CRS_PER_DAYS:
      case ACC_GBL_PER_DAYS:
         sprintf (Query,"SELECT DATE_FORMAT(ClickTime,'%%Y%%m%%d') AS Day,%s",StrQueryCountType);
	 break;
      case ACC_CRS_PER_DAYS_AND_HOUR:
      case ACC_GBL_PER_DAYS_AND_HOUR:
         sprintf (Query,"SELECT DATE_FORMAT(ClickTime,'%%Y%%m%%d') AS Day,DATE_FORMAT(ClickTime,'%%H') AS Hour,%s",StrQueryCountType);
	 break;
      case ACC_CRS_PER_WEEKS:
      case ACC_GBL_PER_WEEKS:
	 /* With %v the weeks always are counted from monday to sunday.
	    01/01/2006 was sunday => it's counted in the week 52 of 2005, that goes from 26/12/2005 (monday) to 01/01/2006 (sunday).
	    The week 1 of 2006 goes from 02/01/2006 (monday) to 08/01/2006 (sunday) */
         sprintf (Query,"SELECT DATE_FORMAT(ClickTime,'%%x%%v') AS Week,%s",StrQueryCountType);
	 break;
      case ACC_CRS_PER_MONTHS:
      case ACC_GBL_PER_MONTHS:
         sprintf (Query,"SELECT DATE_FORMAT(ClickTime,'%%Y%%m') AS Month,%s",StrQueryCountType);
	 break;
      case ACC_CRS_PER_HOUR:
      case ACC_GBL_PER_HOUR:
         sprintf (Query,"SELECT DATE_FORMAT(ClickTime,'%%H') AS Hour,%s",StrQueryCountType);
	 break;
      case ACC_CRS_PER_MINUTE:
      case ACC_GBL_PER_MINUTE:
         sprintf (Query,"SELECT DATE_FORMAT(ClickTime,'%%H%%i') AS Minute,%s",StrQueryCountType);
	 break;
      case ACC_CRS_PER_ACTION:
      case ACC_GBL_PER_ACTION:
         sprintf (Query,"SELECT Action,%s AS Num",StrQueryCountType);
	 break;
      case ACC_GBL_PER_DEGREE:
         sprintf (Query,"SELECT DegCod,%s AS Num",StrQueryCountType);
	 break;
      case ACC_GBL_PER_COURSE:
	 sprintf (Query,"SELECT CrsCod,%s AS Num",StrQueryCountType);
	 break;
     }
   strcat (Query," FROM ");
   if (LogType == LOG_RECENT)
      strcat (Query,"log_recent");
   else
      strcat (Query,"log");
   strcat (Query," WHERE ClickTime >= '");
   sprintf (StrDate,"%04u%02u%02u",
            Gbl.DateRange.DateIni.Year,
            Gbl.DateRange.DateIni.Month,
            Gbl.DateRange.DateIni.Day);
   strcat (Query,StrDate);
   strcat (Query,"' AND ClickTime <= ");
   sprintf (StrDate,"'%04u%02u%02u235959'",
            Gbl.DateRange.DateEnd.Year,
            Gbl.DateRange.DateEnd.Month,
            Gbl.DateRange.DateEnd.Day);
   strcat (Query,StrDate);

   switch (StatsGlobalOrCourse)
     {
      case STAT_GLOBAL:
	 /* All titulations or only this titulation? */
         if (Gbl.CurrentDeg.Deg.DegCod >= 0 && Gbl.Stat.LocationRange == Sta_LOCATION_RANGE_THIS_DEGREE)
           {
	    sprintf (QueryAux," AND DegCod='%ld'",Gbl.CurrentDeg.Deg.DegCod);
            strcat (Query,QueryAux);
           }
         else if (Gbl.CurrentCrs.Crs.CrsCod >= 0 && Gbl.Stat.LocationRange == Sta_LOCATION_RANGE_THIS_COURSE)
           {
	    sprintf (QueryAux," AND CrsCod='%ld'",Gbl.CurrentCrs.Crs.CrsCod);
            strcat (Query,QueryAux);
           }
         /* Type of users */
	 switch (Gbl.Stat.UsrType)
	   {
	    case Sta_IDENTIFIED_USRS:
               sprintf (StrUsrType," AND UsrType<>'%u'",(unsigned) Usr_UNKNOWN);
	       break;
	    case Sta_ALL_USRS:
               switch (Gbl.Stat.CountType)
                 {
                  case Sta_TOTAL_CLICKS:
                  case Sta_GENERATION_TIME:
                  case Sta_SEND_TIME:
                     StrUsrType[0] = '\0';
	             break;
                  case Sta_DISTINCT_USRS:
                  case Sta_CLICKS_PER_USR:
                     sprintf (StrUsrType," AND UsrType<>'%u'",(unsigned) Usr_UNKNOWN);
                    }
	       break;
	    case Sta_ADMINISTRATORS:
               sprintf (StrUsrType," AND UsrType='%u'",(unsigned) Usr_DEG_ADMIN);
	       break;
	    case Sta_TEACHERS:
               sprintf (StrUsrType," AND UsrType='%u'",(unsigned) Usr_TEACHER);
	       break;
	    case Sta_STUDENTS:
               sprintf (StrUsrType," AND UsrType='%u'",(unsigned) Usr_STUDENT);
	       break;
	    case Sta_INVITED_USRS:
               sprintf (StrUsrType," AND UsrType='%u'",(unsigned) Usr_INVITED);
               break;
	    case Sta_UNKNOWN_USRS:
               sprintf (StrUsrType," AND UsrType='%u'",(unsigned) Usr_UNKNOWN);
               break;
	    case Sta_ME:
               sprintf (StrUsrType," AND DNI='%s'",Gbl.Usrs.Me.UsrDat.DNI);
	       break;
	   }
         strcat (Query,StrUsrType);
	 break;
      case STAT_COURSE:
	 strcat (Query," AND CrsCod='");
         sprintf (CrsCodStr,"%ld",Gbl.CurrentCrs.Crs.CrsCod);
	 strcat (Query,CrsCodStr);
	 strcat (Query,"'");
	 LengthQuery = strlen (Query);
	 NumUsr = 0;
	 Ptr = Gbl.Usrs.Select.All;
	 while (*Ptr)
	   {
	    Str_GetNextStrUntilSeparParamMult (&Ptr,UsrDat.EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64);
            Usr_GetUsrCodFromEncryptedUsrCod (&UsrDat);
            Usr_GetDNIFromUsrCod (&UsrDat);
	    if (UsrDat.DNI[0])
	      {
	       LengthQuery = LengthQuery + 9 + strlen (UsrDat.DNI) + 1;
	       if (LengthQuery > MAX_LENGTH_QUERY_ACCESS - 128)
                  Lay_ShowErrorAndExit ("Query is too large.");
	       strcat (Query,NumUsr ? " OR DNI='" : " AND (DNI='");
	       strcat (Query,UsrDat.DNI);
	       strcat (Query,"'");
	       NumUsr++;
	      }
	   }
	 strcat (Query,")");
	 break;
     }

   /* Select action */
   if (Gbl.Stat.NumAction != ActAll)
     {
      strcat (Query," AND Action='");
      strcat (Query,Actions[Gbl.Stat.NumAction].LogMessage);
      strcat (Query,"'");
     }

   /* End the query */
   switch (Gbl.Stat.ClicksStatType)
     {
      case ACC_CRS_LISTING:
	 strcat (Query," ORDER BY F");
	 break;
      case ACC_CRS_PER_USR:
	 strcat (Query," GROUP BY DNI ORDER BY Num DESC");
	 break;
     case ACC_CRS_PER_DAYS:
     case ACC_GBL_PER_DAYS:
	 strcat (Query," GROUP BY Day DESC");
	 break;
      case ACC_CRS_PER_DAYS_AND_HOUR:
      case ACC_GBL_PER_DAYS_AND_HOUR:
	 strcat (Query," GROUP BY Day DESC,Hour");
	 break;
     case ACC_CRS_PER_WEEKS:
     case ACC_GBL_PER_WEEKS:
	 strcat (Query," GROUP BY Week DESC");
	 break;
     case ACC_CRS_PER_MONTHS:
     case ACC_GBL_PER_MONTHS:
	 strcat (Query," GROUP BY Month DESC");
	 break;
      case ACC_CRS_PER_HOUR:
      case ACC_GBL_PER_HOUR:
	 strcat (Query," GROUP BY Hour");
	 break;
      case ACC_CRS_PER_MINUTE:
      case ACC_GBL_PER_MINUTE:
	 strcat (Query," GROUP BY Minute");
	 break;
      case ACC_CRS_PER_ACTION:
      case ACC_GBL_PER_ACTION:
         strcat (Query," GROUP BY Action ORDER BY Num DESC");
	 break;
      case ACC_GBL_PER_DEGREE:
	 strcat (Query," GROUP BY DegCod ORDER BY Num DESC");
	 break;
      case ACC_GBL_PER_COURSE:
	 strcat (Query," GROUP BY CrsCod ORDER BY Num DESC");
	 break;
     }

   /***** Make the query *****/
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get clicks");

   /***** Write query for debug *****/
/*
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
      Lay_ShowAlert (INFO,Query);
*/

   /***** Count the number of rows in result *****/
   if (NumRows == 0)
      Lay_ShowAlert (INFO,"No hay accesos con el criterio de b&uacute;squeda seleccionado.");	// Need translation!
   else
     {
      if (Gbl.Stat.ClicksStatType == ACC_CRS_LISTING)
        {
         sprintf (Gbl.Message,"Listado de accesos detallados a <strong>%s</strong>",	// Need translation!
                  Gbl.CurrentCrs.Crs.FullName);
         Lay_WriteTitle (Gbl.Message);
         Sta_WriteSelectedRangeOfDates (NumDays);
        }
      else
        {
         switch (Gbl.Stat.ClicksStatType)
           {
            case ACC_CRS_PER_USR:
            case ACC_CRS_PER_DAYS:
            case ACC_CRS_PER_DAYS_AND_HOUR:
            case ACC_CRS_PER_WEEKS:
            case ACC_CRS_PER_MONTHS:
            case ACC_CRS_PER_HOUR:
            case ACC_CRS_PER_MINUTE:
            case ACC_CRS_PER_ACTION:
               sprintf (Gbl.Message,"Estad&iacute;stica de acceso a la asignatura <strong>%s</strong>",	// Need translation!
                        Gbl.CurrentCrs.Crs.FullName);
               Lay_WriteTitle (Gbl.Message);
               Sta_WriteSelectedRangeOfDates (NumDays);
	       break;
            default:
               switch (Gbl.Stat.LocationRange)
                 {
                  case Sta_LOCATION_RANGE_ALL:
                     strcpy (Gbl.Message,"Estad&iacute;stica de todos los accesos");	// Need translation!
                     break;
                  case Sta_LOCATION_RANGE_THIS_DEGREE:
                     sprintf (Gbl.Message,"Estad&iacute;stica de accesos a la titulaci&oacute;n <srong>%s</strong>",	// Need translation!
                              Gbl.CurrentDeg.Deg.ShortName);
                     break;
                  case Sta_LOCATION_RANGE_THIS_COURSE:
                     sprintf (Gbl.Message,"Estad&iacute;stica de accesos a la asignatura <strong>%s</strong>",	// Need translation!
                              Gbl.CurrentCrs.Crs.ShortName);
                     break;
                  default:
                     break;
                 }
               Lay_WriteTitle (Gbl.Message);
               Sta_WriteSelectedRangeOfDates (NumDays);
               if (Gbl.Stat.UsrType == Sta_ME)
                  sprintf (Gbl.Message,"%s: %s",
                           Txt_User[Gbl.Prefs.Language],
                           Gbl.Usrs.Me.UsrDat.FullName);
               else
                  sprintf (Gbl.Message,"%s: %s",
                           Txt_Users[Gbl.Prefs.Language],
                           Txt_USR_TYPE_STATS[Gbl.Stat.UsrType][Gbl.Prefs.Language]);
               Lay_WriteTitle (Gbl.Message);
           }

         sprintf (Gbl.Message,"%s: %s",
                  Txt_Action[Gbl.Prefs.Language],
                  Actions[Gbl.Stat.NumAction].LogMessage);
         Lay_WriteTitle (Gbl.Message);

         sprintf (Gbl.Message,"%s %s, %s %s",
                  Txt_The_graph_shows_the_NUMBER[Gbl.Prefs.Language],
                  Txt_TYPE_COUNT_STAT_SMALL[Gbl.Stat.CountType][Gbl.Prefs.Language],
                  Txt_distributed_by[Gbl.Prefs.Language],
                  Txt_STA_CLICK_STAT_TYPES[Gbl.Stat.ClicksStatType][Gbl.Prefs.Language]);
         Lay_WriteTitle (Gbl.Message);
  	}

      /***** Put the table with the clicks *****/
      /* Write start of table frame */
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,(Gbl.Stat.ClicksStatType == ACC_CRS_LISTING) ? "100%" : NULL);
      switch (Gbl.Stat.ClicksStatType)
	{
	 case ACC_CRS_LISTING:
	    Sta_ShowDetailedAccessesList (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_USR:
	    Sta_ShowNumAccessesPerUsr (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_DAYS:
	 case ACC_GBL_PER_DAYS:
	    Sta_ShowNumAccessesPerDays (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_DAYS_AND_HOUR:
	 case ACC_GBL_PER_DAYS_AND_HOUR:
	    Sta_ShowDistrAccessesPerDaysAndHour (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_WEEKS:
	 case ACC_GBL_PER_WEEKS:
	    Sta_ShowNumAccessesPerWeeks (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_MONTHS:
	 case ACC_GBL_PER_MONTHS:
	    Sta_ShowNumAccessesPerMonths (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_HOUR:
	 case ACC_GBL_PER_HOUR:
	    Sta_ShowNumAccessesPerHour (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_MINUTE:
	 case ACC_GBL_PER_MINUTE:
	    Sta_ShowAverageAccessesPerMinute (NumRows,mysql_res);
	    break;
	 case ACC_CRS_PER_ACTION:
	 case ACC_GBL_PER_ACTION:
	    Sta_ShowNumAccessesPerAction (NumRows,mysql_res);
	    break;
	 case ACC_GBL_PER_DEGREE:
	    Sta_ShowNumAccessesPerDegree (NumRows,mysql_res);
	    break;
	 case ACC_GBL_PER_COURSE:
	    Sta_ShowNumAccessesPerCourse (NumRows,mysql_res);
	    break;
	}

      /* End of frame */
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Free the memory used by the list of users *****/
   if (Gbl.CurrentAct == ActSeeAccCrs)
      Usr_FreeListsEncryptedUsrCods ();

   /***** Free memory used by the data of the user *****/
   Usr_UsrDataDestructor (&UsrDat);

   return true;	// No error
  }

/****************************************************************************/
/****************** Show a listing of detailed clicks ***********************/
/****************************************************************************/

static void Sta_ShowDetailedAccessesList (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_No_INDEX[Txt_NUM_LANGUAGES];
   extern const char *Txt_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type[Txt_NUM_LANGUAGES];
   extern const char *Txt_Date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Action[Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   unsigned long NumRow;
   long	FirstRow,LastRow; // First and last rows to show
   long NumPagesBefore,NumPagesAfter,NumPagsTotal;
   struct UsrData UsrDat;
   MYSQL_ROW row;
   unsigned UsrTypeLog;

   /***** Initialize estructura of data of the user *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** Compute the first and the last row to show *****/
   FirstRow = Gbl.Stat.FirstRow;
   LastRow = Gbl.Stat.LastRow;
   if (FirstRow == 0 && LastRow == 0) // Call from main form
     {
      // Show last clicks
      FirstRow = (NumRows / Gbl.Stat.RowsPerPage - 1) * Gbl.Stat.RowsPerPage + 1;
      if ((FirstRow + Gbl.Stat.RowsPerPage - 1) < NumRows)
	 FirstRow += Gbl.Stat.RowsPerPage;
      LastRow = NumRows;
     }
   if (FirstRow < 1) // For security reasons; really it should never be less than 1
      FirstRow = 1;
   if (LastRow > NumRows)
      LastRow = NumRows;
   if ((LastRow - FirstRow) >= Gbl.Stat.RowsPerPage) // For if there have been clicks that have increased the number of rows
      LastRow = FirstRow + Gbl.Stat.RowsPerPage - 1;

   /***** Compute the number total of pages *****/
   /* Number of pages before the current one */
   NumPagesBefore = (FirstRow-1) / Gbl.Stat.RowsPerPage;
   if (NumPagesBefore * Gbl.Stat.RowsPerPage < (FirstRow-1))
      NumPagesBefore++;
   /* Number of pages after the current one */
   NumPagesAfter = (NumRows - LastRow) / Gbl.Stat.RowsPerPage;
   if (NumPagesAfter * Gbl.Stat.RowsPerPage < (NumRows - LastRow))
      NumPagesAfter++;
   /* Count the total number of pages */
   NumPagsTotal = NumPagesBefore + 1 + NumPagesAfter;

   /***** Put heading with backward and forward buttons *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"7\">");
   fprintf (Gbl.F.Out,"<table width=\"100%%\" cellspacing=\"0\" cellpadding=\"0\">\n");
   fprintf (Gbl.F.Out,"<tr>\n");

   /* Put link to jump to previous page (older clicks) */
   if (FirstRow > 1)
     {
      Act_FormStart (ActSeeAccCrs);
      Sta_WriteParamsDatesSeeAccesses ();
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ClickStatType\" value=\"%u\" />\n",(unsigned) ACC_CRS_LISTING);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"StatAct\" value=\"%u\" />\n",(unsigned) Gbl.Stat.NumAction);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FirstRow\" value=\"%ld\" />\n",FirstRow-Gbl.Stat.RowsPerPage);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"LastRow\" value=\"%ld\" />\n",FirstRow-1);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"RowsPage\" value=\"%ld\" />\n",Gbl.Stat.RowsPerPage);
      Usr_HiddenParUsrCodAll (ActSeeAccCrs,Gbl.Usrs.Select.All);
     }
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"20%%\">\n");
   if (FirstRow > 1)
     {
      sprintf (Gbl.MsgWindowStatus,"Mostrar %ld resultados anteriores",Gbl.Stat.RowsPerPage);		// Need translation!
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,"TIT_TBL");
      fprintf (Gbl.F.Out,"<strong>&lt;Anteriores</B></acronym></a>");	// Need translation!
     }
   fprintf (Gbl.F.Out,"</td>\n");
   if (FirstRow > 1)
      fprintf (Gbl.F.Out,"</form>\n");

   /* Write number of current page */
   fprintf (Gbl.F.Out,"<td align=\"center\" width=\"60%%\" class=\"TIT_TBL\"><strong>" \
                      "Accesos %ld-%ld de %lu (p&aacute;gina %ld de %ld)" \
                      "</strong></td>\n",	// Need translation!
            FirstRow,LastRow,NumRows,NumPagesBefore+1,NumPagsTotal);

   /* Put link to jump to next page (more recent clicks) */
   if (LastRow < NumRows)
     {
      Act_FormStart (ActSeeAccCrs);
      Sta_WriteParamsDatesSeeAccesses ();
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ClickStatType\" value=\"%u\" />\n",(unsigned) ACC_CRS_LISTING);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"StatAct\" value=\"%u\" />\n",(unsigned) Gbl.Stat.NumAction);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FirstRow\" value=\"%ld\" />\n",LastRow+1);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"LastRow\" value=\"%ld\" />\n",LastRow+Gbl.Stat.RowsPerPage);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"RowsPage\" value=\"%ld\" />\n",Gbl.Stat.RowsPerPage);
      Usr_HiddenParUsrCodAll (ActSeeAccCrs,Gbl.Usrs.Select.All);
     }
   fprintf (Gbl.F.Out,"<td align=\"right\" width=\"20%%\">\n");
   if (LastRow < NumRows)
     {
      sprintf (Gbl.MsgWindowStatus,"Mostrar %ld resultados siguientes",Gbl.Stat.RowsPerPage);	// Need translation!
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,"TIT_TBL");
      fprintf (Gbl.F.Out,"<strong>Siguientes&gt;</strong></acronym></a>");	// Need translation!
     }
   fprintf (Gbl.F.Out,"</td>\n");
   if (LastRow < NumRows)
      fprintf (Gbl.F.Out,"</form>\n");

   fprintf (Gbl.F.Out,"</tr>\n");
   fprintf (Gbl.F.Out,"</table>\n");
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">DNI&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" width=\"25%%\" class=\"TIT_TBL\">Observaciones</td></tr>\n",	// Need translation!
            Txt_No_INDEX[Gbl.Prefs.Language],
            Txt_Name[Gbl.Prefs.Language],
            Txt_Type[Gbl.Prefs.Language],
            Txt_Date[Gbl.Prefs.Language],
            Txt_Action[Gbl.Prefs.Language]);

   /***** Write rows back *****/
   for (NumRow = LastRow, Gbl.GlobalRowEvenOdd = 0; NumRow >= FirstRow; NumRow--, Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd)
     {
      mysql_data_seek (mysql_res, (my_ulonglong) (NumRow-1));
      row = mysql_fetch_row (mysql_res);

      strcpy (UsrDat.DNI,row[0]);

      /* Get user's data of the base of data */
      Usr_ChkDNIAndGetAllUsrDataFromDNI (&UsrDat);

      /* Get logged user type */
      if (sscanf (row[1],"%u",&UsrTypeLog) != 1)
	 Lay_ShowErrorAndExit ("Wrong user's type.");

      /* Write the number of row */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%ld&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],NumRow);
      /* Write the DNI si se trata of a student */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],UsrTypeLog == Usr_STUDENT ? UsrDat.DNI : "********");
      /* Write the nombre and the apellidos */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],UsrDat.FullName);
      /* Write the type of user (in row[1] is the number of type of user) */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],(UsrTypeLog < Usr_NUM_USR_TYPES) ? Txt_USR_TYPES_SINGULAR_Abc[UsrTypeLog][UsrDat.Gender][Gbl.Prefs.Language] : "?");
      /* Write the date (in row[2] is the date in formato YYYYMMDDHHMMSS) */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">",Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      Dat_WriteDate (row[2]);
      fprintf (Gbl.F.Out,"&nbsp;");
      Dat_WriteHourMinute (&row[2][8]);
      fprintf (Gbl.F.Out,":%c%c&nbsp;</td>",
               row[2][12],row[2][13]);
      /* Write the action realizada */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],row[3]);
      /* Write las observaciones of the acceso */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s</td></tr>\n",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],row[4] ? row[4] : "&nbsp;");
     }

   /***** Free memory used by the data of the user *****/
   Usr_UsrDataDestructor (&UsrDat);
  }

/****************************************************************************/
/***** Show a listing of with the number of clicks of cada user *****/
/****************************************************************************/

static void Sta_ShowNumAccessesPerUsr (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_No_INDEX[Txt_NUM_LANGUAGES];
   extern const char *Txt_Photo[Txt_NUM_LANGUAGES];
   extern const char *Txt_Name[Txt_NUM_LANGUAGES];
   extern const char *Txt_Type[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   unsigned long NumRow;
   float NumPagesGenerated,MaxPagesGenerated = 0.0;
   unsigned BarWidth;
   struct UsrData UsrDat;
   MYSQL_ROW row;
   char PhotoURL[PATH_MAX+1];
   bool ShowPhoto;

   /***** Initialize estructura of data of the user *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">DNI&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" colspan=\"2\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_No_INDEX[Gbl.Prefs.Language],
            Txt_Photo[Gbl.Prefs.Language],
            Txt_Name[Gbl.Prefs.Language],
            Txt_Type[Gbl.Prefs.Language],
            Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);

   /***** Write rows *****/
   for (NumRow = 1, Gbl.GlobalRowEvenOdd = 0; NumRow <= NumRows; NumRow++, Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd)
     {
      row = mysql_fetch_row (mysql_res);

      strcpy (UsrDat.DNI,row[0]);

      /* Get user's data of the base of data */
      Usr_ChkDNIAndGetAllUsrDataFromDNI (&UsrDat);		// Get of the base of data the data of the user

      /* Write the number of row */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%ld&nbsp;</td>",Gbl.ColorRows[Gbl.GlobalRowEvenOdd],NumRow);
      /* Write the DNI si se trata of a student */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],UsrDat.UsrTypeInDB == Usr_STUDENT ? UsrDat.DNI : "********");
      /* Show the foto */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\">",Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (&UsrDat,PhotoURL);
      Usr_ShowUsrPhoto (&UsrDat,ShowPhoto ? PhotoURL : NULL,12,16,true);
      fprintf (Gbl.F.Out,"</td>");
      /* Write the nombre and the apellidos */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],UsrDat.FullName);
      /* Write the type of user */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">%s&nbsp;</td>",
	       Gbl.ColorRows[Gbl.GlobalRowEvenOdd],Txt_USR_TYPES_SINGULAR_Abc[UsrDat.UsrTypeInDB][UsrDat.Gender][Gbl.Prefs.Language]);
      /* Write the number of clicks */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      if (NumRow == 1)
	 MaxPagesGenerated = NumPagesGenerated;
      if (MaxPagesGenerated > 0.0)
        {
         BarWidth = (unsigned) (((NumPagesGenerated * 300.0) / MaxPagesGenerated) + 0.5);
         if (BarWidth == 0)
            BarWidth = 1;
        }
      else
         BarWidth = 0;
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" bgcolor=\"#%s\" class=\"LOG\">",Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      if (BarWidth)
	 fprintf (Gbl.F.Out,"<img src=\"%s/%c1x14.gif\" width=\"%u\" height=\"14\" align=\"top\" alt=\"\" />&nbsp;",
		  Gbl.Prefs.IconsURL,UsrDat.UsrTypeInDB == Usr_STUDENT ? 'c' : 'v',BarWidth);
      Sta_WriteFloatNum (NumPagesGenerated);
      fprintf (Gbl.F.Out,"&nbsp;</td></tr>\n");
     }

   /***** Free memory used by the data of the user *****/
   Usr_UsrDataDestructor (&UsrDat);
  }

/****************************************************************************/
/****** Show a listing of with the number of clicks in cada date ******/
/****************************************************************************/

static void Sta_ShowNumAccessesPerDays (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_Date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Day[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   extern const char *Txt_DAYS_SMALL[7][Txt_NUM_LANGUAGES];
   unsigned long NumRow;
   struct Date ReadDate,LastDate,Date;
   unsigned D,NumDaysFromLastDateToCurrDate,NumDayWeek;
   float NumPagesGenerated,MaxPagesGenerated = 0.0,TotalPagesGenerated = 0.0;
   MYSQL_ROW row;

   /***** Initialize LastDate *****/
   Dat_AssignDate (&LastDate,&(Gbl.DateRange.DateEnd));

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_Date[Gbl.Prefs.Language],
            Txt_Day[Gbl.Prefs.Language],
            Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);

   /***** Compute maximum number of pages generated per day *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      if (NumPagesGenerated > MaxPagesGenerated)
	 MaxPagesGenerated = NumPagesGenerated;
      TotalPagesGenerated += NumPagesGenerated;
     }

   /***** Write rows beginning by the most recent day and ending by the oldest *****/
   mysql_data_seek (mysql_res,0);
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get year, month and day (row[0] holds the date in YYYYMMDD format) */
      if (!(Dat_GetDateFromYYYYMMDD (&ReadDate,row[0])))
	 Lay_ShowErrorAndExit ("Wrong date.");

      /* Get number of pages generated (in row[1]) */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);

      Dat_AssignDate (&Date,&LastDate);
      NumDaysFromLastDateToCurrDate = Dat_GetNumDaysBetweenDates (&ReadDate,&LastDate);
      /* In the next loop (NumDaysFromLastDateToCurrDate-1) das (the more recent) with 0 clicks are shown
         and a last day (the oldest) with NumPagesGenerated */
      for (D=1; D<=NumDaysFromLastDateToCurrDate; D++)
        {
         NumDayWeek = Dat_GetDayOfWeek (Date.Year,Date.Month,Date.Day);

         /* Write the date */
         fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%02u/%02u/%02u&nbsp;</td>",
	          (NumDayWeek == 6) ? "LOG_R" : "LOG",Date.Day,Date.Month,Date.Year % 100);

         /* Write the day of the week */
         fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">%s&nbsp;</td>",
                  (NumDayWeek == 6) ? "LOG_R" : "LOG",Txt_DAYS_SMALL[NumDayWeek][Gbl.Prefs.Language]);
         /* Draw bar proportional to number of pages generated */
         Sta_DrawBarNumAccesses ((NumDayWeek == 6) ? 'r' : 'c',(D == NumDaysFromLastDateToCurrDate) ? NumPagesGenerated : 0.0,MaxPagesGenerated,TotalPagesGenerated,400);

         /* Decrease day */
         Dat_GetDayBefore (&Date,&Date);
        }
      Dat_AssignDate (&LastDate,&Date);
     }
   NumDaysFromLastDateToCurrDate = Dat_GetNumDaysBetweenDates (&Gbl.DateRange.DateIni,&LastDate);

   /***** Finally NumDaysFromLastDateToCurrDate days are shown with 0 clicks
          (the oldest days from the requested initial day until the first with clicks) *****/
   for (D=1; D<=NumDaysFromLastDateToCurrDate; D++)
     {
      NumDayWeek = Dat_GetDayOfWeek (Date.Year,Date.Month,Date.Day);

      /* Write the date */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%02u/%02u/%02u&nbsp;</td>",
               (NumDayWeek == 6) ? "LOG_R" : "LOG",Date.Day,Date.Month,Date.Year % 100);

      /* Write the day of the week */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">%s&nbsp;</td>",
               (NumDayWeek == 6) ? "LOG_R" : "LOG",Txt_DAYS_SMALL[NumDayWeek][Gbl.Prefs.Language]);

      /* Draw bar proportional to number of pages generated */
      Sta_DrawBarNumAccesses ((NumDayWeek == 6) ? 'r' : 'c',0.0,MaxPagesGenerated,TotalPagesGenerated,400);

      /* Decrease day */
      Dat_GetDayBefore (&Date,&Date);
     }
  }

/****************************************************************************/
/**************** Show graphic of number of pages generated per hour ***************/
/****************************************************************************/

#define GRAPH_DISTRIBUTION_PER_HOUR_HOUR_WIDTH 20
#define GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH (GRAPH_DISTRIBUTION_PER_HOUR_HOUR_WIDTH*24)

static void Sta_ShowDistrAccessesPerDaysAndHour (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Color_of_the_graphic[Txt_NUM_LANGUAGES];
   extern const char *Txt_STAT_COLOR_TYPES[Sta_NUM_COLOR_TYPES][Txt_NUM_LANGUAGES];
   extern const char *Txt_Date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Day[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   extern const char *Txt_DAYS_SMALL[7][Txt_NUM_LANGUAGES];
   Sta_ColorType_t ColorType,SelectedColorType;
   unsigned long NumRow;
   struct Date PreviousReadDate,CurrentReadDate,LastDate,Date;
   unsigned D,NumDaysFromLastDateToCurrDate=1,NumDayWeek;
   unsigned Hour,ReadHour=0;
   float NumPagesGenerated,MaxPagesGenerated = 0.0;
   float NumAccPerHour[24],NumAccPerHourZero[24];
   MYSQL_ROW row;

   /***** Get selected color type *****/
   SelectedColorType = Sta_GetStatColorType ();

   /***** Put a selector for the type of color *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" colspan=\"26\" class=\"%s\">",
            The_ClassFormul[Gbl.Prefs.Theme]);

   Act_FormStart (Gbl.CurrentAct);
   Sta_WriteParamsDatesSeeAccesses ();
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ClickStatType\" value=\"%u\" />\n",
            (unsigned) Gbl.Stat.ClicksStatType);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CountType\" value=\"%u\" />\n",
            (unsigned) Gbl.Stat.CountType);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"StatAct\" value=\"%u\" />\n",
            (unsigned) Gbl.Stat.NumAction);
   if (Gbl.CurrentAct == ActSeeAccCrs)
      Usr_HiddenParUsrCodAll (ActSeeAccCrs,Gbl.Usrs.Select.All);
   else // Gbl.CurrentAct == ActSeeAccGbl
     {
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"UsrType\" value=\"%u\" />\n",
               (unsigned) Gbl.Stat.UsrType);
      fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"LocationRange\" value=\"%u\" />",
               (unsigned) Gbl.Stat.LocationRange);
     }

   fprintf (Gbl.F.Out,"%s: ",Txt_Color_of_the_graphic[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<select name=\"ColorType\" onchange=\"javascript:document.forms[%d].submit();\">",
            Gbl.NumForm);
   for (ColorType=(Sta_ColorType_t) 0; ColorType<Sta_NUM_COLOR_TYPES; ColorType++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) ColorType);
      if (ColorType == SelectedColorType)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s",Txt_STAT_COLOR_TYPES[ColorType][Gbl.Prefs.Language]);
     }
   fprintf (Gbl.F.Out,"</select></form></td></tr>\n");

   /***** Compute maximum number of pages generated per day-hour *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[2]);
      if (NumPagesGenerated > MaxPagesGenerated)
	 MaxPagesGenerated = NumPagesGenerated;
     }

   /***** Initialize LastDate *****/
   Dat_AssignDate (&LastDate,&(Gbl.DateRange.DateEnd));

   /***** Reset number of pages generated per hour *****/
   for (Hour=0; Hour<24; Hour++)
      NumAccPerHour[Hour] = NumAccPerHourZero[Hour] = 0.0;

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" valign=\"top\" rowspan=\"3\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"center\" valign=\"top\" rowspan=\"3\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" colspan=\"24\" width=\"%u\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_Date[Gbl.Prefs.Language],
            Txt_Day[Gbl.Prefs.Language],
            GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH,Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"24\" width=\"%u\">",GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH);
   Sta_DrawBarColors (SelectedColorType,MaxPagesGenerated);
   fprintf (Gbl.F.Out,"</td></tr><tr>");
   for (Hour=0; Hour<24; Hour++)
      fprintf (Gbl.F.Out,"<td width=\"%u\" align=\"center\" valign=\"top\" class=\"LOG\">%02uh</td>",
               GRAPH_DISTRIBUTION_PER_HOUR_HOUR_WIDTH,Hour);
   fprintf (Gbl.F.Out,"</tr>");

   /***** Write rows comenzando por the day ms reciente and terminando por the ms antiguo *****/
   mysql_data_seek (mysql_res,0);

   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get year, month and day (row[0] holds the date in YYYYMMDD format) */
      if (!(Dat_GetDateFromYYYYMMDD (&CurrentReadDate,row[0])))
	 Lay_ShowErrorAndExit ("Wrong date.");

      /* Get the hour (in row[1] is the hour in formato HH) */
      if (sscanf (row[1],"%02u",&ReadHour) != 1)
	 Lay_ShowErrorAndExit ("Wrong hour.");

      /* Get number of pages generated (in row[2]) */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[2]);

      /* If this is the first read date, initialize PreviousReadDate */
      if (NumRow == 1)
         Dat_AssignDate (&PreviousReadDate,&CurrentReadDate);

      /* Update number of pages generated per hour */
      if (PreviousReadDate.Year  != CurrentReadDate.Year  ||
          PreviousReadDate.Month != CurrentReadDate.Month ||
          PreviousReadDate.Day   != CurrentReadDate.Day)	// Current read date (CurrentReadDate) is older than previous read date (PreviousReadDate) */
        {
         /* In the next loop we show (NumDaysFromLastDateToCurrDate-1) days (the menos antiguos) with 0 clicks
            and a last day (older) with NumPagesGenerated */
         Dat_AssignDate (&Date,&LastDate);
         NumDaysFromLastDateToCurrDate = Dat_GetNumDaysBetweenDates (&PreviousReadDate,&LastDate);
         for (D=1; D<=NumDaysFromLastDateToCurrDate; D++)
           {
            NumDayWeek = Dat_GetDayOfWeek (Date.Year,Date.Month,Date.Day);

            /* Write the date */
            fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%02u/%02u/%02u&nbsp;</td>",
	             (NumDayWeek == 6) ? "LOG_R" : "LOG",Date.Day,Date.Month,Date.Year % 100);

            /* Write the day of the week */
            fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">%s&nbsp;</td>",
                     (NumDayWeek == 6) ? "LOG_R" : "LOG",Txt_DAYS_SMALL[NumDayWeek][Gbl.Prefs.Language]);

            /* Draw a cell with the color proportional to the number of clicks */
            if (D == NumDaysFromLastDateToCurrDate)
               Sta_DrawAccessesPerHourForADay (SelectedColorType,NumAccPerHour,MaxPagesGenerated);
            else	// D < NumDaysFromLastDateToCurrDate
               Sta_DrawAccessesPerHourForADay (SelectedColorType,NumAccPerHourZero,MaxPagesGenerated);
            fprintf (Gbl.F.Out,"</tr>\n");

            /* Decrease day */
            Dat_GetDayBefore (&Date,&Date);
           }
         Dat_AssignDate (&LastDate,&Date);
         Dat_AssignDate (&PreviousReadDate,&CurrentReadDate);

         /* Reset number of pages generated per hour */
         for (Hour=0; Hour<24; Hour++)
            NumAccPerHour[Hour] = 0.0;
        }
      NumAccPerHour[ReadHour] = NumPagesGenerated;
     }

   /***** Show the clicks of the oldest day with clicks *****/
   /* In the next loop we show (NumDaysFromLastDateToCurrDate-1) days (more recent) with 0 clicks
      and a last day (older) with NumPagesGenerated clicks */
   Dat_AssignDate (&Date,&LastDate);
   NumDaysFromLastDateToCurrDate = Dat_GetNumDaysBetweenDates (&PreviousReadDate,&LastDate);
   for (D=1; D<=NumDaysFromLastDateToCurrDate; D++)
     {
      NumDayWeek = Dat_GetDayOfWeek (Date.Year,Date.Month,Date.Day);

      /* Write the date */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%02u/%02u/%02u&nbsp;</td>",
               (NumDayWeek == 6) ? "LOG_R" : "LOG",Date.Day,Date.Month,Date.Year % 100);

      /* Write the day of the week */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">%s&nbsp;</td>",
               (NumDayWeek == 6) ? "LOG_R" : "LOG",Txt_DAYS_SMALL[NumDayWeek][Gbl.Prefs.Language]);

      /* Draw the color proporcional al number of clicks */
      if (D == NumDaysFromLastDateToCurrDate)
         Sta_DrawAccessesPerHourForADay (SelectedColorType,NumAccPerHour,MaxPagesGenerated);
      else	// D < NumDaysFromLastDateToCurrDate
         Sta_DrawAccessesPerHourForADay (SelectedColorType,NumAccPerHourZero,MaxPagesGenerated);
      fprintf (Gbl.F.Out,"</tr>\n");

      /* Decrease day */
      Dat_GetDayBefore (&Date,&Date);
     }

   /***** Finally se muestran NumDaysFromLastDateToCurrDate das with 0 clicks
          (the oldest days since the initial day requested by the user until the first with clicks) *****/
   Dat_AssignDate (&LastDate,&Date);
   NumDaysFromLastDateToCurrDate = Dat_GetNumDaysBetweenDates (&Gbl.DateRange.DateIni,&LastDate);
   for (D=1; D<=NumDaysFromLastDateToCurrDate; D++)
     {
      NumDayWeek = Dat_GetDayOfWeek (Date.Year,Date.Month,Date.Day);

      /* Write the date */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"%s\">%02u/%02u/%02u&nbsp;</td>",
               (NumDayWeek == 6) ? "LOG_R" : "LOG",Date.Day,Date.Month,Date.Year % 100);

      /* Write the day of the week */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"%s\">%s&nbsp;</td>",
               (NumDayWeek == 6) ? "LOG_R" : "LOG",Txt_DAYS_SMALL[NumDayWeek][Gbl.Prefs.Language]);

      /* Draw the color proporcional to number of clicks */
      Sta_DrawAccessesPerHourForADay (SelectedColorType,NumAccPerHourZero,MaxPagesGenerated);
      fprintf (Gbl.F.Out,"</tr>\n");

      /* Decrease day */
      Dat_GetDayBefore (&Date,&Date);
     }
  }

/****************************************************************************/
/********************** Get type of color for statistics ********************/
/****************************************************************************/

static Sta_ColorType_t Sta_GetStatColorType (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"ColorType",UnsignedStr,10);
   if (UnsignedStr[0])
     {
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         Lay_ShowErrorAndExit ("Type of color is missing.");
      if (UnsignedNum >= Sta_NUM_COLOR_TYPES)
         Lay_ShowErrorAndExit ("Type of color is missing.");
      return (Sta_ColorType_t) UnsignedNum;
     }
   return (Sta_ColorType_t) 0;
  }

/****************************************************************************/
/************************* Draw a bar with colors ***************************/
/****************************************************************************/

static void Sta_DrawBarColors (Sta_ColorType_t ColorType,float MaxPagesGenerated)
  {
   unsigned Interval,NumColor,R,G,B;

   /***** Write numbers from 0 to MaxPagesGenerated *****/
   fprintf (Gbl.F.Out,"<table width=\"100%%\" cellspacing=\"0\" cellpadding=\"0\">");
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"bottom\" width=\"%u\" colspan=\"%u\" class=\"LOG\">0</td>",
            (GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH/5)/2,(GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH/5)/2);
   for (Interval=1; Interval<=4; Interval++)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"bottom\" width=\"%u\" colspan=\"%u\" class=\"LOG\">",
               GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH/5,GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH/5);
      Sta_WriteFloatNum ((float) Interval * MaxPagesGenerated / 5.0);
      fprintf (Gbl.F.Out,"</td>");
     }
   fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"bottom\" width=\"%u\" colspan=\"%u\" class=\"LOG\">",
            (GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH/5)/2,(GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH/5)/2);
   Sta_WriteFloatNum (MaxPagesGenerated);
   fprintf (Gbl.F.Out,"</td></tr><tr>");

   /***** Draw colors *****/
   for (NumColor=0; NumColor<GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH; NumColor++)
     {
      Sta_SetColor (ColorType,(float) NumColor,(float) GRAPH_DISTRIBUTION_PER_HOUR_TOTAL_WIDTH,&R,&G,&B);
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"1\" bgcolor=\"#%02X%02X%02X\"><img src=\"%s/tr1x14.gif\" alt=\"\" /></td>",
               R,G,B,Gbl.Prefs.IconsURL);
     }
   fprintf (Gbl.F.Out,"</tr></table>");
  }

/****************************************************************************/
/********************* Draw accesses per hour for a day *********************/
/****************************************************************************/

static void Sta_DrawAccessesPerHourForADay (Sta_ColorType_t ColorType,float NumPagesGenerated[24],float MaxPagesGenerated)
  {
   unsigned Hour,R,G,B;

   for (Hour=0; Hour<24; Hour++)
     {
      Sta_SetColor (ColorType,NumPagesGenerated[Hour],MaxPagesGenerated,&R,&G,&B);
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u\" bgcolor=\"#%02X%02X%02X\">" \
                         "<acronym class=\"LOG\" title=\"",
               GRAPH_DISTRIBUTION_PER_HOUR_HOUR_WIDTH,R,G,B);
      Sta_WriteFloatNum (NumPagesGenerated[Hour]);
      fprintf (Gbl.F.Out,"\"><img src=\"%s/tr20x14.gif\" width=\"%u\" height=\"14\" alt=\"",
               Gbl.Prefs.IconsURL,GRAPH_DISTRIBUTION_PER_HOUR_HOUR_WIDTH);
      Sta_WriteFloatNum (NumPagesGenerated[Hour]);
      fprintf (Gbl.F.Out,"\" /></acronym></td>");
     }
  }

/****************************************************************************/
/************************* Set color depending on ratio *********************/
/****************************************************************************/
// MaxPagesGenerated must be > 0
/*
Black         Blue         Cyan        Green        Yellow        Red
  +------------+------------+------------+------------+------------+
  |     0.2    |     0.2    |     0.2    |     0.2    |     0.2    |
  +------------+------------+------------+------------+------------+
 0.0          0.2          0.4          0.6          0.8          1.0
*/

static void Sta_SetColor (Sta_ColorType_t ColorType,float NumPagesGenerated,float MaxPagesGenerated,unsigned *R,unsigned *G,unsigned *B)
  {
   float Result = (NumPagesGenerated / MaxPagesGenerated);

   switch (ColorType)
     {
      case Sta_COLOR:
         if (Result < 0.2)		// Black -> Blue
           {
            *R = 0;
            *G = 0;
            *B = (unsigned) (Result * 256.0 / 0.2 + 0.5);
            if (*B == 256)
               *B = 255;
           }
         else if (Result < 0.4)	// Blue -> Cyan
           {
            *R = 0;
            *G = (unsigned) ((Result-0.2) * 256.0 / 0.2 + 0.5);
            if (*G == 256)
               *G = 255;
            *B = 255;
           }
         else if (Result < 0.6)	// Cyan -> Green
           {
            *R = 0;
            *G = 255;
            *B = 256 - (unsigned) ((Result-0.4) * 256.0 / 0.2 + 0.5);
            if (*B == 256)
               *B = 255;
           }
         else if (Result < 0.8)	// Green -> Yellow
           {
            *R = (unsigned) ((Result-0.6) * 256.0 / 0.2 + 0.5);
            if (*R == 256)
               *R = 255;
            *G = 255;
            *B = 0;
           }
         else			// Yellow -> Red
           {
            *R = 255;
            *G = 256 - (unsigned) ((Result-0.8) * 256.0 / 0.2 + 0.5);
            if (*G == 256)
               *G = 255;
            *B = 0;
           }
         break;
      case Sta_BLACK_TO_WHITE:
         *B = (unsigned) (Result * 256.0 + 0.5);
         if (*B == 256)
            *B = 255;
         *R = *G = *B;
         break;
      case Sta_WHITE_TO_BLACK:
         *B = 256 - (unsigned) (Result * 256.0 + 0.5);
         if (*B == 256)
            *B = 255;
         *R = *G = *B;
     }
  }

/****************************************************************************/
/********** Show listing with number of pages generated per week ************/
/****************************************************************************/

static void Sta_ShowNumAccessesPerWeeks (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_Week[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   unsigned long NumRow;
   struct Date ReadDate,LastDate,Date;
   unsigned W,NumWeeksBetweenLastDateAndCurrentDate;
   float NumPagesGenerated,MaxPagesGenerated = 0.0,TotalPagesGenerated = 0.0;
   MYSQL_ROW row;

   /***** Initialize LastDate to avoid warning *****/
   Dat_CalculateWeekOfYear (&Gbl.DateRange.DateEnd);	// Changes Week and Year
   Dat_AssignDate (&LastDate,&(Gbl.DateRange.DateEnd));

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_Week[Gbl.Prefs.Language],
            Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);

   /***** Compute maximum number of pages generated per week *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      if (NumPagesGenerated > MaxPagesGenerated)
	 MaxPagesGenerated = NumPagesGenerated;
      TotalPagesGenerated += NumPagesGenerated;
     }

   /***** Write rows *****/
   mysql_data_seek (mysql_res,0);
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get year and week (row[0] holds date in YYYYWW format) */
      if (sscanf (row[0],"%04u%02u",&ReadDate.Year,&ReadDate.Week) != 2)
	 Lay_ShowErrorAndExit ("Wrong date.");

      /* Get number of pages generated (in row[1]) */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);

      Dat_AssignDate (&Date,&LastDate);
      NumWeeksBetweenLastDateAndCurrentDate = Dat_GetNumWeeksBetweenDates (&ReadDate,&LastDate);
      for (W=1; W<=NumWeeksBetweenLastDateAndCurrentDate; W++)
        {
         /* Write week */
         fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"LOG\">%02u/%02u&nbsp;</td>",
	          Date.Week,Date.Year % 100);

         /* Draw bar proportional to number of pages generated */
         Sta_DrawBarNumAccesses ('c',(W == NumWeeksBetweenLastDateAndCurrentDate) ? NumPagesGenerated : 0.0,MaxPagesGenerated,TotalPagesGenerated,400);

         /* Decrement week */
         Dat_GetWeekBefore (&Date,&Date);
        }
      Dat_AssignDate (&LastDate,&Date);
     }

  /***** Finally, show the old weeks without pages generated *****/
  Dat_CalculateWeekOfYear (&Gbl.DateRange.DateIni);	// Changes Week and Year
  NumWeeksBetweenLastDateAndCurrentDate = Dat_GetNumWeeksBetweenDates (&Gbl.DateRange.DateIni,&LastDate);
  for (W=1; W<=NumWeeksBetweenLastDateAndCurrentDate; W++)
    {
     /* Write week */
     fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"LOG\">%02u/%02u&nbsp;</td>",
              Date.Week,Date.Year % 100);

     /* Draw bar proportional to number of pages generated */
     Sta_DrawBarNumAccesses ('c',0.0,MaxPagesGenerated,TotalPagesGenerated,400);

     /* Decrement week */
     Dat_GetWeekBefore (&Date,&Date);
    }
  }

/****************************************************************************/
/********** Show a graph with the number of clicks in each month ************/
/****************************************************************************/

static void Sta_ShowNumAccessesPerMonths (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_Month[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   unsigned long NumRow;
   struct Date ReadDate,LastDate,Date;
   unsigned M,NumMesesEntreLastDateYAct;
   float NumPagesGenerated,MaxPagesGenerated = 0.0,TotalPagesGenerated = 0.0;
   MYSQL_ROW row;

   /***** Initialize LastDate *****/
   Dat_AssignDate (&LastDate,&(Gbl.DateRange.DateEnd));

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_Month[Gbl.Prefs.Language],
            Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);

   /***** Compute maximum number of pages generated per month *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      if (NumPagesGenerated > MaxPagesGenerated)
	 MaxPagesGenerated = NumPagesGenerated;
      TotalPagesGenerated += NumPagesGenerated;
     }

   /***** Write rows *****/
   mysql_data_seek (mysql_res,0);
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get the year and the month (in row[0] is the date in YYYYMM format) */
      if (sscanf (row[0],"%04u%02u",&ReadDate.Year,&ReadDate.Month) != 2)
	 Lay_ShowErrorAndExit ("Wrong date.");

      /* Get number of pages generated (in row[1]) */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);

      Dat_AssignDate (&Date,&LastDate);
      NumMesesEntreLastDateYAct = Dat_GetNumMonthsBetweenDates (&ReadDate,&LastDate);
      for (M=1; M<=NumMesesEntreLastDateYAct; M++)
        {
         /* Write the month */
         fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"LOG\">%02u/%02u&nbsp;</td>",
	          Date.Month,Date.Year % 100);

         /* Draw bar proportional to number of pages generated */
         Sta_DrawBarNumAccesses ('c',(M == NumMesesEntreLastDateYAct) ? NumPagesGenerated : 0.0,MaxPagesGenerated,TotalPagesGenerated,400);

         /* Decrease month */
         Dat_GetMonthBefore (&Date,&Date);
        }
      Dat_AssignDate (&LastDate,&Date);
     }

  /***** Finally, show the oldest months without clicks *****/
  NumMesesEntreLastDateYAct = Dat_GetNumMonthsBetweenDates (&Gbl.DateRange.DateIni,&LastDate);
  for (M=1; M<=NumMesesEntreLastDateYAct; M++)
    {
     /* Write the month */
     fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"LOG\">%02u/%02u&nbsp;</td>",
              Date.Month,Date.Year % 100);

     /* Draw bar proportional to number of pages generated */
     Sta_DrawBarNumAccesses ('c',0.0,MaxPagesGenerated,TotalPagesGenerated,400);

     /* Decrease month */
     Dat_GetMonthBefore (&Date,&Date);
    }
  }

/****************************************************************************/
/**************** Show graphic of number of pages generated per hour ***************/
/****************************************************************************/

#define DIGIT_WIDTH 5

static void Sta_ShowNumAccessesPerHour (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   unsigned long NumRow;
   float NumPagesGenerated,MaxPagesGenerated = 0.0,TotalPagesGenerated = 0.0;
   unsigned NumDays;
   unsigned Hour=0,ReadHour=0,H;
   unsigned NumDigits,ColumnWidth;
   MYSQL_ROW row;

   if ((NumDays = Dat_GetNumDaysBetweenDates (&Gbl.DateRange.DateIni,&Gbl.DateRange.DateEnd)) == 0)
      return;

   /***** Compute maximum number of pages generated per hour *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]) / (float) NumDays;
      if (NumPagesGenerated > MaxPagesGenerated)
	 MaxPagesGenerated = NumPagesGenerated;
      TotalPagesGenerated += NumPagesGenerated;
     }

   /***** Compute width of columns (one for each hour) *****/
   /* Maximum number of dgits. If less than 4, set it to 4 to ensure a minimum width */
   NumDigits = (MaxPagesGenerated >= 1000) ? (unsigned) floor (log10 ((double) MaxPagesGenerated)) + 1 : 4;
   ColumnWidth = NumDigits * DIGIT_WIDTH + 2;

   /***** Draw the graphic *****/
   mysql_data_seek (mysql_res, 0);
   NumRow = 1;
   fprintf (Gbl.F.Out,"<tr>");
   while (Hour < 24)
     {
      if (NumRow <= NumRows)	// If not read yet all the results of the query
	{
	 row = mysql_fetch_row (mysql_res); // Get next result
	 NumRow++;
	 if (sscanf (row[0],"%02u",&ReadHour) != 1)   // In row[0] is the date in HH format
	    Lay_ShowErrorAndExit ("Wrong hour.");
         NumPagesGenerated = Str_GetFloatNumFromStr (row[1]) / (float) NumDays;
	 for (H = Hour; H < ReadHour; H++, Hour++)
	    Sta_WriteAccessHour (H,0.0,MaxPagesGenerated,TotalPagesGenerated,ColumnWidth);
	 Sta_WriteAccessHour (ReadHour,NumPagesGenerated,MaxPagesGenerated,TotalPagesGenerated,ColumnWidth);
	 Hour++;
	}
      else
	 for (H = ReadHour+1; H < 24; H++, Hour++)
	    Sta_WriteAccessHour (H,0.0,MaxPagesGenerated,TotalPagesGenerated,ColumnWidth);
     }
   fprintf (Gbl.F.Out,"</tr>");
  }

/****************************************************************************/
/**** Write a column of the graphic of the number of clicks in each hour ****/
/****************************************************************************/

static void Sta_WriteAccessHour (unsigned Hour,float NumPagesGenerated,float MaxPagesGenerated,float TotalPagesGenerated,unsigned ColumnWidth)
  {
   unsigned AltoBarra;

   fprintf (Gbl.F.Out,"<td width=\"%u\" align=\"center\" valign=\"bottom\" class=\"DAT_SMALL\">",ColumnWidth);

   /* Draw bar with a height porportional to the number of clicks */
   if (NumPagesGenerated > 0.0)
     {
      fprintf (Gbl.F.Out,"%u%%<br />",(unsigned) (((NumPagesGenerated * 100.0) / TotalPagesGenerated) + 0.5));
      Sta_WriteFloatNum (NumPagesGenerated);
      fprintf (Gbl.F.Out,"<br />");
      AltoBarra = (unsigned) (((NumPagesGenerated * 400.0) / MaxPagesGenerated) + 0.5);
      if (AltoBarra == 0)
         AltoBarra = 1;
      fprintf (Gbl.F.Out,"<img src=\"%s/c8x1.gif\" width=\"8\" height=\"%u\" alt=\"\" /><br />",Gbl.Prefs.IconsURL,AltoBarra);
     }
   else
      fprintf (Gbl.F.Out,"0%%<br />0<br />");

   /* Write the hour */
   fprintf (Gbl.F.Out,"%uh</td>",Hour);
  }

/****************************************************************************/
/**** Show a listing with the number of clicks in every minute of the day ***/
/****************************************************************************/

#define NUM_MINUTES_PER_DAY		(60*24)			// 1440 minutes in a day
#define WIDTH_SEMIDIVISION_GRAPHIC	24
#define NUM_DIVISIONS_X		10

static void Sta_ShowAverageAccessesPerMinute (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   unsigned long NumRow=1;
   MYSQL_ROW row;
   unsigned NumDays;
   unsigned MinuteDay=0,ReadHour,MinuteRead,MinuteDayRead=0,i;
   float NumPagesGenerated;
   float NumClicksPerMin[NUM_MINUTES_PER_DAY],MaxPagesGenerated = 0.0,
         Power10LeastOrEqual,MaxX,IncX;
   char *Format;

   if ((NumDays = Dat_GetNumDaysBetweenDates (&Gbl.DateRange.DateIni,&Gbl.DateRange.DateEnd)) == 0)
      return;

   /***** Compute number of clicks (and mximo) in every minute *****/
   while (MinuteDay < NUM_MINUTES_PER_DAY)
     {
      if (NumRow <= NumRows)	// If not all the result of the query are yet read
	{
	 row = mysql_fetch_row (mysql_res); // Get next result
	 NumRow++;
	 if (sscanf (row[0],"%02u%02u",&ReadHour,&MinuteRead) != 2)   // In row[0] is the date in formato HHMM
	    Lay_ShowErrorAndExit ("Wrong hour-minute.");
         /* Get number of pages generated */
         NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
         MinuteDayRead = ReadHour*60 + MinuteRead;
	 for (i = MinuteDay; i < MinuteDayRead; i++, MinuteDay++)
            NumClicksPerMin[i] = 0.0;
         NumClicksPerMin[MinuteDayRead] = NumPagesGenerated / (float) NumDays;
         if (NumClicksPerMin[MinuteDayRead] > MaxPagesGenerated)
	    MaxPagesGenerated = NumClicksPerMin[MinuteDayRead];
	 MinuteDay++;
	}
      else
	 for (i = MinuteDayRead+1; i < NUM_MINUTES_PER_DAY; i++, MinuteDay++)
            NumClicksPerMin[i] = 0.0;
     }

   /***** Compute the mximo valor of X and the incremento of the X axis *****/
   if (MaxPagesGenerated <= 0.000001)
      MaxX = 0.000001;
   else
     {
      Power10LeastOrEqual = (float) pow (10.0,floor (log10 ((double) MaxPagesGenerated)));
      MaxX = ceil (MaxPagesGenerated / Power10LeastOrEqual) * Power10LeastOrEqual;
     }
   IncX = MaxX / (float) NUM_DIVISIONS_X;
   if (IncX >= 1.0)
      Format = "%.0f";
   else if (IncX >= 0.1)
      Format = "%.1f";
   else if (IncX >= 0.01)
      Format = "%.2f";
   else if (IncX >= 0.001)
      Format = "%.3f";
   else
      Format = "%f";

   /***** Rtulos of the X axis *****/
   Sta_WriteLabelsXAxisAccMin (IncX,Format);

   /***** Eje Y and graphic *****/
   for (i=0; i<NUM_MINUTES_PER_DAY; i++)
      Sta_WriteAccessMinute (i,NumClicksPerMin[i],MaxX);

   /***** Eje X *****/
   /* First division (left) */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" width=\"%u\"><img src=\"%s/ejexizq24x1.gif\" width=\"%u\" height=\"1\" alt=\"\" /></td>",
            WIDTH_SEMIDIVISION_GRAPHIC,Gbl.Prefs.IconsURL,WIDTH_SEMIDIVISION_GRAPHIC);
   /* All the intermediate divisions */
   for (i=0; i<NUM_DIVISIONS_X*2; i++)
      fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u\"><img src=\"%s/ejex24x1.gif\" width=\"%u\" height=\"1\" alt=\"\" /></td>",
	       WIDTH_SEMIDIVISION_GRAPHIC,Gbl.Prefs.IconsURL,WIDTH_SEMIDIVISION_GRAPHIC);
   /* Last division (right) */
   fprintf (Gbl.F.Out,"<td align=\"left\" width=\"%u\"><img src=\"%s/tr24x1.gif\" width=\"%u\" height=\"1\" alt=\"\" /></td></tr>",
            WIDTH_SEMIDIVISION_GRAPHIC,Gbl.Prefs.IconsURL,WIDTH_SEMIDIVISION_GRAPHIC);

   /***** Write again the labels of the X axis *****/
   Sta_WriteLabelsXAxisAccMin (IncX,Format);
  }

/****************************************************************************/
/****** Write labels of the X axis in the graphic of clicks per minute ******/
/****************************************************************************/

#define WIDTH_DIVISION_GRAPHIC	(WIDTH_SEMIDIVISION_GRAPHIC*2)	// 48

static void Sta_WriteLabelsXAxisAccMin (float IncX,const char *Format)
  {
   unsigned i;
   float NumX;

   fprintf (Gbl.F.Out,"<tr>");
   for (i=0, NumX=0; i<=NUM_DIVISIONS_X; i++, NumX+=IncX)
     {
      fprintf (Gbl.F.Out,"<td colspan=\"2\" width=\"%u\" align=\"center\" valign=\"bottom\" class=\"LOG\">",
               WIDTH_DIVISION_GRAPHIC);
      fprintf (Gbl.F.Out,Format,NumX);
      fprintf (Gbl.F.Out,"</td>");
     }
   fprintf (Gbl.F.Out,"</tr>");
  }

/****************************************************************************/
/***** Write a row of the graphic with number of clicks in every minute *****/
/****************************************************************************/

#define WIDTH_GRAPHIC		(WIDTH_DIVISION_GRAPHIC*NUM_DIVISIONS_X)	// 48*10=480

static void Sta_WriteAccessMinute (unsigned Minute,float NumPagesGenerated,float MaxX)
  {
   unsigned BarWidth;

   /***** Start row *****/
   fprintf (Gbl.F.Out,"<tr>");

   /***** Labels of the Y axis, and Y axis *****/
   if (!Minute)						// If minute 0
      fprintf (Gbl.F.Out,"<td align=\"left\" rowspan=\"30\" width=\"%u\" style=\"background-image: url('%s/ejey24x30.gif'); background-repeat: repeat;\" valign=\"top\" class=\"LOG\">00h</td>",
               WIDTH_SEMIDIVISION_GRAPHIC,Gbl.Prefs.IconsURL);
   else if (Minute == (NUM_MINUTES_PER_DAY - 30))	// If 23:30
      fprintf (Gbl.F.Out,"<td align=\"left\" rowspan=\"30\" width=\"%u\" style=\"background-image: url('%s/ejey24x30.gif'); background-repeat: repeat;\" valign=\"bottom\" class=\"LOG\">24h</td>",
               WIDTH_SEMIDIVISION_GRAPHIC,Gbl.Prefs.IconsURL);
   else if (!(Minute % 30) && (Minute % 60))		// If minute is multiple of 30 but not of 60 (i.e.: 30, 90, 150...)
      fprintf (Gbl.F.Out,"<td align=\"left\" rowspan=\"60\" width=\"%u\" style=\"background-image: url('%s/ejey24x60.gif'); background-repeat: repeat;\" valign=\"middle\" class=\"LOG\">%02uh</td>",
               WIDTH_SEMIDIVISION_GRAPHIC,Gbl.Prefs.IconsURL,(Minute + 30) / 60);

   /***** Start of cell for the graphic *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" colspan=\"%u\" width=\"%u\" height=\"1\" valign=\"bottom\" style=\"background-image: url('%s/malla%c48x1.gif'); background-repeat: repeat;\">",
	    NUM_DIVISIONS_X*2,WIDTH_GRAPHIC,Gbl.Prefs.IconsURL,((Minute % 60) == 0) ? 'v' : 'h');

   /***** Draw bar with anchura proporcional al number of clicks *****/
   if (NumPagesGenerated != 0.0)
      if ((BarWidth = (unsigned) (((NumPagesGenerated * (float) WIDTH_GRAPHIC / MaxX)) + 0.5)) != 0)
	 fprintf (Gbl.F.Out,"<img src=\"%s/b%c1x1.gif\" width=\"%u\" height=\"1\" alt=\"\" />",Gbl.Prefs.IconsURL,((Minute % 60) == 0) ? 'g' : 'b',BarWidth);

   /***** End of cell of graphic and end of row *****/
   fprintf (Gbl.F.Out,"</td></tr>");
  }

/****************************************************************************/
/****** Show a listing of with the number of clicks a cada action ******/
/****************************************************************************/

static void Sta_ShowNumAccessesPerAction (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_Action[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   unsigned long NumRow;
   float NumPagesGenerated,MaxPagesGenerated = 0.0,TotalPagesGenerated = 0.0;
   MYSQL_ROW row;

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_Action[Gbl.Prefs.Language],
            Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);

   /***** Compute maximum number of pages generated per day *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      if (NumRow == 1)
	 MaxPagesGenerated = NumPagesGenerated;
      TotalPagesGenerated += NumPagesGenerated;
     }

   /***** Write rows *****/
   mysql_data_seek (mysql_res, 0);
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Write the action */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"LOG\">%s&nbsp;</td>",row[0]);

      /* Draw bar proportional to number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      Sta_DrawBarNumAccesses ('c',NumPagesGenerated,MaxPagesGenerated,TotalPagesGenerated,400);
     }
  }

/****************************************************************************/
/**** Show a listing with the number of clicks a cada degree ****/
/****************************************************************************/

static void Sta_ShowNumAccessesPerDegree (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_No_INDEX[Txt_NUM_LANGUAGES];
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   unsigned long NumRow,Ranking;
   float NumPagesGenerated,MaxPagesGenerated = 0.0,TotalPagesGenerated = 0.0;
   MYSQL_ROW row;
   long DegCod;

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_No_INDEX[Gbl.Prefs.Language],
            Txt_Degree[Gbl.Prefs.Language],
            Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);

   /***** Compute maximum number of pages generated per degree *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);

      if (NumRow == 1)
	 MaxPagesGenerated = NumPagesGenerated;
      TotalPagesGenerated += NumPagesGenerated;
     }

   /***** Write rows *****/
   mysql_data_seek (mysql_res, 0);
   for (Ranking=0, NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      /* Get degree */
      row = mysql_fetch_row (mysql_res);

      /* Get degree code */
      DegCod = Str_ConvertStrCodToLongCod (row[0]);

      /* Write ranking of this degree */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"LOG\">");
      if (DegCod >= 0)
         fprintf (Gbl.F.Out,"%lu",++Ranking);
      fprintf (Gbl.F.Out,"&nbsp;</td>");

      /* Write degree */
      Sta_WriteDegree (DegCod);

      /* Draw bar proportional to number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      Sta_DrawBarNumAccesses ('c',NumPagesGenerated,MaxPagesGenerated,TotalPagesGenerated,300);
     }
  }

/****************************************************************************/
/********* Show a listing with the number of clicks to each course **********/
/****************************************************************************/

static void Sta_ShowNumAccessesPerCourse (unsigned long NumRows,MYSQL_RES *mysql_res)
  {
   extern const char *Txt_No_INDEX[Txt_NUM_LANGUAGES];
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Year[Txt_NUM_LANGUAGES];
   extern const char *Txt_Course[Txt_NUM_LANGUAGES];
   extern const char *Txt_TYPE_COUNT_STAT_CAPS[Sta_NUM_STAT_COUNT_TYPES][Txt_NUM_LANGUAGES];
   extern const char *Txt_Go_to_X[Txt_NUM_LANGUAGES];
   extern const char *Deg_StrsDegYear[1+Deg_MAX_YEARS_PER_DEGREE];	// Declaration in swad_deg.c
   unsigned long NumRow,Ranking;
   float NumPagesGenerated,MaxPagesGenerated = 0.0,TotalPagesGenerated = 0.0;
   MYSQL_ROW row;
   bool CrsOK;
   struct Course Crs;

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"left\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>\n",
            Txt_No_INDEX[Gbl.Prefs.Language],
            Txt_Degree[Gbl.Prefs.Language],
            Txt_Year[Gbl.Prefs.Language],
            Txt_Course[Gbl.Prefs.Language],
            Txt_TYPE_COUNT_STAT_CAPS[Gbl.Stat.CountType][Gbl.Prefs.Language]);

   /***** Compute maximum number of pages generated per course *****/
   for (NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);

      /* Get number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);

      if (NumRow == 1)
	 MaxPagesGenerated = NumPagesGenerated;
      TotalPagesGenerated += NumPagesGenerated;
     }

   /***** Write rows *****/
   mysql_data_seek (mysql_res, 0);
   for (Ranking=0, NumRow = 1; NumRow <= NumRows; NumRow++)
     {
      /* Get degree, the year and the course */
      row = mysql_fetch_row (mysql_res);

      /* Get course code */
      Crs.CrsCod = Str_ConvertStrCodToLongCod (row[0]);

      /* Get data of current degree */
      CrsOK = Deg_GetDataOfCourseByCod (&Crs);

      /* Write ranking of this course */
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"LOG\">");
      if (CrsOK)
         fprintf (Gbl.F.Out,"%lu",++Ranking);
      fprintf (Gbl.F.Out,"&nbsp;</td>");

      /* Write degree */
      Sta_WriteDegree (Crs.DegCod);

      /* Write degree year */
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" class=\"LOG\">%s&nbsp;</td>",
               CrsOK ? Deg_StrsDegYear[Crs.Year] : "-");

      /* Write course, including link */
      fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\" class=\"LOG\">");
      if (CrsOK)
        {
         Act_FormStart ((Gbl.Usrs.Me.Logged) ? ActMnu : ActUnk);
         fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"CrsCod\" value=\"%ld\" />",Crs.CrsCod);
         sprintf (Gbl.MsgWindowStatus,Txt_Go_to_X[Gbl.Prefs.Language],Crs.FullName);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,"LOG");
         fprintf (Gbl.F.Out,"%s</acronym></a>",Crs.ShortName);
        }
      else
         fprintf (Gbl.F.Out,"-");
      fprintf (Gbl.F.Out,"&nbsp;");
      if (CrsOK)
         fprintf (Gbl.F.Out,"</form>");
      fprintf (Gbl.F.Out,"</td>");

      /* Draw bar proportional to number of pages generated */
      NumPagesGenerated = Str_GetFloatNumFromStr (row[1]);
      Sta_DrawBarNumAccesses ('c',NumPagesGenerated,MaxPagesGenerated,TotalPagesGenerated,300);
     }
  }

/****************************************************************************/
/*********************** Write titulation with an icon **********************/
/****************************************************************************/

static void Sta_WriteDegree (long DegCod)
  {
   struct Degree Deg;

   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\">");
   if (DegCod >= 0)
     {
      Deg.DegCod = DegCod;
      Deg_GetDataOfDegreeByCod (&Deg);
      fprintf (Gbl.F.Out,"<a href=\"%s\" class=\"LOG\" target=\"_blank\"><acronym class=\"LOG\" title=\"%s\">" \
                         "<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"%s\" />&nbsp;%s&nbsp;</acronym></a>",
               Deg.WWW,Deg.FullName,Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Deg.Logo,Deg.FullName,Deg.ShortName);
     }
   else
      fprintf (Gbl.F.Out,"<span class=\"LOG\"><acronym class=\"LOG\" title=\"%s\">" \
                         "<img src=\"%s/tr16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"%s\" />&nbsp;-&nbsp;</acronym></span>",
               "Accesos sin seleccionar titulaci&oacute;n",Gbl.Prefs.IconsURL,
               "Accesos sin seleccionar titulaci&oacute;n");
   fprintf (Gbl.F.Out,"</td>");
  }

/****************************************************************************/
/***************** Draw a bar with the number of clicks ***************/
/****************************************************************************/

static void Sta_DrawBarNumAccesses (char Color,float NumPagesGenerated,float MaxPagesGenerated,float TotalPagesGenerated,unsigned MaxBarWidth)
  {
   unsigned BarWidth;

   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"top\">");
   if (NumPagesGenerated != 0.0)
     {
      /* Draw bar with a with proportional to the number of clicks */
      BarWidth = (unsigned) (((NumPagesGenerated * (float) MaxBarWidth) / MaxPagesGenerated) + 0.5);
      if (BarWidth == 0)
         BarWidth = 1;
      fprintf (Gbl.F.Out,"<img src=\"%s/%c1x14.gif\" width=\"%u\" height=\"14\" align=\"top\" alt=\"\" /><span class=\"LOG\">&nbsp;",
	       Gbl.Prefs.IconsURL,Color,BarWidth);

      /* Write the number of clicks */
      Sta_WriteFloatNum (NumPagesGenerated);
      fprintf (Gbl.F.Out,"&nbsp;(%u",
               (unsigned) (((NumPagesGenerated * 100.0) / TotalPagesGenerated) + 0.5));
     }
   else
      /* Write the number of clicks */
      fprintf (Gbl.F.Out,"<span class=\"LOG\">0&nbsp;(0");
   fprintf (Gbl.F.Out,"%%)&nbsp;</span></td></tr>\n");
  }

/****************************************************************************/
/**** Write parameters of initial and final dates in the query of clicks ****/
/****************************************************************************/

void Sta_WriteParamsDatesSeeAccesses (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"StartDay\" value=\"%u\" />\n",
            Gbl.DateRange.DateIni.Day);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"StartMonth\" value=\"%u\" />\n",
            Gbl.DateRange.DateIni.Month);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"StartYear\" value=\"%u\" />\n",
            Gbl.DateRange.DateIni.Year);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"EndDay\" value=\"%u\" />\n",
            Gbl.DateRange.DateEnd.Day);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"EndMonth\" value=\"%u\" />\n",
            Gbl.DateRange.DateEnd.Month);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"EndYear\" value=\"%u\" />\n",
            Gbl.DateRange.DateEnd.Year);
  }

/****************************************************************************/
/******************** Write the selected range of dates *********************/
/****************************************************************************/

static void Sta_WriteSelectedRangeOfDates (unsigned NumDays)
  {
   extern const char *Txt_Date[Txt_NUM_LANGUAGES];
   extern const char *Txt_Dates[Txt_NUM_LANGUAGES];
   extern const char *Txt_DATES_RANGE[Txt_NUM_LANGUAGES];
   extern const char *Txt_one_day[Txt_NUM_LANGUAGES];
   extern const char *Txt_days[Txt_NUM_LANGUAGES];
   char StrDateIni[2+1+2+1+4+1];
   char StrDateEnd[2+1+2+1+4+1];
   char StrDatesRange[1024];

   sprintf (StrDateIni,"%02u/%02u/%04u",Gbl.DateRange.DateIni.Day,Gbl.DateRange.DateIni.Month,Gbl.DateRange.DateIni.Year);
   if (NumDays == 1)
      sprintf (Gbl.Message,"%s: %s (%s)",Txt_Date[Gbl.Prefs.Language],StrDateIni,Txt_one_day[Gbl.Prefs.Language]);
   else
     {
      sprintf (StrDateEnd,"%02u/%02u/%04u",Gbl.DateRange.DateEnd.Day,Gbl.DateRange.DateEnd.Month,Gbl.DateRange.DateEnd.Year);
      sprintf (StrDatesRange,Txt_DATES_RANGE[Gbl.Prefs.Language],StrDateIni,StrDateEnd);
      sprintf (Gbl.Message,"%s: %s (%u %s)",
               Txt_Dates[Gbl.Prefs.Language],StrDatesRange,NumDays,Txt_days[Gbl.Prefs.Language]);
     }
   Lay_WriteTitle (Gbl.Message);
  }

/****************************************************************************/
/******** Write a number in floating point with the correct accuracy ********/
/****************************************************************************/

static void Sta_WriteFloatNum (float Number)
  {
   double IntegerPart,FractionaryPart;
   char *Format;

   FractionaryPart = modf ((double) Number,&IntegerPart);

   if (FractionaryPart == 0.0)
      fprintf (Gbl.F.Out,"%.0f",IntegerPart);
   else
     {
      if (IntegerPart != 0.0 || FractionaryPart >= 0.1)
         Format = "%.2f";
      else if (FractionaryPart >= 0.01)
         Format = "%.3f";
      else if (FractionaryPart >= 0.001)
         Format = "%.4f";
      else if (FractionaryPart >= 0.0001)
         Format = "%.5f";
      else if (FractionaryPart >= 0.00001)
         Format = "%.6f";
      else if (FractionaryPart >= 0.000001)
         Format = "%.7f";
      else
         Format = "%e";
      fprintf (Gbl.F.Out,Format,Number);
     }
  }

/****************************************************************************/
/************************** Show use of the platform ************************/
/****************************************************************************/

void Sta_ReqUseOfPlatform (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_STA_USE_STAT_TYPES[Sta_NUM_TYPES_USE_STATS][Txt_NUM_LANGUAGES];
   extern const char *Txt_Show_statistic[Txt_NUM_LANGUAGES];
   Sta_UseStatType_t UseStatType;

   /***** Start form *****/
   Act_FormStart (ActSeeUseGbl);
   fprintf (Gbl.F.Out,"<div align=\"center\"><table>\n");

   /***** Compute stats for anywhere, degree or course? *****/
   Sta_WriteSelectorDegCrs (Sta_LOCATION_RANGE_ALL);

   /***** Type of statistic *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   for (UseStatType=(Sta_UseStatType_t) 0; UseStatType<Sta_NUM_TYPES_USE_STATS; UseStatType++)
     {
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" valign=\"top\">" \
                         "<input type=\"radio\" name=\"UseStatType\" value=\"%u\"",
               (unsigned) UseStatType);
      if (UseStatType == Gbl.Stat.UseStatType)
         fprintf (Gbl.F.Out," checked=\"checked\"");
      fprintf (Gbl.F.Out," /></td>" \
                         "<td align=\"left\" class=\"%s\">%s</td>" \
                         "</tr>",
               The_ClassFormul[Gbl.Prefs.Theme],Txt_STA_USE_STAT_TYPES[UseStatType][Gbl.Prefs.Language]);
     }
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   fprintf (Gbl.F.Out,"</td></tr>\n");

   /***** Submit button *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">" \
                      "<input type=\"submit\" value=\"%s\" />" \
                      "</td></tr>\n",
            Txt_Show_statistic[Gbl.Prefs.Language]);

   /***** Form end *****/
   fprintf (Gbl.F.Out,"</table></div></form>\n");
  }

/****************************************************************************/
/************************** Show use of the platform ************************/
/****************************************************************************/

void Sta_ShowUseOfPlatform (void)
  {
   extern const char *Txt_Users[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_users_belonging_to_at_least_one_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_users_not_belonging_to_any_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_Total_number_of_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_Average_number_of_courses_to_which_a_user_belongs[Txt_NUM_LANGUAGES];
   extern const char *Txt_Average_number_of_users_belonging_to_a_course[Txt_NUM_LANGUAGES];
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   /***** Get the type of stat of use ******/
   Par_GetParam (Par_PARAM_SINGLE,"UseStatType",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      Lay_ShowErrorAndExit ("Type of stat is missing.");
   if (UnsignedNum >= Sta_NUM_TYPES_USE_STATS)
      Lay_ShowErrorAndExit ("Type of stat is missing.");
   Gbl.Stat.UseStatType = (Sta_UseStatType_t) UnsignedNum;

   /***** Show again the form to see use of the platform *****/
   Sta_ReqUseOfPlatform ();

   /***** Show the stat of use selected by user *****/
   switch (Gbl.Stat.UseStatType)
     {
      case Sta_DEGREES_AND_COURSES:	// Number of degrees and courses
         /***** Number of degrees and courses *****/
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
         Sta_GetAndShowNumDegsInSWAD ();
         Sta_GetAndShowNumCrssInSWAD ();
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
         break;
      case Sta_USERS:			// Number of users
         /***** Number of users *****/
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
         fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                            "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                            "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                            "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                            "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                            "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td></tr>",
                  Txt_Users[Gbl.Prefs.Language],
                  Txt_No_of_users_belonging_to_at_least_one_course[Gbl.Prefs.Language],
                  Txt_No_of_users_not_belonging_to_any_course[Gbl.Prefs.Language],
                  Txt_Total_number_of_users[Gbl.Prefs.Language],
                  Txt_Average_number_of_courses_to_which_a_user_belongs[Gbl.Prefs.Language],
                  Txt_Average_number_of_users_belonging_to_a_course[Gbl.Prefs.Language]);
         Sta_GetAndShowNumUsrsInPlatform (Usr_STUDENT);
         Sta_GetAndShowNumUsrsInPlatform (Usr_TEACHER);
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
         break;
      case Sta_FOLDERS_AND_FILES:	// Number of folders and files
         /***** Stats of exploration trees *****/
         Sta_ShowStatsExpTrees ();
         break;
      case Sta_MSGS_BETWEEN_USERS:	// Number of users' (sent and received) messages
         /***** Stats of messages *****/
         Sta_ShowMsgsStats ();
         break;
      case Sta_FORUMS:			// Number of forums, threads and posts
         /***** Stats of forums *****/
         Sta_ShowForumStats ();
         break;
      case Sta_NOTICES:			// Number of notices
         /***** Stats of notices *****/
         Sta_ShowNoticesStats ();
         break;
      case Sta_LANGUAGES:		// Number of users per language
         /***** Number of users who have choice a language *****/
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
         Sta_GetAndShowNumUsrsPerLanguage ();
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
         break;
      case Sta_THEMES:			// Number of users per theme
         /***** Number of users who have choice a theme *****/
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
         Sta_GetAndShowNumUsrsPerTheme ();
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
         break;
      case Sta_COLUMNS:			// Number of users per design of columns
         /***** Number of users who have choice a layout of columns *****/
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
         Sta_GetAndShowNumUsrsPerLayout ();
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
         break;
      case Sta_NOTIFY_EVENTS:			// Number of users per notify event
         /***** Number of users who want to be notified by e-mail on each event *****/
         Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
         Sta_GetAndShowNumUsrsPerNotifyEvent ();
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
  }

/****************************************************************************/
/********************* Get and show total number of degrees *****************/
/****************************************************************************/

static void Sta_GetAndShowNumDegsInSWAD (void)
  {
   extern const char *Txt_Degrees[Txt_NUM_LANGUAGES];
   unsigned NumDegrees;

   /***** Get number of degrees *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         NumDegrees = Deg_GetTotalNumberOfDegrees ();
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         NumDegrees = 1;
     }

   /***** Write number of degrees *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">&nbsp;%s:&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">%u&nbsp;</td>" \
                      "</tr>",
            Txt_Degrees[Gbl.Prefs.Language],NumDegrees);
  }

/****************************************************************************/
/****************** Get and show total number of courses ********************/
/****************************************************************************/

static void Sta_GetAndShowNumCrssInSWAD (void)
  {
   extern const char *Txt_Courses[Txt_NUM_LANGUAGES];
   extern const char *Txt_Courses_with_teachers[Txt_NUM_LANGUAGES];
   extern const char *Txt_Courses_with_students[Txt_NUM_LANGUAGES];
   unsigned NumCourses;

   /***** Get number of courses *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         NumCourses = Deg_GetTotalNumberOfCourses ();
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         NumCourses = Gbl.CurrentDeg.Deg.NumCourses;
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         NumCourses = 1;
     }

   /***** Write number of courses *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">&nbsp;%s:&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">%u&nbsp;</td></tr>",
            Txt_Courses[Gbl.Prefs.Language],NumCourses);

   /***** Get number of courses with teachers *****/
   NumCourses = Sta_GetNumCoursesWithUsrs (Usr_TEACHER);

   /***** Write total number of courses with teachers *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">&nbsp;%s:&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">%u&nbsp;</td></tr>",
            Txt_Courses_with_teachers[Gbl.Prefs.Language],NumCourses);

   /***** Get number of courses with students *****/
   NumCourses = Sta_GetNumCoursesWithUsrs (Usr_STUDENT);

   /***** Write total number of courses with students *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">&nbsp;%s:&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">%u&nbsp;</td></tr>",
            Txt_Courses_with_students[Gbl.Prefs.Language],NumCourses);
  }

/****************************************************************************/
/*************** Get number of courses with users of a type *****************/
/****************************************************************************/

static unsigned Sta_GetNumCoursesWithUsrs (Usr_UsrType_t UsrType)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumCourses;

   /***** Get number of courses with users *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT COUNT(DISTINCT crs_usr.CrsCod) FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod",
                  (unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT COUNT(DISTINCT courses.CrsCod) FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         return ((UsrType == Usr_TEACHER) ? (Gbl.CurrentCrs.Crs.NumTchs ? 1 : 0) : (Gbl.CurrentCrs.Crs.NumStds ? 1 : 0));
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of courses with users");

   /***** Get number of courses *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumCourses) != 1)
     Lay_ShowErrorAndExit ("Error when getting number of courses with users.");
   
   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumCourses;
  }

/****************************************************************************/
/************************ Get total number of users *************************/
/****************************************************************************/

unsigned Sta_GetTotalNumberOfUsers (Sta_LocationRange_t LocationRange,Usr_UsrType_t UsrType)
  {
   char Query[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrs;

   /***** Get number of users from database *****/
   switch (LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         if (UsrType == Usr_UNKNOWN)
            sprintf (Query,"SELECT COUNT(*) FROM usr_data");
         else
            sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE UsrType='%u'",
                     (unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         if (UsrType == Usr_UNKNOWN)
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM courses,crs_usr,usr_data" \
                           " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod",
                     Gbl.CurrentDeg.Deg.DegCod);
         else
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM courses,crs_usr,usr_data" \
                           " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
                     Gbl.CurrentDeg.Deg.DegCod,(unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         if (UsrType == Usr_UNKNOWN)
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM crs_usr,usr_data" \
                           " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod",
                     Gbl.CurrentCrs.Crs.CrsCod);
         else
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM crs_usr,usr_data" \
                           " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
                     Gbl.CurrentCrs.Crs.CrsCod,(unsigned) UsrType);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of users");

   /***** Get number of teachers *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumUsrs) != 1)
      NumUsrs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumUsrs;
  }

/****************************************************************************/
/************************ See stats about the platform **********************/
/****************************************************************************/

static void Sta_GetAndShowNumUsrsInPlatform (Usr_UsrType_t UsrType)
  {
   extern const char *Txt_USR_TYPES_PLURAL_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   unsigned NumUsrsTotal,NumUsrsWithCrs;
   float NumCrssPerUsr,NumUsrsPerCrs;

   /***** Get the number of users belonging to any course *****/
   NumUsrsWithCrs = Sta_GetNumUsrsBelongingToAnyCrs (UsrType);

   /***** Get total number of users *****/
   NumUsrsTotal = Sta_GetTotalNumberOfUsers (Sta_LOCATION_RANGE_ALL,UsrType);

   /***** Get average number of courses per user *****/
   NumCrssPerUsr = Sta_GetNumCrssPerUsr (UsrType);

   /***** Query the number of users por course *****/
   NumUsrsPerCrs = Sta_GetNumUsrsPerCrs (UsrType);

   /***** Write the total number of users *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%s&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%.2f&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%.2f&nbsp;</td>" \
                      "</tr>",
            Txt_USR_TYPES_PLURAL_Abc[UsrType][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
            NumUsrsWithCrs,NumUsrsTotal - NumUsrsWithCrs,NumUsrsTotal,NumCrssPerUsr,NumUsrsPerCrs);
  }

/****************************************************************************/
/*************** Get number of courses with users of a type *****************/
/****************************************************************************/

static unsigned Sta_GetNumUsrsBelongingToAnyCrs (Usr_UsrType_t UsrType)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrsWithCrs;

   /***** Get number of users who belong to any course *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod",
                  (unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         return ((UsrType == Usr_TEACHER) ? Gbl.CurrentCrs.Crs.NumTchs : Gbl.CurrentCrs.Crs.NumStds);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of users who belong to any course");

   /* Get number of users */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumUsrsWithCrs) != 1)
      Lay_ShowErrorAndExit ("Error when getting number of users who belong to any course.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   return NumUsrsWithCrs;
  }

/****************************************************************************/
/************ Get average number of courses with users of a type ************/
/****************************************************************************/

static float Sta_GetNumCrssPerUsr (Usr_UsrType_t UsrType)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   float NumCrssPerUsr;

   /***** Get number of courses per user from database *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT AVG(NumCrss) FROM" \
                        " (SELECT COUNT(crs_usr.CrsCod) AS NumCrss FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod GROUP BY crs_usr.UsrCod) AS NumCrssTable",
                  (unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT AVG(NumCrss) FROM" \
                        " (SELECT COUNT(crs_usr.CrsCod) AS NumCrss FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u' GROUP BY crs_usr.UsrCod) AS NumCrssTable",
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         return 1.0;
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of courses per user");

   /***** Get number of courses *****/
   row = mysql_fetch_row (mysql_res);
   NumCrssPerUsr = Str_GetFloatNumFromStr (row[0]);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumCrssPerUsr;
  }

/****************************************************************************/
/************ Get average number of courses with users of a type ************/
/****************************************************************************/

static float Sta_GetNumUsrsPerCrs (Usr_UsrType_t UsrType)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   float NumUsrsPerCrs;

   /***** Get number of users per course from database *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT AVG(NumUsrs) FROM" \
                        " (SELECT COUNT(crs_usr.UsrCod) AS NumUsrs FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod GROUP BY crs_usr.CrsCod) AS NumUsrsTable",
                  (unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT AVG(NumUsrs) FROM" \
                        " (SELECT COUNT(crs_usr.UsrCod) AS NumUsrs FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u' GROUP BY crs_usr.CrsCod) AS NumUsrsTable",
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) UsrType);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         return (float) ((UsrType == Usr_TEACHER) ? Gbl.CurrentCrs.Crs.NumTchs : Gbl.CurrentCrs.Crs.NumStds);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get number of users per course");

   /***** Get number of users *****/
   row = mysql_fetch_row (mysql_res);
   NumUsrsPerCrs = Str_GetFloatNumFromStr (row[0]);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumUsrsPerCrs;
  }

/*****************************************************************************/
/********************** Show stats of exploration trees **********************/
/*****************************************************************************/

static void Sta_ShowStatsExpTrees (void)
  {
   extern const char *Txt_STAT_COURSE_FILE_ZONES[][Txt_NUM_LANGUAGES];
   extern const char *Txt_Virtual_pendrives[Txt_NUM_LANGUAGES];
   static const Brw_FileBrowser_t StatCrsFileZones[Sta_NUM_STAT_CRS_FILE_ZONES] =
     {
      Brw_FILE_BRW_ADMIN_DOWNLOAD_CRS,
      Brw_FILE_BRW_ADMIN_DOWNLOAD_GRP,
      Brw_FILE_BRW_COMMON_CRS,
      Brw_FILE_BRW_COMMON_GRP,
      Brw_FILE_BRW_ADMIN_MARKS_CRS,
      Brw_FILE_BRW_ADMIN_MARKS_GRP,
      Brw_FILE_BRW_ASSIGNMENTS_USR,
      Brw_FILE_BRW_WORKS_USR,
      Brw_FILE_BRW_UNKNOWN,
     };
   unsigned NumStat;

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write table heading *****/
   Sta_WriteStatsExpTreesTableHead ();

   /***** Write sizes of course file zones *****/
   for (NumStat=0; NumStat<Sta_NUM_STAT_CRS_FILE_ZONES; NumStat++)
      Sta_WriteRowStatsExpTrees (StatCrsFileZones[NumStat],Txt_STAT_COURSE_FILE_ZONES[NumStat][Gbl.Prefs.Language]);

   /***** Write table heading *****/
   Sta_WriteStatsExpTreesTableHead ();

   /***** Write size of briefcases *****/
   Sta_WriteRowStatsExpTrees (Brw_FILE_BRW_BRIEFCASE_USR,Txt_Virtual_pendrives[Gbl.Prefs.Language]);

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/*********** Write table heading for stats of exploration trees **************/
/*****************************************************************************/

static void Sta_WriteStatsExpTreesTableHead (void)
  {
   extern const char *Txt_File_zones[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_courses[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_Max_levels[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_folders[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_files[Txt_NUM_LANGUAGES];
   extern const char *Txt_Size[Txt_NUM_LANGUAGES];
   extern const char *Txt_STAT_COURSE_FILE_ZONES[][Txt_NUM_LANGUAGES];
   extern const char *Txt_crs[Txt_NUM_LANGUAGES];
   extern const char *Txt_usr[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s/<br />%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s/<br />%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s/<br />%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s/<br />%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s/<br />%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s/<br />%s</td>" \
                      "</tr>",
            Txt_File_zones[Gbl.Prefs.Language],
            Txt_No_of_courses[Gbl.Prefs.Language],
            Txt_No_of_users[Gbl.Prefs.Language],
            Txt_Max_levels[Gbl.Prefs.Language],
            Txt_No_of_folders[Gbl.Prefs.Language],
            Txt_No_of_files[Gbl.Prefs.Language],
            Txt_Size[Gbl.Prefs.Language],
            Txt_No_of_folders[Gbl.Prefs.Language],Txt_crs[Gbl.Prefs.Language],
            Txt_No_of_files[Gbl.Prefs.Language],Txt_crs[Gbl.Prefs.Language],
            Txt_Size[Gbl.Prefs.Language],Txt_crs[Gbl.Prefs.Language],
            Txt_No_of_folders[Gbl.Prefs.Language],Txt_usr[Gbl.Prefs.Language],
            Txt_No_of_files[Gbl.Prefs.Language],Txt_usr[Gbl.Prefs.Language],
            Txt_Size[Gbl.Prefs.Language],Txt_usr[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/*************** Write a row of stats of exploration trees *******************/
/*****************************************************************************/

static void Sta_WriteRowStatsExpTrees (Brw_FileBrowser_t FileZone,const char *NameOfFileZones)
  {
   int NumCrss,NumUsrs;	// -1 stands for not aplicable
   char StrNumCrss[10+1];
   char StrNumUsrs[10+1];
   char StrNumFoldersPerCrs[10+1];
   char StrNumFoldersPerUsr[10+1];
   char StrNumFilesPerCrs[10+1];
   char StrNumFilesPerUsr[10+1];
   unsigned MaxLevels;
   unsigned long NumFolders;
   unsigned long NumFiles;
   unsigned long long int Size;

   Sta_GetSizeOfFileZoneFromDB (Gbl.Stat.LocationRange,FileZone,
                                &NumCrss,&NumUsrs,&MaxLevels,&NumFolders,&NumFiles,&Size);
   if (NumCrss == -1)
     {
      strcpy (StrNumCrss         ,"-");
      strcpy (StrNumFoldersPerCrs,"-");
      strcpy (StrNumFilesPerCrs  ,"-");
     }
   else
     {
      sprintf (StrNumCrss         ,"%d"  ,NumCrss);
      sprintf (StrNumFoldersPerCrs,"%.1f",NumCrss ? (double) NumFolders / (double) NumCrss : 0.0);
      sprintf (StrNumFilesPerCrs  ,"%.1f",NumCrss ? (double) NumFiles   / (double) NumCrss : 0.0);
     }
   if (NumUsrs == -1)
     {
      strcpy (StrNumUsrs         ,"-");
      strcpy (StrNumFoldersPerUsr,"-");
      strcpy (StrNumFilesPerUsr  ,"-");
     }
   else
     {
      sprintf (StrNumUsrs         ,"%d"  ,NumUsrs);
      sprintf (StrNumFoldersPerUsr,"%.1f",NumUsrs ? (double) NumFolders / (double) NumUsrs : 0.0);
      sprintf (StrNumFilesPerUsr  ,"%.1f",NumUsrs ? (double) NumFiles   / (double) NumUsrs : 0.0);
     }
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%lu</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%lu</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;",
            NameOfFileZones,
            StrNumCrss,StrNumUsrs,MaxLevels,
            NumFolders,NumFiles);
   Str_WriteSizeInBytes ((double) Size);
   fprintf (Gbl.F.Out,"</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;",
            StrNumFoldersPerCrs,StrNumFilesPerCrs);
   if (NumCrss == -1)
      fprintf (Gbl.F.Out,"-");
   else
      Str_WriteSizeInBytes (NumCrss ? (double) Size / (double) NumCrss : 0.0);
   fprintf (Gbl.F.Out,"</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;",
            StrNumFoldersPerUsr,StrNumFilesPerUsr);
   if (NumUsrs == -1)
      fprintf (Gbl.F.Out,"-");
   else
      Str_WriteSizeInBytes (NumUsrs ? (double) Size / (double) NumUsrs : 0.0);
   fprintf (Gbl.F.Out,"</td></tr>");
  }

/*****************************************************************************/
/**************** Get the size of a file zone from database ******************/
/*****************************************************************************/

static void Sta_GetSizeOfFileZoneFromDB (Sta_LocationRange_t LocationRange,Brw_FileBrowser_t FileBrowser,
                                         int *NumCrss,int *NumUsrs,unsigned *MaxLevels,unsigned long *NumFolders,unsigned long *NumFiles,unsigned long long int *Size)
  {
   char Query[2048];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;

   /***** Get the size of a file browser *****/
   switch (LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         if (FileBrowser == Brw_FILE_BRW_UNKNOWN)
            sprintf (Query,"SELECT COUNT(DISTINCT CrsCod),'-1',MAX(NumLevels),SUM(NumFolders),SUM(NumFiles),SUM(TotalSize)" \
                           " FROM file_browser_size WHERE FileBrowser<>'%u'",
                     (unsigned) Brw_FILE_BRW_BRIEFCASE_USR);
         else if (FileBrowser == Brw_FILE_BRW_BRIEFCASE_USR)
            sprintf (Query,"SELECT '-1',COUNT(DISTINCT UsrCod),MAX(NumLevels),SUM(NumFolders),SUM(NumFiles),SUM(TotalSize)" \
                           " FROM file_browser_size" \
                           " WHERE FileBrowser='%u'",
                     (unsigned) FileBrowser);
         else if (FileBrowser == Brw_FILE_BRW_ASSIGNMENTS_USR ||
                  FileBrowser == Brw_FILE_BRW_WORKS_USR)
            sprintf (Query,"SELECT COUNT(DISTINCT CrsCod),COUNT(DISTINCT UsrCod),MAX(NumLevels),SUM(NumFolders),SUM(NumFiles),SUM(TotalSize)" \
                           " FROM file_browser_size" \
                           " WHERE FileBrowser='%u'",
                     (unsigned) FileBrowser);
         else
            sprintf (Query,"SELECT COUNT(DISTINCT CrsCod),'-1',MAX(NumLevels),SUM(NumFolders),SUM(NumFiles),SUM(TotalSize)" \
                           " FROM file_browser_size" \
                           " WHERE FileBrowser='%u'",
                     (unsigned) FileBrowser);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         if (FileBrowser == Brw_FILE_BRW_UNKNOWN)
            sprintf (Query,"SELECT COUNT(DISTINCT file_browser_size.CrsCod),'-1',MAX(file_browser_size.NumLevels),SUM(file_browser_size.NumFolders),SUM(file_browser_size.NumFiles),SUM(file_browser_size.TotalSize)" \
                           " FROM file_browser_size,courses" \
                           " WHERE courses.DegCod='%ld' AND courses.CrsCod=file_browser_size.CrsCod",
                     Gbl.CurrentDeg.Deg.DegCod);
         else if (FileBrowser == Brw_FILE_BRW_BRIEFCASE_USR)
            sprintf (Query,"SELECT '-1',COUNT(DISTINCT file_browser_size.UsrCod),MAX(file_browser_size.NumLevels),SUM(file_browser_size.NumFolders),SUM(file_browser_size.NumFiles),SUM(file_browser_size.TotalSize)" \
                           " FROM file_browser_size,courses,crs_usr" \
                           " WHERE file_browser_size.FileBrowser='%u' AND courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND file_browser_size.UsrCod=crs_usr.UsrCod",
                     (unsigned) FileBrowser,Gbl.CurrentDeg.Deg.DegCod);
         else if (FileBrowser == Brw_FILE_BRW_ASSIGNMENTS_USR ||
                  FileBrowser == Brw_FILE_BRW_WORKS_USR)
            sprintf (Query,"SELECT COUNT(DISTINCT file_browser_size.CrsCod),COUNT(DISTINCT file_browser_size.UsrCod),MAX(file_browser_size.NumLevels),SUM(file_browser_size.NumFolders),SUM(file_browser_size.NumFiles),SUM(file_browser_size.TotalSize)" \
                           " FROM file_browser_size,courses" \
                           " WHERE file_browser_size.FileBrowser='%u' AND courses.DegCod='%ld' AND courses.CrsCod=file_browser_size.CrsCod",
                     (unsigned) FileBrowser,Gbl.CurrentDeg.Deg.DegCod);
         else
            sprintf (Query,"SELECT COUNT(DISTINCT file_browser_size.CrsCod),'-1',MAX(file_browser_size.NumLevels),SUM(file_browser_size.NumFolders),SUM(file_browser_size.NumFiles),SUM(file_browser_size.TotalSize)" \
                           " FROM file_browser_size,courses" \
                           " WHERE file_browser_size.FileBrowser='%u' AND courses.DegCod='%ld' AND courses.CrsCod=file_browser_size.CrsCod",
                     (unsigned) FileBrowser,Gbl.CurrentDeg.Deg.DegCod);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         if (FileBrowser == Brw_FILE_BRW_UNKNOWN)
            sprintf (Query,"SELECT '1','-1',MAX(NumLevels),SUM(NumFolders),SUM(NumFiles),SUM(TotalSize)" \
                           " FROM file_browser_size" \
                           " WHERE CrsCod='%ld'",
                     Gbl.CurrentCrs.Crs.CrsCod);
         else if (FileBrowser == Brw_FILE_BRW_BRIEFCASE_USR)
            sprintf (Query,"SELECT '-1',COUNT(DISTINCT file_browser_size.UsrCod),MAX(file_browser_size.NumLevels),SUM(file_browser_size.NumFolders),SUM(file_browser_size.NumFiles),SUM(file_browser_size.TotalSize)" \
                           " FROM file_browser_size,crs_usr" \
                           " WHERE file_browser_size.FileBrowser='%u' AND crs_usr.CrsCod='%ld' AND file_browser_size.UsrCod=crs_usr.UsrCod",
                     (unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod);
         else if (FileBrowser == Brw_FILE_BRW_ASSIGNMENTS_USR ||
                  FileBrowser == Brw_FILE_BRW_WORKS_USR)
            sprintf (Query,"SELECT '1',COUNT(DISTINCT UsrCod),MAX(NumLevels),SUM(NumFolders),SUM(NumFiles),SUM(TotalSize)" \
                           " FROM file_browser_size" \
                           " WHERE FileBrowser='%u' AND CrsCod='%ld'",
                     (unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod);
         else
            sprintf (Query,"SELECT '1','-1',MAX(NumLevels),SUM(NumFolders),SUM(NumFiles),SUM(TotalSize)" \
                           " FROM file_browser_size" \
                           " WHERE FileBrowser='%u' AND CrsCod='%ld'",
                     (unsigned) FileBrowser,Gbl.CurrentCrs.Crs.CrsCod);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get size of a file browser");

   /* Get row */
   row = mysql_fetch_row (mysql_res);

   /* Reset default values to zero */
   *NumCrss = *NumUsrs = 0;
   *MaxLevels = 0;
   *NumFolders = *NumFiles = 0;
   *Size = 0;

   /* Get number of courses (row[0]) */
   if (row[0])
      if (sscanf (row[0],"%u",NumCrss) != 1)
         Lay_ShowErrorAndExit ("Error when getting number of courses.");

   /* Get number of users (row[1]) */
   if (row[1])
      if (sscanf (row[1],"%u",NumUsrs) != 1)
         Lay_ShowErrorAndExit ("Error when getting number of users.");

   /* Get maximum number of levels (row[2]) */
   if (row[2])
      if (sscanf (row[2],"%u",MaxLevels) != 1)
         Lay_ShowErrorAndExit ("Error when getting maximum number of levels.");

   /* Get number of folders (row[3]) */
   if (row[3])
      if (sscanf (row[3],"%lu",NumFolders) != 1)
         Lay_ShowErrorAndExit ("Error when getting number of folders.");

   /* Get number of files (row[4]) */
   if (row[4])
      if (sscanf (row[4],"%lu",NumFiles) != 1)
         Lay_ShowErrorAndExit ("Error when getting number of files.");

   /* Get total size (row[5]) */
   if (row[5])
      if (sscanf (row[5],"%llu",Size) != 1)
         Lay_ShowErrorAndExit ("Error when getting toal size.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/******* Get the number of files of a course file zone from database *********/
/*****************************************************************************/

static unsigned long Sta_GetNumFilesOfCrsFileZoneFromDB (Brw_FileBrowser_t FileBrowser,long CrsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumFiles = 0UL;

   /***** Get number of files in a file browser from database *****/
   sprintf (Query,"SELECT SUM(NumFiles)" \
                  " FROM file_browser_size" \
                  " WHERE FileBrowser='%u' AND CrsCod='%ld'",
            (unsigned) FileBrowser,CrsCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of files in a file browser");

   /***** Get row *****/
   row = mysql_fetch_row (mysql_res);

   /***** Get number of files (row[0]) *****/
   if (row[0])
      if (sscanf (row[0],"%lu",&NumFiles) != 1)
         Lay_ShowErrorAndExit ("Error when getting the number of files in a file browser.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumFiles;
  }

/*****************************************************************************/
/*************************** Show stats of messages **************************/
/*****************************************************************************/

static void Sta_ShowMsgsStats (void)
  {
   extern const char *Txt_Messages[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSGS_Not_deleted[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSGS_Deleted[Txt_NUM_LANGUAGES];
   extern const char *Txt_Total[Txt_NUM_LANGUAGES];
   extern const char *Txt_Number_of_BR_notifications[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSGS_Sent[Txt_NUM_LANGUAGES];
   extern const char *Txt_MSGS_Received[Txt_NUM_LANGUAGES];
   unsigned NumMsgsSent,NumMsgsSentAndDeleted;
   unsigned NumMsgsReceived,NumMsgsReceivedAndDeleted;
   unsigned NumMsgsReceivedAndNotified;

   /***** Get the number of unique messages sent from this location (all the platform, current degree or current course) *****/
   NumMsgsSent                = Msg_GetNumMsgsSent     (Gbl.Stat.LocationRange,Msg_STATUS_ALL     );
   NumMsgsSentAndDeleted      = Msg_GetNumMsgsSent     (Gbl.Stat.LocationRange,Msg_STATUS_DELETED );

   NumMsgsReceived            = Msg_GetNumMsgsReceived (Gbl.Stat.LocationRange,Msg_STATUS_ALL     );
   NumMsgsReceivedAndDeleted  = Msg_GetNumMsgsReceived (Gbl.Stat.LocationRange,Msg_STATUS_DELETED );
   NumMsgsReceivedAndNotified = Msg_GetNumMsgsReceived (Gbl.Stat.LocationRange,Msg_STATUS_NOTIFIED);

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write table heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "</tr>",
            Txt_Messages[Gbl.Prefs.Language],
            Txt_MSGS_Not_deleted[Gbl.Prefs.Language],
            Txt_MSGS_Deleted[Gbl.Prefs.Language],
            Txt_Total[Gbl.Prefs.Language],
            Txt_Number_of_BR_notifications[Gbl.Prefs.Language]);

   /***** Write number of messages *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;-</td>" \
                      "</tr>" \
                      "</tr>" \
                      "<td align=\"left\" class=\"DAT\">%s</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "</tr>",
            Txt_MSGS_Sent[Gbl.Prefs.Language],NumMsgsSent-NumMsgsSentAndDeleted,NumMsgsSentAndDeleted,NumMsgsSent,
            Txt_MSGS_Received[Gbl.Prefs.Language],NumMsgsReceived-NumMsgsReceivedAndDeleted,NumMsgsReceivedAndDeleted,NumMsgsReceived,NumMsgsReceivedAndNotified);

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/***************************** Show stats of forums **************************/
/*****************************************************************************/

static void Sta_ShowForumStats (void)
  {
   extern const char *Txt_Forums[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_forums[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_threads[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_messages[Txt_NUM_LANGUAGES];
   extern const char *Txt_Number_of_BR_notifications[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_threads_BR_per_forum[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_messages_BR_per_thread[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_messages_BR_per_forum[Txt_NUM_LANGUAGES];

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write table heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" class=\"TIT_TBL\">" \
                      "<img src=\"%s/forum16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />" \
                      " %s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "</tr>",
            Gbl.Prefs.IconsURL,Txt_Forums[Gbl.Prefs.Language],
            Txt_No_of_forums[Gbl.Prefs.Language],
            Txt_No_of_threads[Gbl.Prefs.Language],
            Txt_No_of_messages[Gbl.Prefs.Language],
            Txt_Number_of_BR_notifications[Gbl.Prefs.Language],
            Txt_No_of_threads_BR_per_forum[Gbl.Prefs.Language],
            Txt_No_of_messages_BR_per_thread[Gbl.Prefs.Language],
            Txt_No_of_messages_BR_per_forum[Gbl.Prefs.Language]);

   /***** Write a row for each type of forum *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         Sta_ShowStatOfAForumType (For_FORUM_GLOBAL_USRS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_GLOBAL_TCHS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_SWAD_USRS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_SWAD_TCHS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_INSTITUTION_USRS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_INSTITUTION_TCHS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_DEGREE_USRS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_DEGREE_TCHS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_COURSE_USRS,-1L,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_COURSE_TCHS,-1L,-1L);
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         Sta_ShowStatOfAForumType (For_FORUM_DEGREE_USRS,Gbl.CurrentDeg.Deg.DegCod,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_DEGREE_TCHS,Gbl.CurrentDeg.Deg.DegCod,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_COURSE_USRS,Gbl.CurrentDeg.Deg.DegCod,-1L);
         Sta_ShowStatOfAForumType (For_FORUM_COURSE_TCHS,Gbl.CurrentDeg.Deg.DegCod,-1L);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         Sta_ShowStatOfAForumType (For_FORUM_COURSE_USRS,-1L,Gbl.CurrentCrs.Crs.CrsCod);
         Sta_ShowStatOfAForumType (For_FORUM_COURSE_TCHS,-1L,Gbl.CurrentCrs.Crs.CrsCod);
     }

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/************************* Show stats of a forum type ************************/
/*****************************************************************************/

static void Sta_ShowStatOfAForumType (For_ForumType_t ForumType,long DegCod,long CrsCod)
  {
   extern const char *Txt_Courses[Txt_NUM_LANGUAGES];
   extern const char *Txt_Degrees[Txt_NUM_LANGUAGES];
   extern const char *Txt_Centres[Txt_NUM_LANGUAGES];
   extern const char *Txt_Institutions[Txt_NUM_LANGUAGES];
   extern const char *Txt_General[Txt_NUM_LANGUAGES];
   extern const char *Txt_only_teachers[Txt_NUM_LANGUAGES];
   char Icon[512];

   switch (ForumType)
     {
      case For_FORUM_COURSE_USRS:
         sprintf (Icon,"<img src=\"%s/course16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Courses[Gbl.Prefs.Language],"");
         break;
      case For_FORUM_COURSE_TCHS:
         sprintf (Icon,"<img src=\"%s/course16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Courses[Gbl.Prefs.Language],Txt_only_teachers[Gbl.Prefs.Language]);
         break;
      case For_FORUM_DEGREE_USRS:
         sprintf (Icon,"<img src=\"%s/pack16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Degrees[Gbl.Prefs.Language],"");
         break;
      case For_FORUM_DEGREE_TCHS:
         sprintf (Icon,"<img src=\"%s/pack16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Degrees[Gbl.Prefs.Language],Txt_only_teachers[Gbl.Prefs.Language]);
         break;
      case For_FORUM_CENTRE_USRS:
         sprintf (Icon,"<img src=\"%s/house16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Centres[Gbl.Prefs.Language],"(sin implementar)");	//  Change !!!!
         break;
      case For_FORUM_CENTRE_TCHS:
         sprintf (Icon,"<img src=\"%s/house16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Centres[Gbl.Prefs.Language],Txt_only_teachers[Gbl.Prefs.Language]);	//  Change !!!!
         break;
      case For_FORUM_INSTITUTION_USRS:
         sprintf (Icon,"<img src=\"%s/house16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Institutions[Gbl.Prefs.Language],"");
         break;
      case For_FORUM_INSTITUTION_TCHS:
         sprintf (Icon,"<img src=\"%s/house16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_Institutions[Gbl.Prefs.Language],Txt_only_teachers[Gbl.Prefs.Language]);
         break;
      case For_FORUM_GLOBAL_USRS:
         sprintf (Icon,"<img src=\"%s/ballon16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_General[Gbl.Prefs.Language],"");
         break;
      case For_FORUM_GLOBAL_TCHS:
         sprintf (Icon,"<img src=\"%s/ballon16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Txt_General[Gbl.Prefs.Language],Txt_only_teachers[Gbl.Prefs.Language]);
         break;
      case For_FORUM_SWAD_USRS:
         sprintf (Icon,"<img src=\"%s/swad16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Cfg_PLATFORM_SHORT_NAME,"");
         break;
      case For_FORUM_SWAD_TCHS:
         sprintf (Icon,"<img src=\"%s/swad16x16.gif\" width=\"16\" height=\"16\" align=\"top\" />",Gbl.Prefs.IconsURL);
         Sta_WriteForumTitleAndStats (ForumType,DegCod,CrsCod,Icon,Cfg_PLATFORM_SHORT_NAME,Txt_only_teachers[Gbl.Prefs.Language]);
         break;
     }
  }

/*****************************************************************************/
/************************* Write title of a forum type ***********************/
/*****************************************************************************/

static void Sta_WriteForumTitleAndStats (For_ForumType_t ForumType,long DegCod,long CrsCod,const char *Icon,const char *ForumName1,const char *ForumName2)
  {
   unsigned NumForums;
   unsigned NumThreads;
   unsigned NumPosts;
   unsigned NumUsrsToBeNotified;
   float NumThrsPerForum;
   float NumPostsPerThread;
   float NumPostsPerForum;

   /***** Compute number of forums, number of threads and number of posts *****/
   NumForums  = For_GetNumTotalForumsOfType       (ForumType,DegCod,CrsCod);
   NumThreads = For_GetNumTotalThrsInForumsOfType (ForumType,DegCod,CrsCod);
   NumPosts   = For_GetNumTotalPstsInForumsOfType (ForumType,DegCod,CrsCod,&NumUsrsToBeNotified);

   /***** Compute number of threads per forum, number of posts per forum and number of posts per thread *****/
   NumThrsPerForum   = (NumForums  ? (float) NumThreads / (float) NumForums  : 0.0);
   NumPostsPerThread = (NumThreads ? (float) NumPosts   / (float) NumThreads : 0.0);
   NumPostsPerForum  = (NumForums  ? (float) NumPosts   / (float) NumForums  : 0.0);

   /***** Write forum name and stats *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\" valign=\"middle\" class=\"DAT\">%s&nbsp;%s%s</td>" \
                      "<td align=\"right\" class=\"DAT\">%u</td>" \
                      "<td align=\"right\" class=\"DAT\">%u</td>" \
                      "<td align=\"right\" class=\"DAT\">%u</td>" \
                      "<td align=\"right\" class=\"DAT\">%u</td>" \
                      "<td align=\"right\" class=\"DAT\">%.2f</td>" \
                      "<td align=\"right\" class=\"DAT\">%.2f</td>" \
                      "<td align=\"right\" class=\"DAT\">%.2f</td>" \
                      "</tr>",
            Icon,ForumName1,ForumName2,
            NumForums,NumThreads,NumPosts,NumUsrsToBeNotified,
            NumThrsPerForum,NumPostsPerThread,NumPostsPerForum);
  }

/*****************************************************************************/
/***************************** Show stats of notices *************************/
/*****************************************************************************/

static void Sta_ShowNoticesStats (void)
  {
   extern const char *Txt_NOTICE_Active_BR_notices[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTICE_Obsolete_BR_notices[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTICE_Deleted_BR_notices[Txt_NUM_LANGUAGES];
   extern const char *Txt_Total[Txt_NUM_LANGUAGES];
   extern const char *Txt_Number_of_BR_notifications[Txt_NUM_LANGUAGES];
   Not_NoticeStatus_t NoticeStatus;
   unsigned NumNotices[Not_NUM_STATUS_NOTICE];
   unsigned NumTotalNotices = 0;
   unsigned NumNotif;
   unsigned NumTotalNotifications = 0;

   /***** Get the number of notices active and obsolete
          from this location (all the platform, current degree or current course) *****/
   for (NoticeStatus=(Not_NoticeStatus_t) 0; NoticeStatus<Not_NUM_STATUS_NOTICE; NoticeStatus++)
     {
      NumNotices[NoticeStatus] = Not_GetNumNotices (Gbl.Stat.LocationRange,NoticeStatus,&NumNotif);
      NumTotalNotices += NumNotices[NoticeStatus];
      NumTotalNotifications += NumNotif;
     }

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write table heading *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "</tr>",
            Txt_NOTICE_Active_BR_notices[Gbl.Prefs.Language],
            Txt_NOTICE_Obsolete_BR_notices[Gbl.Prefs.Language],
            Txt_NOTICE_Deleted_BR_notices[Gbl.Prefs.Language],
            Txt_Total[Gbl.Prefs.Language],
            Txt_Number_of_BR_notifications[Gbl.Prefs.Language]);

   /***** Write number of notices *****/
   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "<td align=\"right\" class=\"DAT\">&nbsp;%u</td>" \
                      "</tr>",
            NumNotices[Not_ACTIVE_NOTICE],
            NumNotices[Not_OBSOLETE_NOTICE],
            NumNotices[Not_DELETED_NOTICE],
            NumTotalNotices,
            NumTotalNotifications);

   /***** End table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/********* Get and show number of users who have chosen a language **********/
/****************************************************************************/

static void Sta_GetAndShowNumUsrsPerLanguage (void)
  {
   extern const char *Txt_StrLangId[Txt_NUM_LANGUAGES];
   extern const char *Txt_StrLangName[Txt_NUM_LANGUAGES];
   extern const char *Txt_Language[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_PERCENT_of_users[Txt_NUM_LANGUAGES];
   Txt_Language_t Lan;
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrs[Txt_NUM_LANGUAGES];
   unsigned NumUsrsTotal = 0;

   /***** Heading row *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td></tr>",
            Txt_Language[Gbl.Prefs.Language],
            Txt_No_of_users[Gbl.Prefs.Language],
            Txt_PERCENT_of_users[Gbl.Prefs.Language]);

   /***** For each language... *****/
   for (Lan=(Txt_Language_t) 0; Lan<Txt_NUM_LANGUAGES; Lan++)
     {
      /***** Get the number of users who have chosen this language from database *****/
      switch (Gbl.Stat.LocationRange)
        {
         case Sta_LOCATION_RANGE_ALL:
            sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE Language='%s'",Txt_StrLangId[Lan]);
            break;
         case Sta_LOCATION_RANGE_THIS_DEGREE:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM courses,crs_usr,usr_data" \
                           " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.Language='%s'",
                     Gbl.CurrentDeg.Deg.DegCod,Txt_StrLangId[Lan]);
            break;
         case Sta_LOCATION_RANGE_THIS_COURSE:
         default:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM crs_usr,usr_data" \
                           " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.Language='%s'",
                     Gbl.CurrentCrs.Crs.CrsCod,Txt_StrLangId[Lan]);
        }
      DB_QuerySELECT (Query,&mysql_res,"can not get the number of users who have chosen a language");

      /* Get number of users */
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%u",&NumUsrs[Lan]) != 1)
         Lay_ShowErrorAndExit ("Error when getting the number of users who have chosen a language.");

      /* Free structure that stores the query result */
      DB_FreeMySQLResult (&mysql_res);

      /* Update total number of users */
      NumUsrsTotal += NumUsrs[Lan];
     }

   /***** Write number of users who have choice each language *****/
   for (Lan=(Txt_Language_t) 0; Lan<Txt_NUM_LANGUAGES; Lan++)
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">%s&nbsp;</td>" \
                         "<td align=\"right\" class=\"DAT\">%u</td>" \
                         "<td align=\"right\" class=\"DAT\">%5.2f%%</td></tr>",
               Txt_StrLangName[Lan],NumUsrs[Lan],NumUsrsTotal ? 100.0 * (float) NumUsrs[Lan] / (float) NumUsrsTotal : 0);
  }

/****************************************************************************/
/********** Get and show number of users who have choice a theme ************/
/****************************************************************************/

static void Sta_GetAndShowNumUsrsPerTheme (void)
  {
   extern const char *The_ThemeId[The_NUM_THEMES];
   extern const char *The_ThemeNames[The_NUM_THEMES];
   extern const char *Txt_Design[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_PERCENT_of_users[Txt_NUM_LANGUAGES];
   tTheme Theme;
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrs[The_NUM_THEMES];
   unsigned NumUsrsTotal = 0;

   /***** Heading row *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td></tr>",
            Txt_Design[Gbl.Prefs.Language],
            Txt_No_of_users[Gbl.Prefs.Language],
            Txt_PERCENT_of_users[Gbl.Prefs.Language]);

   /***** For each theme... *****/
   for (Theme=(tTheme) 0; Theme<The_NUM_THEMES; Theme++)
     {
      /***** Get number of users who have chosen this theme from database *****/
      switch (Gbl.Stat.LocationRange)
        {
         case Sta_LOCATION_RANGE_ALL:
            sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE Theme='%s'",
                     The_ThemeId[Theme]);
            break;
         case Sta_LOCATION_RANGE_THIS_DEGREE:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod)" \
                           " FROM courses,crs_usr,usr_data" \
                           " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.Theme='%s'",
                     Gbl.CurrentDeg.Deg.DegCod,The_ThemeId[Theme]);
            break;
         case Sta_LOCATION_RANGE_THIS_COURSE:
         default:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod)" \
                           " FROM crs_usr,usr_data" \
                           " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.Theme='%s'",
                     Gbl.CurrentCrs.Crs.CrsCod,The_ThemeId[Theme]);
        }
      DB_QuerySELECT (Query,&mysql_res,"can not get the number of users who have chosen a theme");

      /* Get number of users */
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%u",&NumUsrs[Theme]) != 1)
         Lay_ShowErrorAndExit ("Error when getting the number of users who have chosen a theme.");

      /* Free structure that stores the query result */
      DB_FreeMySQLResult (&mysql_res);

      /* Update total number of users */
      NumUsrsTotal += NumUsrs[Theme];
     }

   /***** Write number of users who have chosen each theme *****/
   for (Theme=(tTheme) 0; Theme<The_NUM_THEMES; Theme++)
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"DAT\"><img src=\"%s/%s/%s/theme_32x20.gif\" width=\"32\" height=\"20\" alt=\"%s\" /></td>" \
                         "<td align=\"right\" class=\"DAT\">%u</td>" \
                         "<td align=\"right\" class=\"DAT\">%5.2f%%</td></tr>",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_THEMES,The_ThemeId[Theme],The_ThemeNames[Theme],
               NumUsrs[Theme],NumUsrsTotal ? 100.0 * (float) NumUsrs[Theme] / (float) NumUsrsTotal : 0);
  }

/****************************************************************************/
/***** Get and show number of users who have chosen a layout of columns *****/
/****************************************************************************/

static void Sta_GetAndShowNumUsrsPerLayout (void)
  {
   extern const char *Txt_Columns[Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_PERCENT_of_users[Txt_NUM_LANGUAGES];
   unsigned SideCols;
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrs[4];
   unsigned NumUsrsTotal = 0;
   extern const char *Txt_LAYOUT_SIDE_COLUMNS[4][Txt_NUM_LANGUAGES];

   /***** Heading row *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td></tr>",
            Txt_Columns[Gbl.Prefs.Language],
            Txt_No_of_users[Gbl.Prefs.Language],
            Txt_PERCENT_of_users[Gbl.Prefs.Language]);

   /***** For each language... *****/
   for (SideCols=0; SideCols<=3; SideCols++)
     {
      /***** Get the number of users who have chosen this layout of columns from database *****/
      switch (Gbl.Stat.LocationRange)
        {
         case Sta_LOCATION_RANGE_ALL:
            sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE SideCols='%u'",SideCols);
            break;
         case Sta_LOCATION_RANGE_THIS_DEGREE:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM courses,crs_usr,usr_data" \
                           " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.SideCols='%u'",
                     Gbl.CurrentDeg.Deg.DegCod,SideCols);
            break;
         case Sta_LOCATION_RANGE_THIS_COURSE:
         default:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM crs_usr,usr_data" \
                           " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.SideCols='%u'",
                     Gbl.CurrentCrs.Crs.CrsCod,SideCols);
        }
      DB_QuerySELECT (Query,&mysql_res,"can not get the number of users who have chosen a layout of columns");

      /* Get number of users */
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%u",&NumUsrs[SideCols]) != 1)
         Lay_ShowErrorAndExit ("Error when getting the number of users who have chosen a layout of columns.");

      /* Free structure that stores the query result */
      DB_FreeMySQLResult (&mysql_res);

      /* Update total number of users */
      NumUsrsTotal += NumUsrs[SideCols];
     }

   /***** Write number of users who have chosen this layout of columns *****/
   for (SideCols=0; SideCols<=3; SideCols++)
      fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"DAT\"><img src=\"%s/layout%u%u_32x20.gif\" width=\"32\" height=\"20\" alt=\"%s\" /></td>" \
                         "<td align=\"right\" class=\"DAT\">%u</td>" \
                         "<td align=\"right\" class=\"DAT\">%5.2f%%</td></tr>",
               Gbl.Prefs.IconsURL,SideCols >> 1,SideCols & 1,Txt_LAYOUT_SIDE_COLUMNS[SideCols][Gbl.Prefs.Language],
               NumUsrs[SideCols],NumUsrsTotal ? 100.0 * (float) NumUsrs[SideCols] / (float) NumUsrsTotal : 0);
  }

/****************************************************************************/
/****** Get and show number of users who want to be notified by e-mail ******/
/****************************************************************************/

static void Sta_GetAndShowNumUsrsPerNotifyEvent (void)
  {
   extern const char *Txt_Event[Txt_NUM_LANGUAGES];
   extern const char *Txt_NOTIFY_EVENTS[Ntf_NUM_NOTIFY_EVENTS][Txt_NUM_LANGUAGES];
   extern const char *Txt_No_of_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_PERCENT_of_users[Txt_NUM_LANGUAGES];
   extern const char *Txt_Number_of_BR_events[Txt_NUM_LANGUAGES];
   extern const char *Txt_Number_of_BR_e_mails[Txt_NUM_LANGUAGES];
   extern const char *Txt_Total[Txt_NUM_LANGUAGES];
   Ntf_NotifyEvent_t NotifyEvent;
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrsTotalInPlatform;
   unsigned NumUsrsTotalWhoWantToBeNotifiedAboutSomeEvent;
   unsigned NumUsrs[Ntf_NUM_NOTIFY_EVENTS];
   unsigned NumEventsTotal = 0;
   unsigned NumEvents[Ntf_NUM_NOTIFY_EVENTS];
   unsigned NumMailsTotal = 0;
   unsigned NumMails[Ntf_NUM_NOTIFY_EVENTS];

   /***** Heading row *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"TIT_TBL\">%s&nbsp;</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td>" \
                      "<td align=\"right\" class=\"TIT_TBL\">&nbsp;%s</td></tr>",
            Txt_Event[Gbl.Prefs.Language],
            Txt_No_of_users[Gbl.Prefs.Language],
            Txt_PERCENT_of_users[Gbl.Prefs.Language],
            Txt_Number_of_BR_events[Gbl.Prefs.Language],
            Txt_Number_of_BR_e_mails[Gbl.Prefs.Language]);

   /***** Get total number of users in platform *****/
   NumUsrsTotalInPlatform = Sta_GetTotalNumberOfUsers (Gbl.Stat.LocationRange,Usr_UNKNOWN);

   /***** Get total number of users who want to be notified by e-mail on some event, from database *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE NotifyEvents<>0");
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod" \
                        " AND usr_data.NotifyEvents<>0",
                  Gbl.CurrentDeg.Deg.DegCod);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM crs_usr,usr_data" \
                        " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod" \
                        " AND usr_data.NotifyEvents<>0",
                  Gbl.CurrentCrs.Crs.CrsCod);
     }
   DB_QuerySELECT (Query,&mysql_res,"can not get the total number of users who want to be notified by e-mail on some event");

   /* Get number of users */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumUsrsTotalWhoWantToBeNotifiedAboutSomeEvent) != 1)
      Lay_ShowErrorAndExit ("Error when getting the total number of users who want to be notified by e-mail on some event.");

   /***** For each notify event... *****/
   for (NotifyEvent=(Ntf_NotifyEvent_t) 0; NotifyEvent<Ntf_NUM_NOTIFY_EVENTS; NotifyEvent++)
     {
      /***** Get the number of users who want to be notified by e-mail on this event, from database *****/
      switch (Gbl.Stat.LocationRange)
        {
         case Sta_LOCATION_RANGE_ALL:
            sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE ((NotifyEvents & '%u') <> 0)",
                     (1 << NotifyEvent));
            break;
         case Sta_LOCATION_RANGE_THIS_DEGREE:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM courses,crs_usr,usr_data" \
                           " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod" \
                           " AND ((usr_data.NotifyEvents & '%u') <> 0)",
                     Gbl.CurrentDeg.Deg.DegCod,(1 << NotifyEvent));
            break;
         case Sta_LOCATION_RANGE_THIS_COURSE:
         default:
            sprintf (Query,"SELECT COUNT(DISTINCT usr_data.UsrCod) FROM crs_usr,usr_data" \
                           " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod" \
                           " AND ((usr_data.NotifyEvents & '%u') <> 0)",
                     Gbl.CurrentCrs.Crs.CrsCod,(1 << NotifyEvent));
        }
      DB_QuerySELECT (Query,&mysql_res,"can not get the number of users who want to be notified by e-mail on an event");

      /* Get number of users */
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%u",&NumUsrs[NotifyEvent]) != 1)
         Lay_ShowErrorAndExit ("Error when getting the number of users who want to be notified by e-mail on an event.");

      /* Free structure that stores the query result */
      DB_FreeMySQLResult (&mysql_res);

      /***** Get number of notifications by e-mail from database *****/
      switch (Gbl.Stat.LocationRange)
        {
         case Sta_LOCATION_RANGE_ALL:
            sprintf (Query,"SELECT SUM(NumEvents),SUM(NumMails) FROM sta_notif WHERE NotifyEvent='%u'",
                     (unsigned) NotifyEvent);
            break;
         case Sta_LOCATION_RANGE_THIS_DEGREE:
            sprintf (Query,"SELECT SUM(NumEvents),SUM(NumMails) FROM sta_notif" \
                           " WHERE sta_notif.DegCod='%ld' AND NotifyEvent='%u'",
                     Gbl.CurrentDeg.Deg.DegCod,(unsigned) NotifyEvent);
            break;
         case Sta_LOCATION_RANGE_THIS_COURSE:
         default:
            sprintf (Query,"SELECT SUM(NumEvents),SUM(NumMails) FROM sta_notif" \
                           " WHERE sta_notif.CrsCod='%ld' AND  NotifyEvent='%u'",
                     Gbl.CurrentCrs.Crs.CrsCod,(unsigned) NotifyEvent);
        }
      DB_QuerySELECT (Query,&mysql_res,"can not get the number of notifications by e-mail");

      row = mysql_fetch_row (mysql_res);

      /* Get number of events notified */
      if (row[0])
        {
         if (sscanf (row[0],"%u",&NumEvents[NotifyEvent]) != 1)
            Lay_ShowErrorAndExit ("Error when getting the number of notifications by e-mail.");
        }
      else
         NumEvents[NotifyEvent] = 0;

      /* Get number of mails sent */
      if (row[1])
        {
         if (sscanf (row[1],"%u",&NumMails[NotifyEvent]) != 1)
            Lay_ShowErrorAndExit ("Error when getting the number of e-mails to notify events3.");
        }
      else
         NumMails[NotifyEvent] = 0;

      /* Free structure that stores the query result */
      DB_FreeMySQLResult (&mysql_res);

      /* Update total number of events and mails */
      NumEventsTotal += NumEvents[NotifyEvent];
      NumMailsTotal += NumMails[NotifyEvent];
     }

   /***** Write number of users who want to be notified by e-mail on each event *****/
   for (NotifyEvent=(Ntf_NotifyEvent_t) 0; NotifyEvent<Ntf_NUM_NOTIFY_EVENTS; NotifyEvent++)
      fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">%s&nbsp;</td>" \
                         "<td align=\"right\" class=\"DAT\">%u</td>" \
                         "<td align=\"right\" class=\"DAT\">%5.2f%%</td>" \
                         "<td align=\"right\" class=\"DAT\">%u</td>" \
                         "<td align=\"right\" class=\"DAT\">%u</td></tr>",
               Txt_NOTIFY_EVENTS[NotifyEvent][Gbl.Prefs.Language],
               NumUsrs[NotifyEvent],
               NumUsrsTotalInPlatform ? 100.0 * (float) NumUsrs[NotifyEvent] / (float) NumUsrsTotalInPlatform : 0.0,
               NumEvents[NotifyEvent],
               NumMails[NotifyEvent]);

   /***** Write total number of users who want to be notified by e-mail on some event *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"DAT\">%s&nbsp;</td>" \
                      "<td align=\"right\" class=\"DAT\">%u</td>" \
                      "<td align=\"right\" class=\"DAT\">%5.2f%%</td>" \
                      "<td align=\"right\" class=\"DAT\">%u</td>" \
                      "<td align=\"right\" class=\"DAT\">%u</td></tr>",
            Txt_Total[Gbl.Prefs.Language],
            NumUsrsTotalWhoWantToBeNotifiedAboutSomeEvent,
            NumUsrsTotalInPlatform ? 100.0 * (float) NumUsrsTotalWhoWantToBeNotifiedAboutSomeEvent / (float) NumUsrsTotalInPlatform : 0.0,
            NumEventsTotal,
            NumMailsTotal);
  }

/****************************************************************************/
/******************* Request showing statistics of courses ******************/
/****************************************************************************/

void Sta_ReqStatsCourses (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_STA_USE_STAT_TYPES[Sta_NUM_TYPES_USE_STATS][Txt_NUM_LANGUAGES];
   extern const char *Txt_Show_statistics[Txt_NUM_LANGUAGES];

   /***** Start form *****/
   Act_FormStart (ActSeeStaCrs);
   fprintf (Gbl.F.Out,"<div align=\"center\"><table>\n");

   /***** Compute stats for anywhere, degree or course? *****/
   Sta_WriteSelectorDegCrs (Sta_LOCATION_RANGE_ALL);

   /***** Submit button *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\">" \
                      "<input type=\"submit\" value=\"%s\" />" \
                      "</td></tr>\n",
            Txt_Show_statistics[Gbl.Prefs.Language]);

   /***** Form end *****/
   fprintf (Gbl.F.Out,"</table></div></form>\n");
  }

/****************************************************************************/
/************************ Show statistics of courses ************************/
/****************************************************************************/

void Sta_ShowStatsCourses (void)
  {
   /***** Show again the form to see statistics of courses *****/
   Sta_ReqStatsCourses ();

   /***** Show number of degrees and courses *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   Sta_GetAndShowNumDegsInSWAD ();
   Sta_GetAndShowNumCrssInSWAD ();
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Show the stats of courses *****/
   Sta_GetAndShowTableOfCourses ();
  }

/****************************************************************************/
/****************** Get and show total number of courses ********************/
/****************************************************************************/

static void Sta_GetAndShowTableOfCourses (void)
  {
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_Course[Txt_NUM_LANGUAGES];
   extern const char *Txt_Web_page_of_the_course[Txt_NUM_LANGUAGES];
   extern const char *Txt_Teach[Txt_NUM_LANGUAGES];
   extern const char *Txt_Stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_Indicators[Txt_NUM_LANGUAGES];
   extern const char *Txt_Syllabus[Txt_NUM_LANGUAGES];
   extern const char *Txt_Teaching_guide[Txt_NUM_LANGUAGES];
   extern const char *Txt_Syllabus_lectures[Txt_NUM_LANGUAGES];
   extern const char *Txt_Syllabus_practices[Txt_NUM_LANGUAGES];
   extern const char *Txt_Assignments[Txt_NUM_LANGUAGES];
   extern const char *Txt_Files_assignments[Txt_NUM_LANGUAGES];
   extern const char *Txt_Files_works[Txt_NUM_LANGUAGES];
   extern const char *Txt_Online_tutoring[Txt_NUM_LANGUAGES];
   extern const char *Txt_Forum_threads[Txt_NUM_LANGUAGES];
   extern const char *Txt_Forum_posts[Txt_NUM_LANGUAGES];
   extern const char *Txt_Messages_sent_by_teachers[Txt_NUM_LANGUAGES];
   extern const char *Txt_Assessment[Txt_NUM_LANGUAGES];
   extern const char *Txt_Assessment_system[Txt_NUM_LANGUAGES];
   extern const char *Txt_Yes[Txt_NUM_LANGUAGES];
   extern const char *Txt_No[Txt_NUM_LANGUAGES];
   extern const char *Txt_INFO_SRC_SHORT_TEXT[Crs_NUM_INFO_SOURCES][Txt_NUM_LANGUAGES];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long CrsCod;
   unsigned NumTchs;
   unsigned NumStds;
   bool ThereIsSyllabus;
   Crs_InfoSrc_t TeachingGuideSrc;
   Crs_InfoSrc_t SyllabusTheSrc;
   Crs_InfoSrc_t SyllabusPraSrc;
   bool ThereAreAssignments;
   unsigned NumAssignments;
   unsigned long NumFilesAssignments;
   unsigned long NumFilesWorks;
   bool ThereIsOnlineTutoring;
   unsigned NumThreads;
   unsigned NumPosts;
   unsigned NumUsrsToBeNotified;
   unsigned NumMsgsSentByTchs;
   bool ThereIsAssessment;
   Crs_InfoSrc_t AssessmentSrc;
   bool CoursePartiallyOK;
   bool CourseAllOK;

   /***** Table start *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write table heading *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\"><table cellspacing=\"2\" cellpadding=\"0\">" \
                      "<tr>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\" rowspan=\"3\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\" rowspan=\"3\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\" rowspan=\"3\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\" rowspan=\"3\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\" rowspan=\"3\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\" colspan=\"15\">%s</td>" \
                      "</tr>" \
                      "<tr>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\" colspan=\"4\">1.%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\" colspan=\"4\">2.%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\" colspan=\"4\">3.%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\" colspan=\"3\">4.%s</td>" \
                      "</tr>" \
                      "<tr>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\"></td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\"></td>" \
                      "<td bgcolor=\"#%s\" align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\"></td>" \
                      "<td bgcolor=\"#%s\" align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"right\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"center\" class=\"TIT_TBL\"></td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "<td bgcolor=\"#%s\" align=\"left\" class=\"TIT_TBL\">%s</td>" \
                      "</tr>",
            Gbl.ColorRows[0],Txt_Degree[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Course[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Web_page_of_the_course[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Teach[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Stud[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Indicators[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Syllabus[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Assignments[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Online_tutoring[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Assessment[Gbl.Prefs.Language],
            Gbl.ColorRows[0],
            Gbl.ColorRows[0],Txt_Teaching_guide[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Syllabus_lectures[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Syllabus_practices[Gbl.Prefs.Language],
            Gbl.ColorRows[0],
            Gbl.ColorRows[0],Txt_Assignments[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Files_assignments[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Files_works[Gbl.Prefs.Language],
            Gbl.ColorRows[0],
            Gbl.ColorRows[0],Txt_Forum_threads[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Forum_posts[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Messages_sent_by_teachers[Gbl.Prefs.Language],
            Gbl.ColorRows[0],
            Gbl.ColorRows[0],Txt_Teaching_guide[Gbl.Prefs.Language],
            Gbl.ColorRows[0],Txt_Assessment_system[Gbl.Prefs.Language]);

   /***** Get courses from database *****/
   switch (Gbl.Stat.LocationRange)
     {
      case Sta_LOCATION_RANGE_ALL:
         sprintf (Query,"SELECT degrees.FullName,courses.FullName,courses.CrsCod" \
                        " FROM degrees,courses" \
                        " WHERE degrees.DegCod=courses.DegCod" \
                        " ORDER BY degrees.FullName,courses.FullName");
         break;
      case Sta_LOCATION_RANGE_THIS_DEGREE:
         sprintf (Query,"SELECT degrees.FullName,courses.FullName,courses.CrsCod" \
                        " FROM degrees,courses" \
                        " WHERE degrees.DegCod='%ld' AND degrees.DegCod=courses.DegCod" \
                        " ORDER BY degrees.FullName,courses.FullName",
                  Gbl.CurrentDeg.Deg.DegCod);
         break;
      case Sta_LOCATION_RANGE_THIS_COURSE:
      default:
         sprintf (Query,"SELECT degrees.FullName,courses.FullName,courses.CrsCod" \
                        " FROM degrees,courses" \
                        " WHERE courses.CrsCod='%ld' AND degrees.DegCod=courses.DegCod" \
                        " ORDER BY degrees.FullName,courses.FullName",
                  Gbl.CurrentCrs.Crs.CrsCod);
     }
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get courses");

   /***** List courses *****/
   for (Gbl.GlobalRowEvenOdd = 1, NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Get next course */
      row = mysql_fetch_row (mysql_res);

      /* Get course code (row[2]) */
      if ((CrsCod = Str_ConvertStrCodToLongCod (row[2])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of course.");

      /* Get number of users */
      NumStds = Usr_GetNumStdsInCrs (CrsCod);
      NumTchs = Usr_GetNumTchsInCrs (CrsCod);

      /* Get information about syllabus */
      TeachingGuideSrc = Crs_GetInfoSrcFromDB (CrsCod,Crs_TEACHING_GUIDE);
      SyllabusTheSrc   = Crs_GetInfoSrcFromDB (CrsCod,Crs_LECTURES);
      SyllabusPraSrc   = Crs_GetInfoSrcFromDB (CrsCod,Crs_PRACTICALS);
      ThereIsSyllabus = (TeachingGuideSrc != Crs_INFO_SRC_NONE) ||
                        (SyllabusTheSrc   != Crs_INFO_SRC_NONE) ||
                        (SyllabusPraSrc   != Crs_INFO_SRC_NONE);
      // Discard syllabus with internal editor selected but no xml file !!!!!!!!!!!!!!!!!!!!!!!!!

      /* Get information about assignments */
      NumAssignments = Asg_GetNumAssigmentsInCrs (CrsCod);
      NumFilesAssignments = Sta_GetNumFilesOfCrsFileZoneFromDB (Brw_FILE_BRW_ASSIGNMENTS_USR,CrsCod);
      NumFilesWorks       = Sta_GetNumFilesOfCrsFileZoneFromDB (Brw_FILE_BRW_WORKS_USR      ,CrsCod);
      ThereAreAssignments = (NumAssignments != 0) || (NumFilesAssignments != 0) || (NumFilesWorks != 0);

      /* Get information about online tutoring */
      NumThreads = For_GetNumTotalThrsInForumsOfType (For_FORUM_COURSE_USRS,-1L,CrsCod);
      NumPosts   = For_GetNumTotalPstsInForumsOfType (For_FORUM_COURSE_USRS,-1L,CrsCod,&NumUsrsToBeNotified);
      NumMsgsSentByTchs = Msg_GetNumMsgsSentByTchsCrs (CrsCod);
      ThereIsOnlineTutoring = (NumThreads != 0) || (NumPosts != 0) || (NumMsgsSentByTchs != 0);

      /* Get information about assessment */
      AssessmentSrc = Crs_GetInfoSrcFromDB (CrsCod,Crs_ASSESSMENT);
      ThereIsAssessment = (TeachingGuideSrc != Crs_INFO_SRC_NONE) ||
                          (AssessmentSrc    != Crs_INFO_SRC_NONE);

      /* All the indicators are OK? */
      CoursePartiallyOK = ThereIsSyllabus || ThereAreAssignments || ThereIsOnlineTutoring || ThereIsAssessment;
      CourseAllOK       = ThereIsSyllabus && ThereAreAssignments && ThereIsOnlineTutoring && ThereIsAssessment;

      /* Write a row for this course */
      fprintf (Gbl.F.Out,"<tr>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"DAT_SMALL\"><a href=\"https://%s/?CrsCod=%ld\" target=\"_blank\">https://%s/?CrsCod=%ld</a></td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%u</td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%u</td>" \
                         "<td bgcolor=\"#%s\" align=\"center\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"center\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%u</td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%lu</td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%lu</td>" \
                         "<td bgcolor=\"#%s\" align=\"center\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%u</td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%u</td>" \
                         "<td bgcolor=\"#%s\" align=\"right\" class=\"%s\">%u</td>" \
                         "<td bgcolor=\"#%s\" align=\"center\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"%s\">%s</td>" \
                         "<td bgcolor=\"#%s\" align=\"left\" class=\"%s\">%s</td>" \
                         "</tr>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],CourseAllOK ? "DAT_SMALL_GREEN" : (CoursePartiallyOK ? "DAT_SMALL" : "DAT_SMALL_RED"),row[0],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],CourseAllOK ? "DAT_SMALL_GREEN" : (CoursePartiallyOK ? "DAT_SMALL" : "DAT_SMALL_RED"),row[1],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],Cfg_PLATFORM_URL,CrsCod,Cfg_PLATFORM_URL,CrsCod,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],(NumTchs != 0) ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumTchs,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],(NumStds != 0) ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumStds,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsSyllabus ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",ThereIsSyllabus ? Txt_Yes[Gbl.Prefs.Language] : Txt_No[Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsSyllabus ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",Txt_INFO_SRC_SHORT_TEXT[TeachingGuideSrc][Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsSyllabus ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",Txt_INFO_SRC_SHORT_TEXT[SyllabusTheSrc][Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsSyllabus ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",Txt_INFO_SRC_SHORT_TEXT[SyllabusPraSrc][Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereAreAssignments ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",ThereAreAssignments ? Txt_Yes[Gbl.Prefs.Language] : Txt_No[Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereAreAssignments ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumAssignments,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereAreAssignments ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumFilesAssignments,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereAreAssignments ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumFilesWorks,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsOnlineTutoring ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",ThereIsOnlineTutoring ? Txt_Yes[Gbl.Prefs.Language] : Txt_No[Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsOnlineTutoring ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumThreads,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsOnlineTutoring ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumPosts,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsOnlineTutoring ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",NumMsgsSentByTchs,
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsAssessment ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",ThereIsAssessment ? Txt_Yes[Gbl.Prefs.Language] : Txt_No[Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsAssessment ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",Txt_INFO_SRC_SHORT_TEXT[TeachingGuideSrc][Gbl.Prefs.Language],
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],ThereIsAssessment ? "DAT_SMALL_GREEN" : "DAT_SMALL_RED",Txt_INFO_SRC_SHORT_TEXT[AssessmentSrc][Gbl.Prefs.Language]);

      Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** End table *****/
   fprintf (Gbl.F.Out,"</table></td></tr>");
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/****************************************************************************/
/******** Calculate average photos of all students from each degree *********/
/****************************************************************************/

void Sta_CalcPhotoDegree (void)
  {
   Sta_AvgPhotoTypeOfAverage_t TypeOfAverage;
   long DegCod = -1L;
   char DirAvgPhotosRelPath[Sta_NUM_AVERAGE_PHOTO_TYPES][PATH_MAX+1];
   unsigned NumStds,NumStdsWithPhoto;
   Usr_Gender_t Gender;
   long TotalTimeToComputeAvgPhotoInMicroseconds,PartialTimeToComputeAvgPhotoInMicroseconds;

   /***** Get type of average *****/
   Gbl.Stat.DegPhotos.TypeOfAverage = Sta_GetPhotoAvgTypeFromForm ();

   /***** Creates public directories for average photos if not exist *****/
   Fil_CreateDirIfNotExists (Cfg_PATH_REL_PHOTOS_PUBLIC);
   for (TypeOfAverage=(Sta_AvgPhotoTypeOfAverage_t) 0; TypeOfAverage<Sta_NUM_AVERAGE_PHOTO_TYPES; TypeOfAverage++)
     {
      sprintf (DirAvgPhotosRelPath[TypeOfAverage],"%s/%s",Cfg_PATH_REL_PHOTOS_PUBLIC,Sta_StrAvgPhotoDirs[TypeOfAverage]);
      Fil_CreateDirIfNotExists (DirAvgPhotosRelPath[TypeOfAverage]);
     }

   /***** Creates private directory for lists of users' photos if not exists *****/
   Fil_CreateDirIfNotExists (Cfg_PATH_REL_PHOTOS_TMP_PRIVATE);

   /***** Remove old private files used for lists *****/
   Brw_RemoveOldTmpFiles (Cfg_PATH_REL_PHOTOS_TMP_PRIVATE,false);

   /***** Get the degree which photo will be computed *****/
   if ((DegCod = Deg_GetParamOtherDegCod ()) == -1L)
      Lay_ShowErrorAndExit ("Code of degree is missing.");

   /***** Prevent the computing of an average photo too recently updated *****/
   if (Sta_GetTimeAvgPhotoWasComputed (DegCod) >=
       Gbl.TimeStartExecution - Cfg_MIN_TIME_TO_RECOMPUTE_AVG_PHOTO)
      Lay_ShowErrorAndExit ("Average photo has been computed recently.");

   /***** Get list of students in this degree *****/
   Usr_GetStdsLstDeg (DegCod);

   for (Gender=(Usr_Gender_t) 0; Gender<Usr_NUM_GENDERS; Gender++)
     {
      TotalTimeToComputeAvgPhotoInMicroseconds = 0;
      for (TypeOfAverage=(Sta_AvgPhotoTypeOfAverage_t) 0; TypeOfAverage<Sta_NUM_AVERAGE_PHOTO_TYPES; TypeOfAverage++)
        {
         /***** Compute average photos of students belonging this degree *****/
         Sta_ComputeAveragePhoto (DegCod,Gender,&Gbl.Usrs.LstStds,
                                  TypeOfAverage,DirAvgPhotosRelPath[TypeOfAverage],
                                  &NumStds,&NumStdsWithPhoto,&PartialTimeToComputeAvgPhotoInMicroseconds);
         TotalTimeToComputeAvgPhotoInMicroseconds += PartialTimeToComputeAvgPhotoInMicroseconds;
        }

      /***** Store stats in database *****/
      Sta_UpdateDegStats (DegCod,Gender,NumStds,NumStdsWithPhoto,TotalTimeToComputeAvgPhotoInMicroseconds);
     }

   /***** Free memory for students list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /***** Show photos *****/
   Sta_ShowOrPrintPhotoDegree (Sta_DEGREES_SEE);
  }

/*****************************************************************************/
/******** Get degree code with average photo least recently updated **********/
/*****************************************************************************/
// Returns -1 when error or when all the degrees have been updated too recently

static long Sta_GetDegWithAvgPhotoLeastRecentlyUpdated (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows = 0;
   long DegCod = -1L;

   /***** Delete all the degrees in sta_degrees table not present in degrees table *****/
   Sta_RemoveObsoleteStatDegrees ();

   /***** 1. If any degree is not in table, choose it as least recently updated *****/
   /* Get degrees from database */
   sprintf (Query,"SELECT DegCod FROM degrees WHERE DegCod NOT IN (SELECT DISTINCT DegCod FROM sta_degrees) ORDER BY RAND() LIMIT 1");
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees");

   /* If number of rows is 1, then get the degree code */
   if (NumRows == 1)
     {
      /* Get row */
      row = mysql_fetch_row (mysql_res);

      /* Get degree code (row[0]) */
      if ((DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of degree.");
     }
   else   // NumRows == 0
     {
      /* Free structure that stores the query result */
      DB_FreeMySQLResult (&mysql_res);

      /***** 2. If all the degrees are in table, choose the least recently updated *****/
      /* Get degrees from database */
      sprintf (Query,"SELECT DegCod FROM sta_degrees WHERE UNIX_TIMESTAMP(TimeAvgPhoto)<UNIX_TIMESTAMP()-%ld ORDER BY TimeAvgPhoto LIMIT 1",
               Cfg_MIN_TIME_TO_RECOMPUTE_AVG_PHOTO);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees");

      /* If number of rows is 1, then get the degree code */
      if (NumRows == 1)
        {
         /* Get row */
        row = mysql_fetch_row (mysql_res);

         /* Get degree code (row[0]) */
         if ((DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
            Lay_ShowErrorAndExit ("Wrong code of degree.");
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return DegCod;
  }

/*****************************************************************************/
/* Delete all the degrees in sta_degrees table not present in degrees table **/
/*****************************************************************************/

void Sta_RemoveObsoleteStatDegrees (void)
  {
   char Query[512];

   sprintf (Query,"DELETE FROM sta_degrees" \
                  " WHERE DegCod NOT IN (SELECT DegCod FROM degrees)");
   DB_QueryDELETE (Query,"can not remove old degrees from stats");
  }

/*****************************************************************************/
/***************** Get time when average photo was computed ******************/
/*****************************************************************************/

static long Sta_GetTimeAvgPhotoWasComputed (long DegCod)
  {
   char Query[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   long TimeAvgPhotoWasComputed = 0;

   /***** Get last time an average photo was computed from database *****/
   sprintf (Query,"SELECT MIN(UNIX_TIMESTAMP(TimeAvgPhoto))" \
                  " FROM sta_degrees WHERE DegCod='%ld'",DegCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get last time an average photo was computed");

   if (NumRows == 1)
     {
      /***** Get row *****/
      row = mysql_fetch_row (mysql_res);

      /***** Get time when average photo was computed (row[0]) *****/
      if (row[0])
         if (row[0][0])
            if (sscanf (row[0],"%ld",&TimeAvgPhotoWasComputed) != 1)
               TimeAvgPhotoWasComputed = 0;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return TimeAvgPhotoWasComputed;
  }

/*****************************************************************************/
/********************* Get time to compute average photo *********************/
/*****************************************************************************/

static long Sta_GetTimeToComputeAvgPhoto (long DegCod)
  {
   char Query[256];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   Usr_Gender_t Gender;
   long PartialTimeToComputeAvgPhoto,TotalTimeToComputeAvgPhoto = -1L;

   /***** Get time to compute average photo from database *****/
   sprintf (Query,"SELECT TimeToComputeAvgPhoto FROM sta_degrees" \
                  " WHERE DegCod='%ld'",DegCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get time to compute average photo");

   /***** Count number of rows in result *****/
   if (NumRows == Usr_NUM_GENDERS)
     {
      TotalTimeToComputeAvgPhoto = 0;
      for (Gender=(Usr_Gender_t) 0; Gender<Usr_NUM_GENDERS; Gender++)
        {
         /***** Get row *****/
         row = mysql_fetch_row (mysql_res);

         /***** Get time to compute average photo (row[0]) *****/
         PartialTimeToComputeAvgPhoto = -1L;
         if (row[0][0])
            if (sscanf (row[0],"%ld",&PartialTimeToComputeAvgPhoto) != 1)
               PartialTimeToComputeAvgPhoto = -1L;
         if (PartialTimeToComputeAvgPhoto < 0)
           {
            TotalTimeToComputeAvgPhoto = -1L;
            break;
           }
         TotalTimeToComputeAvgPhoto += PartialTimeToComputeAvgPhoto;
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return TotalTimeToComputeAvgPhoto;
  }

/*****************************************************************************/
/********************** Compute the average photo of a list ******************/
/*****************************************************************************/
// Returns number of users in list with photo

static void Sta_ComputeAveragePhoto (long DegCod,Usr_Gender_t Gender,struct ListUsers *LstUsrs,
                                     Sta_AvgPhotoTypeOfAverage_t TypeOfAverage,const char *DirAvgPhotosRelPath,
                                     unsigned *NumStds,unsigned *NumStdsWithPhoto,long *TimeToComputeAvgPhotoInMicroseconds)
  {
   extern const char *StringsGenderDB[Usr_NUM_GENDERS];
   unsigned NumUsr;
   char PathPrivRelPhoto[PATH_MAX+1];	// Relative path to private photo, to calculate average face
   struct UsrData *UsrDat;
   char AvgPhotoRelPath[PATH_MAX+1];
   char FileNamePhotoNames[PATH_MAX+1];
   FILE *FilePhotoNames = NULL;	// Initialized to avoid warning
   char StrCallToProgram[256];	// Call to photo processing program
   int ReturnCode;
   /* To compute execution time of this function */
   struct timeval tvStartComputingStat,tvEndComputingStat;
   struct timezone tz;

   /***** Set start time to compute the stats of this degree *****/
   gettimeofday (&tvStartComputingStat,&tz);

   /***** Reset number of students in this degree *****/
   *NumStds = *NumStdsWithPhoto = 0;

   /***** Build name for file with average photo *****/
   sprintf (AvgPhotoRelPath,"%s/%ld_%s.jpg",DirAvgPhotosRelPath,DegCod,StringsGenderDB[Gender]);

   /***** Remove old file if exists *****/
   if (Fil_CheckIfPathExists (AvgPhotoRelPath))  // If file exists
      unlink (AvgPhotoRelPath);

   if (Gbl.Usrs.LstStds.NumUsrs >= Cfg_MIN_PHOTOS_TO_COMPUTE_AVERAGE)
     {
      /***** Build names for text file with photo paths *****/
      sprintf (FileNamePhotoNames,"%s/%ld.txt",Cfg_PATH_REL_PHOTOS_TMP_PRIVATE,DegCod);
      if ((FilePhotoNames = fopen (FileNamePhotoNames,"wt")) == NULL)
         Lay_ShowErrorAndExit ("Can not open file to compute average photo.");

      /***** Loop writing file names in text file *****/
      for (NumUsr=0; NumUsr<LstUsrs->NumUsrs; NumUsr++)
        {
         UsrDat = &(LstUsrs->Lst[LstUsrs->Ind[NumUsr]]);
         if (Gender == Usr_GENDER_ALL || Gender == UsrDat->Gender)
           {
            (*NumStds)++;
            /***** Add photo to file for average face calculation *****/
            if (Usr_CheckIfPrivPhotoExists (UsrDat,PathPrivRelPhoto))
              {
               (*NumStdsWithPhoto)++;
               fprintf (FilePhotoNames,"%s\n",PathPrivRelPhoto);
              }
           }
        }
      fclose (FilePhotoNames);

      /***** Call to program to calculate average photo *****/
      if (*NumStdsWithPhoto >= Cfg_MIN_PHOTOS_TO_COMPUTE_AVERAGE)
        {
         sprintf (StrCallToProgram,"%s %s %s",Sta_StrAvgPhotoPrograms[TypeOfAverage],FileNamePhotoNames,AvgPhotoRelPath);

         ReturnCode = system (StrCallToProgram);
         if (ReturnCode == -1)
            Lay_ShowErrorAndExit ("Error when running program that computes the average photo.");

         /* Write message depending on the return code */
         if (WEXITSTATUS(ReturnCode))
            Lay_ShowErrorAndExit ("The average photo has not been computed successfully.");
        }
     }

   /***** Time used to compute the stats of this degree *****/
   if (gettimeofday (&tvEndComputingStat, &tz))
      // Error in gettimeofday
      *TimeToComputeAvgPhotoInMicroseconds = -1L;
   else
     {
      if (tvEndComputingStat.tv_usec < tvStartComputingStat.tv_usec)
	{
	 tvEndComputingStat.tv_sec--;
	 tvEndComputingStat.tv_usec += 1000000L;
	}
      *TimeToComputeAvgPhotoInMicroseconds = (tvEndComputingStat.tv_sec - tvStartComputingStat.tv_sec) * 1000000L + tvEndComputingStat.tv_usec - tvStartComputingStat.tv_usec;
     }
  }

/****************************************************************************/
/** Show class photo with average photos of all students from each degree ***/
/****************************************************************************/

void Sta_ShowPhotoDegree (void)
  {
   /***** Get type of average *****/
   Gbl.Stat.DegPhotos.TypeOfAverage = Sta_GetPhotoAvgTypeFromForm ();

   Sta_ShowOrPrintPhotoDegree (Sta_DEGREES_SEE);
  }

/****************************************************************************/
/** Print class photo with average photos of all students from each degree **/
/****************************************************************************/

void Sta_PrintPhotoDegree (void)
  {
   /***** Get type of average *****/
   Gbl.Stat.DegPhotos.TypeOfAverage = Sta_GetPhotoAvgTypeFromForm ();

   Sta_ShowOrPrintPhotoDegree (Sta_DEGREES_PRINT);
  }

/****************************************************************************/
/** Show class photo with average photos of all students from each degree ***/
/****************************************************************************/

void Sta_ShowOrPrintPhotoDegree (Sta_AvgPhotoSeeOrPrint_t SeeOrPrint)
  {
   /***** Get photo size from form *****/
   Gbl.Stat.DegPhotos.HowComputePhotoSize = Sta_GetHowComputePhotoSizeFromForm ();

   /***** Get how to order degrees from form *****/
   Gbl.Stat.DegPhotos.HowOrderDegrees = Sta_GetHowOrderDegreesFromForm ();

   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   if (SeeOrPrint == Sta_DEGREES_SEE)
     {
      /***** Put a selector for the type of average *****/
      Sta_PutSelectorForTypeOfAvg ();

      /***** Put a selector for the size of photos *****/
      Sta_PutSelectorForHowComputePhotoSize ();

      /***** Put a selector for the order of degrees *****/
      Sta_PutSelectorForHowOrderDegrees ();

      /***** Form to select type of list used to display degree photos *****/
      Usr_ShowFormsToSelectUsrListType (ActSeePhoDeg);

      /***** Link to print view *****/
      Sta_PutLinkToPrintViewOfDegreeStats ();

      /***** Link to computation of average photos *****/
      Sta_PutLinkToCalculateDegreeStats ();
     }

   /***** Get maximum number of students and maximum number of students with photo in all degrees *****/
   Sta_GetMaxStdsPerDegree ();

   /***** Draw the classphoto/list *****/
   switch (Gbl.Usrs.Me.ListType)
     {
      case Usr_CLASS_PHOTO:
         Sta_ShowOrPrintClassPhotoDegrees (SeeOrPrint);
         break;
      case Usr_LIST:
         Sta_ShowOrPrintListDegrees (SeeOrPrint);
         break;
     }
  }

/****************************************************************************/
/******************* Put a selector for the type of average *****************/
/****************************************************************************/

static void Sta_PutSelectorForTypeOfAvg (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Average_type[Txt_NUM_LANGUAGES];
   Sta_AvgPhotoTypeOfAverage_t TypeOfAvg;

   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActSeePhoDeg);
   Sta_PutHiddenParamPhotoSize ();
   Sta_PutHiddenParamOrderDegrees ();
   Usr_PutParamColsClassPhoto ();
   fprintf (Gbl.F.Out,"<span class=\"%s\">%s: </span>" \
                      "<select name=\"AvgType\" onchange=\"javascript:document.forms[%d].submit();\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Average_type[Gbl.Prefs.Language],
            Gbl.NumForm);
   for (TypeOfAvg=(Sta_AvgPhotoTypeOfAverage_t) 0; TypeOfAvg<Sta_NUM_AVERAGE_PHOTO_TYPES; TypeOfAvg++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) TypeOfAvg);
      if (TypeOfAvg == Gbl.Stat.DegPhotos.TypeOfAverage)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s",Sta_StrAvgPhotoTypes[TypeOfAvg]);
     }
   fprintf (Gbl.F.Out,"</select></form></div>\n");
  }

/****************************************************************************/
/**************** Put hidden parameter for the type of average **************/
/****************************************************************************/

void Sta_PutHiddenParamTypeOfAvg (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"AvgType\" value=\"%u\" />",(unsigned) Gbl.Stat.DegPhotos.TypeOfAverage);
  }

/****************************************************************************/
/*********************** Get type of average for photos *********************/
/****************************************************************************/

static Sta_AvgPhotoTypeOfAverage_t Sta_GetPhotoAvgTypeFromForm (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"AvgType",UnsignedStr,10);
   if (UnsignedStr[0])
     {
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         Lay_ShowErrorAndExit ("Type of average is missing.");
      if (UnsignedNum >= Sta_NUM_AVERAGE_PHOTO_TYPES)
         Lay_ShowErrorAndExit ("Type of average is missing.");
      return (Sta_AvgPhotoTypeOfAverage_t) UnsignedNum;
     }
   return (Sta_AvgPhotoTypeOfAverage_t) 0;
  }

/****************************************************************************/
/****************** Put a selector for the size of photos *******************/
/****************************************************************************/

static void Sta_PutSelectorForHowComputePhotoSize (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Size_of_photos[Txt_NUM_LANGUAGES];
   Sta_HowComputePhotoSize_t PhoSi;

   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActSeePhoDeg);
   Sta_PutHiddenParamTypeOfAvg ();
   Sta_PutHiddenParamOrderDegrees ();
   Usr_PutParamColsClassPhoto ();
   fprintf (Gbl.F.Out,"<span class=\"%s\">%s: </span>" \
                      "<select name=\"PhotoSize\" onchange=\"javascript:document.forms[%d].submit();\">",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Size_of_photos[Gbl.Prefs.Language],
            Gbl.NumForm);
   for (PhoSi=(Sta_HowComputePhotoSize_t) 0; PhoSi<Sta_NUM_HOW_COMPUTE_PHOTO_SIZES; PhoSi++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) PhoSi);
      if (PhoSi == Gbl.Stat.DegPhotos.HowComputePhotoSize)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s",Sta_StrPhotoSizes[PhoSi]);
     }
   fprintf (Gbl.F.Out,"</select></form></div>\n");
  }

/****************************************************************************/
/**************** Put hidden parameter for the size of photos ***************/
/****************************************************************************/

void Sta_PutHiddenParamPhotoSize (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"PhotoSize\" value=\"%u\" />",(unsigned) Gbl.Stat.DegPhotos.HowComputePhotoSize);
  }

/****************************************************************************/
/********************* Get how to compute size of photos ********************/
/****************************************************************************/

static Sta_HowComputePhotoSize_t Sta_GetHowComputePhotoSizeFromForm (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"PhotoSize",UnsignedStr,10);
   if (UnsignedStr[0])
     {
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         Lay_ShowErrorAndExit ("Size of photo is missing.");
      if (UnsignedNum >= Sta_NUM_HOW_COMPUTE_PHOTO_SIZES)
         Lay_ShowErrorAndExit ("Size of photo is missing.");
      return (Sta_HowComputePhotoSize_t) UnsignedNum;
     }
   return (Sta_HowComputePhotoSize_t) 0;
  }

/****************************************************************************/
/****************** Put a selector for the order of degrees *****************/
/****************************************************************************/

static void Sta_PutSelectorForHowOrderDegrees (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   Sta_HowOrderDegrees_t Order;

   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActSeePhoDeg);
   Sta_PutHiddenParamTypeOfAvg ();
   Sta_PutHiddenParamPhotoSize ();
   Usr_PutParamColsClassPhoto ();
   fprintf (Gbl.F.Out,"<span class=\"%s\">Ordenar titulaciones por: </span>" \
                      "<select name=\"OrdDeg\" onchange=\"javascript:document.forms[%d].submit();\">",
            The_ClassFormul[Gbl.Prefs.Theme],Gbl.NumForm);
   for (Order=(Sta_HowOrderDegrees_t) 0; Order<Sta_NUM_HOW_ORDER_DEGREES; Order++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) Order);
      if (Order == Gbl.Stat.DegPhotos.HowOrderDegrees)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s",Sta_StrOrderDegrees[Order]);
     }
   fprintf (Gbl.F.Out,"</select></form></div>\n");
  }

/****************************************************************************/
/**************** Put hidden parameter for the order of degrees *************/
/****************************************************************************/

void Sta_PutHiddenParamOrderDegrees (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OrdDeg\" value=\"%u\" />",(unsigned) Gbl.Stat.DegPhotos.HowOrderDegrees);
  }

/****************************************************************************/
/*************************** Get how to order degrees ***********************/
/****************************************************************************/

static Sta_HowOrderDegrees_t Sta_GetHowOrderDegreesFromForm (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Par_GetParam (Par_PARAM_SINGLE,"OrdDeg",UnsignedStr,10);
   if (UnsignedStr[0])
     {
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
         Lay_ShowErrorAndExit ("Sort order is missing.");
      if (UnsignedNum >= Sta_NUM_HOW_ORDER_DEGREES)
         Lay_ShowErrorAndExit ("Sort order is missing.");
      return (Sta_HowOrderDegrees_t) UnsignedNum;
     }
   return (Sta_HowOrderDegrees_t) 0;
  }

/****************************************************************************/
/*************** Put a link to print view the stats of degrees **************/
/****************************************************************************/

static void Sta_PutLinkToPrintViewOfDegreeStats (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_print_view[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<div align=\"center\">" \
                      "<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"left\" valign=\"middle\">");
   Act_FormStart (ActPrnPhoDeg);
   Sta_PutHiddenParamTypeOfAvg ();
   Sta_PutHiddenParamPhotoSize ();
   Sta_PutHiddenParamOrderDegrees ();
   Usr_PutParamUsrListType (Gbl.Usrs.Me.ListType);
   Usr_PutParamColsClassPhoto ();
   Usr_PutParamListWithPhotos ();
   strcpy (Gbl.MsgWindowStatus,Txt_Show_print_view[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_NEW_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/print12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a></form></td></tr></table></div>",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
  }

/****************************************************************************/
/*************** Put a link to calculate the stats of degrees ***************/
/****************************************************************************/

static void Sta_PutLinkToCalculateDegreeStats (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   unsigned NumDeg;
   struct Degree Deg;
   long EstimatedTimeToComputeAvgPhotoInMicroseconds;
   char StrEstimatedTimeToComputeAvgPhoto[64];

   if ((Deg.DegCod = Sta_GetDegWithAvgPhotoLeastRecentlyUpdated ()) > 0)
     {
      /***** Get list of all the degrees *****/
      Deg_GetListAllDegs ();

      /***** Get data of the degree from database *****/
      Deg_GetDataOfDegreeByCod (&Deg);

      /***** Start div and table *****/
      fprintf (Gbl.F.Out,"<div align=\"center\">" \
                         "<table cellspacing=\"0\" cellpadding=\"0\">" \
                         "<tr><td align=\"center\">");

      /***** Start form *****/
      Act_FormStart (ActCalPhoDeg);
      Sta_PutHiddenParamTypeOfAvg ();
      Sta_PutHiddenParamPhotoSize ();
      Sta_PutHiddenParamOrderDegrees ();
      Usr_PutParamUsrListType (Gbl.Usrs.Me.ListType);
      Usr_PutParamColsClassPhoto ();
      Usr_PutParamListWithPhotos ();
      strcpy (Gbl.MsgWindowStatus,"Calcular fotograf&iacute;a promedio de una titulaci&oacute;n");	// Need translation!
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
      fprintf (Gbl.F.Out,"<img src=\"%s/update12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                         " Calcular fotograf&iacute;a promedio de </acronym></a>",	// Need translation!
               Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus);

      /***** Put selector with all the degrees *****/
      fprintf (Gbl.F.Out,"<select name=\"OthDegCod\">");
      for (NumDeg=0; NumDeg<Gbl.Degs.AllDegs.Num; NumDeg++)
        {
         /* Get time to compute average photo of this degree */
         EstimatedTimeToComputeAvgPhotoInMicroseconds = Sta_GetTimeToComputeAvgPhoto (Gbl.Degs.AllDegs.Lst[NumDeg].DegCod);
         if (EstimatedTimeToComputeAvgPhotoInMicroseconds == -1L)
            strcpy (StrEstimatedTimeToComputeAvgPhoto,"desconocido");
         else
            Sta_WriteTime (StrEstimatedTimeToComputeAvgPhoto,EstimatedTimeToComputeAvgPhotoInMicroseconds);

         fprintf (Gbl.F.Out,"<option value=\"%ld\"%s>%s (tiempo: %s)</option>",
                  Gbl.Degs.AllDegs.Lst[NumDeg].DegCod,
                  (Gbl.Degs.AllDegs.Lst[NumDeg].DegCod == Deg.DegCod) ? " selected=\"selected\"" :
                  ((Sta_GetTimeAvgPhotoWasComputed (Gbl.Degs.AllDegs.Lst[NumDeg].DegCod) >= Gbl.TimeStartExecution - Cfg_MIN_TIME_TO_RECOMPUTE_AVG_PHOTO) ? " disabled=\"disabled\"" : ""),
                  Gbl.Degs.AllDegs.Lst[NumDeg].ShortName,
                  StrEstimatedTimeToComputeAvgPhoto);
        }

      /***** End selector, form, table and div *****/
      fprintf (Gbl.F.Out,"</select></form></td></tr></table></div>");
 
      /***** Free list of all the degrees *****/
      Deg_FreeListAllDegs ();
     }
  }

/*****************************************************************************/
/*** Get number of students and number of students with photo in a degree ****/
/*****************************************************************************/

static void Sta_GetMaxStdsPerDegree (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get maximum number of students in a degree from database *****/
   sprintf (Query,"SELECT MAX(NumStds),MAX(NumStdsWithPhoto),MAX(NumStdsWithPhoto/NumStds) FROM sta_degrees WHERE Gender='all' AND NumStds>'0'");
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get maximum number of students in a degree");

   /***** Count number of rows in result *****/
   if (NumRows == 1)
     {
      row = mysql_fetch_row (mysql_res);

      if (row[0] == NULL)
	 Gbl.Stat.DegPhotos.MaxStds = -1;
      else if (sscanf (row[0],"%d",&Gbl.Stat.DegPhotos.MaxStds) != 1)
	 Gbl.Stat.DegPhotos.MaxStds = -1;

      if (row[1] == NULL)
	 Gbl.Stat.DegPhotos.MaxStdsWithPhoto = -1;
      else if (sscanf (row[1],"%d",&Gbl.Stat.DegPhotos.MaxStdsWithPhoto) != 1)
	 Gbl.Stat.DegPhotos.MaxStdsWithPhoto = -1;

      if (row[2] == NULL)
	 Gbl.Stat.DegPhotos.MaxPercent = -1.0;
      else if (sscanf (row[2],"%f",&Gbl.Stat.DegPhotos.MaxPercent) != 1)
	 Gbl.Stat.DegPhotos.MaxPercent = -1.0;

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
   else
     {
      Gbl.Stat.DegPhotos.MaxStds = Gbl.Stat.DegPhotos.MaxStdsWithPhoto = -1;
      Gbl.Stat.DegPhotos.MaxPercent = -1.0;
     }
  }

/****************************************************************************/
/************ Show or print the stats of degrees as class photo *************/
/****************************************************************************/

static void Sta_ShowOrPrintClassPhotoDegrees (Sta_AvgPhotoSeeOrPrint_t SeeOrPrint)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct Degree Deg;
   unsigned NumDegsNotEmpty;
   int NumStds,NumStdsWithPhoto;
   struct Date DateAvgPhoto;
   bool TRIsOpen = false;

   /***** Class photo start *****/
   if (SeeOrPrint == Sta_DEGREES_SEE)
     {
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
      Lay_WriteHeaderClassPhoto (Gbl.Usrs.ClassPhoto.Cols,false,true,-1L,-1L,-1L);
     }
   else
     {
      Lay_StartTableFrame (DARK_BLUE,WHITE,NULL);
      Lay_WriteHeaderClassPhoto (Gbl.Usrs.ClassPhoto.Cols,true,true,-1L,-1L,-1L);
     }

   /***** Get degrees from database *****/
   Sta_BuildQueryOfDegrees (Query);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees");

   /***** Get degrees *****/
   for (NumDegsNotEmpty=0, NumRow=0; NumRow<NumRows; NumRow++)
     {
      /***** Get next degree *****/
      row = mysql_fetch_row (mysql_res);

      /* Get degree code (row[0]) */
      if ((Deg.DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of degree.");

      /* Get data of degree */
      Deg_GetDataOfDegreeByCod (&Deg);

      /* Get number of students and number of students with photo in this degree */
      Sta_GetNumStdsInDegree (Deg.DegCod,Usr_GENDER_ALL,&NumStds,&NumStdsWithPhoto);

      if (NumStds > 0)
        {
         /* Get year, month and day (row[1] holds the date in YYYYMMDD format) */
         if (!(Dat_GetDateFromYYYYMMDD (&DateAvgPhoto,row[1])))
            Lay_ShowErrorAndExit ("Wrong date.");

         if ((NumDegsNotEmpty % Gbl.Usrs.ClassPhoto.Cols) == 0)
           {
            fprintf (Gbl.F.Out,"<tr>\n");
            TRIsOpen = true;
           }

         /***** Show average photo of students belonging to this degree *****/
         fprintf (Gbl.F.Out,"<td align=\"center\">");
         Sta_ShowDegreeAvgPhotoAndStat (&Deg,SeeOrPrint,Usr_GENDER_ALL,NumStds,NumStdsWithPhoto,&DateAvgPhoto);
         fprintf (Gbl.F.Out,"</td>");

         if ((++NumDegsNotEmpty % Gbl.Usrs.ClassPhoto.Cols) == 0)
           {
            fprintf (Gbl.F.Out,"</tr>\n");
            TRIsOpen = false;
           }
        }
     }
   if (TRIsOpen)
      fprintf (Gbl.F.Out,"</tr>\n");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Photos end *****/
   if (SeeOrPrint == Sta_DEGREES_SEE)
     {
      Lay_WriteFootClassPhoto ();
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
   else
     {
      Lay_WriteFootClassPhoto ();
      Lay_EndTableFrame ();
     }
  }

/****************************************************************************/
/**************** Show or print the stats of degrees as list ****************/
/****************************************************************************/

static void Sta_ShowOrPrintListDegrees (Sta_AvgPhotoSeeOrPrint_t SeeOrPrint)
  {
   extern const char *Txt_No_INDEX[Txt_NUM_LANGUAGES];
   extern const char *Txt_Degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_GENDER_PLURAL_Abc[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   unsigned NumDegsNotEmpty;
   int NumStds,NumStdsWithPhoto;
   struct Date DateAvgPhoto;
   struct Degree Deg;
   Usr_Gender_t Gender;

   /***** Class photo start *****/
   if (SeeOrPrint == Sta_DEGREES_SEE)
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
   else
      fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");

   /***** Write heading *****/
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" valign=\"top\" class=\"TIT_TBL\">%s</td>" \
                      "<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>",
            Txt_No_INDEX[Gbl.Prefs.Language],
            Txt_Degree[Gbl.Prefs.Language]);
   for (Gender=(Usr_Gender_t) 0; Gender<Usr_NUM_GENDERS; Gender++)
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" class=\"TIT_TBL\">%s&nbsp;</td>",Txt_GENDER_PLURAL_Abc[Gender][Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</tr>\n");

   /***** Get degrees from database *****/
   Sta_BuildQueryOfDegrees (Query);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get degrees");

   /***** Get degrees *****/
   for (Gbl.GlobalRowEvenOdd = 0,NumDegsNotEmpty=0, NumRow=0; NumRow<NumRows; NumRow++)
     {
      /***** Get next degree *****/
      row = mysql_fetch_row (mysql_res);

      /* Get degree code (row[0]) */
      if ((Deg.DegCod = Str_ConvertStrCodToLongCod (row[0])) < 0)
         Lay_ShowErrorAndExit ("Wrong code of degree.");

      /* Get year, month and day (row[1] holds the date in YYYYMMDD format) */
      if (!(Dat_GetDateFromYYYYMMDD (&DateAvgPhoto,row[1])))
         Lay_ShowErrorAndExit ("Wrong date.");

      /* Get data of degree */
      Deg_GetDataOfDegreeByCod (&Deg);

      /***** Show logo and name of this degree *****/
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" class=\"DAT\" bgcolor=\"#%s\">%u&nbsp;</td>",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd],++NumDegsNotEmpty);

      /***** Show logo and name of this degree *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" class=\"DAT\" bgcolor=\"#%s\">",
               Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
      if (SeeOrPrint == Sta_DEGREES_SEE)
         fprintf (Gbl.F.Out,"<a href=\"%s\" class=\"DAT\" target=\"_blank\"><acronym class=\"DAT\" title=\"%s\">",
                  Deg.WWW,Deg.FullName);
      fprintf (Gbl.F.Out,"<img src=\"%s/%s/%s16x16.gif\" width=\"16\" height=\"16\" align=\"top\" alt=\"%s\" />&nbsp;%s&nbsp;",
               Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_DEGREES,Deg.Logo,Deg.FullName,Deg.ShortName);
      if (SeeOrPrint == Sta_DEGREES_SEE)
         fprintf (Gbl.F.Out,"</acronym></a>");
      fprintf (Gbl.F.Out,"</td");

      for (Gender=(Usr_Gender_t) 0; Gender<Usr_NUM_GENDERS; Gender++)
        {
         /***** Show average photo of students belonging to this degree *****/
         Sta_GetNumStdsInDegree (Deg.DegCod,Gender,&NumStds,&NumStdsWithPhoto);
         fprintf (Gbl.F.Out,"<td align=\"center\" bgcolor=\"#%s\">",Gbl.ColorRows[Gbl.GlobalRowEvenOdd]);
         if (Gbl.Usrs.Listing.WithPhotos)
            Sta_ShowDegreeAvgPhotoAndStat (&Deg,SeeOrPrint,Gender,NumStds,NumStdsWithPhoto,&DateAvgPhoto);
         else
            Sta_ShowDegreeStat (NumStds,NumStdsWithPhoto);
         fprintf (Gbl.F.Out,"</td>");
        }
      fprintf (Gbl.F.Out,"</tr>\n");

      Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Photos end *****/
   if (SeeOrPrint == Sta_DEGREES_SEE)
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
   else
      fprintf (Gbl.F.Out,"</table>\n");
  }

/*****************************************************************************/
/****** Build a query to get the degrees ordered by different criteria *******/
/*****************************************************************************/

static void Sta_BuildQueryOfDegrees (char *Query)
  {
   switch (Gbl.Stat.DegPhotos.HowOrderDegrees)
     {
      case Sta_NUMBER_OF_STUDENTS:
         sprintf (Query,"SELECT degrees.DegCod,DATE_FORMAT(sta_degrees.TimeAvgPhoto,'%%Y%%m%%d') FROM degrees,sta_degrees" \
                        " WHERE sta_degrees.Gender='all' AND sta_degrees.NumStds>'0' AND degrees.DegCod=sta_degrees.DegCod" \
                        " ORDER BY sta_degrees.NumStds DESC,sta_degrees.NumStdsWithPhoto DESC,degrees.ShortName");
         break;
      case Sta_NUMBER_OF_PHOTOS:
         sprintf (Query,"SELECT degrees.DegCod,DATE_FORMAT(sta_degrees.TimeAvgPhoto,'%%Y%%m%%d') FROM degrees,sta_degrees" \
                        " WHERE sta_degrees.Gender='all' AND sta_degrees.NumStds>'0' AND degrees.DegCod=sta_degrees.DegCod" \
                        " ORDER BY sta_degrees.NumStdsWithPhoto DESC,sta_degrees.NumStds DESC,degrees.ShortName");
         break;
      case Sta_PERCENT:
         sprintf (Query,"SELECT degrees.DegCod,DATE_FORMAT(sta_degrees.TimeAvgPhoto,'%%Y%%m%%d') FROM degrees,sta_degrees" \
                        " WHERE sta_degrees.Gender='all' AND sta_degrees.NumStds>'0' AND degrees.DegCod=sta_degrees.DegCod" \
                        " ORDER BY sta_degrees.NumStdsWithPhoto/sta_degrees.NumStds DESC,degrees.ShortName");
         break;
      case Sta_DEGREE_NAME:
         sprintf (Query,"SELECT degrees.DegCod,DATE_FORMAT(sta_degrees.TimeAvgPhoto,'%%Y%%m%%d') FROM degrees,sta_degrees" \
                        " WHERE sta_degrees.Gender='all' AND sta_degrees.NumStds>'0' AND degrees.DegCod=sta_degrees.DegCod" \
                        " ORDER BY degrees.ShortName");
         break;
     }
  }

/*****************************************************************************/
/*** Get number of students and number of students with photo in a degree ****/
/*****************************************************************************/

static void Sta_GetNumStdsInDegree (long DegCod,Usr_Gender_t Gender,int *NumStds,int *NumStdsWithPhoto)
  {
   extern const char *StringsGenderDB[Usr_NUM_GENDERS];
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get the number of students in a degree from database *****/
   sprintf (Query,"SELECT NumStds,NumStdsWithPhoto FROM sta_degrees" \
                  " WHERE DegCod='%ld' AND Gender='%s'",
            DegCod,StringsGenderDB[Gender]);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the number of students in a degree");

   if (NumRows == 0)
      *NumStds = *NumStdsWithPhoto = -1;
   else
     {
      row = mysql_fetch_row (mysql_res);
      if (sscanf (row[0],"%d",NumStds) != 1)
	 *NumStds = -1;
      if (sscanf (row[1],"%d",NumStdsWithPhoto) != 1)
	 *NumStdsWithPhoto = -1;

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
  }

/*****************************************************************************/
/*********************** Update statistics of a degree ***********************/
/*****************************************************************************/

static void Sta_UpdateDegStats (long DegCod,Usr_Gender_t Gender,unsigned NumStds,unsigned NumStdsWithPhoto,long TimeToComputeAvgPhotoInMicroseconds)
  {
   extern const char *StringsGenderDB[Usr_NUM_GENDERS];
   char Query[1024];

   sprintf (Query,"REPLACE INTO sta_degrees" \
                  " (DegCod,Gender,NumStds,NumStdsWithPhoto,TimeAvgPhoto,TimeToComputeAvgPhoto)" \
                  " VALUES('%ld','%s','%u','%u',NOW(),'%ld')",
	    DegCod,StringsGenderDB[Gender],NumStds,NumStdsWithPhoto,TimeToComputeAvgPhotoInMicroseconds);
   DB_QueryREPLACE (Query,"can not save stats of a degree");
  }

/*****************************************************************************/
/******************* Show the average photo of a degree **********************/
/*****************************************************************************/

static void Sta_ShowDegreeStat (int NumStds,int NumStdsWithPhoto)
  {
   extern const char *Txt_photos[Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<span class=\"DAT\">%d&nbsp;</span>" \
                      "<span class=\"DAT_SMALL\">(%d&nbsp;%s,&nbsp;%d%%)</span>",
            NumStds,NumStdsWithPhoto,Txt_photos[Gbl.Prefs.Language],
            (NumStds > 0) ? (int) (((NumStdsWithPhoto * 100.0) / NumStds) + 0.5) : 0);
  }

/*****************************************************************************/
/******************* Show the average photo of a degree **********************/
/*****************************************************************************/

static void Sta_ShowDegreeAvgPhotoAndStat (struct Degree *Deg,Sta_AvgPhotoSeeOrPrint_t SeeOrPrint,Usr_Gender_t Gender,int NumStds,int NumStdsWithPhoto,struct Date *DateAvgPhoto)
  {
   extern const char *StringsGenderDB[Usr_NUM_GENDERS];
   extern const char *Txt_stud[Txt_NUM_LANGUAGES];
   extern const char *Txt_GENDER_PLURAL_abc[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_photos[Txt_NUM_LANGUAGES];
   unsigned PhotoWidth,PhotoHeight;
   char AvgPhotoRelPath[PATH_MAX+1];
   char PhotoURL[PATH_MAX+1];
   char CopyOfDegShortName[Deg_MAX_LENGTH_DEGREE_SHORT_NAME+1];	// Short name of degree
   char PhotoCaption[512];

   /***** Compute photo width and height to be proportional to number of students *****/
   Sta_ComputePhotoSize (NumStds,NumStdsWithPhoto,&PhotoWidth,&PhotoHeight);

   /***** Make a copy of the degree short name *****/
   strncpy (CopyOfDegShortName,Deg->ShortName,Deg_MAX_LENGTH_DEGREE_SHORT_NAME);
   CopyOfDegShortName[Deg_MAX_LENGTH_DEGREE_SHORT_NAME] = '\0';
   Str_LimitLengthHTMLStr (CopyOfDegShortName,(SeeOrPrint == Sta_DEGREES_SEE) ? 10 : 15);

   if (SeeOrPrint == Sta_DEGREES_SEE)
      fprintf (Gbl.F.Out,"<a href=\"%s\" class=\"ORL\" target=\"_blank\"><acronym title=\"%s\">",Deg->WWW,Deg->FullName);
   fprintf (Gbl.F.Out,"<img src=\"");

   if (NumStds && NumStdsWithPhoto)
     {
      sprintf (AvgPhotoRelPath,"%s/%s/%ld_%s.jpg",Cfg_PATH_REL_PHOTOS_PUBLIC,Sta_StrAvgPhotoDirs[Gbl.Stat.DegPhotos.TypeOfAverage],Deg->DegCod,StringsGenderDB[Gender]);

      if (Fil_CheckIfPathExists (AvgPhotoRelPath))
        {
         sprintf (PhotoURL,"%s/%s/%ld_%s.jpg",Cfg_URL_PHOTOS_PUBLIC,Sta_StrAvgPhotoDirs[Gbl.Stat.DegPhotos.TypeOfAverage],Deg->DegCod,StringsGenderDB[Gender]);
         fprintf (Gbl.F.Out,"%s\" width=\"%u\" height=\"%u\"",PhotoURL,PhotoWidth,PhotoHeight);
         if (SeeOrPrint == Sta_DEGREES_SEE)
           {
            sprintf (PhotoCaption,"%s<br />%d&nbsp;%s&nbsp;(%s)<br />%d&nbsp;%s&nbsp;(%d%%)<br />%02u/%02u/%04u",
                     Deg->ShortName,
                     NumStds,Txt_stud[Gbl.Prefs.Language],Txt_GENDER_PLURAL_abc[Gender][Gbl.Prefs.Language],
                     NumStdsWithPhoto,Txt_photos[Gbl.Prefs.Language],
                     (NumStds > 0) ? (int) (((NumStdsWithPhoto * 100.0) / NumStds) + 0.5) : 0,
                     DateAvgPhoto->Day,DateAvgPhoto->Month,DateAvgPhoto->Year);
            fprintf (Gbl.F.Out," onmouseover=\"zoom(this,'%s','%s')\" onmouseout=\"noZoom(this);\"",
                     PhotoURL,PhotoCaption);
           }
        }
      else
         fprintf (Gbl.F.Out,"%s/usr_bl.jpg\" width=\"%u\" height=\"%u\"",Gbl.Prefs.IconsURL,PhotoWidth,PhotoHeight);
     }
   else
      fprintf (Gbl.F.Out,"%s/usr_bl.jpg\" width=\"%u\" height=\"%u\"",Gbl.Prefs.IconsURL,PhotoWidth,PhotoHeight);
   fprintf (Gbl.F.Out," alt=\"%s\" />",Deg->FullName);
   if (SeeOrPrint == Sta_DEGREES_PRINT)
      fprintf (Gbl.F.Out,"<span class=\"ORL\">");
   fprintf (Gbl.F.Out,"<br />%s<br />%d&nbsp;%s<br />%d&nbsp;%s<br />(%d%%)",
            CopyOfDegShortName,
            NumStds,Txt_stud[Gbl.Prefs.Language],
            NumStdsWithPhoto,Txt_photos[Gbl.Prefs.Language],
            (NumStds > 0) ? (int) (((NumStdsWithPhoto * 100.0) / NumStds) + 0.5) : 0);
   if (SeeOrPrint == Sta_DEGREES_SEE)
      fprintf (Gbl.F.Out,"</acronym></a>");
   else
      fprintf (Gbl.F.Out,"</span>");
  }

/*****************************************************************************/
/******** Compute photo width and height for class photo of degrees **********/
/*****************************************************************************/
#define MIN_WIDTH_PHOTO 9
#define MIN_HEIGHT_PHOTO 12
#define MIN_PIXELS_PHOTO (MIN_WIDTH_PHOTO * MIN_HEIGHT_PHOTO)
#define DEF_WIDTH_PHOTO 36
#define DEF_HEIGHT_PHOTO 48
#define DEF_PIXELS_PHOTO (DEF_WIDTH_PHOTO * DEF_HEIGHT_PHOTO)
#define MAX_WIDTH_PHOTO 72
#define MAX_HEIGHT_PHOTO 96
#define MAX_PIXELS_PHOTO (MAX_WIDTH_PHOTO * MAX_HEIGHT_PHOTO)

static void Sta_ComputePhotoSize (int NumStds,int NumStdsWithPhoto,unsigned *PhotoWidth,unsigned *PhotoHeight)
  {
   unsigned PhotoPixels = DEF_PIXELS_PHOTO;

   switch (Gbl.Stat.DegPhotos.HowComputePhotoSize)
     {
      case Sta_PROPORTIONAL_TO_NUMBER_OF_STUDENTS:
         if (Gbl.Stat.DegPhotos.MaxStds > 0)
            PhotoPixels = (unsigned) (((float) (MAX_PIXELS_PHOTO - MIN_PIXELS_PHOTO) / Gbl.Stat.DegPhotos.MaxStds) * NumStds + MIN_PIXELS_PHOTO + 0.5);
         break;
      case Sta_PROPORTIONAL_TO_NUMBER_OF_PHOTOS:
         if (Gbl.Stat.DegPhotos.MaxStdsWithPhoto > 0)
            PhotoPixels = (unsigned) (((float) (MAX_PIXELS_PHOTO - MIN_PIXELS_PHOTO) / Gbl.Stat.DegPhotos.MaxStdsWithPhoto) * NumStdsWithPhoto + MIN_PIXELS_PHOTO + 0.5);
         break;
      case Sta_PROPORTIONAL_TO_PERCENT:
         if (Gbl.Stat.DegPhotos.MaxPercent > 0.0)
           {
            if (NumStds)
               PhotoPixels = (unsigned) (((float) (MAX_PIXELS_PHOTO - MIN_PIXELS_PHOTO) / Gbl.Stat.DegPhotos.MaxPercent) * ((float) NumStdsWithPhoto / NumStds) + MIN_PIXELS_PHOTO + 0.5);
            else
              {
               *PhotoWidth  = MIN_WIDTH_PHOTO;
               *PhotoHeight = MIN_HEIGHT_PHOTO;
               return;
              }
           }
         break;
      case Sta_UNIQUE_SIZE:
         *PhotoWidth  = DEF_WIDTH_PHOTO;
         *PhotoHeight = DEF_HEIGHT_PHOTO;
         return;
     }
   *PhotoWidth  = (unsigned) (sqrt (((double) Usr_PHOTO_REAL_WIDTH / (double) Usr_PHOTO_REAL_HEIGHT) * (double) PhotoPixels) + 0.5);
   *PhotoHeight = (unsigned) (((double) Usr_PHOTO_REAL_HEIGHT / (double) Usr_PHOTO_REAL_WIDTH) * (double) *PhotoWidth + 0.5);
  }

/****************************************************************************/
/**************** Compute the time used to generate the page ****************/
/****************************************************************************/

void Sta_ComputeTimeToGeneratePage (void)
  {
   if (gettimeofday (&Gbl.tvPageCreated, &Gbl.tz))
      // Error in gettimeofday
      Gbl.TimeGenerationInMicroseconds = -1;
   else
     {
      if (Gbl.tvPageCreated.tv_usec < Gbl.tvStart.tv_usec)
	{
	 Gbl.tvPageCreated.tv_sec--;
	 Gbl.tvPageCreated.tv_usec += 1000000;
	}
      Gbl.TimeGenerationInMicroseconds = (Gbl.tvPageCreated.tv_sec  - Gbl.tvStart.tv_sec) * 1000000L +
                                          Gbl.tvPageCreated.tv_usec - Gbl.tvStart.tv_usec;
     }
  }

/****************************************************************************/
/****************** Compute the time used to send the page ******************/
/****************************************************************************/

void Sta_ComputeTimeToSendPage (void)
  {
   if (gettimeofday (&Gbl.tvPageSent, &Gbl.tz))
      // Error in gettimeofday
      Gbl.TimeSendInMicroseconds = -1;
   else
     {
      if (Gbl.tvPageSent.tv_usec < Gbl.tvPageCreated.tv_usec)
	{
	 Gbl.tvPageSent.tv_sec--;
	 Gbl.tvPageSent.tv_usec += 1000000;
	}
      Gbl.TimeSendInMicroseconds = (Gbl.tvPageSent.tv_sec  - Gbl.tvPageCreated.tv_sec) * 1000000L +
                                    Gbl.tvPageSent.tv_usec - Gbl.tvPageCreated.tv_usec;
     }
  }

/****************************************************************************/
/************** Write the time to generate and send the page ****************/
/****************************************************************************/

void Sta_WriteTimeToGenerateAndSendPage (void)
  {
   extern const char *Txt_PAGE1_Page_generated_in[Txt_NUM_LANGUAGES];
   extern const char *Txt_PAGE2_and_sent_in[Txt_NUM_LANGUAGES];
   char StrTimeGenerationInMicroseconds[64];
   char StrTimeSendInMicroseconds[64];

   if (Gbl.TimeGenerationInMicroseconds >= 0 &&
       Gbl.TimeSendInMicroseconds >= 0)
     {
      Sta_WriteTime (StrTimeGenerationInMicroseconds,Gbl.TimeGenerationInMicroseconds);
      Sta_WriteTime (StrTimeSendInMicroseconds,Gbl.TimeSendInMicroseconds);
      fprintf (Gbl.F.Out,"%s %s %s %s",
               Txt_PAGE1_Page_generated_in[Gbl.Prefs.Language],StrTimeGenerationInMicroseconds,
               Txt_PAGE2_and_sent_in[Gbl.Prefs.Language],StrTimeSendInMicroseconds);
     }
  }

/****************************************************************************/
/********* Write time (given in microseconds) depending on amount ***********/
/****************************************************************************/

static void Sta_WriteTime (char *Str,long TimeInMicroseconds)
  {
   if (TimeInMicroseconds < 1000L)
      sprintf (Str,"%ld &micro;s",TimeInMicroseconds);
   else if (TimeInMicroseconds < 1000000L)
      sprintf (Str,"%ld ms",TimeInMicroseconds / 1000);
   else if (TimeInMicroseconds < (60*1000000L))
      sprintf (Str,"%.1f s",(float) TimeInMicroseconds / 1E6);
   else
      sprintf (Str,"%ld min, %ld s",TimeInMicroseconds / (60*1000000L),(TimeInMicroseconds/1000000L) % 60);
  }
