// swad_usr.c: users

/*
    SWAD ("Sistema Web de Apoyo a la Docencia" in Spanish,
    "Web System for Teaching Support" in English),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.
    Copyright (C) 1999-2010 Antonio Caas-Vargas
    & Daniel J. Calandria-Hernndez,
    University of Granada (SPAIN) (acanas@ugr.es)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************************************************************/
/*********************************** Headers ********************************/
/****************************************************************************/

#include <ctype.h>		// For isalnum, isdigit, etc.
#include <limits.h>		// For maximum values
#include <stdlib.h>		// For exit, system, malloc, free, rand, etc.
#include <string.h>		// For string functions
#include <sys/wait.h>		// For the macro WEXITSTATUS
#include <unistd.h>		// For access, lstat, getpid, chdir, symlink, unlink

#include "swad_cfg.h"
#include "swad_con.h"
#include "swad_db.h"
#include "swad_gbl.h"
#include "swad_grp.h"
#include "swad_ntf.h"
#include "swad_par.h"
#include "swad_prf.h"
#include "swad_rec.h"
#include "swad_usr.h"

/****************************************************************************/
/***************************** Internal constants ***************************/
/****************************************************************************/

// Permissions to see ranges of users: 1 means permitted and 0 means forbidden
// The order is Superuser-Administrator-Teacher-Student-Invited-Unknown from msb to lsb
unsigned ListUsrsRangesAllowed[Usr_NUM_LIST_USRS_RANGES][Usr_NUM_USR_TYPES] =
  {
//	Type of users in list
//	Usr_UNKNOWN	Usr_INVITED	Usr_STUDENT	Usr_TEACHER	Usr_DEG_ADMIN	Usr_SUPERUSER
	{0x00		,0x00		,0x20		,0x3F		,0x3F		,0x00		,},	// Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL
	{0x00		,0x00		,0x00		,0x3F		,0x3F		,0x00		,},	// Usr_LIST_USRS_RANGE_ANY_DEGREE_SEARCH
	{0x00		,0x00		,0x20		,0x3F		,0x3F		,0x00		,},	// Usr_LIST_USRS_RANGE_NO_COURSE_ALL
	{0x00		,0x00		,0x30		,0x3F		,0x3F		,0x00		,},	// Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL
	{0x00		,0x00		,0x00		,0x3F		,0x3F		,0x00		,},	// Usr_LIST_USRS_RANGE_THIS_DEGREE_SEARCH
	{0x00		,0x00		,0x38		,0x3F		,0x3F		,0x00		,},	// Usr_LIST_USRS_RANGE_THIS_COURSE_ALL
	{0x00		,0x00		,0x00		,0x3F		,0x3F		,0x00		,},	// Usr_LIST_USRS_RANGE_THIS_COURSE_ONLY_ONE
  };

/* Special codes for global administrator and superuser deg_admin table */
#define Usr_SPECIAL_CODE_GLOBAL_ADMIN	-1L
#define Usr_SPECIAL_CODE_SUPERUSER	-2L

const char *StringsGenderDB[Usr_NUM_GENDERS] =
  {
   "unknown",
   "female",
   "male",
   "all",
   };
const char *StringsUsrListTypeInDB[NUM_USR_LIST_TYPES] =
  {
   "classphoto",
   "list"
  };
const char *StringsUsrListType[NUM_USR_LIST_TYPES] =
  {
   "Orla",
   "Lista"
  };

/****************************************************************************/
/****************************** Internal types ******************************/
/****************************************************************************/

/****************************************************************************/
/************** External global variables from others modules ***************/
/****************************************************************************/

extern struct Globals Gbl;
extern struct Actions Actions[Act_NUM_ACTIONS];
const char *Txt_RECORD_FIELD_VISIBILITY_RECORD[Rec_NUM_TYPES_VISIBILITY][Txt_NUM_LANGUAGES];

/****************************************************************************/
/************************* Internal global variables ************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Internal prototypes **************************/
/****************************************************************************/

static void Usr_ClearPhotoName (long UsrCod);
static void Usr_UpdatePhotoName (struct UsrData *UsrDat);

static bool Usr_CheckIfUsrSharesAnyOfMyCrs (long UsrCod);
// static bool Usr_ChangeDNIOfUsr (char *DNIOld,char *DNINew);
static bool Usr_ChkIfEncryptedUsrCodExists (const char *EncryptedUsrCod);
static bool Usr_CheckCurrentPassword (void);
static bool Usr_CheckPendingPassword (void);
static void Usr_AssignMyPendingPasswordToMyCurrentPassword (void);
static void Usr_CreateANewPassword (char PlainPassword[Usr_MAX_LENGTH_PLAIN_PASSWORD+1]);
static void Usr_SetMyPendingPassword (char PlainPassword[Usr_MAX_LENGTH_PLAIN_PASSWORD+1]);
static bool Usr_SlowCheckIfPasswordIsGood (const char *PlainPassword,const char *EncryptedPassword,long UsrCod);
static bool Usr_CheckIfPasswdIsDNIorName (const char *PlainPassword);
static unsigned Usr_GetNumOtherUsrsWhoUseThisPassword (const char *EncryptedPassword,long UsrCod);

static void Usr_UpdatePhoto1 (struct UsrData *UsrDat);
static void Usr_UpdatePhoto2 (void);

static void Usr_CreateNewEncryptedUsrCod (struct UsrData *UsrDat);

static void Usr_AllocateUsrsList (struct ListUsers *LstUsrs,unsigned NumUsrs);

static void Usr_FormToSelectUsrListType (Act_Action_t NextAction,Usr_ShowUsrsType_t ListType);

static void Usr_StartFormLinkToTchCard (const char *EncryptedUsrCod);

static void Usr_GetAndUpdateUsrListType (void);
static bool Usr_GetUsrListTypeFromForm (void);
static void Usr_GetMyUsrListTypeFromDB (void);
static void Usr_UpdateMyUsrListTypeInDB (void);

static bool Usr_GetParamColsClassPhotoFromForm (void);
static void Usr_GetMyColsClassPhotoFromDB (void);
static void Usr_UpdateMyColsClassPhotoInDB (void);

static void Usr_GetAndUpdatePrefAboutListWithPhotos (void);
static bool Usr_GetParamListWithPhotosFromForm (void);
static void Usr_GetMyPrefAboutListWithPhotosFromDB (void);
static void Usr_UpdateMyPrefAboutListWithPhotosPhotoInDB (void);

static void Usr_ShowLinkToPrintPreviewOfCards (void);
static void Usr_ParamUsrsRange (void);

static void Usr_PutAreaToEnterStudentsIDs (void);
static void Usr_PutActionsRegRemUsrsCrs (void);
static void Usr_PutAllUsrTypesRegRemUsrsCrs (void);
static void Usr_PutOneUsrTypeRegRemUsrsCrs (Usr_UsrType_t UsrType,bool Checked);

static void Usr_RegisterUsrUsingHisDNI (Usr_UsrType_t RegRemUsrType,unsigned NumGrpsInCurrentCrs,struct ListCodGrps *LstGrps,unsigned *NumUsrsRegistered);
static void Usr_MarkOfficialStdsAsRemovable (long ImpGrpCod,bool RemoveSpecifiedUsrs);
static void Usr_ShowLstGrpsToRegisterRemoveStds (void);

/****************************************************************************/
/**** Show alert about number of clicks remaining before sending my photo ***/
/****************************************************************************/

void Usr_InformAboutNumClicksBeforePhoto (void)
  {
   char BreadcrumbStr[512];

   if (Gbl.Usrs.Me.NumAccWithoutPhoto)
     {
      if (Gbl.Usrs.Me.NumAccWithoutPhoto >= Usr_MAX_CLICKS_WITHOUT_PHOTO)
         Lay_ShowAlert (WARNING,"Tiene que enviar su fotograf&iacute;a," \
                                " ya que ha agotado el n&uacute;mero de accesos" \
                                " permitido sin fotograf&iacute;a.");	// Need translation!
      else if (Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW)
        {
         Act_GetBreadcrumbStrForAction (ActReqMyPho,Gbl.Prefs.Language,true,BreadcrumbStr);
         sprintf (Gbl.Message,"S&oacute;lo podr&aacute; realizar" \
                              " %u acciones m&aacute;s" \
                              " si no env&iacute;a su fotograf&iacute;a" \
                              " (puede hacerlo en %s).",	// Need translation!
                  Usr_MAX_CLICKS_WITHOUT_PHOTO-Gbl.Usrs.Me.NumAccWithoutPhoto,
                  BreadcrumbStr);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
  }

/****************************************************************************/
/************************** Create data for a user **************************/
/****************************************************************************/

void Usr_UsrDataConstructor (struct UsrData *UsrDat)
  {
   /* Asignar memoria to the comments */
   if ((UsrDat->Std.Comments = malloc (Cte_MAX_BYTES_TEXT+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store user's data.");

   /* Inicializar a cero the data of the user */
   Usr_ResetUsrDataExceptUsrCodAndDNI (UsrDat);
  }

/****************************************************************************/
/****************** Reset user's data except UsrCod and DNI *****************/
/****************************************************************************/
// UsrCod and DNI are not changed

void Usr_ResetUsrDataExceptUsrCodAndDNI (struct UsrData *UsrDat)
  {
   UsrDat->EncryptedUsrCod[0] = '\0';

   UsrDat->Password[0] = '\0';

   UsrDat->UsrTypeInDB = Usr_UNKNOWN;
   UsrDat->Surname1[0] = '\0';
   UsrDat->Surname2[0] = '\0';
   UsrDat->FirstName[0] = '\0';
   UsrDat->FullName[0] = '\0';
   UsrDat->Gender = Usr_GENDER_UNKNOWN;
   UsrDat->E_mail[0] = '\0';
   UsrDat->WWW[0] = '\0';
   UsrDat->Skype[0] = '\0';
   UsrDat->Photo[0] = '\0';
   UsrDat->PublicPhoto = false;
   UsrDat->InsCod = -1L;

   UsrDat->Tch.CtrCod = -1L;
   UsrDat->Tch.DptCod = -1L;
   UsrDat->Tch.Office[0] = '\0';
   UsrDat->Tch.OfficePhone[0] = '\0';

   UsrDat->Std.LocalAddress[0] = '\0';
   UsrDat->Std.LocalPhone[0] = '\0';
   UsrDat->Std.FamilyAddress[0] = '\0';
   UsrDat->Std.FamilyPhone[0] = '\0';
   UsrDat->Std.OriginPlace[0] = '\0';
   UsrDat->Std.StrBirthday[0] = '\0';
   UsrDat->Std.Birthday.Day = 0;
   UsrDat->Std.Birthday.Month = 0;
   UsrDat->Std.Birthday.Year = 0;
   if (UsrDat->Std.Comments)
      UsrDat->Std.Comments[0] = '\0';

   UsrDat->NumNewMsgs = 0;
   UsrDat->Accepted = true;

   UsrDat->Prefs.Language = Txt_LANGUAGE_DEFAULT;
   UsrDat->Prefs.Theme = THEME_DEFAULT;
   UsrDat->Prefs.SideCols = 3;
   UsrDat->Prefs.SecureIcons = true;
   UsrDat->Prefs.NotifyEvents = 0;	// By default, don't notify anything
  }

/****************************************************************************/
/**************** Free memory used to store the data of a user **************/
/****************************************************************************/

void Usr_UsrDataDestructor (struct UsrData *UsrDat)
  {
   if (UsrDat->Std.Comments)
     {
      free ((void *) UsrDat->Std.Comments);
      UsrDat->Std.Comments = NULL;
     }
  }

/****************************************************************************/
/************* Get all the user's data from a given user's code *************/
/****************************************************************************/
// Input: UsrDat->UsrCod must hold user's code

void Usr_GetAllUsrDataFromUsrCod (struct UsrData *UsrDat)
  {
   Usr_GetDNIFromUsrCod (UsrDat);
   Usr_GetUsrDataFromUsrCod (UsrDat);
  }

/****************************************************************************/
/************** Get all the user's data from a given user's DNI *************/
/****************************************************************************/
// Input: UsrDat->DNI must hold user's DNI

void Usr_GetAllUsrDataFromDNI (struct UsrData *UsrDat)
  {
   Usr_GetUsrCodFromDNI (UsrDat);
   Usr_GetUsrDataFromUsrCod (UsrDat);
  }

/****************************************************************************/
/*************** Get user code from user's record in database *****************/
/****************************************************************************/

void Usr_GetDNIFromUsrCod (struct UsrData *UsrDat)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   if (UsrDat->UsrCod > 0)
     {
      /***** Get user's ID from database *****/
      sprintf (Query,"SELECT DNI FROM usr_data WHERE UsrCod='%ld'",UsrDat->UsrCod);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get user's ID");

      if (NumRows == 0)
         UsrDat->DNI[0] = '\0';
      else if (NumRows == 1)
        {
         /***** Get DNI *****/
         row = mysql_fetch_row (mysql_res);
         strncpy (UsrDat->DNI,row[0],Usr_MAX_LENGTH_USR_ID);
         UsrDat->DNI[Usr_MAX_LENGTH_USR_ID] = '\0';
        }
      else if (NumRows > 1)
         Lay_ShowErrorAndExit ("Error when getting user's ID.");

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
   else
      UsrDat->DNI[0] = '\0';
  }

/****************************************************************************/
/*************** Get user code from user's record in database *****************/
/****************************************************************************/

void Usr_GetUsrCodFromDNI (struct UsrData *UsrDat)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   if (UsrDat->DNI[0])
     {
      /***** Get user's code from database *****/
      sprintf (Query,"SELECT UsrCod FROM usr_data WHERE DNI='%s'",UsrDat->DNI);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get user's code");

      if (NumRows != 1)
         Lay_ShowErrorAndExit ("Error when getting user's code.");

      /***** Get user code *****/
      row = mysql_fetch_row (mysql_res);
      UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
   else
      UsrDat->UsrCod = -1L;
  }

/****************************************************************************/
/** Get user code from user's record in database using encrypted user code ****/
/****************************************************************************/
// Input: UsrDat->EncryptedUsrCod must hold user's encrypted code

void Usr_GetUsrCodFromEncryptedUsrCod (struct UsrData *UsrDat)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   if (UsrDat->EncryptedUsrCod[0])
     {
      /***** Get user's code from database *****/
      sprintf (Query,"SELECT UsrCod FROM usr_data WHERE EncryptedUsrCod='%s'",
               UsrDat->EncryptedUsrCod);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get user's code");

      if (NumRows != 1)
         Lay_ShowErrorAndExit ("Error when getting user's code.");

      /***** Get user code *****/
      row = mysql_fetch_row (mysql_res);
      UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
   else
      UsrDat->UsrCod = -1L;
  }

/****************************************************************************/
/******************* Get user type from unsigned string *********************/
/****************************************************************************/

Usr_UsrType_t Usr_ConvertUnsignedStrToUsrType (const char *UnsignedStr)
  {
   unsigned UnsignedNum;

   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      return (UnsignedNum >= Usr_NUM_USR_TYPES) ? Usr_UNKNOWN : (Usr_UsrType_t) UnsignedNum;
   return Usr_UNKNOWN;
  }

/****************************************************************************/
/************ Get user's data from database giving a user's code ************/
/****************************************************************************/
// UsrDat->UsrCod must contain an existing user's code

void Usr_GetUsrDataFromUsrCod (struct UsrData *UsrDat)
  {
   extern const char *Txt_StrLangId[Txt_NUM_LANGUAGES];
   extern const char *The_ThemeId[The_NUM_THEMES];
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   Usr_Gender_t Gender;
   tTheme Theme;
   Txt_Language_t Lan;
   char YN[1+1];
   char StrBirthday[4+1+2+1+2+1];
   unsigned UnsignedNum;

   /***** Get user's data from database *****/
   sprintf (Query,"SELECT EncryptedUsrCod,Password,UsrType,Surname1,Surname2,FirstName,Gender,Theme,Language,E_mail,WWW,Skype,Photo,PublicPhoto," \
                  "InsCod,DptCod,CtrCod,Office,OfficePhone," \
                  "LocalAddress,LocalPhone,FamilyAddress,FamilyPhone,OriginPlace,Birthday,Comments," \
                  "NumNewMsgs,SideCols,SecureIcons,NotifyEvents,LastTab" \
                  " FROM usr_data WHERE UsrCod='%ld'",
            UsrDat->UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get user's data");

   /***** Check number of rows in result *****/
   if (NumRows != 1)
      Lay_ShowErrorAndExit ("Error when getting user's data.");

   /***** Read the data comunes a all the users *****/
   row = mysql_fetch_row (mysql_res);

   /* Get encrypted user code */
   strncpy (UsrDat->EncryptedUsrCod,row[0],sizeof (UsrDat->EncryptedUsrCod)-1);
   UsrDat->EncryptedUsrCod[sizeof (UsrDat->EncryptedUsrCod)-1] = '\0';

   /* Get encrypted password */
   strncpy (UsrDat->Password,row[1],sizeof (UsrDat->Password)-1);
   UsrDat->Password[sizeof (UsrDat->Password)-1] = '\0';

   /* Get user type */
   UsrDat->UsrTypeInDB = Usr_ConvertUnsignedStrToUsrType (row[2]);

   /* Get name */
   strncpy (UsrDat->Surname1 ,row[3],sizeof (UsrDat->Surname1 )-1);
   strncpy (UsrDat->Surname2 ,row[4],sizeof (UsrDat->Surname2 )-1);
   strncpy (UsrDat->FirstName,row[5],sizeof (UsrDat->FirstName)-1);

   /* Get gender */
   UsrDat->Gender = Usr_GENDER_UNKNOWN;
   for (Gender=(Usr_Gender_t) 0; Gender<Usr_NUM_GENDERS; Gender++)
      if (!strcasecmp (row[6],StringsGenderDB[Gender]))
        {
	 UsrDat->Gender = Gender;
         break;
        }

   /* Get theme */
   UsrDat->Prefs.Theme = THEME_YELLOW;
   for (Theme=(tTheme) 0; Theme<The_NUM_THEMES; Theme++)
      if (!strcasecmp (row[7],The_ThemeId[Theme]))
        {
	 UsrDat->Prefs.Theme = Theme;
         break;
        }

   /* Get language */
   UsrDat->Prefs.Language = Txt_LANGUAGE_DEFAULT;
   for (Lan=(Txt_Language_t) 0; Lan<Txt_NUM_LANGUAGES; Lan++)
      if (!strcasecmp (row[8],Txt_StrLangId[Lan]))
        {
	 UsrDat->Prefs.Language = Lan;
         break;
        }
 
   /* Get rest of data */
   strncpy (UsrDat->E_mail           ,row[ 9],sizeof (UsrDat->E_mail           )-1);
   strncpy (UsrDat->WWW              ,row[10],sizeof (UsrDat->WWW              )-1);
   strncpy (UsrDat->Skype            ,row[11],sizeof (UsrDat->Skype            )-1);
   strncpy (UsrDat->Photo            ,row[12],sizeof (UsrDat->Photo            )-1);
   strncpy (YN                       ,row[13],1);
   UsrDat->PublicPhoto = (Str_ConvertToUpperLetter (YN[0]) == 'Y');
   UsrDat->InsCod = Str_ConvertStrCodToLongCod (row[14]);

   UsrDat->Tch.DptCod = Str_ConvertStrCodToLongCod (row[15]);
   UsrDat->Tch.CtrCod = Str_ConvertStrCodToLongCod (row[16]);
   strncpy (UsrDat->Tch.Office       ,row[17],sizeof (UsrDat->Tch.Office       )-1);
   strncpy (UsrDat->Tch.OfficePhone  ,row[18],sizeof (UsrDat->Tch.OfficePhone  )-1);

   strncpy (UsrDat->Std.LocalAddress ,row[19],sizeof (UsrDat->Std.LocalAddress )-1);
   strncpy (UsrDat->Std.LocalPhone   ,row[20],sizeof (UsrDat->Std.LocalPhone   )-1);
   strncpy (UsrDat->Std.FamilyAddress,row[21],sizeof (UsrDat->Std.FamilyAddress)-1);
   strncpy (UsrDat->Std.FamilyPhone  ,row[22],sizeof (UsrDat->Std.FamilyPhone  )-1);
   strncpy (UsrDat->Std.OriginPlace  ,row[23],sizeof (UsrDat->Std.OriginPlace  )-1);
   strcpy (StrBirthday,row[24] ? row[24] : "0000-00-00");
   Usr_GetStudentCommentsFromString (row[25] ? row[25] : "",UsrDat);	// Get the comments comunes a todas the courses

   /* Get number of new messages */
   if (sscanf (row[26],"%u",&UsrDat->NumNewMsgs) != 1)
      UsrDat->NumNewMsgs = 0;

   /* Get if user wants to show side columns */
   if (sscanf (row[27],"%u",&UsrDat->Prefs.SideCols) == 1)
     {
      if (UsrDat->Prefs.SideCols > 3)
         UsrDat->Prefs.SideCols = 3;	// Show both side columns
     }
   else
      UsrDat->Prefs.SideCols = 3;	// Show both side columns

   /* Get secure / unsecure icons */
   UsrDat->Prefs.SecureIcons = (Str_ConvertToUpperLetter (row[28][0]) == 'Y');

   /* Get on which events I want to be notified by e-mail */
   if (sscanf (row[29],"%u",&UsrDat->Prefs.NotifyEvents) != 1)
      UsrDat->Prefs.NotifyEvents = 0;
   if (UsrDat->Prefs.NotifyEvents >= (1 << Ntf_NUM_NOTIFY_EVENTS))	// Maximum binary value for NotifyEvents is 000...0011...11
      UsrDat->Prefs.NotifyEvents = 0;

   /* Get last tab */
   UsrDat->LastTab = TabPer;	// By default, set last tab to the personal tab
   if (sscanf (row[30],"%u",&UnsignedNum) == 1)
      if (UnsignedNum >= 1 || UnsignedNum <= Act_NUM_TABS)
         UsrDat->LastTab = (Act_Tab_t) UnsignedNum;

   UsrDat->Surname1 [sizeof (UsrDat->Surname1 )-1] = '\0';
   UsrDat->Surname2 [sizeof (UsrDat->Surname2 )-1] = '\0';
   UsrDat->FirstName[sizeof (UsrDat->FirstName)-1] = '\0';
   UsrDat->E_mail   [sizeof (UsrDat->E_mail   )-1] = '\0';
   UsrDat->WWW      [sizeof (UsrDat->WWW      )-1] = '\0';
   UsrDat->Skype    [sizeof (UsrDat->Skype    )-1] = '\0';
   UsrDat->Photo    [sizeof (UsrDat->Photo    )-1] = '\0';

   UsrDat->Tch.Office     [sizeof (UsrDat->Tch.Office     )-1] = '\0';
   UsrDat->Tch.OfficePhone[sizeof (UsrDat->Tch.OfficePhone)-1] = '\0';

   UsrDat->Std.LocalAddress [sizeof (UsrDat->Std.LocalAddress )-1] = '\0';
   UsrDat->Std.LocalPhone   [sizeof (UsrDat->Std.LocalPhone   )-1] = '\0';
   UsrDat->Std.FamilyAddress[sizeof (UsrDat->Std.FamilyAddress)-1] = '\0';
   UsrDat->Std.FamilyPhone  [sizeof (UsrDat->Std.FamilyPhone  )-1] = '\0';
   UsrDat->Std.OriginPlace  [sizeof (UsrDat->Std.OriginPlace  )-1] = '\0';
   if (sscanf (StrBirthday,"%u-%u-%u",&(UsrDat->Std.Birthday.Year),&(UsrDat->Std.Birthday.Month),&(UsrDat->Std.Birthday.Day)) != 3)
      Lay_ShowErrorAndExit ("Wrong date.");
   Dat_ConvDateToDateStr (&(UsrDat->Std.Birthday),UsrDat->Std.StrBirthday);

   Str_ConvertToTitleType (UsrDat->Surname1 );
   Str_ConvertToTitleType (UsrDat->Surname2 );
   Str_ConvertToTitleType (UsrDat->FirstName);

   /* Create full name using FirstName, Surname1 and Surname2 */
   Usr_BuildFullName (UsrDat);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/********** Build full name using FirstName, Surname1 and Surname2 **********/
/****************************************************************************/

void Usr_BuildFullName (struct UsrData *UsrDat)
  {
   strcpy (UsrDat->FullName,UsrDat->FirstName);
   if (UsrDat->Surname1[0])
     {
      strcat (UsrDat->FullName," ");
      strcat (UsrDat->FullName,UsrDat->Surname1);
     }
   if (UsrDat->Surname2[0])
     {
      strcat (UsrDat->FullName," ");
      strcat (UsrDat->FullName,UsrDat->Surname2);
     }
  }

/****************************************************************************/
/** Get the comments in the record of a student from database from a string ***/
/****************************************************************************/

void Usr_GetStudentCommentsFromString (char *Str,struct UsrData *UsrDat)
  {
   /***** Check that memory for comments is allocated *****/
   if (UsrDat->Std.Comments)
     {
      /***** Copy comments from Str to Comments *****/
      strncpy (UsrDat->Std.Comments,Str,Cte_MAX_BYTES_TEXT);
      UsrDat->Std.Comments[Cte_MAX_BYTES_TEXT] = '\0';
     }
  }

/*****************************************************************************/
/******** Restrict length of firstname and surnames and write then ***********/
/*****************************************************************************/

void Usr_RestrictLengthAndWriteName (struct UsrData *UsrDat,unsigned MaxChars)
  {
   char FirstName[Usr_MAX_BYTES_NAME+1];
   char Surnames[2*(Usr_MAX_BYTES_NAME+1)];

   /***** Restrict length of firstname and surnames *****/
   strcpy (FirstName,UsrDat->FirstName);
   strcpy (Surnames,UsrDat->Surname1);
   if (UsrDat->Surname2[0])
     {
      strcat (Surnames," ");
      strcat (Surnames,UsrDat->Surname2);
     }
   Str_LimitLengthHTMLStr (FirstName,MaxChars);
   Str_LimitLengthHTMLStr (Surnames,MaxChars);

   /***** Replace tildes, , etc. by codes, because some browsers (i.e., IE5) don't show correctly tildes with AJAX *****/
   Str_ReplaceSpecialCharByCodes (FirstName,Usr_MAX_BYTES_NAME);
   Str_ReplaceSpecialCharByCodes (Surnames,2*Usr_MAX_BYTES_NAME+1);

   /***** Write shorted firstname, then return, then shorted surnames *****/
   fprintf (Gbl.F.Out,"%s<br />%s",FirstName,Surnames);
  }

/*****************************************************************************/
/********************* Check if user's photo can be shown ********************/
/*****************************************************************************/
// Returns true if the photo can be shown and false if not.
// Public photo means two different things depending on the user's type
// Student: if his/her photo is public ==> it can be seen by other students in the same course. His/her teachers always can see it.
// teacher: if his/her photo is public ==> it can be seen by any other identified user. His/her students always can see it.

bool Usr_ShowUsrPhotoIsAllowed (struct UsrData *UsrDat,char *PhotoURL)
  {
   bool ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat->UsrCod);
   bool IAmAStudent = (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT || (Gbl.Usrs.Me.UsrTypeLog == Usr_INVITED && Gbl.Usrs.Me.UsrDat.UsrTypeInDB == Usr_STUDENT));
   bool IAmATeacher = (Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER || (Gbl.Usrs.Me.UsrTypeLog == Usr_INVITED && Gbl.Usrs.Me.UsrDat.UsrTypeInDB == Usr_TEACHER));
   bool IAmADegreeAdministrator = (Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN);
   bool HeIsStudent = (UsrDat->UsrTypeInDB == Usr_STUDENT);
   bool ICanSeePhoto = false;

   /***** Check if I can see the other's photo *****/
   if (ItsMe || Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)	// A superuser always can see any user's photo
      ICanSeePhoto = true;
   else if (IAmAStudent || IAmATeacher || IAmADegreeAdministrator)
     {
      if (IAmAStudent && HeIsStudent)
	 // A student only can see the photo of other student if both photos are public
         ICanSeePhoto = (Gbl.Usrs.Me.MyPhotoExists && Gbl.Usrs.Me.UsrDat.PublicPhoto && UsrDat->PublicPhoto);
      else
         // A user only can see the photo of other user if it's public or if both users share any course
         ICanSeePhoto = UsrDat->PublicPhoto ? true : Usr_CheckIfUsrSharesAnyOfMyCrs (UsrDat->UsrCod);
     }

   /***** Photo is shown if I can see it or it's me, and photo exists *****/
   return ICanSeePhoto ? Usr_BuildLinkToPhoto (UsrDat,PhotoURL) : false;
  }

/****************************************************************************/
/***************** Create a public link to a private photo ******************/
/****************************************************************************/
// Returns false if photo doesn't exists
// Returns true if link is created successfully

bool Usr_BuildLinkToPhoto (const struct UsrData *UsrDat,char *PhotoURL)
  {
   char PathPublRelPhoto[PATH_MAX+1];
   char PathPrivRelFotoDesdeLink[PATH_MAX+1];

   if (UsrDat->Photo[0])
     {
      /***** Make path to public photo *****/
      sprintf (PathPublRelPhoto,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PUBLIC,UsrDat->Photo);

      /***** Make path to private photo from public directory *****/
      sprintf (PathPrivRelFotoDesdeLink,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE_FROM_PHOTOS_PUBLIC,UsrDat->DNI);

      /***** Create a symbolic link to the private photo, if not exists *****/
      if (!Fil_CheckIfPathExists (PathPublRelPhoto))
         if (symlink (PathPrivRelFotoDesdeLink,PathPublRelPhoto) != 0)
            Lay_ShowErrorAndExit ("Can not create public link" \
                                 " to access to user's private photo");

      /***** Create the public URL of the photo *****/
      sprintf (PhotoURL,"%s/%s.jpg",Cfg_URL_PHOTOS_PUBLIC,UsrDat->Photo);

      return true;
     }
   return false;
  }

/****************************************************************************/
/*************** Check if photo exists and return path to it ****************/
/****************************************************************************/
// Returns false if photo doesn't exists
// Returns true if photo exists

bool Usr_CheckIfPrivPhotoExists (const struct UsrData *UsrDat,char *PathPrivRelPhoto)
  {
   /***** Make path to photo privada *****/
   sprintf (PathPrivRelPhoto,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,UsrDat->DNI);

   return Fil_CheckIfPathExists (PathPrivRelPhoto);
  }

/****************************************************************************/
/************************** Remove a user's photo ***************************/
/****************************************************************************/
// Returns true on success, false on error

bool Usr_RemovePhoto (struct UsrData *UsrDat)
  {
   char PathPrivRelFoto[PATH_MAX+1];
   char PathPublRelPhoto[PATH_MAX+1];
   unsigned NumErrors = 0;

   if (UsrDat->Photo[0])
     {
      /***** Clear photo name in database *****/
      Usr_ClearPhotoName (UsrDat->UsrCod);

      /***** Remove public link *****/
      sprintf (PathPublRelPhoto,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PUBLIC,UsrDat->Photo);
      if (Fil_CheckIfPathExists (PathPublRelPhoto))	// Public link exists
         if (unlink (PathPublRelPhoto))			// Remove public link
           { 
	    Lay_ShowAlert (ERROR,"Error removing public link to photo.");
            NumErrors++;
           }

      /***** Remove photo *****/
      sprintf (PathPrivRelFoto,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,UsrDat->DNI);
      if (Fil_CheckIfPathExists (PathPrivRelFoto))	// Photo exists
        {
         if (unlink (PathPrivRelFoto))			// Remove photo
           { 
	    Lay_ShowAlert (ERROR,"Error removing photo.");
            NumErrors++;
           }
        }

      /***** Remove original photo *****/
      sprintf (PathPrivRelFoto,"%s/%s_original.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,UsrDat->DNI);
      if (Fil_CheckIfPathExists (PathPrivRelFoto))	// Original photo exists
         if (unlink (PathPrivRelFoto))			// Remove original photo
           {
	    Lay_ShowAlert (ERROR,"Error removing the original photo.");
            NumErrors++;
           }

      /***** Clear photo name in user's data *****/
      UsrDat->Photo[0] = '\0';
     }

   return (NumErrors == 0);
  }

/****************************************************************************/
/****************** Clear photo name of an user in database *****************/
/****************************************************************************/

static void Usr_ClearPhotoName (long UsrCod)
  {
   char Query[128];

   /***** Clear photo name in user's data *****/
   sprintf (Query,"UPDATE usr_data SET Photo='' WHERE UsrCod='%ld'",UsrCod);
   DB_QueryUPDATE (Query,"can not clear the name of a user's photo");
  }

/****************************************************************************/
/***************** Update photo name of an user in database *****************/
/****************************************************************************/

static void Usr_UpdatePhotoName (struct UsrData *UsrDat)
  {
   char Query[512];
   char PathPublRelPhoto[PATH_MAX+1];

   /***** Update photo name in database *****/
   sprintf (Query,"UPDATE usr_data SET Photo='%s' WHERE UsrCod='%ld'",
            Gbl.UniqueNameEncrypted,UsrDat->UsrCod);
   DB_QueryUPDATE (Query,"can not update the name of a user's photo");

   /***** Remove the old symbolic link to photo *****/
   sprintf (PathPublRelPhoto,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PUBLIC,UsrDat->Photo);
   unlink (PathPublRelPhoto);		// Remove public link

   /***** Update photo name in user's data *****/
   strcpy (UsrDat->Photo,Gbl.UniqueNameEncrypted);
  }

/*****************************************************************************/
/****************** Write code to show the photo of a user *******************/
/*****************************************************************************/

void Usr_ShowUsrPhoto (const struct UsrData *UsrDat,const char *PhotoURL,int Width,int Height,bool Zoom)
  {
   char SpecialFullName [3*(Usr_MAX_BYTES_NAME+1)+1];
   char SpecialShortName[3*(Usr_MAX_BYTES_NAME+1)+6];
   char SpecialSurnames [2*(Usr_MAX_BYTES_NAME+1)+1];

   /* Replace tildes, , etc. in full name by codes, because some browsers (i.e., IE5) don't show correctly tildes with AJAX */
   strcpy (SpecialFullName,UsrDat->FullName);
   Str_ReplaceSpecialCharByCodes (SpecialFullName,sizeof (SpecialFullName)-1);

   if (SpecialFullName[0])
      fprintf (Gbl.F.Out,"<acronym title=\"%s\">",SpecialFullName);
   fprintf (Gbl.F.Out,"<img src=\"");
   if (PhotoURL)
      fprintf (Gbl.F.Out,"%s",PhotoURL);
   else
      fprintf (Gbl.F.Out,"%s/usr_bl.jpg",Gbl.Prefs.IconsURL);
   if      (Width ==  12 && Height ==  16)
      fprintf (Gbl.F.Out,"\" class=\"F12x16\"");
   else if (Width ==  15 && Height ==  20)
      fprintf (Gbl.F.Out,"\" class=\"F15x20\"");
   else if (Width ==  18 && Height ==  24)
      fprintf (Gbl.F.Out,"\" class=\"F18x24\" width=\"18\" height=\"24\"");	// Here width and height are written explicitely in order to allow copy-paste to Excel
   else if (Width ==  24 && Height ==  32)
      fprintf (Gbl.F.Out,"\" class=\"F24x32\"");
   else if (Width ==  36 && Height ==  48)
      fprintf (Gbl.F.Out,"\" class=\"F36x48\"");
   else if (Width ==  75 && Height == 100)
      fprintf (Gbl.F.Out,"\" class=\"F75x100\"");
   else if (Width == 120 && Height == 160)
      fprintf (Gbl.F.Out,"\" class=\"F120x160\"");
   else if (Width == 150 && Height == 200)
      fprintf (Gbl.F.Out,"\" class=\"F150x200\"");
   else
      fprintf (Gbl.F.Out,"\" width=\"%d\" height=\"%d\" align=\"middle\"",Width,Height);
   if (Zoom && PhotoURL && (Width != Usr_PHOTO_REAL_WIDTH || Height != Usr_PHOTO_REAL_HEIGHT))
     {
      strcpy (SpecialShortName,UsrDat->FirstName);
      Str_LimitLengthHTMLStr (SpecialShortName,23);
      Str_ReplaceSpecialCharByCodes (SpecialShortName,Usr_MAX_BYTES_NAME);
      SpecialSurnames[0] = '\0';
      if (UsrDat->Surname1[0])
         strcpy (SpecialSurnames,UsrDat->Surname1);
      if (UsrDat->Surname2[0])
        {
         strcat (SpecialSurnames," ");
         strcat (SpecialSurnames,UsrDat->Surname2);
        }
      Str_LimitLengthHTMLStr (SpecialSurnames,23);
      Str_ReplaceSpecialCharByCodes (SpecialSurnames,2*Usr_MAX_BYTES_NAME+1);
      strcat (SpecialShortName,"<br />");
      strcat (SpecialShortName,SpecialSurnames);
      fprintf (Gbl.F.Out," onmouseover=\"zoom(this,'%s','%s')\" onmouseout=\"noZoom(this);\"",
               PhotoURL,SpecialShortName);
     }
   fprintf (Gbl.F.Out," />");
   if (SpecialFullName[0])
      fprintf (Gbl.F.Out,"</acronym>");
  }

/****************************************************************************/
/********** Check if a user is an administrator of current degree ***********/
/****************************************************************************/

bool Usr_CheckIfUsrIsAdmOfDeg (long UsrCod,long DegCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a user is administrator of a degree from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM deg_admin" \
                  " WHERE UsrCod='%ld' AND DegCod='%ld'",
            UsrCod,DegCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user is administrator of a degree");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error while checking if a user is administrator of a degree.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/****************************************************************************/
/********** Check if a user is an administrator of current degree ***********/
/****************************************************************************/

bool Usr_CheckIfUsrIsAdmOfAllDegs (long UsrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a user is administrator of all degrees from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM deg_admin WHERE UsrCod='%ld' AND DegCod='%ld'",
            UsrCod,Usr_SPECIAL_CODE_GLOBAL_ADMIN);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user is administrator of all degrees");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a user is administrator of all degrees.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/****************************************************************************/
/********************* Check if a user is a superuser ***********************/
/****************************************************************************/

bool Usr_CheckIfUsrIsSuperuser (long UsrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a user is superuser from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM deg_admin" \
                  " WHERE UsrCod='%ld' AND DegCod='%ld'",
            UsrCod,Usr_SPECIAL_CODE_SUPERUSER);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user is superuser");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a user is superuser.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/****************************************************************************/
/*************** Check if a user belongs to any of my courses ***************/
/****************************************************************************/

static bool Usr_CheckIfUsrSharesAnyOfMyCrs (long UsrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a user shares any course with me from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_usr WHERE UsrCod='%ld'" \
                  " AND CrsCod IN (SELECT CrsCod FROM crs_usr WHERE UsrCod='%ld')",
            UsrCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user shares any course with you");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a user shares any course with you.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (bool) (NumRows != 0);
  }

/*****************************************************************************/
/** Get all my institutions (those of my courses) and store them in a list ***/
/*****************************************************************************/

void Usr_GetMyInstitutions (void)
  {
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long InsCod;

   /***** If my institutions are yet filled, there's nothing to do *****/
   if (Gbl.Usrs.Me.MyInstitutions.Filled)
      return;

   Gbl.Usrs.Me.MyInstitutions.Num = 0;

   /***** Get my institutions from database *****/
   if ((NumRows = Usr_GetInssFromUsr (Gbl.Usrs.Me.UsrDat.UsrCod,&mysql_res)) > 0) // Institutions found
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
	 /* Get next institution */
	 row = mysql_fetch_row (mysql_res);

         /* Get institution code */
	 if ((InsCod = Str_ConvertStrCodToLongCod (row[0])) > 0)
	   {
            if (Gbl.Usrs.Me.MyInstitutions.Num == MAX_INSTITUTIONS_PER_USR)
               Lay_ShowErrorAndExit ("Maximum number of institutions of a user exceeded.");

	    Gbl.Usrs.Me.MyInstitutions.InsCods[Gbl.Usrs.Me.MyInstitutions.Num] = InsCod;
            Gbl.Usrs.Me.MyInstitutions.Num++;
	   }
	}

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Set boolean that indicates that my institutions are yet filled *****/
   Gbl.Usrs.Me.MyInstitutions.Filled = true;
  }

/*****************************************************************************/
/***** Get all my degrees (those of my courses) and store them in a list *****/
/*****************************************************************************/

void Usr_GetMyDegrees (void)
  {
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long DegCod;

   /***** If my degrees are yet filled, there's nothing to do *****/
   if (Gbl.Usrs.Me.MyDegrees.Filled)
      return;

   Gbl.Usrs.Me.MyDegrees.Num = 0;

   /***** Get my degrees from database *****/
   if ((NumRows = Usr_GetDegsFromUsr (Gbl.Usrs.Me.UsrDat.UsrCod,-1L,&mysql_res)) > 0) // Degrees found
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
	 /* Get next degree */
	 row = mysql_fetch_row (mysql_res);

         /* Get degree code */
	 if ((DegCod = Str_ConvertStrCodToLongCod (row[0])) > 0)
	   {
            if (Gbl.Usrs.Me.MyDegrees.Num == Deg_MAX_DEGREES_PER_USR)
               Lay_ShowErrorAndExit ("Maximum number of degrees of a user exceeded.");

	    Gbl.Usrs.Me.MyDegrees.DegCods[Gbl.Usrs.Me.MyDegrees.Num] = DegCod;
            Gbl.Usrs.Me.MyDegrees.Num++;
	   }
	}

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Set boolean that indicates that my degrees are yet filled *****/
   Gbl.Usrs.Me.MyDegrees.Filled = true;
  }

/*****************************************************************************/
/*************** Get all my courses and store them in a list *****************/
/*****************************************************************************/

void Usr_GetMyCourses (void)
  {
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long CrsCod;

   /***** If my courses are yet filled, there's nothing to do *****/
   if (Gbl.Usrs.Me.MyCourses.Filled)
      return;

   Gbl.Usrs.Me.MyCourses.Num = 0;

   /***** Get my courses from database *****/
   if ((NumRows = Usr_GetCrssFromUsr (Gbl.Usrs.Me.UsrDat.UsrCod,-1L,&mysql_res)) > 0) // Courses found
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
	 /* Get next course */
	 row = mysql_fetch_row (mysql_res);

         /* Get course code */
	 if ((CrsCod = Str_ConvertStrCodToLongCod (row[0])) > 0)
	   {
            if (Gbl.Usrs.Me.MyCourses.Num == Deg_MAX_COURSES_PER_USR)
               Lay_ShowErrorAndExit ("Maximum number of courses of a user exceeded.");

	    Gbl.Usrs.Me.MyCourses.CrsCods[Gbl.Usrs.Me.MyCourses.Num] = CrsCod;
            Gbl.Usrs.Me.MyCourses.Num++;
	   }
	}

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Set boolean that indicates that my courses are yet filled *****/
   Gbl.Usrs.Me.MyCourses.Filled = true;
  }

/****************************************************************************/
/******************* Check if a user belongs to a degree ********************/
/****************************************************************************/

bool Usr_CheckIfUsrBelongsToDeg (long UsrCod,long DegCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get is a user belongs to a degree from database *****/
   sprintf (Query,"SELECT COUNT(DISTINCT courses.DegCod) FROM crs_usr,courses" \
                  " WHERE crs_usr.UsrCod='%ld' AND courses.DegCod='%ld' AND crs_usr.CrsCod=courses.CrsCod",
            UsrCod,DegCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user belongs to a degree");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a user belongs to a degree.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows > 0);
  }

/****************************************************************************/
/******************** Check if a user belongs to a course *******************/
/****************************************************************************/

bool Usr_CheckIfUsrBelongsToCrs (long UsrCod,long CrsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a user belongs to a course from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
            CrsCod,UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user belongs to a course");

   /***** Get number of rows *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a user belongs to a course.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/******************** Check if I belong to an institution *******************/
/****************************************************************************/
// The list of my institutions must be filled before calling this function

bool Usr_CheckIfIBelongToIns (long InsCod)
  {
   unsigned NumMyIns;

   /***** Fill the list with the institutions I belong to *****/ 
   Usr_GetMyInstitutions ();

   /***** Check if the institution passed as parameter is any of my institutions *****/
   for (NumMyIns=0; NumMyIns<Gbl.Usrs.Me.MyInstitutions.Num; NumMyIns++)
      if (Gbl.Usrs.Me.MyInstitutions.InsCods[NumMyIns] == InsCod)
	 return true;
   return false;
  }

/****************************************************************************/
/*********************** Check if I belong to a degree **********************/
/****************************************************************************/
// The list of my degrees must be filled before calling this function

bool Usr_CheckIfIBelongToDeg (long DegCod)
  {
   unsigned NumMyDeg;

   /***** Fill the list with the degrees I belong to *****/ 
   Usr_GetMyDegrees ();

   /***** Check if the degree passed as parameter is any of my degrees *****/
   for (NumMyDeg=0; NumMyDeg<Gbl.Usrs.Me.MyDegrees.Num; NumMyDeg++)
      if (Gbl.Usrs.Me.MyDegrees.DegCods[NumMyDeg] == DegCod)
	 return true;
   return false;
  }

/****************************************************************************/
/*********************** Check if I belong to a course **********************/
/****************************************************************************/
// The list of my courses must be filled before calling this function

bool Usr_CheckIfIBelongToCrs (long CrsCod)
  {
   unsigned NumMyCrs;

   /***** Fill the list with the courses I belong to *****/ 
   Usr_GetMyCourses ();

   /***** Check if the course passed as parameter is any of my courses *****/
   for (NumMyCrs=0; NumMyCrs<Gbl.Usrs.Me.MyCourses.Num; NumMyCrs++)
      if (Gbl.Usrs.Me.MyCourses.CrsCods[NumMyCrs] == CrsCod)
	 return true;
   return false;
  }

/*****************************************************************************/
/************** Get the institutions of a user from database *****************/
/*****************************************************************************/
// Returns the number of rows of the result

unsigned long Usr_GetInssFromUsr (long UsrCod,MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get the institutions a user belongs to from database *****/
   sprintf (Query,"SELECT DISTINCT(institutions.InsCod) FROM crs_usr,courses,degrees,institutions" \
                  " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod=courses.CrsCod AND courses.DegCod=degrees.DegCod AND degrees.InsCod=institutions.InsCod" \
                  " ORDER BY institutions.ShortName",
            UsrCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get the institutions a user belongs to");
  }

/*****************************************************************************/
/***************** Get the degrees of a user from database *******************/
/*****************************************************************************/
// Returns the number of rows of the result

unsigned long Usr_GetDegsFromUsr (long UsrCod,long InsCod,MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get degress a user belongs to from database *****/
   if (InsCod > 0)
      sprintf (Query,"SELECT DISTINCT(degrees.DegCod) FROM crs_usr,courses,degrees" \
                     " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod=courses.CrsCod AND courses.DegCod=degrees.DegCod AND degrees.InsCod='%ld'" \
                     " ORDER BY degrees.ShortName",
               UsrCod,InsCod);
   else
      sprintf (Query,"SELECT DISTINCT(degrees.DegCod) FROM crs_usr,courses,degrees" \
                     " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod=courses.CrsCod AND courses.DegCod=degrees.DegCod" \
                     " ORDER BY degrees.ShortName",
               UsrCod);
   return DB_QuerySELECT (Query,mysql_res,"can not check the degrees a user belongs to");
  }

/*****************************************************************************/
/************** Get all the courses of a user from database ******************/
/*****************************************************************************/
// Returns the number of rows of the result

unsigned long Usr_GetCrssFromUsr (long UsrCod,long DegCod,MYSQL_RES **mysql_res)
  {
   char Query[512];

   /***** Get courses a user belongs to from database *****/
   if (DegCod > 0)
      sprintf (Query,"SELECT crs_usr.CrsCod FROM crs_usr,courses" \
                     " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod=courses.CrsCod AND courses.DegCod='%ld'" \
                     " ORDER BY courses.ShortName",
               UsrCod,DegCod);
   else
      sprintf (Query,"SELECT crs_usr.CrsCod FROM crs_usr,courses" \
                     " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod=courses.CrsCod" \
                     " ORDER BY courses.ShortName",
               UsrCod);
   return DB_QuerySELECT (Query,mysql_res,"can not get the courses a user belongs to");
  }

/****************************************************************************/
/*********************** Write form to remove old users *********************/
/****************************************************************************/

void Usr_AskRemoveOldUsrs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_USR_TYPES_PLURAL_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_Eliminate[Txt_NUM_LANGUAGES];
   Usr_UsrType_t UsrType;
   unsigned i;

   /***** Form to request number of months without access *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Gbl.Usrs.MonthsWithoutAccess = Usr_DEF_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS;
   Act_FormStart (ActRemOldUsr);
   fprintf (Gbl.F.Out,"<span class=\"%s\">Eliminar los </span><select name=\"UsrType\">",	// Need translation!
            The_ClassFormul[Gbl.Prefs.Theme]);
   for (UsrType=Usr_STUDENT; UsrType<=Usr_TEACHER; UsrType++)
     {
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) UsrType);
      if (UsrType == Usr_STUDENT)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%s</option>",Txt_USR_TYPES_PLURAL_abc[UsrType][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language]);
     }   
   fprintf (Gbl.F.Out,"</select><span class=\"%s\"> que no est&eacute;n dados de alta en ninguna asignatura<br />" \
                      " y que lleven m&aacute;s de </span><select name=\"Months\">",	// Need translation!
            The_ClassFormul[Gbl.Prefs.Theme]);
   for (i=Usr_MIN_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS; i<=Usr_MAX_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS; i++)
     {
      fprintf (Gbl.F.Out,"<option");
      if (i == Gbl.Usrs.MonthsWithoutAccess)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%u</option>",i);
     }
   fprintf (Gbl.F.Out,"</select><span class=\"%s\"> meses sin acceder a %s.</span>",
            The_ClassFormul[Gbl.Prefs.Theme],Cfg_PLATFORM_SHORT_NAME);

   /***** Send button*****/
   Lay_PutSendButton (Txt_Eliminate[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form></div>\n");
  }

/****************************************************************************/
/***************************** Remove old users *****************************/
/****************************************************************************/

void Usr_RemoveOldUsrs (void)
  {
   extern const char *Txt_USR_TYPES_PLURAL_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   char UnsignedStr[10+1];
   unsigned long SecondsWithoutAccess;
   Usr_UsrType_t UsrType;
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   unsigned long NumUsrsRemoved = 0UL;
   struct UsrData UsrDat;

   /***** Get parameter with the type of users to remove *****/
   Par_GetParam (Par_PARAM_SINGLE,"UsrType",UnsignedStr,10);
   UsrType = Usr_ConvertUnsignedStrToUsrType (UnsignedStr);
   if (UsrType != Usr_STUDENT &&
       UsrType != Usr_TEACHER)
      Lay_ShowErrorAndExit ("Wrong users' type.");

   /***** Get parameter with number of months without access *****/
   Par_GetParam (Par_PARAM_SINGLE,"Months",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.Usrs.MonthsWithoutAccess) != 1)
      Lay_ShowErrorAndExit ("Number of months without accesses is missing.");
   if (Gbl.Usrs.MonthsWithoutAccess < Usr_MIN_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS ||
       Gbl.Usrs.MonthsWithoutAccess > Usr_MAX_MONTHS_WITHOUT_ACCESS_TO_REMOVE_OLD_USRS)
      Lay_ShowErrorAndExit ("Wrong number of months without accesses.");
   SecondsWithoutAccess = (unsigned long) Gbl.Usrs.MonthsWithoutAccess * Dat_SECONDS_IN_ONE_MONTH;

   /***** Get old users from database *****/
   sprintf (Query,"SELECT UsrCod FROM usr_data WHERE" \
                  " UsrType='%u' AND" \
                  " UNIX_TIMESTAMP(LastTime) < UNIX_TIMESTAMP()-%lu" \
                  " AND UsrCod NOT IN (SELECT DISTINCT UsrCod FROM crs_usr)",
            (unsigned) UsrType,SecondsWithoutAccess);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get old users");

   sprintf (Gbl.Message,"Eliminando %lu %s que no estaban dados de alta en ninguna asignatura" \
                        " y que llevaban m&aacute;s de %u meses sin acceder a %s.",	// Need translation!
            NumRows,Txt_USR_TYPES_PLURAL_abc[UsrType][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language],
            Gbl.Usrs.MonthsWithoutAccess,Cfg_PLATFORM_SHORT_NAME);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Initialize structure with user's data *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** Remove users *****/
   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      row = mysql_fetch_row (mysql_res);
      UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);
      if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat))	// If user's data exist...
        {
         Usr_DeleteUsrFromPlatform (&UsrDat,Cte_QUIET);
         NumUsrsRemoved++;
        }
     }

   /***** Free memory used for user's data *****/
   Usr_UsrDataDestructor (&UsrDat);
	
   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Write end message *****/
   sprintf (Gbl.Message,"Eliminados %lu %s.",	// Need translation!
            NumUsrsRemoved,Txt_USR_TYPES_PLURAL_abc[UsrType][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language]);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/**************** Check if a user exists with a given DNI *******************/
/****************************************************************************/
/*
static bool Usr_ChangeDNIOfUsr (char *DNIOld,char *DNINew)
  {
   struct UsrData UsrDatOld;
   bool Result = false;
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   long CrsCod;
   char PathRelPhotoOld[PATH_MAX+1];
   char PathRelPhotoNew[PATH_MAX+1];
   char PathRelUsrOld[PATH_MAX+1];
   char PathRelUsrNew[PATH_MAX+1];
   char PathRelPrivWorksOld[PATH_MAX+1];
   char PathRelPrivWorksNew[PATH_MAX+1];

   if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (DNIOld))
     {
      strncpy (UsrDatOld.DNI,DNIOld,Usr_MAX_LENGTH_USR_ID);
      UsrDatOld.DNI[Usr_MAX_LENGTH_USR_ID] = '\0';

      // ***** Initialize structure with user's data *****
      Usr_UsrDataConstructor (&UsrDatOld);

      if (Usr_ChkDNIAndGetAllUsrDataFromDNI (&UsrDatOld))	// If user's with source DNI exists...
        {
         if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (DNINew))
           {
            if (!Usr_ChkIfDNIExists (DNINew))	// If user's with source DNI exists...
              {
               // ***** Change DNI in user's data table *****
               fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s -&gt; %s</td><tr>\n",
                        DNIOld,DNINew);

               sprintf (Query,"UPDATE usr_data SET DNI='%s' WHERE UsrCod='%ld' AND DNI='%s'",
                        DNINew,UsrDatOld.UsrCod,DNIOld);
               DB_QueryUPDATE (Query,"can not update user's ID");

               // ***** Change photo *****

// !!!!! Remove old public symbolic link to private photo !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

               sprintf (PathRelPhotoOld,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,DNIOld);
               if (Fil_CheckIfPathExists (PathRelPhotoOld))
                 {
                  sprintf (PathRelPhotoNew,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,DNINew);
                  fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s -&gt; %s</td><tr>\n",
                           PathRelPhotoOld,PathRelPhotoNew);
                  if (Fil_RenameFileOrDir (PathRelPhotoOld,PathRelPhotoNew))
                     fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">El archivo <strong>%s</strong> se ha renombrado como <strong>%s</strong></td><tr>\n",
                              PathRelPhotoOld,PathRelPhotoNew);
                  else
                     fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s</td><tr>\n",Gbl.Message);
                 }
               else
                  fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s no existe</td><tr>\n",
                           PathRelPhotoOld);

               sprintf (PathRelPhotoOld,"%s/%s_original.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,DNIOld);
               if (Fil_CheckIfPathExists (PathRelPhotoOld))
                 {
                  sprintf (PathRelPhotoNew,"%s/%s_original.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,DNINew);
                  fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s -&gt; %s</td><tr>\n",
                           PathRelPhotoOld,PathRelPhotoNew);

                  // * Rename the file *
                  if (Fil_RenameFileOrDir (PathRelPhotoOld,PathRelPhotoNew))
                     fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">El archivo <strong>%s</strong> se ha renombrado como <strong>%s</strong></td><tr>\n",
                              PathRelPhotoOld,PathRelPhotoNew);
                  else
                     fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s</td><tr>\n",Gbl.Message);
                 }
               else
                  fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s no existe</td><tr>\n",
                           PathRelPhotoOld);

               // ***** Change briefcase folder name *****
               sprintf (PathRelUsrOld,"%s/%c/%s",Cfg_PATH_REL_USR_PRIVATE,DNIOld[strlen (DNIOld)-1],DNIOld);
               if (Fil_CheckIfPathExists (PathRelUsrOld))
                 {
                  sprintf (PathRelUsrNew,"%s/%c/%s",Cfg_PATH_REL_USR_PRIVATE,DNINew[strlen (DNINew)-1],DNINew);
                  fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s -&gt; %s</td><tr>\n",
                           PathRelUsrOld,PathRelUsrNew);

                  // * Rename the directory *
                  if (Fil_RenameFileOrDir (PathRelUsrOld,PathRelUsrNew))
                     fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">La carpeta <strong>%s</strong> se ha renombrado como <strong>%s</strong></td><tr>\n",
                              PathRelUsrOld,PathRelUsrNew);
                  else
                     fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s</td><tr>\n",Gbl.Message);
                 }
               else
                  fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s no existe</td><tr>\n",
                           PathRelUsrOld);

               // ***** Get user's courses from database *****
               if ((NumRows = Usr_GetCrssFromUsr (UsrDatOld.UsrCod,-1L,&mysql_res)) > 0) // Courses found
                  for (NumRow=0; NumRow<NumRows; NumRow++)
	            {
	             // * Get next course *
	             row = mysql_fetch_row (mysql_res);

                     // * Get course code *
	             if ((CrsCod = Str_ConvertStrCodToLongCod (row[0])) > 0)
	               {
                        fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">Asignatura: %ld</td><tr>\n",
                                 CrsCod);

                        // * Move user's works folder inside this course, if exists *
                        sprintf (PathRelPrivWorksOld,"%s/%ld/usr/%s",Cfg_PATH_REL_CRS_PRIVATE,CrsCod,DNIOld);
                        if (Fil_CheckIfPathExists (PathRelPrivWorksOld))
                          {
                           sprintf (PathRelPrivWorksNew,"%s/%ld/usr/%s",Cfg_PATH_REL_CRS_PRIVATE,CrsCod,DNINew);
                           fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s -&gt; %s</td><tr>\n",
                                    PathRelPrivWorksOld,PathRelPrivWorksNew);

                           // * Rename the directory *
                           if (Fil_RenameFileOrDir (PathRelPrivWorksOld,PathRelPrivWorksNew))
                              fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">La carpeta <strong>%s</strong> se ha renombrado como <strong>%s</strong></td><tr>\n",
                                       PathRelPrivWorksOld,PathRelPrivWorksNew);
                           else
                              fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s</td><tr>\n",Gbl.Message);
                          }
                        else
                           fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"5\">%s no existe</td><tr>\n",
                                    PathRelPrivWorksOld);
	               }
	            }

               // ***** Free structure that stores the query result *****
               DB_FreeMySQLResult (&mysql_res);

               Result = true;
              }
           }
        }

      // ***** Free memory used for user's data *****
      Usr_UsrDataDestructor (&UsrDatOld);
     }

   return Result;
  }
*/
/****************************************************************************/
/**************** Check if a user exists with a given DNI *******************/
/****************************************************************************/

static bool Usr_ChkIfEncryptedUsrCodExists (const char *EncryptedUsrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if an encrypted user's code already existed in database *****/
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE EncryptedUsrCod='%s'",
            EncryptedUsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if an encrypted user's code already existed");

   /***** Get number of rows (0/1) with this DNI *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if an encrypted user's code already existed.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/****** Request acceptation / refusion of register in current course ********/
/****************************************************************************/

void Usr_ReqAcceptRegisterInCrs (void)
  {
   extern const char *Txt_Your_registration_in_course_X_has_been_requested[Txt_NUM_LANGUAGES];
   extern const char *Txt_Register_me_IN_A_COURSE[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_me[Txt_NUM_LANGUAGES];

   sprintf (Gbl.Message,Txt_Your_registration_in_course_X_has_been_requested[Gbl.Prefs.Language],
            Gbl.CurrentCrs.Crs.FullName);
   Lay_ShowAlert (WARNING,Gbl.Message);

   fprintf (Gbl.F.Out,"<div align=\"center\">");

   /***** Send button to accept register in the current course *****/
   Act_FormStart (ActAccRegCrs);
   Lay_PutSendButton (Txt_Register_me_IN_A_COURSE[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");

   /***** Send button to refuse register in the current course *****/
   Act_FormStart (ActRefRegCrs);
   Lay_PutSendButton (Txt_Remove_me[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form></div>\n");
  }

/****************************************************************************/
/************* Accept my registration in the current course *****************/
/****************************************************************************/

void Usr_AcceptRegisterMeInCrs (void)
  {
   char Query[512];

   /***** Confirm my registration *****/
   sprintf (Query,"UPDATE crs_usr SET Accepted='Y' WHERE CrsCod='%ld' AND UsrCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not confirm your registration");

   /***** Confirmation message *****/
   sprintf (Gbl.Message,"Usted ha confirmado su alta en la asignatura <strong>%s</strong>",	// Need translation!
            Gbl.CurrentCrs.Crs.FullName);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/*********************** Write form for user log in *************************/
/****************************************************************************/

void Usr_WriteFormLogin (void)
  {
   extern const char *The_ClassUsr[The_NUM_THEMES];
   extern const char *Txt_ID_or_nick[Txt_NUM_LANGUAGES];
   extern const char *Txt_Password[Txt_NUM_LANGUAGES];
   extern const char *Txt_Log_in[Txt_NUM_LANGUAGES];
   extern const char *The_ClassHead[The_NUM_THEMES];
   extern const char *Txt_I_forgot_my_password[Txt_NUM_LANGUAGES];

   /***** DNI, password and submit button *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"center\" valign=\"middle\">");
   Act_FormStart (ActAutUsrInt);
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"left\" valign=\"middle\" class=\"%s\">" \
                      "%s:" \
                      "</td>",
            The_ClassUsr[Gbl.Prefs.Theme],Txt_ID_or_nick[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\">" \
                      "<input type=\"text\" name=\"UsrId\" size=\"8\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td>",
            Usr_MAX_LENGTH_USR_ID,Gbl.Usrs.Me.UsrIdLogin);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" class=\"%s\">" \
                      "&nbsp;%s:" \
                      "</td>",
            The_ClassUsr[Gbl.Prefs.Theme],Txt_Password[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\">" \
                      "<input type=\"password\" name=\"UsrPwd\" size=\"8\" maxlength=\"%u\" />" \
                      "</td>",
            Usr_MAX_LENGTH_PLAIN_PASSWORD);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" class=\"%s\">" \
                      "&nbsp;<input type=\"submit\" value=\"%s\" />" \
                      "</td>",
            The_ClassUsr[Gbl.Prefs.Theme],Txt_Log_in[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</tr></table></form></td>" \
                      "</tr></table>");

   /***** Link user for sending a new password *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"center\" class=\"%s\">",
            The_ClassHead[Gbl.Prefs.Theme]);
   Act_FormStart (ActFrmSndNewPwd);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"UsrId\" value=\"%s\" />",Gbl.Usrs.Me.UsrIdLogin);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_I_forgot_my_password[Gbl.Prefs.Language],The_ClassHead[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"%s</acronym></a></form></td></tr></table>",
            Txt_I_forgot_my_password[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/****************** Write type and name of logged user **********************/
/****************************************************************************/

void Usr_WriteLoggedUsr (void)
  {
   extern const char *The_ClassUsr[The_NUM_THEMES];
   extern const char *The_ClassHead[The_NUM_THEMES];
   extern const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_Log_out[Txt_NUM_LANGUAGES];
   Usr_UsrType_t UsrType;
   unsigned NumAvailableUsrTypes = 0;
   bool IHaveWebPage = Str_URLLooksValid (Gbl.Usrs.Me.UsrDat.WWW);
   bool ShowPhoto;
   char PhotoURL[PATH_MAX+1];

   /***** Count user types availables *****/
   for (UsrType=Usr_INVITED; UsrType<Usr_NUM_USR_TYPES; UsrType++)
      if (Gbl.Usrs.Me.AvailableUsrTypes & (1 << UsrType))
         NumAvailableUsrTypes++;

   /***** User's type *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"right\" class=\"%s\">",
            The_ClassUsr[Gbl.Prefs.Theme]);
   if (NumAvailableUsrTypes == 1)
      fprintf (Gbl.F.Out,"%s:",Txt_USR_TYPES_SINGULAR_Abc[Gbl.Usrs.Me.UsrTypeLog][Gbl.Usrs.Me.UsrDat.Gender][Gbl.Prefs.Language]);
   else
     {
      Act_FormStart (ActChgUsrTypLog);
      fprintf (Gbl.F.Out,"<select name=\"UsrTyp\" onchange=\"javascript:document.forms[%d].submit();\">",Gbl.NumForm);
      for (UsrType=Usr_INVITED; UsrType<Usr_NUM_USR_TYPES; UsrType++)
         if (Gbl.Usrs.Me.AvailableUsrTypes & (1 << UsrType))
           {
            fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) UsrType);
            if (UsrType == Gbl.Usrs.Me.UsrTypeLog)
               fprintf (Gbl.F.Out," selected=\"selected\"");
            fprintf (Gbl.F.Out,">%s</option>",
                     Txt_USR_TYPES_SINGULAR_Abc[UsrType][Gbl.Usrs.Me.UsrDat.Gender][Gbl.Prefs.Language]);
           }
      fprintf (Gbl.F.Out,"</select></form>");
     }
   fprintf (Gbl.F.Out,"</td>");

   /***** Show photo *****/
   fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" width=\"30\">");
   if (IHaveWebPage)
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\">",Gbl.Usrs.Me.UsrDat.WWW);
   ShowPhoto = Usr_ShowUsrPhotoIsAllowed (&Gbl.Usrs.Me.UsrDat,PhotoURL);
   Usr_ShowUsrPhoto (&Gbl.Usrs.Me.UsrDat,ShowPhoto ? PhotoURL : NULL,15,20,true);
   if (IHaveWebPage)
      fprintf (Gbl.F.Out,"</a>");
   fprintf (Gbl.F.Out,"</td>");

   /***** User's name *****/
   /* Link to my web page, if exists */
   fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",
            The_ClassUsr[Gbl.Prefs.Theme]);
   if (IHaveWebPage)
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"%s\">",
               Gbl.Usrs.Me.UsrDat.WWW,The_ClassUsr[Gbl.Prefs.Theme]);

   /* Name */
   fprintf (Gbl.F.Out,"%s",Gbl.Usrs.Me.UsrDat.FullName);

   /* End link to my web page, if exists */
   if (IHaveWebPage)
      fprintf (Gbl.F.Out,"</a>");
   fprintf (Gbl.F.Out,"</td></tr></table>");

   /***** Number of new messages (not read) *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"right\" class=\"%s\">",
            The_ClassHead[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<div id=\"msg\">");		// Used for AJAX based refresh
   Msg_WriteNumberOfNewMsgs ();
   fprintf (Gbl.F.Out,"</div>");			// Used for AJAX based refresh
   fprintf (Gbl.F.Out,"</td>");

   /***** Column used as separation *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">&nbsp;&nbsp;&nbsp;&nbsp;</td>",
            The_ClassHead[Gbl.Prefs.Theme]);

   /***** Link to close current session *****/
   fprintf (Gbl.F.Out,"<td align=\"left\" class=\"%s\">",The_ClassHead[Gbl.Prefs.Theme]);
   Act_FormStart (ActLogOut);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,Txt_Log_out[Gbl.Prefs.Language],The_ClassHead[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/close12x12.gif\" width=\"12\" height=\"12\" align=\"bottom\" alt=\"%s\" />" \
                      "&nbsp;%s</acronym></a></form></td></tr></table>",
            Gbl.Prefs.IconsURL,Txt_Log_out[Gbl.Prefs.Language],Txt_Log_out[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/* Get parameter with my identification (plain DNI or nickname) from a form **/
/*****************************************************************************/

void Usr_GetParamUsrIdLogin (void)
  {
   Par_GetParam (Par_PARAM_SINGLE,"UsrId",Gbl.Usrs.Me.UsrIdLogin,Usr_MAX_LENGTH_USR_ID);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.Me.UsrIdLogin,Usr_MAX_LENGTH_USR_ID,true);
  }

/*****************************************************************************/
/************* Get parameter with my plain password from a form **************/
/*****************************************************************************/

void Usr_GetParamUsrPwdLogin (void)
  {
   /***** Get plain password from form *****/
   Par_GetParam (Par_PARAM_SINGLE,"UsrPwd",Gbl.Usrs.Me.LoginPlainPassword,Usr_MAX_LENGTH_PLAIN_PASSWORD);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.Me.LoginPlainPassword,Usr_MAX_LENGTH_PLAIN_PASSWORD,true);

   /***** Encrypt password *****/
   Cry_EncryptSHA512Base64 (Gbl.Usrs.Me.LoginPlainPassword,Gbl.Usrs.Me.LoginEncryptedPassword);
  }

/*****************************************************************************/
/*** Get parameter plain DNI of other user from a form and get user's data ***/
/*****************************************************************************/
// Return true if DNI is valid and user exists

bool Usr_GetParamOtherDNIPlainAndGetUserData (void)
  {
   /***** Get parameter with the plain DNI of another user *****/
   Usr_GetParamOtherUsrDNIPlain ();

   /***** Check if DNI is valid *****/
   if (!Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Other.UsrDat.DNI))	// DNI not valid
     {
      sprintf (Gbl.Message,"El DNI <strong>%s</strong> no es v&aacute;lido.",	// Need translation!
               Gbl.Usrs.Other.UsrDat.DNI);
      Lay_ShowAlert (WARNING,Gbl.Message);
      return false;
     }

   /***** Check if user exists and get his data *****/
   if (!Usr_ChkDNIAndGetAllUsrDataFromDNI (&Gbl.Usrs.Other.UsrDat))	// Existing user
     {
      sprintf (Gbl.Message,"No existe ning&uacute;n usuario con DNI <strong>%s</strong>.",	// Need translation!
               Gbl.Usrs.Other.UsrDat.DNI);
      Lay_ShowAlert (WARNING,Gbl.Message);
      return false;
     }
 
   return true;
  }

/*****************************************************************************/
/********** Put hidden parameter with the plain DNI of other user ************/
/*****************************************************************************/

void Usr_PutParamOtherUsrDNIPlain (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OtherUsrDNI\" value=\"%s\" />",
            Gbl.Usrs.Other.UsrDat.DNI);
  }

/*****************************************************************************/
/************* Get parameter plain DNI of other user from a form *************/
/*****************************************************************************/

void Usr_GetParamOtherUsrDNIPlain (void)
  {
   Par_GetParam (Par_PARAM_SINGLE,"OtherUsrDNI",Gbl.Usrs.Other.UsrDat.DNI,Usr_MAX_LENGTH_USR_ID);
            
   Str_ConvertToUpperText (Gbl.Usrs.Other.UsrDat.DNI);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.Other.UsrDat.DNI,Usr_MAX_LENGTH_USR_ID,true);
  }

/*****************************************************************************/
/********* Put hidden parameter encrypted user's code of other user **********/
/*****************************************************************************/

void Usr_PutParamOtherUsrCodEncrypted (const char EncryptedUsrCod[Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1])
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"OtherUsrCod\" value=\"%s\" />",
            EncryptedUsrCod);
  }

/*****************************************************************************/
/********* Get hidden parameter encrypted user's code of other user **********/
/*****************************************************************************/

void Usr_GetParamOtherUsrCodEncrypted (void)
  {
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"OtherUsrCod",Gbl.Usrs.Other.UsrDat.EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64),Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64,true);
   if (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod[0])	// If parameter exists...
     {
      Usr_GetUsrCodFromEncryptedUsrCod (&Gbl.Usrs.Other.UsrDat);
      if (Gbl.Usrs.Other.UsrDat.UsrCod < 0)	// Check is user's code is valid
         Lay_ShowErrorAndExit ("Wrong user's code.");
      Usr_GetDNIFromUsrCod (&Gbl.Usrs.Other.UsrDat);
     }
   else	// Parameter doesn't exists
     {
      Gbl.Usrs.Other.UsrDat.UsrCod = -1L;
      Gbl.Usrs.Other.UsrDat.DNI[0] = '\0';	// Return empty plain DNI
     }
  }

/****************************************************************************/
/**** Check if users exists, if his password is correct, get his data... ****/
/****************************************************************************/
// Returns true if I have been successfully autentified

bool Usr_CheckUsrAndGetUsrData (void)
  {
   extern const char *The_ThemeId[The_NUM_THEMES];
   extern const char *Txt_USR_TYPES_SINGULAR_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   static const char *MsgBadLogin = "El DNI o apodo introducido no existe o bien la contrase&ntilde;a es incorrecta.";	// Need translation!
   char Command[2048];
   int ReturnCode;
   bool ICanBeAdmin = false;
   bool PasswordIsCorrect;

   /***** Check user's identification *****/
   switch (Gbl.CurrentAct)
     {
      case ActAutUsrExt:
         /* Copy user's identifier received from external site */
         strncpy (Gbl.Usrs.Me.UsrIdLogin,Gbl.Imported.UsrId,Usr_MAX_LENGTH_USR_ID);
         Gbl.Usrs.Me.UsrIdLogin[Usr_MAX_LENGTH_USR_ID] = '\0';
	 // No break!
      case ActAutUsrInt:
         /* Check if user has typed his DNI or his nickname */
         if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Me.UsrIdLogin))
           {
            // User is trying to log using his/her DNI
	    strcpy (Gbl.Usrs.Me.UsrDat.DNI,Gbl.Usrs.Me.UsrIdLogin);

            /* Check if user's DNI exists in database, and get user's data */
            if (!Usr_ChkDNIAndGetAllUsrDataFromDNI (&Gbl.Usrs.Me.UsrDat))
              {
	       Lay_ShowAlert (WARNING,MsgBadLogin);
               return false;
              }
           }
         else if (Usr_CheckIfNickIsValid (Gbl.Usrs.Me.UsrIdLogin))
	   {
            // Useris trying to log using his/her nickname
	    if (!Usr_GetUsrCodFromNickname (Gbl.Usrs.Me.UsrIdLogin,&Gbl.Usrs.Me.UsrDat))
              {
	       Lay_ShowAlert (WARNING,MsgBadLogin);
               return false;
              }

            /* Get user's data */
            Usr_GetAllUsrDataFromUsrCod (&Gbl.Usrs.Me.UsrDat);
   	   }
         else
           {
	    Lay_ShowAlert (WARNING,MsgBadLogin);
            return false;
           }
         break;
      default:	// Session is open and user's code is get from session
         Gbl.Usrs.Me.UsrDat.UsrCod = Gbl.Session.UsrCod;

         /* Check if user exists in database, and get his/her data */
         if (!Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Me.UsrDat))
           {
            sprintf (Gbl.Message,"No existe ning&uacute;n usuario con el c&oacute;digo <strong>%ld</strong>.",	// Need translation!
                     Gbl.Usrs.Me.UsrDat.UsrCod);
            Lay_ShowAlert (WARNING,Gbl.Message);
            return false;
           }
         break;
     }

   switch (Gbl.CurrentAct)
     {
      case ActAutUsrExt:
         /* Validate session:
            the call to SWAD is really coming from external site? */
         if (Gbl.Imported.UsrId[0] &&
             Gbl.Imported.SessionId[0])
           {
            sprintf (Command,"%s 1 0 %s %s",
                     Cfg_EXTERNAL_LOGIN_CLIENT_COMMAND,Gbl.Imported.UsrId,Gbl.Imported.SessionId);
            ReturnCode = system (Command);
            if (ReturnCode == -1)
               Lay_ShowErrorAndExit ("Error when running command to validate imported session.");
            ReturnCode = WEXITSTATUS(ReturnCode);
            switch (ReturnCode)
              {
               case 2:	// Student
               case 3:  // Teacher
                  Gbl.Imported.UsrType = (Usr_UsrType_t) ReturnCode;
                  break;
               default:
                  sprintf (Gbl.Message,"Error %d while validating session.",ReturnCode);
                  Lay_ShowAlert (ERROR,Gbl.Message);
                  return false;
              }
           }
         else
           {
            Lay_ShowAlert (WARNING,MsgBadLogin);
            return false;
           }

         if (Gbl.Imported.UsrType != Gbl.Usrs.Me.UsrDat.UsrTypeInDB)
           {
            sprintf (Gbl.Message,"El tipo de usuario externo de %s (%s) no coincide con el interno de %s (%s).",	// Need translation!
                     Cfg_EXTERNAL_LOGIN_SERVICE_SHORT_NAME,
                     Txt_USR_TYPES_SINGULAR_abc[Gbl.Imported.UsrType          ][Gbl.Usrs.Me.UsrDat.Gender][Gbl.Prefs.Language],
                     Cfg_PLATFORM_SHORT_NAME,
                     Txt_USR_TYPES_SINGULAR_abc[Gbl.Usrs.Me.UsrDat.UsrTypeInDB][Gbl.Usrs.Me.UsrDat.Gender][Gbl.Prefs.Language]);
            Lay_ShowAlert (WARNING,Gbl.Message);
            return false;
           }
         break;
      default:
         /* Check user's password:
            the encrypted password typed by user or coming from the session
            is the same as the stored in database? */
         PasswordIsCorrect = Usr_CheckCurrentPassword ();
         if (!PasswordIsCorrect && Gbl.CurrentAct == ActAutUsrInt)	// Login password is not the same as current password
            if ((PasswordIsCorrect = Usr_CheckPendingPassword ()))
               Usr_AssignMyPendingPasswordToMyCurrentPassword ();

         if (!PasswordIsCorrect)	// If my password is not correct...
           {
            if (Gbl.CurrentAct == ActAutUsrInt)
               Lay_ShowAlert (WARNING,MsgBadLogin);
            return false;
           }
     }

   // Password is correct
   Gbl.Usrs.Me.Logged = true;

   /***** Set user's preferences *****/
   Gbl.Prefs.SecureIcons = Gbl.Usrs.Me.UsrDat.Prefs.SecureIcons;
   Prf_SetIconsURL ();
   Gbl.Prefs.Theme = Gbl.Usrs.Me.UsrDat.Prefs.Theme;
   sprintf (Gbl.Prefs.PathTheme,"%s/%s/%s",Gbl.Prefs.IconsURL,Cfg_ICON_FOLDER_THEMES,The_ThemeId[Gbl.Prefs.Theme]);
   Gbl.Prefs.Language = Gbl.Usrs.Me.UsrDat.Prefs.Language;
   Gbl.Prefs.SideCols = Gbl.Usrs.Me.UsrDat.Prefs.SideCols;

   /***** Set the user's type I am logged *****/
   Gbl.Usrs.Me.UsrTypeLog = Gbl.Usrs.Me.UsrTypeFromSession;	// Try to get logged user type from session
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_UNKNOWN)			// If no logged user type retrieved from session...
      Gbl.Usrs.Me.UsrTypeLog = Gbl.Usrs.Me.UsrDat.UsrTypeInDB;	// ...set logged user type to user type in database

   /***** Construct the path to my directory *****/
   Str_ConvertToUpperText (Gbl.Usrs.Me.UsrDat.DNI);
   Usr_ConstructPathUsr (Gbl.Usrs.Me.UsrDat.DNI,Gbl.Usrs.Me.PathRelDir,Gbl.Usrs.Me.PathAbsDir);

   /***** Check if my photo exists and create a link to it ****/
   Gbl.Usrs.Me.MyPhotoExists = Usr_BuildLinkToPhoto (&Gbl.Usrs.Me.UsrDat,Gbl.Usrs.Me.PhotoURL);

   /***** Check if I belong to current degree and if I am administrator of current degree *****/
   if (Gbl.CurrentDeg.Deg.DegCod > 0)
      /* Check if I am and administrator of current degree */
      ICanBeAdmin = Usr_CheckIfUsrIsAdmOfDeg (Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.CurrentDeg.Deg.DegCod);
   if (!ICanBeAdmin)
      ICanBeAdmin = Usr_CheckIfUsrIsAdmOfAllDegs (Gbl.Usrs.Me.UsrDat.UsrCod);

   /***** Check if I belong to current course *****/
   if (Gbl.CurrentCrs.Crs.CrsCod > 0)
      Gbl.Usrs.Me.IBelongToThisCrs = Usr_CheckIfIBelongToCrs (Gbl.CurrentCrs.Crs.CrsCod);

   /***** Build my list of available types of user for current *****/
   if (Gbl.CurrentCrs.Crs.CrsCod > 0 && !Gbl.Usrs.Me.IBelongToThisCrs)
      Gbl.Usrs.Me.AvailableUsrTypes = 1 << Usr_INVITED;
   else
      Gbl.Usrs.Me.AvailableUsrTypes = 1 << Gbl.Usrs.Me.UsrDat.UsrTypeInDB;
   if (ICanBeAdmin)
      Gbl.Usrs.Me.AvailableUsrTypes |= 1 << Usr_DEG_ADMIN;
   if (Usr_CheckIfUsrIsSuperuser (Gbl.Usrs.Me.UsrDat.UsrCod))
      Gbl.Usrs.Me.AvailableUsrTypes |= 1 << Usr_SUPERUSER;

   /***** Check if the user type I am logged is now available for me *****/
   if (!(Gbl.Usrs.Me.AvailableUsrTypes & (1 << Gbl.Usrs.Me.UsrTypeLog)))	// Current type I am logged is not available for me
      /* Set the least prioritary type available for me */
      for (Gbl.Usrs.Me.UsrTypeLog=Usr_UNKNOWN; Gbl.Usrs.Me.UsrTypeLog<Usr_NUM_USR_TYPES; Gbl.Usrs.Me.UsrTypeLog++)
         if (Gbl.Usrs.Me.AvailableUsrTypes & (1 << Gbl.Usrs.Me.UsrTypeLog))
            break;

   /***** Set other variables related with my user type *****/
   Gbl.Usrs.Me.UsrTypeForumsChat = (Gbl.Usrs.Me.UsrTypeLog == Usr_INVITED) ? Gbl.Usrs.Me.UsrDat.UsrTypeInDB :
                                                                             Gbl.Usrs.Me.UsrTypeLog;
   if (Gbl.CurrentDeg.Deg.DegCod > 0)
     {
      if (Gbl.CurrentCrs.Crs.CrsCod > 0)
        {
         Gbl.Usrs.Me.IHaveAccessToCurrentCrs = ((Gbl.Usrs.Me.IBelongToThisCrs && (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT ||
                                                                                  Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER)) ||
                                                Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER);
         if (Gbl.Usrs.Me.IBelongToThisCrs)
            Gbl.Usrs.Me.UsrDat.Accepted = Usr_GetIfUserHasAccepted (Gbl.Usrs.Me.UsrDat.UsrCod);
        }
     }

   return true;
  }

/****************************************************************************/
/************************ Change my logged user type ************************/
/****************************************************************************/

void Usr_ChgUsrTypeLog1 (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   /***** Get parameter with the new logged user type ******/
   Par_GetParam (Par_PARAM_SINGLE,"UsrTyp",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
     {
      /* Check if new user type is a correct type *****/
      if (UnsignedNum >= Usr_NUM_USR_TYPES)
         return;

      /* Check if new user type is allowed for me */
      if (!(Gbl.Usrs.Me.AvailableUsrTypes & (1 << UnsignedNum)))
         return;

      /* New user type is correct and is allowed for me, so change my logged user type */
      Gbl.Usrs.Me.UsrTypeLog = (Usr_UsrType_t) UnsignedNum;

      /* Update logged user type in session */
      Ses_UpdateSessionDataInDB ();
     }
  }

void Usr_ChgUsrTypeLog2 (void)
  {
   extern const char *Txt_You_are_now_LOGGED_as_X[Txt_NUM_LANGUAGES];
   extern const char *Txt_logged[Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_SINGULAR_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];

   /***** Write message with my new logged user type *****/
   sprintf (Gbl.Message,Txt_You_are_now_LOGGED_as_X[Gbl.Prefs.Language],
            Txt_logged[Gbl.Usrs.Me.UsrDat.Gender][Gbl.Prefs.Language],
            Txt_USR_TYPES_SINGULAR_abc[Gbl.Usrs.Me.UsrTypeLog][Gbl.Usrs.Me.UsrDat.Gender][Gbl.Prefs.Language]);
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/****************** Update number of accesses without photo *****************/
/****************************************************************************/

unsigned Usr_UpdateMyClicksWithoutPhoto (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   unsigned NumClicks;

   /***** Get number of clicks without photo from database *****/
   sprintf (Query,"SELECT NumClicks FROM clicks_without_photo" \
                  " WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get number of clicks without photo");

   /***** Update the list of clicks without photo *****/
   if (NumRows)	// The user exists ==> update number of clicks without photo
     {
      /* Get current number of accesses */
      row = mysql_fetch_row (mysql_res);
      sscanf (row[0],"%u",&NumClicks);

      /* Update number of accesses */
      if (NumClicks <= Usr_MAX_CLICKS_WITHOUT_PHOTO)
        {
         sprintf (Query,"UPDATE clicks_without_photo SET NumClicks=NumClicks+1 WHERE UsrCod='%ld'",
                  Gbl.Usrs.Me.UsrDat.UsrCod);
         DB_QueryUPDATE (Query,"can not update number of clicks without photo");
         NumClicks++;
        }
     }
   else					// The user doesn't exist ==> add him/her
     {
      /* Add the user, with one access */
      sprintf (Query,"INSERT INTO clicks_without_photo (UsrCod,NumClicks) VALUES ('%ld',1)",
               Gbl.Usrs.Me.UsrDat.UsrCod);
      DB_QueryINSERT (Query,"can not create number of clicks without photo");
      NumClicks = 1;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   /***** Return the number of rows of the result *****/
   return NumClicks;
  }

/****************************************************************************/
/******* Remove user from table with number of accesses without photo *******/
/****************************************************************************/

void Usr_RemoveUsrFromTableClicksWithoutPhoto (long UsrCod)
  {
   char Query[512];

   sprintf (Query,"DELETE FROM clicks_without_photo WHERE UsrCod='%ld'",UsrCod);
   DB_QueryDELETE (Query,"can not remove a user from the list of users without photo");
  }

/****************************************************************************/
/*********** Asign users listing type depending on current action ***********/
/****************************************************************************/

void Usr_AsignUsrListType (void)
  {
   switch (Gbl.CurrentAct)
     {
      case ActSeeStd:
      case ActLstStdAll:
         Gbl.Usrs.Listing.Type = Usr_USER_LIST_DATA_COURSE_STUDENTS;
         break;
      case ActSeeTch:
      case ActLstTchAll:
         Gbl.Usrs.Listing.Type = Usr_USER_LIST_DATA_TEACHERS;
         break;
      case ActLstAdm:
         Gbl.Usrs.Listing.Type = Usr_USER_LIST_DATA_ADMINISTRATORS;
         break;
      case ActReqCrdCrsStd:
      case ActSeeCrdCrsStd:
      case ActPrnCrdCrsStd:
         Gbl.Usrs.Listing.Type = Usr_USER_LIST_RECORDS_COURSE_STUDENTS;
         break;
      case ActSeeCrdCrsTch:
      case ActPrnCrdCrsTch:
         Gbl.Usrs.Listing.Type = Usr_USER_LIST_RECORDS_COURSE_TEACHERS;
         break;
      default:
         Gbl.Usrs.Listing.Type = Usr_USER_LIST_UNKNOWN;
         break;
     }
  }

/****************************************************************************/
/**************** Write a form to request another user's DNI ****************/
/****************************************************************************/

void Usr_WriteFormToReqAnotherUserDNI (Act_Action_t NextAction)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_User_DNI[Txt_NUM_LANGUAGES];
   extern const char *Txt_Continue[Txt_NUM_LANGUAGES];

   /***** Form to request DNI *****/
   Act_FormStart (NextAction);
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr>" \
                      "<td align=\"right\" class=\"%s\">%s:</td>" \
                      "<td align=\"left\">" \
                      "<input type=\"text\" name=\"OtherUsrDNI\" size=\"%u\" maxlength=\"%u\" />" \
                      "</td>" \
                      "</tr>" \
                      "</table></div>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_User_DNI[Gbl.Prefs.Language],
            Usr_MAX_LENGTH_USR_ID,Usr_MAX_LENGTH_USR_ID);

   /***** Send button*****/
   Lay_PutSendButton (Txt_Continue[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/****************************************************************************/
/******************* Show connected users and right menu ********************/
/****************************************************************************/

void Usr_ShowConnectedUsrsAndRightMenu (void)
  {
   extern const char *The_ClassConnected[The_NUM_THEMES];
   extern const char *Txt_Connected_PLURAL[Txt_NUM_LANGUAGES];

   /***** Connected users *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" valign=\"top\" width=\"128\"" \
                      " style=\"background-image: url('%s/head_base_background_1x20.gif'); background-repeat: repeat-x;\">" \
                      "<table width=\"100%%\" cellspacing=\"4\" cellpadding=\"0\">" \
                      "<tr><td align=\"center\" width=\"120\" class=\"%s\"><br />",
            Gbl.Prefs.PathTheme,The_ClassConnected[Gbl.Prefs.Theme]);
   if (Gbl.CurrentCrs.Crs.CrsCod > 0)
      fprintf (Gbl.F.Out,"%s<br />",Txt_Connected_PLURAL[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"<div id=\"courseconnected\">");	// Used for AJAX based refresh
   if (Gbl.CurrentCrs.Crs.CrsCod > 0)
     {
      Con_ShowConnectedUsrsBelongingToCurrentCrs (Usr_TEACHER);
      fprintf (Gbl.F.Out,"<img src=\"%s/tr1x4.gif\" width=\"1\" height=\"4\" alt=\"\" />",Gbl.Prefs.IconsURL);	// Image used as separation
      Con_ShowConnectedUsrsBelongingToCurrentCrs (Usr_STUDENT);
     }
   fprintf (Gbl.F.Out,"</div>");			// Used for AJAX based refresh
   fprintf (Gbl.F.Out,"</td></tr>");

   /***** Right menu: institutional links *****/
   Lnk_WriteMenuWithInstitutionalLinks ();
     
   fprintf (Gbl.F.Out,"</table></td>\n");
  }

/****************************************************************************/
/*********************** Check whether a DNI is valid ***********************/
/****************************************************************************/

bool Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (char *DNI)
  {
   unsigned Length = strlen (DNI);
   char DNIWithoutLeadingZeros[Usr_MAX_LENGTH_USR_ID+1];

   /***** Must have Usr_MIN_LENGTH_USR_ID <= characters <= Usr_MAX_LENGTH_USR_ID *****/
   if (Length < Usr_MIN_LENGTH_USR_ID ||
       Length > Usr_MAX_LENGTH_USR_ID)
      return false;

   /***** If DNI starts by 0 ******/
   if (DNI[0] == '0')	// If DNI starts by '0'
     {
      strncpy (DNIWithoutLeadingZeros,DNI,Length);
      DNIWithoutLeadingZeros[Length] = '\0';
      Str_RemoveLeadingZeros (DNIWithoutLeadingZeros);	// DNIs are always stored internally without leading zeros
      if (Usr_CheckIfDNIWithoutLeadingZerosIsValid (DNIWithoutLeadingZeros))
        {
         strcpy (DNI,DNIWithoutLeadingZeros);
         return true;
        }
      return false;
     }

   /***** If DNI doesn't starts by '0' ******/
   return Usr_CheckIfDNIWithoutLeadingZerosIsValid (DNI);
  }

/****************************************************************************/
/*********************** Check whether a DNI is valid ***********************/
/****************************************************************************/

bool Usr_CheckIfDNIWithoutLeadingZerosIsValid (char *DNI)
  {
   unsigned Length = strlen (DNI);
   char DNIWithoutEndAlpha[Usr_MAX_LENGTH_USR_ID+1];

   /***** Must have Usr_MIN_LENGTH_USR_ID <= characters <= Usr_MAX_LENGTH_USR_ID *****/
   if (Length < Usr_MIN_LENGTH_USR_ID ||
       Length > Usr_MAX_LENGTH_USR_ID)
      return false;

   /***** If DNI ends in alpha letter ******/
   if ((DNI[Length-1]>='A' && DNI[Length-1]<='Z') || (DNI[Length-1]>='a' && DNI[Length-1]<='z'))	// If DNI ends in alpha letter
     {
      strncpy (DNIWithoutEndAlpha,DNI,Length-1);
      DNIWithoutEndAlpha[Length-1] = '\0';
      if (Usr_CheckIfDNIWithoutEndAlphaIsValid (DNIWithoutEndAlpha,Length-1))
        {
         DNI[Length-1] = '\0';		// If it's a valid DNI without the end alpha letter ==> remove the end alpha letter
         return true;
        }
      if ((DNI[Length-2]>='A' && DNI[Length-2]<='Z') || (DNI[Length-2]>='a' && DNI[Length-2]<='z'))	// If DNI ends in two alpha letters
        {
         DNIWithoutEndAlpha[Length-2] = '\0';
         if (Usr_CheckIfDNIWithoutEndAlphaIsValid (DNIWithoutEndAlpha,Length-2))
           {
            DNI[Length-2] = '\0';	// If it's a valid DNI without both end alpha letters ==> remove both end alpha letters
            return true;
           }
        }
      return false;
     }

   /***** If DNI doesn't end in alpha letter ******/
   return Usr_CheckIfDNIWithoutEndAlphaIsValid (DNI,Length);
  }

/****************************************************************************/
/********* Check whether a DNI without the ending letter is valid ***********/
/****************************************************************************/
// Returns true if the DNI string is valid, or false if not
// A valid DNI must...:
// 1. All characters must be digits or letters
// 2. Must be Usr_MIN_LENGTH_USR_ID <= characters <= Usr_MAX_LENGTH_USR_ID.
// 3. Must end in digit.
// 4. Can start by digit or by letter.
// 5. Can have groups of 1...Usr_MAX_CONSECUTIVE_LETTERS_USR_ID consecutive letters, at start or in the middle, but not at the end

bool Usr_CheckIfDNIWithoutEndAlphaIsValid (const char *DNI,unsigned Length)
  {
   const char *Ptr;
   unsigned NumDigits = 0;
   unsigned NumConsecutiveLetters = 0;

   /***** 2. Must be Usr_MIN_LENGTH_USR_ID <= characters <= Usr_MAX_LENGTH_USR_ID *****/
   if (Length < Usr_MIN_LENGTH_USR_ID ||
       Length > Usr_MAX_LENGTH_USR_ID)
      return false;

   /***** 3. Must end in digit ******/
   if (!isdigit ((int) DNI[Length-1]))
      return false;

   /***** 4. Can start by digit or by letter *****/
   Ptr = DNI;
   while (*Ptr)		// Loop while digit, until end or first letter found
      if (isdigit ((int) *Ptr))	// If character is digit again
        {
         NumDigits++;
         Ptr++;
        }
      else if ((*Ptr>='A' && *Ptr<='Z') || (*Ptr>='a' && *Ptr<='z'))	// If character is letter
        {
         // 5. Can have groups of 1...Usr_MAX_CONSECUTIVE_LETTERS_USR_ID consecutive letters, at start or in the middle, but not at the end
         NumConsecutiveLetters = 1;
         Ptr++;
         while ((*Ptr>='A' && *Ptr<='Z') || (*Ptr>='a' && *Ptr<='z'))	// Loop while letter, until first digit found
           {
            NumConsecutiveLetters++;
            if (NumConsecutiveLetters > Usr_MAX_CONSECUTIVE_LETTERS_USR_ID)
               return false;
            Ptr++;
           }
        }
      else		// 1. All characters must be digits or letters
         return false;

   return (NumDigits >= Usr_MIN_DIGITS_USR_ID);	// 2. Must have Usr_MIN_DIGITS_USR_ID digits at least
  }

/****************************************************************************/
/********************* Check whether a nickname if valid ********************/
/****************************************************************************/

bool Usr_CheckIfNickIsValid (const char *Nickname)
  {
   unsigned Length = strlen (Nickname);
   unsigned i;
   unsigned NumDigits = 0;
   unsigned NumConsecutiveSpaces = 0;
   const char *Ptr;
   char Ch;

   /***** A nick must have Usr_MIN_LENGTH_NICKNAME characters or more *****/
   if (Length < Usr_MIN_LENGTH_NICKNAME)
      return false;

   /***** A nick can not have more than Usr_MIN_DIGITS_USR_ID consecutive digits */
   for (i=0, Ptr=Nickname; i<Length; i++, Ptr++)
     {
      Ch = *Ptr;
      if (!(isalnum (Ch) ||
	    Ch == ' ' || Ch == '-' ||
	    Ch == '' || Ch == '' || Ch == '' || Ch == '' || Ch == '' || Ch == '' || Ch == '' ||
	    Ch == '' || Ch == '' || Ch == '' || Ch == '' || Ch == '' || Ch == '' || Ch == ''))
	 return false;
      else if (isdigit ((int) Ch))
	{
	 if (++NumDigits == Usr_MIN_DIGITS_USR_ID)	// More than Usr_MIN_DIGITS_USR_ID-1 digits are not allowed
	    return 0;
	 NumConsecutiveSpaces = 0;
	}
      else if (Ch == ' ')
	{
	 if (++NumConsecutiveSpaces == 2) // 2 or more consecutive spaces are not allowed
	    return 0;
	}
      else
	 NumConsecutiveSpaces = 0;
     }

   return true;
  }

/****************************************************************************/
/*** Check if login password is the same as current password in database ****/
/****************************************************************************/
// Returns true if there's no current password in database, or if login password is the same
// Returns false if there's a current password in database and is not the same as the login password

static bool Usr_CheckCurrentPassword (void)
  {
   return (Gbl.Usrs.Me.UsrDat.Password[0] ?
           !strcmp (Gbl.Usrs.Me.LoginEncryptedPassword,Gbl.Usrs.Me.UsrDat.Password) : true);
  }

/****************************************************************************/
/*** Check if login password is the same as current password in database ****/
/****************************************************************************/
// Returns true if there's pending password in database and login password is the same
// Returns false if there's no a pending password in database, or if pending password is not the same as the login password

static bool Usr_CheckPendingPassword (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** Get pending password from database *****/
   sprintf (Query,"SELECT PendingPassword FROM pending_passwd" \
                  " WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get pending password");

   if (NumRows == 0)
      Gbl.Usrs.Me.PendingPassword[0] = '\0';
   else
     {
      /* Get encrypted pending password */
      row = mysql_fetch_row (mysql_res);
      strcpy (Gbl.Usrs.Me.PendingPassword,row[0]);
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (Gbl.Usrs.Me.PendingPassword[0] ?
           !strcmp (Gbl.Usrs.Me.LoginEncryptedPassword,Gbl.Usrs.Me.PendingPassword) : false);
  }

/****************************************************************************/
/************ Assign my pending password to my current password *************/
/****************************************************************************/

static void Usr_AssignMyPendingPasswordToMyCurrentPassword (void)
  {
   char Query[512];

   /***** Update my current password in database *****/
   sprintf (Query,"UPDATE usr_data SET Password='%s' WHERE UsrCod='%ld'",
            Gbl.Usrs.Me.PendingPassword,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update your password");

   /***** Update my current password *****/
   strcpy (Gbl.Usrs.Me.UsrDat.Password,Gbl.Usrs.Me.PendingPassword);
  }

/****************************************************************************/
/*********************** Change my password in database *********************/
/****************************************************************************/

void Usr_ActChgMyPwd1 (void)
  {
   char PlainPassword[Usr_MAX_LENGTH_PLAIN_PASSWORD+1];
   char NewPlainPassword[2][Usr_MAX_LENGTH_PLAIN_PASSWORD+1];
   char NewEncryptedPassword[Cry_LENGTH_ENCRYPTED_STR_SHA512_BASE64+1];

   /***** Get plain password from form *****/
   Par_GetParam (Par_PARAM_SINGLE,"UsrPwd",PlainPassword,Usr_MAX_LENGTH_PLAIN_PASSWORD);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,PlainPassword,Usr_MAX_LENGTH_PLAIN_PASSWORD,true);

   /***** Encrypt password *****/
   Cry_EncryptSHA512Base64 (PlainPassword,Gbl.Usrs.Me.LoginEncryptedPassword);

   Gbl.Usrs.Error = true;
   if (Usr_CheckCurrentPassword ())
     {
      Par_GetParam (Par_PARAM_SINGLE,"Paswd1",NewPlainPassword[0],Usr_MAX_LENGTH_PLAIN_PASSWORD);
      Par_GetParam (Par_PARAM_SINGLE,"Paswd2",NewPlainPassword[1],Usr_MAX_LENGTH_PLAIN_PASSWORD);

      /***** Check if I have written twice the same password *****/
      if (strcmp (NewPlainPassword[0],NewPlainPassword[1]))
         // Passwords don't match
	 sprintf (Gbl.Message,"No ha escrito dos veces la misma contrase&ntilde;a nueva.");	// Need translation!
      else
        {
	 Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewPlainPassword[0],Usr_MAX_LENGTH_PLAIN_PASSWORD,true);
         Cry_EncryptSHA512Base64 (NewPlainPassword[0],NewEncryptedPassword);
         if (Usr_SlowCheckIfPasswordIsGood (NewPlainPassword[0],NewEncryptedPassword,Gbl.Usrs.Me.UsrDat.UsrCod))	// New password is good?
	   {
            strcpy (Gbl.Usrs.Me.UsrDat.Password,NewEncryptedPassword);
	    Ses_UpdateSessionDataInDB ();
            Usr_UpdateCommonCard (&Gbl.Usrs.Me.UsrDat,Usr_REPLACE_DATA);
	    sprintf (Gbl.Message,"Su contrase&ntilde;a se ha modificado con &eacute;xito.");	// Need translation!
	    Gbl.Usrs.Error = false;
	   }
        }
     }
   else
      sprintf (Gbl.Message,"Su contrase&ntilde;a actual no se ha introducido correctamente.");	// Need translation!
  }

void Usr_ActChgMyPwd2 (void)
  {
   /***** Write error message when updating password *****/
   Lay_ShowAlert (Gbl.Usrs.Error ? WARNING : INFO,Gbl.Message);

   /***** Retry? *****/
   if (Gbl.Usrs.Error)
      Usr_ShowFormChgPwd ();
  }

/****************************************************************************/
/*************** Show form to send a new password by e-mail *****************/
/****************************************************************************/

void Usr_ShowFormSendNewPwd (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_If_you_have_forgotten_your_password_[Txt_NUM_LANGUAGES];
   extern const char *Txt_ID_or_nick[Txt_NUM_LANGUAGES];
   extern const char *Txt_E_mail_new_password[Txt_NUM_LANGUAGES];

   /***** Help text *****/
   Lay_ShowAlert (INFO,Txt_If_you_have_forgotten_your_password_[Gbl.Prefs.Language]);

   /***** Start of form *****/
   Act_FormStart (ActSndNewPwd);

   /***** DNI/nickname *****/
   fprintf (Gbl.F.Out,"<div align=\"center\" class=\"%s\">%s:&nbsp;<input type=\"text\" name=\"UsrId\" size=\"8\" maxlength=\"%u\" value=\"%s\" /></div>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_ID_or_nick[Gbl.Prefs.Language],Usr_MAX_LENGTH_USR_ID,Gbl.Usrs.Me.UsrIdLogin);

   /***** End form *****/
   Lay_PutSendButton (Txt_E_mail_new_password[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/****************************************************************************/
/*********************** Send a new password by e-mail **********************/
/****************************************************************************/

void Usr_SendNewPwdByEMail (void)
  {
   extern const char *Txt_You_must_enter_your_ID_or_your_nickname[Txt_NUM_LANGUAGES];
   extern const char *Txt_If_you_have_written_your_ID_or_nickname_correctly_[Txt_NUM_LANGUAGES];
   extern const char *Txt_New_password_NO_HTML[Txt_NUM_LANGUAGES];
   bool UsrExists = false;
   char NewRandomPlainPassword[Usr_MAX_LENGTH_PLAIN_PASSWORD+1];
   char BreadcrumbStr[512];
   char Command[256]; // Command to execute for sending an e-mail
   int ReturnCode;

   /***** Check if DNI or nickname is not empty *****/
   if (!Gbl.Usrs.Me.UsrIdLogin[0])
     {
      Lay_ShowAlert (WARNING,Txt_You_must_enter_your_ID_or_your_nickname[Gbl.Prefs.Language]);
      Usr_ShowFormSendNewPwd ();
      return;
     }

   /***** Check if users exists *****/
   /* Check if user has typed his DNI or his nickname */
   if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Me.UsrIdLogin))
     {
      // User has typed a DNI
      strcpy (Gbl.Usrs.Me.UsrDat.DNI,Gbl.Usrs.Me.UsrIdLogin);

      /* Check if user's DNI exists in database, and get user's data */
      UsrExists = Usr_ChkDNIAndGetAllUsrDataFromDNI (&Gbl.Usrs.Me.UsrDat);
     }
   else if (Usr_CheckIfNickIsValid (Gbl.Usrs.Me.UsrIdLogin))
     {
      // User has typed a nickname
      if ((UsrExists = Usr_GetUsrCodFromNickname (Gbl.Usrs.Me.UsrIdLogin,&Gbl.Usrs.Me.UsrDat)))
         /* Get user's data */
         Usr_GetAllUsrDataFromUsrCod (&Gbl.Usrs.Me.UsrDat);
     }

   /***** Send a new password via e-mail when user exists *****/
   if (!UsrExists || !Gbl.Usrs.Me.UsrDat.E_mail[0])	// If user don't exists or he/she have no mail address
     {
      /***** Help message *****/
      Lay_ShowAlert (INFO,Txt_If_you_have_written_your_ID_or_nickname_correctly_[Gbl.Prefs.Language]);
      return;
     }

   /***** Create temporary file for mail content *****/
   Msg_CreateFileNameMail ();
      
   /***** Create a new random password *****/
   Usr_CreateANewPassword (NewRandomPlainPassword);
      
   /***** Write mail content into file and close file *****/
   /* Welcome note */
   Msg_WriteWelcomeNoteEMail (&Gbl.Usrs.Me.UsrDat);

   /* Message body */
   Act_GetBreadcrumbStrForAction (ActFrmChgPwd,Gbl.Prefs.Language,false,BreadcrumbStr);
   fprintf (Gbl.Msg.FileMail,"Se le ha asignado la siguiente contrasea para entrar en %s: %s\n" \
                             "Si desea activar esta nueva contrasea, debe entrar en https://%s/" \
                             " con su DNI/apodo y esta nueva contrasea antes de %u das." \
                             " Una vez que haya entrado, puede cambiar la contrasea por otra en %s.\n" \
                             "Si no accede con esta nueva contrasea, seguir teniendo la contrasea que tuviera hasta ahora.\n" \
                             "Si ha recibido este mensaje sin solicitarlo," \
                             " se debe a que alguien que conoce su DNI o su apodo" \
                             " ha solicitado el envo de una nueva contrasea para usted.\n" \
                             "Esta nueva contrasea se ha enviado slo a la direccin de correo %s, que es la que aparece en su ficha.\n\n",	// Need translation!
            Cfg_PLATFORM_SHORT_NAME,NewRandomPlainPassword,
            Cfg_PLATFORM_URL,(unsigned) (Cfg_TIME_TO_DELETE_OLD_PENDING_PASSWORDS/(24L*60L*60L)),
            BreadcrumbStr,
            Gbl.Usrs.Me.UsrDat.E_mail);

   /* Footer note */
   Msg_WriteFootNoteEMail (Gbl.Prefs.Language);

   fclose (Gbl.Msg.FileMail);

   /***** Call the script programmed by Antonio F. Daz Garca to send an e-mail *****/
   sprintf (Command,"%s \"%s\" \"%s\" \"[%s] %s\" \"%s\"",
            Cfg_COMMAND_SEND_AUTOMATIC_E_MAIL,
            Cfg_AUTOMATIC_E_MAIL_FROM,Gbl.Usrs.Me.UsrDat.E_mail,Cfg_PLATFORM_SHORT_NAME,
            Txt_New_password_NO_HTML[Gbl.Prefs.Language],
            Gbl.Msg.FileNameMail);

   ReturnCode = system (Command);
   if (ReturnCode == -1)
      Lay_ShowErrorAndExit ("Error when running script to send e-mail.");

   /***** Remove temporary file *****/
   unlink (Gbl.Msg.FileNameMail);

   /***** Write message depending on return code *****/
   ReturnCode = WEXITSTATUS(ReturnCode);
   switch (ReturnCode)
     {
      case 0: // Message sent successfully
         Usr_SetMyPendingPassword (NewRandomPlainPassword);
         Lay_ShowAlert (INFO,Txt_If_you_have_written_your_ID_or_nickname_correctly_[Gbl.Prefs.Language]);
         break;
      case 1:
         Lay_ShowAlert (ERROR,"Ha ocurrido un problema enviando autom&aacute;ticamente un correo electr&oacute;nico.");	// Need translation!
         break;
      default:
         sprintf (Gbl.Message,"Error interno: el mensaje no ha podido enviarse correctamente." \
                              " C&oacute;digo de error devuelto por el script de env&iacute;o: %d",	// Need translation!
                  ReturnCode);
         Lay_ShowAlert (ERROR,Gbl.Message);
         break;
     }
  }

/****************************************************************************/
/*********************** Create a new random password ***********************/
/****************************************************************************/

static void Usr_CreateANewPassword (char PlainPassword[Usr_MAX_LENGTH_PLAIN_PASSWORD+1])
  {
   Str_CreateRandomAlphanumStr (PlainPassword,Usr_MIN_LENGTH_PLAIN_PASSWORD);
  }

/****************************************************************************/
/************************* Set my pending password **************************/
/****************************************************************************/

static void Usr_SetMyPendingPassword (char PlainPassword[Usr_MAX_LENGTH_PLAIN_PASSWORD+1])
  {
   char Query[512];

   /***** Encrypt my pending password *****/
   Cry_EncryptSHA512Base64 (PlainPassword,Gbl.Usrs.Me.PendingPassword);

   /***** Remove expired pending passwords from database *****/
   sprintf (Query,"DELETE FROM pending_passwd" \
                  " WHERE (UNIX_TIMESTAMP() > UNIX_TIMESTAMP(DateAndTime)+'%ld')",
            Cfg_TIME_TO_DELETE_OLD_PENDING_PASSWORDS);
   DB_QueryDELETE (Query,"can not remove expired pending passwords");

   /***** Update my current password in database *****/
   sprintf (Query,"REPLACE INTO pending_passwd (UsrCod,PendingPassword,DateAndTime)" \
                  " VALUES ('%ld','%s',NOW())",
            Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Me.PendingPassword);
   DB_QueryREPLACE (Query,"can not create pending password");
  }

/****************************************************************************/
/******************************** Update password ***************************/
/****************************************************************************/

void Usr_UpdateOtherPwd1 (void)
  {
   char NewPlainPassword[2][Usr_MAX_LENGTH_PLAIN_PASSWORD+1];
   char NewEncryptedPassword[Cry_LENGTH_ENCRYPTED_STR_SHA512_BASE64+1];

   Gbl.Usrs.Error = true;

   /***** Get other user's DNI from form and get user's data *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      Par_GetParam (Par_PARAM_SINGLE,"Paswd1",NewPlainPassword[0],Usr_MAX_LENGTH_PLAIN_PASSWORD);
      Par_GetParam (Par_PARAM_SINGLE,"Paswd2",NewPlainPassword[1],Usr_MAX_LENGTH_PLAIN_PASSWORD);

      if (strcmp (NewPlainPassword[0],NewPlainPassword[1]))
         // Paswords don't match
         sprintf (Gbl.Message,"No ha escrito dos veces la misma contrase&ntilde;a nueva.");	// Need translation!
      else
        {
         Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,NewPlainPassword[0],Usr_MAX_LENGTH_PLAIN_PASSWORD,true);
         Cry_EncryptSHA512Base64 (NewPlainPassword[0],NewEncryptedPassword);
         if (Usr_SlowCheckIfPasswordIsGood (NewPlainPassword[0],NewEncryptedPassword,Gbl.Usrs.Other.UsrDat.UsrCod))	// Good password
           {
            /* Update other user's data */
            strcpy (Gbl.Usrs.Other.UsrDat.Password,NewEncryptedPassword);
            Usr_UpdateCommonCard (&Gbl.Usrs.Other.UsrDat,Usr_REPLACE_DATA);

	    sprintf (Gbl.Message,"La contrase&ntilde;a de %s se ha modificado con &eacute;xito.",	// Need translation!
                     Gbl.Usrs.Other.UsrDat.FullName);
	    Gbl.Usrs.Error = false;

            /* Free memory used for other user' data */
            Usr_UsrDataDestructor (&Gbl.Usrs.Other.UsrDat);
           }
        }
     }
   else
      Gbl.Message[0] = '\0';
  }

void Usr_UpdateOtherPwd2 (void)
  {
   /***** Write message of error al actualizar the password and reintentar *****/
   if (Gbl.Message[0])
      Lay_ShowAlert (Gbl.Usrs.Error ? WARNING : INFO,Gbl.Message);

   if (Gbl.Usrs.Error)
     {
      Gbl.Usrs.Error = false;
      Usr_ShowFormOthPwd2 ();
     }
  }

/****************************************************************************/
/************************ Check if a password is good ***********************/
/****************************************************************************/

#define MAX_OTHER_USERS_USING_THE_SAME_PASSWORD 4

static bool Usr_SlowCheckIfPasswordIsGood (const char *PlainPassword,const char *EncryptedPassword,long UsrCod)
  {
   /***** Check if password sey makink fast checks *****/
   if (!Usr_FastCheckIfPasswordSeemsGood (PlainPassword))
      return false;

   /***** Check if password is found in DNI, first name or surnames of anybody *****/
   if (Usr_CheckIfPasswdIsDNIorName (PlainPassword))	// PlainPassword is a DNI, name or surname
     {
      sprintf (Gbl.Message,"La contrase&ntilde;a nueva no puede ser un DNI, ni un nombre, ni un apellido.");	// Need translation!
      return false;
     }

   /***** Check if password is used by too many other users *****/
   if (Usr_GetNumOtherUsrsWhoUseThisPassword (EncryptedPassword,UsrCod) >
       MAX_OTHER_USERS_USING_THE_SAME_PASSWORD)
     {
      sprintf (Gbl.Message,"La contrase&ntilde;a nueva es demasiado trivial. Use otra m&aacute;s dif&iacute;cil de averiguar.");	// Need translation!
      return false;
     }

   return true;
  }

/****************************************************************************/
/******** Check if a password is a DNI, a first name or a surname ***********/
/****************************************************************************/

static bool Usr_CheckIfPasswdIsDNIorName (const char *PlainPassword)
  {
   char Query[512+4*Usr_MAX_LENGTH_PLAIN_PASSWORD];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if password is found in user's ID, first name or surnames of anybody, from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE DNI='%s' OR FirstName='%s' OR Surname1='%s' OR Surname2='%s'",
            PlainPassword,PlainPassword,PlainPassword,PlainPassword);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a password matches a user's ID, a first name or a surname");

   /* Get number of rows */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a password matches a user's ID, a first name or a surname.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   if (NumRows)	// PlainPassword is a DNI, name or surname
      return true;

   return false;
  }

/****************************************************************************/
/************* Get the number of users who use yet a password ***************/
/****************************************************************************/

static unsigned Usr_GetNumOtherUsrsWhoUseThisPassword (const char *EncryptedPassword,long UsrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get number of other users who use a password from database *****/
   /* Query database */
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE Password='%s' AND UsrCod<>'%ld'",
            EncryptedPassword,UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a password is trivial");

   /* Get number of rows */
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a password is trivial.");

   /* Free structure that stores the query result */
   DB_FreeMySQLResult (&mysql_res);

   return NumRows;
  }

/****************************************************************************/
/********************** Check if a password seems good **********************/
/****************************************************************************/

bool Usr_FastCheckIfPasswordSeemsGood (const char *PlainPassword)
  {
   unsigned LengthPassword = strlen (PlainPassword),i;
   bool ItsANumber;

   /***** Check length of password *****/
   if (LengthPassword < Usr_MIN_LENGTH_PLAIN_PASSWORD)			// PlainPassword too short
     {
      sprintf (Gbl.Message,"La contrase&ntilde;a nueva ha de contener al menos %u caracteres.",	// Need translation!
               Usr_MIN_LENGTH_PLAIN_PASSWORD);
      return false;
     }

   /***** Check spaces in password *****/
   if (strchr (PlainPassword,(int) ' ') != NULL)	// PlainPassword with spaces
     {
      sprintf (Gbl.Message,"La contrase&ntilde;a nueva no puede contener espacios.");	// Need translation!
      return false;
     }

   /***** Check if password is a number *****/
   for (i=0, ItsANumber=true; i<LengthPassword && ItsANumber; i++)
      if (PlainPassword[i] < '0' || PlainPassword[i] > '9')
         ItsANumber = false;
   if (ItsANumber)
     {
      sprintf (Gbl.Message,"La contrase&ntilde;a nueva no puede estar formada s&oacute;lo por d&iacute;gitos.");	// Need translation!
      return false;
     }

   return true;
  }

/****************************************************************************/
/************* Get nickname of a user from his/her user's code **************/
/****************************************************************************/

bool Usr_GetNicknameFromUsrCod (long UsrCod,char *Nickname)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool Found;

   /***** Get user's nickname from database *****/
   sprintf (Query,"SELECT Nickname FROM usr_data WHERE UsrCod='%ld'",UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get nickname");

   if (NumRows == 0)
     {
      *Nickname = '\0';
      Found = false;
     }
   else
     {
      /* Get nickname */
      row = mysql_fetch_row (mysql_res);
      strcpy (Nickname,row[0]);
      Found = true;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return Found;
  }

/****************************************************************************/
/************* Get user's code of a user from his/her nickname **************/
/****************************************************************************/
// Returns true if nickname found in database

bool Usr_GetUsrCodFromNickname (const char *Nickname,struct UsrData *UsrDat)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool Found;

   /***** Get user's code from database *****/
   sprintf (Query,"SELECT UsrCod FROM usr_data WHERE Nickname='%s'",Nickname);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get user's code");

   if (NumRows == 0)
     {
      /* Set user's code to void */
      UsrDat->UsrCod = -1L;

      Found = false;
     }
   else
     {
      /* Get row */
      row = mysql_fetch_row (mysql_res);

      /* Get user's code */
      UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

      Found = true;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return Found;
  }

/****************************************************************************/
/************* Show form to the cambio of apodo *****************/
/****************************************************************************/

void Usr_ShowFormChangeNick (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Current_nickname[Txt_NUM_LANGUAGES];
   extern const char *Txt_New_nickname[Txt_NUM_LANGUAGES];
   extern const char *Txt_Change_nickname[Txt_NUM_LANGUAGES];

   /***** Get current nickname of user *****/
   Usr_GetNicknameFromUsrCod (Gbl.Usrs.Me.UsrDat.UsrCod,Gbl.Usrs.Me.Nickname);

   /***** Write message of inicio *****/
   sprintf (Gbl.Message,"Su apodo o <I>nickname</I> es personal y debe elegirlo cuidadosamente.<br />" \
                        "Si no le gustan los apodos, puede usar su nombre de pila, sus apellidos, sus iniciales, etc.<br />" \
                        "Puede usar su apodo para identificarse en %s" \
                        " y para que otros usuarios le env&iacute;en mensajes.",	// Need translation!
            Cfg_PLATFORM_SHORT_NAME);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Start form *****/
   Act_FormStart (ActRecNic);

   /***** Contenido of the form *****/
   fprintf (Gbl.F.Out,"<div align=\"center\"><table>");
   if (Gbl.Usrs.Me.Nickname[0])
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" width=\"50%%\" class=\"%s\">%s: </td>" \
                         "<td align=\"left\" width=\"50%%\" class=\"DAT\">%s</td>",
               The_ClassFormul[Gbl.Prefs.Theme],
               Txt_Current_nickname[Gbl.Prefs.Language],
               Gbl.Usrs.Me.Nickname);
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" width=\"50%%\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\" width=\"50%%\">" \
                      "<input type=\"text\" name=\"NewNick\" size=\"%d\" maxlength=\"%u\" value=\"%s\" />" \
                      "</td></tr></table></div>\n",
	    The_ClassFormul[Gbl.Prefs.Theme],
            Txt_New_nickname[Gbl.Prefs.Language],
            Usr_MAX_LENGTH_NICKNAME,Usr_MAX_LENGTH_NICKNAME,Gbl.Usrs.Me.Nickname);

   /***** End form *****/
   Lay_PutSendButton (Txt_Change_nickname[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/****************************************************************************/
/******************************* Update nickname ****************************/
/****************************************************************************/

void Usr_UpdateNick (void)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumNicks;
   char OldNickname[Usr_MAX_LENGTH_NICKNAME+1];
   bool Error = false;

   /***** Get new nickname from form *****/
   Par_GetParam (Par_PARAM_SINGLE,"NewNick",Gbl.Usrs.NewNickname,Usr_MAX_LENGTH_NICKNAME);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.NewNickname,Usr_MAX_LENGTH_NICKNAME,true);

   if (Usr_CheckIfNickIsValid (Gbl.Usrs.NewNickname))	// if new nickname is valid
     {
      /***** Check if new nickname exists in database *****/
      Usr_GetNicknameFromUsrCod (Gbl.Usrs.Me.UsrDat.UsrCod,OldNickname);
      if (!strcmp (Gbl.Usrs.NewNickname,OldNickname)) // My nickname match exactly the new nickname
	{
	 Error = true;
	 sprintf (Gbl.Message,"El apodo <strong>%s</strong> coincide con el que ya ten&iacute;a registrado.",	// Need translation!
                  Gbl.Usrs.NewNickname);
	}
      else if (strcasecmp (Gbl.Usrs.NewNickname,OldNickname)) // My nickname doesn't match, not even case insensitive, the new nickname
	{
	 /***** Get if the new nickname matches any of the nicknames of other users from database *****/
	 sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE Nickname='%s' AND UsrCod<>'%ld'",
                  Gbl.Usrs.NewNickname,Gbl.Usrs.Me.UsrDat.UsrCod);
         DB_QuerySELECT (Query,&mysql_res,"can not check if nickname already existed");

         /***** Get if nickname is repeated (row[0]) *****/
         row = mysql_fetch_row (mysql_res);
         if (sscanf (row[0],"%u",&NumNicks) != 1)
            Lay_ShowErrorAndExit ("Error when checking if nickname already existed.");

	 /***** Free structure that stores the query result *****/
         DB_FreeMySQLResult (&mysql_res);

         if (NumNicks) // Un apodo of another user coincide with the mo
	   {
	    Error = true;
	    sprintf (Gbl.Message,"El apodo <strong>%s</strong> ya ha sido registrado por otro usuario.",	// Need translation!
                     Gbl.Usrs.NewNickname);
	   }
	}
      if (!Error)
	{
	 // Now we know the new nickname is not already in database and is diffente to the current one
	 Usr_UpdateMyNick ();
	 if (OldNickname[0]) // I have already a nickname
	    sprintf (Gbl.Message,"Su nuevo apodo (<strong>%s</strong>)" \
                                 " ha sustituido al apodo anterior (<strong>%s</strong>).",
		     Gbl.Usrs.NewNickname,OldNickname);
	 else // No tena apodo
	    sprintf (Gbl.Message,"Su apodo (<strong>%s</strong>) se ha registrado correctamente.",	// Need translation!
                     Gbl.Usrs.NewNickname);
	}
     }
   else	// New nickname is not valid
     {
      Error = true;
      strcpy (Gbl.Message,"El apodo introducido no es v&aacute;lido.");	// Need translation!
     }

   if (Error)
     {
      Lay_ShowAlert (WARNING,Gbl.Message);
      Usr_ShowFormChangeNick ();
     }
   else
      Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/********************** Show form for changing my password ******************/
/****************************************************************************/

void Usr_ShowFormChgPwd (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Current_password[Txt_NUM_LANGUAGES];
   extern const char *Txt_New_password[Txt_NUM_LANGUAGES];
   extern const char *Txt_Retype_new_password[Txt_NUM_LANGUAGES];
   extern const char *Txt_Change_password[Txt_NUM_LANGUAGES];
   bool IHaveAPasswordInDB = (bool) Gbl.Usrs.Me.UsrDat.Password[0];

   /***** Help message *****/
   if (!IHaveAPasswordInDB) // If I don't have a password in database...
      Lay_ShowAlert (WARNING,"Antes de acceder a cualquier otra opci&oacute;n debe crear su contrase&ntilde;a.");	// Need translation!
   else if (Gbl.Usrs.Me.LoginPlainPassword[0])
     {
      if (!Usr_FastCheckIfPasswordSeemsGood (Gbl.Usrs.Me.LoginPlainPassword))
         Lay_ShowAlert (WARNING,"Su contrase&ntilde;a no es suficientemente segura.");	// Need translation!
     }

   sprintf (Gbl.Message,"La contrase&ntilde;a debe tener al menos %u caracteres y no puede contener espacios." \
                        " Es muy importante que no use un n&uacute;mero, ni un nombre o apellido, ni una palabra conocida.",	// Need translation!
            Usr_MIN_LENGTH_PLAIN_PASSWORD);
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Start form *****/
   Act_FormStart (ActRecPwd);

   fprintf (Gbl.F.Out,"<div align=\"center\"><table>");
   if (IHaveAPasswordInDB) // If I have a password in database...
      fprintf (Gbl.F.Out,"<tr><td align=\"right\" width=\"50%%\" class=\"%s\">%s: </td>" \
                         "<td align=\"left\" width=\"50%%\"><input type=\"password\" name=\"UsrPwd\" size=\"%u\" maxlength=\"%u\" /></td></tr>",
               The_ClassFormul[Gbl.Prefs.Theme],
               Txt_Current_password[Gbl.Prefs.Language],
               Usr_INPUT_LENGTH_PLAIN_PASSWORD,Usr_MAX_LENGTH_PLAIN_PASSWORD);
   fprintf (Gbl.F.Out,"<tr><td align=\"right\" width=\"50%%\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\" width=\"50%%\"><input type=\"password\" name=\"Paswd1\" size=\"%u\" maxlength=\"%u\" /></td></tr>" \
                      "<tr><td align=\"right\" width=\"50%%\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\" width=\"50%%\"><input type=\"password\" name=\"Paswd2\" size=\"%u\" maxlength=\"%u\" /></td></tr></table></div>\n",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_New_password[Gbl.Prefs.Language],
            Usr_INPUT_LENGTH_PLAIN_PASSWORD,Usr_MAX_LENGTH_PLAIN_PASSWORD,
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Retype_new_password[Gbl.Prefs.Language],
            Usr_INPUT_LENGTH_PLAIN_PASSWORD,Usr_MAX_LENGTH_PLAIN_PASSWORD);

   /***** Send button *****/
   Lay_PutSendButton (Txt_Change_password[Gbl.Prefs.Language]);

   /***** End form *****/
   fprintf (Gbl.F.Out,"</form>\n");
  }

/****************************************************************************/
/**************** Show form to ask user for another user's ID ***************/
/****************************************************************************/

void Usr_ReqDNIOthUsrPwd (void)
  {
   Lay_ShowAlert (INFO,"Escriba el DNI del usuario al que desea cambiar la contrase&ntilde;a.");	// Need translation!

   Usr_WriteFormToReqAnotherUserDNI (ActFrmPwdOthUsr);
  }

/****************************************************************************/
/***** Show form to the cambio of password of another user ******/
/****************************************************************************/

void Usr_ShowFormOthPwd1 (void)
  {
   /***** Get user's DNI from form *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /***** If I try to change my password,
             then go to the corresponding option in personal tab *****/
      if (Gbl.Usrs.Me.UsrDat.UsrCod == Gbl.Usrs.Other.UsrDat.UsrCod)
        {
         Gbl.CurrentTab = TabPer;
         Gbl.CurrentAct = ActFrmChgPwd;
        }
      Gbl.Usrs.Error = false;
     }
   else
      Gbl.Usrs.Error = true;
  }

void Usr_ShowFormOthPwd2 (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_New_password[Txt_NUM_LANGUAGES];
   extern const char *Txt_Change_password[Txt_NUM_LANGUAGES];
   char BreadcrumbStr[512];

   /***** Warning when DNI is not correct *****/
   if (Gbl.Usrs.Error)
      Usr_ReqDNIOthUsrPwd ();
   else
     {
      switch (Gbl.Usrs.Me.UsrTypeLog)
        {
         case Usr_TEACHER:
            /***** If I am a teacher, I only can change the password of students from current course *****/
            if (Usr_CheckIfUsrBelongsToCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentCrs.Crs.CrsCod))	// Always code reach this point, a course should have been selected
              {
               if (Gbl.Usrs.Other.UsrDat.UsrTypeInDB != Usr_STUDENT)
                  Lay_ShowErrorAndExit ("You can not change the password of this user.");

               if (!(Gbl.Usrs.Other.UsrDat.Accepted = Usr_GetIfUserHasAccepted (Gbl.Usrs.Other.UsrDat.UsrCod)))
                 {
                  Act_GetBreadcrumbStrForAction (ActLstAdm,Gbl.Prefs.Language,true,BreadcrumbStr);
                  sprintf (Gbl.Message,"No le est&aacute; permitido cambiar la contrase&ntilde;a %s con DNI <strong>%s</strong>" \
                                       " porque a&uacute;n no ha aceptado el alta en <strong>%s</strong>." \
                                       " Env&iacute;e el DNI <strong>%s</strong> a un administrador de la titulaci&oacute;n (%s)" \
                                       " o a un administrador de %s si no hay administrador de la titulaci&oacute;n," \
                                       " para que le cambien la contrase&ntilde;a %s.",	// Need translation!
	                   (Gbl.Usrs.Other.UsrDat.Gender == Usr_GENDER_FEMALE) ? "de la alumna" : (Gbl.Usrs.Other.UsrDat.Gender == Usr_GENDER_MALE) ? "del alumno" : "del alumno/a",Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentCrs.Crs.FullName,Gbl.Usrs.Other.UsrDat.DNI,	// Need translation!
                           BreadcrumbStr,
                           Cfg_PLATFORM_SHORT_NAME,
                           (Gbl.Usrs.Other.UsrDat.Gender == Usr_GENDER_FEMALE) ? "a la alumna" : (Gbl.Usrs.Other.UsrDat.Gender == Usr_GENDER_MALE) ? "al alumno" : "al alumno/a");	// Need translation!
                  Lay_ShowAlert (WARNING,Gbl.Message);
                  Usr_ReqDNIOthUsrPwd ();
                  return;
                 }
              }
            else
              {
               sprintf (Gbl.Message,"No existe el usuario con DNI <strong>%s</strong> en <strong>%s</strong>.",	// Need translation!
	                Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentCrs.Crs.FullName);
               Lay_ShowAlert (WARNING,Gbl.Message);
               Usr_ReqDNIOthUsrPwd ();
               return;
              }
            break;
         case Usr_DEG_ADMIN:
            if (!Usr_CheckIfUsrIsAdmOfAllDegs (Gbl.Usrs.Me.UsrDat.UsrCod))	// If I am not admin of all degrees
               /***** If I am an administrator of current degree, but not of all degrees, I only can change the password of users from current degree *****/
               if (!Usr_CheckIfUsrBelongsToDeg (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentDeg.Deg.DegCod))	// Always I am an administrator, a degree should have been selected
                 {
                  sprintf (Gbl.Message,"No existe el usuario con DNI <strong>%s</strong>" \
                                       " en la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
	                   Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentDeg.Deg.FullName);
                  Lay_ShowAlert (WARNING,Gbl.Message);
                  Usr_ReqDNIOthUsrPwd ();
                  return;
                 }
            break;
         case Usr_SUPERUSER:
            break;
         default:	// This shouldn't happen
            Lay_ShowErrorAndExit ("You can not change another user's password.");
            break;
        }

      /***** Message informativo *****/
      Lay_ShowAlert (INFO,"Cambio de la contrase&ntilde;a del usuario siguiente:");	// Need translation!

      /***** Form to change password *****/
      /* Start form */
      fprintf (Gbl.F.Out,"<div align=\"center\">");
      Act_FormStart (ActRecPwdOthUsr);
      Usr_PutParamOtherUsrDNIPlain ();

      /* Show user's record */
      fprintf (Gbl.F.Out,"<table><tr><td colspan=\"2\" align=\"center\">");
      Rec_ShowCommonRecord (RECORD_LIST,&Gbl.Usrs.Other.UsrDat,NULL);

      /* New password */
      fprintf (Gbl.F.Out,"</td></tr><tr><td align=\"right\" width=\"50%%\" class=\"%s\">%s: </td>" \
                         "<td align=\"left\" width=\"50%%\"><input type=\"password\" name=\"Paswd1\" size=\"%u\" maxlength=\"%u\" /></td></tr>" \
                         "<tr><td align=\"right\" width=\"50%%\" class=\"%s\">Reescriba la nueva contrase&ntilde;a: </td>" \
                         "<td align=\"left\" width=\"50%%\"><input type=\"password\" name=\"Paswd2\" size=\"%u\" maxlength=\"%u\" /></td></tr>" \
                         "</table>",	// Need translation!
               The_ClassFormul[Gbl.Prefs.Theme],Txt_New_password[Gbl.Prefs.Language],
               Usr_INPUT_LENGTH_PLAIN_PASSWORD,Usr_MAX_LENGTH_PLAIN_PASSWORD,
               The_ClassFormul[Gbl.Prefs.Theme],
               Usr_INPUT_LENGTH_PLAIN_PASSWORD,Usr_MAX_LENGTH_PLAIN_PASSWORD);

      /* End form */
      Lay_PutSendButton (Txt_Change_password[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</form></div>\n");

      /***** Free memory used for user's data *****/
      Usr_UsrDataDestructor (&Gbl.Usrs.Other.UsrDat);
     }
  }

/****************************************************************************/
/*********************** Form for sending my photo **************************/
/****************************************************************************/

void Usr_ReqMyPhoto (void)
  {
   /* Show the form for sending the photo */
   Usr_ReqPhoto (&Gbl.Usrs.Me.UsrDat,Gbl.Usrs.Me.MyPhotoExists,Gbl.Usrs.Me.PhotoURL);
  }

/****************************************************************************/
/****************** Form for sending other user's photo *********************/
/****************************************************************************/

void Usr_ReqUsrPhoto (struct UsrData *UsrDat)
  {
   bool PhotoExists;
   char PhotoURL[PATH_MAX+1];

   /* Check if user's photo exists and create a link to it */
   PhotoExists = Usr_BuildLinkToPhoto (UsrDat,PhotoURL);

   /* Show the form for sending the photo */
   Usr_ReqPhoto (UsrDat,PhotoExists,PhotoURL);
  }

/****************************************************************************/
/****************** Show a form for sending an user's photo *****************/
/****************************************************************************/

void Usr_ReqPhoto (const struct UsrData *UsrDat,bool PhotoExists,const char *PhotoURL)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_PHOTO_Your_photo_exists[Txt_NUM_LANGUAGES];
   extern const char *Txt_PHOTO_The_photo_of_X_exists[Txt_NUM_LANGUAGES];
   extern const char *Txt_PHOTO_If_you_send_a_new_picture[Txt_NUM_LANGUAGES];
   extern const char *Txt_PHOTO_X_has_no_photo_yet[Txt_NUM_LANGUAGES];
   extern const char *Txt_File_with_your_photo[Txt_NUM_LANGUAGES];
   extern const char *Txt_Upload_photo[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_photo[Txt_NUM_LANGUAGES];
   bool ItsMe = (UsrDat->UsrCod == Gbl.Usrs.Me.UsrDat.UsrCod);
   fprintf (Gbl.F.Out,"<div align=\"center\">\n");

   /***** Write message about photo presence or ausence *****/
   if (PhotoExists)	// User has photo
     {
      if (ItsMe)	// It's me
         sprintf (Gbl.Message,"%s %s",
                  Txt_PHOTO_Your_photo_exists[Gbl.Prefs.Language],
                  Txt_PHOTO_If_you_send_a_new_picture[Gbl.Prefs.Language]);
      else		// It's another user
        {
	 sprintf (Gbl.Message,Txt_PHOTO_The_photo_of_X_exists[Gbl.Prefs.Language],
                  UsrDat->FullName);
         strcat (Gbl.Message," ");
         strcat (Gbl.Message,Txt_PHOTO_If_you_send_a_new_picture[Gbl.Prefs.Language]);
        }
      Lay_ShowAlert (WARNING,Gbl.Message);
      /* Show photo */
      Usr_ShowUsrPhoto (UsrDat,PhotoURL,Usr_PHOTO_REAL_WIDTH,Usr_PHOTO_REAL_HEIGHT,true);
     }
   else	if (!ItsMe)	// It's another user, and he has no photo
     {
      sprintf (Gbl.Message,Txt_PHOTO_X_has_no_photo_yet[Gbl.Prefs.Language],
               UsrDat->FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   sprintf (Gbl.Message,"Puede enviar un archivo con una imagen en formato <I>jpg</I>.<br />" \
                        "La imagen debe contener al menos un rostro en posici&oacute;n frontal," \
		        " y con el fondo blanco o muy claro.");	// Need translation!
   Lay_ShowAlert (INFO,Gbl.Message);

   /***** Write a form to send photo *****/
   if (ItsMe)
      Act_FormStart (ActDetMyPho);
   else
     {
      Act_FormStart (ActDetUsrPho);
      Usr_PutParamOtherUsrCodEncrypted (UsrDat->EncryptedUsrCod);
     }
   fprintf (Gbl.F.Out,"<table>" \
                      "<tr><td align=\"right\" class=\"%s\">%s:<br /></td>" \
                      "<td align=\"left\"><input type=\"file\" name=\"%s\" size=\"40\" maxlength=\"100\" value=\"%s.jpg\" /></td></tr>" \
                      "<tr><td colspan=\"2\" align=\"center\"><input type=\"submit\" value=\"%s\" accept=\"image/jpeg\" /></td></tr>" \
                      "</table></form>\n",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_File_with_your_photo[Gbl.Prefs.Language],Brw_NAME_OF_PARAM_FILENAME_ORG,UsrDat->DNI,
            Txt_Upload_photo[Gbl.Prefs.Language]);

   /***** Write a form to remove photo if it exists *****/
   if (PhotoExists)	// User has photo
     {
      Lay_ShowAlert (INFO,"...o puede eliminar la fotograf&iacute;a actual.");	// Need translation!
      if (ItsMe)
         Act_FormStart (ActRemMyPho);
      else
        {
         Act_FormStart (ActRemUsrPho);
         Usr_PutParamOtherUsrCodEncrypted (UsrDat->EncryptedUsrCod);
        }
      fprintf (Gbl.F.Out,"<input type=\"submit\" value=\"%s\" /></form>\n",
               Txt_Remove_photo[Gbl.Prefs.Language]);
     }

   fprintf (Gbl.F.Out,"</div>\n");
  }

/****************************************************************************/
/**************** Receive my photo and detect faces on it *******************/
/****************************************************************************/

void Usr_RecMyPhotoDetFaces (void)
  {
   /***** Receive my photo and detect faces on it *****/
   Usr_ReceivePhotoAndDetectFaces (true,&Gbl.Usrs.Me.UsrDat);
  }

/****************************************************************************/
/********** Receive another user's photo and detect faces on it *************/
/****************************************************************************/

void Usr_RecUsrPhotoDetFaces (void)
  {
   /***** Get user's code from form *****/
   Usr_GetParamOtherUsrCodEncrypted ();

   /***** Get password, user type and user's data from database *****/
   if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat))
      /***** Receive photo *****/
      Usr_ReceivePhotoAndDetectFaces (false,&Gbl.Usrs.Other.UsrDat);
   else
      Lay_ShowErrorAndExit ("User doesn't exist.");
  }

/****************************************************************************/
/***************************** Remove my photo ******************************/
/****************************************************************************/

void Usr_RemoveMyPhoto1 (void)
  {
   /***** Remove photo *****/
   Gbl.Error = !Usr_RemovePhoto (&Gbl.Usrs.Me.UsrDat);

   /***** The link to my photo is not valid now, so build it again before writing the web page *****/
   Gbl.Usrs.Me.MyPhotoExists = Usr_BuildLinkToPhoto (&Gbl.Usrs.Me.UsrDat,Gbl.Usrs.Me.PhotoURL);
  }

void Usr_RemoveMyPhoto2 (void)
  {
   extern const char *Txt_Photo_removed[Txt_NUM_LANGUAGES];
   
   if (!Gbl.Error)
      Lay_ShowAlert (INFO,Txt_Photo_removed[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/*********************** Remove another user's photo ************************/
/****************************************************************************/

void Usr_RemoveUsrPhoto (void)
  {
   extern const char *Txt_Photo_removed[Txt_NUM_LANGUAGES];

   /***** Get user's code from form *****/
   Usr_GetParamOtherUsrCodEncrypted ();

   /***** Get password, user type and user's data from database *****/
   if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat))
     {
      /***** Remove photo *****/
      if (Usr_RemovePhoto (&Gbl.Usrs.Other.UsrDat))
         Lay_ShowAlert (INFO,Txt_Photo_removed[Gbl.Prefs.Language]);
     }
   else
      Lay_ShowErrorAndExit ("User doesn't exist.");
  }

/****************************************************************************/
/**************** Receive a photo and detect faces on it ********************/
/****************************************************************************/

void Usr_ReceivePhotoAndDetectFaces (bool ItsMe,const struct UsrData *UsrDat)
  {
   char FileNamePhotoSrc[PATH_MAX+1];
   char FileNamePhotoTmp[PATH_MAX+1];	// Full name (including path and .jpg) of the destination temporary file
   char FileNamePhotoMap[PATH_MAX+1];	// Full name (including path) of the temporary file with the original image with faces
   char FileNameTxtMap[PATH_MAX+1];	// Full name (including path) of the temporary file with the text neccesary to make the image map
   char PathRelPhoto[PATH_MAX+1];
   FILE *FileTxtMap = NULL;	// Temporary file with the text neccesary to make the image map. Initialized to avoid warning
   char FileType[Brw_MAX_BYTES_MIME_TYPE+1];
   bool WrongType = false;
   char Command[256]; // Command to call the program of preprocessing of photos
   int ReturnCode;
   int NumLastForm = 0;	// Initialized to avoid warning
   unsigned NumFacesTotal = 0;
   unsigned NumFacesGreen = 0;
   unsigned NumFacesRed = 0;
   unsigned NumFace;
   unsigned X;
   unsigned Y;
   unsigned Radius;
   unsigned BackgroundCode;
   char StrFileName[NAME_MAX+1];

   /***** Creates directories if not exist *****/
   Fil_CreateDirIfNotExists (Cfg_PATH_REL_PHOTOS_PRIVATE);
   Fil_CreateDirIfNotExists (Cfg_PATH_REL_PHOTOS_TMP_PUBLIC);

   /***** Remove old files *****/
   Brw_RemoveOldTmpFiles (Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,false);

   /***** First of all, copy in disk the file received from stdin (really from Gbl.F.Tmp) *****/
   Fil_StartReceptionOfFile (FileNamePhotoSrc,FileType);

   /* Check if the file type is image/jpeg or image/pjpeg or application/octet-stream */
   if (strcmp (FileType,"image/jpeg"))
      if (strcmp (FileType,"image/pjpeg"))
	 if (strcmp (FileType,"application/octet-stream"))
	    WrongType = true;
   if (WrongType)
     {
      Lay_ShowAlert (WARNING,"El archivo no es <em>jpeg</em>.");	// Need translation!
      return;
     }

   /* End the reception of photo in a temporary file */
   sprintf (FileNamePhotoTmp,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted);
   if (!Fil_EndReceptionOfFile (FileNamePhotoTmp))
      return;

   /* Copy the original photo received to private directory.
      The purpose of this copy is only to have a backup used for researching better methods to detect faces in images */
   sprintf (PathRelPhoto,"%s/%s_original.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,UsrDat->DNI);
   Fil_FastCopyOfFiles (FileNamePhotoTmp,PathRelPhoto);

   /***** Call to program that makes photo processing / face detection *****/
   sprintf (Command,Cfg_COMMAND_FACE_DETECTION,FileNamePhotoTmp);
   ReturnCode = system (Command);
   if (ReturnCode == -1)
      Lay_ShowErrorAndExit ("Error when running command to process photo and detect faces.");

   /***** Write message depending on return code *****/
   ReturnCode = WEXITSTATUS(ReturnCode);
   switch (ReturnCode)
     {
      case 0:	// Faces detected
         /***** Open text file with text for image map *****/
         sprintf (FileNameTxtMap,"%s/%s_map.txt",Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted);
         if ((FileTxtMap = fopen (FileNameTxtMap,"rt")) == NULL)
            Lay_ShowErrorAndExit ("Can not read text file with coordinates of detected faces.");

         /***** Read file with coordinates for image map and compute the number of faces *****/
         NumLastForm = Gbl.NumForm;
         while (!feof (FileTxtMap))
           {
            if (fscanf (FileTxtMap,"%u %u %u %u %s\n",&X,&Y,&Radius,&BackgroundCode,StrFileName) != 5)	// Example of StrFileName = "4924a838630e_016"

               break;
            if (BackgroundCode == 1)
              {
               NumFacesGreen++;
               Act_FormStart (ItsMe ? ActUpdMyPho : ActUpdUsrPho);
               if (!ItsMe)
                  Usr_PutParamOtherUsrCodEncrypted (UsrDat->EncryptedUsrCod);
               fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"FileName\" value=\"%s\" />",StrFileName);
               fprintf (Gbl.F.Out,"</form>");
              }
            else
               NumFacesRed++;
           }
         NumFacesTotal = NumFacesGreen + NumFacesRed;
	 break;
      case 1:	// No faces detected
         NumFacesTotal = NumFacesGreen = NumFacesRed = 0;
	 break;
      default:	// Error
         sprintf (Gbl.Message,"Photo could not be processed successfully.<br />" \
                              "Error code returned by the program of processing: %d",
                  ReturnCode);
         Lay_ShowErrorAndExit (Gbl.Message);
         break;
     }

   /***** Message to the user about the number of faces detected in the image*****/
   if (NumFacesTotal == 0)
      Lay_ShowAlert (WARNING,"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.<br />" \
                             "Pruebe a enviar otra imagen donde el rostro aparezca de frente y el fondo detr&aacute;s de la cabeza sea blanco o muy claro.");	// Need translation!
   else if (NumFacesTotal == 1)
     {
      if (NumFacesGreen == 1)
         Lay_ShowAlert (INFO,"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal" \
                             " y con el fondo detr&aacute;s de la cabeza suficientemente claro.<br />" \
                             "Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.");	// Need translation!
      else
         Lay_ShowAlert (WARNING,"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal," \
                                " pero el fondo detr&aacute;s de la cabeza es demasiado oscuro.<br />" \
                                "Pruebe a enviar otra imagen donde el rostro aparezca de frente y el fondo detr&aacute;s de la cabeza sea blanco o muy claro.");	// Need translation!
     }
   else	// NumFacesTotal > 1
     {
      if (NumFacesRed == 0)
        {
         sprintf (Gbl.Message,"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal" \
                              " y con el fondo detr&aacute;s de la cabeza suficientemente claro.<br />" \
                              "Si desea elegir uno de esos rostros, pulse sobre &eacute;l.",	// Need translation!
                  NumFacesGreen);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
      else if (NumFacesGreen == 0)
        {
         sprintf (Gbl.Message,"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal," \
                              " pero tienen el fondo detr&aacute;s de la cabeza demasiado oscuro.<br />" \
                              "Pruebe a enviar otra imagen donde el rostro aparezca de frente" \
                              " y el fondo detr&aacute;s de la cabeza sea blanco o muy claro.",	// Need translation!
                  NumFacesRed);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else if (NumFacesGreen == 1)
        {
         sprintf (Gbl.Message,"Se han detectado %u rostros en posici&oacute;n frontal:<br />" \
                              "%u con el fondo detr&aacute;s de la cabeza suficientemente claro (en color verde),<br />" \
                              "%u con el fondo detr&aacute;s de la cabeza demasiado oscuro (en color rojo).<br />" \
                              "Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.",	// Need translation!
                  NumFacesTotal,NumFacesGreen,NumFacesRed);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
      else
        {
         sprintf (Gbl.Message,"Se han detectado %u rostros en posici&oacute;n frontal:<br />" \
                              "%u con el fondo detr&aacute;s de la cabeza suficientemente claro (en color verde),<br />" \
                              "%u con el fondo detr&aacute;s de la cabeza demasiado oscuro (en color rojo).<br />" \
                              "Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.",	// Need translation!
                  NumFacesTotal,NumFacesGreen,NumFacesRed);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Create map *****/
   fprintf (Gbl.F.Out,"<map name=\"faces_map\">\n");
   if (NumFacesTotal)
     {
      /***** Read again the file with coordinates and create area shapes *****/
      rewind (FileTxtMap);
      for (NumFace=0; !feof (FileTxtMap);)
        {
         if (fscanf (FileTxtMap,"%u %u %u %u %s\n",&X,&Y,&Radius,&BackgroundCode,StrFileName) != 5)
            break;
         if (BackgroundCode == 1)
           {
            NumFace++;
            fprintf (Gbl.F.Out,"<area shape=\"circle\" href=\"javascript:document.forms[%d].submit();\" coords=\"%u,%u,%u\">\n",
                     NumLastForm+NumFace,X,Y,Radius);
           }
        }
      /***** Close text file with text for image map *****/
      fclose (FileTxtMap);
     }
   fprintf (Gbl.F.Out,"</map>\n");

   /***** Show map photo *****/
   sprintf (FileNamePhotoMap,"%s/%s_map.jpg",Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted);
   fprintf (Gbl.F.Out,"<div align=\"center\" class=\"TITULO\">" \
                      "<img src=\"%s/%s_map.jpg\" usemap=\"#faces_map\" />" \
                      "</div>",
            Cfg_URL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted);
  }

/****************************************************************************/
/**************************** Update my photo *******************************/
/****************************************************************************/

void Usr_UpdateMyPhoto1 (void)
  {
   Usr_UpdatePhoto1 (&Gbl.Usrs.Me.UsrDat);

   /***** The link to my photo is not valid now, so build it again before writing the web page *****/
   Gbl.Usrs.Me.MyPhotoExists = Usr_BuildLinkToPhoto (&Gbl.Usrs.Me.UsrDat,Gbl.Usrs.Me.PhotoURL);
  }

void Usr_UpdateMyPhoto2 (void)
  {
   Usr_UpdatePhoto2 ();
  }

/****************************************************************************/
/********************** Update another user's photo *************************/
/****************************************************************************/

void Usr_UpdateUsrPhoto1 (void)
  {
   /***** Get user's code from form *****/
   Usr_GetParamOtherUsrCodEncrypted ();

   /***** Get password, user type and user's data from database *****/
   if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&Gbl.Usrs.Other.UsrDat))
      /***** Update photo *****/
      Usr_UpdatePhoto1 (&Gbl.Usrs.Other.UsrDat);
   else
      Lay_ShowErrorAndExit ("User doesn't exist.");
  }

void Usr_UpdateUsrPhoto2 (void)
  {
   Usr_UpdatePhoto2 ();
  }

/****************************************************************************/
/*************** Update a user's photo with a selected face *****************/
/****************************************************************************/

static void Usr_UpdatePhoto1 (struct UsrData *UsrDat)
  {
   char PathPhotoTmp[PATH_MAX+1];	// Full name (including path and .jpg) of the temporary file with the selected face
   char PathRelPhoto[PATH_MAX+1];

   /***** Get the name of the file with the selected face *****/
   Par_GetParam (Par_PARAM_SINGLE,"FileName",Gbl.Usrs.FileNamePhoto,NAME_MAX);	// Example of FileNamePhoto: "4924a838630e_016"

   /***** Convert the temporary photo resulting of the processing to the current photo of the user *****/
   sprintf (PathPhotoTmp,"%s/%s_paso3.jpg",Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,Gbl.Usrs.FileNamePhoto);
   if (Fil_CheckIfPathExists (PathPhotoTmp))	// The file with the selected photo exists
     {
      /* Copy the temporary file of the third (last) step resulting of the processing to the directory of private photos */
      sprintf (PathRelPhoto,"%s/%s.jpg",Cfg_PATH_REL_PHOTOS_PRIVATE,UsrDat->DNI);
      Fil_FastCopyOfFiles (PathPhotoTmp,PathRelPhoto);

      /* Update public photo name in database */
      Usr_UpdatePhotoName (UsrDat);

      /* Remove the user from the list of users without photo */
      Usr_RemoveUsrFromTableClicksWithoutPhoto (UsrDat->UsrCod);

      Gbl.Error = false;
     }
   else
      Gbl.Error = true;
  }

static void Usr_UpdatePhoto2 (void)
  {
   static const char *CaptionsPhotos[3] =
     {
      "Rostro seleccionado",
      "Contraste y saturaci&oacute;n mejorados",
      "Balance de blancos realizado"
     };	// Need translation!
   unsigned NumPhoto;

   /***** Show the three images resulting of the processing *****/
   fprintf (Gbl.F.Out,"<table border=\"0\" width=\"100%%\" cellpadding=\"0\"><tr>");
   for (NumPhoto=0; NumPhoto<3; NumPhoto++)
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"top\" class=\"DAT\">" \
                         "<img src=\"%s/%s_paso%u.jpg\" width=\"%u\" height=\"%u\" />" \
                         "<br />%s</td>",
               Cfg_URL_PHOTOS_TMP_PUBLIC,Gbl.Usrs.FileNamePhoto,NumPhoto+1,Usr_PHOTO_REAL_WIDTH,Usr_PHOTO_REAL_HEIGHT,
               CaptionsPhotos[NumPhoto]);
   fprintf (Gbl.F.Out,"</tr></table>");

   /***** Show message *****/
   if (Gbl.Error)	// The file with the selected photo don't exists!
      Lay_ShowErrorAndExit ("Selected photo doesn't exist.");

   Lay_ShowAlert (INFO,"La fotograf&iacute;a se ha actualizado.");	// Need translation!
  }

/****************************************************************************/
/********************** Get fields of record from form ************************/
/****************************************************************************/

void Usr_GetUsrCommonDataFromForm (struct UsrData *UsrDat,bool GetUsrTypeFromForm,bool GetGender,bool GetPublicPhoto)
  {
   char UnsignedStr[10+1];
   char LongStr[1+10+1];
   unsigned UnsignedNum;

   if (GetUsrTypeFromForm) // Get user's type from form
     {
      Par_GetParam (Par_PARAM_SINGLE,"UsrType",UnsignedStr,10);
      UsrDat->UsrTypeInDB = Usr_ConvertUnsignedStrToUsrType (UnsignedStr);
     }

   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"Surname1"  ,UsrDat->Surname1  ,Usr_MAX_BYTES_NAME   ),Usr_MAX_BYTES_NAME,true);
   Str_ConvertToTitleType (UsrDat->Surname1);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"Surname2"  ,UsrDat->Surname2  ,Usr_MAX_BYTES_NAME   ),Usr_MAX_BYTES_NAME,true);
   Str_ConvertToTitleType (UsrDat->Surname2);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"FirstName" ,UsrDat->FirstName ,Usr_MAX_BYTES_NAME   ),Usr_MAX_BYTES_NAME,true);
   Str_ConvertToTitleType (UsrDat->FirstName);
   Usr_BuildFullName (UsrDat);

   if (GetGender)	// Get gender from form
     {
      Par_GetParam (Par_PARAM_SINGLE,"Gender",UnsignedStr,10);
      UsrDat->Gender = Usr_GENDER_UNKNOWN;
      if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
         if (UnsignedNum <= Usr_GENDER_MALE)
            UsrDat->Gender = (Usr_Gender_t) UnsignedNum;
     }
   
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"E_mail"       ,UsrDat->E_mail    ,Cte_MAX_BYTES_STRING   ),Cte_MAX_BYTES_STRING,true);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"WWW"          ,UsrDat->WWW       ,Cte_MAX_BYTES_URL      ),Cte_MAX_BYTES_URL   ,true);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"Skype"        ,UsrDat->Skype     ,Cte_MAX_BYTES_STRING   ),Cte_MAX_BYTES_STRING,true);

   if (GetPublicPhoto)	// Get whether photo is public from form
      UsrDat->PublicPhoto = Prf_GetParamPublicPhoto ();

   /***** Get institution code *****/
   Par_GetParam (Par_PARAM_SINGLE,"InsCod",LongStr,1+10);
   UsrDat->InsCod = Str_ConvertStrCodToLongCod (LongStr);
  }

void Usr_GetStdDataFromForm (struct UsrData *UsrDat)
  {
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"LocalAddress" ,UsrDat->Std.LocalAddress ,Cte_MAX_BYTES_STRING   ),Cte_MAX_BYTES_STRING,true);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"LocalPhone"   ,UsrDat->Std.LocalPhone   ,Usr_MAX_BYTES_PHONE      ),Usr_MAX_BYTES_PHONE   ,true);

   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"FamilyAddress",UsrDat->Std.FamilyAddress,Cte_MAX_BYTES_STRING   ),Cte_MAX_BYTES_STRING,true);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"FamilyPhone"  ,UsrDat->Std.FamilyPhone  ,Usr_MAX_BYTES_PHONE      ),Usr_MAX_BYTES_PHONE   ,true);

   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"OriginPlace"  ,UsrDat->Std.OriginPlace  ,Cte_MAX_BYTES_STRING   ),Cte_MAX_BYTES_STRING,true);
   Str_ConvertToTitleType (UsrDat->Std.OriginPlace);

   Dat_GetDateFromForm ("DiaNac","MesNac","AnoNac",&(UsrDat->Std.Birthday.Day),&(UsrDat->Std.Birthday.Month),&(UsrDat->Std.Birthday.Year));
   Dat_ConvDateToDateStr (&(UsrDat->Std.Birthday),UsrDat->Std.StrBirthday);

   Usr_GetUsrObservationsFromForm (UsrDat);
  }

void Usr_GetTchDataFromForm (struct UsrData *UsrDat)
  {
   char LongStr[1+10+1];

   /***** Get centre code *****/
   Par_GetParam (Par_PARAM_SINGLE,"CtrCod",LongStr,1+10);
   UsrDat->Tch.CtrCod = Str_ConvertStrCodToLongCod (LongStr);

   /***** Get department code *****/
   Par_GetParam (Par_PARAM_SINGLE,"DptCod",LongStr,1+10);
   UsrDat->Tch.DptCod = Str_ConvertStrCodToLongCod (LongStr);

   /***** Get office *****/
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"Office"     ,UsrDat->Tch.Office     ,Cte_MAX_BYTES_STRING),Cte_MAX_BYTES_STRING,true);

   /***** Get office phone number *****/
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Par_GetParam (Par_PARAM_SINGLE,"OfficePhone",UsrDat->Tch.OfficePhone,Usr_MAX_BYTES_PHONE ),Usr_MAX_BYTES_PHONE ,true);
  }

/****************************************************************************/
/******* Check a user's code and get all user's data from user's code *******/
/****************************************************************************/
// Input: UsrDat->UsrCod must hold a valid user code
// Output: When true ==> UsrDat will hold all user's data
//         When false ==> UsrDat is reset, except user's code

bool Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (struct UsrData *UsrDat)
  {
   /***** Check if a user exists having this user's code *****/
   if (Usr_ChkIfUsrCodExists (UsrDat->UsrCod))
     {
      /* Get user's data */
      Usr_GetAllUsrDataFromUsrCod (UsrDat);
      return true;
     }

   /***** No user's code found *****/
   UsrDat->DNI[0] = '\0';
   Usr_ResetUsrDataExceptUsrCodAndDNI (UsrDat);
   return false;
  }

/****************************************************************************/
/******** Check a user's DNI and get all user's data from user's DNI ********/
/****************************************************************************/
// Input: UsrDat->DNI must hold a valid DNI
// Output: When true ==> UsrDat will hold all user's data
//         When false ==> UsrDat is reset, except DNI

bool Usr_ChkDNIAndGetAllUsrDataFromDNI (struct UsrData *UsrDat)
  {
   /***** Check if a user exists having this DNI *****/
   if (Usr_ChkIfDNIExists (UsrDat->DNI))
     {
      /* Get user's data */
      Usr_GetAllUsrDataFromDNI (UsrDat);
      return true;
     }

   /***** No DNI found *****/
   UsrDat->UsrCod = -1L;
   Usr_ResetUsrDataExceptUsrCodAndDNI (UsrDat);
   return false;
  }

/****************************************************************************/
/********* Get if a user has accepted to belong to current course ***********/
/****************************************************************************/

bool Usr_GetIfUserHasAccepted (long UsrCod)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   bool Accepted = false;

   /***** Get if a user has accepted belonging to current course from database *****/
   sprintf (Query,"SELECT Accepted FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not check if a users has accepted belonging to course");

   if (NumRows == 1)
     {
      /* Get if accepted */
      row = mysql_fetch_row (mysql_res);
      Accepted = (Str_ConvertToUpperLetter (row[0][0]) != 'N');
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return Accepted;
  }

/****************************************************************************/
/*** Get the comments of the record of a student desde the form ***/
/****************************************************************************/

void Usr_GetUsrObservationsFromForm (struct UsrData *UsrDat)
  {
   /***** Check if memory is allocated for comments *****/
   if (!UsrDat->Std.Comments)
      Lay_ShowErrorAndExit ("Can not read comments of a user.");

   /***** Get the parameter with the comments *****/
   Par_GetParam (Par_PARAM_SINGLE,"Comments",UsrDat->Std.Comments,Cte_MAX_BYTES_TEXT);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_HTML,UsrDat->Std.Comments,Cte_MAX_BYTES_TEXT,true);
  }

/****************************************************************************/
/*************** Update my last accessed tab in database ********************/
/****************************************************************************/

void Usr_UpdateMyLastTabAndTime (void)
  {
   char Query[512];

   /***** Update my last accessed tab and my last time of click in database *****/
   sprintf (Query,"UPDATE usr_data SET LastTab='%u',LastTime=NOW() WHERE UsrCod='%ld'",
            (unsigned) Gbl.CurrentTab,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update tab and time of last click");
  }

/****************************************************************************/
/**************** Update and show data from identified user *****************/
/****************************************************************************/

void Usr_UpdateMyCard (void)
  {
   /***** Get my data from form *****/
   Usr_GetUsrCommonDataFromForm (&Gbl.Usrs.Me.UsrDat,false,true,true);
   switch (Gbl.Usrs.Me.UsrDat.UsrTypeInDB)
     {
      case Usr_STUDENT:
         Usr_GetStdDataFromForm (&Gbl.Usrs.Me.UsrDat);
         break;
      case Usr_TEACHER:
         Usr_GetTchDataFromForm (&Gbl.Usrs.Me.UsrDat);
	 break;
      default:
         break;
     }

   /***** Update my data in database *****/
   Usr_UpdateCommonCard (&Gbl.Usrs.Me.UsrDat,Usr_REPLACE_DATA);
  }

/****************************************************************************/
/******************* Create and show data from other user *******************/
/****************************************************************************/

void Usr_CreatAndShowUsrCardAndRegisterInCrs (void)
  {
   /***** Get user's DNI from form *****/
   Usr_GetParamOtherUsrDNIPlain ();

   if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Other.UsrDat.DNI))	// DNI valid
     {
      Gbl.Usrs.Other.UsrDat.UsrCod = -1L;

      /***** Initialize user's password to zero *****/
      Gbl.Usrs.Other.UsrDat.Password[0] = '\0';

      /***** Get user's data from form *****/
      Usr_GetUsrCommonDataFromForm (&Gbl.Usrs.Other.UsrDat,true,false,false);

      /***** Add user to current course *****/
      Usr_CreatUsrIfNotExistsAndRegisterHimInCurrentCrs (&Gbl.Usrs.Other.UsrDat,Usr_INSERT_NEW_DATA,Cte_VERBOSE);
     }
   else	// DNI not valid
     {
      /***** Write message *****/
      sprintf (Gbl.Message,"El DNI <strong>%s</strong> no es v&aacute;lido.",	// Need translation!
               Gbl.Usrs.Other.UsrDat.DNI);
      Lay_ShowAlert (ERROR,Gbl.Message);
     }
  }

/****************************************************************************/
/****************** Update and show data from other user ********************/
/****************************************************************************/

void Usr_ModifAndShowUsrCardAndRegisterInCrs (void)
  {
   /***** Get user's DNI from form and get user's data *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /***** Get common user's data from form *****/
      Usr_GetUsrCommonDataFromForm (&Gbl.Usrs.Other.UsrDat,false,false,false);

      /***** Update user's data in database *****/
      Usr_UpdateCommonCard (&Gbl.Usrs.Other.UsrDat,Usr_REPLACE_DATA);

      /***** Show user's record *****/
      Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);

      /***** Register user in current course in database *****/
      Usr_RegisterUsrInCurrentCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Cte_VERBOSE);
     }
  }

/****************************************************************************/
/************** Register an existing user in current course *****************/
/****************************************************************************/

void Usr_GetDNIShowUsrCardAndRegisterInCrs (void)
  {
   /***** Get user's DNI from form and get user's data *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /***** Show user's record *****/
      Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);

      /***** Register user in current course in database *****/
      Usr_RegisterUsrInCurrentCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Cte_VERBOSE);
    }
  }

/****************************************************************************/
/********************** Update my nickname in database **********************/
/****************************************************************************/

void Usr_UpdateMyNick (void)
  {
   char Query[512];

   /***** Update my nickname *****/
   sprintf (Query,"UPDATE usr_data SET Nickname='%s' WHERE UsrCod='%ld'",
            Gbl.Usrs.NewNickname,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update your nickname");
  }

/****************************************************************************/
/**************************** Update user's data ****************************/
/****************************************************************************/

void Usr_UpdateCommonCard (struct UsrData *UsrDat,Usr_ReplaceInsert_t ReplaceOrInsert)
  {
   extern const char *Txt_StrLangId[Txt_NUM_LANGUAGES];
   extern const char *The_ThemeId[The_NUM_THEMES];
   char Query[2048];

   Str_InsertHTTP (UsrDat->WWW);

   if (UsrDat->Std.Birthday.Year < Gbl.Today.Year-99 || UsrDat->Std.Birthday.Year > Gbl.Today.Year-16)
      UsrDat->Std.Birthday.Year = UsrDat->Std.Birthday.Month = UsrDat->Std.Birthday.Day = 0;

   switch (ReplaceOrInsert)
     {
      case Usr_REPLACE_DATA:
         /***** Update user's common data *****/
         sprintf (Query,"UPDATE usr_data" \
                        " SET Password='%s',UsrType='%u'," \
                        "Surname1='%s',Surname2='%s',FirstName='%s',Gender='%s',Theme='%s',Language='%s',E_mail='%s',WWW='%s',Skype='%s',PublicPhoto='%c'," \
                        "InsCod='%ld',DptCod='%ld',CtrCod='%ld',Office='%s',OfficePhone='%s'," \
                        "LocalAddress='%s',LocalPhone='%s',FamilyAddress='%s',FamilyPhone='%s',OriginPlace='%s',Birthday='%04u-%02u-%02u',Comments='%s'" \
                        " WHERE UsrCod='%ld'",
                  UsrDat->Password,
                  (unsigned) UsrDat->UsrTypeInDB,
                  UsrDat->Surname1,UsrDat->Surname2,UsrDat->FirstName,
                  StringsGenderDB[UsrDat->Gender],
                  The_ThemeId[UsrDat->Prefs.Theme],
                  Txt_StrLangId[UsrDat->Prefs.Language],
                  UsrDat->E_mail,UsrDat->WWW,UsrDat->Skype,
                  UsrDat->PublicPhoto ? 'Y' : 'N',
                  UsrDat->InsCod,
                  UsrDat->Tch.DptCod,
                  UsrDat->Tch.CtrCod,
                  UsrDat->Tch.Office,
                  UsrDat->Tch.OfficePhone,
                  UsrDat->Std.LocalAddress,UsrDat->Std.LocalPhone,
                  UsrDat->Std.FamilyAddress,UsrDat->Std.FamilyPhone,UsrDat->Std.OriginPlace,
                  UsrDat->Std.Birthday.Year,UsrDat->Std.Birthday.Month,UsrDat->Std.Birthday.Day,
      		  UsrDat->Std.Comments ? UsrDat->Std.Comments : "",
                  UsrDat->UsrCod);
         DB_QueryUPDATE (Query,"can not update user's data");
	 break;
      case Usr_INSERT_NEW_DATA:
         /***** Create encrypted user's code *****/
         Usr_CreateNewEncryptedUsrCod (UsrDat);

         /***** Insert new user in database *****/
         sprintf (Query,"INSERT INTO usr_data (EncryptedUsrCod,DNI,Password,UsrType,Surname1,Surname2,FirstName,Gender,Theme,Language,E_mail,WWW,Skype,PublicPhoto," \
                        "InsCod,DptCod,CtrCod,Office,OfficePhone," \
                        "LocalAddress,LocalPhone,FamilyAddress,FamilyPhone,OriginPlace,Birthday,Comments," \
                        "NumNewMsgs,SideCols,SecureIcons,NotifyEvents,LastTab)" \
                        " VALUES('%s','%s','%s','%u','%s','%s','%s','%s','%s','%s','%s','%s','%s','%c'," \
                        "'%ld','%ld','%ld','%s','%s'," \
                        "'%s','%s','%s','%s','%s','%04u-%02u-%02u','%s'," \
                        "'0','3','Y','0','%u')",
                  UsrDat->EncryptedUsrCod,
                  UsrDat->DNI,UsrDat->Password,
                  (unsigned) UsrDat->UsrTypeInDB,
                  UsrDat->Surname1,UsrDat->Surname2,UsrDat->FirstName,
                  StringsGenderDB[UsrDat->Gender],
                  The_ThemeId[UsrDat->Prefs.Theme],
                  Txt_StrLangId[UsrDat->Prefs.Language],
                  UsrDat->E_mail,UsrDat->WWW,UsrDat->Skype,
                  UsrDat->PublicPhoto ? 'Y' : 'N',
                  UsrDat->InsCod,
                  UsrDat->Tch.DptCod,
                  UsrDat->Tch.CtrCod,
                  UsrDat->Tch.Office,
                  UsrDat->Tch.OfficePhone,
                  UsrDat->Std.LocalAddress,UsrDat->Std.LocalPhone,
                  UsrDat->Std.FamilyAddress,UsrDat->Std.FamilyPhone,
                  UsrDat->Std.OriginPlace,
                  UsrDat->Std.Birthday.Year,UsrDat->Std.Birthday.Month,UsrDat->Std.Birthday.Day,
                  UsrDat->Std.Comments ? UsrDat->Std.Comments : "",
                  (unsigned) TabPer);
         DB_QueryINSERT (Query,"can not create user");

         /***** Get new user code (generated automatically by database (autoincrement field) *****/
         Usr_GetUsrCodFromDNI (UsrDat);

	 break;
     }
  }

/****************************************************************************/
/******************* Create a new encrypted user's code *********************/
/****************************************************************************/

#define LENGTH_RANDOM_STR 32
#define MAX_TRY 10

static void Usr_CreateNewEncryptedUsrCod (struct UsrData *UsrDat)
  {
   char RandomStr[LENGTH_RANDOM_STR+1];
   unsigned NumTry;

   for (NumTry=0; NumTry<MAX_TRY; NumTry++)
     {
      Str_CreateRandomAlphanumStr (RandomStr,LENGTH_RANDOM_STR);
      Cry_EncryptSHA256Base64 (RandomStr,UsrDat->EncryptedUsrCod);
      if (!Usr_ChkIfEncryptedUsrCodExists (UsrDat->EncryptedUsrCod))
          break;
     }
   if (NumTry == MAX_TRY)
      Lay_ShowErrorAndExit ("Can not create a new encrypted user's code.");
   }

/****************************************************************************/
/***** Create user if not exists and register him/her in current course *****/
/****************************************************************************/
// UsrDat must contain user's data

void Usr_CreatUsrIfNotExistsAndRegisterHimInCurrentCrs (struct UsrData *UsrDat,Usr_ReplaceInsert_t ReplaceOrInsert,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char PathRelUsr[PATH_MAX+1];

   /***** Check if current user can make this action *****/
   if (Gbl.Usrs.Me.UsrTypeLog < Usr_TEACHER)	// This check is unnecessary
      Lay_ShowErrorAndExit ("You can't create or register users.");
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER && UsrDat->UsrTypeInDB != Usr_STUDENT)
      Lay_ShowErrorAndExit ("A teacher only can create new users if they are students.");

   /***** Create directory for the user, if not exists *****/
   Usr_ConstructPathUsr (UsrDat->DNI,PathRelUsr,NULL);
   Fil_CreateDirIfNotExists (PathRelUsr);

   /***** Create user if not exists *****/
   Usr_CreateUsrIfNotExists (UsrDat,ReplaceOrInsert,QuietOrVerbose);

   /***** Show user's record *****/
   if (QuietOrVerbose == Cte_VERBOSE)
      Rec_ShowCommonRecordUnmodifiable (UsrDat);

   /***** Register user in current course in database *****/
   Usr_RegisterUsrInCurrentCrs (UsrDat->UsrCod,QuietOrVerbose);
  }

/****************************************************************************/
/*************** Insert a user in the file of records ****************/
/****************************************************************************/

// If user desn't exist in database, create it!
// If user already exists in database:
//	- if ReplaceOrInsert == Usr_REPLACE_DATA ==> update his/her data
//	- if ReplaceOrInsert == Usr_INSERT_NEW_DATA ==> don't update his/her data

void Usr_CreateUsrIfNotExists (struct UsrData *UsrDat,Usr_ReplaceInsert_t ReplaceOrInsert,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   if (Usr_ChkIfDNIExists (UsrDat->DNI))
     {
      // User already exists in database
      if (QuietOrVerbose == Cte_VERBOSE)
        {
         sprintf (Gbl.Message,"Ya existe el usuario con DNI <strong>%s</strong>.",	// Need translation!
                  UsrDat->DNI);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
      if (ReplaceOrInsert == Usr_REPLACE_DATA)	// Update his/her data?
         Usr_UpdateCommonCard (UsrDat,Usr_REPLACE_DATA);
     }
   else	// User desn't exist in database, create it!
      Usr_UpdateCommonCard (UsrDat,Usr_INSERT_NEW_DATA);
  }

/****************************************************************************/
/*********************** Register user in current course ********************/
/****************************************************************************/

void Usr_RegisterUsrInCurrentCrs (long UsrCod,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char Query[1024];

   if (Usr_CheckIfUsrBelongsToCrs (UsrCod,Gbl.CurrentCrs.Crs.CrsCod))
     {
      if (QuietOrVerbose)
        {
         sprintf (Gbl.Message,"El usuario ya estaba dado de alta en <strong>%s</strong>.",	// Need translation!
                  Gbl.CurrentCrs.Crs.FullName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }
   else	// User didn't belong to current course, add it as pending
     {
      /***** Register user in current course in database *****/
      sprintf (Query,"INSERT INTO crs_usr (CrsCod,UsrCod,Accepted,LastAccDownloadCrs,LastAccCommonCrs,LastAccMyWorks,LastAccCrsWorks,LastAccMarksCrs,LastDowGrpCod,LastComGrpCod,LastAssGrpCod,NumAccTst,NextAccTst,UsrListType,ColsClassPhoto,ListWithPhotos)" \
                     " VALUES('%ld','%ld','N','0000-00-00','0000-00-00','0000-00-00','0000-00-00','0000-00-00','-1','-1','-1','0','0000-00-00','%s','%u','%c')",
               Gbl.CurrentCrs.Crs.CrsCod,UsrCod,StringsUsrListTypeInDB[Usr_SHOW_USRS_TYPE_DEFAULT],Usr_CLASS_PHOTO_COLS_DEF,Usr_LIST_WITH_PHOTOS_DEF ? 'Y' : 'N');
      DB_QueryINSERT (Query,"can not register user in course");

      if (QuietOrVerbose == Cte_VERBOSE)
        {
         sprintf (Gbl.Message,"El usuario ha sido a&ntilde;adido a <strong>%s</strong>.",	// Need translation!
                  Gbl.CurrentCrs.Crs.FullName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }
  }

/****************************************************************************/
/******************* Register administrator in current degree ***************/
/****************************************************************************/

void Usr_RegisterAdminInCurrentDeg (long UsrCod)
  {
   char Query[512];

   /***** Check if user was and administrator of current degree *****/
   if (Usr_CheckIfUsrIsAdmOfDeg (UsrCod,Gbl.CurrentDeg.Deg.DegCod))
      sprintf (Gbl.Message,"El usuario con ya era administrador de la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentDeg.Deg.FullName);
   else	// User wasn't administrator of current degree
     {
      /***** Insert or replace administrator in current degree *****/
      sprintf (Query,"REPLACE INTO deg_admin (UsrCod,DegCod)" \
                     " VALUES ('%ld','%ld')",
               UsrCod,Gbl.CurrentDeg.Deg.DegCod);
      DB_QueryREPLACE (Query,"can not create administrator of degree");

      sprintf (Gbl.Message,"El usuario ha sido dado de alta como administrador de <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentDeg.Deg.FullName);
     }
   Lay_ShowAlert (INFO,Gbl.Message);
  }

/****************************************************************************/
/************************ Remove a user from a course ***********************/
/****************************************************************************/

void Usr_EffectivelyRemUsrFromCrs (struct UsrData *UsrDat,struct Course *Crs,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char Query[1024];

   if (Usr_CheckIfUsrBelongsToCrs (UsrDat->UsrCod,Crs->CrsCod))	// User belongs to the course
     {
      /***** Remove user from the table of courses-users *****/
      sprintf (Query,"DELETE FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
               Crs->CrsCod,UsrDat->UsrCod);
      DB_QueryDELETE (Query,"can not remove a user from a course");

      if (QuietOrVerbose == Cte_VERBOSE)
        {
         sprintf (Gbl.Message,"El usuario ha sido dado de baja de <strong>%s</strong>.",	// Need translation!
                  Crs->FullName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }
   else	// User doesn't belong to current course
     {
      if (QuietOrVerbose == Cte_VERBOSE)
        {
         sprintf (Gbl.Message,"El usuario no estaba dado de alta en <strong>%s</strong>.",	// Need translation!
                  Crs->FullName);
         Lay_ShowAlert (INFO,Gbl.Message);
        }
     }

   /***** Remove user from all the groups in current course *****/
   Grp_RemUsrFromAllGrpsInCrs (UsrDat,Crs,QuietOrVerbose);

   /***** Remove works zone of this user in current course *****/
   Brw_RemoveUsrWorksInCrs (UsrDat,Crs,QuietOrVerbose);

   /***** Remnove fields of this user in its current course record *****/
   Rec_RemoveFieldsCrsRecordInCrs (UsrDat->UsrCod,Crs,QuietOrVerbose);

   /***** Remove user's clipboard in current course *****/
   Brw_RemoveUsrClipboardInCrs (UsrDat->UsrCod,Crs->CrsCod);

   /***** Remove user's expanded trees in current course *****/
   Brw_RemoveUsrExpandedFoldersInCrs (UsrDat->UsrCod,Crs->CrsCod);
  }

/****************************************************************************/
/************* Remove an administrator from current degree ******************/
/****************************************************************************/

void Usr_EffectivelyRemAdmFromDeg (long UsrCod)
  {
   char Query[1024];

   if (Usr_CheckIfUsrIsAdmOfDeg (UsrCod,Gbl.CurrentDeg.Deg.DegCod))	// User is administrator of current degree
     {
      /***** Remove user from the table of courses-users *****/
      sprintf (Query,"DELETE FROM deg_admin WHERE UsrCod='%ld' AND DegCod='%ld'",
               UsrCod,Gbl.CurrentDeg.Deg.DegCod);
      DB_QueryDELETE (Query,"can not remove an administrator from a degree");

      sprintf (Gbl.Message,"El usuario ha sido dado de baja como administrador de <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentDeg.Deg.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else	// User doesn't belong to current course
     {
      sprintf (Gbl.Message,"El usuario no era administrador de <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (ERROR,Gbl.Message);
     }
  }

/****************************************************************************/
/********************* Remove a user of all the sitios ********************/
/****************************************************************************/

void Usr_DeleteUsrFromPlatform (struct UsrData *UsrDat,Cte_QuietOrVerbose_t QuietOrVerbose)
  {
   char Query[1024];
   bool PhotoRemoved = false;

   /***** Remove the zonas of trabajos of a user in todas the courses *****/
   Brw_RemoveUsrWorksInAllCrss (UsrDat,QuietOrVerbose);	// Hay que hacerlo antes of remove the user of the courses

   /***** Remove the fields of record of the user especfica of the course in todas the courses *****/
   Rec_RemoveFieldsCrsRecordAll (UsrDat->UsrCod,QuietOrVerbose);

   /***** Quitar the student of all the groups of todas the courses *****/
   Grp_RemUsrFromAllGrps (UsrDat,QuietOrVerbose);

   /***** Borrar user of the table of courses and users *****/
   sprintf (Query,"DELETE FROM crs_usr WHERE UsrCod='%ld'",UsrDat->UsrCod);
   DB_QueryDELETE (Query,"can not remove a user from all courses");

   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ha sido dado de baja de todas sus asignaturas.",	// Need translation!
               UsrDat->DNI);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Remove user as administrator of any degree *****/
   sprintf (Query,"DELETE FROM deg_admin WHERE UsrCod='%ld'",UsrDat->UsrCod);
   DB_QueryDELETE (Query,"can not remove a user as administrator");

   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ha sido dado de baja como administrador.",	// Need translation!
               UsrDat->DNI);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Remove user's clipboards *****/
   Brw_RemoveUsrClipboard (UsrDat->UsrCod);
   For_RemoveUsrFromThrClipboard (UsrDat->UsrCod);

   /***** Remove user's expanded trees *****/
   Brw_RemoveUsrExpandedFolders (UsrDat->UsrCod);

   /***** Delete user's messages sent and received *****/
   Msg_DelAllRecOrSntMsgsUsr (Msg_MESSAGES_RECEIVED,UsrDat->UsrCod,-1L,"","");
   Msg_DelAllRecOrSntMsgsUsr (Msg_MESSAGES_SENT,UsrDat->UsrCod,-1L,"","");
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"Mensajes del usuario con DNI <strong>%s</strong> eliminados.",	// Need translation!
               UsrDat->DNI);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Delete thread read status for this user *****/
   For_DeleteUsrFromReadThrs (UsrDat->UsrCod);

   /***** Remove the file tree of a user *****/
   Usr_RemoveUsrBriefcase (UsrDat);
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"Malet&iacute;n del usuario con DNI <strong>%s</strong> eliminado.",	// Need translation!
               UsrDat->DNI);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Remove user's record *****/
   Usr_RemoveUsrCard (UsrDat);
   if (QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"Ficha del usuario con DNI <strong>%s</strong> eliminada.",	// Need translation!
               UsrDat->DNI);
      Lay_ShowAlert (INFO,Gbl.Message);
     }

   /***** Remove the user of the list of users sin foto *****/
   Usr_RemoveUsrFromTableClicksWithoutPhoto (UsrDat->UsrCod);

   /***** Remove user's photo *****/
   PhotoRemoved = Usr_RemovePhoto (UsrDat);
   if (PhotoRemoved && QuietOrVerbose == Cte_VERBOSE)
     {
      sprintf (Gbl.Message,"Fotograf&iacute;a del usuario con DNI <strong>%s</strong> eliminada.",	// Need translation!
               UsrDat->DNI);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
  }

/****************************************************************************/
/********************* Remove the briefcase of a user ***********************/
/****************************************************************************/

void Usr_RemoveUsrBriefcase (struct UsrData *UsrDat)
  {
   char PathRelUsr[PATH_MAX+1];

   /***** Remove the briefcase of the user *****/
   Usr_ConstructPathUsr (UsrDat->DNI,PathRelUsr,NULL);
   Brw_RemoveTree (PathRelUsr);

   /***** Remove size of the briefcase of the user from database *****/
   Brw_RemoveSizeOfFileTreeFromDB (-1L,-1L,UsrDat->UsrCod);
  }

/****************************************************************************/
/********************* Remove the rbol of a user **********************/
/****************************************************************************/

void Usr_RemoveUsrCard (struct UsrData *UsrDat)
  {
   char Query[512];

   /***** Remove record global common *****/
   sprintf (Query,"DELETE FROM usr_data WHERE UsrCod='%ld'",UsrDat->UsrCod);
   DB_QueryDELETE (Query,"can not remove user's data");
  }

/*****************************************************************************/
/************ Write a row of a table with the data of a student **************/
/*****************************************************************************/

void Usr_WriteRowStdMainData (unsigned NumUsr,struct UsrData *UsrDat,bool PutCheckboxToSelectUsr)
  {
   const char *BgColor;
   char PhotoURL[PATH_MAX+1];
   bool ShowPhoto;
   bool UsrIsTheMsgSender = false;
   char MailLink[7+Cte_MAX_BYTES_STRING+1];	// mailto:dir_correo
   char WWWLink[Cte_MAX_BYTES_URL+1];
   char SkypeLink[6+Cte_MAX_BYTES_STRING+5+1];	// skype:user?call
   struct Institution Ins;
   bool ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat->UsrCod);
   bool ICanSeeDNI = (Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER);
   bool ShowData = ItsMe ||
                   (Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER && UsrDat->Accepted) ||
                   Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN ||
                   Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER;
   bool PutLinkToCard = Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER && !PutCheckboxToSelectUsr;

   /***** Initialize variables *****/
   if (PutLinkToCard)
      sprintf (Gbl.MsgWindowStatus,"Ver ficha de %s",	// Need translation!
               UsrDat->FullName);

   /***** Start row *****/
   fprintf (Gbl.F.Out,"<tr>");

   /***** Checkbox to select user *****/
   if (PutCheckboxToSelectUsr)
      if (Gbl.Usrs.Select.All[0])	// I have selected one or more users
	 if (!strcmp (Gbl.Usrs.Select.All,UsrDat->EncryptedUsrCod))	// I try to send a message to one unique user (Gbl.Usrs.Select.All holds a unique user)
            UsrIsTheMsgSender = true;
   BgColor = UsrIsTheMsgSender ? LIGHT_GREEN : Gbl.ColorRows[Gbl.GlobalRowEvenOdd];   // Two colors are used alternatively to better distinguish the rows
   if (PutCheckboxToSelectUsr)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" bgcolor=\"#%s\">",BgColor);
      Usr_PutCheckboxToSelectUser (Usr_STUDENT,UsrDat->EncryptedUsrCod,UsrIsTheMsgSender);
      fprintf (Gbl.F.Out,"</td>");
     }

   /***** Student has accepted registration in current course? *****/
   fprintf (Gbl.F.Out,"<td class=\"");
   if (UsrIsTheMsgSender)
      fprintf (Gbl.F.Out,"BM_SEL");
   else
      fprintf (Gbl.F.Out,"BM%d",Gbl.GlobalRowEvenOdd);
   fprintf (Gbl.F.Out,"\"><img src=\"%s/%s12x12.gif\" alt=\"\" title=\"Alta %s\" width=\"12\" height=\"12\" /></td>",
            Gbl.Prefs.IconsURL,
            UsrDat->Accepted ? "new" : "tr",
            UsrDat->Accepted ? "confirmada" : "pendiente de confirmaci&oacute;n");	// Need translation!

   /***** Write number of student in the list *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" bgcolor=\"#%s\" class=\"%s\">&nbsp;%u&nbsp;</td>",
            BgColor,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL",NumUsr);

   if (Gbl.Usrs.Listing.WithPhotos)
     {
      /***** Show student's photo *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\">",BgColor);
      if (PutLinkToCard)
        {
         Usr_StartFormLinkToStdCard (UsrDat->EncryptedUsrCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
        }
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
      Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,18,24,Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW);
      if (PutLinkToCard)
         fprintf (Gbl.F.Out,"</acronym></a></form>");
      fprintf (Gbl.F.Out,"</td>");
     }

   /***** Prepare data for brief presentation *****/
   Usr_RestrictLengthMainData (ShowData,UsrDat,MailLink,WWWLink,SkypeLink);

   /****** Write DNI ******/
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">",
            BgColor,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL");
   if (PutLinkToCard)
     {
      Usr_StartFormLinkToStdCard (UsrDat->EncryptedUsrCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL");
     }
   fprintf (Gbl.F.Out,"%s&nbsp;",ICanSeeDNI ? UsrDat->DNI : "********");
   if (PutLinkToCard)
      fprintf (Gbl.F.Out,"</acronym></a></form>");
   fprintf (Gbl.F.Out,"</td>");

   /***** Write rest of main student's data *****/
   Ins.InsCod = UsrDat->InsCod;
   Ins_GetDataOfInstitutionByCod (&Ins);
   Usr_WriteMainDataUsrExceptDNI (UsrDat,BgColor,ShowData,
                                  UsrDat->E_mail[0] ? MailLink  : NULL,
                                  UsrDat->WWW[0]    ? WWWLink   : NULL,
                                  UsrDat->Skype[0]  ? SkypeLink : NULL,
                                  Ins.ShortName,
                                  Ins.WWW[0]        ? Ins.WWW   : NULL);

   /***** End row *****/
   fprintf (Gbl.F.Out,"</tr>\n");

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
  }

/*****************************************************************************/
/************ Write a row of a table with the data of a student **************/
/*****************************************************************************/

void Usr_WriteRowStdAllData (unsigned NumUsr,struct UsrData *UsrDat,char *GroupNames)
  {
   const char *BgColor;
   char PhotoURL[PATH_MAX+1];
   bool ShowPhoto;
   unsigned NumGrpTyp,NumField;
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;
   char Text[Cte_MAX_BYTES_TEXT+1];
   struct Institution Ins;
   bool ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat->UsrCod);
   bool ICanSeeDNI = (Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER);
   bool ShowData = ItsMe ||
                   (Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER && UsrDat->Accepted) ||
                   Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN ||
                   Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER;

   /***** Start row *****/
   BgColor = Gbl.ColorRows[Gbl.GlobalRowEvenOdd];   // Two colors are used alternatively to better distinguish the rows
   fprintf (Gbl.F.Out,"<tr>");

   if (Gbl.Usrs.Listing.WithPhotos)
     {
      /***** Show student's photo *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\">",BgColor);
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
      Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,18,24,Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW);
      fprintf (Gbl.F.Out,"</td>");
     }

   /****** Write DNI ******/
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">%s&nbsp;</td>",
            BgColor,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL",ICanSeeDNI ? UsrDat->DNI : "********");

   /***** Write rest of main student's data *****/
   Ins.InsCod = UsrDat->InsCod;
   Ins_GetDataOfInstitutionByCod (&Ins);
   Usr_WriteMainDataUsrExceptDNI (UsrDat,BgColor,ShowData,NULL,NULL,NULL,Ins.ShortName,NULL);

   /***** Write the rest of the data of the student *****/
   Usr_WriteUsrData (BgColor,UsrDat->Std.LocalAddress[0]  ? (ShowData ? UsrDat->Std.LocalAddress  : "********") : "&nbsp;",NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->Std.LocalPhone[0]    ? (ShowData ? UsrDat->Std.LocalPhone    : "********") : "&nbsp;",NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->Std.FamilyAddress[0] ? (ShowData ? UsrDat->Std.FamilyAddress : "********") : "&nbsp;",NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->Std.FamilyPhone[0]   ? (ShowData ? UsrDat->Std.FamilyPhone   : "********") : "&nbsp;",NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->Std.OriginPlace[0]   ? (ShowData ? UsrDat->Std.OriginPlace   : "********") : "&nbsp;",NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->Std.StrBirthday[0]   ? (ShowData ? UsrDat->Std.StrBirthday   : "********") : "&nbsp;",NULL,true,UsrDat->Accepted);

   if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
     {
      /***** Write the groups a the que pertenece the student *****/
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
         if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps)	 // If current course tiene groups of este type
           {
            Grp_GetNamesGrpsStdBelongsTo (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod,UsrDat->UsrCod,GroupNames);
            Usr_WriteUsrData (BgColor,GroupNames,NULL,true,UsrDat->Accepted);
           }

      /***** Fields of the record dependientes of the course *****/
      for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
        {
         /* Get the text of the field */
         NumRows = Rec_GetFieldFromCrsRecord (UsrDat->UsrCod,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].FieldCod,&mysql_res);
         if (NumRows)
           {
            row = mysql_fetch_row (mysql_res);
            strcpy (Text,row[0]);
            Str_ChangeFormat (Str_FROM_HTML,Str_TO_RIGOROUS_HTML,Text,Cte_MAX_BYTES_TEXT,false);	// Se convierte of HTML a HTML respetuoso
           }
         else
            Text[0] = '\0';
         Usr_WriteUsrData (BgColor,Text,NULL,false,UsrDat->Accepted);

         /* Free structure that stores the query result */
         DB_FreeMySQLResult (&mysql_res);
        }
     }

   /***** End row *****/
   fprintf (Gbl.F.Out,"</tr>\n");

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
  }

/*****************************************************************************/
/*** Write a row of a table with the data of a teacher or an administrator ***/
/*****************************************************************************/

void Usr_WriteRowTchMainData (unsigned NumUsr,struct UsrData *UsrDat,bool PutCheckboxToSelectUsr)
  {
   const char *BgColor;
   char PhotoURL[PATH_MAX+1];
   bool ShowPhoto;
   bool UsrIsTheMsgSender = false;
   char MailLink[7+Cte_MAX_BYTES_STRING+1];	// mailto:dir_correo
   char WWWLink[Cte_MAX_BYTES_URL+1];
   char SkypeLink[6+Cte_MAX_BYTES_STRING+5+1];	// skype:user?call
   struct Institution Ins;
   bool ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat->UsrCod);
   bool ShowData = ItsMe || UsrDat->Accepted || Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN || Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER;
   bool PutLinkToCard;
   bool ICanSeeDNI;

   /***** Check if I can see teacher's DNI *****/
   ICanSeeDNI = Usr_ICanSeeDNIOfATeacher (UsrDat);

   /***** Initialize variables *****/
   PutLinkToCard = PutCheckboxToSelectUsr ? false : ((Gbl.CurrentCrs.Crs.CrsCod > 0) ? Usr_CheckIfUsrBelongsToCrs (UsrDat->UsrCod,Gbl.CurrentCrs.Crs.CrsCod) : false);
   if (PutLinkToCard)
      sprintf (Gbl.MsgWindowStatus,"Ver ficha de %s",UsrDat->FullName);	// Need translation!

   /***** Start row *****/
   fprintf (Gbl.F.Out,"<tr>");

   /***** Checkbox to select user *****/
   if (PutCheckboxToSelectUsr)
     if (Gbl.Usrs.Select.All[0])	// I have selected one or more users
	 if (!strcmp (Gbl.Usrs.Select.All,UsrDat->EncryptedUsrCod))	// I try to send a message to one unique user (Gbl.Usrs.Select.All holds a unique user)
            UsrIsTheMsgSender = true;
   BgColor = UsrIsTheMsgSender ? LIGHT_GREEN : Gbl.ColorRows[Gbl.GlobalRowEvenOdd];   // Two colors are used alternatively to better distinguish the rows
   if (PutCheckboxToSelectUsr)
     {
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"middle\" bgcolor=\"#%s\">",BgColor);
      Usr_PutCheckboxToSelectUser (Usr_TEACHER,UsrDat->EncryptedUsrCod,UsrIsTheMsgSender);
      fprintf (Gbl.F.Out,"</td>");
     }

   /***** Teacher has accepted registration in current course/in any course in degree/in any course? *****/
   fprintf (Gbl.F.Out,"<td class=\"");
   if (UsrIsTheMsgSender)
      fprintf (Gbl.F.Out,"BM_SEL");
   else
      fprintf (Gbl.F.Out,"BM%d",Gbl.GlobalRowEvenOdd);
   fprintf (Gbl.F.Out,"\"><img src=\"%s/%s12x12.gif\" alt=\"\" title=\"Alta(s) %s\" width=\"12\" height=\"12\" /></td>",
            Gbl.Prefs.IconsURL,
            UsrDat->Accepted ? "new" : "tr",
            UsrDat->Accepted ? "confirmada(s)" : "pendiente(s) de confirmaci&oacute;n");	// Need translation!

   /***** Write number of user *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" bgcolor=\"#%s\" class=\"DAT_SMALL_N\">&nbsp;%u&nbsp;</td>",
            BgColor,NumUsr);

   if (Gbl.Usrs.Listing.WithPhotos)
     {
      /***** Show teacher's photo *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\">",BgColor);
      if (PutLinkToCard)
        {
         Usr_StartFormLinkToTchCard (UsrDat->EncryptedUsrCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
        }
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
      Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,18,24,Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW);
      if (PutLinkToCard)
         fprintf (Gbl.F.Out,"</acronym></a></form>");
      fprintf (Gbl.F.Out,"</td>");
     }

   /***** Prepare data for brief presentation *****/
   Usr_RestrictLengthMainData (ShowData,UsrDat,MailLink,WWWLink,SkypeLink);

   /****** Write the DNI ******/
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">",BgColor,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL");
   if (PutLinkToCard)
     {
      Usr_StartFormLinkToTchCard (UsrDat->EncryptedUsrCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL");
     }
   fprintf (Gbl.F.Out,"%s&nbsp;",ICanSeeDNI ? UsrDat->DNI : "********");
   if (PutLinkToCard)
      fprintf (Gbl.F.Out,"</acronym></a></form>");
   fprintf (Gbl.F.Out,"</td>");

   /***** Write rest of main teacher's data *****/
   Ins.InsCod = UsrDat->InsCod;
   Ins_GetDataOfInstitutionByCod (&Ins);
   Usr_WriteMainDataUsrExceptDNI (UsrDat,BgColor,ShowData,
                                  UsrDat->E_mail[0] ? MailLink  : NULL,
                                  UsrDat->WWW[0]    ? WWWLink   : NULL,
                                  UsrDat->Skype[0]  ? SkypeLink : NULL,
                                  Ins.ShortName,
                                  Ins.WWW[0]        ? Ins.WWW   : NULL);
   fprintf (Gbl.F.Out,"</tr>");

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
  }

/*****************************************************************************/
/*** Write a row of a table with the data of a teacher or an administrator ***/
/*****************************************************************************/

void Usr_WriteRowTchAllData (unsigned NumUsr,struct UsrData *UsrDat)
  {
   const char *BgColor;
   char PhotoURL[PATH_MAX+1];
   bool ShowPhoto;
   struct Institution Ins;
   bool ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat->UsrCod);
   bool ShowData = ItsMe || UsrDat->Accepted || Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN || Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER;
   bool ICanSeeDNI;
   struct Centre Ctr;
   struct Department Dpt;

   /***** Check if I can see teacher's DNI *****/
   ICanSeeDNI = Usr_ICanSeeDNIOfATeacher (UsrDat);

   /***** Start row *****/
   BgColor = Gbl.ColorRows[Gbl.GlobalRowEvenOdd];   // Two colors are used alternatively to better distinguish the rows
   fprintf (Gbl.F.Out,"<tr>");
   if (Gbl.Usrs.Listing.WithPhotos)
     {
      /***** Show teacher's photo *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\">",BgColor);
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
      Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,18,24,Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW);
      fprintf (Gbl.F.Out,"</td>");
     }

   /****** Write the DNI ******/
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">%s&nbsp;</td>",
            BgColor,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL",ICanSeeDNI ? UsrDat->DNI : "********");

   /***** Write rest of main teacher's data *****/
   Ins.InsCod = UsrDat->InsCod;
   Ins_GetDataOfInstitutionByCod (&Ins);
   Usr_WriteMainDataUsrExceptDNI (UsrDat,BgColor,ShowData,NULL,NULL,NULL,Ins.ShortName,NULL);

   /***** Write the rest of teacher's data *****/
   if (ShowData && UsrDat->Tch.CtrCod > 0)
     {
      Ctr.CtrCod = UsrDat->Tch.CtrCod;
      Ctr_GetDataOfCentreByCod (&Ctr);
     }
   Usr_WriteUsrData (BgColor,(ShowData && UsrDat->Tch.CtrCod > 0   ) ? Ctr.FullName            : "&nbsp;",NULL,true,UsrDat->Accepted);

   if (ShowData && UsrDat->Tch.DptCod > 0)
     {
      Dpt.DptCod = UsrDat->Tch.DptCod;
      Dpt_GetDataOfDepartmentByCod (&Dpt);
     }
   Usr_WriteUsrData (BgColor,(ShowData && UsrDat->Tch.DptCod > 0   ) ? Dpt.FullName            : "&nbsp;",NULL,true,UsrDat->Accepted);

   Usr_WriteUsrData (BgColor,(ShowData && UsrDat->Tch.Office[0]     ) ? UsrDat->Tch.Office      : "&nbsp;",NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,(ShowData && UsrDat->Tch.OfficePhone[0]) ? UsrDat->Tch.OfficePhone : "&nbsp;",NULL,true,UsrDat->Accepted);

   fprintf (Gbl.F.Out,"</tr>");

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
  }

/*****************************************************************************/
/******************* Check if I can see the DNI of a teacher *****************/
/*****************************************************************************/

bool Usr_ICanSeeDNIOfATeacher (struct UsrData *UsrDat)
  {
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN)
     {
      if (Usr_CheckIfUsrIsAdmOfAllDegs (Gbl.Usrs.Me.UsrDat.UsrCod))	// If I am admin of all degrees
         return true;
      // If I am an administrator of current degree, but not of all degrees, I only can see the DNI of users from current degree
      return Usr_CheckIfUsrBelongsToDeg (UsrDat->UsrCod,Gbl.CurrentDeg.Deg.DegCod);
     }
   return (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER);
  }

/*****************************************************************************/
/*** Write a row of a table with the data of a teacher or an administrator ***/
/*****************************************************************************/

void Usr_WriteRowAdmData (unsigned NumUsr,struct UsrData *UsrDat)
  {
   const char *BgColor;
   char PhotoURL[PATH_MAX+1];
   bool ShowPhoto;
   char MailLink[7+Cte_MAX_BYTES_STRING+1];	// mailto:dir_correo
   char WWWLink[Cte_MAX_BYTES_URL+1];
   char SkypeLink[6+Cte_MAX_BYTES_STRING+5+1];	// skype:user?call
   struct Institution Ins;
   bool PutLinkToCard;
   bool ICanSeeDNI = (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER);

   /***** Initialize variables *****/
   PutLinkToCard = (Gbl.CurrentCrs.Crs.CrsCod > 0 ? Usr_CheckIfUsrBelongsToCrs (UsrDat->UsrCod,Gbl.CurrentCrs.Crs.CrsCod) : false);
   if (PutLinkToCard)
      sprintf (Gbl.MsgWindowStatus,"Ver ficha de %s",UsrDat->FullName);	// Need translation!

   /***** Start row *****/
   BgColor = Gbl.ColorRows[Gbl.GlobalRowEvenOdd];   // Two colors are used alternatively to better distinguish the rows
   fprintf (Gbl.F.Out,"<tr>");

   /***** Write number of user *****/
   fprintf (Gbl.F.Out,"<td align=\"right\" bgcolor=\"#%s\" class=\"DAT_SMALL_N\">&nbsp;%u&nbsp;</td>",BgColor,NumUsr);

   if (Gbl.Usrs.Listing.WithPhotos)
     {
      /***** Show administrator's photo *****/
      fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\">",BgColor);
      if (PutLinkToCard)
        {
         Usr_StartFormLinkToTchCard (UsrDat->EncryptedUsrCod);
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,NULL);
        }
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
      Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,18,24,Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW);
      if (PutLinkToCard)
         fprintf (Gbl.F.Out,"</acronym></a></form>");
      fprintf (Gbl.F.Out,"</td>");
     }

   /***** Prepare data for brief presentation *****/
   Usr_RestrictLengthMainData (true,UsrDat,MailLink,WWWLink,SkypeLink);

   /****** Write the DNI ******/
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">",BgColor,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL");
   if (PutLinkToCard)
     {
      Usr_StartFormLinkToTchCard (UsrDat->EncryptedUsrCod);
      Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,UsrDat->Accepted ? "DAT_SMALL_N" : "DAT_SMALL");
     }
   fprintf (Gbl.F.Out,"%s&nbsp;",ICanSeeDNI ? UsrDat->DNI : "********");
   if (PutLinkToCard)
      fprintf (Gbl.F.Out,"</acronym></a></form>");
   fprintf (Gbl.F.Out,"</td>");

   /***** Write rest of main administrator's data *****/
   Ins.InsCod = UsrDat->InsCod;
   Ins_GetDataOfInstitutionByCod (&Ins);
   Usr_WriteMainDataUsrExceptDNI (UsrDat,BgColor,true,
                                  UsrDat->E_mail[0] ? MailLink  : NULL,
                                  UsrDat->WWW[0]    ? WWWLink   : NULL,
                                  UsrDat->Skype[0]  ? SkypeLink : NULL,
                                  Ins.ShortName,
                                  Ins.WWW[0]        ? Ins.WWW   : NULL);
   fprintf (Gbl.F.Out,"</tr>");

   /***** Write degrees which are administrated by this administrator *****/
   Deg_GetAndWriteDegreesAdminBy (UsrDat->UsrCod,Gbl.Usrs.Listing.WithPhotos ? Usr_NUM_MAIN_FIELDS_DATA_ADM : Usr_NUM_MAIN_FIELDS_DATA_ADM-1);

   Gbl.GlobalRowEvenOdd = 1 - Gbl.GlobalRowEvenOdd;
  }

/*****************************************************************************/
/************** Restrict the length of the main data of a user ***************/
/*****************************************************************************/

void Usr_RestrictLengthMainData (bool ShowData,struct UsrData *UsrDat,char *MailLink,char *WWWLink,char *SkypeLink)
  {
   Str_LimitLengthHTMLStr (UsrDat->FirstName,15);
   Str_LimitLengthHTMLStr (UsrDat->Surname1,15);
   Str_LimitLengthHTMLStr (UsrDat->Surname2,15);
   if (ShowData && UsrDat->E_mail[0])
     {
      sprintf (MailLink,"mailto:%s",UsrDat->E_mail);
      Str_LimitLengthHTMLStr (UsrDat->E_mail,15);
     }
   strcpy (WWWLink,UsrDat->WWW);
   Str_LimitLengthHTMLStr (UsrDat->WWW,15);
   if (ShowData && UsrDat->Skype[0])
     {
      sprintf (SkypeLink,"skype:%s?call",UsrDat->Skype);
      Str_LimitLengthHTMLStr (UsrDat->Skype,12);
     }
  }

/*****************************************************************************/
/************************* Write main data of a user *************************/
/*****************************************************************************/

void Usr_WriteMainDataUsrExceptDNI (struct UsrData *UsrDat,const char *BgColor,bool ShowData,
                                    const char *MailLink,const char *WWWLink,const char *SkypeLink,
                                    const char *InstitutionName,const char *InstitutionLink)
  {
   Usr_WriteUsrData (BgColor,UsrDat->Surname1[0]  ?             UsrDat->Surname1              : "&nbsp;",                             NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->Surname2[0]  ?             UsrDat->Surname2              : "&nbsp;",                             NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->FirstName[0] ?             UsrDat->FirstName             : "&nbsp;",                             NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->E_mail[0]    ? (ShowData ? UsrDat->E_mail  : "********") : "&nbsp;",ShowData ? MailLink        : NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->WWW[0]       ?             UsrDat->WWW                   : "&nbsp;",           WWWLink               ,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,UsrDat->Skype[0]     ? (ShowData ? UsrDat->Skype   : "********") : "&nbsp;",ShowData ? SkypeLink       : NULL,true,UsrDat->Accepted);
   Usr_WriteUsrData (BgColor,(UsrDat->InsCod > 0) ? (ShowData ? InstitutionName : "********") : "&nbsp;",ShowData ? InstitutionLink : NULL,true,UsrDat->Accepted);
  }

/*****************************************************************************/
/********************* Write a cell with data of a user **********************/
/*****************************************************************************/

void Usr_WriteUsrData (const char *BgColor,const char *Dato,const char *Enlace,bool NonBreak,bool Accepted)
  {
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"%s\">",BgColor,
            Accepted ? (NonBreak ? "DAT_SMALL_NOBR_N" : "DAT_SMALL_N") : 
                       (NonBreak ? "DAT_SMALL_NOBR"   : "DAT_SMALL"  ));
   if (Enlace != NULL)
      fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\">",Enlace);
   fprintf (Gbl.F.Out,"%s",Dato);
   if (NonBreak)
      fprintf (Gbl.F.Out,"&nbsp;");
   if (Enlace != NULL)
      fprintf (Gbl.F.Out,"</a>");
   fprintf (Gbl.F.Out,"</td>");
  }

/*****************************************************************************/
/********************** Get number of students in a course *******************/
/*****************************************************************************/

unsigned Usr_GetNumStdsInCrs (long CrsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumStds;

   /***** Get the number of students in a course from database ******/
   sprintf (Query,"SELECT COUNT(*) FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
            CrsCod,(unsigned) Usr_STUDENT);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of students in a course");

   /***** Get number of students (row[0]) *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumStds) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of students in a course.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumStds;
  }

/*****************************************************************************/
/********************** Get number of teachers in a course *******************/
/*****************************************************************************/

unsigned Usr_GetNumTchsInCrs (long CrsCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumTchs;

   /***** Get the number of teachers in a course from database ******/
   sprintf (Query,"SELECT COUNT(*) FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType>='%u'",
            CrsCod,(unsigned) Usr_TEACHER);	// Actually returns number of teachers+administrators+superusers
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of teachers in a course");

   /***** Get number of students (row[0]) *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumTchs) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of teachers in a course.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumTchs;
  }

/*****************************************************************************/
/***************** Count how many students belong to this degree *************/
/*****************************************************************************/

unsigned Usr_CountNumStdsInDeg (long DegCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumStds;

   /***** Get the number of students in a degree from database ******/
   sprintf (Query,"SELECT COUNT(DISTINCT crs_usr.UsrCod) FROM courses,crs_usr,usr_data" \
                  " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
            DegCod,(unsigned) Usr_STUDENT);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of students in a degree");

   /***** Get the number of students *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumStds) != 1)
      Lay_ShowErrorAndExit ("Error when getting the number of students in a degree.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumStds;
  }

/*****************************************************************************/
/********* Get the list of students from current course and order it ************/
/*****************************************************************************/

void Usr_GetAndOrderStdsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod)
  {
   if (ListUsrsRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
      /***** Get list of students in current course *****/
      Usr_GetStdsLstCrs ();
   else
      /***** Get other list of students *****/
      Usr_GetStdsLst (ListUsrsRange,UsrCod);

   /***** Order list of students *****/
   Usr_OrderUsrsList (&Gbl.Usrs.LstStds);
  }

/*****************************************************************************/
/***************** Get the list of teachers and order it *********************/
/*****************************************************************************/

void Usr_GetAndOrderTchsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod)
  {
   /***** Get list of teachers *****/
   Usr_GetTchsLst (ListUsrsRange,UsrCod,NULL);

   /***** Order list of teachers *****/
   Usr_OrderUsrsList (&Gbl.Usrs.LstTchs);
  }

/*****************************************************************************/
/************** Get the list of administrators and order it ******************/
/*****************************************************************************/

void Usr_GetAndOrderAdmsLst (Usr_ListUsrsRange_t ListUsrsRange)
  {
   /***** Get list of administrators *****/
   Usr_GetAdmsLst (ListUsrsRange);

   /***** Order list of administrators *****/
   Usr_OrderUsrsList (&Gbl.Usrs.LstAdms);
  }

/*****************************************************************************/
/************* Get the DNI of a random student from current course ***********/
/*****************************************************************************/

void Usr_GetDNIRandomStdInCrs (char *DNI)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   struct UsrData UsrDat;

   /***** Get a random student from current course from database *****/
   sprintf (Query,"SELECT crs_usr.UsrCod FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u' ORDER BY RAND(NOW()) LIMIT 1",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Usr_STUDENT);
   DB_QuerySELECT (Query,&mysql_res,"can not get a random student from current course");

   /***** Get user code *****/
   row = mysql_fetch_row (mysql_res);
   UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);
   Usr_GetDNIFromUsrCod (&UsrDat);
   strcpy (DNI,UsrDat.DNI);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/*************** Get the DNI of a random student from a group ****************/
/*****************************************************************************/

void Usr_GetDNIRandomStdInGrp (long GrpCod,char *DNI)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   struct UsrData UsrDat;

   /***** Get a random student from a group from database *****/
   sprintf (Query,"SELECT crs_grp_usr.UsrCod FROM crs_grp_usr,usr_data" \
                  " WHERE crs_grp_usr.GrpCod='%ld' AND crs_grp_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u' ORDER BY RAND(NOW()) LIMIT 1",
            GrpCod,(unsigned) Usr_STUDENT);
   DB_QuerySELECT (Query,&mysql_res,"can not get a random student from a group");

   /***** Get user code *****/
   row = mysql_fetch_row (mysql_res);
   UsrDat.UsrCod = Str_ConvertStrCodToLongCod (row[0]);
   Usr_GetDNIFromUsrCod (&UsrDat);
   strcpy (DNI,UsrDat.DNI);

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/***************** Get number of teachers in a department *******************/
/****************************************************************************/

unsigned Usr_GetNumberOfTeachersInDepartment (long DptCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumTchs;

   /***** Get the number of teachers in a department from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE UsrType='%u' AND DptCod='%ld'",
            (unsigned) Usr_TEACHER,DptCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of teachers in a department");

   /***** Get number of teachers *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumTchs) != 1)
      NumTchs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumTchs;
  }

/****************************************************************************/
/******************* Get number of users in a institution *******************/
/****************************************************************************/

unsigned Usr_GetNumberOfUsersInInstitution (long InsCod,Usr_UsrType_t UsrType)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumUsrs;

   /***** Get the number of users in an institution from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE InsCod='%ld' AND UsrType='%u'",
            InsCod,(unsigned) UsrType);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of users in an institution");

   /***** Get number of users *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumUsrs) != 1)
      NumUsrs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumUsrs;
  }

/****************************************************************************/
/******************* Get number of teachers in a centre *********************/
/****************************************************************************/

unsigned Usr_GetNumberOfTeachersInCentre (long CtrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumTchs;

   /***** Get the number of teachers in a centre from database *****/
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE UsrType='%u' AND CtrCod='%ld'",
            (unsigned) Usr_TEACHER,CtrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not get the number of teachers in a centre");

   /***** Get number of teachers *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumTchs) != 1)
      NumTchs = 0;

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return NumTchs;
  }

/*****************************************************************************/
/************** Get list with data of students of current course *************/
/*****************************************************************************/

#define MAX_LENGTH_QUERY_COD_GROUPS (10*1024)

void Usr_GetStdsLstCrs (void)
  {
   char Query[MAX_LENGTH_QUERY_COD_GROUPS+1];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData *UsrDat;
   unsigned NumPositiveCods = 0;
   unsigned NumNegativeCods = 0;
   char LongStr[1+10+1];
   unsigned NumGrpSel;
   long GrpCod;
   unsigned NumGrpTyp;
   bool *AddStdsWithoutGroupOf;

   Gbl.Usrs.LstStds.NumUsrs = 0;

   /***** If there are no groups selected, don't do anything *****/
   if (!Gbl.Usrs.ClassPhoto.AllGroups &&
       !Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)
      return;

   /***** Create query for all the students in the course *****/
   sprintf (Query,"SELECT crs_usr.UsrCod,crs_usr.Accepted FROM crs_usr,usr_data" \
                  " WHERE crs_usr.CrsCod='%ld' AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
            Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Usr_STUDENT);

   if (!Gbl.Usrs.ClassPhoto.AllGroups)
     {
      /***** Get list of groups types in current course *****/
      Grp_GetListGrpTypesInThisCrs ();

      /***** Allocate memory for list of booleans AddStdsWithoutGroupOf *****/
      if ((AddStdsWithoutGroupOf = (bool *) calloc (Gbl.CurrentCrs.Grps.GrpTypes.Num,sizeof (bool))) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store types of group.");

      /***** Initialize vector of booleans that indicates whether it's necessary add to the list
             the students who don't belong to any group of each type *****/
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
         AddStdsWithoutGroupOf[NumGrpTyp] = false;

      /***** Create query with the students who belong to the groups selected *****/
      if (Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps)	// If there are groups selected...
        {
         /* Check if there are positive and negative codes in the list */
         for (NumGrpSel=0; NumGrpSel<Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps; NumGrpSel++)
            if ((GrpCod = Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrpSel]) > 0)
               NumPositiveCods++;
            else
               for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
                  if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod == -GrpCod)
                    {
                     AddStdsWithoutGroupOf[NumGrpTyp] = true;
                     break;
                    }
         /* If there are positive codes, add the students who belong to those codes */
         if (NumPositiveCods)
           {
            strcat (Query," AND (crs_usr.UsrCod IN (SELECT DISTINCT UsrCod FROM crs_grp_usr WHERE");
            NumPositiveCods = 0;
            for (NumGrpSel=0; NumGrpSel<Gbl.CurrentCrs.Grps.LstGrpsSel.NumGrps; NumGrpSel++)
               if ((GrpCod = Gbl.CurrentCrs.Grps.LstGrpsSel.GrpCod[NumGrpSel]) > 0)
                 {
                  strcat (Query,NumPositiveCods ? " OR GrpCod='" : " GrpCod='");
                  sprintf (LongStr,"%ld",GrpCod);
                  strcat (Query,LongStr);
                  strcat (Query,"'");
                  NumPositiveCods++;
                 }
            strcat (Query,")");
           }
        }

      /***** Create a query with the students who don't belong to any group *****/
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
         if (AddStdsWithoutGroupOf[NumGrpTyp])
           {
            if (NumPositiveCods || NumNegativeCods)
               strcat (Query," OR ");
            else
               strcat (Query," AND (");
            /* Select all the students of the course who don't belong to any group of type GrpTypCod */
            strcat (Query,"crs_usr.UsrCod NOT IN (SELECT DISTINCT crs_grp_usr.UsrCod FROM crs_grp,crs_grp_usr WHERE crs_grp.GrpTypCod='");
            sprintf (LongStr,"%ld",Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypCod);
            strcat (Query,LongStr);
            strcat (Query,"' AND crs_grp.GrpCod=crs_grp_usr.GrpCod)");
            NumNegativeCods++;
           }
      if (NumPositiveCods || NumNegativeCods)
         strcat (Query,")");

      /***** Free memory used by the list of booleans AddStdsWithoutGroupOf *****/
      free ((void *) AddStdsWithoutGroupOf);

      /***** Free list of groups types in current course *****/
      Grp_FreeListGrpTypesAndGrps ();
     }

   /***** Query database *****/
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get students belonging to current course");

   /***** Allocate memory for the list of students *****/
   Usr_AllocateUsrsList (&(Gbl.Usrs.LstStds),(unsigned) NumRows);

   for (NumRow=0; NumRow<NumRows; NumRow++)
     {
      /* Add student to list of students */
      UsrDat = &Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.NumUsrs];

      /* Get next student */
      row = mysql_fetch_row (mysql_res);

      /* Initialize structure with student's data */
      Usr_UsrDataConstructor (UsrDat);

      /* Get user code */
      UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

      /* Get if student has accepted registration in current course */
      UsrDat->Accepted = (Str_ConvertToUpperLetter (row[1][0]) == 'Y');

      if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (UsrDat))               // Get from the database the data of the student
        {
         Gbl.Usrs.LstStds.Ind[Gbl.Usrs.LstStds.NumUsrs] = Gbl.Usrs.LstStds.NumUsrs;
         Gbl.Usrs.LstStds.Remove[Gbl.Usrs.LstStds.NumUsrs] = false;  // By default, users are not removed
         Gbl.Usrs.LstStds.NumUsrs++;
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/*********************** Get list with data of students **********************/
/*****************************************************************************/

void Usr_GetStdsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod)
  {
   char Query[4096];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData *UsrDat;

   Gbl.Usrs.LstStds.NumUsrs = 0;

   /***** Build query *****/
   switch (ListUsrsRange)
     {
      case Usr_LIST_USRS_RANGE_THIS_COURSE_ONLY_ONE:
         /* Select a specific student of current course */
         sprintf (Query,"SELECT crs_usr.UsrCod,crs_usr.Accepted FROM crs_usr,usr_data" \
                        " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod='%ld' AND usr_data.UsrCod='%ld' AND usr_data.UsrType='%u'",
                  UsrCod,Gbl.CurrentCrs.Crs.CrsCod,UsrCod,(unsigned) Usr_STUDENT);
         break;
      case Usr_LIST_USRS_RANGE_THIS_COURSE_ALL:	// To see the students in a course, use the function Usr_GetStdsLstCrs ()
         return;
      case Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL:
         /* Select students of current degree with any courses of current degree not accepted + students of current degree with all their courses of current degree accepted */
         sprintf (Query,"SELECT DISTINCT crs_usr.UsrCod,'N' FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N'" \
                        " UNION " \
                        "SELECT DISTINCT crs_usr.UsrCod,'Y' FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.UsrCod NOT IN " \
                        "(SELECT DISTINCT crs_usr.UsrCod FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N')",
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_STUDENT,
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_STUDENT,
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_STUDENT);
         break;
      case Usr_LIST_USRS_RANGE_THIS_DEGREE_SEARCH:
         return;
      case Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL:
         /* Select students with any of their courses not accepted + students with all their courses accepted */
         sprintf (Query,"SELECT DISTINCT crs_usr.UsrCod,'N' FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N'" \
                        " UNION " \
                        "SELECT DISTINCT crs_usr.UsrCod,'Y' FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.UsrCod NOT IN " \
                        "(SELECT DISTINCT crs_usr.UsrCod FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N')",
                  (unsigned) Usr_STUDENT,(unsigned) Usr_STUDENT,(unsigned) Usr_STUDENT);
         break;
      case Usr_LIST_USRS_RANGE_ANY_DEGREE_SEARCH:
         return;
      case Usr_LIST_USRS_RANGE_NO_COURSE_ALL:
         /* Select students with no courses */
         sprintf (Query,"SELECT UsrCod,'N' FROM usr_data" \
                        " WHERE UsrType='%u' AND UsrCod NOT IN (SELECT DISTINCT UsrCod FROM crs_usr)",
                  (unsigned) Usr_STUDENT);
         break;
     }
/*
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
      Lay_ShowAlert (INFO,Query);
*/
   /***** Query database *****/
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get list of students");

   if (NumRows) // If students found
     {
      if (NumRows > Cfg_MAX_USRS_IN_LIST)
        {
         sprintf (Gbl.Message,"La lista de %lu usuarios es demasiado grande para mostrarla.",	// Need translation!
                  NumRows);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
      else
        {
         /***** Allocate memory for the list of students *****/
         Usr_AllocateUsrsList (&(Gbl.Usrs.LstStds),(unsigned) NumRows);

         /***** Get the list of students *****/
         for (NumRow=0; NumRow<NumRows; NumRow++)
	   {
            /* Add teacher to list of teachers */
	    UsrDat = &Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.NumUsrs];

            /* Get next student */
            row = mysql_fetch_row (mysql_res);

            /* Initialize structure with student's data */
            Usr_UsrDataConstructor (UsrDat);

            /* Get user code */
            UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

            /* Get if student has accepted registration */
            UsrDat->Accepted = (Str_ConvertToUpperLetter (row[1][0]) != 'N');

	    if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (UsrDat))		// Get from the database students's data
	      {
	       Gbl.Usrs.LstStds.Ind[Gbl.Usrs.LstStds.NumUsrs] = Gbl.Usrs.LstStds.NumUsrs;
               Gbl.Usrs.LstStds.Remove[Gbl.Usrs.LstStds.NumUsrs] = false;  // By default, users are not removed
	       Gbl.Usrs.LstStds.NumUsrs++;
	      }
           }
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/************************ Get list with data of teachers *********************/
/*****************************************************************************/

void Usr_GetTchsLst (Usr_ListUsrsRange_t ListUsrsRange,long UsrCod,const char *TchQuery)
  {
   char Query[4096];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData *UsrDat;

   Gbl.Usrs.LstTchs.NumUsrs = 0;

   /***** Build query *****/
   switch (ListUsrsRange)
     {
      case Usr_LIST_USRS_RANGE_THIS_COURSE_ONLY_ONE:
         /* Select a specific teacher of current course */
         sprintf (Query,"SELECT crs_usr.UsrCod,crs_usr.Accepted FROM crs_usr,usr_data" \
                        " WHERE crs_usr.UsrCod='%ld' AND crs_usr.CrsCod='%ld' AND usr_data.UsrCod='%ld' AND usr_data.UsrType='%u'",
                  UsrCod,Gbl.CurrentCrs.Crs.CrsCod,UsrCod,(unsigned) Usr_TEACHER);
         break;
      case Usr_LIST_USRS_RANGE_THIS_COURSE_ALL:
         /* Select teachers of current course */
         sprintf (Query,"SELECT crs_usr.UsrCod,crs_usr.Accepted FROM crs_usr,usr_data" \
                        " WHERE crs_usr.CrsCod='%ld' AND usr_data.UsrType='%u' AND crs_usr.UsrCod=usr_data.UsrCod",
                  Gbl.CurrentCrs.Crs.CrsCod,(unsigned) Usr_TEACHER);
         break;
      case Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL:
         /* Select teachers of current degree with any courses of current degree not accepted + teachers of current degree with all their courses of current degree accepted */
         sprintf (Query,"SELECT DISTINCT crs_usr.UsrCod,'N' FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N'" \
                        " UNION " \
                        "SELECT DISTINCT crs_usr.UsrCod,'Y' FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.UsrCod NOT IN " \
                        "(SELECT DISTINCT crs_usr.UsrCod FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N')",
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_TEACHER,
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_TEACHER,
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_TEACHER);
         break;
      case Usr_LIST_USRS_RANGE_THIS_DEGREE_SEARCH:
         /* Select teachers of current degree with any courses of current degree not accepted + teachers of current degree with all their courses of current degree accepted */
         sprintf (Query,"SELECT DISTINCT crs_usr.UsrCod,'N' FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND %s AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N'" \
                        " UNION " \
                        "SELECT DISTINCT crs_usr.UsrCod,'Y' FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND %s  AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.UsrCod NOT IN " \
                        "(SELECT DISTINCT crs_usr.UsrCod FROM courses,crs_usr,usr_data" \
                        " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND usr_data.UsrType='%u' AND %s  AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N')",
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_TEACHER,TchQuery,
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_TEACHER,TchQuery,
                  Gbl.CurrentDeg.Deg.DegCod,(unsigned) Usr_TEACHER,TchQuery);
         break;
      case Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL:
         /* Select teachers with any of their courses not accepted + teachers with all their courses accepted */
         sprintf (Query,"SELECT DISTINCT crs_usr.UsrCod,'N' FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N'" \
                        " UNION " \
                        "SELECT DISTINCT crs_usr.UsrCod,'Y' FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.UsrCod NOT IN " \
                        "(SELECT DISTINCT crs_usr.UsrCod FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N')",
                  (unsigned) Usr_TEACHER,(unsigned) Usr_TEACHER,(unsigned) Usr_TEACHER);
         break;
      case Usr_LIST_USRS_RANGE_ANY_DEGREE_SEARCH:
         /* Select teachers with any of their courses not accepted + teachers with all their courses accepted */
         sprintf (Query,"SELECT DISTINCT crs_usr.UsrCod,'N' FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND %s AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N'" \
                        " UNION " \
                        "SELECT DISTINCT crs_usr.UsrCod,'Y' FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND %s AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.UsrCod NOT IN " \
                        "(SELECT DISTINCT crs_usr.UsrCod FROM usr_data,crs_usr" \
                        " WHERE usr_data.UsrType='%u' AND %s AND usr_data.UsrCod=crs_usr.UsrCod AND crs_usr.Accepted='N')",
                  (unsigned) Usr_TEACHER,TchQuery,(unsigned) Usr_TEACHER,TchQuery,(unsigned) Usr_TEACHER,TchQuery);
         break;
      case Usr_LIST_USRS_RANGE_NO_COURSE_ALL:
         /* Select teachers with no courses */
         sprintf (Query,"SELECT UsrCod,'N' FROM usr_data" \
                        " WHERE UsrType='%u' AND UsrCod NOT IN (SELECT DISTINCT UsrCod FROM crs_usr)",
                  (unsigned) Usr_TEACHER);
         break;
     }
/*
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
      Lay_ShowAlert (INFO,Query);
*/
   /***** Query database *****/
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get list of teachers");

   if (NumRows) // If teachers found in current course
     {
      /***** Allocate memory for the list of teachers *****/
      Usr_AllocateUsrsList (&(Gbl.Usrs.LstTchs),(unsigned) NumRows);

      /***** Get the list of teachers *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
         /* Add teacher to list of teachers */
	 UsrDat = &Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.NumUsrs];

         /* Get next teacher */
         row = mysql_fetch_row (mysql_res);

         /* Initialize structure with teacher's data */
         Usr_UsrDataConstructor (UsrDat);

         /* Get user code */
         UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

         /* Get if teacher has accepted registration */
         UsrDat->Accepted = (Str_ConvertToUpperLetter (row[1][0]) != 'N');

	 if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (UsrDat))		// Get from the database teacher's data
	   {
	    Gbl.Usrs.LstTchs.Ind[Gbl.Usrs.LstTchs.NumUsrs] = Gbl.Usrs.LstTchs.NumUsrs;
            Gbl.Usrs.LstTchs.Remove[Gbl.Usrs.LstTchs.NumUsrs] = false;  // By default, users are not removed
	    Gbl.Usrs.LstTchs.NumUsrs++;
	   }
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/******************** Get list with data of administrators *******************/
/*****************************************************************************/

void Usr_GetAdmsLst (Usr_ListUsrsRange_t ListUsrsRange)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData *UsrDat;

   Gbl.Usrs.LstAdms.NumUsrs = 0;

   /***** Build query *****/
   switch (ListUsrsRange)
     {
      case Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL:
         strcpy (Query,"SELECT DISTINCT UsrCod FROM deg_admin");
         break;
      case Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL:
         sprintf (Query,"SELECT UsrCod FROM deg_admin WHERE DegCod='%ld' OR DegCod='%ld'",
                  Gbl.CurrentDeg.Deg.DegCod,Usr_SPECIAL_CODE_GLOBAL_ADMIN);
         break;
      default:	// not aplicable
	 return;
     }

   /***** Query database *****/
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get list of administrators");

   if (NumRows) // If administrators found
     {
      /***** Allocate memory for the list of teachers *****/
      Usr_AllocateUsrsList (&(Gbl.Usrs.LstAdms),(unsigned) NumRows);

      /***** Get the list of administrators *****/
      for (NumRow=0; NumRow<NumRows; NumRow++)
	{
         /* Add teacher to list of administrators */
	 UsrDat = &Gbl.Usrs.LstAdms.Lst[Gbl.Usrs.LstAdms.NumUsrs];

         /* Get next administrator */
         row = mysql_fetch_row (mysql_res);

         /* Initialize structure with administrator's data */
         Usr_UsrDataConstructor (UsrDat);

         /* Get user code */
         UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

	 if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (UsrDat))		// Get from the database administrator's data
	   {
	    Gbl.Usrs.LstAdms.Ind[Gbl.Usrs.LstAdms.NumUsrs] = Gbl.Usrs.LstAdms.NumUsrs;
            Gbl.Usrs.LstAdms.Remove[Gbl.Usrs.LstAdms.NumUsrs] = false;  // By default, users are not removed
	    Gbl.Usrs.LstAdms.NumUsrs++;
	   }
        }
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/*****************************************************************************/
/********* Get the user's codes of all the students of a degree **********/
/*****************************************************************************/

void Usr_GetStdsLstDeg (long DegCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRow,NumRows;
   struct UsrData *UsrDat;

   Gbl.Usrs.LstStds.NumUsrs = 0;

   if (Usr_CountNumStdsInDeg (DegCod))
     {
      /***** Get the students in a degree from database *****/
      sprintf (Query,"SELECT DISTINCT crs_usr.UsrCod FROM courses,crs_usr,usr_data" \
                     " WHERE courses.DegCod='%ld' AND courses.CrsCod=crs_usr.CrsCod AND crs_usr.UsrCod=usr_data.UsrCod AND usr_data.UsrType='%u'",
               DegCod,(unsigned) Usr_STUDENT);
      NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get the students of a degree");

      /***** Allocate memory for the list of students *****/
      Usr_AllocateUsrsList (&(Gbl.Usrs.LstStds),(unsigned) NumRows);

      for (NumRow=0; NumRow<NumRows; NumRow++)
        {
         /* Get next user */
         row = mysql_fetch_row (mysql_res);

         /* Initialize structure with user's data */
         UsrDat = &Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.NumUsrs];
         Usr_UsrDataConstructor (UsrDat);

         /* Get user code */
         UsrDat->UsrCod = Str_ConvertStrCodToLongCod (row[0]);

         /* Add user to list */
         if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (UsrDat))               // Get from the database the data of the student
           {
            Gbl.Usrs.LstStds.Ind[Gbl.Usrs.LstStds.NumUsrs] = Gbl.Usrs.LstStds.NumUsrs;
            Gbl.Usrs.LstStds.Remove[Gbl.Usrs.LstStds.NumUsrs] = false;  // By default, users are not removed
            Gbl.Usrs.LstStds.NumUsrs++;
           }
        }

      /***** Free structure that stores the query result *****/
      DB_FreeMySQLResult (&mysql_res);
     }
  }

/*****************************************************************************/
/*********************** Ordenar the list of users ************************/
/*****************************************************************************/

void Usr_OrderUsrsList (struct ListUsers *LstUsrs)
  {
   unsigned i,j,IndMin,NumUsrs=LstUsrs->NumUsrs,Tmp;
   unsigned *PtrInd;
   struct UsrData *PtrLst;

   if (NumUsrs)
     {
      for (i=0, PtrInd=LstUsrs->Ind; i<NumUsrs; i++)
	 *PtrInd++ = i;
      for (i=0, PtrInd=LstUsrs->Ind, PtrLst = LstUsrs->Lst; i<NumUsrs-1; i++)
	{
	 for (IndMin = i, j=i+1; j<NumUsrs; j++)
	    if (Usr_NameIsLess (PtrLst + PtrInd[j],PtrLst + PtrInd[IndMin]))
	       IndMin = j;
	 if (i != IndMin)
	   {	// Exchange PtrInd[i] and PtrInd[IndMin]
            Tmp            = PtrInd[  i   ];
	    PtrInd[  i   ] = PtrInd[IndMin];
	    PtrInd[IndMin] = Tmp;
	   }
	}
     }
  }

/*****************************************************************************/
/***** Compare two users' names returning true if the first one is less ******/
/*****************************************************************************/

bool Usr_NameIsLess (struct UsrData *UsrData1,struct UsrData *UsrData2)
  {
   struct UsrData ComparableData1;
   struct UsrData ComparableData2;
   int Result; // Result of the comparison between two strings

   /***** Check the cases where the surname 1 is empty in order to lead to the end those users without surname 1 *****/
   if (UsrData1->Surname1[0])
     {
      if (!UsrData2->Surname1[0])		// If the first user does have surname 1 and the second user don't
	 return true;
     }
   else if (UsrData2->Surname1[0])		// If the first user don't have surname 1 and the second user does
      return false;
   else						// If none of the two users have surname 1...
     {
      if (UsrData1->Surname2[0])
	{
	 if (!UsrData2->Surname2[0])		// If the first user does have surname 2 and the second user don't
	    return true;
	}
      else if (UsrData2->Surname2[0])		// If the first user don't have surname 2 and the second user does
	 return false;
      else					// If none of the two users have surnames...
	{
	 if (UsrData1->FirstName[0])
	   {
	    if (!UsrData2->FirstName[0])		// If the first user does have firstname and the second user don't
	       return true;
	   }
	 else if (UsrData2->FirstName[0])		// If the first user don't have firstname and the second user does
	    return false;
	 else						// If none of the two users have firstname nor surnames...
	   {
	    if (strcoll (UsrData1->DNI,UsrData2->DNI) < 0)	// ...the user with the less DNI will be the first
	       return true;
	    return false;
	   }
	}
     }

   /***** Check the normal cases: those with surname 1 not empty *****/
   Str_ConvertToComparable (strcpy (ComparableData1.Surname1,UsrData1->Surname1));
   Str_ConvertToComparable (strcpy (ComparableData2.Surname1,UsrData2->Surname1));
   Result = strcoll (ComparableData1.Surname1,ComparableData2.Surname1);
   if (Result < 0)         // Surname 1 from user 1 less than that of user 2
      return true;
   if (Result == 0)        // Surnames 1 are the same
     {
      Str_ConvertToComparable (strcpy (ComparableData1.Surname2,UsrData1->Surname2));
      Str_ConvertToComparable (strcpy (ComparableData2.Surname2,UsrData2->Surname2));
      Result = strcoll (ComparableData1.Surname2,ComparableData2.Surname2);
      if (Result < 0)     // Surnames 1 are the same and not empty surnames; surname 2 from user 1 less than that of user 2
	 return true;
      if (Result == 0)    // Surnames 1 and Surnames 2 are the same
	{
	 Str_ConvertToComparable (strcpy (ComparableData1.FirstName,UsrData1->FirstName));
	 Str_ConvertToComparable (strcpy (ComparableData2.FirstName,UsrData2->FirstName));
	 Result = strcoll (ComparableData1.FirstName,ComparableData2.FirstName);
	 if (Result < 0)  // Surnames 1 and Surnames 2 are the same; firstname from user 1 less than that of user 2
	    return true;
	}
     }
   return false;
  }

/*****************************************************************************/
/********************** Allocate space for list of users *********************/
/*****************************************************************************/

static void Usr_AllocateUsrsList (struct ListUsers *LstUsrs,unsigned NumUsrs)
  {
/*
if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
   {
    sprintf (Gbl.Message,"Memory used by list = %lu",(long) sizeof (struct UsrData) * NumUsrs);
    Lay_ShowAlert (INFO,Gbl.Message);
   }
*/
   if ((LstUsrs->Ind = (unsigned *) calloc (NumUsrs,sizeof (unsigned))) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store users' data.");
   if ((LstUsrs->Remove = (bool *) calloc (NumUsrs,sizeof (bool))) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store users' data.");
   if ((LstUsrs->Lst = (struct UsrData *) calloc (NumUsrs,sizeof (struct UsrData))) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store users' data.");
  }

/*****************************************************************************/
/********************* Free space used for list of users *********************/
/*****************************************************************************/

void Usr_FreeUsrsList (struct ListUsers *LstUsrs)
  {
   unsigned NumUsr;

   if (LstUsrs->NumUsrs)
     {
      /***** Free memory used for users data *****/
      for (NumUsr=0; NumUsr<LstUsrs->NumUsrs; NumUsr++)
         Usr_UsrDataDestructor (&(LstUsrs->Lst[NumUsr]));

      /***** Free the list itself *****/
      if (LstUsrs->Lst)
        {
         free ((void *) LstUsrs->Lst);
         LstUsrs->Lst = NULL;
        }

      /***** Free booleans vector *****/
      if (LstUsrs->Remove)
        {
         free ((void *) LstUsrs->Remove);
         LstUsrs->Remove = NULL;
        }

      /***** Free index vector *****/
      if (LstUsrs->Ind)
        {
         free ((void *) LstUsrs->Ind);
         LstUsrs->Ind = NULL;
        }

      /***** Reset number of users *****/
      LstUsrs->NumUsrs = 0;
     }
  }

/*****************************************************************************/
/********** Write parameter with the list of users seleccionados ************/
/*****************************************************************************/

void Usr_HiddenParUsrCodAll (Act_Action_t NextAction,const char *ListUsrCods)
  {
   Ses_InsertHiddenParInDB (NextAction,"UsrCodAll",ListUsrCods);
  }

/*****************************************************************************/
/************************* Get list of selected users ************************/
/*****************************************************************************/

void Usr_GetListSelectedUsrs (void)
  {
   unsigned Length;

   /***** Allocate memory for the lists of users *****/
   Usr_AllocateListEncryptedUsrCodAll ();
   Usr_AllocateListEncryptedUsrCodTch ();
   Usr_AllocateListEncryptedUsrCodStd ();

   /***** Get selected users *****/
   Par_GetParam (Par_PARAM_MULTIPLE,"UsrCodAll",Gbl.Usrs.Select.All,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.Select.All,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS,true);
   Par_GetParam (Par_PARAM_MULTIPLE,"UsrCodTch",Gbl.Usrs.Select.Tch,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.Select.Tch,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS,true);
   Par_GetParam (Par_PARAM_MULTIPLE,"UsrCodStd",Gbl.Usrs.Select.Std,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.Select.Std,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS,true);

   /***** Add teachers to the list with all selected users *****/
   if (Gbl.Usrs.Select.Tch[0])
     {
      if (Gbl.Usrs.Select.All[0])
         if ((Length = strlen (Gbl.Usrs.Select.All)) < Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS)
           {
            Gbl.Usrs.Select.All[Length  ] = Par_SEPARATOR_PARAM_MULTIPLE;
            Gbl.Usrs.Select.All[Length+1] = '\0';
           }
      strncat (Gbl.Usrs.Select.All,Gbl.Usrs.Select.Tch,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS-strlen (Gbl.Usrs.Select.All));
     }

   /***** Add students to the list with all selected users *****/
   if (Gbl.Usrs.Select.Std[0])
     {
      if (Gbl.Usrs.Select.All[0])
         if ((Length = strlen (Gbl.Usrs.Select.All)) < Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS)
           {
            Gbl.Usrs.Select.All[Length  ] = Par_SEPARATOR_PARAM_MULTIPLE;
            Gbl.Usrs.Select.All[Length+1] = '\0';
           }
      strncat (Gbl.Usrs.Select.All,Gbl.Usrs.Select.Std,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS-strlen (Gbl.Usrs.Select.All));
     }
  }

/*****************************************************************************/
/************ Get list of DNI or nicknames written explicitely ***************/
/*****************************************************************************/
// Returns true if no errors

bool Usr_GetListDNIOrNicks (bool WriteErrorMsgs)
  {
   unsigned Length;
   const char *Ptr;
   char DNIOrNickname[1024+1];
   struct UsrData UsrDat;
   bool ValidUsr;
   bool Error = false;

   /***** Allocate memory for the lists of DNI of users *****/
   Usr_AllocateListEncryptedUsrCodAll ();

   /***** Allocate memory for the lists of nicknames of users *****/
   Usr_AllocateListNicks ();

   /***** Get nicknames of users *****/
   Par_GetParam (Par_PARAM_SINGLE,"Nicks",Gbl.Usrs.ListDNIOrNicks,Usr_MAX_BYTES_LIST_NICKS);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Usrs.ListDNIOrNicks,Usr_MAX_BYTES_LIST_NICKS,true);

   /***** Add encrypted DNIs of users to the list with all selected users *****/
   if (Gbl.Usrs.ListDNIOrNicks[0])
     {
      /* Initialize structure with user's data */
      Usr_UsrDataConstructor (&UsrDat);

      /* Get next plain DNI or nickname */
      Ptr = Gbl.Usrs.ListDNIOrNicks;
      while (*Ptr)
        {
         /* Find next string in text until comma or semicolon (leading and trailing spaces are removed) */
         Str_GetNextStringUntilComma (&Ptr,DNIOrNickname,1024);

         /* Check if string is plain DNI or nickname and get encrypted DNI */
         if (DNIOrNickname[0])
           {
            if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (DNIOrNickname))		// String is a DNI?
              {
               strncpy (UsrDat.DNI,DNIOrNickname,Usr_MAX_LENGTH_USR_ID);
               UsrDat.DNI[Usr_MAX_LENGTH_USR_ID] = '\0';

               if (!(ValidUsr = Usr_ChkDNIAndGetAllUsrDataFromDNI (&UsrDat)))
                 {
                  if (WriteErrorMsgs)
                    {
                     sprintf (Gbl.Message,"No existe ning&uacute;n usuario con el DNI <strong>%s</strong>.",	// Need translation!
                              DNIOrNickname);
	             Lay_ShowAlert (ERROR,Gbl.Message);
                    }
                  Error = true;
                 } 
              }
            else if (Usr_CheckIfNickIsValid (DNIOrNickname))	// String is a nickname?
	      {
	       if ((ValidUsr = Usr_GetUsrCodFromNickname (DNIOrNickname,&UsrDat)))
                  Usr_GetAllUsrDataFromUsrCod (&UsrDat);	// To get UsrDat.EncryptedUsrCod
               else
                 {
                  if (WriteErrorMsgs)
                    {
                     sprintf (Gbl.Message,"No existe ning&uacute;n usuario con el apodo <strong>%s</strong>.",	// Need translation!
                              DNIOrNickname);
	             Lay_ShowAlert (ERROR,Gbl.Message);
                    }
                  Error = true;
                 }
	      }
            else						// String is not a valid DNI or nickname
              {
               ValidUsr = false;
               if (WriteErrorMsgs)
                 {
                  sprintf (Gbl.Message,"El DNI o apodo <strong>%s</strong> no es v&aacute;lido.",	// Need translation!
                           DNIOrNickname);
	          Lay_ShowAlert (ERROR,Gbl.Message);
                 }
               Error = true;
              }
            if (ValidUsr)	// Only if user is valid
              {
               /* Find if encrypted user's code is yet in list */
               if (!Usr_FindEncryptedUsrCodInList (UsrDat.EncryptedUsrCod))	// If not in list ==> add it
                 {
                  /* Add encrypted user's code to list of users */
                  if ((Length = strlen (Gbl.Usrs.Select.All)) == 0)	// First user in list
                    {
                     if (strlen (UsrDat.EncryptedUsrCod) < Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS)
                        strcpy (Gbl.Usrs.Select.All,UsrDat.EncryptedUsrCod);	// Add first user
                    }
                  else	// Not first user in list
                     if (Length < Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS + 1 + strlen (UsrDat.EncryptedUsrCod))
                       {
	                // Add another user
                        Gbl.Usrs.Select.All[Length] = Par_SEPARATOR_PARAM_MULTIPLE;
                        strcat (Gbl.Usrs.Select.All,UsrDat.EncryptedUsrCod);
                       }
                 }
              }
           }
        }

      /* Free memory used for user's data */
      Usr_UsrDataDestructor (&UsrDat);
     }
   return Error;
  }

/*****************************************************************************/
/************** Find if encrypted user's code is yet in list *****************/
/*****************************************************************************/
// Returns true if EncryptedUsrCodToFind is in Gbl.Usrs.Select.All

bool Usr_FindEncryptedUsrCodInList (const char *EncryptedUsrCodToFind)
  {
   const char *Ptr;
   char EncryptedUsrCod[Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64+1];

   Ptr = Gbl.Usrs.Select.All;
   while (*Ptr)
     {
      Str_GetNextStrUntilSeparParamMult (&Ptr,EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64);
      if (!strcmp (EncryptedUsrCodToFind,EncryptedUsrCod))
         return true;	// Found!
     }
   return false;	// Not found
  }

/*****************************************************************************/
/*************** Count number of valid DNIs in encrypted list ****************/
/*****************************************************************************/

int Usr_CountNumUsrsInEncryptedList (void)
  {
   const char *Ptr;
   int NumUsrs = 0;
   struct UsrData UsrDat;

   /***** Recorrer the lista Gbl.Usrs.Select.All to contar the number of users destino *****/
   Ptr = Gbl.Usrs.Select.All;
   while (*Ptr)
     {
      Str_GetNextStrUntilSeparParamMult (&Ptr,UsrDat.EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64);
      Usr_GetUsrCodFromEncryptedUsrCod (&UsrDat);
      if (UsrDat.UsrCod > 0)
	 NumUsrs++;
     }
   return NumUsrs;
  }

/*****************************************************************************/
/********************* Allocate memory for list of users *********************/
/*****************************************************************************/

void Usr_AllocateListEncryptedUsrCodAll (void)
  {
   if (!Gbl.Usrs.Select.All)
     {
      if ((Gbl.Usrs.Select.All = malloc (Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS+1)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store list of users.");
      Gbl.Usrs.Select.All[0] = '\0';
     }
  }

/*****************************************************************************/
/****************** Allocate memory for list of students *********************/
/*****************************************************************************/

void Usr_AllocateListEncryptedUsrCodStd (void)
  {
   if (!Gbl.Usrs.Select.Std)
     {
      if ((Gbl.Usrs.Select.Std = malloc (Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS+1)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store list of students.");
      Gbl.Usrs.Select.Std[0] = '\0';
     }
  }

/*****************************************************************************/
/****************** Allocate memory for list of teachers *********************/
/*****************************************************************************/

void Usr_AllocateListEncryptedUsrCodTch (void)
  {
   if (!Gbl.Usrs.Select.Tch)
     {
      if ((Gbl.Usrs.Select.Tch = malloc (Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS+1)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store list of teachers.");
      Gbl.Usrs.Select.Tch[0] = '\0';
     }
  }

/*****************************************************************************/
/********** Allocate memory for list of DNI or nicknames of users ************/
/*****************************************************************************/

void Usr_AllocateListNicks (void)
  {
   if (!Gbl.Usrs.ListDNIOrNicks)
     {
      if ((Gbl.Usrs.ListDNIOrNicks = malloc (Usr_MAX_BYTES_LIST_NICKS+1)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store nicknames.");
      Gbl.Usrs.ListDNIOrNicks[0] = '\0';
     }
  }

/*****************************************************************************/
/********************** Free memory for lists of users ***********************/
/*****************************************************************************/

void Usr_FreeListsEncryptedUsrCods (void)
  {
   if (Gbl.Usrs.Select.All)
     {
      free ((void *) Gbl.Usrs.Select.All);
      Gbl.Usrs.Select.All = NULL;
     }
   if (Gbl.Usrs.Select.Tch)
     {
      free ((void *) Gbl.Usrs.Select.Tch);
      Gbl.Usrs.Select.Tch = NULL;
     }
   if (Gbl.Usrs.Select.Std)
     {
      free ((void *) Gbl.Usrs.Select.Std);
      Gbl.Usrs.Select.Std = NULL;
     }
  }

/*****************************************************************************/
/****************** Free memory for the list of nicknames ********************/
/*****************************************************************************/

void Usr_FreeListNicks (void)
  {
   if (Gbl.Usrs.ListDNIOrNicks)
     {
      free ((void *) Gbl.Usrs.ListDNIOrNicks);
      Gbl.Usrs.ListDNIOrNicks = NULL;
     }
  }

/*****************************************************************************/
/******************** List users to select some of them **********************/
/*****************************************************************************/

void Usr_ShowFormsToSelectUsrListType (Act_Action_t NextAction)
  {
   /***** Start table *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** 1st row *****/
   /* Put a button to select USR_CLASS_ROOM */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"%s\">" \
                      "<table cellspacing=\"4\" cellpadding=\"0\"><tr><td align=\"left\" width=\"24\">",
            (Gbl.Usrs.Me.ListType == Usr_CLASS_PHOTO) ? "USR_LIST_TYPE_ON" : "USR_LIST_TYPE_OFF");
   Usr_FormToSelectUsrListType (NextAction,Usr_CLASS_PHOTO);
   fprintf (Gbl.F.Out,"</td>");

   /* Number of columns in the class photo */
   fprintf (Gbl.F.Out,"<td align=\"left\">");
   Act_FormStart (NextAction);
   Grp_PutParamsCodGrps ();
   Usr_PutParamUsrListType (Usr_CLASS_PHOTO);
   Usr_PutParamListWithPhotos ();
   Usr_PutExtraParamsUsrList (NextAction);
   Usr_PutSelectorNumColsClassPhoto ();
   fprintf (Gbl.F.Out,"</form></td></tr></table></td></tr>\n");

   /***** 2nd row *****/
   /* Put a button to select Usr_LIST */
   fprintf (Gbl.F.Out,"<tr><td align=\"left\" class=\"%s\">" \
                      "<table cellspacing=\"4\" cellpadding=\"0\"><tr><td align=\"left\" width=\"24\">",
            (Gbl.Usrs.Me.ListType == Usr_LIST) ? "USR_LIST_TYPE_ON" : "USR_LIST_TYPE_OFF");
   Usr_FormToSelectUsrListType (NextAction,Usr_LIST);
   fprintf (Gbl.F.Out,"</td>");

   /* See the photos in list? */
   fprintf (Gbl.F.Out,"<td align=\"left\">");
   Act_FormStart (NextAction);
   Grp_PutParamsCodGrps ();
   Usr_PutParamUsrListType (Usr_LIST);
   Usr_PutExtraParamsUsrList (NextAction);
   Usr_PutCheckboxListWithPhotos ();
   fprintf (Gbl.F.Out,"</form></td></tr></table></td></tr>\n");

   /***** End of table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
  }

/*****************************************************************************/
/************* Put a radio element to select a users' list type **************/
/*****************************************************************************/

static void Usr_FormToSelectUsrListType (Act_Action_t NextAction,Usr_ShowUsrsType_t ListType)
  {
   Act_FormStart (NextAction);
   Grp_PutParamsCodGrps ();
   Usr_PutParamListWithPhotos ();
   Usr_PutParamUsrListType (ListType);
   Usr_PutExtraParamsUsrList (NextAction);
   Act_LinkFormSubmit (Act_MAIN_WINDOW,StringsUsrListType[ListType],NULL);
   fprintf (Gbl.F.Out,"<img src=\"%s/%s24x24.gif\" width=\"24\" height=\"24\" alt=\"%s\" align=\"bottom\" />" \
                      "</acronym></a></form>",
            Gbl.Prefs.IconsURL,StringsUsrListTypeInDB[ListType],StringsUsrListType[ListType]);
  }

/*****************************************************************************/
/***************** Put extra parameters for a list of users ******************/
/*****************************************************************************/

void Usr_PutExtraParamsUsrList (Act_Action_t NextAction)
  {
   switch (Gbl.CurrentAct)
     {
      case ActSeeStd:
      case ActSeeTch:
         Usr_ParamUsrsRange ();
         break;
      case ActReqAsgWrkCrs:
      case ActEdiAsgWrkCrs:
         Brw_ParamWorksZoneCrs ();
         break;
      case ActReqMsgUsr:
	 Usr_HiddenParUsrCodAll (NextAction,Gbl.Usrs.Select.All);
         break;
      case ActSeePhoDeg:
         Sta_PutHiddenParamTypeOfAvg ();
         Sta_PutHiddenParamPhotoSize ();
         Sta_PutHiddenParamOrderDegrees ();
         break;
      default:
         break;
     }
  }

/*****************************************************************************/
/******************** List users to select some of them **********************/
/*****************************************************************************/

void Usr_ListUsersToSelect (Usr_UsrType_t UsrType)
  {
   /***** If there are no users, don't list anything *****/
   if ((UsrType == Usr_STUDENT && !Gbl.Usrs.LstStds.NumUsrs) ||
       (UsrType == Usr_TEACHER && !Gbl.Usrs.LstTchs.NumUsrs))
       return;

   /***** Put a row to select all the users *****/
   Usr_PutCheckboxToSelectAllTheUsers (UsrType);

   /***** Draw the classphoto/list  *****/
   switch (Gbl.Usrs.Me.ListType)
     {
      case Usr_CLASS_PHOTO:
         Usr_DrawClassPhoto (Usr_CLASS_PHOTO_SEL,UsrType);
         break;
      case Usr_LIST:
         Usr_ListUsrsForSelection (UsrType);
         break;
     }
  }

/*****************************************************************************/
/****** Put a row, in a classphoto or a list, to select all the users ********/
/*****************************************************************************/

void Usr_PutCheckboxToSelectAllTheUsers (Usr_UsrType_t UsrType)
  {
   extern const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_PLURAL_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   struct ListUsers *LstUsrs;
   Usr_Gender_t Gender;

   fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"%u\" bgcolor=\"#%s\" class=\"TIT_TBL\">",
            Usr_GetColumnsForSelectUsrs (),VERY_LIGHT_BLUE);
   if (UsrType == Usr_STUDENT)
     {
      fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"SEL_DESEL_ALUMS\" value=\"\" onclick=\"togglecheckChildren(this,'UsrCodStd')\" />");
      LstUsrs = &Gbl.Usrs.LstStds;
     }
   else
     {
      fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"SEL_DESEL_PROFS\" value=\"\" onclick=\"togglecheckChildren(this,'UsrCodTch')\" />");
      LstUsrs = &Gbl.Usrs.LstTchs;
     }
   Gender = Usr_GetGenderOfUsrsLst (LstUsrs);
   fprintf (Gbl.F.Out,"%s:</td></tr>",(LstUsrs->NumUsrs == 1) ? Txt_USR_TYPES_SINGULAR_Abc[UsrType][Gender][Gbl.Prefs.Language] :
                                                                Txt_USR_TYPES_PLURAL_Abc  [UsrType][Gender][Gbl.Prefs.Language]);
  }


/*****************************************************************************/
/************************ Get gender of a list of users **********************/
/*****************************************************************************/

Usr_Gender_t Usr_GetGenderOfUsrsLst (struct ListUsers *LstUsrs)
  {
   Usr_Gender_t Gender;
   unsigned NumUsr;

   /***** If no users, gender is undefined *****/
   if (LstUsrs->NumUsrs == 0)
      return Usr_GENDER_UNKNOWN;

   /***** Initialize gender to that of first user in list *****/
   Gender = LstUsrs->Lst[0].Gender;

   /***** Search if there is one user at least with different gender than the first one *****/
   for (NumUsr=1; NumUsr<LstUsrs->NumUsrs; NumUsr++)
      if (LstUsrs->Lst[NumUsr].Gender != Gender)
         return Usr_GENDER_UNKNOWN;

   return Gender;
  }

/*****************************************************************************/
/**** Get number of table columns, in classphoto or list, to select users ****/
/*****************************************************************************/

unsigned Usr_GetColumnsForSelectUsrs (void)
  {
   return (Gbl.Usrs.Me.ListType == Usr_CLASS_PHOTO) ? Gbl.Usrs.ClassPhoto.Cols :
                                                      (Gbl.Usrs.Listing.WithPhotos ? 1+Usr_NUM_MAIN_FIELDS_DATA_USR : Usr_NUM_MAIN_FIELDS_DATA_USR);
  }

/*****************************************************************************/
/******* Put a checkbox, in a classphoto or a list, to select a user *********/
/*****************************************************************************/

void Usr_PutCheckboxToSelectUser (Usr_UsrType_t UsrType,const char *EncryptedUsrCod,bool UsrIsTheMsgSender)
  {
   fprintf (Gbl.F.Out,"<input type=\"checkbox\" name=\"");
   if (UsrType == Usr_STUDENT)
      fprintf (Gbl.F.Out,"UsrCodStd\" value=\"%s\" onclick=\"checkParent(this,'SEL_DESEL_ALUMS')\"",EncryptedUsrCod);
   else
      fprintf (Gbl.F.Out,"UsrCodTch\" value=\"%s\" onclick=\"checkParent(this,'SEL_DESEL_PROFS')\"",EncryptedUsrCod);
   if (UsrIsTheMsgSender)	// Reply to a user
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," />");
  }

/*****************************************************************************/
/********* Put a checkbox to select whether list users with photos ***********/
/*****************************************************************************/

void Usr_PutCheckboxListWithPhotos (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Display_photos[Txt_NUM_LANGUAGES];
 
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"WithPhotosExists\" value=\"Y\" />");

   /***** Put checkbox to select  whether list users with photos *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"left\" valign=\"middle\">" \
                      "<input type=\"checkbox\" name=\"WithPhotos\" value=\"Y\"");
   if (Gbl.Usrs.Listing.WithPhotos)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   fprintf (Gbl.F.Out," onclick=\"javascript:document.forms[%d].submit();\" /></td>",Gbl.NumForm);
   fprintf (Gbl.F.Out,"<td align=\"left\" valign=\"middle\" class=\"%s\">&nbsp;%s</td>" \
                      "</tr></table>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_Display_photos[Gbl.Prefs.Language]);
  }

/*****************************************************************************/
/*************************** List main students' data ************************/
/*****************************************************************************/

void Usr_ListMainDataStds (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_all_data[Txt_NUM_LANGUAGES];
   extern const char *Txt_NAMES_MAIN_FIELDS_DATA_USR[Usr_NUM_MAIN_FIELDS_DATA_USR][Txt_NUM_LANGUAGES];
   unsigned NumCol;
   unsigned NumUsr;
   char *GroupNames;

   GroupNames = NULL;	// To avoid warning

   /***** Asign users listing type depending on current action *****/
   Usr_AsignUsrListType ();

   if (Gbl.Usrs.LstStds.NumUsrs)
     {
      if (Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER)
        {
         /****** Link to see all the data ******/
         fprintf (Gbl.F.Out,"<div align=\"center\">");
         Act_FormStart (ActLstStdAll);
         Grp_PutParamsCodGrps ();
         Usr_PutParamListWithPhotos ();
         Usr_PutExtraParamsUsrList (ActLstStdAll);
         fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">");
         fprintf (Gbl.F.Out,"<tr><td align=\"center\" valign=\"middle\">");
         Act_LinkFormSubmit (Act_NEW_WINDOW,Txt_Show_all_data[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"%s</acronym></a></td></tr></table></form></div>",
                  Txt_Show_all_data[Gbl.Prefs.Language]);
        }

      /***** Allocate memory for the string with the list of group names where student belongs to *****/
      if ((GroupNames = (char *) malloc ((MAX_LENGTH_GROUP_NAME+3)*Gbl.CurrentCrs.Grps.GrpTypes.NumGrpsTotal)) == NULL)
         Lay_ShowErrorAndExit ("Not enough memory to store names of groups.");

      /***** Start table with list of students *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
      if (!Gbl.Usrs.ClassPhoto.AllGroups)
        {
         fprintf (Gbl.F.Out,"<tr><td colspan=\"%u\" align=\"center\" class=\"TITULO\">",Usr_NUM_MAIN_FIELDS_DATA_USR);
         Grp_WriteNamesOfSelectedGrps ();
         fprintf (Gbl.F.Out,"</td></tr>");
        }

      /***** Heading row with column names *****/
      /* Start row */
      fprintf (Gbl.F.Out,"<tr>");
      /* 1. Columns for the data */
      for (NumCol=0; NumCol<Usr_NUM_MAIN_FIELDS_DATA_USR; NumCol++)
         if (NumCol != 2 || Gbl.Usrs.Listing.WithPhotos)	// Skip photo column if I don't want this column
            fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                     VERY_LIGHT_BLUE,Txt_NAMES_MAIN_FIELDS_DATA_USR[NumCol][Gbl.Prefs.Language]);

      /* End row */
      fprintf (Gbl.F.Out,"</tr>\n");

      /***** List students' data *****/
      for (NumUsr=0; NumUsr<Gbl.Usrs.LstStds.NumUsrs; NumUsr++)
         Usr_WriteRowStdMainData (NumUsr+1,&(Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.Ind[NumUsr]]),false);

      /***** End of table *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /***** Free memory used by the string with the list of group names where student belongs to *****/
      free ((void *) GroupNames);
     }
   else	// Gbl.Usrs.LstStds.NumUsrs == 0
      Usr_ShowWarningNoUsersFound (Usr_STUDENT);

   /***** Free memory for students list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);
  }

/*****************************************************************************/
/***************************** List students' data ***************************/
/*****************************************************************************/

void Usr_ListAllDataStds (void)
  {
   extern const char *Txt_NAMES_ALL_FIELDS_DATA_STD[Usr_NUM_ALL_FIELDS_DATA_STD][Txt_NUM_LANGUAGES];
   extern const char *Txt_Group[Txt_NUM_LANGUAGES];
   unsigned NumColumnsCommonCard;
   unsigned NumColumnsCardAndGroups;
   unsigned NumColumnsTotal;
   unsigned NumCol;
   unsigned NumUsr;
   char *GroupNames;
   unsigned NumGrpTyp,NumField;

   GroupNames = NULL;	// To avoid warning

   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Assign users listing type depending on current action *****/
   Usr_AsignUsrListType ();

   /***** Get list of groups types and groups in current course *****/
   Grp_GetListGrpTypesInThisCrs ();

   /***** Get groups to show *****/
   Grp_GetParCodsSeveralGrpsToShowUsrs ();

   /***** Get students range *****/
   Usr_GetListUsrsRange (Usr_STUDENT,
                         Gbl.CurrentCrs.Crs.CrsCod > 0 ? Usr_LIST_USRS_RANGE_THIS_COURSE_ALL : (Gbl.CurrentDeg.Deg.DegCod > 0 ? Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL : Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL));

   /****** Get and order list of students in current course ******/
   Usr_GetAndOrderStdsLst (Gbl.Usrs.ListRange,-1L);

   if (Gbl.Usrs.LstStds.NumUsrs)
     {
      if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
         /***** Get list of record fields in current course *****/
         Rec_GetListRecordFieldsInCurrentCrs ();

      /***** Set number of columns *****/
      NumColumnsCommonCard = Usr_NUM_ALL_FIELDS_DATA_STD;
      if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
        {
         NumColumnsCardAndGroups = NumColumnsCommonCard + Gbl.CurrentCrs.Grps.GrpTypes.Num;
         NumColumnsTotal = NumColumnsCardAndGroups + Gbl.CurrentCrs.Records.LstFields.Num;
        }
      else
         NumColumnsTotal = NumColumnsCardAndGroups = NumColumnsCommonCard;

      /***** Allocate memory for the string with the list of group names where student belongs to *****/
      if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
         if ((GroupNames = (char *) malloc ((MAX_LENGTH_GROUP_NAME+3)*Gbl.CurrentCrs.Grps.GrpTypes.NumGrpsTotal)) == NULL)
            Lay_ShowErrorAndExit ("Not enough memory to store names of groups.");

      /***** Start table with list of students *****/
      Lay_StartTableFrame (NULL,NULL,NULL);
      if (!Gbl.Usrs.ClassPhoto.AllGroups)
        {
         fprintf (Gbl.F.Out,"<tr><td colspan=\"%u\" align=\"center\" class=\"TITULO\">",
                  NumColumnsTotal);
         Grp_WriteNamesOfSelectedGrps ();
         fprintf (Gbl.F.Out,"</td></tr>");
        }

      /***** Heading row with column names *****/
      /* Start row */
      fprintf (Gbl.F.Out,"<tr>");

      /* 1. Columns fot the data */
      for (NumCol=(Gbl.Usrs.Listing.WithPhotos ? 0 : 1); NumCol<NumColumnsCommonCard; NumCol++)
         fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                  VERY_LIGHT_BLUE,Txt_NAMES_ALL_FIELDS_DATA_STD[NumCol][Gbl.Prefs.Language]);

      /* 2. Columns for the groups */
      if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
        {
         if (Gbl.CurrentCrs.Grps.GrpTypes.Num)
            for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
               if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps)	 // If current course tiene groups of este type
                  fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s %s&nbsp;</td>",
                           VERY_LIGHT_BLUE,Txt_Group[Gbl.Prefs.Language],
                           Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].GrpTypName);

         if (Gbl.CurrentCrs.Records.LstFields.Num)
           {
            /* 3. Names of record fields that depend on the course */
            for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
               fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                        VERY_LIGHT_BLUE,Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Name);
            /* 4. Visibility type for the record fields that depend on the course, in other row */
            fprintf (Gbl.F.Out,"</tr>\n<tr>");
            for (NumCol=0; NumCol<NumColumnsCardAndGroups; NumCol++)
               if (NumCol != 1 || Gbl.Usrs.Listing.WithPhotos)	// Skip photo column if I don't want it in listing
                  fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\"></td>",LIGHTEST_BLUE);
            for (NumField=0; NumField<Gbl.CurrentCrs.Records.LstFields.Num; NumField++)
               fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">(%s)&nbsp;</td>",
                        LIGHTEST_BLUE,Txt_RECORD_FIELD_VISIBILITY_RECORD[Gbl.CurrentCrs.Records.LstFields.Lst[NumField].Visibility][Gbl.Prefs.Language]);
           }
        }

      /* End row */
      fprintf (Gbl.F.Out,"</tr>\n");

      /***** List students' data *****/
      for (NumUsr=0; NumUsr<Gbl.Usrs.LstStds.NumUsrs; NumUsr++)
         Usr_WriteRowStdAllData (NumUsr+1,&(Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.Ind[NumUsr]]),GroupNames);

      /***** End of table *****/
      Lay_EndTableFrame ();

      /***** Free memory used by the string with the list of group names where student belongs to *****/
      if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
         free ((void *) GroupNames);
     }
   else	// Gbl.Usrs.LstStds.NumUsrs == 0
      Usr_ShowWarningNoUsersFound (Usr_STUDENT);

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();

   /***** Free memory for students list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /***** Free list of groups types and groups in current course *****/
   Grp_FreeListGrpTypesAndGrps ();
  }

/*****************************************************************************/
/*************** List users (of current course) for selection ****************/
/*****************************************************************************/

void Usr_ListUsrsForSelection (Usr_UsrType_t UsrType)
  {
   extern const char *Txt_NAMES_MAIN_FIELDS_DATA_USR[Usr_NUM_MAIN_FIELDS_DATA_USR][Txt_NUM_LANGUAGES];
   unsigned NumCol;
   unsigned NumUsr;

   /***** Heading row with column names *****/
   /* Start row */
   fprintf (Gbl.F.Out,"<tr>");
   /* First column used for selection */
   fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">&nbsp;</td>",VERY_LIGHT_BLUE);
   /* Columns for the data */
   for (NumCol=0; NumCol<Usr_NUM_MAIN_FIELDS_DATA_USR; NumCol++)
      if (NumCol != 2 || Gbl.Usrs.Listing.WithPhotos)	// Skip photo column if I don't want this column
         fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                  VERY_LIGHT_BLUE,Txt_NAMES_MAIN_FIELDS_DATA_USR[NumCol][Gbl.Prefs.Language]);
   /* End row */
   fprintf (Gbl.F.Out,"</tr>\n");

   /***** List users' data *****/
   switch (UsrType)
     {
      case Usr_STUDENT:
         for (NumUsr=0; NumUsr<Gbl.Usrs.LstStds.NumUsrs; NumUsr++)
            Usr_WriteRowStdMainData (NumUsr+1,&(Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.Ind[NumUsr]]),true);
         break;
      case Usr_TEACHER:
         for (NumUsr=0; NumUsr<Gbl.Usrs.LstTchs.NumUsrs; NumUsr++)
            Usr_WriteRowTchMainData (NumUsr+1,&(Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.Ind[NumUsr]]),true);
         break;
      default:
         break;
     }
  }

/*****************************************************************************/
/************************** List main teachers' data *************************/
/*****************************************************************************/

void Usr_ListMainDataTchs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_all_data[Txt_NUM_LANGUAGES];
   extern const char *Txt_NAMES_MAIN_FIELDS_DATA_USR[Usr_NUM_MAIN_FIELDS_DATA_USR][Txt_NUM_LANGUAGES];
   unsigned NumColumns;
   unsigned NumCol;
   unsigned NumUsr;

   /***** Asign users listing type depending on current action *****/
   Usr_AsignUsrListType ();

   if (Gbl.Usrs.LstTchs.NumUsrs)
     {
      if (Gbl.Usrs.Me.UsrTypeLog >= Usr_TEACHER)
        {
         /****** Link to see all the data ******/
         fprintf (Gbl.F.Out,"<div align=\"center\">");
         Act_FormStart (ActLstTchAll);
         Usr_ParamUsrsRange ();
         Usr_PutParamListWithPhotos ();
         fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                            "<tr><td align=\"center\" valign=\"middle\">");
         Act_LinkFormSubmit (Act_NEW_WINDOW,Txt_Show_all_data[Gbl.Prefs.Language],The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"%s</acronym></a></td></tr></table></form></div>",
                  Txt_Show_all_data[Gbl.Prefs.Language]);
        }

      /***** Initialize number of columns *****/
      NumColumns = Usr_NUM_MAIN_FIELDS_DATA_USR;

      /***** Start table with list of teachers *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /***** Heading row with column names *****/
      /* Start row */
      fprintf (Gbl.F.Out,"<tr>");
      for (NumCol=0; NumCol<NumColumns; NumCol++)
         if (NumCol != 2 || Gbl.Usrs.Listing.WithPhotos)	// Skip photo column if I don't want this column
            fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                     VERY_LIGHT_BLUE,Txt_NAMES_MAIN_FIELDS_DATA_USR[NumCol][Gbl.Prefs.Language]);

      /* End row */
      fprintf (Gbl.F.Out,"</tr>\n");

      /***** List data of teachers *****/
      for (NumUsr=0; NumUsr<Gbl.Usrs.LstTchs.NumUsrs; NumUsr++)
         Usr_WriteRowTchMainData (NumUsr+1,&(Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.Ind[NumUsr]]),false);

      /***** End of table *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
   else	// Gbl.Usrs.LstTchs.NumUsrs == 0
      Lay_ShowAlert (INFO,"No hay profesores.");	// Need translation!

   /***** Free memory for teachers list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
  }

/*****************************************************************************/
/************************** List all teachers' data **************************/
/*****************************************************************************/

void Usr_ListAllDataTchs (void)
  {
   extern const char *Txt_NAMES_ALL_FIELDS_DATA_TCH[Usr_NUM_ALL_FIELDS_DATA_TCH][Txt_NUM_LANGUAGES];
   unsigned NumColumns;
   unsigned NumCol;
   unsigned NumUsr;

   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Asign users listing type depending on current action *****/
   Usr_AsignUsrListType ();

   /***** Get teachers range *****/
   Usr_GetListUsrsRange (Usr_TEACHER,
                         Gbl.CurrentCrs.Crs.CrsCod > 0 ? Usr_LIST_USRS_RANGE_THIS_COURSE_ALL : (Gbl.CurrentDeg.Deg.DegCod > 0 ? Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL : Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL));

   /***** Get and order list of teachers *****/
   Usr_GetAndOrderTchsLst (Gbl.Usrs.ListRange,-1L);

   if (Gbl.Usrs.LstTchs.NumUsrs)
     {
      /***** Initialize number of columns *****/
      NumColumns = Usr_NUM_ALL_FIELDS_DATA_TCH;

      /***** Start table with list of teachers *****/
      Lay_StartTableFrame (NULL,NULL,NULL);

      /***** Heading row with column names *****/
      /* Start row */
      fprintf (Gbl.F.Out,"<tr>");
      for (NumCol=(Gbl.Usrs.Listing.WithPhotos ? 0 : 1); NumCol<NumColumns; NumCol++)
         fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                  VERY_LIGHT_BLUE,Txt_NAMES_ALL_FIELDS_DATA_TCH[NumCol][Gbl.Prefs.Language]);

      /* End row */
      fprintf (Gbl.F.Out,"</tr>\n");

      /***** List data of teachers *****/
      for (NumUsr=0; NumUsr<Gbl.Usrs.LstTchs.NumUsrs; NumUsr++)
         Usr_WriteRowTchAllData (NumUsr+1,&(Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.Ind[NumUsr]]));

      /***** End of table *****/
      Lay_EndTableFrame ();
     }
   else	// Gbl.Usrs.LstTchs.NumUsrs == 0
      Lay_ShowAlert (INFO,"No hay profesores.");	// Need translation!

   /***** Free memory for teachers list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
  }

/***************************************************************************/
/*************************** List teachers' data *****************************/
/*****************************************************************************/
// Return number of teachers found

void Usr_ListTchsFound (Usr_ListUsrsRange_t ListUsrsRange,const char *TchQuery)
  {
   extern const char *Txt_USR_TYPES_SINGULAR_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_USR_TYPES_PLURAL_abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];
   extern const char *Txt_NAMES_MAIN_FIELDS_DATA_USR[Usr_NUM_MAIN_FIELDS_DATA_USR][Txt_NUM_LANGUAGES];
   Usr_Gender_t Gender;
   unsigned NumCol;
   unsigned NumUsrsFound;
   unsigned NumUsr;

   /***** Search for teachers *****/
   Usr_GetTchsLst (ListUsrsRange,-1L,TchQuery);
   NumUsrsFound = Gbl.Usrs.LstTchs.NumUsrs;

   /***** Order list of teachers *****/
   Usr_OrderUsrsList (&Gbl.Usrs.LstTchs);

   /***** Start table with list of teachers *****/
   Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

   /***** Write header ******/
   fprintf (Gbl.F.Out,"<tr><td align=\"center\" class=\"TIT_TBL\" colspan=\"%u\">",Usr_NUM_MAIN_FIELDS_DATA_USR);
   Gender = Usr_GetGenderOfUsrsLst (&Gbl.Usrs.LstTchs);
   if (NumUsrsFound == 1)
      fprintf (Gbl.F.Out,"1 %s",Txt_USR_TYPES_SINGULAR_abc[Usr_TEACHER][Gender][Gbl.Prefs.Language]);
   else
      fprintf (Gbl.F.Out,"%u %s",NumUsrsFound,Txt_USR_TYPES_PLURAL_abc[Usr_TEACHER][Gender][Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</td></tr>");

   if (NumUsrsFound)
     {
      /***** Heading row with column names *****/
      /* Start row */
      fprintf (Gbl.F.Out,"<tr>");
      for (NumCol=0; NumCol<Usr_NUM_MAIN_FIELDS_DATA_USR; NumCol++)
         fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                  VERY_LIGHT_BLUE,Txt_NAMES_MAIN_FIELDS_DATA_USR[NumCol][Gbl.Prefs.Language]);
      /* End row */
      fprintf (Gbl.F.Out,"</tr>\n");

      /***** List data of teachers *****/
      for (NumUsr=0; NumUsr<NumUsrsFound; NumUsr++)
        {
         /* Write data of this teacher */
         Usr_WriteRowTchMainData (NumUsr+1,&(Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.Ind[NumUsr]]),false);

         /* Write all the courses given by this teacher */
         fprintf (Gbl.F.Out,"<tr><td align=\"left\" colspan=\"3\"></td>" \
                            "<td align=\"left\" colspan=\"%u\">",Usr_NUM_MAIN_FIELDS_DATA_USR-3);
         Lay_StartRoundFrameTable10 ("frame_white_corner","frame_white",WHITE,"100%");
         Deg_GetAndWriteCoursesOf (Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.Ind[NumUsr]].UsrCod);
         Lay_EndRoundFrameTable10 ("frame_white_corner","frame_white");
         fprintf (Gbl.F.Out,"</td></tr>");
        }
     }

   /***** End of table *****/
   Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

   /***** Free memory for teachers list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
  }

/*****************************************************************************/
/**************************** List administrators ****************************/
/*****************************************************************************/

void Usr_ListDataAdms (void)
  {
   extern const char *Txt_NAMES_MAIN_FIELDS_DATA_ADM[Usr_NUM_MAIN_FIELDS_DATA_ADM][Txt_NUM_LANGUAGES];
   unsigned NumColumns;
   unsigned NumCol;
   unsigned NumUsr;

   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Asign users' listing type depending on current action *****/
   Usr_AsignUsrListType ();

   /***** Get administrators range *****/
   Usr_GetListUsrsRange (Usr_DEG_ADMIN,
                         Gbl.CurrentDeg.Deg.DegCod > 0 ? Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL :
                                                         Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL);

   /***** Form to select range of administrators *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActLstAdm);
   Usr_PutSelectorListUsrsRange (true,Usr_DEG_ADMIN);
   Usr_PutParamListWithPhotos ();
   fprintf (Gbl.F.Out,"</form></div>\n");

   /***** Get and order list of administrators *****/
   Usr_GetAndOrderAdmsLst (Gbl.Usrs.ListRange);
   if (Gbl.Usrs.LstAdms.NumUsrs)
     {
      /****** See the photos? *****/
      fprintf (Gbl.F.Out,"<div align=\"center\">");
      Act_FormStart (ActLstAdm);
      Usr_ParamUsrsRange ();
      Usr_PutCheckboxListWithPhotos ();
      fprintf (Gbl.F.Out,"</form></div>");

      /***** Initialize number of columns *****/
      NumColumns = Usr_NUM_MAIN_FIELDS_DATA_ADM;

      /***** Start table with list of administrators *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /***** Heading row with column names *****/
      /* Start row */
      fprintf (Gbl.F.Out,"<tr>");
      for (NumCol=0; NumCol<NumColumns; NumCol++)
         if (NumCol != 1 || Gbl.Usrs.Listing.WithPhotos)	// Skip photo column if I don't want this column
            fprintf (Gbl.F.Out,"<td align=\"left\" bgcolor=\"#%s\" class=\"TIT_TBL\">%s&nbsp;</td>",
                     VERY_LIGHT_BLUE,Txt_NAMES_MAIN_FIELDS_DATA_ADM[NumCol][Gbl.Prefs.Language]);

      /* End row */
      fprintf (Gbl.F.Out,"</tr>\n");

      /***** List data of administrators *****/
      for (NumUsr=0; NumUsr<Gbl.Usrs.LstAdms.NumUsrs; NumUsr++)
         Usr_WriteRowAdmData (NumUsr+1,&(Gbl.Usrs.LstAdms.Lst[Gbl.Usrs.LstAdms.Ind[NumUsr]]));

      /***** End of table *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
   else	// Gbl.Usrs.LstAdms.NumUsrs == 0
      Lay_ShowAlert (INFO,"No hay administradores.");	// Need translation!

   /***** Free memory for teachers list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstAdms);
  }

/*****************************************************************************/
/******************** Select students to see their records *********************/
/*****************************************************************************/

void Usr_ReqListCardsStdsCrs (void)
  {
   extern const char *Txt_Show_records[Txt_NUM_LANGUAGES];

   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Asign users listing type depending on current action *****/
   Usr_AsignUsrListType ();

   /***** Get list of selected users *****/
   Usr_GetListSelectedUsrs ();

   /***** Form to select groups *****/
   Grp_ShowFormToSelectSeveralGroups (ActReqCrdCrsStd);

   /***** Form to select type of list used for select several users *****/
   Usr_ShowFormsToSelectUsrListType (ActReqCrdCrsStd);

   /***** Set students range *****/
   Gbl.Usrs.ListRange = Usr_LIST_USRS_RANGE_THIS_COURSE_ALL;

   /***** Get and order lists of users from current course *****/
   Usr_GetAndOrderStdsLst (Gbl.Usrs.ListRange,-1L);

   if (Gbl.Usrs.LstStds.NumUsrs)
     {
      /***** Draw a class photo with students of the course *****/
      /* Form start */
      fprintf (Gbl.F.Out,"<div align=\"center\">");
      Act_FormStart (ActSeeCrdCrsStd);
      Grp_PutParamsCodGrps ();

      /* Start table */
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /* Put list of students to select some of them */
      Usr_ListUsersToSelect (Usr_STUDENT);

      /* End of table */
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");

      /* Send button */
      Lay_PutSendButton (Txt_Show_records[Gbl.Prefs.Language]);
      fprintf (Gbl.F.Out,"</form></div>\n");
     }
   else
      Usr_ShowWarningNoUsersFound (Usr_STUDENT);

   /***** Free memory for students list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /***** Free memory used for por the list of users *****/
   Usr_FreeListsEncryptedUsrCods ();

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();
  }

/*****************************************************************************/
/*************** Start form to access to the record of a student ***************/
/*****************************************************************************/

void Usr_StartFormLinkToStdCard (const char *EncryptedUsrCod)
  {
   Act_FormStart (ActSeeCrdCrsStd);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"UsrCodStd\" value=\"%s\" />",EncryptedUsrCod);
  }

/*****************************************************************************/
/*************** Start form to access to the record of a teacher ***************/
/*****************************************************************************/

static void Usr_StartFormLinkToTchCard (const char *EncryptedUsrCod)
  {
   Act_FormStart (ActSeeCrdCrsTch);
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"UsrCodTch\" value=\"%s\" />",EncryptedUsrCod);
  }

/*****************************************************************************/
/********************* Draw the records of the students **********************/
/*****************************************************************************/

void Usr_ListCardsStdsCrs (void)
  {
   extern const char *Txt_You_must_select_one_ore_more_students[Txt_NUM_LANGUAGES];
   unsigned NumUsrs = 0,
	    EspacioEntreFichas = (Gbl.CurrentAct == ActPrnCrdCrsStd) ? 5 : 0;
   struct UsrData UsrDat;
   const char *Ptr;
   Rec_RecordViewType_t TypeOfView = (Gbl.CurrentAct == ActSeeCrdCrsStd) ? RECORD_LIST : RECORD_PRINT;

   /***** Asign users listing type depending on current action *****/
   Usr_AsignUsrListType ();

   /***** Get parameter with number of user records per page (only for printing) *****/
   if (Gbl.CurrentAct == ActPrnCrdCrsStd)
      Usr_GetParamCardsPerPage ();

   /***** Get list of selected students *****/
   Usr_GetListSelectedUsrs ();

   /* Check the number of students cuyas records se van a mostrar */
   if (Usr_CountNumUsrsInEncryptedList () == 0)	// If no students selected...
     {						// ...write warning notice
      Lay_ShowAlert (WARNING,Txt_You_must_select_one_ore_more_students[Gbl.Prefs.Language]);
      Usr_ReqListCardsStdsCrs ();		// ...show again the form
      return;
     }

   /***** Get list of fields of records in current course *****/
   Rec_GetListRecordFieldsInCurrentCrs ();

   if (Gbl.CurrentAct == ActSeeCrdCrsStd)
     {
      /* Link to print view */
      fprintf (Gbl.F.Out,"<div align=\"center\">");
      Act_FormStart (ActPrnCrdCrsStd);
      Usr_HiddenParUsrCodAll (ActPrnCrdCrsStd,Gbl.Usrs.Select.All);
      Usr_ShowLinkToPrintPreviewOfCards ();
      fprintf (Gbl.F.Out,"</form></div>");
     }
   fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"%u\" cellpadding=\"0\">\n",EspacioEntreFichas);

   /***** Initialize structure with user's data *****/
   Usr_UsrDataConstructor (&UsrDat);

   /***** List the records *****/
   Ptr = Gbl.Usrs.Select.All;
   while (*Ptr)
     {
      Str_GetNextStrUntilSeparParamMult (&Ptr,UsrDat.EncryptedUsrCod,Cry_LENGTH_ENCRYPTED_STR_SHA256_BASE64);
      Usr_GetUsrCodFromEncryptedUsrCod (&UsrDat);
      if (Usr_ChkUsrCodAndGetAllUsrDataFromUsrCod (&UsrDat))		// Get from the database the data of the student
         if (Usr_CheckIfUsrBelongsToCrs (UsrDat.UsrCod,Gbl.CurrentCrs.Crs.CrsCod))
           {
            UsrDat.Accepted = Usr_GetIfUserHasAccepted (UsrDat.UsrCod);

            /* Common record */
            fprintf (Gbl.F.Out,"<tr");
            if (Gbl.CurrentAct == ActPrnCrdCrsStd && NumUsrs != 0 && (NumUsrs % Gbl.Usrs.Listing.CrdsPerPag) == 0)
               fprintf (Gbl.F.Out," style=\"page-break-before:always\"");
            fprintf (Gbl.F.Out,"><td align=\"center\">");
            Rec_ShowCommonRecord (TypeOfView,&UsrDat,NULL);
            fprintf (Gbl.F.Out,"</td></tr>");

            /* Record of the course */
            if (Gbl.CurrentCrs.Records.LstFields.Num) // There are fields in the record particulares a the course...
              {
               fprintf (Gbl.F.Out,"<tr><td align=\"center\">");
               Rec_ShowCrsRecord (TypeOfView,&UsrDat);
	       fprintf (Gbl.F.Out,"</td></tr><tr><td align=\"left\" class=\"DAT_SMALL\">&nbsp;</td></tr>");
              }

            NumUsrs++;
           }
     }

   /***** End of table *****/
   fprintf (Gbl.F.Out,"</table></div>\n");

   /***** Free memory used for user's data *****/
   Usr_UsrDataDestructor (&UsrDat);

   /***** Liberar list of fields of records *****/
   if (Gbl.Usrs.Listing.Type == Usr_USER_LIST_RECORDS_COURSE_STUDENTS)
      Rec_FreeListFields ();

   /***** Free memory used for por the list of users *****/
   Usr_FreeListsEncryptedUsrCods ();
  }

/*****************************************************************************/
/********************** Draw the records of the teachers *********************/
/*****************************************************************************/

void Usr_ListCardsTchsCrs (void)
  {
   unsigned NumUsr,NumUsrs = 0,
	    EspacioEntreFichas = (Gbl.CurrentAct == ActPrnCrdCrsTch) ? 5 : 0;
   struct UsrData *UsrDat;
   Rec_RecordViewType_t TypeOfView = (Gbl.CurrentAct == ActSeeCrdCrsTch) ? RECORD_LIST : RECORD_PRINT;

   /***** Asign users listing type depending on current action *****/
   Usr_AsignUsrListType ();

   /***** Get parameter with number of user records per page (only for printing) *****/
   if (Gbl.CurrentAct == ActPrnCrdCrsTch)
      Usr_GetParamCardsPerPage ();

   /***** Get parameter with teacher's code when viewing the record of one teacher *****/
   Usr_GetParamOtherUsrCodEncrypted ();

   /***** Get and order list of students in current course *****/
   Gbl.Usrs.ListRange = (Gbl.Usrs.Other.UsrDat.UsrCod > 0) ? Usr_LIST_USRS_RANGE_THIS_COURSE_ONLY_ONE : Usr_LIST_USRS_RANGE_THIS_COURSE_ALL;
   Usr_GetAndOrderTchsLst (Gbl.Usrs.ListRange,Gbl.Usrs.Other.UsrDat.UsrCod);

   if (Gbl.Usrs.LstTchs.NumUsrs)
     {
      if (Gbl.CurrentAct == ActSeeCrdCrsTch)
        {
         /* Link to print view */
         fprintf (Gbl.F.Out,"<div align=\"center\">");
         Act_FormStart (ActPrnCrdCrsTch);
         if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ONLY_ONE)
            Usr_PutParamOtherUsrCodEncrypted (Gbl.Usrs.Other.UsrDat.EncryptedUsrCod);
         Usr_ShowLinkToPrintPreviewOfCards ();
         fprintf (Gbl.F.Out,"</form></div>");
        }

      /***** List the records *****/
      fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"%u\" cellpadding=\"0\">",EspacioEntreFichas);
      for (NumUsr=0; NumUsr<Gbl.Usrs.LstTchs.NumUsrs; NumUsr++)
        {
         UsrDat = &(Gbl.Usrs.LstTchs.Lst[Gbl.Usrs.LstTchs.Ind[NumUsr]]);

         /* Common record */
         fprintf (Gbl.F.Out,"<tr");
         if (Gbl.CurrentAct == ActPrnCrdCrsTch && NumUsrs != 0 && (NumUsrs % Gbl.Usrs.Listing.CrdsPerPag) == 0)
            fprintf (Gbl.F.Out," style=\"page-break-before:always\"");
         fprintf (Gbl.F.Out,"><td align=\"center\">");
         Rec_ShowCommonRecord (TypeOfView,UsrDat,NULL);
         fprintf (Gbl.F.Out,"</td></tr>");

         NumUsrs++;
        }
      fprintf (Gbl.F.Out,"</table></div>\n");
     }
   else	// Gbl.Usrs.LstTchs.NumUsrs == NULL
      Lay_ShowAlert (INFO,"No hay profesores.");	// Need translation!

   /***** Free memory for teachers list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
  }

/*****************************************************************************/
/*** Get parameter with number of user records per page (only for printing) ****/
/*****************************************************************************/

void Usr_GetParamCardsPerPage (void)
  {
   char UnsignedStr[10+1];

   Par_GetParam (Par_PARAM_SINGLE,"CrdsPerPag",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.Usrs.Listing.CrdsPerPag) != 1)
      Lay_ShowErrorAndExit ("Number of rows per page is missing.");
  }

/*****************************************************************************/
/***** Get and update type of list, number of columns in class photo and preference about view photos *****/
/*****************************************************************************/

void Usr_GetAndUpdatePrefsAboutUsrList (void)
  {
    /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
    Usr_GetAndUpdateUsrListType ();
    Usr_GetAndUpdateColsClassPhoto ();
    Usr_GetAndUpdatePrefAboutListWithPhotos ();
  }

/*****************************************************************************/
/****************** Get from form the type of users' list ********************/
/*****************************************************************************/

static void Usr_GetAndUpdateUsrListType (void)
  {
   /***** Get type of list used to select users from form *****/
   if (Usr_GetUsrListTypeFromForm ())
      /* Save in the database the type of list preferred by me */
      Usr_UpdateMyUsrListTypeInDB ();
   else
      /* If parameter can't be retrieved from, get my preference from database */
      Usr_GetMyUsrListTypeFromDB ();
  }

/*****************************************************************************/
/************* Put a hidden parameter with the users' list type **************/
/*****************************************************************************/

void Usr_PutParamUsrListType (Usr_ShowUsrsType_t ListType)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"UsrListType\" value=\"%u\" />",(unsigned) ListType);
  }

/*****************************************************************************/
/****************** Get from form the type of users' list ********************/
/*****************************************************************************/

static bool Usr_GetUsrListTypeFromForm (void)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Gbl.Usrs.Me.ListType = Usr_SHOW_USRS_TYPE_DEFAULT;

   /***** Get param with number of users per row *****/
   Par_GetParam (Par_PARAM_SINGLE,"UsrListType",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) != 1)
      return false;
   if (UnsignedNum >= NUM_USR_LIST_TYPES)
      return false;
   Gbl.Usrs.Me.ListType = UnsignedNum;
   return true;
  }

/*****************************************************************************/
/************** Get my preference about type of users' list ******************/
/*****************************************************************************/

static void Usr_GetMyUsrListTypeFromDB (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;
   Usr_ShowUsrsType_t ListType;

   /***** Get type of listing of users from database *****/
   sprintf (Query,"SELECT UsrListType FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get type of listing of users");

   if (NumRows == 1)		// Should be one only row
     {
      /* Get type of users' listing used to select some of them */
      Gbl.Usrs.Me.ListType = Usr_SHOW_USRS_TYPE_DEFAULT;
      row = mysql_fetch_row (mysql_res);
      if (row[0])
         for (ListType = (Usr_ShowUsrsType_t) 0; ListType < NUM_USR_LIST_TYPES; ListType++)
            if (!strcasecmp (row[0],StringsUsrListTypeInDB[ListType]))
              {
	       Gbl.Usrs.Me.ListType = ListType;
               break;
              }
     }
   else if (NumRows == 0)	// If I am an administrator or superuser and I don't belong to current course, then the result will be the default
      Gbl.Usrs.Me.ListType = Usr_SHOW_USRS_TYPE_DEFAULT;
   else	// Error in databse: more than one row for a user in course
      Lay_ShowErrorAndExit ("Error when getting type of listing of users.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/***************** Save my preference about type of users' list *************/
/****************************************************************************/

static void Usr_UpdateMyUsrListTypeInDB (void)
  {
   char Query[512];

   /***** Update type of users listing *****/
   sprintf (Query,"UPDATE crs_usr SET UsrListType='%s' WHERE CrsCod='%ld' AND UsrCod='%ld'",
            StringsUsrListTypeInDB[Gbl.Usrs.Me.ListType],Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   DB_QueryUPDATE (Query,"can not update type of listing");
  }

/*****************************************************************************/
/************* Get and update number of columns in class photo ***************/
/*****************************************************************************/

void Usr_GetAndUpdateColsClassPhoto (void)
  {
   /***** Get the number of columns in class photo from form *****/
   if (Usr_GetParamColsClassPhotoFromForm ())
      /***** Save the number of columns into the database  *****/
      Usr_UpdateMyColsClassPhotoInDB ();
   else
      /***** If parameter can't be retrieved from form, get my preference from database *****/
      Usr_GetMyColsClassPhotoFromDB ();
  }

/*****************************************************************************/
/****** Put a hidden parameter with the number of colums in class photo ******/
/*****************************************************************************/

void Usr_PutParamColsClassPhoto (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"ColsClassPhoto\" value=\"%u\" />",Gbl.Usrs.ClassPhoto.Cols);
  }

/*****************************************************************************/
/************* Get from form the number of colums in class photo *************/
/*****************************************************************************/

static bool Usr_GetParamColsClassPhotoFromForm (void)
  {
   char UnsignedStr[10+1];

   /***** Get parameter with number of users per row *****/
   Par_GetParam (Par_PARAM_SINGLE,"ColsClassPhoto",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&Gbl.Usrs.ClassPhoto.Cols) != 1)
     {
      Gbl.Usrs.ClassPhoto.Cols = Usr_CLASS_PHOTO_COLS_DEF;
      return false;
     }
   if (Gbl.Usrs.ClassPhoto.Cols < 1 ||
       Gbl.Usrs.ClassPhoto.Cols > Usr_CLASS_PHOTO_COLS_MAX)
      Gbl.Usrs.ClassPhoto.Cols = Usr_CLASS_PHOTO_COLS_DEF;
   return true;
  }

/*****************************************************************************/
/** Get my prefs. about number of colums in class photo for current course ***/
/*****************************************************************************/

static void Usr_GetMyColsClassPhotoFromDB (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** If no user logged or not course selected... *****/
   if (Gbl.Usrs.Me.UsrDat.UsrCod < 0 || Gbl.CurrentCrs.Crs.CrsCod < 0)
     {
      Gbl.Usrs.ClassPhoto.Cols = Usr_CLASS_PHOTO_COLS_DEF;
      return;
     }

   /***** Get number of columns in class photo from database *****/
   sprintf (Query,"SELECT ColsClassPhoto FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not get number of columns in class photo");

   if (NumRows == 1)		// Should be one only row
     {
      /* Get number of columns in class photo */
      Gbl.Usrs.ClassPhoto.Cols = Usr_CLASS_PHOTO_COLS_DEF;
      row = mysql_fetch_row (mysql_res);
      if (row[0])
         if (sscanf (row[0],"%u",&Gbl.Usrs.ClassPhoto.Cols) == 1)
            if (Gbl.Usrs.ClassPhoto.Cols < 1 ||
                Gbl.Usrs.ClassPhoto.Cols > Usr_CLASS_PHOTO_COLS_MAX)
	       Gbl.Usrs.ClassPhoto.Cols = Usr_CLASS_PHOTO_COLS_DEF;
     }
   else if (NumRows == 0)	// If I am an administrator or superuser and I don't belong to current course, then the result will be 0 rows
      Gbl.Usrs.ClassPhoto.Cols = Usr_CLASS_PHOTO_COLS_DEF;
   else	// Error in databse: more than one row for a user in course
      Lay_ShowErrorAndExit ("Error when getting number of columns in class photo.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/* Save my prefs. about number of colums in class photo for current course **/
/****************************************************************************/

static void Usr_UpdateMyColsClassPhotoInDB (void)
  {
   char Query[512];

   if (Gbl.Usrs.Me.UsrDat.UsrCod > 0 && Gbl.CurrentCrs.Crs.CrsCod > 0)
     {
      /***** Update number of colums in class photo for current course *****/
      sprintf (Query,"UPDATE crs_usr SET ColsClassPhoto='%u' WHERE CrsCod='%ld' AND UsrCod='%ld'",
               Gbl.Usrs.ClassPhoto.Cols,Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
      DB_QueryUPDATE (Query,"can not update number of columns in class photo");
     }
  }

/*****************************************************************************/
/********** Get and update preference about photos in users' list ************/
/*****************************************************************************/

static void Usr_GetAndUpdatePrefAboutListWithPhotos (void)
  {
   /***** Get he preference about photos in users' list from form *****/
   if (Usr_GetParamListWithPhotosFromForm ())
      /***** Save preference about photos in users' list into the database  *****/
      Usr_UpdateMyPrefAboutListWithPhotosPhotoInDB ();
   else
      /***** If parameter can't be retrieved from form, get my preference from database *****/
      Usr_GetMyPrefAboutListWithPhotosFromDB ();
  }

/*****************************************************************************/
/** Put a hidden parameter with the preference about photos in users' list ***/
/*****************************************************************************/

void Usr_PutParamListWithPhotos (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"WithPhotosExists\" value=\"Y\" />");
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"WithPhotos\" value=\"%c\" />",
            Gbl.Usrs.Listing.WithPhotos ? 'Y' : 'N');
  }

/*****************************************************************************/
/********* Get from form the preference about photos in users' list **********/
/*****************************************************************************/

static bool Usr_GetParamListWithPhotosFromForm (void)
  {
   char YN[1+1];

   /***** Get if exists parameter with preference about photos in users' list *****/
   Par_GetParam (Par_PARAM_SINGLE,"WithPhotosExists",YN,1);
   if (Str_ConvertToUpperLetter (YN[0]) != 'Y')
     {
      Gbl.Usrs.Listing.WithPhotos = Usr_LIST_WITH_PHOTOS_DEF;
      return false;
     }

   /***** Parameter with preference about photos in users' list exists, so get it *****/
   Par_GetParam (Par_PARAM_SINGLE,"WithPhotos",YN,1);
   Gbl.Usrs.Listing.WithPhotos = (Str_ConvertToUpperLetter (YN[0]) == 'Y');
   return true;
  }

/*****************************************************************************/
/***** Get my preference about photos in users' list for current course ******/
/*****************************************************************************/

static void Usr_GetMyPrefAboutListWithPhotosFromDB (void)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned long NumRows;

   /***** If no user logged or not course selected... *****/
   if (Gbl.Usrs.Me.UsrDat.UsrCod < 0 || Gbl.CurrentCrs.Crs.CrsCod < 0)
     {
      Gbl.Usrs.Listing.WithPhotos = Usr_LIST_WITH_PHOTOS_DEF;
      return;
     }

   /***** Get if listing of users must show photos from database *****/
   sprintf (Query,"SELECT ListWithPhotos FROM crs_usr WHERE CrsCod='%ld' AND UsrCod='%ld'",
            Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
   NumRows = DB_QuerySELECT (Query,&mysql_res,"can not check if listing of users must show photos");

   if (NumRows == 1)		// Should be one only row
     {
      /* Get number of columns in class photo */
      Gbl.Usrs.Listing.WithPhotos = Usr_LIST_WITH_PHOTOS_DEF;
      row = mysql_fetch_row (mysql_res);
      Gbl.Usrs.Listing.WithPhotos = (Str_ConvertToUpperLetter (row[0][0]) != 'N');
     }
   else if (NumRows == 0)	// If I am an administrator or superuser and I don't belong to current course, then the result will be 0 rows
      Gbl.Usrs.Listing.WithPhotos = Usr_LIST_WITH_PHOTOS_DEF;
   else	// Error in databse: more than one row for a user in course
      Lay_ShowErrorAndExit ("Error when checking if listing of users must show photos.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/**** Save my preference about photos in users' list for current course *****/
/****************************************************************************/

static void Usr_UpdateMyPrefAboutListWithPhotosPhotoInDB (void)
  {
   char Query[512];

   if (Gbl.Usrs.Me.UsrDat.UsrCod > 0 && Gbl.CurrentCrs.Crs.CrsCod > 0)
     {
      /***** Update number of colums in class photo for current course *****/
      sprintf (Query,"UPDATE crs_usr SET ListWithPhotos='%c' WHERE CrsCod='%ld' AND UsrCod='%ld'",
               Gbl.Usrs.Listing.WithPhotos ? 'Y' : 'N',Gbl.CurrentCrs.Crs.CrsCod,Gbl.Usrs.Me.UsrDat.UsrCod);
      DB_QueryUPDATE (Query,"can not update your preference about photos in listing");
     }
  }

/*****************************************************************************/
/**************** Show a link to print preview of users' records ***************/
/*****************************************************************************/

static void Usr_ShowLinkToPrintPreviewOfCards (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_print_view[Txt_NUM_LANGUAGES];
   unsigned i;

   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr><td align=\"center\" valign=\"middle\" class=\"%s\">",The_ClassFormul[Gbl.Prefs.Theme]);
   strcpy (Gbl.MsgWindowStatus,Txt_Show_print_view[Gbl.Prefs.Language]);
   Act_LinkFormSubmit (Act_NEW_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
   fprintf (Gbl.F.Out,"<img src=\"%s/print12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                      " %s</acronym></a> (<select name=\"CrdsPerPag\">",
            Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
   for (i=1; i<=10; i++)
     {
      fprintf (Gbl.F.Out,"<option");
      if (i == Gbl.Usrs.Listing.CrdsPerPag)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%u</option>",i);
     }
   fprintf (Gbl.F.Out,"</select>&nbsp;fichas por p&aacute;gina)</td></tr></table>");
  }

/*****************************************************************************/
/******************** how list or class photo of students *******************/
/*****************************************************************************/

void Usr_SeeStudents (void)
  {
   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   Gbl.Usrs.ListRange = Usr_LIST_USRS_RANGE_THIS_COURSE_ALL;

   switch (Gbl.Usrs.Me.UsrTypeLog)
     {
      case Usr_DEG_ADMIN:
      case Usr_SUPERUSER:
         /***** Get students's range *****/
         Usr_GetListUsrsRange (Usr_STUDENT,
                               Gbl.CurrentCrs.Crs.CrsCod > 0 ? Usr_LIST_USRS_RANGE_THIS_COURSE_ALL :
                                                               (Gbl.CurrentDeg.Deg.DegCod > 0 ? Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL :
                                                                                                Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL));

         /***** Form to select range of students *****/
         fprintf (Gbl.F.Out,"<div align=\"center\">");
         Act_FormStart (ActSeeStd);
         Usr_PutParamUsrListType (Gbl.Usrs.Me.ListType);
         Usr_PutParamColsClassPhoto ();
         Usr_PutParamListWithPhotos ();
         Usr_PutSelectorListUsrsRange (true,Usr_STUDENT);
         fprintf (Gbl.F.Out,"</form></div>\n");
         break;
      default:
         Gbl.Usrs.ListRange = Usr_LIST_USRS_RANGE_THIS_COURSE_ALL;
     }

   /***** Form to select groups *****/
   if (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL)
      Grp_ShowFormToSelectSeveralGroups (ActSeeStd);

   /***** Form to select type of list of users *****/
   Usr_ShowFormsToSelectUsrListType (ActSeeStd);

   /***** Get and order list of students in current course *****/
   Usr_GetAndOrderStdsLst (Gbl.Usrs.ListRange,-1L);

   if (Gbl.Usrs.LstStds.NumUsrs)
     {
      /***** Draw the classphoto/list  *****/
      switch (Gbl.Usrs.Me.ListType)
        {
         case Usr_CLASS_PHOTO:
            Usr_DrawClassPhoto (Usr_CLASS_PHOTO_SEE,Usr_STUDENT);
            break;
         case Usr_LIST:
            Usr_ListMainDataStds ();
            break;
        }

      /***** Free memory for students list *****/
      Usr_FreeUsrsList (&Gbl.Usrs.LstStds);
     }
   else
      Usr_ShowWarningNoUsersFound (Usr_STUDENT);

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();
  }

/*****************************************************************************/
/******************** how list or class photo of teachers *******************/
/*****************************************************************************/

void Usr_SeeTeachers (void)
  {
   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Get teachers's range *****/
   Usr_GetListUsrsRange (Usr_TEACHER,
                         Gbl.CurrentCrs.Crs.CrsCod > 0 ? Usr_LIST_USRS_RANGE_THIS_COURSE_ALL :
                                                         (Gbl.CurrentDeg.Deg.DegCod > 0 ? Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL :
                                                                                          Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL));

   /***** Form to select range of teachers *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">");
   Act_FormStart (ActSeeTch);
   Usr_PutParamUsrListType (Gbl.Usrs.Me.ListType);
   Usr_PutParamColsClassPhoto ();
   Usr_PutParamListWithPhotos ();
   Usr_PutSelectorListUsrsRange (true,Usr_TEACHER);
   fprintf (Gbl.F.Out,"</form></div>\n");

   /***** Form to select type of list of users *****/
   Usr_ShowFormsToSelectUsrListType (ActSeeTch);

   /***** Get and order list of teachers *****/
   Usr_GetAndOrderTchsLst (Gbl.Usrs.ListRange,-1L);
   if (Gbl.Usrs.LstTchs.NumUsrs)
     {
      /***** Draw the classphoto/list  *****/
      switch (Gbl.Usrs.Me.ListType)
        {
         case Usr_CLASS_PHOTO:
            Usr_DrawClassPhoto (Usr_CLASS_PHOTO_SEE,Usr_TEACHER);
            break;
         case Usr_LIST:
            Usr_ListMainDataTchs ();
            break;
        }

      /***** Free memory for teachers list *****/
      Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
     }
   else
      Lay_ShowAlert (INFO,"No hay profesores.");	// Need translation!
  }

/*****************************************************************************/
/**************** Draw class photo with students ready to print **************/
/*****************************************************************************/

void Usr_SeeStdClassPhotoPrn (void)
  {
   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /****** Get groups to show ******/
   Grp_GetParCodsSeveralGrpsToShowUsrs ();

   /***** Get and order list of students in current course *****/
   Usr_GetAndOrderStdsLst (Gbl.Usrs.ListRange,-1L);
   if (Gbl.Usrs.LstStds.NumUsrs)
     {
      /***** Draw the students' class photo *****/
      Usr_DrawClassPhoto (Usr_CLASS_PHOTO_PRN,Usr_STUDENT);

      /***** Free memory for students list *****/
      Usr_FreeUsrsList (&Gbl.Usrs.LstStds);
     }
   else
      Usr_ShowWarningNoUsersFound (Usr_STUDENT);

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();
  }

/*****************************************************************************/
/**************** Draw class photo with teachers ready to print **************/
/*****************************************************************************/

void Usr_SeeTchClassPhotoPrn (void)
  {
   /***** Get and update type of list, number of columns in class photo and preference about view photos *****/
   Usr_GetAndUpdatePrefsAboutUsrList ();

   /***** Get teachers range *****/
   Usr_GetListUsrsRange (Usr_TEACHER,
                         Gbl.CurrentCrs.Crs.CrsCod > 0 ? Usr_LIST_USRS_RANGE_THIS_COURSE_ALL :
                                                         (Gbl.CurrentDeg.Deg.DegCod > 0 ? Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL :
                                                                                          Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL));

   /***** Get and order list of teachers *****/
   Usr_GetAndOrderTchsLst (Gbl.Usrs.ListRange,-1L);
   if (Gbl.Usrs.LstTchs.NumUsrs)
     {
      /***** Draw the teachers' class photo *****/
      Usr_DrawClassPhoto (Usr_CLASS_PHOTO_PRN,Usr_TEACHER);

      /***** Free memory for teachers list *****/
      Usr_FreeUsrsList (&Gbl.Usrs.LstTchs);
     }
   else
      Lay_ShowAlert (INFO,"No hay profesores.");	// Need translation!
  }

/*****************************************************************************/
/****************** Draw class photo with students or teachers ***************/
/*****************************************************************************/

void Usr_DrawClassPhoto (Usr_ClassPhotoType_t ClassPhotoType,Usr_UsrType_t UsrTypeInClassPhoto)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Show_print_view[Txt_NUM_LANGUAGES];
   struct ListUsers *LstUsrs;
   unsigned NumUsr;
   bool TRIsOpen = false;
   bool PutLinkToWeb = false,	// Initialized to avoid warning
        IAmLoggedAsTeacherOrAbove = (Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER ||
                                     Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN ||
                                     Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER),
        PutLinkToIndexCard = (IAmLoggedAsTeacherOrAbove && ClassPhotoType == Usr_CLASS_PHOTO_SEE && UsrTypeInClassPhoto == Usr_STUDENT),
        ICanSeePhotos = (IAmLoggedAsTeacherOrAbove || (Gbl.Usrs.Me.MyPhotoExists && Gbl.Usrs.Me.UsrDat.PublicPhoto)),
        ItsMe,
        ShowPhoto,
        ShowData,
        UsrIsTheMsgSender;
   int PhotoWidth = 36,PhotoHeight = 48,	// Default photo width and height
       LengthUsrData = 10; 		// Maximum number of characters of user data
   char PhotoURL[PATH_MAX+1];
   char PathPrivRelPhoto[PATH_MAX+1];	// Relative path to private photo, to calculate average face
   struct UsrData *UsrDat;
   bool CalcAvgPhoto = false;
   char FileNamePublAveragePhoto[PATH_MAX+1];
   char FileNamePublMedianPhoto[PATH_MAX+1];
   char FileNamePhotoNames[PATH_MAX+1];
   FILE *FilePhotoNames = NULL;	// Initialized to avoid warning
   char Command[256]; // Command to call the programa of preprocessing of photos
   int ReturnCode;
   unsigned NumPhotos = 0;
   char BreadcrumbStr[512];

   /***** Show students or teachers? *****/
   LstUsrs = (UsrTypeInClassPhoto == Usr_STUDENT) ? &Gbl.Usrs.LstStds : &Gbl.Usrs.LstTchs;

   /***** Set width and height of photos *****/
   switch (ClassPhotoType)
     {
      case Usr_CLASS_PHOTO_SEE:
         CalcAvgPhoto = (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER && LstUsrs->NumUsrs >= 10 && LstUsrs->NumUsrs <= 100);
	 /* Link to print view */
	 fprintf (Gbl.F.Out,"<div align=\"center\"><table cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"left\" valign=\"middle\">");
	 Act_FormStart ((UsrTypeInClassPhoto == Usr_STUDENT) ? ActPrnStdPho : ActPrnTchPho);
         if (UsrTypeInClassPhoto == Usr_STUDENT)
            Grp_PutParamsCodGrps ();
         else
            Usr_ParamUsrsRange ();
         strcpy (Gbl.MsgWindowStatus,Txt_Show_print_view[Gbl.Prefs.Language]);
         Act_LinkFormSubmit (Act_NEW_WINDOW,Gbl.MsgWindowStatus,The_ClassFormul[Gbl.Prefs.Theme]);
         fprintf (Gbl.F.Out,"<img src=\"%s/print12x12.gif\" width=\"12\" height=\"12\" alt=\"%s\" />" \
                            " %s</acronym></a></form></td></tr></table></div>",
                  Gbl.Prefs.IconsURL,Gbl.MsgWindowStatus,Gbl.MsgWindowStatus);
	 Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);
	 PhotoWidth = 36; PhotoHeight = 48; LengthUsrData = 10;
         Lay_WriteHeaderClassPhoto (Gbl.Usrs.ClassPhoto.Cols,false,true,
                                  (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL || Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL) ? Gbl.CurrentDeg.Deg.InsCod : -1L,
                                  (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL || Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL) ? Gbl.CurrentDeg.Deg.DegCod : -1L,
                                   Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL                                                           ? Gbl.CurrentCrs.Crs.CrsCod : -1L);
	 break;
      case Usr_CLASS_PHOTO_PRN:
         CalcAvgPhoto = (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER && LstUsrs->NumUsrs >= 10);
         Lay_StartTableFrame (DARK_BLUE,WHITE,NULL);
	 PhotoWidth = 36; PhotoHeight = 48; LengthUsrData = 15;
         Lay_WriteHeaderClassPhoto (Gbl.Usrs.ClassPhoto.Cols,true,true,
                                  (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL || Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL) ? Gbl.CurrentDeg.Deg.InsCod : -1L,
                                  (Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL || Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL) ? Gbl.CurrentDeg.Deg.DegCod : -1L,
                                   Gbl.Usrs.ListRange == Usr_LIST_USRS_RANGE_THIS_COURSE_ALL                                                           ? Gbl.CurrentCrs.Crs.CrsCod : -1L);
	 break;
      case Usr_CLASS_PHOTO_SEL:
	 PhotoWidth = 18; PhotoHeight = 24; LengthUsrData = 10;
	 break;
     }

   if (CalcAvgPhoto)
     {
      /***** Creates public directory for average photo if not exists *****/
      Fil_CreateDirIfNotExists (Cfg_PATH_REL_PHOTOS_TMP_PUBLIC);

      /***** Remove old files in public directory *****/
      Brw_RemoveOldTmpFiles (Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,false);

      /***** Creates private directory for lists of users' photos if not exists *****/
      Fil_CreateDirIfNotExists (Cfg_PATH_REL_PHOTOS_TMP_PRIVATE);

      /***** Remove old private files used for lists *****/
      Brw_RemoveOldTmpFiles (Cfg_PATH_REL_PHOTOS_TMP_PRIVATE,false);

      /***** Make unique names for text file with photo paths and for file with average photo *****/
      sprintf (FileNamePublAveragePhoto,"%s/%s_media.jpg",Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted);
      sprintf (FileNamePublMedianPhoto,"%s/%s_mediana.jpg",Cfg_PATH_REL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted);
      sprintf (FileNamePhotoNames,"%s/%s.txt",Cfg_PATH_REL_PHOTOS_TMP_PRIVATE,Gbl.UniqueNameEncrypted);	// !!! Don't make it public !!!
      if ((FilePhotoNames = fopen (FileNamePhotoNames,"wt")) == NULL)
	 Lay_ShowErrorAndExit ("Can not create file to compute average photo.");
     }

   /***** Loop for showing users photos, names and place of birth *****/
   for (NumUsr=0; NumUsr<LstUsrs->NumUsrs; )
     {
      if ((NumUsr % Gbl.Usrs.ClassPhoto.Cols) == 0)
        {
         fprintf (Gbl.F.Out,"<tr>\n");
         TRIsOpen = true;
        }

      UsrDat = &(LstUsrs->Lst[LstUsrs->Ind[NumUsr]]);

      ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == UsrDat->UsrCod);
      ShowData = ItsMe || UsrDat->Accepted || Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN || Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER;

      /***** Begin user's cell *****/
      fprintf (Gbl.F.Out,"<td align=\"center\" valign=\"bottom\" class=\"ORL\"");
      UsrIsTheMsgSender = false;
      if (ClassPhotoType == Usr_CLASS_PHOTO_SEL)
	 if (Gbl.Usrs.Select.All[0])	// I have selected one or more users
	    if (!strcmp (Gbl.Usrs.Select.All,UsrDat->EncryptedUsrCod))	// I try to send a message to one unique user (Gbl.Usrs.Select.All holds a unique user)
              {
               UsrIsTheMsgSender = true;
	       fprintf (Gbl.F.Out," bgcolor=\"#%s\"",LIGHT_GREEN);
              }
      fprintf (Gbl.F.Out,">");

      /***** If we have to put a link to user's index record... *****/
      if (PutLinkToIndexCard)
        {
	 Usr_StartFormLinkToStdCard (UsrDat->EncryptedUsrCod);
         sprintf (Gbl.MsgWindowStatus,"Ver ficha de %s",UsrDat->FullName);	// Need translation!
         Act_LinkFormSubmit (Act_MAIN_WINDOW,Gbl.MsgWindowStatus,"ORL");
        }
      else
         PutLinkToWeb = ShowData && Str_URLLooksValid (UsrDat->WWW);

      /***** Checkbox to select this user in case of message *****/
      if (ClassPhotoType == Usr_CLASS_PHOTO_SEL)
         Usr_PutCheckboxToSelectUser (UsrTypeInClassPhoto,UsrDat->EncryptedUsrCod,UsrIsTheMsgSender);

      /***** Put a link to user's personal web page *****/
      if (PutLinkToWeb)
         fprintf (Gbl.F.Out,"<a href=\"%s\" target=\"_blank\" class=\"ORL\">",UsrDat->WWW);

      /***** Add photo to file for average face calculation *****/
      if (CalcAvgPhoto)
         if (Usr_CheckIfPrivPhotoExists (UsrDat,PathPrivRelPhoto))
           {
            fprintf (FilePhotoNames,"%s\n",PathPrivRelPhoto);
            NumPhotos++;
           }

      /***** Show photo *****/
      ShowPhoto = Usr_ShowUsrPhotoIsAllowed (UsrDat,PhotoURL);
      Usr_ShowUsrPhoto (UsrDat,ShowPhoto ? PhotoURL : NULL,PhotoWidth,PhotoHeight,Actions[Gbl.CurrentAct].BrowserWindow == Act_MAIN_WINDOW);

      /***** Photo foot *****/
      fprintf (Gbl.F.Out,"<br />");

      /* Name */
      if (UsrDat->FirstName[0])
        {
         Str_LimitLengthHTMLStr (UsrDat->FirstName,LengthUsrData);
         fprintf (Gbl.F.Out,"%s",UsrDat->FirstName);
        }
      else
	 fprintf (Gbl.F.Out,"&nbsp;");
      fprintf (Gbl.F.Out,"<br />");
      if (UsrDat->Surname1[0])
        {
         Str_LimitLengthHTMLStr (UsrDat->Surname1,LengthUsrData);
         fprintf (Gbl.F.Out,"%s",UsrDat->Surname1);
        }
      else
	 fprintf (Gbl.F.Out,"&nbsp;");
      fprintf (Gbl.F.Out,"<br />");
      if (UsrDat->Surname2[0])
        {
         Str_LimitLengthHTMLStr (UsrDat->Surname2,LengthUsrData);
         fprintf (Gbl.F.Out,"%s",UsrDat->Surname2);
        }
      else
         fprintf (Gbl.F.Out,"&nbsp;");

      /* Origin place */
      if (UsrTypeInClassPhoto == Usr_STUDENT &&
          (ClassPhotoType == Usr_CLASS_PHOTO_SEE ||
           ClassPhotoType == Usr_CLASS_PHOTO_PRN))
        {
         fprintf (Gbl.F.Out,"<br />");
	 if (UsrDat->Std.OriginPlace[0])
	   {
	    fprintf (Gbl.F.Out,"<em>");
            if (ShowData)
              {
	       Str_LimitLengthHTMLStr (UsrDat->Std.OriginPlace,LengthUsrData);
	       fprintf (Gbl.F.Out,"%s",UsrDat->Std.OriginPlace);
              }
            else
	       fprintf (Gbl.F.Out,"-");
	    fprintf (Gbl.F.Out,"</em>");            
	   }
	 else // There's no origin place
	    fprintf (Gbl.F.Out,"&nbsp;");
        }

      /***** End of user's cell *****/
      if (PutLinkToIndexCard)
         fprintf (Gbl.F.Out,"</acronym></a></form>");
      else if (PutLinkToWeb)
         fprintf (Gbl.F.Out,"</a>");
      fprintf (Gbl.F.Out,"</td>");

      if ((++NumUsr % Gbl.Usrs.ClassPhoto.Cols) == 0)
        {
         fprintf (Gbl.F.Out,"</tr>\n");
         TRIsOpen = false;
        }
     }
   if (TRIsOpen)
      fprintf (Gbl.F.Out,"</tr>\n");

   /***** Write message in case of the user can not view the photos of others *****/
   if (!ICanSeePhotos && UsrTypeInClassPhoto == Usr_STUDENT &&
       (ClassPhotoType == Usr_CLASS_PHOTO_SEE ||
        ClassPhotoType == Usr_CLASS_PHOTO_SEL))
     {
      fprintf (Gbl.F.Out," <tr><td colspan=\"%d\" align=\"center\" class=\"MSJ_AVISO\">" \
	                 "(para ver las fotos de otros tiene que ",	// Need translation!
	       Gbl.Usrs.ClassPhoto.Cols);
      if (Gbl.Usrs.Me.MyPhotoExists)
        {
         Act_GetBreadcrumbStrForAction (ActEdiPrf,Gbl.Prefs.Language,true,BreadcrumbStr);
         fprintf (Gbl.F.Out,"permitir que otros vean la suya; puede hacerlo en %s",BreadcrumbStr);	// Need translation!
        }
      else
        {
         Act_GetBreadcrumbStrForAction (ActReqMyPho,Gbl.Prefs.Language,true,BreadcrumbStr);
         fprintf (Gbl.F.Out,"enviar la suya; puede hacerlo en %s",BreadcrumbStr);	// Need translation!
        }
      fprintf (Gbl.F.Out,")</td></tr>");
     }

   /***** End table *****/
   switch (ClassPhotoType)
     {
      case Usr_CLASS_PHOTO_SEE:
         Lay_WriteFootClassPhoto ();
         Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
         break;
      case Usr_CLASS_PHOTO_PRN:
         Lay_WriteFootClassPhoto ();
         Lay_EndTableFrame ();
         break;
      default:
	 break;
     }

   if (CalcAvgPhoto)
     {
      fclose (FilePhotoNames);

      if (NumPhotos >= 10)
        {
         /***** Call to program to calculate average photo *****/
         sprintf (Command,"nice -n 19 ./foto_promedio %s %s",FileNamePhotoNames,FileNamePublAveragePhoto);

         ReturnCode = system (Command);
         if (ReturnCode == -1)
            Lay_ShowErrorAndExit ("Error when running command to compute average photo.");

         /* Write message depending on return code */
         if (WEXITSTATUS(ReturnCode))
            Lay_ShowErrorAndExit ("The average photo has not been computed successfully.");

         fprintf (Gbl.F.Out,"<div align=\"center\"><img src=\"%s/%s_media.jpg\" width=\"%u\" height=\"%u\" /></div>\n",
                  Cfg_URL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted,Usr_PHOTO_REAL_WIDTH,Usr_PHOTO_REAL_HEIGHT);

         /***** Call to program to calculate median photo *****/
         sprintf (Command,"nice -n 19 ./foto_mediana %s %s",FileNamePhotoNames,FileNamePublMedianPhoto);

         ReturnCode = system (Command);
         if (ReturnCode == -1)
            Lay_ShowErrorAndExit ("Error when running command to compute average photo.");

         /* Write message depending on return code */
         if (WEXITSTATUS(ReturnCode))
            Lay_ShowErrorAndExit ("The average photo has not been computed successfully.");

         fprintf (Gbl.F.Out,"<div align=\"center\"><img src=\"%s/%s_mediana.jpg\" width=\"%u\" height=\"%u\" /></div>\n",
                  Cfg_URL_PHOTOS_TMP_PUBLIC,Gbl.UniqueNameEncrypted,Usr_PHOTO_REAL_WIDTH,Usr_PHOTO_REAL_HEIGHT);
        }
     }
  }

/*****************************************************************************/
/** Put a selector to choice between ranges when getting users for listing ***/
/*****************************************************************************/

void Usr_PutSelectorListUsrsRange (bool SendOnChange,Usr_UsrType_t UsrTypeToList)
  {
   extern const char *Txt_any_course_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_no_course_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_degree[Txt_NUM_LANGUAGES];
   extern const char *Txt_course[Txt_NUM_LANGUAGES];
   Usr_ListUsrsRange_t ListUsrsRange;

   fprintf (Gbl.F.Out,"<select name=\"LocationRange\"");
   if (SendOnChange)
      fprintf (Gbl.F.Out," onchange=\"javascript:document.forms[%d].submit();\"",Gbl.NumForm);
   fprintf (Gbl.F.Out,">");

   for (ListUsrsRange=(Usr_ListUsrsRange_t) 0; ListUsrsRange<Usr_NUM_LIST_USRS_RANGES; ListUsrsRange++)
     {
      /* Don't put forbidden options in selectable list */
      if (!(ListUsrsRangesAllowed[ListUsrsRange][UsrTypeToList] & (1 << Gbl.Usrs.Me.UsrTypeLog)))
         continue;
      switch (ListUsrsRange)
        {
         case Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL:
            break;
         case Usr_LIST_USRS_RANGE_ANY_DEGREE_SEARCH:
            continue;
         case Usr_LIST_USRS_RANGE_NO_COURSE_ALL:
            break;
         case Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL:
            if (Gbl.CurrentDeg.Deg.DegCod < 0)	// No degree selected
               continue;
            break;
         case Usr_LIST_USRS_RANGE_THIS_DEGREE_SEARCH:
            continue;
         case Usr_LIST_USRS_RANGE_THIS_COURSE_ALL:
            if (Gbl.CurrentCrs.Crs.CrsCod < 0)	// No course selected
               continue;
            break;
         case Usr_LIST_USRS_RANGE_THIS_COURSE_ONLY_ONE:
            continue;
        }

      /* Write allowed option */
      fprintf (Gbl.F.Out,"<option value=\"%u\"",(unsigned) ListUsrsRange);
      if (Gbl.Usrs.ListRange == ListUsrsRange)
         fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">");
      switch (ListUsrsRange)
        {
         case Usr_LIST_USRS_RANGE_ANY_DEGREE_ALL:
            fprintf (Gbl.F.Out,"%s",Txt_any_course_degree[Gbl.Prefs.Language]);
            break;
         case Usr_LIST_USRS_RANGE_NO_COURSE_ALL:
            fprintf (Gbl.F.Out,"%s",Txt_no_course_degree[Gbl.Prefs.Language]);
            break;
         case Usr_LIST_USRS_RANGE_THIS_DEGREE_ALL:
            fprintf (Gbl.F.Out,"%s: %s",Txt_degree[Gbl.Prefs.Language],Gbl.CurrentDeg.Deg.FullName);
            break;
         case Usr_LIST_USRS_RANGE_THIS_COURSE_ALL:
            fprintf (Gbl.F.Out,"%s: %s",Txt_course[Gbl.Prefs.Language],Gbl.CurrentCrs.Crs.FullName);
            break;
         default:
            break;
        }
      fprintf (Gbl.F.Out,"</option>");
     }

   fprintf (Gbl.F.Out,"</select>");
  }

/*****************************************************************************/
/*************** Put hidden parameter with users' range  *********************/
/*****************************************************************************/

static void Usr_ParamUsrsRange (void)
  {
   fprintf (Gbl.F.Out,"<input type=\"hidden\" name=\"LocationRange\" value=\"%u\" />",
            (unsigned) Gbl.Usrs.ListRange);
  }

/*****************************************************************************/
/************************* Get users range for listing ***********************/
/*****************************************************************************/

void Usr_GetListUsrsRange (Usr_UsrType_t UsrTypeToList,Usr_ListUsrsRange_t DefaultLocationRange)
  {
   char UnsignedStr[10+1];
   unsigned UnsignedNum;

   Gbl.Usrs.ListRange = DefaultLocationRange;

   Par_GetParam (Par_PARAM_SINGLE,"LocationRange",UnsignedStr,10);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      if (UnsignedNum < Usr_NUM_LIST_USRS_RANGES)
         if (ListUsrsRangesAllowed[UnsignedNum][UsrTypeToList] & (1 << Gbl.Usrs.Me.UsrTypeLog))	// If I have permission to see this range of users
            Gbl.Usrs.ListRange = (Usr_ListUsrsRange_t) UnsignedNum;
  }

/*****************************************************************************/
/***************** Write selector of columns in class photo ******************/
/*****************************************************************************/

void Usr_PutSelectorNumColsClassPhoto (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_columns[Txt_NUM_LANGUAGES];   
   unsigned Cols;

   /***** Start selector *****/
   fprintf (Gbl.F.Out,"<table cellspacing=\"0\" cellpadding=\"0\"><tr>" \
                      "<td align=\"left\" valign=\"middle\">" \
                      "<select name=\"ColsClassPhoto\" onchange=\"javascript:document.forms[%d].submit();\">",
            Gbl.NumForm);

   /***** Put a row in selector for every number from 1 to Usr_CLASS_PHOTO_COLS_MAX *****/
   for (Cols=1; Cols<=Usr_CLASS_PHOTO_COLS_MAX; Cols++)
     {
      fprintf (Gbl.F.Out,"<option");
      if (Cols == Gbl.Usrs.ClassPhoto.Cols)
	 fprintf (Gbl.F.Out," selected=\"selected\"");
      fprintf (Gbl.F.Out,">%u</option>",Cols);
     }

   /***** End selector *****/
   fprintf (Gbl.F.Out,"</select></td>" \
                      "<td align=\"left\" valign=\"middle\" class=\"%s\">&nbsp;%s</td>" \
                      "</tr></table>",
            The_ClassFormul[Gbl.Prefs.Theme],Txt_columns[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/***** Register/remove users (taken from a list) in/from current course *****/
/****************************************************************************/

void Usr_RegisterRemoveUsrsCrs (void)
  {
   extern const char *The_ClassTitle[The_NUM_THEMES];
   extern const char *Txt_Register_remove[Txt_NUM_LANGUAGES];

   /***** Form to send students to be registered / removed *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">\n");
   Act_FormStart (ActRecFrmMdfStdCrs);

   /***** Step 1: List of students to be registered / removed *****/
   fprintf (Gbl.F.Out,"<div align=\"left\" class=\"%s\">" \
                      "<br />Paso 1: Proporcione una lista de usuarios</div>",	// Need translation!
            The_ClassTitle[Gbl.Prefs.Theme]);

   /* Option a: get students from official lists */
   fprintf (Gbl.F.Out,"<div align=\"left\" class=\"%s\">" \
                      "<br />Opci&oacute;n a): Obtenga alumnos de las listas oficiales<br />&nbsp;</div>",	// Need translation!
            The_ClassTitle[Gbl.Prefs.Theme]);
   Lay_WriteTitle ("Seleccione los grupos de alumnos que desee dar de alta/baja en esta asignatura.");	// Need translation!
   Imp_ListMyImpGrpsAndStdsForm ();

   /* Option b: get students' IDs from pasted text */
   fprintf (Gbl.F.Out,"<div align=\"left\" class=\"%s\">" \
                      "<br />Opci&oacute;n b): Escriba o pegue una lista de DNI<br />&nbsp;</div>",	// Need translation!
            The_ClassTitle[Gbl.Prefs.Theme]);
   Lay_WriteTitle ("Escriba o pegue una lista con DNI de usuarios que desee dar de alta/baja en esta asignatura." \
                   " Los DNI pueden ir separados por cadenas de caracteres arbitrarias" \
                   " (por ej. nombres, apellidos, espacios, etc.)," \
                   " siempre que las cadenas separadoras no contengan muchos n&uacute;meros juntos" \
                   " (por ej. no deben aparecer n&uacute;meros de tel&eacute;fono)." \
                   " SWAD s&oacute;lo tomar los DNI, ignorando el resto del texto.");	// Need translation!
   Usr_PutAreaToEnterStudentsIDs ();

   /***** Step 2: Select type of user to register/remove to/from current course *****/
   fprintf (Gbl.F.Out,"<div align=\"left\" class=\"%s\">" \
                      "<br />Paso 2: Seleccione el tipo de usuario a dar de alta/baja<br />&nbsp;</div>",	// Need translation!
            The_ClassTitle[Gbl.Prefs.Theme]);
   Usr_PutAllUsrTypesRegRemUsrsCrs ();

   /***** Step 3: Put different actions to register/remove students to/from current course *****/
   fprintf (Gbl.F.Out,"<div align=\"left\" class=\"%s\">" \
                      "<br />Paso 3: Seleccione la acci&oacute;n que desea realizar<br />&nbsp;</div>",	// Need translation!
            The_ClassTitle[Gbl.Prefs.Theme]);
   Usr_PutActionsRegRemUsrsCrs ();

   /***** Step 4: Select groups in which register / remove students *****/
   fprintf (Gbl.F.Out,"<div align=\"left\" class=\"%s\">" \
                      "<br />Paso 4: Opcionalmente, seleccione grupos<br />&nbsp;</div>",	// Need translation!
            The_ClassTitle[Gbl.Prefs.Theme]);
   Usr_ShowLstGrpsToRegisterRemoveStds ();

   /***** Step 5: Button to register / remove students *****/
   fprintf (Gbl.F.Out,"<div align=\"left\" class=\"%s\">" \
                      "<br />Paso 5: Confirme el alta/baja<br />&nbsp;</div>",	// Need translation!
            The_ClassTitle[Gbl.Prefs.Theme]);
   Lay_PutSendButton (Txt_Register_remove[Gbl.Prefs.Language]);

   /***** End of form *****/
   fprintf (Gbl.F.Out,"</form></div>\n");
  }

/****************************************************************************/
/** Put text area to enter/paste IDs of students to be registered/removed ***/
/****************************************************************************/

static void Usr_PutAreaToEnterStudentsIDs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_List_of_IDs[Txt_NUM_LANGUAGES];

   /***** Start of text area *****/
   fprintf (Gbl.F.Out,"<table><tr>" \
                      "<td align=\"right\" valign=\"top\" class=\"%s\">%s: </td>" \
                      "<td align=\"left\"><textarea name=\"Txt\" cols=\"50\" rows=\"10\">",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_List_of_IDs[Gbl.Prefs.Language]);

   /***** End of text area *****/
   fprintf (Gbl.F.Out,"</textarea></td></tr></table>\n");
  }

/****************************************************************************/
/** Put different actions to register/remove users to/from current course ***/
/****************************************************************************/

static void Usr_PutActionsRegRemUsrsCrs (void)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_Register_users_who_are_on_the_list[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_users_who_are_on_the_list[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_users_who_are_not_on_the_list[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_users_who_are_not_on_the_list_and_register_users_who_are_on_the_list[Txt_NUM_LANGUAGES];

   /***** Register / remove users listed or not listed *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">" \
                      "<table cellspacing=\"0\" cellpadding=\"0\">" \
                      "<tr>" \
                      "<td align=\"left\"><input type=\"radio\" name=\"RegRemAction\" value=\"1\" checked=\"checked\" /></td>" \
                      "<td align=\"left\" class=\"%s\">%s</td>" \
                      "</tr>" \
                      "<tr>" \
                      "<td align=\"left\"><input type=\"radio\" name=\"RegRemAction\" value=\"2\" /></td>" \
                      "<td align=\"left\" class=\"%s\">%s</td>" \
                      "</tr>" \
                      "<tr>" \
                      "<td align=\"left\"><input type=\"radio\" name=\"RegRemAction\" value=\"3\" /></td>" \
                      "<td align=\"left\" class=\"%s\">%s</td>" \
                      "</tr>" \
                      "<tr>" \
                      "<td align=\"left\"><input type=\"radio\" name=\"RegRemAction\" value=\"4\" /></td>" \
                      "<td align=\"left\" class=\"%s\">%s</td>" \
                      "</tr>" \
                      "</table>" \
                      "</div>",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Register_users_who_are_on_the_list[Gbl.Prefs.Language],
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Remove_users_who_are_on_the_list[Gbl.Prefs.Language],
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Remove_users_who_are_not_on_the_list[Gbl.Prefs.Language],
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_Remove_users_who_are_not_on_the_list_and_register_users_who_are_on_the_list[Gbl.Prefs.Language]);
  }

/****************************************************************************/
/*** Select types of user to register/remove users to/from current course ***/
/****************************************************************************/

static void Usr_PutAllUsrTypesRegRemUsrsCrs (void)
  {
   /***** Students or teachers *****/
   fprintf (Gbl.F.Out,"<div align=\"center\">" \
	              "<table cellspacing=\"0\" cellpadding=\"0\">");
   Usr_PutOneUsrTypeRegRemUsrsCrs (Usr_STUDENT,true);
   Usr_PutOneUsrTypeRegRemUsrsCrs (Usr_TEACHER,false);
   fprintf (Gbl.F.Out,"</table>" \
                      "</div>");
  }

/****************************************************************************/
/*** Select types of user to register/remove users to/from current course ***/
/****************************************************************************/

static void Usr_PutOneUsrTypeRegRemUsrsCrs (Usr_UsrType_t UsrType,bool Checked)
  {
   extern const char *The_ClassFormul[The_NUM_THEMES];
   extern const char *Txt_USR_TYPES_SINGULAR_Abc[Usr_NUM_USR_TYPES][Usr_NUM_GENDERS][Txt_NUM_LANGUAGES];

   fprintf (Gbl.F.Out,"<tr>" \
                      "<td align=\"left\"><input type=\"radio\" name=\"RegRemUsrType\" value=\"%u\"",
            (unsigned) UsrType);
   if (Checked)
      fprintf (Gbl.F.Out," checked=\"checked\"");
   if (Gbl.Usrs.Me.UsrTypeLog != Usr_SUPERUSER)	// Can I select type of user?
      fprintf (Gbl.F.Out," disabled=\"disabled\"");
   fprintf (Gbl.F.Out," /></td>" \
                      "<td align=\"left\" class=\"%s\">%s</td>" \
                      "</tr>",
            The_ClassFormul[Gbl.Prefs.Theme],
            Txt_USR_TYPES_SINGULAR_Abc[UsrType][Usr_GENDER_UNKNOWN][Gbl.Prefs.Language]);
  }

/****************************************************************************/
/****** Receive the list of users of the course to register/remove **********/
/****************************************************************************/

#define MAX_BYTES_LIST_OFFICIAL_GRP_CODS (10*1024)

void Usr_ReceiveFormUsrsCrs (void)
  {
   char ListOfficialGrpCods[MAX_BYTES_LIST_OFFICIAL_GRP_CODS+1];
   char UnsignedStr[10+1];
   unsigned UnsignedNum;
   Usr_UsrType_t RegRemUsrType = Usr_STUDENT;
   bool RemoveUsrs = false;
   bool RemoveSpecifiedUsrs = true;
   bool RegisterUsrs = false;
   const char *Ptr;
   unsigned NumUsr;
   long GrpCod;
   unsigned NumGrpsInCurrentCrs;
   unsigned NumUsrsRegistered = 0;
   unsigned NumUsrsRemoved = 0;
   struct ListUsers *LstUsrs;
   struct ListCodGrps LstGrps;

   /***** Get the type of user to register / remove *****/
   Par_GetParam (Par_PARAM_SINGLE,"RegRemUsrType",UnsignedStr,1);
   if (UnsignedStr[0])
     {
      RegRemUsrType = Usr_ConvertUnsignedStrToUsrType (UnsignedStr);
      switch (RegRemUsrType)
        {
         case Usr_STUDENT:
            break;
         case Usr_TEACHER:
            if (Gbl.Usrs.Me.UsrTypeLog != Usr_SUPERUSER)
               Lay_ShowErrorAndExit ("You are not allowed to create teachers.");	// If user manipulated the form
            break;
         default:
            Lay_ShowErrorAndExit ("Wrong type of users.");				// If user manipulated the form
        }
     }

   /***** Get the action to do *****/
   Par_GetParam (Par_PARAM_SINGLE,"RegRemAction",UnsignedStr,1);
   if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
      switch (UnsignedNum)
        {
         case 1:
            RemoveUsrs = false;
            RemoveSpecifiedUsrs = false;	// Ignored
            RegisterUsrs = true;
            break;
         case 2:
            RemoveUsrs = true;
            RemoveSpecifiedUsrs = true;
            RegisterUsrs = false;
            break;
         case 3:
            RemoveUsrs = true;
            RemoveSpecifiedUsrs = false;
            RegisterUsrs = false;
            break;
         case 4:
            RemoveUsrs = true;
            RemoveSpecifiedUsrs = false;
            RegisterUsrs = true;
            break;
         default:
            Lay_ShowErrorAndExit ("Wrong registering / removing specification.");
        }
   else
      Lay_ShowErrorAndExit ("Registering / removing is missing.");

   /***** Get official groups of students *****/
   Par_GetParam (Par_PARAM_MULTIPLE,"ImpGrpCod",ListOfficialGrpCods,MAX_BYTES_LIST_OFFICIAL_GRP_CODS);

   LstGrps.NumGrps = 0;
   if ((NumGrpsInCurrentCrs = Grp_CountNumGrpsInCurrentCrs ())) // This course has groups
     {
      /***** Get list of groups types and groups in current course *****/
      Grp_GetListGrpTypesAndGrpsInThisCrs ();

      /***** Get the list of groups to which register/remove students *****/
      Grp_GetLstCodsGrpIWant (&LstGrps);

      /***** A student can't belong to more than one group when the type of group only allows to register in one group *****/
      if (RegisterUsrs && (RegRemUsrType == Usr_STUDENT) && (LstGrps.NumGrps >= 2))
         /* Check if I have selected more than one group of single registration */
         if (!Grp_CheckIfSelectionGrpsIsValid (&LstGrps))
           {
            /* Show warning message and exit */
            Lay_ShowAlert (WARNING,"Para un tipo de grupo de adscrici&oacute;n &uacute;nica" \
                                   " no puede apuntar alumnos a m&aacute;s de un grupo.");	// Need translation!

            /* Free memory used by lists of groups and abort */
            Grp_FreeListCodGrp (&LstGrps);
            Grp_FreeListGrpTypesAndGrps ();
            return;
           }
     }

   /***** Get list of users' IDs *****/
   if ((Gbl.Txt = malloc (Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS+1)) == NULL)
      Lay_ShowErrorAndExit ("Not enough memory to store users' IDs.");
   Par_GetParam (Par_PARAM_SINGLE,"Txt",Gbl.Txt,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS);
   Str_ChangeFormat (Str_FROM_FORM,Str_TO_TEXT,Gbl.Txt,Usr_MAX_BYTES_LIST_ENCRYPTED_USR_CODS,true);

   /***** Remove users *****/
   if (RemoveUsrs)
     {
      /***** Get list of users in current course *****/
      switch (RegRemUsrType)
        {
         case Usr_STUDENT:
            Usr_GetStdsLstCrs ();
            LstUsrs = &Gbl.Usrs.LstStds;
            break;
         case Usr_TEACHER:
            Usr_GetTchsLst (Usr_LIST_USRS_RANGE_THIS_COURSE_ALL,-1L,NULL);
            LstUsrs = &Gbl.Usrs.LstTchs;
            break;
         default:
            LstUsrs = NULL;	// To avoid warning
            Lay_ShowErrorAndExit ("Wrong type of users.");	// If user manipulated the form
        }

      if (LstUsrs->NumUsrs)
        {
         /***** Initialize list of users to remove *****/
         for (NumUsr=0; NumUsr<LstUsrs->NumUsrs; NumUsr++)
            LstUsrs->Remove[NumUsr] = !RemoveSpecifiedUsrs;

         /***** Loop 1: go through form list setting if a student must be removed *****/
         /* Step a: Get students from a list of official groups */
         if (RegRemUsrType == Usr_STUDENT)
           {
            Ptr = ListOfficialGrpCods;
            while (*Ptr)
              {
               /* Find next official group code in list */
               Str_GetNextStringUntilSeparator (&Ptr,UnsignedStr,10);
               if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
                 {
                  GrpCod = (long) UnsignedNum;
                  /* Mark users belonging to this official group as removable */
                  Usr_MarkOfficialStdsAsRemovable (GrpCod,RemoveSpecifiedUsrs);
                 }
              }
           }

         /* Step b: Get users from a list of DNIs */
         Ptr = Gbl.Txt;
         while (*Ptr)
           {
            // Find next DNI in text
            Str_GetNextStringUntilSeparator (&Ptr,Gbl.Usrs.Other.UsrDat.DNI,Usr_MAX_LENGTH_USR_ID);
            if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Other.UsrDat.DNI))	// If not a DNI, skip it
               for (NumUsr=0; NumUsr<LstUsrs->NumUsrs; NumUsr++)
                  if (!strcasecmp (LstUsrs->Lst[NumUsr].DNI,Gbl.Usrs.Other.UsrDat.DNI))	// Student found
                     LstUsrs->Remove[NumUsr] = RemoveSpecifiedUsrs;
           }

         /***** Loop 2: go through users list removing users *****/
         for (NumUsr=0; NumUsr<LstUsrs->NumUsrs; NumUsr++)
            if (LstUsrs->Remove[NumUsr])	// If this student must be removed
              {
               if (NumGrpsInCurrentCrs)	// If there are groups in the course
                 {
                  if (LstGrps.NumGrps)	// If the teacher has selected groups
                    {
                     if (Grp_RemoveUsrFromGroups (&(LstUsrs->Lst[NumUsr]),&LstGrps))		// Remove user from the selected groups, not from the whole course
                        NumUsrsRemoved++;
                    }
                  else	// The teacher has not selected groups
                    {
                     Usr_EffectivelyRemUsrFromCrs (&(LstUsrs->Lst[NumUsr]),&Gbl.CurrentCrs.Crs,Cte_QUIET);	// Remove user from the course
                     NumUsrsRemoved++;
                    }
                 }
               else	// No groups
                 {
                  Usr_EffectivelyRemUsrFromCrs (&(LstUsrs->Lst[NumUsr]),&Gbl.CurrentCrs.Crs,Cte_QUIET);	// Remove user from the course
                  NumUsrsRemoved++;
                 }
              }
        }

      /***** Free memory for users list *****/
      Usr_FreeUsrsList (LstUsrs);
     }

   /***** Register users *****/
   if (RegisterUsrs)
     {
      /***** Step a: Get users from a list of official groups *****/
      if (RegRemUsrType == Usr_STUDENT)
        {
         Ptr = ListOfficialGrpCods;
         while (*Ptr)
           {
            /* Find next official group code in list */
            Str_GetNextStringUntilSeparator (&Ptr,UnsignedStr,10);
            if (sscanf (UnsignedStr,"%u",&UnsignedNum) == 1)
              {
               GrpCod = (long) UnsignedNum;
               /* Import official group of users */
               Imp_ImportStdsFromAnImpGrp (GrpCod,NumGrpsInCurrentCrs,&LstGrps,&NumUsrsRegistered);
              }
           }
        }

      /***** Step b: Get users from a list of DNIs ******/
      Ptr = Gbl.Txt;
      while (*Ptr)
        {
         /* Find next DNI in text */
         Str_GetNextStringUntilSeparator (&Ptr,Gbl.Usrs.Other.UsrDat.DNI,Usr_MAX_LENGTH_USR_ID);
         Str_ConvertToUpperText (Gbl.Usrs.Other.UsrDat.DNI);
         if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Other.UsrDat.DNI))	// If it's not a DNI, skip it
            Usr_RegisterUsrUsingHisDNI (RegRemUsrType,NumGrpsInCurrentCrs,&LstGrps,&NumUsrsRegistered);
        }
     }

   /***** Write messages with the number of users registered/removed *****/
   if (RemoveUsrs)
      switch (NumUsrsRemoved)
        {
         case 0:
            Lay_ShowAlert (INFO,"No se ha dado de baja ning&uacute;n usuario.");	// Need translation!
            break;
         case 1:
            Lay_ShowAlert (INFO,"Se ha dado de baja un usuario.");	// Need translation!
            break;
         default:
            sprintf (Gbl.Message,"Se han dado de baja %u usuarios.",	// Need translation!
                     NumUsrsRemoved);
            Lay_ShowAlert (INFO,Gbl.Message);
        }
   if (RegisterUsrs)
      switch (NumUsrsRegistered)
        {
         case 0:
            Lay_ShowAlert (INFO,"No se ha dado de alta ning&uacute;n usuario.");	// Need translation!
            break;
         case 1:
            Lay_ShowAlert (INFO,"Se ha dado de alta un usuario.");	// Need translation!
            break;
         default:
            sprintf (Gbl.Message,"Se han dado de alta %u usuarios (incluyendo las posibles repeticiones).",	// Need translation!
                     NumUsrsRegistered);
            Lay_ShowAlert (INFO,Gbl.Message);
        }

   /***** Free memory used by the body of the message *****/
   Str_FreeTxt ();

   /***** Free memory with the list of groups to/from which register/remove users *****/
   Grp_FreeListCodGrp (&LstGrps);

   /***** Free list of groups types and groups in current course *****/
   Grp_FreeListGrpTypesAndGrps ();
  }

/****************************************************************************/
/********************** Register a user using his/her ID ********************/
/****************************************************************************/

static void Usr_RegisterUsrUsingHisDNI (Usr_UsrType_t RegRemUsrType,unsigned NumGrpsInCurrentCrs,struct ListCodGrps *LstGrps,unsigned *NumUsrsRegistered)
  {
   bool UsrHasTheSpecifiedType = true;

   /* Check if the record of the user exists and get the type of user */
   if (Usr_ChkDNIAndGetAllUsrDataFromDNI (&Gbl.Usrs.Other.UsrDat))		// Get from the database the data of the user to register
     {
      // The user exists. Only can be registered if he/she is a user of the specified type
      if (Gbl.Usrs.Other.UsrDat.UsrTypeInDB != RegRemUsrType)
        {
         UsrHasTheSpecifiedType = false;
         sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong>" \
                              " no se ha dado de alta por no ser del tipo especificado.",	// Need translation!
                  Gbl.Usrs.Other.UsrDat.DNI);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else
      // The user doesn't exist. Register him/her as user of the specified type
      Gbl.Usrs.Other.UsrDat.UsrTypeInDB = RegRemUsrType;
   if (UsrHasTheSpecifiedType)
     {
      Usr_CreatUsrIfNotExistsAndRegisterHimInCurrentCrs (&Gbl.Usrs.Other.UsrDat,Usr_INSERT_NEW_DATA,Cte_QUIET);	// Register user in course, and create his/her record if user did not exist
      if (NumGrpsInCurrentCrs)	// If there are groups in the course
         Grp_RegisterUsrIntoGroups (&Gbl.Usrs.Other.UsrDat,LstGrps);				// Register student in the selected groups
      (*NumUsrsRegistered)++;
     }
  }

/****************************************************************************/
/****** Mark the students belonging to an official group as removable *******/
/****************************************************************************/

static void Usr_MarkOfficialStdsAsRemovable (long ImpGrpCod,bool RemoveSpecifiedUsrs)
  {
   char Query[1024];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumStds,NumStd;
   unsigned NumUsr;
   char DNI[Usr_MAX_LENGTH_USR_ID+1];

   /***** Get imported students belonging to this group from database *****/
   sprintf (Query,"SELECT DNI FROM imported_students WHERE GrpCod='%ld'",
            ImpGrpCod);
   NumStds = (unsigned) DB_QuerySELECT (Query,&mysql_res,"can not get imported students");

   /***** Mark the students from this official group as removable *****/
   for (NumStd=0; NumStd<NumStds; NumStd++)	// For each student inside the official group...
     {
      /* Get DNI of the student from official list */
      row = mysql_fetch_row (mysql_res);
      strncpy (DNI,row[0],Usr_MAX_LENGTH_USR_ID);
      DNI[Usr_MAX_LENGTH_USR_ID] = '\0';

      /* Check if this official DNI is in the list of students */
      if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (DNI))
         for (NumUsr=0; NumUsr<Gbl.Usrs.LstStds.NumUsrs; NumUsr++)
            if (!strcasecmp (Gbl.Usrs.LstStds.Lst[NumUsr].DNI,DNI))	// Student found
               Gbl.Usrs.LstStds.Remove[NumUsr] = RemoveSpecifiedUsrs;
     }

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);
  }

/****************************************************************************/
/**** Show list of groups to register/remove students to/from the course ****/
/****************************************************************************/

static void Usr_ShowLstGrpsToRegisterRemoveStds (void)
  {
   unsigned NumGrpTyp;

   /***** Check if current course has groups *****/
   if (Grp_CountNumGrpsInCurrentCrs ()) // This course has groups
     {
      /***** Get list of groups types and groups in current course *****/
      Grp_GetListGrpTypesAndGrpsInThisCrs ();

      /***** Write help message *****/
      Lay_WriteTitle ("Seleccione los grupos en/de los que quiere dar de alta/baja usuarios." \
                      " S&oacute;lo puede seleccionar aquellos grupos" \
                      " a los que usted pertenezca.<br />" \
                      "Si selecciona grupos," \
                      " los usuarios ser&aacute;n dados de alta en la asignatura y en los grupos seleccionados," \
                      " o bien dados de baja los grupos seleccionados pero no de la asignatura.<br />" \
                      "Si no selecciona ning&uacute;n grupo," \
                      " los usuarios ser&aacute;n dados de alta en la asignatura pero no en grupos concretos," \
                      " o bien dados de baja de la asignatura y de todos los grupos.");	// Need translation!

      /***** Start table *****/
      Lay_StartRoundFrameTable10 ("frame_central_corner","frame_central",WHITE,NULL);

      /***** List to select the groups the user belongs to *****/
      for (NumGrpTyp=0; NumGrpTyp<Gbl.CurrentCrs.Grps.GrpTypes.Num; NumGrpTyp++)
         if (Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp].NumGrps)
            Grp_ListGrpsToAddOrRemUsrs (&Gbl.CurrentCrs.Grps.GrpTypes.LstGrpTypes[NumGrpTyp]);

      /***** Free list of groups types and groups in current course *****/
      Grp_FreeListGrpTypesAndGrps ();

      /***** End table *****/
      Lay_EndRoundFrameTable10 ("frame_central_corner","frame_central");
     }
   else
     {
      /***** Write help message *****/
      sprintf (Gbl.Message,"No se han creado grupos en <strong>%s</strong>." \
                           " Por tanto, los usuarios especificados ser&aacute;n dados de alta/baja en/de la asignatura.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_WriteTitle (Gbl.Message);
     }
  }

/****************************************************************************/
/********** Ask for removing all the students from current course ***********/
/****************************************************************************/

void Usr_AskRemAllStdsThisCrs (void)
  {
   extern const char *Txt_Remove_students[Txt_NUM_LANGUAGES];

   /***** Write message to confirm the removing *****/
   sprintf (Gbl.Message,"&iquest;Realmente desea dar de baja todos los alumnos de la asignatura <strong>%s</strong>?" \
                        " Aunque no se eliminar&aacute;n los datos de los alumnos en otras asignaturas," 
                        " ya no aparecer&aacute;n adscritos a esta asignatura," \
                        " y adem&aacute;s <strong>se eliminar&aacute;n los trabajos enviados por ellos en esta asignatura," \
                        " sus fichas personalizadas para esta asignatura, y su adcripci&oacute;n a grupos de esta asignatura.</strong>.",	// Need translation!
            Gbl.CurrentCrs.Crs.FullName);
   Lay_ShowAlert (WARNING,Gbl.Message);

   /***** Form to remove all the students in current course *****/
   Act_FormStart (ActRemAllStdCrs);
   Grp_PutParamAllGroups ();
   Lay_PutSendButton (Txt_Remove_students[Gbl.Prefs.Language]);
   fprintf (Gbl.F.Out,"</form>\n");
  }

/****************************************************************************/
/******************** Remove all the students from current course **************/
/****************************************************************************/

void Usr_RemAllStdsThisCrs (void)
  {
   unsigned NumUsr;

   /***** Get list of students in current course *****/
   Grp_GetParCodsSeveralGrpsToShowUsrs ();
   Usr_GetStdsLstCrs ();

   /***** Remove all the students *****/
   if (Gbl.Usrs.LstStds.NumUsrs)
     {
      for (NumUsr=0; NumUsr<Gbl.Usrs.LstStds.NumUsrs; NumUsr++)
         Usr_EffectivelyRemUsrFromCrs (&Gbl.Usrs.LstStds.Lst[Gbl.Usrs.LstStds.Ind[NumUsr]],&Gbl.CurrentCrs.Crs,Cte_QUIET);
      sprintf (Gbl.Message,"Se han dado de baja los %u alumnos que hab&iacute;a en <strong>%s</strong>.",	// Need translation!
               Gbl.Usrs.LstStds.NumUsrs,Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);
     }
   else
     {
      sprintf (Gbl.Message,"No hab&iacute;a alumnos en <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
      Lay_ShowAlert (WARNING,Gbl.Message);
     }

   /***** Free memory for students list *****/
   Usr_FreeUsrsList (&Gbl.Usrs.LstStds);

   /***** Free memory for list of selected groups *****/
   Grp_FreeListCodSelectedGrps ();
  }

/****************************************************************************/
/****************** Form to request the DNI of another user *****************/
/****************************************************************************/

void Usr_ReqDNINewUsr (void)
  {
   sprintf (Gbl.Message,"Escriba el DNI del usuario que desea dar de alta en <strong>%s</strong>" \
                        " o cuyos datos desea modificar.",	// Need translation!
            Gbl.CurrentCrs.Crs.FullName);
   Lay_ShowAlert (INFO,Gbl.Message);

   Usr_WriteFormToReqAnotherUserDNI (ActAskNewUsrCrs);
  }

/****************************************************************************/
/************** Form to request the DNI of an administrator *****************/
/****************************************************************************/

void Usr_ReqDNINewAdm (void)
  {
   if (Gbl.CurrentDeg.Deg.DegCod > 0)
     {
      sprintf (Gbl.Message,"Escriba el DNI del usuario que desea dar de alta" \
                           " como administrador de la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentDeg.Deg.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);

      Usr_WriteFormToReqAnotherUserDNI (ActAskNewAdm);
     }
   else
      Lay_ShowAlert (WARNING,"Debe seleccionar una titulaci&oacute;n.");	// Need translation!
  }

/****************************************************************************/
/****************** Form to request the DNI of another user *****************/
/****************************************************************************/

void Usr_ReqDNIPhotoUsr (void)
  {
   Lay_ShowAlert (INFO,"Escriba el DNI del usuario cuya fotograf&iacute;a desea enviar o eliminar.");

   Usr_WriteFormToReqAnotherUserDNI (ActUsrPho);
  }

/****************************************************************************/
/******************* Add/modify another user to current course *****************/
/****************************************************************************/

void Usr_AskIfAddUsrToCrs (void)
  {
   extern const char *Txt_Register_user_IN_A_COURSE[Txt_NUM_LANGUAGES];
   bool UsrBelongsToCrs = false;

   /***** Get plain DNI of the user to add/modify *****/
   Usr_GetParamOtherUsrDNIPlain ();

   /***** Check if DNI is valid *****/
   if (Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (Gbl.Usrs.Other.UsrDat.DNI))	// DNI valid
     {
      /* Check if user exists and get his data */
      if (Usr_ChkDNIAndGetAllUsrDataFromDNI (&Gbl.Usrs.Other.UsrDat))	// Existing user
        {
         /* Check if it's allowed to regisrer this user in course */
         if (((Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER || Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN) &&
              (Gbl.Usrs.Other.UsrDat.UsrTypeInDB == Usr_STUDENT || Gbl.Usrs.Other.UsrDat.UsrTypeInDB == Usr_TEACHER)) ||
             Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
           {
            if ((UsrBelongsToCrs = Usr_CheckIfUsrBelongsToCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentCrs.Crs.CrsCod)))
              {
               Gbl.Usrs.Other.UsrDat.Accepted = Usr_GetIfUserHasAccepted (Gbl.Usrs.Other.UsrDat.UsrCod);
               if (Gbl.Usrs.Other.UsrDat.Accepted)
                 {
                  sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ya est&aacute; dado de alta en <strong>%s</strong>.",	// Need translation!
                           Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentCrs.Crs.FullName);
                  Lay_ShowAlert (INFO,Gbl.Message);
                 }
               else
                 {
                  sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ya est&aacute; en <strong>%s</strong>," \
                                       " pero a&uacute;n no ha aceptado el alta.",	// Need translation!
                           Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentCrs.Crs.FullName);
                  Lay_ShowAlert (WARNING,Gbl.Message);
                 }
              }
            else	// User don't belong to current course
              {
               sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ya tiene ficha," \
                                    " pero a&uacute;n no est&aacute; en <strong>%s</strong>.",	// Need translation!
                        Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentCrs.Crs.FullName);
               Lay_ShowAlert (INFO,Gbl.Message);
              }

            if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER || Gbl.Usrs.Me.UsrTypeLog == Usr_DEG_ADMIN || 
                (Gbl.Usrs.Me.UsrTypeLog == Usr_TEACHER && Gbl.Usrs.Other.UsrDat.UsrTypeInDB == Usr_STUDENT && UsrBelongsToCrs && Gbl.Usrs.Other.UsrDat.Accepted))
               /* Show form to modify the current data */
               Rec_ShowFormOtherExistingCommonRecord (&Gbl.Usrs.Other.UsrDat);
            else if (UsrBelongsToCrs) // A teacher wants to add another user to current course, and that user yet belongs to current course
               Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);
            else // A teacher wants to add another user to current course, but he/her can not modify the other teacher's data
              {
               /* Present (not modifiable) teacher's data */
               Act_FormStart (ActRegUsr);
               Usr_PutParamOtherUsrDNIPlain ();
               Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);
               Lay_PutSendButton (Txt_Register_user_IN_A_COURSE[Gbl.Prefs.Language]);
               fprintf (Gbl.F.Out,"</form>\n");
              }
           }
         else
           {
            sprintf (Gbl.Message,"No le est&aacute; permitido dar de alta en <strong>%s</strong>" \
                                 " al usuario con DNI <strong>%s</strong>.",	// Need translation!
                     Gbl.CurrentCrs.Crs.FullName,Gbl.Usrs.Other.UsrDat.DNI);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
        }
      else	// New user
        {
         /***** Initialize some data of this new user *****/
         Gbl.Usrs.Other.UsrDat.UsrTypeInDB = Usr_STUDENT;

         /***** Show form to enter the data of a new user *****/
         sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> es nuevo, a&uacute;n no tiene ficha.",	// Need translation!
                  Gbl.Usrs.Other.UsrDat.DNI);
         Lay_ShowAlert (INFO,Gbl.Message);
         Rec_ShowFormOtherNewCommonRecord (&Gbl.Usrs.Other.UsrDat);
        }
     }
   else	// DNI not valid
     {
      /***** Write message and request a new DNI *****/
      sprintf (Gbl.Message,"El DNI <strong>%s</strong> no es v&aacute;lido.",	// Need translation!
               Gbl.Usrs.Other.UsrDat.DNI);
      Lay_ShowAlert (WARNING,Gbl.Message);
      Usr_ReqDNINewUsr ();
     }
  }

/****************************************************************************/
/******************* Add an administrator to current degree *****************/
/****************************************************************************/

void Usr_AddAdmToDeg (void)
  {
   if (Gbl.CurrentDeg.Deg.DegCod < 0)
     {
      Lay_ShowAlert (WARNING,"Debe seleccionar una titulaci&oacute;n.");	// Need translation!
      return;
     }

   /***** Get plain DNI of the user to add/modify *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /* Check if it's allowed to register this administrator in degree */
      if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
        {
         /***** Register administrator in current degree in database *****/
         Usr_RegisterAdminInCurrentDeg (Gbl.Usrs.Other.UsrDat.UsrCod);

         /***** Show user's record *****/
         Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);
        }
      else
         Lay_ShowErrorAndExit ("You can not register a user as administrator of degree.");
     }
   else
      Usr_ReqDNINewAdm ();
  }

/****************************************************************************/
/********************** Send another user's photograph **********************/
/****************************************************************************/

void Usr_SendPhotoUsr1 (void)
  {
   /***** Get plain DNI of the user whose photo must be sent or removed and get user's data *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /* If intento cambiar mi foto, ir a the opcin correspondiente in the pestaa of data personales */
      if (Gbl.Usrs.Me.UsrDat.UsrCod == Gbl.Usrs.Other.UsrDat.UsrCod)
        {
         Gbl.CurrentTab = TabPer;
         Gbl.CurrentAct = ActReqMyPho;
        }
      Gbl.Usrs.Error = false;
     }
   else
      Gbl.Usrs.Error = true;
  }

/****************************************************************************/
/********************** Send another user's photograph **********************/
/****************************************************************************/

void Usr_SendPhotoUsr2 (void)
  {
   bool ICanChangePhoto;

   if (Gbl.Usrs.Error)
      Usr_ReqDNIPhotoUsr ();
   else
     {
      /* Check if I have permission to change user's photo */
      switch (Gbl.Usrs.Me.UsrTypeLog)
        {
         case Usr_SUPERUSER:
            ICanChangePhoto = true;
            break;
         case Usr_DEG_ADMIN:
            ICanChangePhoto = Usr_CheckIfUsrBelongsToDeg (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentDeg.Deg.DegCod);
            break;
         case Usr_TEACHER:
            ICanChangePhoto = Usr_CheckIfUsrBelongsToCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentCrs.Crs.CrsCod);
            break;
         default:
            ICanChangePhoto = false;
        }

      /* Check if user exists and get his data */
      if (ICanChangePhoto)	// If I have permission to change user's photo...
        {
         if (Usr_ChkDNIAndGetAllUsrDataFromDNI (&Gbl.Usrs.Other.UsrDat))	// If user's data exist...
           {
            /* Check if uploading the other user's photo is allowed */
            if (Gbl.Usrs.Me.UsrTypeLog > Gbl.Usrs.Other.UsrDat.UsrTypeInDB)
              {
               Gbl.Usrs.Other.UsrDat.Accepted = Usr_GetIfUserHasAccepted (Gbl.Usrs.Other.UsrDat.UsrCod);
               Usr_ReqUsrPhoto (&Gbl.Usrs.Other.UsrDat);	// Request user's photograph
              }
            else
              {
               sprintf (Gbl.Message,"Usted no tiene permiso para cambiar la fotograf&iacute;a del usuario" \
                                    " con DNI <strong>%s</strong>.",	// Need translation!
                        Gbl.Usrs.Other.UsrDat.DNI);
               Lay_ShowAlert (WARNING,Gbl.Message);
              }
           }
         else
            Lay_ShowErrorAndExit ("User doesn't exist.");
        }
      else	// The user doesn't belong to current course
        {
         sprintf (Gbl.Message,"Usted no tiene permiso para cambiar la fotograf&iacute;a del usuario" \
                              " con DNI <strong>%s</strong>.",	// Need translation!
                  Gbl.Usrs.Other.UsrDat.DNI);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
  }

/****************************************************************************/
/*** Show form to ask for the DNI of a user to remove from current course ***/
/****************************************************************************/

void Usr_ReqDNIRemUsrFromCrs (void)
  {
   if (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT)
      sprintf (Gbl.Message,"Escriba su DNI si desea darse de baja" \
                           " (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.) de <strong>%s</strong>.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
   else
      sprintf (Gbl.Message,"Escriba el DNI del usuario que desea dar de baja" \
                           " (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.) de <strong>%s</strong>," \
                           " o su DNI si es usted quien desea darse de baja.",	// Need translation!
               Gbl.CurrentCrs.Crs.FullName);
   Lay_ShowAlert (INFO,Gbl.Message);

   Usr_WriteFormToReqAnotherUserDNI (ActAskRemUsrCrs);
  }

/****************************************************************************/
/*** Show form to ask for the DNI of a user to remove from current course ***/
/****************************************************************************/

void Usr_ReqDNIRemAdm (void)
  {
   if (Gbl.CurrentDeg.Deg.DegCod > 0)
     {
      if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
         sprintf (Gbl.Message,"Escriba el DNI del usuario que desea dar de baja" \
                              " como administrador de la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
                  Gbl.CurrentDeg.Deg.FullName);
      else
         sprintf (Gbl.Message,"Escriba su DNI si desea darse de baja" \
                              " como administrador de la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
                 Gbl.CurrentDeg.Deg.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);

      Usr_WriteFormToReqAnotherUserDNI (ActAskRemAdm);
     }
   else
      Lay_ShowAlert (WARNING,"Debe seleccionar una titulaci&oacute;n.");	// Need translation!
  }

/****************************************************************************/
/************** Ask for remove of a user from current course *************/
/****************************************************************************/

void Usr_ReqRemUsrFromCrs (void)
  {
   Usr_ReqRemOrRemUsrFromCrs (Usr_REQUEST_REMOVE_USR);
  }

/****************************************************************************/
/********************* Remove a user from current course ********************/
/****************************************************************************/

void Usr_RemUsrFromCrs (void)
  {
   Usr_ReqRemOrRemUsrFromCrs (Usr_REMOVE_USR);
  }

/****************************************************************************/
/******************** Remove of a user from current course ******************/
/****************************************************************************/

void Usr_ReqRemOrRemUsrFromCrs (Usr_ReqDelOrDelUsr_t ReqDelOrDelUsr)
  {
   bool ItsMe;
   bool ICanRemove;

   /***** Get user's DNI from form *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /* Check if it's forbidden remove that user */
      // A teacher can remove a student or himself
      // An administrator can remove anyone
      ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == Gbl.Usrs.Other.UsrDat.UsrCod);
      ICanRemove = (Gbl.Usrs.Me.UsrTypeLog == Usr_STUDENT) ? ItsMe : (Gbl.Usrs.Me.UsrTypeLog >= Gbl.Usrs.Other.UsrDat.UsrTypeInDB);
      if (ICanRemove)
        {
         /* Check if the other user belong to current course */
         if (Usr_CheckIfUsrBelongsToCrs (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentCrs.Crs.CrsCod))
           {		// The other user belongs to current course ==> ask for removing or remove him
            switch (ReqDelOrDelUsr)
              {
               case Usr_REQUEST_REMOVE_USR:	// Ask if remove user from current course
                  Usr_AskIfRemUsrFromCrsOrPlatform (ItsMe);		
		  break;
	       case Usr_REMOVE_USR:		// Remove user from current course
                  Usr_EffectivelyRemUsrFromCrs (&Gbl.Usrs.Other.UsrDat,&Gbl.CurrentCrs.Crs,Cte_VERBOSE);
              }
           }
         else	// The other user don't belong to current course
           {
            sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> no estaba dado de alta en <strong>%s</strong>.",	// Need translation!
                     Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentCrs.Crs.FullName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
        }
      else
        {
         sprintf (Gbl.Message,"No le est&aacute; permitido dar de baja de <strong>%s</strong> al usuario con DNI <strong>%s</strong>.",	// Need translation!
                  Gbl.CurrentCrs.Crs.FullName,Gbl.Usrs.Other.UsrDat.DNI);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else
      Usr_ReqDNIRemUsrFromCrs ();
  }

/****************************************************************************/
/********* Ask for remove of an administrator from current degree ***********/
/****************************************************************************/

void Usr_ReqRemAdm (void)
  {
   Usr_ReqRemOrRemAdm (Usr_REQUEST_REMOVE_USR);
  }

/****************************************************************************/
/********************* Remove a user from current course ********************/
/****************************************************************************/

void Usr_RemAdm (void)
  {
   Usr_ReqRemOrRemAdm (Usr_REMOVE_USR);
  }

/****************************************************************************/
/******************** Remove of a user from current course ******************/
/****************************************************************************/

void Usr_ReqRemOrRemAdm (Usr_ReqDelOrDelUsr_t ReqDelOrDelUsr)
  {
   bool ItsMe;
   bool ICanRemove;

   if (Gbl.CurrentDeg.Deg.DegCod < 0)
     {
      Lay_ShowAlert (WARNING,"Debe seleccionar una titulaci&oacute;n.");	// Need translation!
      return;
     }

   /***** Get user's DNI from form and get user's data *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /* Check if it's forbidden remove that administrator */
      // A superuser can remove any administrator
      // An administrator only can remove itself
      ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == Gbl.Usrs.Other.UsrDat.UsrCod);
      ICanRemove = ItsMe || Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER;
      if (ICanRemove)
        {
         /* Check if the other user belong to current course */
         if (Usr_CheckIfUsrIsAdmOfDeg (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentDeg.Deg.DegCod))
           {		// The other user is an administrator of current degree ==> ask for removing or remove him
            switch (ReqDelOrDelUsr)
              {
               case Usr_REQUEST_REMOVE_USR:     // Ask if remove administrator from current degree
                  Usr_AskIfRemAdmFromDeg (ItsMe);               
                  break;
               case Usr_REMOVE_USR:             // Remove administrator from current degree
                     Usr_EffectivelyRemAdmFromDeg (Gbl.Usrs.Other.UsrDat.UsrCod);
                 }
              }
         else        // The other user is not an administrator of current degree
           {
            sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> no es un administrador de la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
                     Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentDeg.Deg.FullName);
            Lay_ShowAlert (WARNING,Gbl.Message);
           }
        }
      else
        {
         sprintf (Gbl.Message,"No le est&aacute; permitido dar de baja como administrador" \
                              " de la titulaci&oacute;n <strong>%s</strong> al usuario con DNI <strong>%s</strong>.",	// Need translation!
                  Gbl.CurrentDeg.Deg.FullName,Gbl.Usrs.Other.UsrDat.DNI);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else
      Usr_ReqDNIRemAdm ();
  }

/****************************************************************************/
/********* Write form to ask for a user's DNI to remove it globally *********/
/****************************************************************************/

void Usr_ReqDNIRemGbl (void)
  {
   Lay_ShowAlert (INFO,"Escriba el DNI del usuario que desea eliminar.");	// Need translation!

   Usr_WriteFormToReqAnotherUserDNI (ActAskRemUsrGbl);
  }

/****************************************************************************/
/********* Request definite removing of a user from the platform ************/
/****************************************************************************/

void Usr_ReqRemUsrGbl (void)
  {
   Usr_ReqDelOrDelUsrGbl (Usr_REQUEST_REMOVE_USR);
  }

/****************************************************************************/
/************** Definite removing of a user from the platform ***************/
/****************************************************************************/

void Usr_RemUsrGbl (void)
  {
   Usr_ReqDelOrDelUsrGbl (Usr_REMOVE_USR);
  }

/****************************************************************************/
/**************************** Removing of a student *************************/
/****************************************************************************/

void Usr_ReqDelOrDelUsrGbl (Usr_ReqDelOrDelUsr_t ReqDelOrDelUsr)
  {
   bool ItsMe;

   /***** Get user's DNI from form *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      ItsMe = (Gbl.Usrs.Me.UsrDat.UsrCod == Gbl.Usrs.Other.UsrDat.UsrCod);
      if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER && !ItsMe)
         switch (ReqDelOrDelUsr)
           {
            case Usr_REQUEST_REMOVE_USR:
               Usr_AskIfRemUsrFromCrsOrPlatform (ItsMe);    // Ask if remove user
               break;
            case Usr_REMOVE_USR:
               Usr_DeleteUsrFromPlatform (&Gbl.Usrs.Other.UsrDat,Cte_VERBOSE); // Remove definitely the user from the platform
           }
      else
        {
         sprintf (Gbl.Message,"No le est&aacute; permitido eliminar completamente al usuario con DNI <strong>%s</strong>.",	// Need translation!
                  Gbl.Usrs.Other.UsrDat.DNI);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else
      Usr_ReqDNIRemGbl ();
  }

/****************************************************************************/
/******************* Ask if really wanted to remove a user ******************/
/****************************************************************************/

void Usr_AskIfRemUsrFromCrsOrPlatform (bool ItsMe)
  {
   extern const char *Txt_Remove_me[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_user[Txt_NUM_LANGUAGES];

   if (Usr_ChkIfDNIExists (Gbl.Usrs.Other.UsrDat.DNI))
     {
      if (Gbl.CurrentAct == ActAskRemUsrCrs)
	 sprintf (Gbl.Message,"&iquest;Realmente desea %s (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.) de la asignatura <strong>%s</strong>?",	// Need translation!
                  ItsMe ? "darse de baja" :
                          "dar de baja el siguiente usuario",Gbl.CurrentCrs.Crs.FullName);	// Need translation!
      else // Gbl.CurrentAct == ActAskRemUsrGbl
   	 sprintf (Gbl.Message,"&iquest;Realmente desea %s?",
                  ItsMe ? "eliminar completamente su cuenta de usuario" :
                          "eliminar por completo el siguiente usuario");	// Need translation!
      Lay_ShowAlert (WARNING,Gbl.Message);

      Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);

      Act_FormStart ((Gbl.CurrentAct == ActAskRemUsrCrs) ? ActRemUsrCrs : ActRemUsrGbl);
      Usr_PutParamOtherUsrDNIPlain ();
      fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"%s\" /></div>\n</form>\n",
               (Gbl.CurrentAct == ActAskRemUsrCrs) ?
                (ItsMe ? Txt_Remove_me[Gbl.Prefs.Language] :
                         Txt_Remove_user[Gbl.Prefs.Language]) :
                (ItsMe ? "Eliminarme completamente" :
                         "Eliminar completamente"));	// Need translation!
     }
   else
      Lay_ShowErrorAndExit ("User doesn't exist.");
  }

/****************************************************************************/
/****** Ask if really wanted to add an administrator to current degree ******/
/****************************************************************************/

void Usr_AskIfAddAdmToDeg (void)
  {
   extern const char *Txt_Register_user_IN_A_COURSE[Txt_NUM_LANGUAGES];

   if (Gbl.CurrentDeg.Deg.DegCod < 0)
     {
      Lay_ShowAlert (WARNING,"Debe seleccionar una titulaci&oacute;n.");	// Need translation!
      return;
     }

   /***** Get plain DNI of the user to add/modify *****/
   if (Usr_GetParamOtherDNIPlainAndGetUserData ())
     {
      /* Check if it's allowed to register this administrator in degree */
      if (Gbl.Usrs.Me.UsrTypeLog == Usr_SUPERUSER)
        {
         if (Usr_CheckIfUsrIsAdmOfDeg (Gbl.Usrs.Other.UsrDat.UsrCod,Gbl.CurrentDeg.Deg.DegCod))        // User is yet an administrator of current degree
           {
            sprintf (Gbl.Message,"El usuario con DNI <strong>%s</strong> ya era administrador de la titulaci&oacute;n <strong>%s</strong>.",	// Need translation!
                     Gbl.Usrs.Other.UsrDat.DNI,Gbl.CurrentDeg.Deg.FullName);
            Lay_ShowAlert (INFO,Gbl.Message);
            Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);
           }
         else
           {
            sprintf (Gbl.Message,"&iquest;Realmente desea dar de alta el siguiente administrador en la titulaci&oacute;n <strong>%s</strong>?",	// Need translation!
                     Gbl.CurrentDeg.Deg.FullName);
            Lay_ShowAlert (INFO,Gbl.Message);
            Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);

            Act_FormStart (ActNewAdm);
            Usr_PutParamOtherUsrDNIPlain ();
            fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"%s\" /></div>\n</form>\n",
                     Txt_Register_user_IN_A_COURSE[Gbl.Prefs.Language]);
           }
        }
      else
        {
         sprintf (Gbl.Message,"No le est&aacute; permitido dar de alta como administrador" \
                              " de la titulaci&oacute;n <strong>%s</strong> al usuario con DNI <strong>%s</strong>.",	// Need translation!
                  Gbl.CurrentDeg.Deg.FullName,Gbl.Usrs.Other.UsrDat.DNI);
         Lay_ShowAlert (WARNING,Gbl.Message);
        }
     }
   else
      Usr_ReqDNINewAdm ();
  }

/****************************************************************************/
/*** Ask if really wanted to remove an administrator from current degree ****/
/****************************************************************************/

void Usr_AskIfRemAdmFromDeg (bool ItsMe)
  {
   extern const char *Txt_Remove_me[Txt_NUM_LANGUAGES];
   extern const char *Txt_Remove_user[Txt_NUM_LANGUAGES];

   if (Usr_ChkIfDNIExists (Gbl.Usrs.Other.UsrDat.DNI))
     {
      sprintf (Gbl.Message,"&iquest;Realmente desea %s de la titulaci&oacute;n <strong>%s</strong>?",	// Need translation!
               ItsMe ? "darse de baja como administrador" :
                       "dar de baja el siguiente administrador",	// Need translation!
               Gbl.CurrentDeg.Deg.FullName);
      Lay_ShowAlert (INFO,Gbl.Message);

      Rec_ShowCommonRecordUnmodifiable (&Gbl.Usrs.Other.UsrDat);

      Act_FormStart (ActRemAdm);
      Usr_PutParamOtherUsrDNIPlain ();
      fprintf (Gbl.F.Out,"<div align=\"center\"><input type=\"submit\" value=\"%s\" /></div>\n</form>\n",
               ItsMe ? Txt_Remove_me[Gbl.Prefs.Language] :
                       Txt_Remove_user[Gbl.Prefs.Language]);
     }
   else
      Lay_ShowErrorAndExit ("User doen't exist.");
  }

/****************************************************************************/
/************* Build the relative path of a user from his DNI ***************/
/****************************************************************************/

void Usr_ConstructPathUsr (char *DNI,char *PathRelUsr,char *PathAbsUsr)
  {
   char DNI_LSB; // Least significant character of the DNI

   if (!Usr_CheckIfDNIIsValidThenRemZerosAndAlpha (DNI))
      Lay_ShowErrorAndExit ("User's ID is not valid.");
   DNI_LSB = DNI[strlen (DNI)-1];
   sprintf (PathRelUsr,"%s/%c/%s",Cfg_PATH_REL_USR_PRIVATE,DNI_LSB,DNI);
   if (PathAbsUsr)
      sprintf (PathAbsUsr,"%s/%c/%s",Cfg_PATH_ABS_USR_PRIVATE,DNI_LSB,DNI);
  }

/****************************************************************************/
/************** Check if a user exists with a given user's code *************/
/****************************************************************************/

bool Usr_ChkIfUsrCodExists (long UsrCod)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a user exists in database *****/
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE UsrCod='%ld'",UsrCod);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user exists");

   /***** Get number of rows (0/1) with this DNI *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a user exists.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/**************** Check if a user exists with a given DNI *******************/
/****************************************************************************/

bool Usr_ChkIfDNIExists (const char *DNI)
  {
   char Query[512];
   MYSQL_RES *mysql_res;
   MYSQL_ROW row;
   unsigned NumRows;

   /***** Get if a user exists in database *****/
   sprintf (Query,"SELECT COUNT(*) FROM usr_data WHERE DNI='%s'",DNI);
   DB_QuerySELECT (Query,&mysql_res,"can not check if a user exists");

   /***** Get number of rows (0/1) with this DNI *****/
   row = mysql_fetch_row (mysql_res);
   if (sscanf (row[0],"%u",&NumRows) != 1)
      Lay_ShowErrorAndExit ("Error when checking if a user exists.");

   /***** Free structure that stores the query result *****/
   DB_FreeMySQLResult (&mysql_res);

   return (NumRows != 0);
  }

/****************************************************************************/
/********************** Show warning "no users found" ***********************/
/****************************************************************************/
// Use Usr_UNKNOWN type to display "no users found"

void Usr_ShowWarningNoUsersFound (Usr_UsrType_t UsrType)
  {
   extern const char *Txt_No_users_found[Usr_NUM_USR_TYPES][Txt_NUM_LANGUAGES];

   Lay_ShowAlert (WARNING,Txt_No_users_found[UsrType][Gbl.Prefs.Language]);
  }
